/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.rtc.foundation.web.ui.gridviewer.NativeStyleSheetManager");

dojo.require("dojox.html.styles");

(function() {
/**
 * Native implementation of a dynamic style sheet manager. As this uses lots of non standard API,
 * the implementation is done using pure JavaScript.
 * We do not use dojox.html.styles, as it full of errors, does not allow updating rules and logs
 * each change made to a dynamic style sheet.
 * @since 3.0
 */
var NativeStyleSheetManager= dojo.declare("com.ibm.team.rtc.foundation.web.ui.gridviewer.NativeStyleSheetManager", null, {

});

var _sheet= null;
function _getStyleSheet() {
	if (_sheet == null) {
		if (dojo.doc.createStyleSheet) {
			_sheet= dojo.doc.createStyleSheet();
			_sheet.title= 'GridViewerDynamicStyleManager';
		} else {
			var element= dojo.doc.createElement('style');
			element.setAttribute('type', 'text/css');
			element.setAttribute('title', 'GridViewerDynamicStyleManager');
			dojo.doc.getElementsByTagName('head')[0].appendChild(element);
			_sheet= element.sheet;
		}
	}
	return _sheet;
}


NativeStyleSheetManager.getRules= function() {
	// see http://www.quirksmode.org/dom/w3c_css.html#access

	var sheet= _getStyleSheet();
	return sheet.cssRules || sheet.rules;
}

NativeStyleSheetManager.insertRule= function(selector, rule) {
	// see http://www.quirksmode.org/dom/w3c_css.html#change

	var sheet= _getStyleSheet();
	if (sheet.insertRule) {
		sheet.insertRule(selector + ' {' + rule + '}', sheet.cssRules.length);
	} else {
		sheet.addRule(selector, rule);
	}
}

NativeStyleSheetManager.deleteRule= function(index) {
	// see http://www.quirksmode.org/dom/w3c_css.html#change

	var sheet= _getStyleSheet();
	if (sheet.deleteRule) {
		sheet.deleteRule(index);
	} else {
		sheet.removeRule(index);
	}
}

})();
