 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "ItemProxyMessages");
(function() { //module container

com.ibm.team.workitem.web.cache.internal.Cache = {	
	addItem: function(id, object) {
		var cache = this.getCache();
		var id = this.getId(id);
		if(cache[id]) {
			return cache[id];
		}
		object._cacheId = id;
		cache[id] = object; 
		this.useCounts[id] =  0;

		return cache[id];
	},
	removeItem: function(id) {
	    if(id == null) return;
		var cache = this.getCache();
		var id = this.getId(id);
		if(cache[id]) {
			cache[id] = false;
			delete cache[id];
			this.useCounts[id] = 0;
			delete this.useCounts[id];
			return true; /* was removed */
		}
		return false; /* not found */
	},
	renameItem: function(oldId, newId) {
	    if(oldId == newId) return;
		oldId = this.getId(oldId);
		newId = this.getId(newId);
		var cache = this.getCache();
		if(cache[oldId]) {
			cache[newId] = cache[oldId];
			cache[oldId] = false;
			delete cache[oldId];
			this.useCounts[newId] = this.useCounts[oldId];
			this.useCounts[oldId] = 0;
			delete this.useCounts[oldId];
		}
		return false;
	},
	getCache: function() {
		if(!this.cache) {
			this.cache = {};
			this.useCounts = {}
		}
		return this.cache;		
	},
	getId: function(id) {
		return id + "";
	},
	getItem: function(id, object) {

		var cache = this.getCache();
		var id = this.getId(id);
		if(cache[id]) {
			this._showDebugMessage("getItem(" + id + ")=found");
			return cache[id];
		}
		if(object) {
			this.addItem(id, object);
			this._showDebugMessage("getItem(" + id + ")=add item");
			return object;
		}
		this._showDebugMessage("getItem(" + id + ")=null");
		return null;
	},
	getItemByItemId: function(itemId) {
		var cache = this.getCache();
		for(var key in cache) {
			if(cache[key].getItemId && cache[key].getItemId() == itemId) {
				return cache[key];
			}
		}
		return null;
	},
	/*
	 * Get all items that meet all the criteria
	 * Must match every filter
	 *     filterSelectors: Match methods (selector called must return true)
	 *     filterAttributes: Match attribute values (not null or true)
	 */
	getAllItems: function(parms /* filterSelectors, filterAttributes, excludeSelectors */) {
		parms = parms || {};
		var cache = this.getCache();
		var result = [];
		for(var key in cache) {
			if(cache[key]) {
				var fail = false;
				if(parms.filterSelectors) {
					for(var i=0;i<parms.filterSelectors.length;i++) {
						var selector = parms.filterSelectors[i];
						if(!dojo.isFunction(cache[key][selector]) || !cache[key][selector]()) {
						    fail = true;
							break;
						}						
					}
				}
				if (!fail) {
					if (parms.filterAttributes) {
						for (var i = 0; i < parms.filterAttributes.length; i++) {
							var attributeName = parms.filterAttributes[i];
							if (!cache[key][attributeName]) {
								fail = true;
								break;
							}
						}
					}
				}
				if(!fail) {
					result.push(cache[key]);
				}				
			} 
		}
		if(!parms.excludeSelectors) return result;
		var exclusionResult = [];
		for(var i=0;i<result.length;i++) {
			var fail = false;
			for(var j=0;j<parms.excludeSelectors.length;j++) {
					var selector = parms.excludeSelectors[j];
					if(dojo.isFunction(result[i][selector]) && result[i][selector]()) {
					    fail = true;
					    break;
					}						
			}
			if(!fail) {
				exclusionResult.push(result[i]);
			}
		}
		return exclusionResult;
	},
	getUseCount: function(id) {
		var cache = this.getCache();
		return this.useCounts[id];
	},
	incrementUseCount: function(id) {
		var cache = this.getCache();
		var item = cache[id];
		if(!item) return;
		this.useCounts[id]++
		this._showDebugMessage("increment(" + id + ")=" + this.useCounts[id]);
	},
	decrementUseCount: function(id, removeEvenIfChanged) {
		var cache = this.getCache();
		var item = cache[id];
		if(!item) return;
		
		this.useCounts[id]--
		if(this.useCounts[id]< 0) { 
			this.useCounts[id];
			console.debug(dojo.string.substitute(this._getMessages().debugUseCountLessThanZero, [id]));
		}
		
		var isChanged = item.isChanged;
		if(dojo.isFunction(item.isChanged)) {
			isChanged = item.isChanged();
		}
		var isNew = item.isNew;
		if(dojo.isFunction(isNew)) {
			isNew = item.isNew();
		}
		if(!removeEvenIfChanged && isChanged && !isNew) return; // keep unsaved items, unless it's new and zero
		if(this.useCounts[id] <= 0) {
			this._showDebugMessage("decrement(" + id + ")=" + this.useCounts[id] + " (removed)");
			if(cache[id].destroy) {
				cache[id].destroy();
			}
			cache[id] = false;
			this.useCounts[id] = 0;
		} else {
			this._showDebugMessage("decrement(" + id + ")=" + this.useCounts[id]);			
		}

	},
	_showDebugMessage: function(msg) {
	    if(this._isDebugging === false) return;
		if(this._isDebugging === true || window.location.href.indexOf("debugCache") !=-1) {
		     this._isDebugging = true;
		     console.debug(msg);	
		} else {
			this._isDebugging = false;
		}
	},
	_getMessages: function() {
		if(this.messages) return this.messages;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "ItemProxyMessages");
		return this.messages;
	}
};

})();