 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory");

(function() { //module container

com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory = {	
	/*
	 * Create a new link type proxy
	 */
	getLinkTypeProxy: function(creationParms /* linkType */) {
		var linkTypeProxy = {
			object: creationParms.linkType || [],
			isLinkTypeEqual: function(parms /* linkType */) {
				if(!parms.linkType && this.object) return false;
				if(!this.object && parms.linkType) return false;
				if(!this.object && !parms.linkType) return true;
				
				var otherLinkType = parms.linkType;
				if(this.object.id == otherLinkType.id && (this.object.isSymmetric || (this.object.isSource === otherLinkType.isSource))) return true;
				
				return false;
			},
			/*
			 * Merge any changes in the fromLinkType to this.object
			 * Return true if the link type was found in this.object
			 */
			mergeChangesFromParameterToObject: function(parms /* fromLinkType */ ) {
				var fromLinkType = parms.fromLinkType;
				if(!fromLinkType || !fromLinkType.linkDTOs || fromLinkType.linkDTOs.length == 0) return false;

				/* If they are different types, skip them */
				if(fromLinkType.id != this.object.id) return false;
				if (fromLinkType.isSource != null) { 
					if(fromLinkType.isSource != this.object.isSource) return false;
				}
				if (fromLinkType.isSymmetric != null) { 
					if(fromLinkType.isSymmetric != this.object.isSymmetric) return false;
				}
						
				/*
				 * Merge the from dtos with the current ones
				 */								
				var linkChanged = false;
				var newDTOs = [];				
				for(var i=0;i<fromLinkType.linkDTOs.length;i++) {
					var fromLinkDTO = fromLinkType.linkDTOs[i];
					
					var wasFound = false;
					for(var j=0;j<this.object.linkDTOs.length;j++) {
						if(this.object.linkDTOs[j].itemId == fromLinkDTO.itemId ) {
							this.object.linkDTOs[j].wasFound = true;
							wasFound = true;
							newDTOs.push(fromLinkDTO).itemId; // this will handle the isDeleted case
							break;
						} 
					}
					if(!wasFound || fromLinkDTO._isNew) {
						newDTOs.push(fromLinkDTO);
					}
				}
				
				for(var j=0;j<this.object.linkDTOs.length;j++) {
					if(!this.object.linkDTOs[j].wasFound) {
						newDTOs.push(this.object.linkDTOs[j]);
					}	
				}
				
				this.object.linkDTOs = newDTOs;
				
				this.object.linkDTOs = this.object.linkDTOs ||  [];
				fromLinkType.linkDTOs = fromLinkType.linkDTOs || [];
				
				return true;
			},
			getLinkTypeLike: function(fromLinkDTO){
				if(this.object || this.object.linkDTOs || this.object.linkDTOs.length == 0) { return null }
				
				for(var j=0;j<this.object.linkDTOs.length;j++) {
					console.debug(linkDTO);	
				}
			},
			mergeLinkDTO: function() {},
			/*
			 * Merge the passed in DTO to the current object
			 */
			mergeDTOsFromRepositoryToCache: function(parms /* linkType */) {
				if(!this.object || !parms.linkType || !parms.linkType.linkDTOs || parms.linkType.linkDTOs.length == 0) return;
				this.object.linkDTOs = this.object.linkDTOs ||  [];
				var cachedLinkType = parms.linkType;
				cachedLinkType.linkDTOs = cachedLinkType.linkDTOs || [];
				var newCachedLinkDTOs = [];
				var repoFoundDTOs = {};
				for(var i=0;i<cachedLinkType.linkDTOs.length; i++) {
					var cachedLinkDTO = cachedLinkType.linkDTOs[i];
					/*
					 * See if the cached dto's are still in the repo
					 * if so, add it to the new list
					 * if not, add it to the new list if it is "new"
					 */
					 var repoLinkDTO;
					 var found = false;
					 for(var j=0;j<this.object.linkDTOs.length; j++) {
					 	repoLinkDTO = this.object.linkDTOs[j];
					 	if(cachedLinkDTO.itemId == repoLinkDTO.itemId) {
				            repoFoundDTOs[repoLinkDTO.itemId] = repoLinkDTO;
				  			repoLinkDTO._isDeleted = cachedLinkDTO._isDeleted;	
					 		found = true;
					 		break;
					 	}
					 }
					 if(found) {
					 	newCachedLinkDTOs.push(repoLinkDTO);
					 } else {
					 	if(cachedLinkDTO._isNew) {
					 		newCachedLinkDTOs.push(cachedLinkDTO);
					 	} // else it was deleted in the repo and is now removed here
					 }
				}
				/*
				 * Now add any new dto's from the repo to the cache
				 */
				for(var j=0;j<this.object.linkDTOs.length; j++) {
					if(!repoFoundDTOs[this.object.linkDTOs[j].itemId]) {
						newCachedLinkDTOs.push(this.object.linkDTOs[j]);
					}
				}
				
				cachedLinkType.linkDTOs = newCachedLinkDTOs;			
			}
		}
		linkTypeProxy.id = creationParms.linkType.id; // TODO: Verify that id is correct
		return linkTypeProxy;
	}
};

})();