/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.QueryProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;

dojo.declare("com.ibm.team.workitem.web.cache.internal.QueryProxy", [com.ibm.team.workitem.web.cache.internal.ItemProxy], {
	/*
	 * Construct the proxy passing the object
	 */
	isQuery: true,
	constructor: function(parms){
		this.object = parms.queryDto;
		this.storedObject = dojo.clone(parms.queryDto);
		this.storedObject.name = this.object._oldName
		this.storedObject.attributes = {};
        this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
	},
	getItemId: function(){
		return this.object.itemId || this.object.clientItemId;
	},
	getLinkLabel: function() {
		if(this.object && this.object.name && this.object.name.length > 0) {
			if(!this.isNew() && this.object.name == this.messages.labelDefaultQueryName) {
				return this.messages.labelNoName;
			}
			return this.object.name;
		}
		if(this.isNew()) {
			return this.messages.labelNewQuery;
		}
		return this.messages.labelNoName;
	},
	getSummaryLabel: function() {
		if(this.object) {
			if(this.object.description && this.object.description.length > 0) {
				return this.object.name + ": " + this.object.description ;
			} else {
				return this.object.name;
			}
		}
		return this.messages.labelNewQuery;
	}, 
	setIsBeingEdited: function(isBeingEdited) {
		this._isBeingEdited = isBeingEdited;
	},
	getIsBeingEdited: function() {
		return this._isBeingEdited == true;
	},
	getIconUrl: function() {
		return com.ibm.team.workitem.web.ui.internal.view.common.ImagePool.WORKITEM_QUERY;
	},
	getUrl: function() {		
		if(this._queryUrl) {		
			if(this.isNew() 
				|| this._queryUrl.indexOf("action=com.ibm.team.workitem.search") != -1
				|| this._queryUrl.indexOf("action=com.ibm.team.workitem.similarityQuery") != -1) return this._queryUrl;
		}
		if(this.isNew() && this.object.attributes.hasNewQueryUrl || this.getItemId() == null) {
			return WorkItemUtil.getNewQueryUri();
		}	
		if(this.isChanged() || this._isBeingEdited) {
			return WorkItemUtil.getEditQueryUri(this.getItemId());
		}
		return WorkItemUtil.getQueryUri(this.object);
	},
	/* 
	 * Answer true if the value has changed
	 */
	isChanged: function(){
		return this.getValue({path:"attributes.isChanged"});
	},
	/* 
	 * Answer true if the value is new
	 */
	isNew: function(){
		if(this.object._isSaved) return false;
		return !this.object || this.object.isNew || this.object.clientItemId == "" || (this._queryUrl && this._queryUrl.indexOf("action=com.ibm.team.workitem.newQuery") != -1) ;
	},
	resetToStored: function() {
	    var wasNew = this.object.isNew
		this.object = this.storedObject;
		this.object.isNew = wasNew;
	},
	__sential: true
}
);

})();
