/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.WorkItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemListProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItem");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil"); // TODO: remove
dojo.require("com.ibm.team.workitem.web.cache.internal.CacheUtil"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.RequiredPropertiesUpdater"); //$NON-NLS-1$
dojo.require("com.ibm.jdojo.lang.Runtime"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI");

(function() {

// These variables must match LinkCommand.java
var ADD_LINK= "addLink";
var REMOVE_LINK= "removeLink";
var UPDATE_LINK= "updateLink";
var USERS_URI_STUB = "/users/";
var SUBSCRIBERS_ID= "internalSubscriptions";
// The following must match WorkItemRestService.java
var UPDATE_LINKS = "updateLinks";
var ATTACHMENT_TYPE= "com.ibm.team.workitem.linktype.attachment";   // $NON-NLS-1$
/* 
 * C/ALM link types can fail on save because of backlink errors
 */
    var BACK_LINK_TYPES_MAP = {
    	"com.ibm.team.workitem.linktype.relatedChangeManagement": true,
		"com.ibm.team.workitem.linktype.blocksTestExecutionRecord": true,
		"com.ibm.team.workitem.linktype.affectsExecutionResult": true,
		"com.ibm.team.workitem.linktype.testedByTestCase": true,
		"com.ibm.team.workitem.linktype.implementsRequirement": true
    }

 
var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var RequiredPropertiesUpdater= com.ibm.team.workitem.web.cache.internal.RequiredPropertiesUpdater;

dojo.declare("com.ibm.team.workitem.web.cache.internal.WorkItemProxy", [com.ibm.team.workitem.web.cache.internal.ItemListProxy, com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient], {
	/*
	 * Construct the proxy passing the object
	 */
	isWorkItem: true,
	constructor: function(object){
			this.changedAttributePaths  = {};
			this.__workItemAdapter= new com.ibm.team.workitem.web.client.internal.WorkItem(this);
			dojo.publish(EventTopics.WORKITEM_CREATED, [ { workingCopy: this } ]);
	},
	/*
	 * Set an attribute value, notifiying listeners
	 * The path is an array of attribute names, e.g., ["attributes", "internalPriority"]
	 */
	getValue: function(parms /* path */){
		var path = this._getPath(parms.path);
		if (path[0] == "linkTypes") {
			return this._getLinkType(parms)
		};
		if(parms.resolveStateWithWorkflowAction && parms.path && parms.path.length > 1 && path[0] == "attributes" && parms.path[1] == "internalState") {
			return this._getWorkItemState(parms)
		}
		// Special case for contributors/owners
		// If the request is for an id
		//            and the id is null
		//            and the itemId is not null
		//            set the id to the itemId
		if(parms.path && parms.path.length > 2 && path[0] == "attributes" && path[parms.path.length-1] == "id") {
			var value = this.inherited(arguments);
			if(value) return value;
			path[parms.path.length-1] = "itemId";
			parms.path = path;
			return this.inherited(arguments);
		}
		
		return this.inherited(arguments);
	},
	_getWorkItemState: function(parms) {
		var value = this.inherited(arguments);
		var workflowAction = this.getValue({
			path: ["attributes", "workflowAction"]
		});
		if(!workflowAction || !this.workItemSpec || !this.workItemSpec.editProps || !this.workItemSpec.editProps.allValues) return value;
		var allValues = this.workItemSpec.editProps.allValues;
		for(var i=0;i<allValues.length;i++) {
			if(allValues[i].attributeName == "actionValues") {
				var uiItems = allValues[i].uiItems;
				for(var j=0;j<uiItems.length;j++) {
					if(uiItems[j].id == workflowAction.id ) {
						return uiItems[j];
					}
				}
			}
		}
		return value;
	},
	/*
	 * Called to fix anomolies that may exist after move/copy transmorgrification.
	 */
	prepareToBeEdited: function() {
		if(this._lastId && this._lastId != this.object.id) {
			this.object.id = this._lastId;
		}
	},
	handleObjectRefresh: function() {
		this.clearRetainedState();
	},
	/*
	 * Return true if the work item has a link to, from or with the work item in the itemId
	 */
	hasLinkToWorkItem: function(workItemProxy) {
		var workItemItemId = workItemProxy.getItemId();
		if(!this.object.linkTypes ||  this.object.linkTypes.length == 0) return false;
		for(var i=0;i<this.object.linkTypes.length;i++) {
			if(this.object.linkTypes[i].linkDTOs && this.object.linkTypes[i].linkDTOs.length > 0) {
				for(var j=0;j< this.object.linkTypes[i].linkDTOs.length;j++) {
					if(this.object.linkTypes[i].linkDTOs[j] 
						&& this.object.linkTypes[i].linkDTOs[j].target 
						&&  this.object.linkTypes[i].linkDTOs[j].target.itemId === workItemItemId) {
						return true;
					}
				}
			}
		}
		return false;
	},
	/*
	 * Return true if the two items don't link to weach other or the links both ways match 
	 */
	doLinksMatch: function(workItemProxy) {
		/*
		 *  Most of the time they are not linked, so check that first
		 */
		if(!this.hasLinkToWorkItem(workItemProxy)  && !workItemProxy.hasLinkToWorkItem(this)) return true;

		var  thisLinksToThat = this.getLinkTypesToWorkItem(workItemProxy); 
		var  thatLinksToThis = workItemProxy.getLinkTypesToWorkItem(this);	
		if(thisLinksToThat.length != thatLinksToThis.length) return false;	
		for(var i=0;i<thisLinksToThat.length;i++) {
			var thisLinkToThat = thisLinksToThat[i];
			var found = false
			for(var j=0;j<thatLinksToThis.length;j++) {
				var thatLinkToThis = thatLinksToThis[j];
				if(this.isLinkTypeReciprocal(thisLinkToThat.linkType, thatLinkToThis.linkType)) {
					found = true;
					continue;
				}
			}
			if(found == false) return false;
		}
		return true;
	},
	isLinkTypeReciprocal: function(thisLinkType, thatLinkType) {
		if(!thisLinkType || !thatLinkType) return false;
		
		if(thisLinkType.id == thatLinkType.id 
			&& ((thisLinkType.isSymmetric && thatLinkType.isSymmetric)
			 || (thisLinkType.isSource === thatLinkType.isTarget)
			 || (thisLinkType.isTarget === thatLinkType.isSource))) return true;
		
		return false;
	},
	
	/*
	 * Retun the list of links to work items
	 * Return an array of:
	 *    [ { linkType: aLinkType, linkDTO: aLinkDTO } ]
	 */
	getLinkTypesToWorkItem: function(workItemProxy) {
		var workItemItemId = workItemProxy.getItemId();
		var result = [];
		if(!this.object.linkTypes ||  this.object.linkTypes.length == 0) return result;
		for(var i=0;i<this.object.linkTypes.length;i++) {
			if(this.object.linkTypes[i].linkDTOs && this.object.linkTypes[i].linkDTOs.length > 0) {
				for(var j=0;j< this.object.linkTypes[i].linkDTOs.length;j++) {
					if(this.object.linkTypes[i].linkDTOs[j] 
						&& this.object.linkTypes[i].linkDTOs[j].target 
						&&  this.object.linkTypes[i].linkDTOs[j].target.itemId === workItemItemId) {
						result.push({linkType:this.object.linkTypes[i], linkDTO: this.object.linkTypes[i].linkDTOs[j] });
					}
				}
			}
		}
		return result;
	},

	getAttachmentInfoById: function(itemId ) {
			if(!itemId) return;
			var workItem = this.object;
			if(!workItem || !workItem.linkTypes ||  workItem.linkTypes.length == 0) return;
			for(var i=0;i<workItem.linkTypes.length;i++) {
				if(workItem.linkTypes[i].id=="com.ibm.team.workitem.linktype.attachment") {
					if(workItem.linkTypes[i].linkDTOs) {
						for(var j=0;j< workItem.linkTypes[i].linkDTOs.length;j++) {
							if(workItem.linkTypes[i].linkDTOs[j] 
								&& workItem.linkTypes[i].linkDTOs[j].target 
								&&  workItem.linkTypes[i].linkDTOs[j].target.itemId === itemId) {
								return workItem.linkTypes[i].linkDTOs[j].target;
							}
						}
					}					
				}
			}
		},
	getAttachmentInfo: function(number ) {
			if(!number) return;
			if(!isNaN(number)) {
				try{
					number = parseInt(number);
				} catch(e) { return };
			}
			var workItem = this.object;
			if(!workItem || !workItem.linkTypes ||  workItem.linkTypes.length == 0) return;
			for(var i=0;i<workItem.linkTypes.length;i++) {
				if(workItem.linkTypes[i].id=="com.ibm.team.workitem.linktype.attachment") {
					if(workItem.linkTypes[i].linkDTOs) {
						for(var j=0;j< workItem.linkTypes[i].linkDTOs.length;j++) {
							if(workItem.linkTypes[i].linkDTOs[j] 
								&& workItem.linkTypes[i].linkDTOs[j].target 
								&&  workItem.linkTypes[i].linkDTOs[j].target.id === number) {
								return workItem.linkTypes[i].linkDTOs[j].target;
							}
						}
					}					
				}
			}
		},
	getId: function(skipLabel) {
		if(this.isDestroyed) return this._cacheId;
		var id = -1;
		if(this.object && this.object.id) {
			return this.object.id;
		}
		if(this.object && this.object.attributes && this.object.attributes.id) {
			id = this.object.attributes.id.id || -1
		}
		if(!id && this.object && this.object.attributes && this.object.attributes.id && this.object.attributes.id.label) {
				id = this.object.attributes.id.label;
			} 
		if(!skipLabel && this._lastId != id) {
			this.getIdLabel(id);
		}
		this._lastId = id;
		return id;
	},
	getIdLabel: function(id){
		if((id==null) && this.idLabel != null) {
			return this.idLabel;
		}
		if(this._isInIdLabel) return this.idLabel;
		this._isInIdLabel = true;
		id = id || this.getId(true);
		var idLabel = id;
		if(id < 0) {
				idLabel = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil.getNewWorkItemTimestampLabel(this.object);
			} 
		this.idLabel = idLabel + "";
		this._isInIdLabel = false;
		return idLabel;
	},
	getItemId: function(){
		return this.object.itemId;
	},
	getLinkLabel: function() {
		var typeLabel = "Work Item"; // TODO:NLS
		if(this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.label) {
			typeLabel =  this.object.attributes.workItemType.label;
		}
		return typeLabel + " " + this.getIdLabel(); // TODO:NLS
	}, 
	getTypeIconUrl: function() {
		if (this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.iconUrl) {
			return this.object.attributes.workItemType.iconUrl;
		}
		return null;
	},
	getSummaryContent: function() {
		if (this.object && this.object.attributes && this.object.attributes.summary && this.object.attributes.summary.content) {
			return com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil.restoreTags(this.object.attributes.summary.content);
		}
		return ""
	}, 
	getSummaryLabel: function() {
		return this.getShortSummaryLabel(-1);
	},
	/*
	 * Return a summary label with the maximum length
	 */
	getShortSummaryLabel: function (maxLength) {
		if(!this.object){
			return "";
		}
				
		var summaryText =  this.getSummaryContent() || "";
		
		if(maxLength && maxLength > 0 && summaryText.length > maxLength) {
			summaryText = summaryText.substring(0, maxLength) + "...";
		}
		return summaryText
	}, 
	getIsRequired: function(attributeId) {
		if(!this.workItemSpec || !this.workItemSpec.editProps || !this.workItemSpec.editProps.requiredProperties) return false;
		for(var i=0;i<this.workItemSpec.editProps.requiredProperties.length;i++) {
			if(this.workItemSpec.editProps.requiredProperties[i] == attributeId) return true;
		}
		return false;
	},
	/* 
	 * Answer true if the value has changed
	 * TODO: Optimize this
	 */
	isChanged: function(){
		if(this.isDestroyed) return false;
		
		if (this.getIsDirty())
			return true;
		
		// if there are any changed attributes, return true
		if(this.changedAttributePaths) {
			for(var key in this.changedAttributePaths) {
				return true;
			}
		}

		var isChanged = false;
		var cmds = {};
		var saveCommands = this.getSaveCommands(cmds);
		for (var x in saveCommands) {
			isChanged = true;
			break;
		}
		if(isChanged) return true;
		for (var x in this.changedAttributes) {
			isChanged = true;
			break;
		}
		if(isChanged) return true;
		/*
		 * If the item was moved
		 */
		if(this.object && this.object.attributes && this.object.attributes.projectArea &&
		   this.storedObject && this.storedObject.attributes && this.storedObject.attributes.projectArea) {
			if(this.object.attributes.projectArea.id != this.storedObject.attributes.projectArea.id) return true;
		}
		return false;
	},
	isNotChanged: function(){
		return !this.isChanged();
	},
getProjectAreaItemId: function() {
	if(this.object && this.object.attributes && this.object.attributes.projectArea && this.object.attributes.projectArea.id) return this.object.attributes.projectArea.id;
	return null;
},
isInMove: function() {
	var currentProjectAreaItemId = this.getProjectAreaItemId();
	var storedProjectAreaItemId = -1;
	
	if(this.storedObject && this.storedObject.attributes && this.storedObject.attributes.projectArea && this.storedObject.attributes.projectArea.id){ 
		storedProjectAreaItemId = this.storedObject.attributes.projectArea.id;
	}
	
	return storedProjectAreaItemId != currentProjectAreaItemId;
},
 _isLinkTypeEqual: function(linkType1, linkType2) {
        	return (linkType1.id == linkType2.id) &&
        	       ((linkType1.isSource == linkType2.isSource)  || (linkType1.isSymmetric == linkType2.isSymmetric))
},
_findLinkTypeLike: function(linkTypes, linkType) {
			for(var j=0; j< linkTypes.length; j++ ) {
				if(this._isLinkTypeEqual(linkTypes[j], linkType)) {
					return linkTypes[j];
				}
        	}
        	return null;
        },
        /*
         * Remove any changes to CALM links in the work item
         *  - For each backlink type, replace the o
         */
_revertBackLinkChanges: function(parms /* widgetCommands: */) {
        	var linkTypes = this.object.linkTypes;
        	if(linkTypes) {
        		for(var i=0;i<linkTypes.length; i++) {
        			if(BACK_LINK_TYPES_MAP[linkTypes[i].id] === true) {
        				var storedLinkType = this._findLinkTypeLike(this.storedObject.linkTypes, linkTypes[i]);
        				if(storedLinkType) {
        					this._setLinkTypeValue({ path: [ "linkTypes", linkTypes[i].id, "linkDTOs" ] , value: storedLinkType.linkDTOs, notifyEvenIfUnchanged: true} );        					
        				}
        			}
        		}
        	}
        	if(parms && parms.widgetCommands && parms.widgetCommands.updateLinks) {
        		var newUpdateLinks = [];
        		for(var i=0;i<newUpdateLinks.length;i++) {
        			var id = parms.widgetCommands.updateLinks[i].id;
        			if(!BACK_LINK_TYPES_MAP[id]) {
        				newUpdateLinks.push(parms.widgetCommands.updateLinks[i]);
        			}
        		}
        		parms.widgetCommands.updateLinks = newUpdateLinks;
        	}
        },
	/*
	 * Return an array of link types that match the parameters
	 */
	_filterLinkTypes: function(parms /* any properties of the link types */) {
		var result = parms.result || [];
		var linkTypes = this.object.linkTypes;
		if(!linkTypes) return result;

		for(var i=0;i<linkTypes.length;i++){
			var found;
			for(var key in parms) {
				found = false;
				if(linkTypes[i][key] == null) break; // no match
				if(parms[key] != linkTypes[i][key]) break; // no match
				found = true; // used when dropping out of the loop
			}
			if(found) {
				result.push(linkTypes[i]);
			}
		}
		return result;
	},
	_getLinkType: function(parms){
		if (!parms || !parms.path) 
			return null;
		if (!this.object.linkTypes) {
			this.object.linkTypes = [];
		}
		var path = this._getPath(parms.path);
		if (path.length == 1) {
			return this.object.linkTypes;
		}
	
		var result = null;
		/* find the referenced linktype */
		for (var i = 0; i < this.object.linkTypes.length; i++) {
			if (this.object.linkTypes[i].id == path[1]) {
					if(this.object.linkTypes[i].isSymmetric || parms.isSource == undefined || this.object.linkTypes[i].isSource == parms.isSource) {
					result = this.object.linkTypes[i].linkDTOs;
					if(!result && parms.defaultValue) {
						this.object.linkTypes[i].linkDTOs = parms.defaultValue;
						return this.object.linkTypes[i].linkDTOs;
					}
					break;					
				}
			}
		}
		if(!result && parms.defaultValue) {
			var type = {
				id: path[1],
				linkDTOs: parms.defaultValue
			}
			this.object.linkTypes.push(type);
			return type.linkDTOs;
		}

		return result;
	},
	setValue: function(parms /* caller, attributeId, path, savedAttributePath, notifyEvenIfUnchanged, value */){
		var path = this._getPath(parms.path);
		var isChanged = parms.notifyEvenIfUnchanged || false;
		if (path[0] == "linkTypes") {
			isChanged = this._setLinkTypeValue(parms);
			if(isChanged) this._checkForListeners(parms); // Call attribute listeners
			
		}
		else {
			// approval is a special case. Find the value in the approvals.approvals array
			if (path[0] == "approvals") {
				isChanged = this._setApprovalsValue(parms);
				this._checkForListeners(parms); // Call attribute listeners, always for approvals
			}
		 else {
			isChanged = this.inherited(arguments) || isChanged; 
		}}
		
		isChanged = isChanged && !parms.ignoreChange;
		
		if (isChanged || parms.forceNotify) {
			this._handleAttributeChange(parms.attributeId, path);
			this.notifyOfItemChanged(); // Publsh event
		} 
		if(parms.ignoreChange != true) {
			if(dojo.getObject("net.jazz.ajax.ui.PlatformUI") != null && net.jazz.ajax.ui.PlatformUI.getWorkbench() != null) {
				if(isChanged) {
					net.jazz.ajax.ui.PlatformUI.getWorkbench().setPageDirty("com.ibm.team.workitem", isChanged); //$NON-NLS-1$
				} else {		
					com.ibm.team.workitem.web.cache.internal.CacheUtil.setPageDirty();
				}
			}
		}
		
		return isChanged;
	},
	
	getAttribute: function(attributeId) {
		var attributeMap= this._getAttributeMap();
		return attributeMap[attributeId];
	},
	
	_handleAttributeChange: function(attributeId, path) {
		var attributes= this.findDependentAttributes(attributeId);
		if (attributes.length > 0) {
			for (var i= 0; i < attributes.length; i++) {
				this.__workItemAdapter.updateValue(attributes[i].getIdentifier());
				this.notifyListeners({
					path: attributes[i]._getPath(),
					event: EventTopics.ON_DEPENDENCY_CHANGE
				});
			}
		}
		
		/*
		 * If the work item is "active", e.g., in the cache, then validate the required properties
		 * Otherwise assume it is a working copy used in dialogs etc. where the validation is not needed
		 * (since the updater breaks if it isn't "in cache")
		 */
		if(!this.isNotCached) {			
			var updater= RequiredPropertiesUpdater.getRequiredPropertiesUpdater(this.__workItemAdapter.getProjectAreaItemId());
			var callback= dojo.hitch(this, this._handleRequiredAttributesChange);
			updater.handleAttributeChange(this.__workItemAdapter, attributeId, callback);
		}
	},
    _handleRequiredAttributesChange: function(/* String[] */ requiredAttributes) {
		if(!this.workItemSpec || !this.workItemSpec.editProps || !this.workItemSpec.editProps.requiredProperties) return;
		var existing= this.workItemSpec.editProps.requiredProperties;
		if (this._hasChanged(existing, requiredAttributes)) {
			this.workItemSpec.editProps.requiredProperties= requiredAttributes;
			dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_REQUIRED_PROPERTIES_CHANGED, [{ workingCopy: this}]);
		}
    },
    _hasChanged: function(a, b) {
    	if (a.length != b.length) {
    		return true;
    	}
		var setA= {};
		for (var i= 0; i < a.length; i++){
			setA[a[i]]= true;
		}
		for (var k= 0; k < b.length; k++){
			delete setA[b[k]];
		}
		return com.ibm.jdojo.lang.Runtime.getProperties(setA).length > 0;
    },
	/*
	 * Look through creator, owner, comments and subscribers to find the user, else return null
	 */
	findContributor: function(userId) {
		var user = this.getValue({path: "attributes.creator"});
		if(user && user.userId == userId) return user;
		var user = this.getValue({path: "attributes.owner"});
		if(user && user.userId == userId) return user;
		var comments = this.getValue({path: "attributes.internalComments.items"});
		if(comments) {
			for(var i=0;i<comments.length;i++) {
				if(comments[i].creator && comments[i].creator.userId == userId) return comments[i].creator;
			}
		}
		var subscribers = this.getValue({path: "attributes.internalSubscriptions.items"});
		if(subscribers) {
			for(var i=0;i<subscribers.length;i++) {
				if(subscribers[i].userId == userId) return subscribers[i];
			}
		}
		return null; // not found
	},
	
	findDependentAttributes: function(attributeId) {
		var dependents= new Array();
		var attributeMap= this._getAttributeMap();
		for (var id in attributeMap) {
			var attribute= attributeMap[id];
			if (attribute.dependsOn(attributeId)) {
				dependents.push(attribute);
			}
		}
		return dependents;
	},
	
	_getAttributeMap: function() {
		this._attributeMap = this._attributeMap || {}; // TODO: AttributeMap should not be null
		return this._attributeMap;
	},
	
	_validate: function(attributeId) {
		return this.__workItemAdapter.validate(attributeId);
	},
	
	notifyOfItemChanged: function() {
		dojo.publish(EventTopics.WORKITEM_CHANGED, [this])
	},
	_setLinkTypeValue: function(parms){ /* path: value: isSource: | returns isChanged */
		var path = this._getPath(parms.path);
		if (path.length == 1) {
			this.object.linkTypes = parms.value;
			this._checkForListeners(parms)
			return true;
		}
		if (!this.object.linkTypes) {
			this.object.linkTypes = [];
		}
		
		/* find the referenced linktype */
		var isFound = false;
		for (var i = 0; i < this.object.linkTypes.length; i++) {
			if(parms.isSource != null) {
				if (this.object.linkTypes[i].id == path[1] && this.object.linkTypes[i].isSource == parms.isSource ) {
					isFound = true;
					this.object.linkTypes[i].linkDTOs = parms.value || [];
					break;
				}				
			} else { // find the first in the list
				if (this.object.linkTypes[i].id == path[1]) {
					isFound = true;
					this.object.linkTypes[i].linkDTOs = parms.value || [];
					break;
				}
			}
		}
		if (!isFound) {
			var linkType = {
				id: path[1],
				linkDTOs: parms.value || []
			}
			this.object.linkTypes.push(linkType);
		}
		this._checkForListeners(parms);
		return true;
	},
	// An approval value is a single approval within an approval's approvals array
	_setApprovalsValue: function(parms){
		if (!this.object.approvals) {
			this.object.approvals = [];
		}

		if (!parms || !parms.path) 
			return false;
			
		var path = this._getPath(parms.path);
		if (path.length < 2) 
			return false;
			
		var isChanged;	
		if (this.object.approvals.length==0) {
			this.object.approvals = [parms.value];
			parms.value._approvalId = 0;
			isChanged = true;
		}
		else {
			var id = path[1];
			if (id < this.object.approvals.length) {
				this.object.approvals[id] = parms.value;
				isChanged = true;
			}
			else {
				// not found
				parms.value._approvalId = id;
				this.object.approvals.push(parms.value);
				isChanged = true;
			}
		}
		this._checkForListeners(parms);
		return isChanged;
	},
	// A work item was refreshed in a multi save so merge in the changes
	mergeResultDTO: function(resultDTO) {
		
	},
	getSaveCommands: function(allCommands){
		this._getApprovalSaveCommands(allCommands);
		this._getAttachmentSaveCommands(allCommands);
		this._getLinkTypeSaveCommands(allCommands);
//		this._getSubscriberSaveCommands(allCommands);
		return allCommands;
	},
	/*
	 * Add commands to the updateLinks commands
	 * Each command needs an attachment descriptor:
	 *      itemId: linkDTO.itemId
	 *      name: linkDTO.target.name
	 */
	_getAttachmentSaveCommands: function(allCommands){
	
		var attachmentDTOs = this.getValue({
			path: ["linkTypes", /* id */ "com.ibm.team.workitem.linktype.attachment", "linkDTOs"]
		});
		
		if (!attachmentDTOs) 
			return allCommands;
		
		var commands = [];
		for (var i = 0; i < attachmentDTOs.length; i++) {
			var attachmentDTO = attachmentDTOs[i];
			if (attachmentDTO._isNew) {
				var command = {
					cmd: ADD_LINK,
					type: ATTACHMENT_TYPE,
					itemId: attachmentDTO.itemId,
					name: attachmentDTO.target.name // Used in history pending changes
				}
				commands.push(command);
			}
			else { // Not new
				if (attachmentDTO.isRemoved) {
					var command = {
						cmd: REMOVE_LINK,
						type: ATTACHMENT_TYPE,
						itemId: attachmentDTO.itemId,
						name: attachmentDTO.target.name // Used in history pending changes
					}
					commands.push(command);
				}
			}
		}
		
		var updateLinks = allCommands[UPDATE_LINKS] ? allCommands[UPDATE_LINKS] : [];
		
		for (var i = 0; i < commands.length; i++) {
			updateLinks.push(commands[i]);
		}
		
		if (updateLinks.length > 0) {
			allCommands[UPDATE_LINKS] = updateLinks;
		}
		
		return allCommands;
	},
	/*
	 * Add values to the updateLinks commands
	 *
	 */
	_getLinkTypeSaveCommands: function(allCommands){
	
		var linkTypes = this.getValue({
			path: ["linkTypes"]
		});
		if (!linkTypes || linkTypes.length == 0) 
			return allCommands;
		
		var commands = [];
		for (var i = 0; i < linkTypes.length; i++) {
			this._getSaveCommandsForType(commands, linkTypes[i]);
		}
		
		var updateLinks = allCommands[UPDATE_LINKS] ? allCommands[UPDATE_LINKS] : [];
		
		for (var i = 0; i < commands.length; i++) {
			updateLinks.push(commands[i]);
		}
		
		if (commands.length > 0) {
			allCommands[UPDATE_LINKS] = updateLinks;
		}
		return allCommands;
	},
	_getSaveCommandsForType: function(commands, linkType){
		var linkDTOs = linkType.linkDTOs;
		if (!linkDTOs || linkDTOs.length == 0) 
			return;
		var linkDTO;
		for (var i = 0; i < linkDTOs.length; i++) {
			linkDTO = linkDTOs[i]
			if (linkDTO.id == "com.ibm.team.workitem.linktype.attachment") 
				continue;
			if (linkDTO._isNew || this.getId() < 0) {
				if (!linkDTO._isDeleted) {
					// For a new link, consolidate all commands into one command
					var command = {
						cmd: ADD_LINK,
						type: linkType.id,
						end: linkType.isSource ? "source" : "target",
						name: linkType.displayName || "link"// Used in history pending commands
					}
					if (linkDTO.itemId) 
						command.itemId = linkDTO.itemId;
					if (linkDTO.url) 
						command.uri = linkDTO.url;
					if (linkDTO.comment) 
						command.comment = linkDTO.comment;
					if (linkDTO.target && linkDTO._updateTarget)
						command.item = linkDTO.target;
					
					commands.push(command);
				}
			}
			else { // Not new
				if (linkDTO._isDeleted) {
					var command = {
						cmd: REMOVE_LINK,
						type: linkType.id,
						end: linkType.isSource ? "source" : "target",
						name: linkType.displayName || "link"// Used in history pending commands
					}
					if (linkDTO.itemId) 
						command.itemId = linkDTO.itemId;
					if (linkDTO.url) 
						command.uri = linkDTO.url;
					
					commands.push(command);
					
				} else if (linkDTO._isUpdated) {
					var command = {
						cmd: UPDATE_LINK,
						type: linkType.id,
						end: linkType.isSource ? "source" : "target",
						name: linkType.displayName || "link"// Used in history pending commands
					}
					if (linkDTO.itemId) 
						command.itemId = linkDTO.itemId;
					if (linkDTO.url) 
						command.uri = linkDTO.url;
					if (linkDTO.target && linkDTO._updateTarget)
						command.item = linkDTO.target;

					commands.push(command);
				}
			}
		}
	},
	/*
	 * Return an array  of commands. See work item 40464.
	 *
	 * Command Format is:
	 *
	 *      Create a new approval request for a specific user
	 *		var approvalType= ... // e.g. getApprovalTypes()[0].id
	 *		var contributorItemId= ...
	 *		var command= { cmd: "createApproval", param: { type: approvalType, name: "M5 Approval", approvers: [ { user: contributorItemId } ] }};
	 *
	 *		Update an approval descriptor with another name and dueDate
	 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var date= ...
	 *		var command= { id: approvalId, cmd: "updateApproval", param: { name: "New name", dueDate: date.getTime() } };
	 *
	 *		Add a user to an existing approval request
	 *		var contributorItemId= ...
	 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId } };
	 *
	 *		Change an approval state
	 *		var contributorItemId= ...
	 *		var state= ... // e.g. getApprovalStates()[0].id
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId, state: state } };
	 *
	 *		Remove an approver
	 *		var contributorItemId= ...
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "removeApprover", param: { user: contributorItemId } };
	 *
	 *		Remove an approval descriptor
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "removeApproval" };
	 *
	 *    A removed approval stays on the approvals list but is not shown, it returns its own
	 *    removal command
	 *
	 */
	_getApprovalSaveCommands: function(allCommands){
		if (!this.object.approvals) 
			return allCommands;
		var commands = [];
		for (var i = 0; i < this.object.approvals.length; i++) {
			var approval = this.object.approvals[i];
			if (!approval) 
				continue;
			if (approval._isNew) {
				if (!approval._isDeleted) {
					// For a new approval, consolidate all commands into one command
					var command = {
						cmd: "createApproval",
						param: {
							type: approval.approvalType,
							name: approval.name
						}
					}
					if (approval.dueDate) {
						command.param.dueDate = approval.dueDate;
					}
					if (approval.approvals) {
						command.param.approvers = [];
						for (var j = 0; j < approval.approvals.length; j++) {
							subapproval = approval.approvals[j];
							if (subapproval && subapproval.approver) {
								var approverCommand = {
									user: subapproval.approver.itemId,
									state: subapproval.state
								}
								command.param.approvers.push(approverCommand)
							}
						}
					}
					commands.push(command);
				}
			}
			else {
				// For approval updates, send all the commands normally
				if (approval._commands) {
					for (var j = 0; j < approval._commands.length; j++) {
						if (approval._commands[j]) {
							commands.push(approval._commands[j]);
						}
					}
				}
				if (approval.approvals) {
					for (var j = 0; j < approval.approvals.length; j++) {
						var subapproval = approval.approvals[j];
						if (subapproval && subapproval.approver && subapproval.approver._command) {
							commands.push(subapproval.approver._command);
						}
					}
				}
				
			}
		}
		if (commands.length > 0) {
			allCommands.updateApprovals = commands;
		}
		return allCommands;
	},
	getFormElementValues: function(formElements, skipSubscribers){
		var self = this;
		for (var key in this.changedAttributes) {
			if(key == "internalComments") continue; // special case
			var savedAttributePath =  this.changedAttributes[key].savedAttributePath || this.changedAttributes[key].path || this.changedAttributePaths[key];
			self._addFormElementValues({
				path: savedAttributePath,
				attributeId: key,
				formElements: formElements
			})
		}
		
		// If current user made a comment, subscribe the current user
		var addCommentString = formElements[this.SYNTHETIC_COMMENT_ID];
		if (!skipSubscribers && !isGuest() && addCommentString && addCommentString.length > 0) {
			var currentUser = getAuthenticatedContributor();
			if (currentUser.itemId != this.object.attributes.owner.itemId &&
			currentUser.itemId != this.object.attributes.creator.itemId) {
				this.addItems({ items: [currentUser], attributeId: SUBSCRIBERS_ID } );
			}
		}
	},
	/*
	 * Get the form element association for an attribute path
	 */
	_addFormElementValues: function(parms /* path, formElements, attributeId */){
		if (this.changedAttributes[parms.attributeId]) {
			if(parms.path[parms.path.length-1] == "items") {
				parms.formElements[parms.attributeId] = this._getItemsFormElements(this._getAttribute(this._getPath(parms.path)));
			} else {
				parms.formElements[parms.attributeId] = this._getAttribute(this._getPath(parms.path));
			}
		}
	},
	clearRetainedState: function(ignoreComment) {
		if(ignoreComment && this.object && this.object.attributes && this.object.attributes[this.SYNTHETIC_COMMENT_ID]) {
			delete this.object.attributes[this.SYNTHETIC_COMMENT_ID];
		}
		var oldComment;
		if(!ignoreComment && this.changedAttributes && this.changedAttributes[this.SYNTHETIC_COMMENT_ID]) {
			oldComment = this.changedAttributes[this.SYNTHETIC_COMMENT_ID];
		}
		this.changedAttributes = {};
		this.changedAttributePaths = {};
		if(oldComment) {
			this.changedAttributes[this.SYNTHETIC_COMMENT_ID] = oldComment;
			var listenerParms = {
				path: ["attributes", this.SYNTHETIC_COMMENT_ID],
				event: "onchange",
				attributeId: this.SYNTHETIC_COMMENT_ID
			};
			this.notifyListeners(listenerParms);
		}
	},
	getRetainedState: function(retainedState){
		if (this.changedAttributes && this.changedAttributes[this.SYNTHETIC_COMMENT_ID]) {
			retainedState[this.SYNTHETIC_COMMENT_ID] = this.changedAttributes[this.SYNTHETIC_COMMENT_ID].value;
		}
	},
	setRetainedState: function(retainedState){
		if (retainedState && retainedState.internalComments != null) {
			this.setValue({
				path: ["attributes", this.SYNTHETIC_COMMENT_ID, "content"],
				attributeId: this.SYNTHETIC_COMMENT_ID,
				isInitialization: false,
				value: retainedState.internalComments
			})
		}
	},
	/*
	 * This method is copied from WorkItemProxyFactory to avoid circular dependencies
	 */
	_getNewWorkItemProxy: function(parms /* id, createIfNeeded */) {
		if(parms.isShownInRecentlyViewed != false) {
		   parms.isShownInRecentlyViewed = true;
		}
		var id = parms.id || lastId--;
		var workItemProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
		if(workItemProxy || !parms.createIfNeeded) {
			if(workItemProxy.isShownInRecentlyViewed != true) {
				workItemProxy.isShownInRecentlyViewed = parms.isShownInRecentlyViewed;
			} 
			return workItemProxy;
		}
		
		/* if ID is -1 then it is a new item and should be converted to a unique id */
		if(id == -1) {
			id = -1 * Date.parse(new  Date().toString())
		}
		
		var object= {
			attributes: {
				id: {
					id: id
				}
			}
		}
		workItemProxy =  new com.ibm.team.workitem.web.cache.internal.WorkItemProxy({ object: object, isFetched: false}); // empty object initially
		com.ibm.team.workitem.web.cache.internal.Cache.addItem(id, workItemProxy);
		return workItemProxy;
	},
	getJfsResourceUri: function(parms /* locationUri, userid */) {
		if(!parms.uri) return parms.uri;
		var prefix = parms.uri.split("/resource");
		if(prefix.length == 0) return parms.uri; // give up
		prefix = prefix[0]; // http://example.com/jazz
		if(parms.userId) {
			return prefix + USERS_URI_STUB + parms.userId;
		}
		return parms.uri;
	},
	applyResourceLinkToAnchor: function(parms, anchor) {
// Parms:
//			uri: resource's href for the hover,
//			presentationUri: anchor's href for the click,
//			label: label
//			retainLinkText: true/false, Keep the anchor's link text (false), or replace it with the label
//          userId: if contributor, use a JFS resource link based on userid
		if(parms.presentationUri) {
			if (this._isLocalUrl(parms.presentationUri)) {
				var m = /com.ibm.team.scm.ChangeSet\/([A-Za-z0-9_\-]+)\?(.*)/.exec(parms.presentationUri);
				if (m) {
					var locationParams= dojo.queryToObject(m[2]);
					var presentationParams= {
						workspaceItemId: locationParams.workspace || locationParams.Workspace,
						changeSetItemId: m[1],
						projectAreaItemId: locationParams.projectAreaId
					}
					parms.presentationUri = "#action=com.ibm.team.scm.viewChangeSet&" + dojo.objectToQuery(presentationParams);
				} else {
					parms.presentationUri = com.ibm.team.workitem.web.ui.internal.module.WindowContext._convertOidUrlToLocationUrl(parms.presentationUri);
				}
			}
		} 
		if(parms.userId) { // create a jfs uri
			parms.uri = this.getJfsResourceUri(parms); 
		}
		var resourceLinkArgs= parms;
		 return new jazz.ui.ResourceLink(resourceLinkArgs, anchor);
	},
	_isLocalUrl: function(uri) {
		var root= window.location.protocol + "//" + window.location.host + net.jazz.ajax._contextRoot + "/";
		return uri.indexOf(root) == 0;
	},
	getEditPropsAllValues: function() {
	 	if(!this.workItemSpec || !this.workItemSpec.editProps || !this.workItemSpec.editProps.allValues) return [];
		return this.workItemSpec.editProps.allValues;
	},
	getEditableLinkTypes: function() {
		return this.workItemSpec ? this.workItemSpec.editableLinkTypes : []; 
	},
	setEditableLinkTypes: function(linkTypes) {
			linkTypes.sort(function(a, b) {
				try{
					if(!a || !a.displayName || !b || !b.displayName) return 1;
					return a.displayName.toLowerCase() > b.displayName.toLowerCase() ? -1 : 1;
				} catch(e) { 
					return -1 
				}
			});
		this.workItemSpec = this.workItemSpec || {};
		this.workItemSpec.editableLinkTypes = linkTypes;
	},
	/*
	 * This is used in WorkItemProxyClient to filter work items that have links pending
	 */
	setHasNewLinksPending: function(aBoolean) {
		this.hasNewLinksPending = aBoolean
	},
	addLinkTarget: function(parms /* addLinkType addLinkedProjectAreaItemId addLinkedItemId addLinkIsSource*/) {
		parms.linkIsSource = parms.addLinkIsSource == "true";
		var linkType = this._getLinkType({path: [ "linkTypes",  parms.addLinkType], isSource: parms.linkIsSource });
		/*
		 * If the link type does not exist we need to get it from the cache
		 */
		if(!linkType ) {
			var self = this;
			var serviceObject = {
				onSuccess: function(linkTypes) {
					for(var i=0;i<linkTypes.length;i++) {
						if(linkTypes[i].id == parms.addLinkType && (linkTypes[i].isSymmetric || linkTypes[i].isSource == parms.linkIsSource)) {
							parms.linkType = linkTypes[i];
							break;
						}
					}
					self._addLinkTarget2(parms);
				}
			}
			var args = {
				caller: serviceObject,
				onSuccess: "onSuccess"
			}
			args.isNew = true;
		    this.fetchEditableLinkTypes(args /* caller, onSuccess */);
		} else {
			parms.linkType = linkType;
			this._addLinkTarget2(parms)
		}
	},
	_addLinkTarget2: function (parms) {
		if(!parms.linkType) {
			console.debug("WorkItemProxy: Could not find link type:" + parms.linkType);// %NON-NLS-1%
			return;
		}
		if(parms.isNew) {
			/*
			 * See if it was added since we asked for a server copy
			 */
			var currentLinkType = this._getLinkType({path: [ "linkTypes",  parms.linkType], isSource: parms.linkIsSource });
			if(currentLinkType) {
			    parms.linkType = currentLinkType;
			} else {				
				this.object.linkTypes = this.object.linkTypes || [];
				this.object.linkTypes.push(parms.linkType)
			}
		}
		var sourceWorkItem = com.ibm.team.workitem.web.cache.internal.Cache.getItemByItemId(parms.addLinkedItemId);
		if(!sourceWorkItem) {
			console.debug("WorkItemProxy: Could not find source for new link item " + parms.linkType);// %NON-NLS-1%
			return;
		}
		var linkDTO = {
			_isNew: true,
			id: sourceWorkItem.getId(),
			itemId: sourceWorkItem.getItemId(),
			comment: sourceWorkItem.getSummaryLabel(),
			linkTypeId: parms.linkTypeId,
			linkTypeIsSource: parms.linkIsSource,
			linkTypeIsUserDeletable: true,
			target: {
				attributes: [
					{
						key: "id", // %NON-NLS-1%
						value: {
							id: sourceWorkItem.getId()
						}
					}
				]
			}
		}
		var linkType = parms.linkType;
		linkType.linkDTOs = linkType.linkDTOs || [];
		linkType.linkDTOs.push(linkDTO);
		
		var linkTypes = this.getValue({
				path: ["linkTypes"],
				defaultValue: []
			});

		this.setValue({
			path: ["linkTypes"],
			value: linkTypes
			});		
	},
	__sential: true
}
);

})();
