/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient");
/*
 * API (not yet finalized)
 * 
 * fetchWorkItem(...
 * fetchWorkItemEditProperties(...
 * 
 * storeWorkItem(...
 * 
 * isGuest()
 * 
 * fetchPresentations({
 *	  onSuccess: callback,
 *	  isInit: isInit,
 * 	  operationMsg: operationMsg
 *	});
 */
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxyClient");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.CacheUtil"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.client.internal.Attribute");
dojo.require("com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory");
dojo.require("com.ibm.team.workitem.web.client.dtos.DTOUtil"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.core.loader");
//dojo.requirX("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); // do not require... circular dependecy
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");


dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function(){
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
    var WorkItemUtil = com.ibm.team.workitem.web.client.util;
    var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var checkNonNegative = net.jazz.ajax.toolbox.lang.checkNonNegative;
	var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
	var Attribute= com.ibm.team.workitem.web.client.internal.Attribute;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	// Important: keep in sync with com.ibm.team.calm.foundation.common.linking.CALMBackLinkUpdater.PROBLEM_MODIFYING_CALM_LINK
	var PROBLEM_MODIFYING_CALM_LINK= 0x50000;
	var SAVE_PARAMETER_UPDATE_BACKLINKS= "com.ibm.team.workitem.common.internal.updateBacklinks"; //$NON-NLS-1$
	var SAVE_PARAMETER_IGNORE_BACKLINK_PROBLEMS= "com.ibm.team.workitem.common.internal.ignoreBacklinkProblems"; //$NON-NLS-1$
	
    
    /*
     * The KNOWN_ATTRIBUTES describe which work item properties to return.
     * If false, skip the property.
     * For properties that are returned, the map identifes the attribute of the property
     * If "true" then just return the property itself, otherwise return the named attribute.
     *
     * Example: duration returns the value, foundIn returns workItem.foundIn.id, creationDate is skipped
     *
     * IE, all attributes other than these are added to form elements
     * This matches some login in the REST service
     */
    var KNOWN_ATTRIBUTES = { //$NON-NLS this list does not need to be translated
        category: "id",
        correctedEstimate: "label",
        creationDate: false, // immutable (id)
        creator: false, // immutable
// 		KNOWN_ATTRIBUTES[SYNTHETIC_COMMENT_ID] = "content"; // "comment" deferred to initialization
        description: "content",
        dueDate: "id",
        duration: "label",
        foundIn: "id",
        id: false, // immutable
        internalComments: "content",
        internalPriority: "id",
        internalResolution: "id",
        internalSeverity: "id",
        internalState: "id",
        internalSubscriptions: "items",
		internalTags: "content", 
        modified: false, // immutable
        modifiedBy: false, // immutable
        owner: "itemId",
        projectArea: "id",
        resolutionDate: false, // immutable
        resolver: false, // immutable	
        startDate: "id", // immutable
        summary: "content",
        tags: "content",
        target: "id",
        teamArea: "id",
        timeSpent: "label",
        "com.ibm.team.apt.estimate.minimal": "label", // HACK see defect 140772
        "com.ibm.team.apt.estimate.maximal": "label", // HACK see defect 140772
        workItemType: false // special case
    };
    var PART_TO_PATH_MAP = {
		"com.ibm.team.workitem.kind.internal.approvals": "approvals",
		 "com.ibm.team.workitem.kind.internal.attachments": "attachments", // verify
		 "com.ibm.team.workitem.kind.internal.discussion": "comments", // verify
		 "com.ibm.team.workitem.kind.internal.history": "history", //verify
		 "com.ibm.team.workitem.kind.internal.links": "links", // verify
		 "com.ibm.team.workitem.kind.internal.teamarea": "attributes.teamArea.id", // verify
		 "com.ibm.team.workitem.kind.workflow.resolution": "attributes.internalResolution.id",
		 "com.ibm.team.workitem.kind.workflow.state": "attributes.internalState.id"
	};
   var PART_TO_ATTRIBUTE_MAP = {
		"com.ibm.team.workitem.kind.internal.approvals": "approvals",
		 "com.ibm.team.workitem.kind.internal.attachments": "attachments", // verify
		 "com.ibm.team.workitem.kind.internal.discussion": "comments", // verify
		 "com.ibm.team.workitem.kind.internal.history": "history", //verify
		 "com.ibm.team.workitem.kind.internal.links": "links", // verify
		 "com.ibm.team.workitem.kind.internal.teamarea": "teamArea", // verify
		 "com.ibm.team.workitem.kind.workflow.resolution": "internalResolution",
		 "com.ibm.team.workitem.kind.workflow.state": "internalState"
	};
    /*
     * DERIVED_ATTRIBUTES (always true) show attributes of work items
     * that are derived and do not need to be added to form elements
     */
    var DERIVED_ATTRIBUTES = {
        stateId: true,
        //			comment: true,
        intervalItemId: true
        //			type: true,
    };
    
    // Key map translates some key changes
    var ATTRIBUTE_KEY_MAP = {        //			severity: "internalSeverity",
        //			priority: "internalPriority",
        //			resolution: "internalResolution",
    };

    
dojo.declare("com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient", [com.ibm.team.workitem.web.cache.internal.ItemProxyClient], {
		SYNTHETIC_COMMENT_ID: "internal_SYNTHETIC_Comment",
        constructor: function(parms){
			this.storedObject = parms.storedObject || {};
			this._operationCount = 0;
			this.object = parms.object;
			this.isFetched = parms.isFetched;
            this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
            this.isDirty = false;
        },
       destroy: function(){
       	 if(this._subscribeToWorkItemRefreshedHandle) dojo.unsubscribe(this._subscribeToWorkItemRefreshedHandle);
       	 if(this._subscribeToWorkItemEditPropsChangedHandle) dojo.unsubscribe(this._subscribeToWorkItemEditPropsChangedHandle);
  		 this._priorFetchPresentationProps = null;
  		 this._priorFetchInlinePresentationProps = null;
		 this.inherited(arguments);
      },
/*
 * Initialize: fetch work item and presentations
 * Called on initial load and when the editor is refreshed
 */
		initialize: function(parms /* onSuccess, onError, type, id ||tempId, [tab], [draftId]  */) {
			KNOWN_ATTRIBUTES[this.SYNTHETIC_COMMENT_ID] = "content"; // deferred 
			this.addListener({
				path: "attributes.workItemType",
				event: "onchange",
				listener: this,
				functionName: "_handleWorkItemTypeChange"
			});	
			this.getUrl(); // initialize the url
			
			parms.callbackOnSuccess = parms.onSuccess;
			if(parms.onError) {
				parms.callbackOnError = parms.onError;
			}
			parms.fetchEditorPresentation = parms.fetchEditorPresentation === true;
			parms.fetchInlineEditorPresentation = parms.fetchInlineEditorPresentation === true;
			
			this.isInitialized = false;
			parms.onError = dojo.hitch(this,this._initializeWorkItemError);
			parms.onSuccess = dojo.hitch(this,this._initializeWorkItemComplete);
			if(!this.isFetched || (!parms.isNew && !this.isFullFetched && parms.isForcedFullFetch)) {
				this.isFetched = false;
			}
			if(!this.isFetched || parms.isRefresh) {
				this.fetchWorkItem(parms);			
			} else {
				parms.fetchWorkItem = false;
				this.fetchPresentations(parms);
			}
			
			if(!this._subscribeToWorkItemRefreshedHandle) {
				this._subscribeToWorkItemRefreshedHandle = dojo.subscribe(EventTopics.WORKITEM_REFRESHED, dojo.hitch(this, this._handleWorkItemRefreshed));	
			}
			if(!this._subscribeToWorkItemEditPropsChangedHandle) {
				this._subscribeToWorkItemEditPropsChangedHandle = dojo.subscribe(EventTopics.WORKITEM_EDITABLE_PROPERTIES_CHANGED, dojo.hitch(this, this._newProxyEditPropsChanged));	
			}


		},
		isFetchedWithPresentation: function(parms) {
			if(!parms || !this.isFetched || !this.isFullFetched) return false;
			if(parms.fetchEditorPresentation) {
				return this.workItemSpec && this.workItemSpec.presentationProps
			}
			if(parms.fetchInlineEditorPresentation) {
				return this.workItemSpec && this.workItemSpec.inlinePresentationProps
			}
		},
		_initializeWorkItemComplete: function(parms) {
			this.isInitialized = true;
			if(dojo.isFunction(parms.callbackOnSuccess)) {
				parms.callbackOnSuccess(parms);
			}
		},
		_initializeWorkItemError: function(error, parms) {
			parms = parms || {};
			var msg = this.messages.errorOpeningWorkItem;
			if(parms.error && parms.error.message ) {
				msg = msg + "\n" +parms.error.message;
			}
			console.debug(msg);
			if(parms.callbackOnError) {
				parms.callbackOnError(parms);
			}
		},
		/*
		 * A work item was saved or refreshed...
		 * If the workitem changed it's links... 
		 * 	- if this work item is a target of the new links, then we need to merge repository changes
		 */
		_handleWorkItemRefreshed: function(workItem) {
			if(!workItem || workItem.getItemId() == this.getItemId()) return; // skip current work item
			if(!this.isFullFetched || !workItem.isFullFetched) return; // optimization: Don't check unless editor is being shown
			
			/*
			 * If that work item has a link to this work item
			 * but this work item does not have a link to the other work item,
			 * or vice versa
			 * then the link was added or deleted and we need to merge the changes
			 */			
			if(!this.doLinksMatch(workItem)) {
				this.mergeRepositoryChanges({ isKeepingChangedValues: true });
			}			

		},
		/*
		 * When the type changes:
		 * If there is not a priorType proxy for the new type:  
		 * 	1) If there is no priorType proxy, fetch a new work item proxy of the new type and initialize
		 *     This gets the attributes, editableProperties and presentationProperties
		 *  2) Copy this proxy as CurrentProxy. Save it as a priorTypeProxy for the current type so that if the user 
		 *     changes the type back we can recover the original attribute shape and values
		 *  3) NewProxy = the proxy from step 1 or the priorType proxy 
		 *     a) Replace the attributes, editProperties, and presentationProperties in this proxy from NewProxy
		 *     b) Replace any any attribute values from CurrentProxy including changedAttributes and changedAttributePaths
		 *  4) Notify listeners that the type has changed 
		 */
		_handleWorkItemTypeChange: function(parms) {
			this.setIsDirty(true);
			var newTypeId = parms.value ? parms.value.id || "" : "";			
			this._priorTypeProxyDataMap = this._priorTypeProxyDataMap || {};
			if(this._priorTypeProxyDataMap[newTypeId]) {
				parms.priorTypeProxy = this._priorTypeProxyDataMap[newTypeId];
				this._handleWorkItemTypeChangePart2(parms);
			} else {			
				newTypeProxy = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({ id: -1, createIfNeeded: true, incrementUseCount: false, doNotCache: true});
				parms.newTypeProxy = newTypeProxy;
				var initParams= {
					id: -1,
					type: newTypeId, 
					fetchEditorPresentation: true,
					onSuccess: dojo.hitch(this,	function() {
							this._handleWorkItemTypeChangePart2(parms);
						}),
					onError: function(e) {
							e = e || {};
							msg = e.message || "";
							console.debug("*** ERROR fetching status in ListWidget: "+msg);
						}
				};
				if (this.moveProjectAreaItemId) {
					initParams.projectAreaItemId= this.moveProjectAreaItemId;
				}
				newTypeProxy._instigator = this;
				newTypeProxy.initialize(initParams);
			}
 

		},
		/*
		 * Parms has either newTypeProxy or priorTypeProxy
		 */
		_handleWorkItemTypeChangePart2: function(parms){
			var currentTypeId = this._lastSetType; 
			if(!currentTypeId) { 
				if(this.storedObject && this.storedObject.attributes && this.storedObject.attributes.workItemType) {
					currentTypeId = this.storedObject.attributes.workItemType.id;
				} else {
					console.log("[WorkItemProxyClient] Missing storedObject attribute workItemType");
					currentTypeId = "unknown"; // $NON-NLS-1 should never happen
				}
			}

			if(this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.value) {
				this._lastSetType = this.object.attributes.workItemType.value.id;
			} else {
				console.log("[WorkItemProxyClient] Missing object attribute workItemType");
				this._lastSetType = "unknown"; // $NON-NLS-1 should never happen
			}
			
			var currentProxyData = {};
			currentProxyData.changedAttributes = {};
			currentProxyData.changedAttributePaths = {};
			currentProxyData.object =  {};
			currentProxyData.object.attributes =  {};
			currentProxyData.workItemSpec =  {};
			this.object = this.object || {};
			this.object.attributes = this.object.attributes || {};
			this.workItemSpec = this.workItemSpec || {};
			
			currentProxyData.changedAttributes = dojo.clone(this.changedAttributes);
			currentProxyData.changedAttributePaths = dojo.clone(this.changedAttributePaths);
			currentProxyData.object.attributes = dojo.clone(this.object.attributes);
			currentProxyData.workItemSpec = dojo.clone(this.workItemSpec);
			this._priorTypeProxyDataMap[currentTypeId] = currentProxyData;

			/*
			 *  Get the attributes from either the priorTypeProxy (can have changed attributes) or newTypeProxy (fresh)
			 * 
			 */			
			var newProxy = parms.newTypeProxy || parms.priorTypeProxy;
			newProxy.object = newProxy.object || {};
			newProxy.object.attributes = newProxy.object.attributes || {};

 			this.object.attributes = newProxy.object.attributes;
			this.changedAttributes = newProxy.changedAttributes || {};
			this.changedAttributePaths = newProxy.changedAttributePaths || {};
			
			this.workItemSpec = newProxy.workItemSpec || {};
			 
			for(var key in currentProxyData.changedAttributes) {
				// Only copy in the current attributes if they exist in the new proxy attributes
			 	if(newProxy.object.attributes[key]) {
			 		this.changedAttributes[key] = currentProxyData.changedAttributes[key];
			 		this.changedAttributePaths[key] = currentProxyData.changedAttributePaths[key];
			 	}
			 }
			for(var key in currentProxyData.object.attributes) {
				// Only copy in the current attributes if they exist in the new proxy attributes
			 	if(newProxy.object.attributes[key]) {
			 		this.object.attributes[key] = currentProxyData.object.attributes[key];
			 	}
			 }

			this.workItemSpec.lastLoadedProjectAreaItemId = newProxy.getProjectAreaItemId();			 			 
			this.workItemSpec.lastLoadedTypeId = this._lastSetType;
			this.isInTypeChange = true; // notify widgets to reload possible values
       		dojo.publish(EventTopics.WORKITEM_TYPE_CHANGED, [{
				workItemProxy: this
			}]);
			this.refreshObject({justNotify: true});
			this.isInTypeChange = false;
		},
		_newProxyEditPropsChanged: function(parms) {
			if(!parms || !parms.workingCopy || parms.workingCopy._instigator != this) return;
			if(!parms.workingCopy.workItemSpec) return;
			this.workItemSpec = this.workItemSpec || {};
			this.workItemSpec.editProps = parms.workingCopy.workItemSpec.editProps || {};
		  	dojo.publish(EventTopics.WORKITEM_REQUIRED_PROPERTIES_CHANGED, [{ workingCopy: this } ]);
		  	dojo.publish(EventTopics.WORKITEM_EDITABLE_PROPERTIES_CHANGED, [{ workingCopy: this } ]);
		},
		getLocationUri: function() {
			return this.object.locationUri;
		},
		getWorkItemSpec: function() {
			if(this.workItemSpec) return this.workItemSpec;
			this.workItemSpec = {};
			return this.workItemSpec;
		},
		// If the editor sets the url, use that instead for new work items
		setEditorUrl: function(url) {
			this._editorUrl = url;
		},
		// Track the tab on the editor, for the url construciton
		setEditorTab: function(tabName) {
			this._editorTab = tabName;
			this.url = null;
		},
		getUrl: function(parms) {
			var reset = parms != null || (parms && parms.isReset ==true); // TODO: remove obsolete "true" parameter
			if(!this.isNew() && this.url && this.url.indexOf("newWorkItem")) {
				reset = true;
			}
			if(!reset && this.url) return this.url;
			if(this.isNewWorkItem()) {
//				if(this._editorUrl) return this._editorUrl;
				this.url = WorkItemUtil.getNewWorkItemUri(Math.abs(this.getId(), this._editorTab));
			}  else {
				this.url = WorkItemUtil.getViewWorkItemUri(this.getId(), this._editorTab);
			}
			return this.url;		
		},
		/* API */
        fetchWorkItem: function(parms /* onSuccess, onError, isInit, operationMsg, id, projectArea, isForced, callback, isInit */){
            checkNotNull(parms.onSuccess, "onSuccess");
			var id = parms.id || parms.tempId ;
			if(!id) {
				id = this.getId();
			}
            checkNotNull(id, "id-2");
			
            var id = parseInt(id, 10);
			if(parms.isRefresh) {
				this.objectBeforeMove = false;	
				this.infos = [];
				this.moveProjectAreaItemId = false;
			}

            parms.isForced = false; // Avoid looping after forced fetch
			parms.url = window.location.href; // remember the url in case it change while loading
			
            var operation = this.createOperation({
				message: parms.operationMsg || this.messages.busyLoading,
				id: "proxy/fetchExistingWorkItem",
				isModal: this.isFetched, // only modal on refresh and save/load operations
				dialogTitle: this.messages.busyProgress
			});
            this.startOperation(operation);
            var self = this;
			
			// Comment is a special attribute that is used to keep the "unsaved" new comment.
			// It is retained between refreshes so a refresh does not destroy a new comment in progress
			var retainedComment;
			if(this.object && this.object.attributes && this.object.attributes[this.SYNTHETIC_COMMENT_ID]) {
				retainedComment = this.object.attributes[this.SYNTHETIC_COMMENT_ID];	
			}
			
            var serviceObject = {
                success: function(workItem){
					if(self.isFullFetched  && !parms.isRefresh) {
						if(self._isFetchComplete(parms)) {
							if (operation) {
	                        	self.finishOperation(operation);
	                    	}
		                    if(parms["onSuccess"]) {
        		            	parms["onSuccess"](parms);
		                    }
						} else {
							parms.operation = operation;
							/*
							 * fetchPresentations invokes _fetchEditableProperties invokes parms.onSuccess
							 */
    	                    self.fetchPresentations(parms);
        	                parms.fetchAttributeDescriptors = true;
            	            self._fetchAttributeDescriptors(parms);
            	         }
						return; // already fetched
					}                	
                	
                    if (workItem) {
                    	/*
                    	 * If it was partially fetched, and is now fully fetched...
                    	 */
                    	 var isMerging = false;
                    	 if(!self.isFullFetched && parms.isForcedFullFetch) {                    	 	
	   						/*
							 * If the state of the work item on the server has changed
							 */
							var isMerging = self.isPartialFetch;
							if(self.isPartialFetch && parms.isForcedFullFetch && self.object && self.object.stateId && workItem.stateId != self.object.stateId && parms.onStateHasChanged) {
								// The above may change the page location (e.g., return to the welcome page if the editor is cancelled)
								var doContinue = parms.onStateHasChanged();
								// If the state change handler chooses to continue, then continue as a full replace
								if(doContinue) {
									isMerging = false;
								} else {
									return; // Page is changing location anyway so bail out
								}
							}
							
							if(isMerging) {
								self.mergeWorkItem(workItem);
							}            	 	
                    	 }
                    	
						if (id && workItem.attributes && workItem.attributes.id != null && workItem.attributes.id.id == -1) {
							workItem.attributes.id.id = id;
						}
						if (parms.draftId) {
							workItem._draftId = parms.draftId;
							self.setIsDirty(true);
						}
	                	self.isFullFetched = true;	                	
						self.isFetched = true;
						if(!isMerging) {						
							self.object = workItem;
						}
						self.storedObject = dojo.clone(workItem); // Same as object if not merged
						/*
						 * The linkTypes linkDTOs are not cloned, so clone them individually
						 */
						if(self.storedObject.linkTypes) {
							for(var i=0;i<self.storedObject.linkTypes.length;i++) {
								if(self.storedObject.linkTypes[i]) {
									var linkDTOs = self.storedObject.linkTypes[i].linkDTOs;
									if(linkDTOs) {
										for(var j=0;j<linkDTOs.length;j++) {
											linkDTOs[j] = dojo.clone(linkDTOs[j]);
										}
									}
								}
								
							}
						}
						if(retainedComment) {
							if(!self.object.attributes) {
								self.object.attributes = {};
							}
							self.object.attributes[self.SYNTHETIC_COMMENT_ID] = retainedComment;
						}
                        self._promoteAttributes(self.object);
						parms.operation = operation;
                        parms.fetchAttributeDescriptors = true;
                        self.fetchPresentations(parms);
                    }
					/*
					 * This path is taken when an invalid work item id is entered in the search box
					 */
                    else {
                    	if (operation) {
                        	self.finishOperation(operation);
                    	}
						self.decrementUseCount();
                        self.handleError({ 
							message: self.messages.errorCouldNotLoadWorkItem
						});
						self.destroy();
                    }
                },
                error: function(error){
					self.isFullFetched = false; // fail
					var args = parms;
					args.error = error; 
					args.message = dojo.string.substitute(self.messages.errorOpeningWorkItemWithMessage, [error.message || ""]);
					console.debug(args.message);
					if(error.exception) {
						console.debug(error.exception);
					}
                    self.handleError(args);
                    if (operation) {
                        self.finishOperation(operation);
                    }
	                 if (dojo.isFunction( parms.onError)) {
						parms.onError(error, args);
					}
                }
            };
            
            var responseHandler = new ServiceResponseHandler(serviceObject, "success", "error");
                
			var args = {
                includeHistory: false
            }
            
            if (parms.draftId)
            	args.draftId= parms.draftId;
            	
			if(id > 0) {
				args.id = id;
			} else {
				args.newWorkItem = true;
				args.typeId = parms.type;
			}
			if (parms.projectAreaItemId) {
				args.projectAreaItemId= parms.projectAreaItemId;
			}
			this._setArgsProjectAreaItemId(args);
			
			if(this.isFetched && this.object && this.getId() < 0) { // load presentation of a new item
				return serviceObject.success(this.object);
			}
			
            WorkItemClient.getWorkItem(responseHandler, args);
        },
        /*
         * Take some attributes (id) in workitem.attributes[theAttribute:value] and store them as workitem.theAttribute
         */
        _promoteAttributes: function(workItem){
            if (!workItem) 
                return;
            workItem.attributes = workItem.attributes ||
            {};
            if (workItem.attributes.id && workItem.attributes.id.id) {
                workItem.id = workItem.attributes.id.id;
            }
            else {
                workItem.id = -1; // will act like a new work item
            }
        },
		/* API */
        fetchPresentations: function(parms /* onSuccess, isInit, operationMsg */){

			if(parms.isForced || !this.isFetched) return this.fetchWorkItem(parms); // will call back with work item and isFetched

			if(!parms.operation && !(parms.showOperation == false)) {
	            var op = this.createOperation({message: this.messages.busyLoading, id: "WorkItemProxy/fetchPresentations"}); // "Fetch work item presentation."
				parms.operation = op;
            	this.startOperation(parms.operation);
			} 
            // parallel requests
            parms.workItem = this.object;
			parms.callback= parms.onSuccess;
			
            if(!(parms.fetchEditorPresentation === false)) {
            	this._fetchPresentationProperties(parms);	
            }
            if(parms.fetchInlineEditorPresentation === true) {
				this._fetchInlinePresentationProperties(parms);            	
            }
            
			if(parms.fetchAttributeDescriptors) {
				this._fetchAttributeDescriptors(parms);
			}
			
			
			/*
			 * Schedule fetching the editable properties 
			 */
 			window.setTimeout(dojo.hitch(this, function() {
				this._fetchEditableProperties(parms);
 				} , 100));
        },

		_isFetchComplete: function(parms) {
			if(parms._isFetchComplete) return false; // avoid double complete
			if( !(parms.fetchEditorPresentation === false) && !parms.presentationProps ) return false;
			if(parms.fetchInlineEditorPresentation === true && !parms.inlinePresentationProps) return false;
			if(parms.fetchAttributeDescriptors == true && !parms._attributeDescriptors) return false;
			return true;
		},
		_fetchInlinePresentationProperties: function(parms) {
        	var self = this;
            var serviceObj = {
                success: function(presentationProps){
                    parms.inlinePresentationProps = presentationProps;
                    if (self._isFetchComplete(parms)) {
                        self._fetchWorkItemComplete(parms);
                    }
                },
                failure: function(error){
					var args = parms;
					args.error = error; 
					args.message = self.messages.errorLoadingPresentation;
                    self.handleError(args);
  					if (dojo.isFunction( parms.onError)) {
						parms.onError(error, args);
					}
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            var args = {}
            var typeId = (parms.workItem.attributes.workItemType && parms.workItem.attributes.workItemType.id != null && parms.workItem.attributes.workItemType.id != "" ? parms.workItem.attributes.workItemType.id : null);
            if (typeId) 
                args.typeId = typeId;
            
            if (!parms.isTypeChange && !parms.isMoveCopy) {
                args.itemId = parms.workItem.itemId;
            }
			args.target = parms.inlinePresentationId || "com.ibm.team.workitem.web.inline"; // $NON-NLS-1
			this._setArgsProjectAreaItemId(args);
            WorkItemClient.getEditorPresentation(srh, args);
		},
		_fetchAttributeDescriptors: function(parms) {
        	var self = this;
            var serviceObj = {
                success: function(attributeDescriptors){
                    parms._attributeDescriptors = attributeDescriptors;
                    if (self._isFetchComplete(parms)) {
                        self._fetchWorkItemComplete(parms);
                    }
                },
                failure: function(error){
					var args = parms;
					args.error = error; 
					args.message = error.message;
                    self.handleError(args);
  					if (dojo.isFunction( parms.onError)) {
						parms.onError(error, args);
					}
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            var args = {};
			this._setArgsProjectAreaItemId(args);
            WorkItemClient.getAttributes(srh, args);
		},
        _fetchEditableProperties: function(parms){
        	
        	/*
        	 * If the values are the same as the last call then skip the call
        	 * This happens when the workFlowAction changes since that is called here then it changes the resolution
        	 */
        	var workflowActionId 		= this.getValue({path: ["attributes", "workflowAction", "id"]});
        	var internalResolutionId 	= this.getValue({path: ["attributes", "internalResolution", "id"]});
        	
            var self = this;
            var serviceObj = {
                success: function(editProps){
					self._priorFetchEditablePropertiesEditProps = editProps;
                    parms.editProps = editProps;
                    self._fetchEditablePropertiesComplete(parms);
//
// TOD: Delete. Removed for async fetchEditableProperties
//                    if(parms["onSuccess"] && !(parms.isRefresh || parms.fetchAttributeDescriptors || parms.fetchEditorPresentation || parms.fetchInlineEditorPresentation)) {
//                    	parms["onSuccess"](parms);
//                    } else {
//	                   if (self._isFetchComplete(parms)) {
//	                        self._fetchWorkItemComplete(parms);
//	                    }
//                    }
                },
                failure: function(error){
					var args = parms;
					args.error = error; 
					args.message = self.messages.errorLoadingProperties;
                    self.handleError(args);
  					if (dojo.isFunction( parms.onError)) {
						parms.onError(error, args);
					}
                }
            };
     
		/*
         * If the values are the same as the last call then skip the call
         */
        var workflowActionId 		= this.getValue({path: ["attributes", "workflowAction", "id"]});
        var internalResolutionId 	= this.getValue({path: ["attributes", "internalResolution", "id"]});
        if(internalResolutionId == "null") internalResolutionId = null;
        
		var typeId = "-1";
		if(this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.id) {
			typeId =  this.object.attributes.workItemType.id;
		}
		
     	if(this._priorFetchEditablePropertiesEditProps && parms.isUsingEditablePropertiesCache 
     			&& this.getItemId() && this.getItemId() == this._priorFetchEditablePropertiesWorkItemItemId 
        		&& workflowActionId != null && workflowActionId == this._priorFetchEditablePropertiesWorkflowActionId
        		&& typeId && typeId == this._priorFetchEditablePropertiesWorkItemTypeId
        		&& (internalResolutionId == null || internalResolutionId == this._priorFetchEditablePropertiesInternalResolutionId)) {
        			serviceObj["success"](this._priorFetchEditablePropertiesEditProps);
        			return;
        }
       	this._priorFetchEditablePropertiesWorkItemItemId = this.getItemId(); 
       	this._priorFetchEditablePropertiesWorkItemTypeId = typeId;
       	this._priorFetchEditablePropertiesWorkflowActionId = workflowActionId;
       	this._priorFetchEditablePropertiesInternalResolutionId = internalResolutionId;
        	      
        var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		var args = {};
		args.includeArchived = false;
		if(this.isNewWorkItem()) {
			args.newWorkItem="true";
			if (this.object && this.object.attributes && this.object.attributes.workItemType) {
				args.typeId = this.object.attributes.workItemType.id;
			} else { 
				if (parms.type) {
					args.typeId = parms.type;
				}
			}
			// Since this is a new work item we can avoid getting the editProps again if it was the same type
			var lastProjectAreaItemId = -1;
			if(parms.workItem && parms.workItem.attributes && parms.workItem.attributes.projectArea) {			
				lastProjectAreaItemId = parms.workItem.attributes.projectArea.id;
			}
			if(this.workItemSpec && this.workItemSpec.editProps && args.typeId == this.workItemSpec.lastLoadedTypeId && this.workItemSpec.lastLoadedProjectAreaItemId == lastProjectAreaItemId) {
				parms.editProps = this.workItemSpec.editProps;
	        	if (this._isFetchComplete(parms)) {
	                 this._fetchWorkItemComplete(parms);
	            }
	          	return;
			}		
		} else { // not new
			if(!parms.isMoveCopy) {
			 	args.itemId = parms.workItem.itemId;	
			} else {
				args.newWorkItem="true"; // fake out
			}	
		}
        if (parms.isTypeChange) {
			args.typeId = this.getValue({path: "attributes.workItemType.id" });
        }

		if(this.object && this.object.attributes) {
				if (workflowActionId)
					args.workflowActionId = workflowActionId;
				if (internalResolutionId)
					args.internalResolutionId = internalResolutionId;
		}

		this._setArgsProjectAreaItemId(args); 
        WorkItemClient.getWorkItemEditableProperties(srh, args);
  },
		/*
		 * Get the Approvals data
		 */
		fetchSimpleInfo: function(fetchArgs) {
//			var fetchArgs = {
//				serviceSelector: "getApprovalTypes",
//				responseObject: this,
//				responseSelector: "refreshApprovalTypes",
//				progressText: "Loading Approval Types"
//			};
	        var op= this.createOperation({
				message: fetchArgs.progressText,
				id: "fetchSimpleInfo"
			});
	    	this.startOperation(op);

			var self=this;
			// fetch the values required to edit the work item
			var serviceObj= {
				success: function(simpleInfo) {
					if(fetchArgs.responseObject[fetchArgs.responseSelector]) {
						fetchArgs.responseObject[fetchArgs.responseSelector](simpleInfo);
					}					
					if (op) { self.finishOperation(op); }
				},
				failure: function(errorObj) {
					self.handleError({error: errorObj, message: dojo.string.substitute(self.messages.errorLoadingSimpleInfo, [ fetchArgs.serviceSelector ])});
					if (op) { self.finishOperation(op); }
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");

			var args = { projectAreaItemId: this.getProjectAreaItemId()};

			if(WorkItemClient[fetchArgs.serviceSelector]) {
				WorkItemClient[fetchArgs.serviceSelector](srh, args);
			} else {
				console.debug(dojo.string.substitute(this.messages.errorInvalidFetchSimpleInputSelector,[serviceSelector]));
			}
		},		
        _setArgsProjectAreaItemId: function(parms) {
			if(parms.projectAreaItemId) return;
			var projectAreaItemId = this.getValue({path: "attributes.projectArea.id"});
			if(projectAreaItemId) {
				parms.projectAreaItemId = projectAreaItemId;
			}
		},
		getProjectAreaItemId: function() {
			return this.getValue({path: "attributes.projectArea.id"});
		},
        _fetchPresentationProperties: function(parms){
            var self = this;
            var serviceObj = {
                success: function(presentationProps){
                    parms.presentationProps = presentationProps;
                     if (self._isFetchComplete(parms))  {
                        // only populate when we have all responses
                        self._fetchWorkItemComplete(parms);
                    }
                },
                failure: function(error){
					var args = parms;
					args.error = error; 
					args.message = self.messages.errorLoadingPresentation;
                    self.handleError(args);
  					if (dojo.isFunction( parms.onError)) {
						parms.onError(error, args);
					}
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            var args = {}
            var typeId = (parms.workItem.attributes.workItemType && parms.workItem.attributes.workItemType.id != null && parms.workItem.attributes.workItemType.id != "" ? parms.workItem.attributes.workItemType.id : null);
            if (typeId) 
                args.typeId = typeId;
            
            if (!parms.isTypeChange && !parms.isMoveCopy) {
                args.itemId = parms.workItem.itemId;
            }
			this._setArgsProjectAreaItemId(args);
            WorkItemClient.getEditorPresentation(srh, args);
        },
		_isPresentationVisible: function(pres) {
			if(!pres.properties) return true;
			var attributePath, attributeValue;
			if (pres.attributeId) {
				var subkey = KNOWN_ATTRIBUTES[pres.attributeId];
				if(subkey == false) {
					attributePath = ["attributes", pres.attributeId];
					attributeValue = this.getValue({path: attributePath});					
				} else {
					if (subkey) { // a known subkey value
						attributePath = ["attributes", pres.attributeId, subkey];
						attributeValue = this.getValue({path: attributePath});
					} else { // now it is either id or content
						attributePath = ["attributes", pres.attributeId, "id"];
						attributeValue = this.getValue({path: attributePath});
						if(!attributeValue) { // try content (e.g., smallString)
							attributePath = ["attributes", pres.attributeId, "content"];
							attributeValue = this.getValue({path: attributePath});						
						}
					}
				}
			}
			else {
				attributePath = PART_TO_PATH_MAP[pres.partId];
				if(attributePath) {
					attributeValue = this.getValue({path: attributePath});
				}
			}

			var propertyMap = {}; 
			this.getPropertyMap(pres, propertyMap);
			if(propertyMap.hideIfEmpty == "true") {
				if(pres.partId != 'com.ibm.team.workitem.kind.separator') {
			     	if(attributeValue == null || attributeValue.length == 0) return false;					
				} 
			}
			if(propertyMap.hideIfEndpointEmpty == "true") {
				var endpointArray = propertyMap.hideIfEndpointEmpty.split("/");
				if(endpointArray && endpointArray.length > 0) {
					var endpointLinkTypeId = false;
					var endpointId = endpointArray[0];
					if(endpointArray.length > 1) {
						endpointLinkTypeId = endpointArray[0];
						endpointId = endpointArray[1];
					}
					var filter = {
							endpointId: endpointId
						}
					if(endpointLinkTypeId) {
						filter.id = endpointLinkTypeId;
						filter.isSource =  endpointId == "children";
					}
					var linkTypes = this._filterLinkTypes(filter);
					
					if (!linkTypes || !linkTypes.length > 0 || !linkTypes[0].linkDTOs || linkTypes[0].linkDTOs.length == 0) {
							return false;
					}
				}
			}
			
			if(propertyMap.hideIfCreation == "true") {
				if(this.isNewWorkItem()) return false;
			}
			if(propertyMap.hideIfInState != null) {
				var state = this.getValue({path: "attributes.internalState"});
				if(state) {
					var hideIfInStates = propertyMap.hideIfInState.split(",");
					for(var i=0;i<hideIfInStates.length;i++) {
						if(!this._checkState(state, dojo.trim(hideIfInStates[i]))) return false;
					}					
				}
			}					
			return true;
		},
	_checkState: function(state, hideIfInState) {
			if(state.id == hideIfInState) return false;
			if(!isNaN(state.id)) {
				if("s" + state.id == hideIfInState) return false;
				if("S" + state.id == hideIfInState) return false;
			}
			if(!isNaN(hideIfInState)) {
				if(state.id == "s" + hideIfInState) return false;
				if(state.id == "S" + hideIfInState) return false;
			}
			return true;
		},
        _filterPresentationProps: function(props) {
			if(!props || !props.pages) return;
			for(var p=0;p<props.pages.length;p++) {
				var sections = props.pages[p].sections;
				if(!sections || sections.length ==0) break;
				for(var s=0;s<sections.length;s++) {
					var presentations = sections[s].presentations;
					if(!presentations || presentations.length ==0) break;
					var filteredPresentations = [];
					for(var i=0;i<presentations.length;i++) {
						if(this._isPresentationVisible(presentations[i])) {
							filteredPresentations.push(presentations[i]);
							/* Fix missing attributeId in status and resolution */
							if(!presentations[i].attributeId && presentations[i].partId && PART_TO_ATTRIBUTE_MAP[presentations[i].partId]) {
								presentations[i].attributeId = PART_TO_ATTRIBUTE_MAP[presentations[i].partId];
							}
						}
					}
					sections[s].presentations = filteredPresentations;
				}
			}
		},
		_arrayContains: function(array, item) {
  			var i = array.length;
  			while (i--) {
    			if (array[i] == item) {
      				return true;
    			}
  			}
  			return false;
		},
        _fetchWorkItemComplete: function(parms){
        	/*
        	 * Asynchronously load the presentation widgets
        	 * When all are fetched, complete the fetch
        	 */
        	var widgetNames = [];
        	/*
        	 * Push the full editor widgets (if any)
        	 */
        	this._pushPresentedWidgetNamesToLoad(parms.presentationProps, widgetNames);
			/*
			 * If there are any full editor presentations loaded, load the actions
			 */
        	if(parms.presentationProps && parms.presentationProps.pages && parms.presentationProps.pages.length > 0) {
       			this._pushActionNamesToLoad(parms, widgetNames);
        	}		
        	/*
        	 * Push the inline editor widgets
        	 */	
			this._pushPresentedWidgetNamesToLoad(parms.inlinePresentationProps, widgetNames);
			
			/*
			 * Load the widgets
			 */
        	var self = this;
        	if(widgetNames.length > 0 && !this._isLoadingWidgets) {
        		this._isLoadingWidgets = true;
	        	jazz.core.loader.batch_load_async(widgetNames, function(){
        			this._isLoadingWidgets = false;
	        		if(self._isLoadingActions) {
	        			self._isLoadingActions = false;
	         			com.ibm.team.workitem.web.cache.internal.Cache.addItem("workitems/extensionClassesLoaded", true);
	      			}
        			self._fetchWorkItemWithDependentClassesComplete(parms);
	    		});
	       		return; // _continues when async load completes
       		} 		
    		this._fetchWorkItemWithDependentClassesComplete(parms);
        },
        _pushPresentedWidgetNamesToLoad: function(presentationProps, widgetNames) {
        	var page, section, widget, widgetName;
        	if(presentationProps && presentationProps.pages && presentationProps.pages.length) {
        		for(var p=0;p<presentationProps.pages.length;p++) {
        			page = presentationProps.pages[p];
        			if(page.sections && page.sections.length > 0) {
        				for(var s=0;s<page.sections.length;s++) {
							section = page.sections[s];
							if(section.presentations && section.presentations.length > 0) {
								for(var w=0;w<section.presentations.length;w++) {
									presentation = section.presentations[w];
									widgetName = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil.getPresentationWidgetName(presentation);
									if(!this._arrayContains(widgetNames, widgetName)) {
										widgetNames.push(widgetName);
									}
								}
							} 
        				}
        			}
        		}
        	}
        },
        _pushActionNamesToLoad: function(parms, widgetNames) {

        	if(com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/extensionClassesLoaded", false)) return;
        	
        	var extensionActions = com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry.getContributedEditorToolbarActions();
			if(extensionActions.length > 0) {
				for(var i=0;i<extensionActions.length; i++) {
					var actionSpec = extensionActions[i];
					if(actionSpec.action) {
						this._isLoadingActions;
						widgetNames.push(actionSpec.action);
					}
				}
			} else {
				com.ibm.team.workitem.web.cache.internal.Cache.addItem("workitems/extensionClassesLoaded", true); // there are none
			}
        },
        _fetchWorkItemWithDependentClassesComplete: function(parms){
        	
        	/*
        	 * If it is a simpleFetch (merge), then we just want the work item
        	 * and we don't want to do anything with it
        	 */
        	if(parms.isSimpleFetch) {
        		if(parms.operation != null) {
     			   this.finishOperation(parms.operation);
     			}
        		parms.onSuccess(parms.workItem);
        		return;
        	}
        	
			this._priorFetchPresentationProps = parms.presentationProps ? dojo.clone(parms.presentationProps) : null; 
			if(parms.presentationProps) {
				this._filterPresentationProps(parms.presentationProps);
			}
			this._priorFetchInlinePresentationProps = parms.inlinePresentationProps ? dojo.clone(parms.inlinePresentationProps) : null;
			if(parms.inlinePresentationProps) {
				this._filterPresentationProps(parms.inlinePresentationProps);
			}
			this.workItemSpec = this.workItemSpec || {};
			this.workItemSpec.lastLoadedTypeId= this.getValue({
				path: "attributes.workItemType.id"
			});
			if(parms.workItem && parms.workItem.attributes && parms.workItem.attributes.projectArea) {			
				this.workItemSpec.lastLoadedProjectAreaItemId = parms.workItem.attributes.projectArea.id;
			}
			this.workItemSpec.editProps= parms.editProps;
			this.workItemSpec.presentationProps= parms.presentationProps ? parms.presentationProps : this.workItemSpec.presentationProps;
			this.workItemSpec.inlinePresentationProps= parms.inlinePresentationProps ? parms.inlinePresentationProps : this.workItemSpec.inlinePresentationProps ;

			if (parms.fetchAttributeDescriptors) {
				this._attributeMap= this._buildAttributeMap(parms._attributeDescriptors);
			}
			
			if(this.getId() != this._cacheId) {
				com.ibm.team.workitem.web.cache.internal.Cache.renameItem(this._cacheId, this.getId());
			}

			if(this.handleObjectRefresh & !parms.isTypeChange) {
				this.handleObjectRefresh();
			}
			if(parms.operation != null) {
			   this.finishOperation(parms.operation);
			}
			
			if(!(parms.fetchWorkItem === false)) {
				if(parms.isRefresh && parms.object) {
					this.object= dojo.clone(parms.object);
				}
// If justNotify is true then the following are not done in refreshObject
//			this.object= dojo.clone(parms.object);
// 			this.changedAttributes = {};
//			this.changedAttributePaths = {};
				this.refreshObject({
				    isInitialization: this.isInitialization,
				    justNotify: parms.isTypeChange || (parms.isForcedFullFetch && !parms.isRefresh),
					object: parms.workItem,
					caller: this
				});
			}
	
			if(parms.isInBackgroundOperation) {
				this.setIsInBackgroundOperation(true);
			}
            if (!parms.isPopulated) {
	            parms.isPopulated = true;
	            parms.workItemProxy = this; /* @deprecated */
	            parms.workingCopy = this;
    	        parms.callback(parms);		
			} 
			var listenerParms = {
				path: ["workItemSpec"],
				event: "onchange"
			};
			this.notifyListeners(listenerParms);
			
            dojo.publish(EventTopics.WORKITEM_CHANGED, [this]);
        },
        _reFilterPresentationProps: function() {
        	if(this._priorFetchPresentationProps) {
        		this.workItemSpec.presentationProps=dojo.clone(this._priorFetchPresentationProps);
				this._filterPresentationProps(this.workItemSpec.presentationProps);
        	}
        	if(this._priorFetchInlinePresentationProps) {
        		this.workItemSpec.inlinePresentationProps=dojo.clone(this._priorFetchInlinePresentationProps);
        		this._filterPresentationProps(this.workItemSpec.inlinePresentationProps);
        	}
        },
        _buildAttributeMap: function(attributeDescriptors) {
        	var attributeMap= new Object();
        	for (var i= 0; i < attributeDescriptors.length; i++) {
        		var attributeDTO= attributeDescriptors[i];
        		attributeMap[attributeDTO.id]= new Attribute(attributeDTO);
        	}
        	return attributeMap;
        },
        /* API */
		moveOrCopyToProjectArea: function(parms /* isMove, projectAreaItemId */) {
            var self = this;
			if(parms.isMove) {
				this.objectBeforeMove = dojo.clone(this.object);	
				this.infos = [];
				this.moveProjectAreaItemId = parms.projectAreaItemId;
			}
			var msg = parms.isMove ? this.messages.busyMoving : this.messages.busyCopying;
            var op = this.createOperation({
				message: msg,
				id: "WorkItemProxyClient/movecopy"
			});// "Fetch work item presentation."
			this.startOperation(op);
            var serviceObj = {
                success: function(workItem){
					// The state of the work item is now the same as if fetched
					// Work Item can be new if copied, or not
					if(parms.isMove && workItem.infos) { self.infos = workItem.infos}
   					self.finishOperation(op);
					self._completeSaveOrMove(parms, workItem, /* isSave */ false);
					
				},
                failure: function(err){
   					self.finishOperation(op);
					if (err && err.workItem && err.severity == 2) { // info message
					    if(parms.isMove) self.infos = err.infos;
						return self._completeSaveOrMove(parms, err.workItem, /* isSave */ false);
					}
					
					var args = dojo.mixin(parms || {}, parms.resultParameters || {});
					args.error = err;
					args.skipErrorMessage = parms.skipErrorMessage;
                    self.handleError(args);
                    if (dojo.isFunction( parms.onError)) {
                        parms.onError(err, args);
                    }
                }
            };
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
          
            var appArgs = this._getSaveAppArgs(parms);
            if(!appArgs) return; // cancelled
			appArgs.move = parms.isMove;
			appArgs.projectAreaItemId = parms.projectAreaItemId;

            WorkItemClient.moveWorkItem(responseHandler, appArgs);
		},

        /* @deprecated */
        splitWorkItem: function(parms) {
          
          var workItemId = parms.workingCopy.getItemId();
          var self = this;
          var msg = this.messages.busySplitting;
          var op = this.createOperation({
            message: msg,
            id: "WorkItemProxyClient/splitworkitem"
          }); // "Fetch work item presentation."
          
          this.startOperation(op);

          var serviceObj = {
            success: function(workItem) {

				self.finishOperation(op);

				var args = { 
					id: workItem.attributes.id.id,
					onSuccess: dojo.hitch(this, this._refreshComplete), 
					isRefresh: true,
					fetchEditorPresentation: false,
					fetchInlineEditorPresentation: false
					};
				self.fetchWorkItem(args);

          	},
		  		
	  		_refreshComplete: function(parms) {
	  			// nothing to do
	  		},

            failure: function(err) {

				self.finishOperation(op);

				var args = parms;
				args.error = err; 
				args.message = err.message;
				self.handleError(args);

				parms.onError(err, parms);
            }
            
          };

          var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");

          var appArgs = this._getSaveAppArgs(parms);
          
          if(!appArgs) return; // cancelled
          
          appArgs.newOwnerId = parms.newOwner.itemId;

          WorkItemClient.splitWorkItem(responseHandler, appArgs);
        },
		_promoteDTOsToKeyValues: function(DTOArray) {
			if(!dojo.isArray(DTOArray)) return DTOArray; // normal properties.
			var result = {};
			for(var i=0;i<DTOArray.length;i++) {
				var key = DTOArray[i].key;
				var value = DTOArray[i].value;
				if(key && value) {
					result[key] = value;
				}
			}
			return result;
		},
		_moveCopyComplete: function(parms) {
					var args = dojo.mixin(parms ||
					{}, parms.resultParameters ||
					{});
					args.workItem = parms.workItem;
				
					if(this.workItemSpec && this.workItemSpec.editableLinkTypes)  this.workItemSpec.editableLinkTypes = null; // reset it
					
				
					if (dojo.isFunction(parms.moveCopyCallerOnSuccess)) {
						parms.moveCopyCallerOnSuccess(args);
					}
					dojo.publish(EventTopics.WORKITEM_PRESENTATION_CHANGED, [{
						workingItemProxy: this
					}]);
		},
		
		_storeWorkItemCompleted: function(workItem, parms) {
			if (workItem.infos) { 
				this.infos = workItem.infos;
			}
			this.moveProjectAreaItemId = false; // clear the entry
			this._completeSaveOrMove(parms, workItem, /* isSave */ true);
		},

        storeWorkItem: function(parms /*	onSuccess: successCallback,	onError: failureCallback */){
			this.infos = [];
            var self = this;
            var op = this.createOperation({
				message: parms.operationMsg,
				id: "WorkItemProxyClient/store"
			});// "Fetch work item presentation."
			this.startOperation(op);
            var serviceObj = {
                success: function(workItem){
// Uncomment the following to test the CALM Back Link error dialog
//					var err = {
//						infos: [ { message: 'error',
//									causeStatusCode:PROBLEM_MODIFYING_CALM_LINK } ]
//					}
//					this.failure(err);
            	 	self.finishOperation(op);
            		self._storeWorkItemCompleted(workItem, parms);
					// The result of store is merged with the current value
				},
                failure: function(err){
                    self.finishOperation(op);
                    /*
                     * If any info has a CALM failure, confirm and retry
                     */
                    var hasCalmProblem = false;
                    if(err && err.infos && parms.instigator && parms.instigator.openCALMBackLinkProblemDialog) {
                    	for (var i=0;i<err.infos.length;i++) {
                    		if(err.infos[i].causeStatusCode == PROBLEM_MODIFYING_CALM_LINK) {
								var calmParms = {
		      						error: err,
		      						onOk: function(saveOption) {
		      							if(saveOption == "1") { // retry
											parms.doNotUpdateBackLinks = false;
		      							   return self.storeWorkItem(parms);
		      							} else if(saveOption == "2") { // revert changed calm links and try again
		      							   	self._revertBackLinkChanges(parms);
											parms.doNotUpdateBackLinks = false;
			   								return self.storeWorkItem(parms);
		      							} else if(saveOption == "3") { // save without updating backlinks
											parms.doNotUpdateBackLinks = true;
			   								return self.storeWorkItem(parms);
		      							}
		      							return self._completeSaveOrMove(parms, err.workItem, /* isSave */ true);
		      						},
		      						onCancel: function() {
										var args = dojo.mixin(parms || {}, parms.resultParameters || {});
										args.error = err;
										args.skipErrorMessage = parms.skipErrorMessage;
		                    			self.handleError(args);
		                    			if (dojo.isFunction( parms.onError)) {
		                        			parms.onError(err, args);
		                    			}
		                    			return;
		                			}	      		
		                    	}
		                    	parms.instigator.openCALMBackLinkProblemDialog(calmParms);
		                        return; // Dialog will open onOK or onCancel above          			
		                    }
                    	}
                    }
					if (err && err.workItem && err.severity == 2) { // info message
					    err.workItem.errorInfos = err.infos;
						return self._completeSaveOrMove(parms, err.workItem, /* isSave */ true);
					}
					
					var args = dojo.mixin(parms || {}, parms.resultParameters || {});
					args.error = err;
					args.skipErrorMessage = parms.skipErrorMessage;
                    self.handleError(args);
                    if (dojo.isFunction( parms.onError)) {
                        parms.onError(err, args);
                        self.setIsDirty(false);
                    }
                }
            };
            var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
            
			if (this.moveProjectAreaItemId) {
				parms.isSaveAll = true; 
			}
            var appArgs = this._getSaveAppArgs(parms);
            if(!appArgs) return; // cancelled
            
			if(this.moveProjectAreaItemId) {
				appArgs.projectAreaItemId = this.moveProjectAreaItemId;
			}

			this._setArgsProjectAreaItemId(appArgs);	
            WorkItemClient.postWorkItem(responseHandler, appArgs);
        },
		_createChangedAttributesFrom: function(attributeObject) {
			this.changedAttributes = this.changedAttributes || {};
			this.changedAttributePaths = this.changedAttributePaths || {};

			for(var key in this.object.attributes) {
				var valueKey = KNOWN_ATTRIBUTES[key] || "id";
				var isExcluded = DERIVED_ATTRIBUTES[key];
				if (this.object.attributes && this.object.attributes[key]) {
					if (!attributeObject || !attributeObject[key] || (!isExcluded && this.object.attributes[key][valueKey] && this.object.attributes[key][valueKey] != attributeObject[key][valueKey])) {
						this.changedAttributes[key] = this.object.attributes[key][valueKey];
						this.changedAttributePaths[key] = ["attributes", key, valueKey];
					}
				}
			}
		},
		_completeSaveOrMove: function(parms, workItem, isSave){
			if(!isSave && !parms.isMove) { // Handle a Copy
				return this._completeCopy(parms, workItem);
			}
			
			// In the case of a type change now that it is saved, throw away the between type attributes
			if(isSave) { 
				this._priorTypeProxyDataMap = {};
			}	
			// merge the loaded work item with those from the save
			if (workItem.attributes != null) {
				var isNew = workItem.attributes && workItem.attributes.id &&  workItem.attributes.id.id == -1;
				if(!isSave) {
				// Promote these to key-value pairs if needed
					workItem.attributes = this._promoteDTOsToKeyValues(workItem.attributes);				
				}
				this.object.attributes = this.object.attributes || {};
				dojo.mixin(this.object.attributes, workItem.attributes);
				this.object.stateId = workItem.stateId || this.object.stateId;
				this.object.itemId = workItem.itemId || this.object.itemId;
				if(workItem.attributes && workItem.attributes.id && workItem.attributes.id.id) {
					this.object.id = workItem.attributes.id.id;
				}
				
				if (parms.isMove) {
					var comment = this.getValue({
						path: [ "attributes" , [this.SYNTHETIC_COMMENT_ID] ]
					});
					var internalComments = this.getValue({
						path: "attributes.internalComments"
					})
					/* fix the comments, the last internalComment.items is a dup of the comment */
					if (comment && comment.content && internalComments && internalComments.items) {
						var items = internalComments.items;
						var commentContent = comment.content;
						var lastIndex = items.length - 1;
						if (commentContent && items && items[lastIndex] ) {
							items.splice(lastIndex, 1);
						}
					}
				}
				
				if (parms.applyDelta) {
					this.storedObject.attributes = this.storedObject.attributes || {};
					// Even if an attribute is set to "unassigned" it is returned in workItem.attributes
					// Merge the newly loaded attributes with the storedObject
					dojo.mixin(this.storedObject.attributes, dojo.clone(workItem.attributes));
					this.storedObject.stateId = workItem.stateId;
					this.storedObject.itemId = workItem.itemId;
					this.storedObject.locationUri = workItem.locationUri;
					if(workItem.attributes && workItem.attributes.id && workItem.attributes.id.id) {
						this.storedObject.id = workItem.attributes.id.id;
					}
					/*
					 * The workflow action is reset after saving
					 */
					 this.object.attributes.workflowAction = null;
					 /*
					  * If the edit props were loaded last time, refresh them after save
					  * This is for status.action
					  */
					 if(this.workItemSpec && this.workItemSpec.editProps) {
						var args = {
							workItem: this.object,
							isUsingEditablePropertiesCache: true,
							onSuccess: dojo.hitch(this, this._fetchEditablePropertiesComplete)
						};
						this._fetchEditableProperties(args);
					 }
					/*
					 * Re filter the presentation properties to handle hideIfInState changes
					 */
					this._reFilterPresentationProps();
				}

// Use the existing links and approvals... TODO: Merge old with new				
//				this.object.linkTypes = workItem.linkTypes;
//				this.object.approvals = workItem.approvals;
			}
			if(!isSave && this.objectBeforeMove && this.objectBeforeMove.attributes) {
				this._createChangedAttributesFrom(this.objectBeforeMove.attributes);				
			}

/* If the object was saved then we need to clear links and approvals to avoid isChanged. Will be reloaded on refresh */
			if(isSave) {
				if (!parms.applyDelta) {
					this.object.linkTypes = false;
					this.object.approvals = false;
					this.isFetched = false;
				} else {
					/*
					 * If linkTypes were returned, update the linkTypes
					 * LinkTypes are returned if they are updated
					 * If all are deleted, then parms.updateLinks is set
					 */
					if(workItem.linkTypes != null || parms.widgetCommands.updateLinks != null) {
						this.object.linkTypes = workItem.linkTypes;
						this.storedObject.linkTypes = dojo.clone(workItem.linkTypes);					
					}
					if(workItem.approvals != null || parms.widgetCommands.updateApprovals != null) {
						this.object.approvals = workItem.approvals;
						this.storedObject.approvals = dojo.clone(workItem.approvals);	
					}

					this.isFetched = true;
				}
			}
			this.objectBeforeMove = {};
			
			if (workItem && workItem.attributes && workItem.attributes.id && workItem.attributes.id.id && workItem.attributes.id.id != this._cacheId) {
				var newId = workItem.attributes.id.id;
				com.ibm.team.workitem.web.cache.internal.Cache.renameItem(this._cacheId, newId);
				this._lastId = newId;
				this._cacheId = newId;
				this.getIdLabel(newId); // reset the id label
				/*
				 * If the saved new work item has links, notify the other work items that have linkPending to update their links
				 */
				var pendingWorkItems = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({ filterAttributes: [ "isWorkItem", "hasNewLinksPending" ]});
				for(var i=0;i<pendingWorkItems.length;i++) {
					pendingWorkItems[i].mergeRepositoryChanges();
				}
			}
			
			if (isSave) {
				var args = dojo.mixin(parms ||
				{}, parms.resultParameters ||
				{});
				args.workItem = workItem;
				this.clearRetainedState(true);
				if (dojo.isFunction(parms.onSuccess)) {
					parms.onSuccess(args);
					this.setIsDirty(false);
				}
				this.refreshObject({justNotify: true});
	            dojo.publish(EventTopics.WORKITEM_STORED, [this]);
			}
			else { // move
				this.setIsDirty(true);
				args = {
					isMove: parms.isMove,
					isMoveCopy: true,
					moveCopyCallerOnSuccess: parms.onSuccess,
					onSuccess: dojo.hitch(this, this._moveCopyComplete),
					isInit: false,
					operationMsg: this.messages.busyMoving,
					fetchAttributeDescriptors: true
				}
				this.fetchPresentations(args);
			}

		},
	_fetchEditablePropertiesComplete: function(parms) {
		  var editProps = parms.editProps;
		  this.workItemSpec = this.workItemSpec || {};
		  this.workItemSpec.editProps = editProps;
		  dojo.publish(EventTopics.WORKITEM_CHANGED, [this]);
		  dojo.publish(EventTopics.WORKITEM_REQUIRED_PROPERTIES_CHANGED, [{ workingCopy: this } ]);
		  dojo.publish(EventTopics.WORKITEM_EDITABLE_PROPERTIES_CHANGED, [{ workingCopy: this } ]);
	},
	_completeCopy: function(parms, workItem){
	/*
	 * Create a clone of the work item and put it in the cache
	 * Open the editor on the new open work item
	 */
			if (workItem.attributes != null) {
				workItem.attributes = this._promoteDTOsToKeyValues(workItem.attributes);
			}
			var id = WorkItemUtil.getUniqueId();
			id =  -1 * Math.abs(parseInt(id,10)); // unique negative number
			var type;
			if(workItem.attributes && workItem.attributes.workItemType) type =  workItem.attributes.workItemType.id
			var uri = WorkItemUtil.getNewWorkItemUri(id,type);
			// Call a local version of the method rather than WorkItemProxyFactory to avoid circular dependencies
			var newProxy = this._getNewWorkItemProxy({
//			var newProxy = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({
						id: id,
						createIfNeeded: true
			});	
			newProxy.incrementUseCount();
			newProxy.isFetched = true;
			newProxy._isWorkItemCopyTarget = true;
			workItem.attributes.id.id = id;
			workItem.id = id;
			var oldObject = newProxy.object;
			newProxy.object = workItem;
			/*
			 * The new proxy now has the id a large negative number attributes.id.id
			 */
			newProxy._createChangedAttributesFrom(oldObject);
			newProxy.fetchPresentations({
				copyUri: uri,
				onSuccess: dojo.hitch(this, this._completeCopy2)
			})
		},
_completeCopy2: function(parms) {				
			location.replace(parms.copyUri);
		},
		// Set a set of values where the attibute list is in the form
		// key: value
		// Unfortunately we have to guess at what value is being set
		setAttributes: function(parms /* attributeList */ ) {
			var formElements = parms.attributeList;
			for (var key in formElements) {
				var parms = {
					caller: this,
					attributeId: key,
					value: formElements[key]
				};
				if (KNOWN_ATTRIBUTES[key]) {
					parms.path = ["attributes", key, KNOWN_ATTRIBUTES[key]]; 
					this.setValue(parms);
				}
				else {
					// otherwise try id, label and content
					if (this.object.attributes && this.object.attributes[key]) {
						if (this.object.attributes[key].id) {
							parms.path = ["attributes", key, "id"]; 
							this.setValue(parms);
						}
						else {
							if (this.object.attributes[key].content) {
								parms.path = ["attributes", key, "content"]; 
								this.setValue(parms);
							}
							else {
								if (this.object.attributes[key].label) {
									parms.path = ["attributes", key, "label"]; 
									this.setValue(parms);
								}
							}
						}
					}
				}
			}
		},
        /*
         * Return an object containing the names and values of all of the form
         * elements in the editor parts.
         */
        _getFormElements: function(isReturningUnchangedAttributes, allFormElements){
            var allFormElements = allFormElements || {};
            this.getFormElementValues(allFormElements);
            var allProperties = {}; // Handle the case where a value is null
//            for (var widget in this._childWidgets) {
//                if (this._childWidgets[widget].getFormElementValues) {
//                    this._childWidgets[widget].getFormElementValues(allFormElements);
//                }
//            }
            
            // Defect 38252: Return all unchanged attributes
            // If is returning unchanged values, include 
            // non-skipped unchanged attributes from the work item
            if (isReturningUnchangedAttributes && this.object && this.object.attributes) {
                for (var attr in this.object.attributes) {
                	if(attr == null) continue; // Sometimes this is null
                    // If it is not already in the form element (e.g., was changed) 
                    // and it is not on the exclusion list
					// subscriptions are a special case 
					if(attr == "internalSubscriptions") {
						if (!allFormElements[attr] && this.object.attributes[attr] && this.object.attributes[attr].items && this.object.attributes[attr].items.length > 0) {
							var subs = this.object.attributes[attr].items;
							var itemIdList = [];
							for (var i = 0; i < subs.length; i++) {
								itemIdList.push(subs[i].itemId)
							}
							allFormElements[attr] = itemIdList;
						}
					} else  if (!allProperties[attr] && KNOWN_ATTRIBUTES[attr]) {
                        if (KNOWN_ATTRIBUTES[attr] == true) {
                            allFormElements[attr] = this.object.attributes[attr];
                        }
                        else {
                            if (this.object.attributes[attr][KNOWN_ATTRIBUTES[attr]]) {
                                allFormElements[attr] = this.object.attributes[attr][KNOWN_ATTRIBUTES[attr]];
                            }
                        }
                    } else {
                    	if(this.object.attributes[attr]) {
	                    	var changedAttributeSelector = false;
	                    	if(this.changedAttributes && this.changedAttributes[attr] && this.changedAttributes[attr].savedAttributePath && this.changedAttributes[attr].savedAttributePath.length > 0) {
	                        	var changedAttributeSelector = this.changedAttributes[attr].savedAttributePath[this.changedAttributes[attr].savedAttributePath.length-1];
	                    	}
	                    	if(changedAttributeSelector) {
	                    		allFormElements[attr] = this.object.attributes[attr][changedAttributeSelector];
	                    	}
	                    	else {
		                       	if(this.object.attributes[attr].id) {
		                    		allFormElements[attr] = this.object.attributes[attr].id;
		                      	} else {
		                          	if(this.object.attributes[attr].itemId) {
		                    			allFormElements[attr] = this.object.attributes[attr].itemId;
		                      		} 
		//                      		else {
		//                   		     	if(this.object.attributes[attr].label) {
		//                   			 		allFormElements[attr] = this.object.attributes[attr].label;
		//                      			} else {
		//                   			     	if(this.object.attributes[attr].content) {
		//                   				 		allFormElements[attr] = this.object.attributes[attr].content;
		//                      				} else {
		//                   				     	if(this.object.attributes[attr].items) {
		//                   					 		allFormElements[attr] = this.object.attributes[attr].items;
		//                      					}
		//                      				}
		//                      			}
		//                      		}
		                      	}
	                    	}
                    	}
                    }
                }
            }
            
            return allFormElements;
        },
        
        _getSaveAppArgs: function(parms /* widgetCommands, widgetFormElements */ ){
			
			if(!parms.workItemProxy) parms.workItemProxy = this;
			parms.widgetCommands = parms.widgetCommands || {};
			parms.widgetFormElements = parms.widgetFormElements || {};
			dojo.publish(EventTopics.WORKITEM_ABOUT_TO_SAVE, [ parms ]);
			if(parms.cancelSave) {
				console.debug("Save cancelled by component: " + parms.cancelSaveReason)
				return null;
			}
			var hasWidgetValues = false;
			for(var x in parms.widgetCommands) {
				hasWidgetValues = true;
				break;
			}
			if (!hasWidgetValues) {
				for (var x in parms.widgetFormElements) {
					hasWidgetValues = true;
					break;
				}
			}
			if(hasWidgetValues) {
				console.debug("The widget functions getFormElementValues and getSaveCommands are deprecated. Use setValue on the working copy instead. To be removed in RTC 3.0.") // $NON-NLS-1
			}
			
			
            var formElements = this._getFormElements(/* includeUnchangedAttributes */true, parms.widgetFormElements);
			
            // TODO: Remove target mapping
            if (formElements.target) {
                formElements.intervalItemId = formElements.target;
            }
            
            var appArgs = {};
            
            // Find all the custom attributes and put them in a the attribute and value arrays
            var attributeIdentifiers = [];
            var attributeValues = [];
            var action = null;
            for (var key in formElements) {
                // If there is a value, and its different than the saved work item, add it to be saved
                if (formElements[key] != null) {
                    if (!DERIVED_ATTRIBUTES[key] && (KNOWN_ATTRIBUTES[key] != false) &&
                    (parms.isSavaAll || this.isNewWorkItem() ||
                    !this._isAttributeEqual(this._getStoredObjectAttributeValue(key), formElements[key], key))) {
                        var newKey = ATTRIBUTE_KEY_MAP[key] ? ATTRIBUTE_KEY_MAP[key] : key;
                        if (newKey == "type") {
                            newKey = "workItemType"; // Type is a special case
                        }
						if (key == "internalState" || key == "workflowAction") {
							// If the workflow id is provided use that
							if(formElements['workflowAction']) {								
								action = formElements['workflowAction']
							} else {
								action = formElements['internalState']
							}
                        }
                        else {
                            if (newKey != this.SYNTHETIC_COMMENT_ID) { // Comments are a special case (so skip them) 
                                // If a widget sends the atttribute wrapped like: { type: "date", value: aValue } e.g. Timestamp 
                                // then use the .value
                                var value = formElements[key].type ? formElements[key].value : formElements[key];
                                attributeIdentifiers.push(newKey);
                                attributeValues.push(value);
                            }
                        }
                    }
                } else {
				   attributeIdentifiers.push(key);
                   attributeValues.push("");
				}
            }
            
            // FIXME: this is a hack to send back an custom attribute of a draft work item without a presentation 
            if (this.isNewWorkItem() && this.object.attributes["com.ibm.rqm.defect"]) {
                attributeIdentifiers.push("com.ibm.rqm.defect");
                attributeValues.push(this.object.attributes["com.ibm.rqm.defect"].content);
            }
            
            // Comment is a special case
            if (formElements[this.SYNTHETIC_COMMENT_ID]) {
                attributeIdentifiers.push("internalComments");
                attributeValues.push(formElements[this.SYNTHETIC_COMMENT_ID]);
            }
            
            
            appArgs.attributeIdentifiers = attributeIdentifiers;
            appArgs.attributeValues = attributeValues;
            
            // Action is required
            if (action) {
                appArgs.action = action;
            }
            
            // ItemId and type are required
            appArgs.itemId = formElements.itemId;
            appArgs.type = formElements.workItemType ? formElements.workItemType : formElements.type;
            
            // For new work items, we always need to send back the category if it was changed
            if (this.getId() < 0 && !this._hasValue(attributeIdentifiers, "category") && formElements.category) {
                attributeIdentifiers.push("category");
                attributeValues.push(formElements.category);
            }
            
            // WorkItemType is a special case
            // Three cases below:
            // 1) Work item type is provided and changed in an enumeration
            // 2) The type was changed in an existing type, and the new type does not have a wi type enumeration
            // 3) The work item is new.
            var workItemTypeFound = this._hasValue(attributeIdentifiers, "workItemType");
            if (!workItemTypeFound && formElements && this.object && this.object.attributes && this.storedObject && this.storedObject.attributes && this.storedObject.attributes.workItemType) {
                if (formElements.workItemType && this.object.attributes.workItemType.id && (formElements.workItemType != this.storedObject.attributes.workItemType.id)) {
                    attributeIdentifiers.push("workItemType");
                    attributeValues.push(formElements.workItemType);
                    workItemTypeFound = true;
                }
            }
            
            // if the presentation doesn't include these form elements, use the defaults
            appArgs.type = appArgs.type ? appArgs.type : this.changedTypeId;
            appArgs.type = appArgs.type ? appArgs.type : this.object.attributes.workItemType.id;
//            appArgs.type = appArgs.type ? appArgs.type : this.parentController.pageController.defaultType;
            appArgs.itemId = appArgs.itemId ? appArgs.itemId : this.object.itemId;
			var stateId = this.object.stateId;
			if (stateId) {
                appArgs.stateId = appArgs.stateId || stateId;
            }
            else { // New work item need a type
                if (!workItemTypeFound) {
                    workItemTypeFound = true;
                    attributeIdentifiers.push("workItemType");
                    attributeValues.push(this.changedTypeId || appArgs.type);
                }
            }
            
            /*
             * Get the commands from the elements
             * Each command is: { appArgsId: [ commands ] }
             */
            var saveCommands = parms.widgetCommands || {};
			this.getSaveCommands(saveCommands);			
			
            for (var key in saveCommands) {
                var cmdArray = saveCommands[key];
                if (cmdArray.length > 0) {
                    var updateArray = [];
                    for (var i = 0; i < cmdArray.length; i++) {
                        updateArray.push(dojo.toJson(cmdArray[i]));
                    }
                    appArgs[key] = updateArray;
                }
            }
            if(!parms.doNotUpdateBackLinks) {
	            appArgs.additionalSaveParameters= [ SAVE_PARAMETER_UPDATE_BACKLINKS ];
            }

			appArgs.sanitizeHTML = true;

            return appArgs;
            
        },
//        _getSaveCommands: function(){
//            var allCommands = {};
//            for (var partName in this._editorParts) {
//                if (this._editorParts[partName].getSaveCommands) {
//                    this._editorParts[partName].getSaveCommands(allCommands);
//                }
//            }
            
            //			for(var i=0; i< this._updateManagers.length; i++ ) {
            //				if(this._updateManagers[i].getSaveCommands) {
            //					this._updateManagers[i].getSaveCommands(allCommands);
            //				}
            //			}
            
//            return allCommands;
//        },
        
        _isAttributeEqual: function(attr, other, key){
        	if(attr && !other) return false;
        	if(other && !attr) return false;
            // attr comes from work item
            // other comes from form elements;
            if (attr == other) 
                return true;
            if (attr == null && dojo.isString(other) && other == "") 
                return true;
            if (other == null && dojo.isString(attr) && attr == "") 
                return true;
            
            // If the attr is an iso string, then convert it to time
            try {
                if (dojo.isString(attr) && attr != null && attr != "") {
                    attrDate = WorkItemUtil.fromISOString(attr)
                    if (attrDate) {
                        var time = attrDate.getTime();
                        if (time == other) 
                            return true;
                    }
                }
            } 
            catch (e) {
            } // ignore
            if (other && other.type == "date") {
                other = other.value;
                var attrDate, otherDate, attrStr, otherStr;
                // If the attr is an iso string, then convert it to time
                try {
                    if (other != null && other != "") {
                        otherDate = WorkItemUtil.fromISOString(other)
                        if (otherDate) {
                            var otherTime = otherDate.getTime();
                            if (time == otherTime) 
                                return true;
                        }
                    }
                } 
                catch (e) {
                } // ignore
                // test for null conditions. dates can be -1 to indicate unset.
                if (attr == null && other == -1) 
                    return true;
                if (attr == -1 && other == null) 
                    return true;
                
                // If the dates are the same, but the hours different, then the dates are equal
                // This time is always local, and this normalizes distant time zones
                try {
                    if (other && !otherDate) {
                        otherDate = new Date(other);
                    }
                    if (attrDate && otherDate) {
                        attrStr = dojox.date.posix.strftime(attrDate, DATE_FORMAT_NO_TIME);
                        otherStr = dojox.date.posix.strftime(otherDate, DATE_FORMAT_NO_TIME);
                        if (attrStr == otherStr) 
                            return true;
                    }
                } 
                catch (e) {
                } // ignore, not dates
            }
            
            
            if (other && other.toLowerCase) {
                other = other.toLowerCase();
            }
            if (attr && attr.toLowerCase) {
                attr = attr.toLowerCase();
            }
            
            if (attr == null && other == "unassigned") 
                return true;
            if (other == null && attr == "unassigned") 
                return true;
            if (attr == null && other == "undefined") 
                return true;
            if (other == null && attr == "undefined") 
                return true;
                
            if(key == "internalSubscriptions"){
            	if(attr.length != other.length) return false;
            	for(var i=0;i<other.length;i++) {
            		if(other[i] != attr[i].itemId) {
            			if(!WorkItemUtil.isInArray(other[i], attr, "itemId")) {
            				return false;
            			}
            		}
            	}
            	for(var i=0;i<attr.length;i++) {
            		if(other[i] != attr[i].itemId) {
            			if(!WorkItemUtil.isInArray(attr[i].itemId, other)) {
            				return false;
            			}
            		}
            	}
            	return true;
            }
            
            //			if(attr == null && other == -1000) return true;
            //			if(other == null && attr == -1000) return true;
            return false;
        },
        _getStoredObjectAttributeValue: function(attr){
        
            // Type is a special case
            //			if(attr == "type") return this.object.workItemType;
            
            var result;
            var key = KNOWN_ATTRIBUTES[attr];
            if (key && this.storedObject.attributes[attr] != null) {
                if (key == true) {
                    result = this.storedObject.attributes[attr]
                }
                else {
                    if (this.storedObject.attributes[attr][key] != null) {
                        result = this.storedObject.attributes[attr][key];
                    }
                }
            }
            else {
                // Unknown attribute
                if (this.storedObject.attributes[attr]) {
                    if (this.storedObject.attributes[attr].id) 
                        result = this.storedObject.attributes[attr].id;
                    else 
                        if (this.storedObject.attributes[attr].itemId) 
                            result = this.storedObject.attributes[attr].itemId;
                        else 
                            if (this.storedObject.attributes[attr].label) 
                                result = this.storedObject.attributes[attr].label;
                            else 
                                result = this.storedObject.attributes[attr].content;
                }
                else {
                    if (key == null) {
                        for (var key in this.storedObject.attributes[attr]) {
                            if (key == attr) {
                                return this.storedObject.attributes[attr].id ? this.storedObject.attributes[attr].id : null;
                            }
                        }
                    }
                }
            }
            return result;
        },
       fetchWorkItemTypes: function(parms /* reset, onSuccess, projectAreaItemId */) {
			this._setArgsProjectAreaItemId(parms);
			var itemTypesCacheKey = "workitems/itemTypes/" + parms.projectAreaItemId;   
	       	if(!parms.isReset) {
				var itemTypes = com.ibm.team.workitem.web.cache.internal.Cache.getItem(itemTypesCacheKey);
				if(itemTypes) {
					parms.onSuccess(itemTypes);
					return;
				}       		
	       	} else {
	       		com.ibm.team.workitem.web.cache.internal.Cache.removeItem(itemTypesCacheKey);
	       	}
	
			// fetch the values required for the create work item drop down
			var serviceObj= {
				self: this,
				success: function(editProps) {
					if(!editProps || editProps.length == 0) {
						parms.onSuccess([]);
						return;
					}
					var itemTypes = editProps[0].uiItems || [];
					com.ibm.team.workitem.web.cache.internal.Cache.addItem(itemTypesCacheKey, itemTypes);
					parms.onSuccess(itemTypes);
				},
				failure: function(errorObj) {
					this.self.handleError(errorObj, this.self.messages.errorFetchingItemTypes);
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
	
			var appArgs = {projectAreaItemId: parms.projectAreaItemId, newWorkItem: true, ids: ["workItemType"]};
			WorkItemClient.getAllValues(srh, appArgs);
	},		
        /**
         * Returns <code>true</code> if the work item editor is displaying a new
         * work item, <code>false</code> if the work item editor is displaying an
         * existing work item.
         */
        isNewWorkItem: function(){
            if (!this.object) 
                return false;
            return this.getId() < 0;
        },
		isNew: function() {
			return this.isNewWorkItem();
		}, 
        _hasValue: function(array, value){
            for (var i = 0; i < array.length; i++) {
                if (array[i] == value) {
                    return true;
                }
            }
            return false;
        },
        fetchEditableLinkTypes: function(parms /* caller, onSuccess */) {
        	if(this.workItemSpec && this.workItemSpec.editableLinkTypes) {
        		if(parms.onSuccess) {
        			 return parms.caller[parms.onSuccess](this.workItemSpec.editableLinkTypes);
        		}
        	}
			// fetch the values required for the create work item drop down
			var serviceObj= {
				self: this,
				success: function(linkTypes) {
					if(!linkTypes || linkTypes.length == 0) {
						parms.onSuccess([]);
						return;
					}
					parms.caller.onSuccess(linkTypes);
				},
				failure: function(errorObj) {
					this.self.handleError(errorObj, this.self.messages.errorFailedToLoadEditableLinkTypes);
				}
			};
        	var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");    
			var args = {
                projectAreaItemId: this.getProjectAreaItemId()
            }
            com.ibm.team.workitem.web.client.internal.WorkItemClient.getLinkTypes(responseHandler, args);
		},
		/*
		 * Merge the parameter work item with the current work item, where this.object has beeh partially loaded and possibley changed (e.g., by plans)
		 * and the parameter is the result of a full fetch.
		 * The stateId of the current and fetched work items must be the same
		 * 
		 */
		mergeWorkItem: function(fetchedWorkItem) {
			if(!this.object || !fetchedWorkItem) return;
			var isChanged = false;
			/*
			 * Merge Attributes
			 */
			for(var key in fetchedWorkItem.attributes) {
//				if(key != "internalComments") { // comments are handled below
					if(this.object.attributes[key] == null) {
						isChanged = true;
//						this.object.attributes[key] = fetchedWorkItem.attributes[key];
//					}
						/*
						 * Must use setValue since there can be timing issues
						 */				
						this.setValue({
							ignoreChange: true, // don't dirty the work item for this
							isQuiet: true,
							isMerge: true,
							notifyEvenIfUnchanged: true,
							path: ["attributes" , key],
							attributeId: key,
							value: 	fetchedWorkItem.attributes[key]
						});
 				}
			}
			
			/*
			 * Merge links
			 */
			if(fetchedWorkItem.linkTypes && fetchedWorkItem.linkTypes.length > 0) {
				this.object.linkTypes = this.object.linkTypes || [];
				var newLinkTypeProxies = [];
				
				/*
				 * For each fetched link type, merge the existing link types
				 */
				for(var i=0;i<fetchedWorkItem.linkTypes.length;i++) {			
					var fetchedLinkTypeProxy = com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory.getLinkTypeProxy({linkType: fetchedWorkItem.linkTypes[i]});
 					newLinkTypeProxies.push(fetchedLinkTypeProxy);
					/*
					 * Add local changes to the fetched items... replace the fetched items of the same type with local 
					 */
					var linkTypeFound = false;
					for(var j=0;j<this.object.linkTypes.length; j++) {
						linkTypeFound = fetchedLinkTypeProxy.mergeChangesFromParameterToObject({ fromLinkType: this.object.linkTypes[j]});
						if(linkTypeFound) {
							this.object.linkTypes[j].__wasMerged = true;
							break;
						}
					} 
				}
				
				// Each fetched linkType is now in a  proxy and includes all the linkDTOs and changes from the current linkType
				// Now add the current link types to the proxy that did not exist
				for(var j=0;j<this.object.linkTypes.length; j++) {
					if(!this.object.linkTypes[j].__wasMerged) {
						var currentLinkTypeProxy = com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory.getLinkTypeProxy({linkType: this.object.linkTypes[j]});
 						newLinkTypeProxies.push(currentLinkTypeProxy);
					}
				}
				
				var newLinkTypes = [];
				for(var i=0;i<newLinkTypeProxies.length;i++) {
					newLinkTypes.push(newLinkTypeProxies[i].object);	
				}
				
				this.object.linkTypes = newLinkTypes;

			}

			/*
			 * Merge appprovals
			 *  - For now, assume that there are no approvals on the partially fetched object
			 */
			 this.object.approvals = fetchedWorkItem.approvals;
			 
			 this.refreshObject({justNotify: true});
		},
		
		/*
		 * This item has changed in the repository
		 * merge what we can (e.g. links)
		 * if keepChangedValues the do not replace current changed values
		 */
		mergeRepositoryChanges: function(parms /* isKeepingChangedValues */) {
			parms = parms || {};
			var workingCopy = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({
				id: this.getId(),
				createIfNeeded: true,
				doNotCache: true,
				doNotGetFromCache: true
			});
			
			var typeId = this.getValue({path: ["attributes", "workItemType", "id"]});
			var args = {
				onSuccess: dojo.hitch(this, function (workItemDTO) {
					parms.workItemDTO = workItemDTO;
					this._mergeRepositoryChangesFetchComplete(parms);
				}),
				isKeepingChangedValues: parms.isKeepingChangedValues,
				isSimpleFetch: true,
				fetchEditorPresentation: false,
				fetchInlineEditorPresentation: false,
				type: typeId
			}			
			workingCopy.fetchWorkItem(args);			
		},
		_mergeRepositoryChangesFetchComplete: function(parms) {
			/*
			 * The workItemDTO returned has attributes as a set of name-value pairs.
			 * Convert this to an object (as it is returned from fetchWorkItem:)
			 */
			
			this.mergeWorkItemDTO(parms);
		},
		mergeWorkItemDTO: function(parms) {
			if(!this.object || !parms ||  !parms.workItemDTO) return;
			
			var workItemDTO = parms.workItemDTO;
			var isKeepingChangedValues = parms.isKeepingChangedValues;
			
			if(workItemDTO.locationUri) {
				this.object.locationUri = workItemDTO.locationUri;
			}
			if(workItemDTO.stateId) {
				this.object.stateId = workItemDTO.stateId;
			}
			if(workItemDTO.itemId) {
				this.object.itemId = workItemDTO.itemId;
			}
			/*
			 * Merge Attributes
			 */
			for(var key in workItemDTO.attributes) {
				if((!isKeepingChangedValues || (!this.changedAttributes) || (this.changedAttributes && !this.changedAttributes[key])) && key != "internalComments") { // comments are handled below
					this.setValue({
							ignoreChange: true, // don't dirty the work item for this
							isQuiet: true,
							notifyEvenIfUnchanged: true,
							path: ["attributes" , key],
							attributeId: key,
							value: 	workItemDTO.attributes[key]
						});
				}
			}
			
			/*
			 * Attachment names are cached by the editor here... clear this, it will be reset on refresh
			 */
			this.object._attachmentNames = null;
			/*
			 * Merge links
			 */
				workItemDTO.linkTypes = workItemDTO.linkTypes || [];
				this.object.linkTypes = this.object.linkTypes || [];
		
				/*
				 * For each repo link type, merge the dtos into the object link types
				 */
				var repoLinkTypeProxies = {};
				for(var i=0;i<workItemDTO.linkTypes.length;i++) {				
					var repoLinkTypeProxy = com.ibm.team.workitem.web.cache.internal.LinkTypeProxyFactory.getLinkTypeProxy({linkType: workItemDTO.linkTypes[i]});
					repoLinkTypeProxies[repoLinkTypeProxy.object.id] = repoLinkTypeProxy;
					/*
					 * Merge the links read with the existing links
					 * If the repo link type is found in the cached link types, merge DTOs
					 * If the repo link type is not found in the cache, add the repo linked type to the cached version
					 * 
					 */
					for(var j=0;j<this.object.linkTypes.length; j++) {
					    if(repoLinkTypeProxy.isLinkTypeEqual({linkType: this.object.linkTypes[j]})) {
							repoLinkTypeProxy.mergeDTOsFromRepositoryToCache({linkType: this.object.linkTypes[j]})
					    	repoLinkTypeProxy._wasFound = true;
					    	break;
					    }
					}
					if(!repoLinkTypeProxy._wasFound) {
						this.object.linkTypes.push(repoLinkTypeProxy.object);
					}
				}
				/*
				 * For each cached linkType, delete the DTOs for any that were removed on the server
				 */
				for(var i=0;i<this.object.linkTypes.length;i++) {
					/*
					 * If a cached linkType is not in the repo, it was deleted, so delete it here
					 * by removing all it's DTOs
					 */
					if(!repoLinkTypeProxies[this.object.linkTypes[i].id]) {
						// preserve the new links
						var newItems = [];
						var dtos = this.object.linkTypes[i].linkDTOs
						for(var j=0;j<dtos.length;j++) {
							if(dtos[j]._isNew) {
								newItems.push(dtos[j]);
							}
						}
						this.object.linkTypes[i].linkDTOs = newItems;
					}
				}
				this.setValue({
					ignoreChange: true, // don't dirty the work item for this
					notifyEvenIfUnchanged: true,
					caller: this,
					path: ["linkTypes"],
					value: 	this.object.linkTypes // Current values
				});
				this.setHasNewLinksPending(false);


			/*
			 * Merge appprovals
			 */
			if(workItemDTO.approvals && workItemDTO.approvals.length > 0) {
				this.object.approvals = this.object.approvals || [];
				var newApprovals = [];
				/*
				 * Keep the new approvals 
				 */
				for(var i=0;i<this.object.approvals.length; i++) {
				    if(this.object.approvals[i] != null && this.object.approvals[i]._isNew) {
						newApprovals.push(this.object.approvals[i]);
				    }
				}
				/*
				 * Merge the dto approvals 
				 */
				for(var i=0;i<workItemDTO.approvals.length;i++) {		
					newApprovals.push(workItemDTO.approvals[i]);
				}
				this.object.approvals = newApprovals; 
				this.setValue({
					ignoreChange: true, // don't dirty the work item for this
					notifyEvenIfUnchanged: true,
					caller: this,
					path: ["approvals"],
					value: 	this.object.approvals // Current values
				});
			}
		
			
			/*
			 * Merge comments
			 */
			if(workItemDTO.attributes && workItemDTO.attributes.internalComments && workItemDTO.attributes.internalComments.items && workItemDTO.attributes.internalComments.items.length > 0) {
				var newComments = workItemDTO.attributes.internalComments;
				if(this.object && this.object.attributes && this.object.attributes.internalComments) {
					var oldComments = this.object.attributes.internalComments;
					oldComments.items = oldComments.items || [];
					var hasCommentItem = function(item) {
						for(var j=0;j<oldComments.items.length;j++) {
							if(oldComments.items[j].creationDate == item.creationDate && oldComments.items[j].content == item.content) {
								return true;
							}
						}
						return false;
					}
					for(var i=0;i<newComments.items.length;i++) {
						if(!hasCommentItem(newComments.items[i])) {
							oldComments.items.push(newComments.items[i]);
						}
					}
					this.setValue({
						ignoreChange: true, // don't dirty the work item for this
						notifyEvenIfUnchanged: true,
						path: ["attributes","internalComments"],
						attributeId: "internalComments",
						value: oldComments
					});

				} else {
					this.setValue({
						ignoreChange: true, // don't dirty the work item for this
						notifyEvenIfUnchanged: true,
						path: ["attributes" , "internalComments"],
						attributeId: "internalComments",
						value: 	workItemDTO.attributes.internalComments
					});
				}
			}
		},
	/*
	 * Return the changes to the value since the last fetch (or updated via save with applyDelta)
	 * Example: if(getCurrentAndFetchedValues({path: ["attributes", "internalPriority"]}).fetchedValue.id == HIGH_PRIORITY) doMoo();
	 * 
	 * This compares the proxy object value to the one in storedObject (using changeAttributes, etc.)
	 * 
	 * (non-Javadoc)
	 * @see com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient#getCurrentAndFetchedValues()
	 */
	//	public ValueChangeSinceFetchArgs getValueChangeSinceFetch(GetValueArgs argsWithPath);
		getCurrentAndFetchedValues: function(argsWithPath) {
			var result = {
				fetchedValue: {},
				currentValue: {},
				isChanged: false			
			}
			
			result.currentValue= this.getValue(argsWithPath);
			argsWithPath.isFromStoredObject = true;
			result.fetchedValue = this.getValue(argsWithPath);
			var key = null;
			if(argsWithPath && argsWithPath.path && argsWithPath.path.length > 0 ) {
				key = argsWithPath.path[argsWithPath.path.length-1];				
			}
			
			if(key=="linkTypes") {
				var saveArgs= this._getSaveAppArgs({widgetCommands: {}, widgetFormElements: {} });		
				result.isChanged = saveArgs.updateLinks && saveArgs.updateLinks.length > 0;
			} else {
				if(key == "approvals") {
					var saveArgs= this._getSaveAppArgs({widgetCommands: {}, widgetFormElements: {} });
					result.isChanged = saveArgs.updateApprovals && saveArgs.updateApprovals.length > 0;
				} else {
					if(key == "internalSubscriptions" && result.currentValue.items && result.fetchedValue.items) {
						if(!this._isAttributeEqual(result.currentValue.items, result.fetchedValue.items, key)) {
							result.isChanged = true;
						}
					} else {
						if(!this._isAttributeEqual(result.currentValue, result.fetchedValue, key)) {
							result.isChanged = true;
						}
					}
				}
			}
			
			
			return result;
		},
		/*
		 * Return the paths to all changed attributes and links, merging those in object and storedObject
		 * Return value is an array of args for "getValue", e.g.,  [ {path: ["attributes", "internalPriority"]} ]
		 * include: attributes, linkTypes, and approvals
		 * 
		 * (non-Javadoc)
		 * @see com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient#getPathsToChangedValues()
		 */
		getPathsToChangedValues: function() {
			var saveArgs= this._getSaveAppArgs({widgetCommands: {}, widgetFormElements: {} });		
			/*
			 * Appargs include:
			 *   attributeIdentifiers: [] // add each as "attributes.id"
			 *   attributeValues: [] // which we don't care about
			 *   updateLinks: [ "{json command object }" ] // includees links and attachments
			 *      if updateLinks, add "linkTypes" to the path
			 *   updateApprovals: [ "{json command object }" ]
			 *      if update links, add "approvals" to the path
			 */
			
			var result = [];
			if(saveArgs.attributeIdentifiers) {				
				for(var i=0;i<saveArgs.attributeIdentifiers.length;i++) {
					var path = {path:  ["attributes" , saveArgs.attributeIdentifiers[i]] };
					result.push(path);
				}
			}
			
			if(saveArgs.updateLinks && saveArgs.updateLinks.length > 0) {
				var path = { path: ["linkTypes"] };
				result.push(path);
			}
			
			if(saveArgs.updateApprovals) {
				var path = {path: ["approvals"] };
				result.push(path);
			}
			return result;
	},
	/*
	 * Return true if the value exists in the work item
	 * Example: if(isValueChangedSinceFetch({path: ["attributes", "internalPriority"]})) doFoo();
	 * 
	 * (non-Javadoc)
	 * @see com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient#isValueChangedSinceFetch()
	 */
//	public native boolean isValueChangedSinceFetch(GetValueArgs argsWithPath);
	isValueChangedSinceFetch: function(argsWithPath) {
		var paths = this.getPathsToChangedValues();
		var testPath = argsWithPath + "";
		for(var i=0;i<paths.length; i++) {
			if(testPath == (paths[i].path + "")) return true;
		}
		return false;
	},
	/*
	 * Return true if the value exists in the work item
	 * Example: if(isValueDefined({path: ["attributes", "internalPriority"]})) doFoo();
	 * 
	 * (non-Javadoc)
	 * @see com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient#isValueDefined()
	 */
//	public native boolean isValueDefined(GetValueArgs argsWithPath);
	isValueDefined: function(argsWithPath) {
		/*
		 * A value is "defined" if there is an object.attribute
		 */
		if(!this.object || !argsWithPath || !argsWithPath.path) return false;
		return this._getAttribute(argsWithPath.path, this.object) != null;
	},
	/*
	 * Return true if the value has been fetched (or updated via save with applyDelta)
	 * Example: if(isValueFetched({path: ["attributes", "internalPriority"]})) doBoo();
	 * 
	 * This says that the value was stored on the server, and is kept in storedObject.
	 * 
	 * (non-Javadoc)
	 * @see com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient#isValueFetched()
	 */
//	public boolean isValueFetched(GetValueArgs argsWithPath);
	isValueFetched: function(argsWithPath) {
		/*
		 * A value is "fetched" if there is an storedObject.attribute
		 */
		if(!this.storedObject || !argsWithPath || !argsWithPath.path) return false;
		return this._getAttribute(argsWithPath.path, this.storedObject) != null;
	},
        _handleDirtyingEvent: function(e){
			if (this._isDirty) 
                return;
            this.setIsDirty(true);
        },
        setIsDirty: function(isDirty){
            this._isDirty = isDirty;
			if(PlatformUI && PlatformUI.getWorkbench() != null) {
				if(isDirty) {
					PlatformUI.getWorkbench().setPageDirty("com.ibm.team.workitem", true); //$NON-NLS-1$
				} else {		
					com.ibm.team.workitem.web.cache.internal.CacheUtil.setPageDirty();
				}
			}
        },
        getIsDirty: function() {
        	return this._isDirty;
        },
		getErrorMessages: function(obj, msg) {
            var humanFriendlyMsg;
			obj = obj || {};
			if(obj.severity <= 2 && obj.infos && obj.infos.length > 0) {
				humanFriendlyMsg = obj.infos[0].summary; // use first message
			} else if (obj.status === 0) {
                humanFriendlyMsg = this.messages.errorServerNotResponding;
            }
			var summaryMsg;
			if(msg) {
			    summaryMsg = dojo.isString(msg) ? msg : null;
			}
            var detailedMsg = humanFriendlyMsg ? humanFriendlyMsg : obj.message;
            if (!detailedMsg || (detailedMsg.toLowerCase() == "not found")) {
                detailedMsg = this.messages.errorServerNotResponding;
            }
            else 
                if (detailedMsg.toLowerCase() == "unknown") {
                    detailedMsg = this.messages.errorUnknownDetails;
                }
			if(!summaryMsg && detailedMsg) summaryMsg = detailedMsg;
			return {
				summaryMsg: summaryMsg,
				detailedMsg: detailedMsg
			}			
		},
        handleError: function(parms){
			var messages = this.getErrorMessages(parms.error, parms.message);
            messages.detailedMsg = console.debug(dojo.string.substitute(this.messages.errorConsoleDetails, [messages.summaryMsg ? messages.summaryMsg : this.messages.labelUnknownError, messages.detailedMsg]));
			if(!parms.skipErrorMessage) {
				var msg =  dojo.string.substitute(this.messages.errorFetchStoreDetails, [this.getLinkLabel(),(messages.detailedMsg || messages.summaryMsg || parms.message) ]);
				dojo.publish(EventTopics.SHOW_ERROR_PAGE, [msg]);
				
				if(parms.error && parms.error.exception) {
					for(var i=0;i<parms.error.exception.length;i++) {
						console.debug(parms.error.exception[i]);
					}
				}
//	            this.showMessage(msg);
			}
        },
        // Show the message in the header (or somewhere)
        showMessage: function(msg){
            alert(msg);
        }
    });
    

})();
