 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxy");

(function() { //module container
var lastId = -1;

com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory = {	
	/*
	 * Create a new work item proxy
	 * This method is repeated in WorkItemProxy as _getNewWorkItemProxy (avoiding circular dependencies)
	 */
	getWorkItemProxy: function(parms /* id, createIfNeeded, incrementUseCount, doNotCache, doNotGetFromCache, isShownInRecentlyViewed */) {
		if(parms.isShownInRecentlyViewed != false) {
		   parms.isShownInRecentlyViewed = true;
		}
		var workItemProxy = null;
		var id = parms.id || lastId--;
		if(!parms.doNotGetFromCache) {	
			workItemProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
		}
		if(workItemProxy || !parms.createIfNeeded) {
			// if the work item is marked show in recently viewed (=false) leave it that way
			if(workItemProxy.isShownInRecentlyViewed != true) {
				workItemProxy.isShownInRecentlyViewed = parms.isShownInRecentlyViewed;
			} 
			return workItemProxy;
		}
		
		/* if ID is -1 then it is a new item and should be converted to a unique id */
		if(id == -1) {
			id = -1 * Date.parse(new  Date().toString())
		}
		
		var object= {
			attributes: {
				id: {
					id: id
				}
			}
		}
		workItemProxy =  new com.ibm.team.workitem.web.cache.internal.WorkItemProxy({ object: object, isFetched: false}); // empty object initially

		if(!parms.doNotCache) {
			workItemProxy.isShownInRecentlyViewed = parms.isShownInRecentlyViewed;
			com.ibm.team.workitem.web.cache.internal.Cache.addItem(id, workItemProxy);			
			if(parms.incrementUseCount) {
				workItemProxy.incrementUseCount();
			}
		} else {
			workItemProxy.isNotCached = true;
		}
		
		return workItemProxy;
	},
	/*
	 * The following is needed for unraveling the multiple inheritance protocol jDojo issue 
	 * TODO: Remove
	 */
	getWorkItemProxyClient: function(parms) {
		return this.getWorkItemProxy(parms); 
	 },
	 __sentinal: false
};

})();