/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.Attribute");

dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.api.common.Status");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ConfigurationParameters");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

(function() {
	
var ConfigurationParameters= com.ibm.team.workitem.web.client.internal.ConfigurationParameters;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var Status= com.ibm.team.workitem.api.common.Status;

var VALUE_PROVIDER= "valueProvider";
var VALIDATOR= "validator";

dojo.declare("com.ibm.team.workitem.web.client.internal.Attribute", null, {
	
	// API -------------------------------------------
	
	constructor: function(attribute /* AttributeDTO */) {
		
		this._attribute= attribute;
		this._descriptors= {};
		
		if (attribute.valueProvider) {
			for (var i= 0; i < attribute.valueProvider.length; i++) {
				var valueProvider= attribute.valueProvider[i];
				var props= com.ibm.team.workitem.web.client.util.buildMap(valueProvider.properties);
				var cfg= new ConfigurationParameters(dojo.fromJson(props.configuration));
				this._descriptors[valueProvider.type]= {
					ctor: null,
					configuration: cfg,
					scriptClass: valueProvider.scriptClass
				};
			}
		}
	},
	
	getIdentifier: function() {
		return this._attribute.id;
	},
	
	getLabel: function(name) {
		return this._attribute.label;
	},
	
	getAttributeTypeId: function() {
		return this._attribute.attributeTypeId;
	},
	
	dependsOn: function(attributeId) {
		var dependencies= this._attribute.dependentAttributes || [];
		return dojo.indexOf(dependencies, attributeId) > -1;
	},
	
	hasValueProvider: function() {
		return this._descriptors[VALUE_PROVIDER] ? true : false;
	},
	
	getValue: function(/* WorkItem */ workItem) {
		var attributeId= this.getIdentifier();
		var descriptor= this._descriptors[VALUE_PROVIDER];
		var valueProvider= this._createProvider(descriptor);
		if (valueProvider) {
			try {
				return valueProvider.getValue(attributeId, workItem, descriptor.configuration);
			} catch (e) {
				console.error(e);
			}
		}	
		return workItem.getValue(attributeId);
	},
	
	validate: function(/* WorkItem */ workItem) {
		var attributeId= this.getIdentifier();
		var descriptor= this._descriptors[VALIDATOR];
		var validator= this._createProvider(descriptor);
		if (validator) {
			try {
				return validator.validate(attributeId, workItem, descriptor.configuration);
			} catch (e) {
				console.error(e);
			}
		}	
		return Status.OK_STATUS;
	},
	
	getValueSet: function(/* WorkItem */ workItem, responseHandler) {
		
		var args= { 
			attributeName: this.getIdentifier(),
			projectAreaItemId: workItem.getProjectAreaItemId(),
			typeId: workItem.getWorkItemTypeId(),
			includeArchived: false
		};
		
		var dependencies= this._attribute.dependentAttributes;
		if (dependencies) {
			
			// use the same format as a work item post
			var identifiers= new Array();
			var values= new Array();
			
			for (var i= 0; i < dependencies.length; i++) {
				var dependencyValue= workItem.getValue(dependencies[i]);
				if (dependencyValue) {
					identifiers.push(dependencies[i]);
					values.push(dependencyValue);
				}
			}
			args.attributeIdentifiers= identifiers;
			args.attributeValues= values;
		}
		WorkItemClient.getValues(responseHandler, args);
	},
	
	isEnumeration: function() {
		return this._attribute.isEnumeration;
	},
	
	// Internal --------------------------------------
	
	_getPath: function() {
		return ["attributes", this.getIdentifier()];
	},
	
	_createProvider: function(descriptor) {
		if (descriptor != null) {
			if (descriptor.ctor == null && descriptor.scriptClass) {
				var clazz= descriptor.scriptClass;
				var ctor= dojo.getObject(clazz);
				if (ctor == null) {
					/*
					 * TODO: change to load_async
					 */
					jazz.core.loader.load_sync(clazz);
					ctor= dojo.getObject(clazz);
				}
				descriptor.ctor= ctor;
			}
			if (descriptor.ctor) {
				return new descriptor.ctor();
			}
		}
		return null;
	},
	
	// -----------------------------------------------
	
	__sentinel: null
});

})();