 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.QueryClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.workitem.web.client.internal.query.AttributeExpression");
dojo.require("com.ibm.team.workitem.web.client.internal.query.SimilarityExpression");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Variable");
dojo.require("jazz.app.context"); //$NON-NLS-1$

(function() { //module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var findIf					= net.jazz.ajax.toolbox.lang.findIf;
var removeIf				= net.jazz.ajax.toolbox.lang.removeIf;

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var QUERY_SERVICE_URI		= "com.ibm.team.workitem.common.internal.rest.IQueryRestService";
var QUERY_EDITOR_SERVICE_URI= "com.ibm.team.workitem.common.internal.rest.IQueryEditorRestService";

var WORKITEM_QUERY_TYPE		= "com.ibm.team.workitem.workItemType";

var Term= com.ibm.team.workitem.web.client.internal.query.Term;
var AttributeExpression= com.ibm.team.workitem.web.client.internal.query.AttributeExpression;
var SimilarityExpression= com.ibm.team.workitem.web.client.internal.query.SimilarityExpression;
var Variable= com.ibm.team.workitem.web.client.internal.query.Variable;

com.ibm.team.workitem.web.client.internal.QueryClient= {

	SCOPE_PERSONAL: 0,
	SCOPE_OTHER_CONTRIBUTORS: 1,
	SCOPE_TEAM_AREA: 2,
	SCOPE_PROJECT_AREA: 3,
	SCOPE_CROSS_REPOSITORY: 4,
	
	PROCESS_REPORT_OK: 0,
	PROCESS_REPORT_WARNING: 2,
	PROCESS_REPORT_ERROR: 4, 
	
	///////////////////////////////////////////////////////////////////////////////////
	// Service invocation helpers
	///////////////////////////////////////////////////////////////////////////////////

	_invokeJsonService: function(responseHandler, method, appArgs) {
		var actualArgs= {};
		if (appArgs) {
			var jsonArgs= dojo.toJson(appArgs); //0.9
			actualArgs.jsonString= jsonArgs;
			actualArgs.projectAreaItemId= jazz.app.context.get().itemId;
		}
		var serviceRequest= new ServiceRequest(QUERY_SERVICE_URI, method, actualArgs);
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	_invokeQueryService: function(responseHandler, method, appArgs) {
		this._invokeService(QUERY_SERVICE_URI, responseHandler, method, appArgs);
	},
	
	_invokeQueryEditorService: function(responseHandler, method, appArgs) {
		this._invokeService(QUERY_EDITOR_SERVICE_URI, responseHandler, method, appArgs);
	},
	
	_invokeService: function(service, responseHandler, method, appArgs) {
		
		var currentPA= jazz.app.context.get().itemId;
		
		// appArgs is optional, so test whether or not it was sent
		if (!appArgs) {
			var serviceRequest= new ServiceRequest(service, method, { projectAreaItemId : currentPA });
		} else {
			if (!appArgs.projectAreaItemId)
				appArgs.projectAreaItemId= currentPA;
			serviceRequest= new ServiceRequest(service, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// IQueryRestService API
	///////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * Loads a query by its itemId in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * @return The query including its common attributes like name, as a QueryDTO in its JavaScript representation.
	 * 
	 * @see IQueryRestService#getQuery
	 */
	getQuery: function(serviceResponseHandler, itemId, includeCount) {
		if (!itemId)
			itemId= "";
		includeCount= includeCount ? true : false;
		this._invokeQueryService(serviceResponseHandler, "getQuery", {itemId: itemId, includeCount: includeCount });
	},
	
	/**
	 * Loads a query by its itemId or id in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * 
	 * @param args.itemId (UUID) The itemId of the QueryDescriptor
	 * @param args.id (String) Alternatively, the id of the QueryDescriptor as specified in the process spec
	 * @param args.includeCount (boolean) Include the current result count of the query 
	 * @return The query including its common attributes like name, as a QueryDTO in its JavaScript representation.
	 * @see IQueryRestService#getQuery
	 */
	getQuery2: function(serviceResponseHandler, args) {
		this._invokeQueryService(serviceResponseHandler, "getQuery", args);
	},
	
	/**
	 * Loads a number of queries by their itemId in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * @return The queries (QueryDTO[])
	 * 
	 * @see IQueryRestService#getQueries
	 */
	getQueries: function(serviceResponseHandler, /* String[] */ itemIds, /* boolean */ includeCount) {
		if (!itemIds)
			itemIds= [];
		includeCount= includeCount ? true : false;
		this._invokeQueryService(serviceResponseHandler, "getQueries", {itemIds: itemIds, includeCount: includeCount });
	},

	/**
	 * Loads a number of queries by their itemId or id in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * 
	 * @param args.itemIds (UUID[]) The itemIds of the QueryDescriptors
	 * @param args.ids (String[]) The ids of the QueryDescriptors as specified in the process spec
	 * @param args.includeCount [boolean] Include the current result count of the queries
	 * @return The queries (QueryDTO[])
	 * @see IQueryRestService#getQueries
	 */
	getQueries2: function(serviceResponseHandler, args) {
		this._invokeQueryService(serviceResponseHandler, "getQueries", args);
	},
	
	/**
	 * Loads a number of queries by their sharing scope in partial state ({@link IQueryDescriptor#SMALL_PROFILE}, 
	 * excluding the expression.
	 * @return The query descriptors (ScopedQueriesDTO[])
	 * 
	 * @param scope See the SCOPE_ constants defined above
	 * 		0: personal queries
	 * 		1: individually shared queries
	 * 		2: queries shared by team area
	 * 		3: queries shared by project area
	 * @param scopeItemId 
	 * 		The itemId of the item corresponding to the scope: Contributor, TeamArea or Project Area
	 * @param includeCount 
	 * 		Include the current query counts
	 * @param includeParentScopes 
	 * 		Include the scopes above the one specified, e.g. queries shared individually with the user,
	 * 		queries shared with the contributor's team areas and queries shared across the current project area if scope == 0
	 * 
	 * @see IQueryRestService#getScopedQueries
	 */
	/* ScopedQueriesDTO[] */ getScopedQueries: function(serviceResponseHandler, /* int */ scope, /* String */ scopeItemId, /* boolean */ includeCount, /* boolean */ includeParentScopes, /* String */ projectAreaItemId, /* boolean */ suppressCrossRepoQueries) {
		includeCount= includeCount ? true : false;
		includeParentScopes= includeParentScopes ? true : false;
		suppressCrossRepoQueries= suppressCrossRepoQueries ? true : false;
		var params= {scope: scope, includeCount: includeCount, includeParentScopes: includeParentScopes, projectAreaItemId: projectAreaItemId, suppressCrossRepoQueries: suppressCrossRepoQueries };
		if (scopeItemId)
			params.scopeItemId= scopeItemId;
		this._invokeQueryService(serviceResponseHandler, "getScopedQueries", params);
	},
	/* ScopedQueriesDTO[] */ getScopedQueries2: function(serviceResponseHandler, args) {
		this.getScopedQueries(serviceResponseHandler, args.scope, args.scopeItemId, args.includeCount, args.includeParentScopes, args.projectAreaItemId, args.suppressCrossRepoQueries);
	},

	
	/**
	 * Breaks the query results down by a specific attribute
	 * 
	 * @param args.queryItemId (UUID) The itemId of the QueryDescriptor
	 * @param args.attributeId (String) The attribute id (see WorkItemAttributes.js)
	 * @param args.includeCount (boolean, optional, default=true) Include the current result count of the query
	 * @return The histogram (see HistogramDTO)
	 * @see IQueryRestService#getHistogram
	 */
	getHistogram: function(serviceResponseHandler, args) {
		this._invokeQueryService(serviceResponseHandler, "getHistogram", args);
	},
	
	
	/**
	 * @deprecated Use getEditableQuery instead
	 * 
	 * Loads a query by its itemId in full state ({@link IQueryDescriptor#FULL_PROFILE}, 
	 * including the expression.
	 * @return The query including its expression tree as a QueryDTO in its JavaScript representation.
	 * 
	 * @see IQueryRestService#getQuery
	 */
	getCompleteQuery: function(serviceResponseHandler, itemId, includeCount) {
		if (!itemId)
			itemId= "";
		includeCount= includeCount ? true : false;
		this._invokeQueryService(serviceResponseHandler, "getQuery", {itemId: itemId, includeExpression: true, includeCount: includeCount});
	},
	
	/**
	 * @deprecated Use getResultSet instead
	 * 
	 * Executes a query and returns one page of the result set. 
	 * The use of the parameters is different when the query is initially
	 * executed and for subsequent calls to fetch additional data.
	 * 
	 * The initial call has to specify startIndex, maxResults and jsonQuery.
	 * Subsequent calls have to specifyx startIndex, maxResults and token. The
	 * jsonQuery parameters is optional and only used if the query result identified
	 * by token is not available anymore and the query has to be executed.
	 * 
	 * @param startIndex The index of the first work item to be included in the result page,
	 * 		relative to the entire result set.
	 * @param maxResults The number of results that should be returned.
	 * @param token The token identifying the result set. This token is available
	 * 		from the WorkItemSummaryPage that was returned in the initial call.
	 *		Note: This is not true yet. The token field has to be added to WorkItemSummaryPage first.
	 * @param query
	 * 		The query as retrieved by getQueriesForOwner or getPredefinedQueries or getQuery. If the
	 * 		query is stored in the repository, a simple object like { itemId: ... } is sufficient, too.
	 * @param orderAttributes
	 *      An array of attribute identifiers that describes which columns are to be sorted.
	 *		E.g. [ "modifiedDate" ]
	 * @param orderDirections
	 *		A boolean array of the same size as orderAttributes, that specifies for each attribute if
	 *		the order direction should be ascending (true) or descending (false).
	 *		E.g. [ true ]
	 * 
	 * Returns the JavaScript representation of an IWorkItemSummaryPage
	 */
	execute: function(startIndex, maxResults, token, query, orderAttributes, orderDirections, serviceResponseHandler) {
		var args= new Object();
		args.startIndex= startIndex;
		args.maxResults= maxResults;
		args.token= token;
		args.jsonQuery= dojo.toJson(query); //0.9
		args.orderAttributes= orderAttributes;
		args.orderDirections= orderDirections;
		this._invokeQueryService(serviceResponseHandler, "postExecute", args);
	},
	
	/**
	 * @param {QueryDTO} query
	 * @param {Object} args
	 * 		An object with the following properties set:
	 * 		- {int} startIndex [0..]
	 * 		- {int} maxResults [0..500]
	 * 		- {String/UUID} projectAreaItemId
	 * 		- {String/UUID} itemId
	 * 		- {String} id
	 * 		- {String[]} columnIdentifiers (optional)
	 * 		- {Object[]} sortOrder (optional)
	 * 		- {UUID} token (optional)
	 * 		
	 * @param {Object} serviceResponseHandler
	 * @return A ResultSetDTO
	 */
	getResultSet: function(query, userArgs, serviceResponseHandler) {
		
		var args= dojo.clone(userArgs);
		
		if (!args.columnIdentifiers)
			args.columnIdentifiers= [];
			
		if (!args.filterAttribute)
			args.filterAttribute= "";

		if (!args.filterValue)
			args.filterValue= "";
			
		if (args.sortOrder) {
			var sortColumns= new Array();
			var sortDirections= new Array();
			for (var i= 0; i < args.sortOrder.length; i++) {
				var sortColumn= args.sortOrder[i];
				sortColumns.push(sortColumn.id);
				sortDirections.push(sortColumn.ascending);
			}
			args.sortColumns= sortColumns;
			args.sortDirections= sortDirections;
			delete args.sortOrder;
		}
			
		if (query) {
			
			if (query.itemId) {
				args.itemId= query.itemId;
			}
			
			if (query.id) {
				args.id= query.id;
			}
			
			if(query.projectAreaItemId) {
				args.projectAreaItemId = query.projectAreaItemId;
			}
			
			if (!args.jsonExpression) {
				if (query.internalExpression) {
					args.jsonExpression= dojo.toJson(query.internalExpression._term); //query.internalExpression.serialize();
				} else {
					if ("com.ibm.team.workitem.query.rest.dto:EditableQueryDTO" == query._eQualifiedClassName && query.expression) {
						var term= this._createTerm(query.expression);
						args.jsonExpression= term.serialize();
					} else {
						args.jsonQuery= dojo.toJson(query); //0.9
					}
				}
			}
		}
		this._invokeQueryService(serviceResponseHandler, "postGetResultSet", args);
	},
	
	_createTerm: function(termDTO) {

		var term= new Term(termDTO.operator);
		
		var attributeExpressionDTOs= termDTO.attributeExpressions || [];
		for (var i= 0; i < attributeExpressionDTOs.length; i++) {
			
			var aexDTO= attributeExpressionDTOs[i];
			var aex= new AttributeExpression(aexDTO.attribute.id, aexDTO.operator.id);
			
			var values= dojo.map(aexDTO.values || [], function(value) {
				return value.id;
			});
			
			var variables= dojo.map(aexDTO.variables || [], function(variable) {
				return new Variable(variable.id, variable.arguments);
			});
			
			aex.setValues(values);
			aex.setVariables(variables);
			term.addAttributeExpression(aex);
		}
		
		var terms= termDTO.termExpressions || [];
		for (var j= 0; j < terms.length; j++) {
			var subTerm= this._createTerm(terms[j]);
			term.addTerm(subTerm);
		}
		
		return term;
	},
	
	/**
	 * @deprecated Use getResultSet instead
	 * See IQueryRestService#getResults
	 */
	getResults: function(args, serviceResponseHandler) {
		this._invokeQueryService(serviceResponseHandler, "getResults", args);
	},
	
	/**
	 * See IQueryRestService#getQueriesForOwner.
	 */
	getQueriesForOwner: function(appArgs, serviceResponseHandler) {
		this._invokeQueryService(serviceResponseHandler, "getQueriesForOwner", appArgs);
	},
	
	/**
	 * See IQueryRestService#getDefaultQuery.
	 */
	getDefaultQuery: function(serviceResponseHandler) {
		this._invokeQueryService(serviceResponseHandler, "getDefaultQuery");
	},
	
	deleteQuery: function(itemId, serviceResponseHandler) {
		this._invokeQueryService(serviceResponseHandler, "postDeleteQuery", {uuid: itemId});
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Template API
	///////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * @param args.projectAreaItemId
	 */
	/* WorkItemTemplateDTO[] */ getWorkItemTemplates: function(serviceResponseHandler, args) {
		this._invokeQueryService(serviceResponseHandler, "getWorkItemTemplates", args);
	},
	
	/**
	 * @param args.projectAreaItemId
	 * @param args.templateId The template id from the WorkItemTemplateDTO
	 * @param args.variables An array of string values for the template variables. Currently the itemId of 'Filed Against' and 'Planned For'
	 * @return An operation report containing either an error message or a query that can be executed to show the template creation result
	 */
	/* OperationReportDTO */ instantiateWorkItemTemplate: function(serviceResponseHandler, args) {
		this._invokeQueryService(serviceResponseHandler, "postInstantiateWorkItemTemplate", args);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// IQueryEditorRestService API
	///////////////////////////////////////////////////////////////////////////////////
	
	/**
	 * @param userArgs Supported properties: includeChildren [true|false]
	 * @return TreeDTO<AttributeDTO>
	 */
	getAttributeTree: function(serviceResponseHandler, userArgs) {
		var args= userArgs ? dojo.clone(userArgs) : new Object();
		args.queryType= WORKITEM_QUERY_TYPE;
		this._invokeQueryEditorService(serviceResponseHandler, "getAttributeTree", args);
	},
	
	/* PresentationDTO */ getPresentation: function(serviceResponseHandler, /* String */ attributeId) {
		checkNotNull(attributeId);
		var args= { queryType: WORKITEM_QUERY_TYPE, attributeIdentifier: attributeId };
		this._invokeQueryEditorService(serviceResponseHandler, "getPresentation", args);
	},
	
	saveQuery: function(serviceResponseHandler, userArgs) {
		
		var args= dojo.clone(userArgs);
		args.queryType= WORKITEM_QUERY_TYPE;
		
		if (args.expression) {
			args.jsonExpression= args.expression.serialize();
			delete args.expression;
		}

		var setArguments= new Array();
		args.setArguments= setArguments;
		
		if (args.teamAreaAssociations)
			setArguments.push("teamAreaAssociations");
			
		if (args.contributorAssociations)
			setArguments.push("contributorAssociations");
			
		if (args.resultColumns)
			setArguments.push("resultColumns");
			
		if (args.sortColumns)
			setArguments.push("sortColumns");
			
		this._invokeQueryEditorService(serviceResponseHandler, "postSaveQuery", args);
	},
	
	getEditableQuery: function(serviceResponseHandler, /* UUID */ itemId) {
		
		var args= new Object();
		args.queryType= WORKITEM_QUERY_TYPE;
		args.itemId= itemId;
		
		this._invokeQueryEditorService(serviceResponseHandler, "getEditableQuery", args);
	},
	
	//////////////////////////////////////////////////////////////////////////////////
	// Client side query creation
	///////////////////////////////////////////////////////////////////////////////////
	
	/* QueryDTO */ createFulltextQuery: function(/* String */ queryName, /* String */ pattern, /* boolean */ openOnly) {
		
		var term= new Term(Term.AND);
		var patternExpression= new AttributeExpression("all", "contains", [ pattern ]);
		term.addAttributeExpression(patternExpression);
		
		if (openOnly) {
			var openStateVariable= new Variable("state", "open or in progress");
			var stateExpression= new AttributeExpression("internalState", "is");
			stateExpression.setVariables([ openStateVariable ]);
			term.addAttributeExpression(stateExpression);
		}
		
		return { name: queryName, internalExpression: term, itemId: ""};
	},
	
	createSimilarityQuery: function(queryName, itemId, summary, content) {

		var similarityExpression= new SimilarityExpression();
		if (arguments.length > 2)
			similarityExpression.setContent(summary, content);
		similarityExpression.setWorkItem(itemId);

		var term= new Term(Term.AND);
		term.addSimilarityExpression(similarityExpression);

		return { name: queryName, internalExpression: term, itemId: ""};
	},
	
	createWorkItemIdQuery: function(queryName, /* Work Item numbers, e.g., 93898 */ idArray) {
		if(!idArray  || idArray.length == 0) checkNotEmpty(idArray, "createWorItemIdQuery.idArray"); // %NON-NLS-1
		
		var attributeExpression= new AttributeExpression("id", "is", idArray);
		var term= new Term(Term.AND);
		term.addAttributeExpression(attributeExpression);
		return { name: queryName, internalExpression: term, values: idArray }
	}
	
};

})();