 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.internal.SimpleCache");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("jazz.app.context");

(function() {

var checkNotNull			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;
var ServiceRequest			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient		= com.ibm.team.repository.web.transport.TeamServerClient;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var SimpleCache				= com.ibm.team.workitem.web.client.internal.SimpleCache;
var Utils					= com.ibm.team.workitem.web.client.util;
var WorkItemAttributes		= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;


// Dynamicallly loaded scripts
var SCRIPT_DELIMITER		= "\r\n\r\n--a1a1bf4aa1b34a2db01610882541df6b----\r\n\r\n"; //$NON-NLS-1$
var PROVIDED_UNIT_REGEXP	= /dojo\.provide\(\"([^\"]+)\"\)/;

function validateWorkItemParams(appArgs, method) {
	
	// validate that we have any request parameters
	if (!appArgs) {
		throw new Error("WorkItemClient#" + method + ": missing request parameters.");
	}
	
	// validate that we have only one of 'id', 'itemId', or 'newWorkItem'
	var count= 0;
	var setStr;
	dojo.forEach(["id", "itemId", "newWorkItem"],
		function(x) {
			if (appArgs[x]) { count++; setStr= x + " "; }
		});
	if (count > 1) {
		throw new Error("WorkItemClient#" + method +
			": may only set one of 'id', 'itemId', or 'newWorkItem' (saw: [" +
			setStr + "]");
	}
}

var scriptsLoaded= false;
if(!dojo.isRhino) { // defect 109897, do not do the setTimeout for Rhino preprocessing
	window.setTimeout(function() {
		// we need to wait until the initialization data is available before we can load the scripts
		var contextProjectArea = jazz.app.context.get(); 
		if (contextProjectArea && contextProjectArea.itemId) {
			var url= TeamServerClient._getServicePrefix();
			url+= "com.ibm.team.workitem.service.internal.rest.IScriptProviderService"; //$NON-NLS-1$
			url+= "?projectAreaItemId=" + contextProjectArea.itemId; //$NON-NLS-1$
	
			dojo.xhrGet({
		        url: url,
				handleAs: "text", //$NON-NLS-1$
				handle: function(response, ioArgs){
					var scripts= String(response).split(SCRIPT_DELIMITER);
					for (var i= 0; i < scripts.length; i++) {
						var match= PROVIDED_UNIT_REGEXP.exec(scripts[i]);
						if (match && dojo._loadedModules[match[1]]) {
							continue;
						}
						if(scripts[i] && scripts[i].length>0) {
							eval(scripts[i]);
						}
					}					
					scriptsLoaded= true;
				},
				error: function() {
					console.error("Failed to load work item process attachment scripts"); //$NON-NLS-1$
					// let subsequent operations fail rather than keeping it in an endless wait
					scriptsLoaded= true;
				}
			});
		} else {
			// do not use setInterval, the scripts might get loaded more than once (see defect 65892)
			window.setTimeout(arguments.callee, 100);
		}
	}, 100);
}
/**
 * Provides helper methods to fetch workitem-related data from the team server.
 */
com.ibm.team.workitem.web.client.internal.WorkItemClient= {

	// service URI
	SERVICE_URI: "com.ibm.team.workitem.common.internal.rest.IWorkItemRestService",
	
	// get methods
	GET_VALUES_METHOD: "getValues",
	GET_CONTRIBUTORS_METHOD: "getContributors",
	
	/**
	 * see #getAttributes. Returns the attributes that can be used in a call to QueryClient.getHistogram().
	 */
	FILTER_HISTOGRAM: "histogram",
	
	_simpleCache: new SimpleCache(),

	/*
	 * Invokes the work item REST service specified by 'method' with optional
	 * request parameters specified by 'appArgs'.  Calls back to the functions
	 * defined by 'responseHandler'.
	 */
	_invokeWorkItemService: function(responseHandler, method, appArgs) {
		
		// appArgs is optional, so test whether or not it was sent
		if (arguments.length === 2) {
			var serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method);
		} else {
			serviceRequest=
					new ServiceRequest(WorkItemClient.SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	_invokeCachingWorkItemService: function(responseHandler, method, appArgs, cacheKey, ttl) {
		if(appArgs && appArgs.projectAreaItemId) {
			cacheKey = cacheKey + "/projectAreaItemId:"  + appArgs.projectAreaItemId; 
		}
		var self= this;
		this._invokeCachingService(responseHandler, cacheKey, ttl, function(handler) {
			self._invokeWorkItemService(handler, method, appArgs);
		});
	},
	
	_invokeCachingService: function(responseHandler, cacheKey, ttl, actualMethod) {
		
		var cache= this._simpleCache;
		var entry= cache.get(cacheKey);
		
		if (entry != null) {
			responseHandler.doCallback(dojo.clone(entry.result));
			return;
		}
		
		var internalHandler= {
			success: function(result) {
				cache.put(cacheKey, result, ttl);
				responseHandler.doCallback(dojo.clone(result));
			},
			failure: function(error) {
				cache.clear(cacheKey);
				responseHandler.doErrorCallback(error);
			}
		};
		
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		actualMethod(handler);
	},

/*
 * Get the project area item id, either from the args or the default
 */
	_setAppArgsProjectAreaItemId: function( appArgs) {
		if (!appArgs) appArgs= {};
		if(appArgs.projectAreaItemId) {
			appArgs.projectAreaItemId = appArgs.projectAreaItemId.replace(/\"\"/gi, "\"" ); // TODO: find source of double quotes
			return;
		} 
		var context = jazz.app.context.get();
		if(context.itemId){
			// context isn't always a project area
			appArgs.projectAreaItemId = context.itemId
		}
	},
	/*
	 * 
	 */
	getWorkItemEditableProperties: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemEditableProperties");
		if (appArgs.newWorkItem) {
			this._setAppArgsProjectAreaItemId(appArgs); // TODO:newonly?
		}
		
		WorkItemClient._invokeWorkItemService(responseHandler, "getWorkItemEditableProperties", appArgs);
	},
	
	getAllValues: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getAllValues");
		if (appArgs.newWorkItem) {
			this._setAppArgsProjectAreaItemId(appArgs); // TODO:newonly?
		}
		if (!appArgs.ids) {
			appArgs.ids= [];
		}
		WorkItemClient._invokeWorkItemService(responseHandler, "getAllValues", appArgs);
	},
	
	getValues: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		WorkItemClient._invokeWorkItemService(responseHandler, "getValues", appArgs);
	},
	
	getTime: function(responseHandler, appArgs) {
		WorkItemClient._invokeWorkItemService(responseHandler, "getTime", appArgs);
	},
	
	/**
	 * Returns a set of owners; dependent on project area, category, and target.
	 */
	getOwners: function(responseHandler, appArgs) {
		appArgs.attributeName= WorkItemAttributes.OWNER;
		return this.getValues(responseHandler, appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemDTO.
	 */
	getWorkItemDTO: function(responseHandler, appArgs) {
		validateWorkItemParams(appArgs, "getWorkItemDTO");
		if (appArgs.newWorkItem) {
			this._setAppArgsProjectAreaItemId(appArgs); // TODO:newonly?
		}
		this._invokeWorkItemService(responseHandler, "getWorkItemDTO", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#getWorkItemDTO2.
	 */
	getWorkItem: function(responseHandler, appArgs) {
		if (appArgs.newWorkItem) {
			this._setAppArgsProjectAreaItemId(appArgs); // TODO:newonly?
		}
		
		var internalHandler= {
			success: function(workItemDTO) {
				if(workItemDTO == null) {
					responseHandler.doCallback(null);
					return;
				}
				var attributeMap= Utils.buildMap(workItemDTO.attributes)
				workItemDTO.attributes= attributeMap;
//				console.debug(workItemDTO);
				responseHandler.doCallback(workItemDTO);
			},
			failure: function(error) {
				responseHandler.doErrorCallback(error);
			}
		};
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		this._invokeWorkItemService(handler, "getWorkItemDTO2", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#postWorkItem2.
	 */
	postWorkItem: function(responseHandler, appArgs) {
		if (!appArgs.stateId && !appArgs.projectAreaItemId) {
			this._setAppArgsProjectAreaItemId(appArgs);
		}
		this._internalPostWorkItem("postWorkItem2", responseHandler, appArgs);
	},

	postWorkItemN: function(responseHandler, appArgsList) {
		var serializedAppArgs= dojo.map(appArgsList, function(appArgs) {
			if (!appArgs.stateId && !appArgs.projectAreaItemId) {
				this._setAppArgsProjectAreaItemId(appArgs);
			}
				
			appArgs.attributeValues= dojo.map(appArgs.attributeValues, function(value) {
				if (dojo.isArray(value))
					return dojo.toJson(value);
				return value;
			});

			return dojo.toJson(appArgs);
		}, this);

		var internalHandler= {
			success: function(operationReportDTO) {
				dojo.forEach(operationReportDTO.workItems, function(workItemDTO) {
					workItemDTO.attributes= Utils.buildMap(workItemDTO.attributes)
				});
				
				responseHandler.doCallback(operationReportDTO);
			},
			failure: function(error) {
				console.debug(error);
				responseHandler.doErrorCallback(error);
			}
		};
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		this._invokeWorkItemService(handler, "postWorkItemN", { postWorkItemParams: serializedAppArgs });
	},

	/**
	 * See IWorkItemRestService#postMoveWorkItem.
	 */
	moveWorkItem: function(responseHandler, appArgs) {
		if (appArgs.move == null|| !appArgs.projectAreaItemId) {
			throw new Error("WorkItemClient#moveWorkItem: 'move' (true/false) and 'projectAreaItemId' must be set");
		}
		this._internalPostWorkItem("postMoveWorkItem", responseHandler, appArgs);
	},

	getUsesTimeRemaining: function(responseHandler, appArgs) {
		appArgs = appArgs || {};
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getUsesTimeRemaining", appArgs);
	},
	
	/**
	 * See IWorkItemRestService#postSplitWorkItem.
	 */
	splitWorkItem: function(responseHandler, appArgs) {
    if (!appArgs.stateId && !appArgs.projectAreaItemId) {
      this._setAppArgsProjectAreaItemId(appArgs);
    }
		this._internalPostWorkItem("postSplitWorkItem", responseHandler, appArgs);
	},
	
	_internalPostWorkItem: function(serviceName, responseHandler, appArgs) {
		
		appArgs.attributeValues= dojo.map(appArgs.attributeValues, function(value) {
			if (dojo.isArray(value))
				return dojo.toJson(value);
			return value;
		});
		
		var internalHandler= {
			success: function(operationReportDTO) {
				if (operationReportDTO.severity <= WorkItemClient.INFO) {
					var workItemDTO= operationReportDTO.workItem;
					if(operationReportDTO.infos) {
						workItemDTO.infos = operationReportDTO.infos;
					}
					var attributeMap= Utils.buildMap(workItemDTO.attributes)
					workItemDTO.attributes= attributeMap;
					responseHandler.doCallback(workItemDTO);
				} else {
					responseHandler.doErrorCallback(operationReportDTO);
				}
			},
			failure: function(error) {
				console.debug(error);
				responseHandler.doErrorCallback(error);
			}
		};
		var handler= new ServiceResponseHandler(internalHandler, "success", "failure");
		this._invokeWorkItemService(handler, serviceName, appArgs);
	},
	
	/**
	 * See IWorkItemRestService#postBulkChangeWorkItems.
	 * @param appArgs
	 * 		itemIds: an array of work item itemIds
	 * 		attributeIdentifiers: the identifiers of the attributes to be changed
	 * 		attributeValues: the new values for the attributes. Can be JS arrays of simple values.
	 */
	postBulkChangeWorkItems: function(responseHandler, appArgs) {
		
		appArgs.attributeValues= dojo.map(appArgs.attributeValues, function(value) {
			if (dojo.isArray(value))
				return dojo.toJson(value);
			return value;
		});
		
		this._invokeWorkItemService(responseHandler, "postBulkChangeWorkItems", appArgs);
	},
	
	getResolutions: function(responseHandler, appArgs) {
		if (!(appArgs.status || appArgs.actionId)) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (none set)");
		}
		if (appArgs.status && appArgs.actionId) {
			throw new Error("WorkItemClient#getResolutions: must set either appArgs.status or appArgs.actionId (both set)");
		}
		
		this._setAppArgsProjectAreaItemId(appArgs);
		appArgs.attributeName= "internalResolution";
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_VALUES_METHOD, appArgs);
	},
	
	getContributors: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, WorkItemClient.GET_CONTRIBUTORS_METHOD, appArgs);
	},
	
	getTeamMembers: function(responseHandler, appArgs) {
		 appArgs = appArgs || {};
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getTeamMembers", appArgs);
	},
	
	getEditorPresentation: function(responseHandler, appArgs) {
		if (!appArgs.typeId && !appArgs.itemId) {
			throw new Error("WorkItemClient#getEditorPresentation: must set appArgs.typeId (the work item type id) or appArgs.itemId (the work item itemId)");
		}
		this._setAppArgsProjectAreaItemId(appArgs);
		
		var cacheKey = "getEditorPresentation/typeId:" + appArgs.typeId + "/target:" + (appArgs.target ? appArgs.target : "(none)");  			

		this._invokeCachingWorkItemService(responseHandler, "getEditorPresentation", appArgs, cacheKey,  SimpleCache.TTL_INFINITE);

	},
	
	getQuickInformationPresentation: function(responseHandler, appArgs) {
		if (!appArgs.configurationId) {
			throw new Error("WorkItemClient#getQuickInformationPresentation: must set appArgs.configurationId");
		}
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeCachingWorkItemService(responseHandler, "getQuickInformationPresentation", appArgs, ("getQuickInformationPresentation/configurationId:"+ appArgs.configurationId),  SimpleCache.TTL_INFINITE);
	},
	
	getRequiredAttributesConfiguration: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, "getRequiredAttributesConfiguration", appArgs);
	},
		
	getApprovalStates: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeCachingWorkItemService(responseHandler, "getApprovalStates", appArgs, "getApprovalStates", SimpleCache.TTL_INFINITE);
	},
		
	getApprovalTypes: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeCachingWorkItemService(responseHandler, "getApprovalTypes", appArgs, "getApprovalTypes", SimpleCache.TTL_INFINITE);
	},
		
	getLinkTypes: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeCachingWorkItemService(responseHandler, "getLinkTypesForProject", appArgs, "getLinkTypesForProject", SimpleCache.TTL_INFINITE);
	},
	
	getReleases: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getReleases", appArgs);
	},
	
	getWorkItemCategories: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getWorkItemCategories", appArgs);
	},
	
	getWorkItemTimeLines: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getWorkItemTimeLines", appArgs);
	},
	
	getWorkItemTeamAreas: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getWorkItemTeamAreas", appArgs);
	},
	
	postDeleteWorkItems: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, "postDeleteWorkItems", appArgs);
	},
	
	postReleases: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "postReleases", appArgs);
	},
	
	postWorkItemCategories: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "postWorkItemCategories", appArgs);
	},

	/*
	 * Post multiple work items
	 * { 
	 *     s: The string form of an {@link IWorkItem} "[itemId];[stateId]".
	 *     a:  list of attributes to return. An empty or null array means all attributes
	 * 	   includeLinks: false: no links will be included (default=false)
	 * 	   includeApprovals: false: no approvals will be included (default=false)
	 * }
	 */
	postGetWorkItems: function(responseHandler, appArgs) {
 		this._setAppArgsProjectAreaItemId(appArgs);
 		this._invokeWorkItemService(responseHandler, "postGetWorkItems", appArgs);
	},
	
	getEnumerationValues: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		this._invokeWorkItemService(responseHandler, "getEnumerationValues", appArgs);
	},
	
	/**
	 * Returns all work item attributes of a project area
	 * @param appArgs.projectAreaItemId the item id of the project area (required)
	 * @param appArgs.filter an optional filter. Currently supported: WorkItemClient.FILTER_HISTOGRAM
	 */
	getAttributes: function(responseHandler, appArgs) {
		this._setAppArgsProjectAreaItemId(appArgs);
		var key= "getAttributes/" + dojo.toJson(appArgs);
		this._invokeCachingWorkItemService(responseHandler, "getAttributes", appArgs, key, SimpleCache.TTL_INFINITE);
	},
	
	getProcessAreaHierarchies: function(responseHandler) {
		
		var cacheKey= "getProcessAreaHierarchies";
		var ttl= SimpleCache.TTL_MINUTE * 30;
		
		this._invokeCachingService(responseHandler, cacheKey, ttl, com.ibm.team.process.web.client.internal.ProcessClient.getProcessAreaHierarchies);
	},
	
	/**
	 * Returns all iterations of the project area. Optionally included archived ones
	 * @param appArgs.projectAreaItemId the item id of the project area (required)
	 * @param appArgs.includeArchived if 'true', also returns the archived iterations
	 */ 
	getIterations: function(responseHandler, appArgs) {
		var ttl= SimpleCache.TTL_MINUTE * 30;
		this._invokeCachingWorkItemService(responseHandler, "getIterations", appArgs, "getIterations", ttl);
	},
	
	transformWikiSource: function(responseHandler, appArgs) {
		this._invokeWorkItemService(responseHandler, "postWikiSource", appArgs);
	}
};

// must be defined after com.ibm.team.workitem.web.client.internal.WorkItemClient
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;

// See IProcessReport
WorkItemClient.INFO= 0;
WorkItemClient.WARNING= 2;
WorkItemClient.ERROR= 4;

})();