 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.client.internal.WorkItemConfigClient");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

var ServiceRequest			= com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient		= com.ibm.team.repository.web.transport.TeamServerClient;

/**
 * Provides helper methods to fetch workitem-related data from the team server.
 */
com.ibm.team.workitem.web.process.client.internal.WorkItemConfigClient = {

	// service URI
	SERVICE_URI: "com.ibm.team.workitem.service.process.internal.rest.IWorkItemConfigRestService",
	
	setCurrentProjectArea: function(){
		var projectAreaEditors = dijit.registry.byClass("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor").toArray();
		if(projectAreaEditors.length > 0){
			this._currentProjectArea = projectAreaEditors[0];
		}
	},
	
	getCurrentProjectArea: function(){
		return this._currentProjectArea;
	},
	
	/*
	 * Invokes the work item REST service specified by 'method' with optional
	 * request parameters specified by 'appArgs'.  Calls back to the functions
	 * defined by 'responseHandler'.
	 */
	_invokeWorkItemService: function(responseHandler, method, appArgs) {
		
		// appArgs is optional, so test whether or not it was sent
		var serviceRequest;
		if (arguments.length === 2) {
			serviceRequest=
					new ServiceRequest(this.SERVICE_URI, method);
		} else {
			serviceRequest=
					new ServiceRequest(this.SERVICE_URI, method, appArgs);
		}
		TeamServerClient.invokeService(serviceRequest, responseHandler);
	},
	
	getWorkItemTypes: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getWorkItemTypes", appArgs);
	},

	getAttributeTypes: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getAttributeTypes", appArgs);
	},

	getProcessAttachments: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getProcessAttachments", appArgs);
	},
	
	getPresentations: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getPresentations", appArgs);
	},
	
	getWorkflows: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getWorkflows", appArgs);
	},
	
	getOSLCConfigurations: function(responseHandler, appArgs) {
		appArgs.projectAreaItemId= this.getCurrentProjectArea().projectAreaItemId;
		this._invokeWorkItemService(responseHandler, "getOSLCConfigurations", appArgs);
	},
	
	postConfigurations: function(responseHandler, appArgs) {
		var projectArea = this.getCurrentProjectArea();
		appArgs.projectAreaItemId= projectArea.projectAreaItemId;
		appArgs.stateId= projectArea.stateId;
		this._invokeWorkItemService(responseHandler, "postWorkItemConfigurations", appArgs);
	}
};

})();