/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.client.util"); 

dojo.require("dojo.date");
dojo.require("dojox.date.posix");
dojo.require("dojo.i18n");
dojo.require("dojox.string.Builder");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative		= net.jazz.ajax.toolbox.lang.checkNonNegative;

var HEX_DIGITS				= ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E","F"];
var WORKITEM_ID_REGEX		= /([0-9]+)/;
var UUID_LENGTH				= 23;

var CALM_QUERY_GLOBAL_IDENTIFIER= "_tuUNgFU_EeCjQbYWf-8ryQ";

// constants for this.breakupLongWords
if (dojo.isIE && dojo.isIE < 7) {
	var MIN_BREAKUP_LEN			= 32;
	var MAX_LEN					= 40;
} else {
	var MIN_BREAKUP_LEN			= 40;
	var MAX_LEN					= 52;
}

var DELIMITERS				= [".", "-", "_", "/", "?", "#"];

// For performance, do not load messages in this util

com.ibm.team.workitem.web.client.util= {
	_basicPrefix: "#action=com.ibm.team.workitem.",
	
	// DO NOT DELETE without first doing a search across the workitem.web
	// plug-in (being used by WorkItemOverview.js as of 8 Dec 2006)
	_viewWorkItemUriPrefix: "#action=com.ibm.team.workitem.viewWorkItem&id=",
	getEditQueryUri: function(queryId) {
		checkNotNull(queryId, "queryId");
		return this._basicPrefix + "editQuery&id=" + queryId;
	},
	getWelcomePageUri: function(msg) {
		var suffix = "";
		if(msg) {
			suffix = "&error=" + escape(msg);
		} 
		return this._basicPrefix + "viewWelcome" + suffix;
	},
	getNewWorkItemUri: function(tempId, typeName) {
		var suffix = typeName ? "&type=" + typeName : "";
		if(!tempId) {
			return this._basicPrefix + "newWorkItem" + suffix;;
		}
		return this._basicPrefix + "newWorkItem" + "&ts=" + tempId + suffix;
	},
	getNewWorkItemUri2: function(parms) {
		parms.typeId = parms.typeId || parms.typeName; // typeName is deprecated
		var suffix = parms.typeId ? "&type=" + parms.typeId : "";
		if(parms.newWorkItemProjectAreaItemId) {
			suffix += "&projectAreaItemId=" + parms.newWorkItemProjectAreaItemId;
		}
		if(parms.tempId) {
			suffix = suffix +  "&ts=" + parms.tempId;
		} else {
			suffix = suffix +  "&ts=" + this.getUniqueId();
		}
		if(parms.addLinkType && parms.addLinkedItemId && parms.addLinkedProjectAreaItemId) {
			suffix = suffix + "&addLinkedItemId=" + parms.addLinkedItemId
			                + "&addLinkType=" + parms.addLinkType
			                + "&addLinkIsSource=" + (parms.addLinkIsSource == true)
			                + "&addLinkedProjectAreaItemId=" + parms.addLinkedProjectAreaItemId; 
		}
	return this._basicPrefix + "newWorkItem" + suffix;
	},
	getNewWorkItemSetUri: function() {
		return this.reuniquifyUrl(this._basicPrefix + "newWorkItemSet");
	},

	getNewQueryUri: function(isUnique) {
		if(!isUnique) {			
			return this._basicPrefix + "newQuery";
		}
		return this.reuniquifyUrl(this._basicPrefix + "newQuery");		
	},
	getShowQueriesUri: function() {
		return this._basicPrefix + "viewQueries";
	},
	getMyQueriesUri: function() {
		return this.getShowQueriesUri() + "&tab=owned";
	},
	getSharedQueriesUri: function() {
		return this.getShowQueriesUri() + "&tab=shared";
	},
	getRunDefaultQueryUri: function(includeTimestamp) {
		if(includeTimestamp) {
			return this.reuniquifyUrl(this._basicPrefix + "runDefaultQuery");			
		} 
		return this._basicPrefix + "runDefaultQuery";
	},
	getShowTemplateQueryUri: function() {
		return this.reuniquifyUrl(this._basicPrefix + "showTemplateQuery");			
	},
	/*
	 * Image background URL's on IE must be fully qualified so IE does not show the mixed-mode message
	 */
	getBackgroundImageUrl: function(fragment) {
		    fragment = fragment || "";
			if(!window.location.href) return fragment;
			var hrefArray = window.location.href.split("/");
			if(hrefArray.length < 3) return fragment;
 			var result = hrefArray[0] + "//"; // https:
// 			skip hrefArray[1] which is a "" between the //
 			result += hrefArray[2]; // server:port
 			if(fragment.indexOf("/") != 0) result += "/"
			return result + fragment;
	},
	isChildOf: function(startNode, parentNode) {
			var testNode = startNode;
			if(parentNode == testNode) return true;

			// Check to see if the popup is a parent of the target
			var priorParent = null;
			while(testNode != null && testNode != priorParent && testNode != document.body && testNode != document) {
				priorParent = testNode;
				if(testNode==parentNode) {
					return true;
				}
				testNode=testNode.parentNode;				
			}
			return false;
	},
	/*
	 * Answer true if the node is a child of a dijit popup 
	 */	
	isChildOfPopup: function(startNode) {

			var testNode = startNode;
			var priorParent = null;
			while(testNode != null && testNode != priorParent && testNode != document.body && testNode != document) {
				priorParent = testNode;
				if(testNode.dijitPopupParent) {
					return true;
				}
				testNode=testNode.parentNode;				
			}
			return false;
	},	
	getRunSavedQueryUri: function(queryId, index, orderBy, direction, itemsPerPage, isUnique, preserveFilter, filterAttribute, filterValue) {
		checkNotNull(queryId, "queryId");
		return this._basicPrefix + "runSavedQuery&id=" + queryId +
				(index ? "&index=" + index : "") +
				(orderBy ? "&orderBy=" + orderBy : "") +  
				(direction ? "&direction=" + direction : "") + 
				(itemsPerPage ? "&itemsPerPage=" + itemsPerPage : "") +
				(preserveFilter ? "&preserveFilter=true" : "") +
				(filterAttribute ? "&filterAttribute=" + filterAttribute : "") +
				(filterValue ? "&filterValue=" + encodeURIComponent(filterValue) : "") +
				this._getUniqueSuffix(isUnique);
	},
	getRelativeQueryLocationUri: function(scopedItemId) {
		// CLM queries need special handling
		var queryItemId= scopedItemId;
		var queryPart= "";
		if (scopedItemId.length > UUID_LENGTH) {
			var itemId= scopedItemId.substring(0, UUID_LENGTH);
			var projectAreaItemId= scopedItemId.substring(UUID_LENGTH);
			queryItemId= CALM_QUERY_GLOBAL_IDENTIFIER;
			queryPart= "?itemId=" + itemId + "&projectAreaItemId=" + projectAreaItemId;
		}
		return "/resource/itemOid/com.ibm.team.workitem.query.QueryDescriptor/" + queryItemId + queryPart;
	},
	getRefreshSavedQueryUri: function(queryId) {
		var uri= this.getRunSavedQueryUri(queryId);
		return uri + "&refresh=true";
	},
	getRunNewQueryUri: function(/* QueryDTO */ query, /* int */ index, /* String */ orderBy, /* String */ direction, /* String */ itemsPerPage, preserveFilter) {
		checkNotNull(query, "query");
		var serialized= this.serializeQuery(query);
		return this._basicPrefix + "runNewQuery&q=" + serialized +
				(index ? "&index=" + index : "") +
				(orderBy ? "&orderBy=" + orderBy : "") + 
				(preserveFilter ? "&preserveFilter=true" : "") +
				(direction ? "&direction=" + direction : "") +
				(itemsPerPage ? "&itemsPerPage=" + itemsPerPage : "");
	},
	getSimilarityQueryUri: function( /* itemId, content, summary, projectAreaItemId */ parms) {
		var projectAreaTerm = parms.projectAreaItemId ? "&projectAreaItemId=" + parms.projectAreaItemId : "";
		if(parms.itemId) { // Existing work item search
			return this._basicPrefix + "similarityQuery&itemId=" + parms.itemId + projectAreaTerm;
		}
		var summary, content;
		if(parms.summary && parms.summary.length>0) {
			summary = "&summary=" + encodeURIComponent(dojo.trim(parms.summary)) 
		} else {
			summary= "";
		}

		if(parms.content && parms.content.length>0) {
			content = "&content=" + encodeURIComponent(dojo.trim(parms.content)) 
		} else {
			content= "";
		}
		
		return this._basicPrefix + "similarityQuery" + projectAreaTerm + summary + content;
	},
	getViewWorkItemUri: function(id, tab) {
		checkNotNull(id, "id-util3");
		return this._getViewSingleWorkItemUri(id, tab);
	},
	getUrlParam: function(url, param) {
	  var exp = "[.*&]"+param+"=([^&#]*)";
  	  var regex = new RegExp(exp);
  	  var result = regex.exec(url);
      if( !result || result.length < 2) return null;
      return result[1];
	},
	_getViewSingleWorkItemUri: function(id, tab) {
		checkNotNull(id, "id-util2");
	
		var idNum= parseInt(id, 10);
		if(isNaN(idNum)) return null;
		checkNonNegative(idNum, "idNum");
		
		return  this._basicPrefix + "viewWorkItem&id=" + id + (tab ? "&tab=" + tab : "");
	},
	getQueryURI: function(query, index, orderBy, direction, itemsPerPage, preserveFilter) {
		return this.getQueryUri(query, index, orderBy, direction, itemsPerPage, preserveFilter);
		},
	getQueryUri: function(queryDto, index, orderBy, direction, itemsPerPage, preserveFilter) {
		checkNotNull(queryDto, "query");
		
  		if(!queryDto.isNew && queryDto.itemId)  {
  			return this.getRunSavedQueryUri(queryDto.itemId, index, orderBy, direction, itemsPerPage, false, preserveFilter);
  		}
	  		
  		return this.getRunNewQueryUri(queryDto, index, orderBy, direction, itemsPerPage, preserveFilter);
	},
	/*
	 * Return the fragment, which is the part after the first #
	 */
	getFragment: function(url) {
		return new dojo._Url(url).fragment; /* TODO: Remove reference to internal dojo method */
	},
	getSearchUri: function(unencodedStr, openOnly, isUnique) {
		checkNotNull(unencodedStr, "unencodedStr");
		var encodedFragment= encodeURIComponent(dojo.trim(unencodedStr));
		
		if (openOnly) {
			return this._basicPrefix + "search&q=" + encodedFragment + "&o=" + openOnly  + this._getUniqueSuffix(isUnique);
		} else {
			return this._basicPrefix + "search&q=" + encodedFragment + this._getUniqueSuffix(isUnique);
		}
	},
	reuniquifyUrl: function(/* string */ url, removePreserveFilter) {
		if(removePreserveFilter) {
			url = url.replace(/&preserveFilter=true/, "");
		}
		return url.replace(/&tq=([0-9]+)/, "") + "&tq=" + this.getUniqueId();
	}, 
	reuniquifyUrlWithTimestamp: function(/* string */ url) {
		var ts = this.getUniqueId();
		return {
			url: url.replace(/&ts=([0-9]+)/, "") + "&ts=" + ts,
			timestamp: ts
		}; 
	},
	_getUniqueSuffix: function(isUnique) {
		if(!isUnique) return "";
		return "&tq=" + this.getUniqueId();
	},
//	Return a unique id for use in urls
	getUniqueId: function() {
		return Date.parse(new  Date().toString());
//		return this._uniqueId++;
	},
	/*
	 * Returns the default query page size for the Work Item vertical.  Should
	 * not be accessed by code outside the workitem.web bundle.
	 */
	_getDefaultQueryPageSize: function() {
		return 50;
	},
	/*
	 * Returns the maximum query page size for the Work Item vertical.  Should
	 * not be accessed by code outside the workitem.web bundle.
	 */
	_getMaximumQueryPageSize: function() {
		return 500;
	},
	// @deprecated, use getHashRequestParams
	_getHashRequestParams: function(href) {
		return this.getHashRequestParams(href);
	},
	getHashRequestParams: function(href) {
		if(!href) {
			href = document.location.href;
		}
		var reqParms= [];
		var frag= this.getFragment(href);
		if (frag) {
			var nvps= frag.split("&");
			for (var i=0; i<nvps.length; i++) {
				var nvp= nvps[i].split("=");
				if (nvp.length === 2) {
					reqParms.push({name: nvp[0], value: nvp[1]});
				}
			}
		}
		return reqParms;
	},
	
	/* boolean */ isWorkItemId: function(/* String */ id) {
		return !isNaN(parseInt(id));
	},
	
	/* int */ getWorkItemId: function(/* String */ id) {
		var match= WORKITEM_ID_REGEX.exec(id);
		return parseInt(match[1]);
	},
	
	/* String */ serializeQuery: function(/* QueryDTO */ query) {
		var json= dojo.toJson(query);
		return encodeURIComponent(json);
	},
	
	/* QueryDTO */ deserializeQuery: function(/* String */ encoded) {
		var someCode= decodeURIComponent(encoded);
		return this._doEval(someCode);
	},
	
	_doEval: function(someCode) {
		try {
			return eval("(" + someCode + ")");
		}catch(e) {
			console.debug(e);
			return someCode;
		}
	},
		
	_breakupAllLongWords: function(paragraph, minLength, useWBR) {
		if(!minLength) {
			minLength =MIN_BREAKUP_LEN;
		}
		// break the overall paragraph into strings based on spaces
		var words= paragraph.split(" ");
		var needsBreakingUp= false;
		// two pass algorithm so that we don't have to do a lot of string
		// copying for the 99% case
		for (var i=0; i<words.length; i++) {
			if (words[i].length > minLength) {
				needsBreakingUp= true;
				break;
			}
		}
		if (!needsBreakingUp) {
			return paragraph;
		}
		var sep = useWBR == true ? "<wbr/>" : " ";
		if ((dojo.isIE && dojo.isIE<7) /* || dojo.isWebKit */) {
			sep= " ";
		} 
		// if we're still running we have some long words, so now we'll do
		// some string manipulation
		var retStr= "";
		var wordSep = "";
		for (var i=0; i<words.length; i++) {
			// if this is a short word, add it to the return string and
			// loop again, else break up the word
			// If the word is an href, then don't split it
			if ( /* (words[i].indexOf("href=") != -1) || */ words[i].length < minLength ) {
				retStr = retStr + wordSep + words[i];
			} else {
				retStr = retStr + wordSep + this._breakupLongWord(words[i], minLength, sep);
			}
			wordSep = " ";
		}
		return retStr;
	},
	
	_breakupLongWord: function(word, minLength, sep) {
		if(!minLength) {
			minLength =MIN_BREAKUP_LEN;
		}
		var remainingWord= word;
		var brokenupWord= "";
		while (remainingWord.length > minLength) {
			var splitPoint= minLength + 1;
//			for (var i=0; i<DELIMITERS.length; i++) {
//				var delim= DELIMITERS[i];
//				var end= (remainingWord.length > MAX_LEN ? MAX_LEN : remainingWord.length) + 1;
//				var candidateSplitSubstr= remainingWord.substring(minLength + 1, end);
//				var delimIndex= candidateSplitSubstr.indexOf(delim);
//				if (delimIndex != -1) {
//					splitPoint=  minLength + delimIndex + 2;	
//					break;
//				}
//			}
			brokenupWord += remainingWord.substring(0, splitPoint) + sep;
			remainingWord = remainingWord.substring(splitPoint, remainingWord.length);
		}
		brokenupWord += remainingWord;
		return brokenupWord;
	},
	isDebugMode: function() {
			return location.href.indexOf("debug=true") != -1;
	},
	/*
	 * Return a date given a formatted string
	 */ 
	fromISOString: function(timestamp) {
		//Example:  2003-10-06T06:46:00.000-0400
		// dojo.date.stamp.fromISOString wants:
		//          2005-06-30T08:05:00-07:00
		if(timestamp.length == 28) {
			timestamp = timestamp.substring(0,19) + "-" + timestamp.substring(24, 26) + ":" + timestamp.substring(26,28);
		}
		return dojo.date.stamp.fromISOString(timestamp);
		
	},	
	getAllTimestamps: function(modified) {
		var resultArgs = {};
		resultArgs.sortDate = this.fromISOString(modified);
// For relative dates ("2 hours ago"), see history of util		
//		resultArgs.printTimestamp = com.ibm.team.workitem.web.client.util.getModifiedLabel(modified, /* isIso */ true);
		resultArgs.formattedTimestamp = dojo.date.locale.format(resultArgs.sortDate, {formatLength: "medium"});
		resultArgs.modifiedTimestamp = modified;
		return resultArgs;
	},
	getDateLabel: function(date) {
// Date must not be null or "Unassigned"
		return dojo.date.locale.format(date, {formatLength: "medium"});
	},
	/*
 	 * Private Method
 	 * 
 	 * @param String text
 	 * @return boolean
 	 */	
	isStringDefined: function(text) {
		if (text == null) {
			return false;
		} else if (typeof(text) != 'string') {
			return false;
		} else if (text.length == 0) {
			return false;
		}
		return true;
	},
	plainToHTML: function(plain) {
		var span= document.createElement("span");
		var text= document.createTextNode(plain);
		span.appendChild(text);
		return span.innerHTML;
	},
		
	buildMap: function(array, keyName, valueName) {
		
		if (!array)
			return {};
		
		if (!keyName)
			keyName= "key";
		
		if (!valueName)
			valueName= "value";
			
		var map= new Object();
		for (var i in array) {
			var element= array[i];
			map[element[keyName]]= element[valueName];
		}
		return map;
	},
	isInArray: function(item, array, arrayItemSelector) {
		for(var i=0;i<array.length;i++){
			if(arrayItemSelector) {				
				if(item.itemId == array[i][arrayItemSelector]) return true;
			} else {
				if(item.itemId == array[i]) return true;
			}
		}
		return false;
	},
	isCurrentContributorAnAdmin: function () {
		// TODO: do not use an internal constant.
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		var isAdmin = false;
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins") {
					return true;
				}
			}
		}
		return false;
	},
	hasAdministratorRole: function () {
		// TODO: do not use an internal constant.
		var roles = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES");
		if (roles) {
			for (var i=0; i < roles.length; i++) {
				if (roles[i] === "JazzAdmins" || roles[i] === "JazzProjectAdmins") {
					return true;
				}
			}
		}
		return false;
	}
};

})();
