/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
// declare the Dojo module
dojo.provide("com.ibm.team.calm.foundation.web.internal.calmdialogs.FrameContainer");

// declare dependencies on other Dojo modules
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

(function() {

/*
 * Local functions/vars
 */
var _PROTOCOL_WINDOWNAME = { 'OSLC1':'#oslc-windowName-1.0', 'OSLC2':'#oslc-core-windowName-1.0' };
var _PROTOCOL_POSTMSG = { 'OSLC1' : '#oslc-postMessage-1.0', 'OSLC2':'#oslc-core-postMessage-1.0' };
var _MESSAGE_PORT = "oslc-response:";
var _MESSAGE_RESIZE = "oslc-resize:";
var _PARAM_WIDTH = "oslc:hintWidth";
var _PARAM_HEIGHT = "oslc:hintHeight";

function _requiredArgument(/*Map*/ args, /*String*/ id) {
	if (!args[id])
		throw new Error('Missing required argument: "' + id + '"');
	try {
		return args[id];
	} finally {
		delete args[id];
	}
}

var FrameContainer= dojo.declare("com.ibm.team.calm.foundation.web.internal.calmdialogs.FrameContainer", [ dijit._Widget, dijit._Templated ], {
	
	//	_remoteURL: [private] String
	//		The URL (as a string) of the remote webpage initially displayed
	_remoteURL: null,
	
	//	_result_callback: [private] Function
	//		The callback function
	_result_callback: null,
	_resize_callback: null,
	_mode : 'OSLC1',
	_onMessageHandle: null,
	
	constructor: function(args) {
		this._remoteURL = _requiredArgument(args, 'remoteLocation');
		this._result_callback = _requiredArgument(args, 'onData');
		this._resize_callback= args.onResize;
		this._count = FrameContainer._counter++;
		if ( args.mode && args.mode == 'OSLC2' ) {
			this._mode='OSLC2';
		}
	},
	
    templateString: '<div dojoAttachPoint="_iframeDiv" class="com-ibm-team-foundation-crossdomain-web-IFrameContainer" style="width: 100%; height: 100%;"> </div>',
		
	postCreate: function() {
		var doc = dojo.doc;
		var self = this;
		var isWindowName = !(dojo.isSafari >= 4 || dojo.isFF >= 3 || dojo.isChrome || dojo.isIE >= 8); // && dojo.isIE < 8;
		var returnLocation = new dojo._Url(window.location,
				dojo.moduleUrl("dojo", "resources/blank.html")).toString();
		
		var elementString = 'iframe';
		
		if (dojo.isIE && isWindowName)
			elementString = dojo.string.substitute(
				'<iframe name="${0}" onload="com.ibm.team.calm.foundation.web.internal.calmdialogs.FrameContainer._onLoad[${1}]()">',
				[returnLocation, this._count]);
		
		var frame = doc.createElement(elementString);
		
		if (isWindowName) {
			frame.name = returnLocation;
			var finished = false;
			
			FrameContainer._onLoad[this._count] = frame.onload = function() {
				if (!finished) {
					try {
						if (frame.contentWindow.location != returnLocation)
							return;
						var data = frame.contentWindow.name;
						finished = true;
						self._result_callback(data);
					} catch (e) {
						//Ignore location changes on the remote server
					};
				}
			};
		} else {
			this._onMessageHandle= dojo.connect(window, 'onmessage',
				function(e) {
					if (e.source == frame.contentWindow) {
						var data= e.data;
						if (data.indexOf(_MESSAGE_PORT) == 0) {
							self._result_callback(data.substr(_MESSAGE_PORT.length));
						} else if (self._resize_callback && (data.indexOf(_MESSAGE_RESIZE) == 0)) {
							try {
								var newSize= dojo.fromJson(data.substr(_MESSAGE_RESIZE.length));
								if (newSize[_PARAM_WIDTH] || newSize[_PARAM_HEIGHT]) {
									self._resize_callback(newSize[_PARAM_WIDTH], newSize[_PARAM_HEIGHT]);
								}	
							} catch (e) {
								// ignore malformed replies
							}
						}
					}						
				},
				false);
		}
		
		dojo.style(frame, 'width', '100%');
		dojo.style(frame, 'height', '99%');
		frame.frameBorder = '0';
		frame.marginWidth = '0';
		frame.marginHeight = '0';
        frame.scrolling = 'auto';
		
		frame.src = this._remoteURL + (isWindowName ? _PROTOCOL_WINDOWNAME[this._mode] : _PROTOCOL_POSTMSG[this._mode]);
		this._iframeDiv.appendChild(frame);
	},
	
	destroy: function() {
		try {
			if (FrameContainer._onLoad[this._count])
				delete FrameContainer._onLoad[this._count];
			if (this._onMessageHandle) {
				dojo.disconnect(this._onMessageHandle);
				this._onMessageHandle= null;
			}
		} finally {
			this.inherited('destroy', arguments);
		}
	}

});

/*
 * Globals
 */
FrameContainer._counter = 0;
FrameContainer._onLoad = [];

})();