/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.policy.internal.OnStateChangePublishRequiredPropertiesChangedPolicy"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.policy.internal.AbstractPolicy"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

/*
 * This policy listens for changes to the internalStatus and updates the required properties. 
 * Required properties are in edit properties, so an asynchronous server call is used to 
 * fetch the work item properties.
 * 
 * The required properties widgets (RequiredPropertyWidget) are found in the dijit registry.
 * 
 */

(function(){

    dojo.declare("com.ibm.team.workitem.web.policy.internal.OnStateChangePublishRequiredPropertiesChangedPolicy", com.ibm.team.workitem.web.policy.internal.AbstractPolicy, { //$NON-NLS-1$
/*
 * Subscribe to Events is called during construction
 */
	     subscribeToEvents: function() {
        	this.subscribeToWorkItemCreatedEvent();
        },
        onWorkItemCreated: function(policySpec /* { workingCopy: aWorkingCopy } */) {
       		// a state change can be an action
			policySpec.workingCopy.addListener({
				path: ["attributes", "workflowAction", "id"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			policySpec.workingCopy.addListener({
				path: ["attributes", "internalResolution", "id"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
        },
        _handleAttributeChange: function(itemProxyEventParms /* { workingCopy: aWorkItemProxy } */) {
          	/*
        	 * The working copy is a work item proxy.
        	 * Request new editor presentations. This will update the work ing copy workItemSpec
        	 * showOperation is false since since Save cannot be enabled while an operation is in progress 
        	 *   and since the user does not need to see the distracting loading message
        	 */
        	itemProxyEventParms.workingCopy.fetchPresentations({
        		onSuccess: dojo.hitch(this, this._handlePresentationFetched),
        		isInit: false,
        		isForced: false,
        		fetchEditorPresentation: false,
        		isUsingEditablePropertiesCache: true,
        		fetchInlineEditorPresentation: false,
        		fetchWorkItem: false,
        		showOperation: false 
        	})
        },
        _handlePresentationFetched: function(fetchParms /* editProps: editProps */) {
        	dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_REQUIRED_PROPERTIES_CHANGED, [{ workingCopy: fetchParms.workingCopy}]);
        },
        _sentinal: true
    });
    
})();
