/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.policy.internal.RefreshWorkItemCachePolicy"); //$NON-NLS-1$ 
dojo.require("com.ibm.team.workitem.web.ui2.internal.dialog.AlertDialogLauncher"); 

dojo.require("com.ibm.team.workitem.web.policy.internal.AbstractPolicy"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");


/*
 * This policy listens for the event "workitem/cache/refresh" and causes the cached items to be 
 * refreshed.
 */

(function(){

dojo.declare("com.ibm.team.workitem.web.policy.internal.RefreshWorkItemCachePolicy", [com.ibm.team.workitem.web.policy.internal.AbstractPolicy,  com.ibm.team.workitem.web.ui2.internal.dialog.AlertDialogLauncher], { //$NON-NLS-1$
/*
 * Subscribe to Events is called during construction
 */
	     subscribeToEvents: function() {
	     	this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
        	dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_REFRESH_CACHE, dojo.hitch(this, this.refreshWorkItemCache));
        },
        /*
         * 1) Run a query on all work item id's in the cache to get the last modified date 
         * 2) For each item where the server-side last modified date is later than the last loaded date, merge the changes
         * 3) For work items that are changed on the client, prompt to either accept or override the changes
         * 
         * Parms can be { itemArray: [ anArrayOfWorkItemIds ] }
         * If the itemArray is provided, then only those items will be updated, if they need to be.
         * The itemId's are the human readable numbers, e.g., 93898 (not the uuid itemId).
         */
        refreshWorkItemCache: function(parms) {
        	parms = parms || [];
        	var allWorkItems = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({ filterAttributes: [ "isWorkItem" ]});
        	if(allWorkItems.length == 0) return; // nothing to do;
        	var idsToQuery = [];
        	for(var i=0;i<allWorkItems.length;i++) {
        		if(allWorkItems[i].isNew()) continue;
        		if(parms.itemIdArray) {
        			
        		} else {
        			idsToQuery.push(allWorkItems[i].getId());
        		}
        	}
        		var serviceObject = {
			self: this,
			success: function(resultSet) {
				parms.resultSet = resultSet;
				this.self._refreshWorkItemCacheSuccess(parms);
			},
			failure: function(error) {
				console.error(error.message);
			}
		}
        	var query = com.ibm.team.workitem.web.client.internal.QueryClient.createWorkItemIdQuery("Refresh Candidates", idsToQuery);
        	var responseHandler = new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObject, "success", "failure");
			var queryArgs = {
				columnIdentifiers: [com.ibm.team.workitem.web.client.internal.WorkItemAttributes.MODIFIED ],
				maxResults: idsToQuery.length
			};
			com.ibm.team.workitem.web.client.internal.QueryClient.getResultSet(query, queryArgs, responseHandler);	
        },
        /*
         * For any work item in the cache with a modified date later than the last read date, 
         * load the work item again and merge the new work item with the old one
         */
        _refreshWorkItemCacheSuccess: function(parms) {
        	var resultSet = parms.resultSet;
        	if(!resultSet || !resultSet.rows || resultSet.rows.length == 0) return; // nothing to do
   	    	var changedItemsToReload = [];
        	var unchangedItemsToReload = [];
        	for(var i=0;i<resultSet.rows.length;i++) {
        		var cachedWorkItem = com.ibm.team.workitem.web.cache.internal.Cache.getItem(resultSet.rows[i].id);
        		if(!cachedWorkItem || !cachedWorkItem.object || !cachedWorkItem.object.attributes || !cachedWorkItem.object.attributes.modified || !cachedWorkItem.object.attributes.modified.id) continue; // removed from cache since query started
        		var cachedTimestamp = com.ibm.team.workitem.web.client.util.fromISOString(cachedWorkItem.object.attributes.modified.id);
        		var serverTimestamp = new Date();
        		serverTimestamp.setTime(resultSet.rows[i].labels[0]);
        		if(cachedTimestamp < serverTimestamp) {
        			if(cachedWorkItem.isChanged()) {
        				changedItemsToReload.push(cachedWorkItem);
        			} else {
        				unchangedItemsToReload.push(cachedWorkItem);
        			}
        		}
        	}
        	this.refreshItems(unchangedItemsToReload);
        	/*
        	 * Warn about changed items
        	 */
        	if(changedItemsToReload.length>0) {
        		for(var i=0;i<changedItemsToReload.length;i++) {
        			changedItemsToReload[i].mergeRepositoryChanges({isKeepingChangedValues: true}); // merge what can be merged
        		}
        		/*
        		 * Tell the editors to update the headers to warn that changes occured
        		 */
         		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_REPOSITORY_ITEMS_CHANGED, [{changedItems: changedItemsToReload}]);
        	} 
        },
       	/*
       	 * Reload each work item
       	 * TODO: Optimize to fetch a bunch of work items in one call
       	 */
        refreshItems: function(itemList) {
        	for(var i=0;i<itemList.length;i++) {
        		this.syncWithRepositoryCopy(itemList[i]);
        	}
        },
        syncWithRepositoryCopy: function(workItem) {
		var args = { 
			id: workItem.getId(),
			onSuccess: dojo.hitch(this, this._refreshComplete), 
			isRefresh: true,
			fetchEditorPresentation: false,
			fetchInlineEditorPresentation: false
			}
		workItem.fetchWorkItem(args);
	},
	_refreshComplete: function(parms) {
		// nothing to do
	},
        _sentinal: true
    });
    	
})();
