/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.WorkItemsViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.SecondIconPreferenceProvider");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2");
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.require("com.ibm.team.workitem.web.client.internal.Constants");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.AttributeUtil");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.QueryResultProblemInfo");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("jazz.app.ResourceLink");
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.context");
dojo.require("jazz.util.format");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.FilterBox");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.util.i18n");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {
	
var I18nUtil = jazz.util.i18n;

var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var WorkItemQueryPreferenceProvider = com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider;
var SecondIconPreferenceProvider = com.ibm.team.workitem.viewlets.web.ui.internal.utils.SecondIconPreferenceProvider;

var Constants = com.ibm.team.workitem.web.client.internal.Constants;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient2 = com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var FormatUtil = jazz.util.format;

var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var UUIDUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility;
var AttributeUtil = com.ibm.team.workitem.viewlets.web.ui.internal.utils.AttributeUtil;

var IMG_PATH = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web","graphics/work-items/images/");
var IMG_URL_CLOSED_OVERLAY = IMG_PATH + "wi_complete_ovr.gif";
var IMG_URL_IN_PROGRESS_OVERLAY = IMG_PATH + "wi_in_progress_ovr.gif";

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

/*
 * (static) Observables to listen for viewlet log-in, to allow viewlets with C/ALM
 * queries to the same server to load as soon as one logs in.
 */
var _loginTrackersByURL;

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.WorkItemsViewlet",
	PagedTableViewlet, {
	
	contentClass: "com-ibm-team-workitem-viewlets-web-ui-internal-WorkItemsViewlet",
	
	init: function() {
		this.inherited(arguments);

		if (jazz.app.context.get().itemId) {
			this._local = true;
		} else {
			this._local = false;
		}
		if (this.getSite().getFilterBox) {
			this.getSite().getFilterBox().onChange = dojo.hitch(this, this._filterBoxChange);
		}

		this._skipOAuth = true;
		// if the viewlet has a default query and it has not been changed, initialize the title and preference provider
		var defaultQuery = this.getPreference("defaultQuery");
		if (defaultQuery != -1 && this.getPreference("query") == -1) {
			var serviceObject = {
				self: this,
				_successCallback: function(/*array*/ queryDTO) {
					if (!this.self.isDestroyed() && queryDTO) {
						this.self._defaultQueryItemId = queryDTO.itemId;
						this.self.getSite().setTitle(queryDTO.name);
						this.self.getPreferenceProvider("query").initDefault(queryDTO);
					}
				},
				_errorCallback: function(error) {
					this.self.getSite().error(error);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
			var args = {
				includeCount: false,
				id: defaultQuery
			};
			// handle templated viewlets moved to the JTS in online migartion. See 131901.
			var queryCat = this.getPreference("queryCategory");
			if (queryCat) {
				if (queryCat.indexOf("|") !== -1) {
					// this should not happen for now since we only record project area during online migration
					// but we should understand all valid values for this property just in case
					queryCat = queryCat.substring(0,queryCat.indexOf("|"));
				}
				args.projectAreaItemId = queryCat;
			}
			QueryClient2.getQuery2(responseHandler, args);
		}
		
		this._serverRoot = null;
		this._imageServerRoot = null;
		this._widgets = [];
	},

	_filterBoxChange: function(newVal) {
		this._filterBoxChanged = true;
		this.reset(true);
	},

	getResultCount: function() {
		if (this.getPreference("query") == -1 && this.getPreference("defaultQuery") == -1)
			return 0;
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (this._lastFilterValue) {
			// No content because of filtering
			return document.createTextNode(messages.noResultsFilter);
		}
		if (this._oauthURL)
			return this._getLoginMessage();
		if (this._deletedQuery)
			return this._getDeletedQueryMessage();
		if (this._unknownError)
			return this._getUnknownErrorMessage();
		if (this._customError)
			return this._getCustomErrorMessage(this._customError);
		if (this.getPreference("query") != -1 || this.getPreference("defaultQuery") != -1)
			return document.createTextNode(messages.noResults);
		return this._getNoQueryMessage();
	},
	
	settingsAreaToggled: function(opened) {
		this._isSettingsAreaOpen = opened;
		this.updateEditMessages();
	},
	
	setDashboardEditing: function(editing) {
		this.updateEditMessages();
	},
	
	updateEditMessages: function() {
		if (this._noQueryEditMessage)
			this._noQueryEditMessage.className = this.getSite().isDashboardEditing() && !this._isSettingsAreaOpen ? "" : "hidden";
		if (this._deletedQueryEditMessage)
			this._deletedQueryEditMessage.className = this.getSite().isDashboardEditing() && !this._isSettingsAreaOpen ? "" : "hidden";
	},
	
	_getUnknownErrorMessage: function() {
		if (!this._unknownErrorMessage) {
			this._unknownErrorMessage = document.createElement("span");
			this._unknownErrorMessage.innerHTML = messages.unknownError + "&nbsp; ";
			var refreshLink = this._createRefreshLink();
			this._unknownErrorMessage.appendChild(refreshLink);
		}
		return this._unknownErrorMessage;
	},
		
	_getCustomErrorMessage: function(error) {
		if (!this._customErrorMessage || this._lastCustomError !== error.message) {
			var span = document.createElement("span");
			span.innerHTML = jazz.util.html.escape(error.message) + "&nbsp; ";
			var refreshLink = this._createRefreshLink();
			span.appendChild(refreshLink);
			this._customErrorMessage = span;
			this._lastCustomError = error.message;
		}
		return this._customErrorMessage;
	},
	
	_createRefreshLink: function() {
		var refreshLink = document.createElement("a");
		refreshLink.href = "#";
		refreshLink.innerHTML = messages.refresh;
		refreshLink.title = messages.refresh;
		this.connect(refreshLink,"onclick",function(e) {
			e.preventDefault();
			this.getSite().refresh();
		});
		return refreshLink;
	},
	
	_getDeletedQueryMessage: function() {
		if (!this._deletedQueryMessage) {
			this._deletedQueryMessage = document.createElement("span");
			this._deletedQueryMessage.appendChild(document.createTextNode(messages.deletedQuery));
			this._deletedQueryEditMessage = this._createEditSettingsMessage(true);
			this.updateEditMessages();
			this._deletedQueryMessage.appendChild(this._deletedQueryEditMessage);
		}
		
		return this._deletedQueryMessage;
	},
	
	_displayCALMQueryProblemsInfoNode: function(problemStatusDTOs) {
		if (this.calmQueryProblemsInfoNode) {
			this._contentDiv.removeChild(this.calmQueryProblemsInfoNode);
			this.calmQueryProblemsInfoNode= null;
		}
	
		var widget= com.ibm.team.workitem.web.ui.internal.view.query.QueryResultProblemInfo.getProblemInfoWidget(problemStatusDTOs);
		if (widget) {
			this.calmQueryProblemsInfoNode= widget.domNode;
			this._contentDiv.appendChild(this.calmQueryProblemsInfoNode);
		}
	},	
	
	
	_getLoginMessage: function() {
		if (!this._loginMessage) {
			// show login required icon
			this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/login.gif"));
			this.getSite().setSecondaryTitle(messages.viewletNotLoggedIn);
			
			// listen for log-ins for this origin
			this._addLoginTracker();
			
			// show log-in section
			this._loginMessage = document.createElement("div");
			this._loginMessage.className = "login";
			this._loginMessage.style.margin = "1px"; // override default margins
			this._loginMessage.innerHTML = "<div class=\"message\">" + dojo.string.substitute(messages.viewletLogin, [this._oauthURL.host]) + "</div>";
			
			// add log-in link
			var linkDiv = document.createElement("div");
			linkDiv.className = "link";
			var a = document.createElement("a");
			a.href = "#";
			a.appendChild(document.createTextNode(messages.viewletLogin2));
			this.connect(a, "onclick", dojo.hitch(this, this._onLoginClick));
			linkDiv.appendChild(a);		
			this._loginMessage.appendChild(linkDiv);
		}
		return this._loginMessage;
	},
	
	_addLoginTracker: function() {
		if (!_loginTrackersByURL) {
			_loginTrackersByURL = new Object();
		}
		var url = this._oauthURL.scheme + "://" + this._oauthURL.host + (this._oauthURL.port ? ":" + this._oauthURL.port : "");
		this._loginTracker = _loginTrackersByURL[url];
		if (!this._loginTracker) {
			this._loginTracker = _loginTrackersByURL[url] = new Observable();
		}
		this._loginCallback = dojo.hitch(this, function() {
			// Workaround for 85382: Observable is not reentrant
			setTimeout(dojo.hitch(this, function() {
				if (!this.isDestroyed()) {
					// reset icon and title back to original
					this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/icons/work-items.gif"));
					this.getSite().setSecondaryTitle("");
					this._removeLoginTracker();
					this.refresh();
				}
			}), 0);
		});
		this._loginTracker.addListener(this._loginCallback, "login");
	},
	
	_removeLoginTracker: function() {
		if (this._loginCallback && this._loginTracker) {
			this._loginTracker.removeListener(this._loginCallback, "login");
			delete this._loginCallback;
			delete this._loginTracker;
		}
	},
	
	_onLoginClick: function(/* Event */ event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// reset icon and title back to original
		this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/icons/work-items.gif"));
		this.getSite().setSecondaryTitle("");
		// fetch with oauth this time
		this._skipOAuth = false;
		this.refresh();
	},
	
	_getNoQueryMessage: function() {
		if (!this._noQueryMessage) {
			this._noQueryMessage = document.createElement("span");
			this._noQueryMessage.appendChild(document.createTextNode(messages.noQuery));
			this._noQueryEditMessage = this._createEditSettingsMessage(false);
			this.updateEditMessages();
			this._noQueryMessage.appendChild(this._noQueryEditMessage);
		}
		return this._noQueryMessage;
	},
	
	_createEditSettingsMessage: function(/* boolean */ isDeleted) {
		var result = document.createElement("span");
		var links = I18nUtil.addMessage(result, isDeleted ? messages.selectNewQuery : messages.selectQuery);
		this.connect(links[0], "onclick", this._doPreferences);
		return result;
	},
	
	_doPreferences: function(event) {
		dojo.stopEvent(event);
		
		this.getSite().toggleSettings();
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	refresh: function() {
		this._destroyWidgets();
		delete this._totalResults;
		delete this._token;
		delete this._deletedQuery;
		delete this._oauthURL;
		delete this._unknownError;
		delete this._customError;
		this.inherited(arguments);
		// update the title on refresh in case the query was renamed
		this.getPreferenceProvider("query").getQueryName(this.getPreference("query"), dojo.hitch(this.getSite(),this.getSite().setTitle));
	},
	
	update: function() {
		if (dojo.isIE) {
			if (this._iframePollTimeouts) {
				this._clearTimeouts();
			}
			this._iframePollTimeouts = {};
		}
		this.inherited(arguments);
	},
	
	_totalResultsChanged: function() {
		if (this._resizeTimeout) {
			clearTimeout(this._resizeTimeout);
			delete this._resizeTimeout;
		}
		this.updateTotalPageFooter();
		// only update the header if there was no filter
		if (!this._lastFilterValue)
			this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
		if (this._filterBoxChanged) {
			this._resizeTimeout = setTimeout(
				dojo.hitch(this.pagedTable, this.pagedTable.updateHeight), 500);
			this._filterBoxChanged = false;
		}
	},
	
	fetch: function(srh, start, count, entries, details) {
		var responseHandler;
		if (this.getSite().getFilterBox)
			var filterValue = this.getSite().getFilterBox().value
		var serviceObject = {
			self: this,
			filterValue: filterValue,
			_onResponse: function() {
				// notify any login-waiting viewlets to load
				var tracker = this.self._loginTracker;
				if (tracker) {
					this.self._removeLoginTracker();
					tracker.propertyChanged("login");
				}
			},
			success: function(resultSet){
				if (!this.self.isDestroyed()) {
					this._onResponse();
					this.self._lastFilterValue = this.filterValue;
					if (resultSet && resultSet.totalCount > 0) {
						this.self._displayCALMQueryProblemsInfoNode(resultSet.problems);
					
						this.self._resultHeaderIndex = {};
						this.self._resultHeaders = resultSet.headers;
						for (var i = 0; i < resultSet.headers.length; i++) {
							this.self._resultHeaderIndex[resultSet.headers[i].attributeId] = i;
							if (resultSet.headers[i].attributeId.substring(0,5) === "link:") {
								this.self._resultHeaderIndex["link"] = i;
							}
						}
						// only update the total result count if we didn't have a token or if the token is different (e.g. a new DB query was run)
						// this is to work around a bug in the service, see work items 66633 and 70982
						if ((!this.self.token || this.self._token != resultSet.token) && this.self._totalResults != resultSet.totalCount) {
							// work item queries only handle 1000 results, but they report more
							// limit the result count
							this.self._totalResults = Math.min(resultSet.totalCount,1000);
							this.self._totalResultsChanged();
						}
						this.self._token = resultSet.token;
						if (resultSet.rows) {
							var dtos = resultSet.rows;
							var result = new Object();
							if (entries) 
								result.entries = dtos;
							if (details)
								result.details = dtos;
							srh.doCallback(result);
						}
					}
					else {
						this.self._totalResults = 0;
						this.self._totalResultsChanged();
						srh.doCallback();
					}
				}
			},
			failure: function(error){
				// C/ALM cross-server queries require oauth login
				var oauthHeader;
				if (responseHandler.xhr) {
					oauthHeader = responseHandler.xhr.getResponseHeader("X-jazz-web-oauth-url");
				}
				if (oauthHeader) {
					this.self._oauthURL = new dojo._Url(oauthHeader);
					this.self._totalResults = 0;
					this.self.updateTotalPageFooter();
					this.self.getSite().setSecondaryTitle("");
					srh.doCallback();
				}
				else {
					this._onResponse();
					// 404 means query from settings area does not exist
					// 400 means some other issue with the query rest service
					if (error.status == 404 || error.status == 400 || error.status == 500) {
						if (error.status == 500 || (error.status == 400 && !error.message))
							this.self._unknownError = true;
						else if (error.status == 400)
							this.self._customError = error;
						else
							this.self._deletedQuery = true;
						this.self._totalResults = 0;
						this.self.updateTotalPageFooter();
						this.self.getSite().setSecondaryTitle("");
						srh.doCallback();
					}
					else {
						srh.doErrorCallback(error);
					}
				}
			}
		};
		responseHandler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		var columns = [WorkItemAttributes.TYPE, WorkItemAttributes.SUMMARY,
				WorkItemAttributes.CREATOR, WorkItemAttributes.OWNER,
				WorkItemAttributes.CREATION_DATE, WorkItemAttributes.ESTIMATE,
				WorkItemAttributes.CATEGORY, WorkItemAttributes.TARGET, WorkItemAttributes.PROJECT_AREA, WorkItemAttributes.TAGS];
		var additionalIcons = this._getSelectedIconsArray(this.getPreference("secondIcon"));
		for (var i = 0; i < additionalIcons.length; i++) {
			columns.push(WorkItemAttributes.getInternalId(AttributeUtil.getAttribute(additionalIcons[i])));
		}
		var args = {
			startIndex: start,
			maxResults: count,
			absoluteURIs: true,
			projectAreaItemId: jazz.app.context.get().itemId ? jazz.app.context.get().itemId : true,
			columnIdentifiers: columns
		};
		if (filterValue) {
			args.filterValue = filterValue;
			args.filterAttribute = 'summary';
		}
		var queryItemId = this.getPreference("query");
		if (queryItemId == -1) {
			var queryStringId = this.getPreference("defaultQuery");
			if (queryStringId == -1) {
				srh.doCallback();
				return;
			}
			args.id = queryStringId;
			if (args.projectAreaItemId === true) {
				var queryCat = this.getPreference("queryCategory");
				if (queryCat) {
					if (queryCat.indexOf("|") !== -1) {
						// this should not happen, see lines 111-112
						queryCat = queryCat.substring(0,queryCat.indexOf("|"));
					}
					args.projectAreaItemId = queryCat;
				}
			}
		} else {
			args.itemId = queryItemId;
		}
		if (this._token)
			args.token = this._token;
		args.skipOAuth = this._skipOAuth;
		QueryClient2.getResultSet(null, args, responseHandler);
	},
	
	/* Node[] */ renderEntry: function(/* object */ resultDTO, /* object */ resultDetailsDTO, /* boolean */ zoomed) {
		var columns = new Array();
		
		var expandCell, expandLink, linkInfo;
		if (typeof this._resultHeaderIndex["link"] === "number" && !zoomed && this.getPreference("showLinks")) {
			expandCell = document.createElement("td");
			expandCell.className = "expandCell"
			linkInfo = resultDTO.info[this._resultHeaderIndex["link"]];
			if (linkInfo && linkInfo.item && linkInfo.item.length) {
				expandLink = document.createElement("a");
				expandLink.className = this.getPreference("expandLinks") ? "open" : "closed";
				expandLink.href = "#";
				expandCell.appendChild(expandLink);
			}
			columns.push(expandCell);
		}
		
		var iconCell = document.createElement("td");
		var iconContainer = document.createElement("div");
		iconContainer.className = "iconContainer";
		
		iconCell.className = "iconCell";
		var icon = document.createElement("img");
		icon.width = 16;
		icon.height = 16;
		icon.style.marginRight = "0.2em";
		var iconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.TYPE]];
		// need to proxy the url for remote viewlets
		icon.src = this._local ? iconSrc : jazz.app.proxy.encodeProxiedURI(iconSrc);
		iconContainer.appendChild(icon);
		
		// add overlay if available/needed
		if (resultDTO.stateGroup) {
			var src;
			switch (resultDTO.stateGroup) {
				case Constants.CLOSED_STATES:	
					src = IMG_URL_CLOSED_OVERLAY;
					dojo.addClass(icon, "closed");
					break;
				case Constants.IN_PROGRESS_STATES:	
					src = IMG_URL_IN_PROGRESS_OVERLAY;
					break;
			}
			if (src) {
				var overlayIcon = document.createElement("img");
				overlayIcon.className = "overlayIcon";
				overlayIcon.src = src;
				iconContainer.appendChild(overlayIcon);
			}
		}
		iconCell.appendChild(iconContainer);
		columns.push(iconCell);
		
		var additionalIcons = this._getSelectedIconsArray(this.getPreference("secondIcon"));
		for (var i = 0; i < additionalIcons.length; i++) {
			var additionalIcon = additionalIcons[i];
			if (additionalIcon != "none") {
				additionalIcon = WorkItemAttributes.getInternalId(additionalIcon);
				var additionalIconCell = document.createElement("td");
				additionalIconCell.className = "iconCell";
				var additionalIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[additionalIcon]];
				if (additionalIconSrc) {
					var additionalIconImg = document.createElement("img");
					additionalIconImg.width = 16;
					additionalIconImg.height = 16;
					additionalIconImg.style.marginRight = "0.2em";
					additionalIconImg.src = additionalIconSrc;
					additionalIconImg.title = dojo.string.substitute(messages.iconTooltip,[this._resultHeaders[this._resultHeaderIndex[additionalIcon]].label, resultDTO.labels[this._resultHeaderIndex[additionalIcon]]]);
					additionalIconCell.appendChild(additionalIconImg);
				} else {
					var noIconImg = document.createElement("img");
					noIconImg.src = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web","graphics/work-items/icons/no-icon_obj.gif");
					additionalIconCell.appendChild(noIconImg);
					additionalIconCell.title = dojo.string.substitute(
						messages.iconTooltip,
						[this._resultHeaders[this._resultHeaderIndex[additionalIcon]].label,
						 resultDTO.labels[this._resultHeaderIndex[additionalIcon]] || messages.valueUndefined]
					);

				}
				columns.push(additionalIconCell);
			}
		}
		
		var titleCell = document.createElement("td");
		titleCell.className = "titleCell";
		var titleWrapper = document.createElement("div");
		titleWrapper.className = "titleWrapper";
		var titleLink = document.createElement("a");
		var presentationUri, resourceUri = this._getQualifiedUriRoot() + "/resource/itemName/com.ibm.team.workitem.WorkItem/" + resultDTO.id;
		if (this._local) {
			// use a presentation uri locally to avoid context switch for cross project queries
			presentationUri = WorkItemUtil.getViewWorkItemUri(resultDTO.id);
			titleLink.href = presentationUri;
		} else {
			titleLink.href = resourceUri;
		}
		titleLink.innerHTML = resultDTO.id + ": " + FormatUtil.insertWordBreaks(resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SUMMARY]], 20, true);
		if (!zoomed) {
			var args = {retainLinkText: true, uri: resourceUri};
			if (presentationUri)
				args.presentationUri = presentationUri;
			this._widgets.push(new jazz.app.ResourceLink(args,titleLink));
		}
		titleWrapper.appendChild(titleLink);
		titleCell.appendChild(titleWrapper);
		
		if (expandLink) {
			var linkWrapper = document.createElement("div");
			linkWrapper.style.display = this.getPreference("expandLinks") ? "" : "none";
			linkWrapper.className = "linkWrapper";
			for (var i = 0; i < linkInfo.item.length; i++) {
				var linkDiv = document.createElement("div");
				linkDiv.className = "linkRow";
				var linkIcon = document.createElement("div");
				linkIcon.className = "iconContainer";
				var rl = new jazz.app.ResourceLink({
					uri: linkInfo.item[i].url,
					label: linkInfo.item[i].comment,
					iconNode: linkIcon
				});
				linkDiv.appendChild(linkIcon);
				linkDiv.appendChild(rl.domNode);
				linkWrapper.appendChild(linkDiv);
				this._widgets.push(rl);
			}
			this.connect(expandLink, "onclick", dojo.hitch(this,function(e) {
				dojo.stopEvent(e);
				if (expandLink.className === "closed") {
					expandLink.className = "open";
					linkWrapper.style.display = "";
				} else {
					expandLink.className = "closed";
					linkWrapper.style.display = "none";
				}
				// TODO: this is an internal PagedTableViewlet function, we should API it or find another solution
				this.pagedTable._clearHeight();
			}));
			titleCell.appendChild(linkWrapper);
		}
		
		columns.push(titleCell);
		
		return columns;
	},
	
	/* Node[] */ renderDetails: function(/* object */ resultDTO) {
		var columns = new Array();
		
		var detailsCell = document.createElement("td");
		detailsCell.className = "detailsCell";
		detailsCell.colSpan = 2 + this._getSelectedIconsArray(this.getPreference("secondIcon")).length;
		var div = document.createElement("div");
		div.className = "iframeWrapper";
		var iframe = document.createElement("iframe");
		var resizeContent = function() {
			if (iframe.contentWindow && iframe.contentWindow.innerHeight && iframe.contentWindow.scrollMaxY) {
				var win = iframe.contentWindow;
				var h = win.innerHeight + win.scrollMaxY;
			}
			else {
				var h = iframe.contentWindow.document.body.scrollHeight;
			}
			iframe.height = h + "px";
		};
		// onload will not fire in IE, workaround with a polling loop
		if (dojo.isIE) {
			var timeouts = this._iframePollTimeouts;
			var poll = function() {
				if (iframe.contentWindow && iframe.contentWindow.document.body && iframe.contentWindow.document.body.scrollHeight) {
					resizeContent();
				} else {
					timeouts[resultDTO.id] = setTimeout(poll,100);
				}
			};
			// yield to ensure the iframe is on dom
			timeouts[resultDTO.id] = setTimeout(poll,0);
		} else {
			dojo.attr(iframe,"onload",resizeContent);
		}
		iframe.src = this._getSmallPreviewUri(resultDTO);
		iframe.style.width = "100%";
		iframe.height = "0";
		iframe.frameBorder = "0";
		iframe.scrolling = "no";
		div.appendChild(iframe);
		detailsCell.appendChild(div);
		columns.push(detailsCell);
		
		return columns;
	},
	
	_getSmallPreviewUri: function(resultDTO) {
		var uriRoot = this._getQualifiedUriRoot();
		var uri = uriRoot + "/resource/itemOid/com.ibm.team.workitem.WorkItem/" + resultDTO.itemId + "?_mediaType=text%2Fhtml&_selector=smallHover&_context=web";
		return this._needsProxy() ? jazz.app.proxy.encodeProxiedURI(uri) : uri;
	},
	
	_getQualifiedUriRoot: function() {
		if (!this.uriRoot) {
			this.uriRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
			if (!this.uriRoot) {
				var winUrl = new dojo._Url(window.location.href);
				this.uriRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" ) + net.jazz.ajax._contextRoot;
			}
		}
		return this.uriRoot;
	},
	
	_needsProxy: function() {
		if (typeof this.needsProxy === "undefined") {
			var viewletRoot = this._getQualifiedUriRoot();
			var winUrl = new dojo._Url(window.location.href);
			var localRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" );
			this.needsProxy = viewletRoot.substring(0,localRoot.length) !== localRoot;
		}
		return this.needsProxy;
	},
	
	_getQueryHref: function(itemId) {
		// if there's a project area defined, stay in the same web ui
		// otherwise use the resource url
		if (this._local)
			return WorkItemUtil.getRunSavedQueryUri(itemId);
		return this._getQualifiedUriRoot() + WorkItemUtil.getRelativeQueryLocationUri(itemId);
	},
	
	settingChanging: function(settingId) {
		if (settingId === "queryCategory")
			return ["secondIcon"];
		return [];
	},
	
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		if (oldSettings.preferences.query != newSettings.preferences.query) {
			if (this.getSite().getFilterBox)
				this.getSite().getFilterBox().clear();
			delete this._lastFilterValue;
			this.reset(true);
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
		}
		else if (oldSettings.preferences.secondIcon != newSettings.preferences.secondIcon) {
			this.refresh();
		}
		else if (oldSettings.preferences.showLinks != newSettings.preferences.showLinks ||
				oldSettings.preferences.expandLinks != newSettings.preferences.expandLinks) {
			this.update();
		}
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the WorkItemQueryPreferenceProvider
		// always persist the query title (when it's available) on setting changes in case it has changed
		if (UUIDUtility.isValidUUID(newSettings.preferences.query) && this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query))
			this.getSite().setTitle(this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query), true);
	},
	
	getTitleHyperlinkURL: function(){
		var queryId = this.getPreference("query");
		if (queryId != "-1")
			return this._getQueryHref(queryId);
		if (this._defaultQueryItemId)
			return this._getQueryHref(this._defaultQueryItemId);
		return null;
	},
	
	getPreferenceProvider: function(/* string */ preferenceId) {
		if (!this._preferenceProviders)
			this._preferenceProviders = {};
		if (!this._preferenceProviders[preferenceId]) {
			this._preferenceProviders[preferenceId] = preferenceId === "secondIcon" ? new SecondIconPreferenceProvider({viewlet: this}) : 
					new WorkItemQueryPreferenceProvider({ prefId: preferenceId, queryCategoryPrefId: "queryCategory" });
		}
		return this._preferenceProviders[preferenceId];
	},
	
	_clearTimeouts: function() {
		if (dojo.isIE && this._iframePollTimeouts) {
			for (var i in this._iframePollTimeouts) {
				clearTimeout(this._iframePollTimeouts[i]);
			}
		}
	},
	
	_getSelectedIconsArray: function(selected) {
		// translate into an array for work item 148794
		if (!selected || selected === "none") selected = "[]";
		else if (selected.indexOf("[") !== 0) {
			// work around problems with migration for work item 67174
			selected = AttributeUtil.getAttribute(selected);
			selected = '["' + selected + '"]';
		}
		return dojo.fromJson(selected);
	},
	
	destroy: function() {
		this._destroyWidgets();
		this._clearTimeouts();
		this.inherited(arguments);
	},
	
	_destroyWidgets: function() {
		while (this._widgets.length)
			this._widgets.pop().destroy();
	}
	
});

})();
