﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
import flash.external.ExternalInterface; 
 
class DebugArea extends LayoutObject {

	private static var _TEXT_FIELD_FORMAT:TextFormat = DebugArea._initTextFieldFormat();
	
	private static var _TEXT_FIELD_HEIGHT:Number = 15;
	
	private var _lineCount:Number;
	
	private static var _OUTLINE_COLOUR:Number = 0xEEEEEE;
	
	
	public function DebugArea() {
		super();
		this._lineCount = 0;
	}		
	
	private static function _initTextFieldFormat():TextFormat {
		var textFieldFormat = new TextFormat();
		textFieldFormat.color = 0xCCCCCC;
		textFieldFormat.font = ChartArea.FONT;
		textFieldFormat.size = 9;
		
		return textFieldFormat;
	}

	public function getPreferredWidth():Number {
		return Stage.width;
	}
	
	public function getPreferredHeight():Number {
		return DebugArea._TEXT_FIELD_HEIGHT;
	}	
	
	public function draw(parentMc:MovieClip):Void {
		if (this.mc) {
			return;
		}		
		
		this.mc = parentMc.createEmptyMovieClip("debugArea", parentMc.getNextHighestDepth());

		
		this.mc.createTextField("debugTextField", this.mc.getNextHighestDepth(), this.startX, this.startY, this.width, this.height);
		this.mc.debugTextField.text = "";
		this.mc.debugTextField.multiline = true;
		this.mc.debugTextField.wordWrap = true;
		this.mc.debugTextField.border = true;
		this.mc.debugTextField.borderColor = DebugArea._OUTLINE_COLOUR;
		this.mc.debugTextField.setTextFormat(DebugArea._TEXT_FIELD_FORMAT);		
		
		this._drawOutline();
	}	

	private function _drawOutline():Void {
		this.mc.lineStyle(1, DebugArea._OUTLINE_COLOUR);
		this.mc.moveTo(this.startX, this.startY);
		this.mc.lineTo(this.startX, this.startY + Stage.height);
		this.mc.lineTo(this.startX + Stage.width, this.startY + Stage.height);
		this.mc.lineTo(this.startX + Stage.width, this.startY);
		this.mc.lineTo(this.startX, this.startY);
	}		
	
	public function clear():Void {
		if (!this.mc) {
			return;
		}
		
		this.mc.debugTextField.text = "";
	}
	
	public function print(text:String):Void {
		if (!this.mc) {
			return;
		} else if (this._isStringDefined(text) == false) {
			return;
		}
		
		if (this._isStringDefined(this.mc.debugTextField.text)) {
			this.mc.debugTextField.text = this.mc.debugTextField.text + text;
		} else {
			this.mc.debugTextField.text = text;
		}
		
		this.mc.debugTextField.setTextFormat(DebugArea._TEXT_FIELD_FORMAT);
		
	}	

	public function println(text:String):Void {
		if (this._isStringDefined(text) == false) {
			return;
		}
		this._lineCount++;
		
		var ln:String = "";
		if (this._lineCount > 1) {
			ln = newline;
		}
		
		this.print(ln + "[" + String(this._lineCount) + "]: " + text);
		// Scroll to the last line
		this.mc.debugTextField.scroll = this._lineCount;
	}

	public function printSeparator():Void {
		this.print("========================================\n");
	}
	
	public function debug(text:String):Void {
		var result:String = Timer.getTimeStamp() +
							" >> " +
							text;
		this.println(result);
	}	
	
	private function _isStringDefined(text:String):Boolean {
		if (!text) {
			return false;
		}
		return true;
	}
	
	public static function alert(text:String):Void {
		var result:Object = ExternalInterface.call("alert", text);
	}
	
	public function destroy():Void {
		// Line count
		delete this._lineCount;
		
		super.destroy();
	}	
	
}
 