/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.page.SearchProvider");

dojo.require("jazz.ui._SearchProvider")

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics"); 
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

/*
 * Aliases
 */
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var EventTopics = com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var html = jazz.util.html;

var SCOPE_ALL = "all";
var SCOPE_UNRESOLVED = "unresolved";
var MAX_SIZE = 100;
var MAX_INLINE_SIZE = 10;

var _messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

/*
 * The quick search section at the top of the left nav.
 */
dojo.declare("com.ibm.team.workitem.web.ui.internal.page.SearchProvider", [jazz.ui._SearchProvider], {
	constructor: function(parms) {
		if(parms && parms.parentWidget) {
			this.parentWidget = parms.parentWidget;
		}
	},
	scopes: [
		{id: SCOPE_ALL, label: _messages.labelAllWorkItems, icon: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/images/workitems-searchscope_obj.png")},
		{id: SCOPE_UNRESOLVED, label: _messages.labelAllOpenWorkItems, icon: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/images/workitems-searchscope_obj.png")}				
				],
	tooltip: _messages.titleSearch,
	defaultText: _messages.labelSearchWorkItems,
	noResultsLabel: _messages.workItemsNoResultsLabel,
	getOnUpdate: function() {
		return dojo.hitch(this, this.onUpdateInline);
	},
	getOnExecute: function() {
		return dojo.hitch(this, this.onExecuteInline);
	},
	onUpdateInline: function(queryString, scope, callback, errback) {
		var args = {
			query: queryString,
			scopes: scope,
			callback: callback,
			errback: errback,
			isInline: true
		}
		this.onUpdate(args);
	},
	onExecuteInline: function(queryString, scope, callback, errback) {
		this.onExecute(queryString, scope);
	},
	onUpdate: function(args) {
			if(args.query == null || args.query.length == 0) return;
		
			var self = this;
			var serviceObj= {
				_callback: args.callback,
				_errorCallback: args.errback,
				success: function(summaryPage){
					if(args.isInline) {
						var results = self._formatQueryResults(summaryPage, args.query, 1, MAX_INLINE_SIZE, /* isInline */ true);
						this._callback(results.results);
					} else {
						var results = self._formatQueryResults(summaryPage, args.query, args.page, args.maxResults);
						this._callback(results);
					}
				},
				failure: function(/* Error */ error) {
					this._errorCallback(error.message);
				}
			};

			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			
			var params;
			if (WorkItemUtil.isWorkItemId(args.query))
				params = {
					id: WorkItemUtil.getWorkItemId(args.query),
					scopeToProject: false // search across project areas
				};	
			else {
				params = {
					maxResults: MAX_SIZE,
					fullText: args.query
				};
				if(args.scope == SCOPE_UNRESOLVED) 
					params.status = "open";
			}
			
			QueryClient.getResults(params, srh);
	},
	onExecute: function(queryOrArgs, scope) {
		   var query;
		    if(!scope) {
		    	scope = queryOrArgs.scope;
		    	query = queryOrArgs.query;
		    } else {
		    	query = queryOrArgs;
		    }
			if(query == null || query.length == 0) return;
			var openOnly= (scope == "unresolved"); // $NON-NLS-1
			var uri = WorkItemUtil.getSearchUri(query, openOnly, false);
			var isGadget= this.parentWidget && this.parentWidget.isGadget;
			com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation(isGadget, false, uri, null, null);
		},
    _createResult: function(/* ResponseDTO */ dto, /*string*/ query, isInline) {
        	var value = jazz.util.html.escape(this._getPlainText(dto.summary));
			
			var currentId = dto.id;
			var aLabel = this._formatLabel(currentId + ": " + value, query);
			var item;			
			if(!isInline) {
				item = {
					icon: dto.typeIconURL,
					label: aLabel,
					uri: dto.locationUri
				}
			} else {
				item = {
					id: currentId,
					value: value.length>53 ? (value.substring(0,50) + "...") : value,
					label: aLabel
				}				
		
				var self = this;
				var handler = function(args) {
					var isGadget= false;
					if(self.parentWidget && self.parentWidget.isGadget) isGadget = true;
					com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation(isGadget, false, 
						WorkItemUtil.getViewWorkItemUri(currentId, query), null, null);
					self.controller.setWindowLocationHref(WorkItemUtil.getViewWorkItemUri(currentId, query));
				};
				item.handler = handler;
			}
			return item;
        },
	_getPlainText: function(html) {
		var div= document.createElement("div");
		div.innerHTML= html;
		return div.textContent || div.innerText;
	},
     _formatLabel: function(/* string */ label, /* string */ query) {
			var index = label.toLowerCase().indexOf(query.toLowerCase());
			if (index != -1) {
				return label.substring(0, index) + "<b>" + label.substring(index, index + query.length) + "</b>" 
							+ label.substring(index + query.length);
			}
			return label;
		},
	_formatQueryResults: function(summaryPage, query, pageNumber, pageSize, isInline){
		    var dtos = summaryPage ? summaryPage.workItemSummaryDTOs : [];
		    var dtos = dtos || [];

            //sort the results
//            dtos.sort(function(a, b){
//                return a.id < b.id ? -1 : 1;
//            });

			var items = [];
			var firstIndex = (pageNumber-1) * pageSize;
			var lastIndex = firstIndex + pageSize - 1;
            if (dtos.length === 0 || dtos.length < (firstIndex + 1)) {
					return {
						paging: true,
						totalResults: summaryPage.totalQueryResultCount,
						results: items
					};
            }
            
            var handler;
            var lastIndex = Math.min(lastIndex, dtos.length);
			for(var i=firstIndex; i< lastIndex; i++ ){				
				var item = this._createResult(dtos[i], query, isInline);
				items.push(item);
			}		
			
           		
		return {
			paging: true,
			totalResults: dtos.length,
			results: items
		};
	}
});

})();
