/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.CreateFromTemplateAction"); //$NON-NLS-1
dojo.require("com.ibm.team.workitem.web.cache.internal.QueryProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.CacheUtil"); //$NON-NLS-1$


dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var EditorUtil            	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var LayoutUtil            	= com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var EventTopics				= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var PlatformUI;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
// matches any text other than a digit
var NON_NUMERIC_REGEX		= /[^0-9]/;

var DEFAULT_ORDER_ATTRIBUTE	= "id";
var DEFAULT_DIRECTION		= "descending";
var DEFERRED_MOUSEOVER_DELAY = 600; // milliseconds to delay the deferred mouseOver

/**
 * Main application controller method for Jazz work item web ui.  Modules
 * *outside* of the com.ibm.team.workitem.web component should *not* access
 * methods in this controller directly; rather external modules should update
 * the browser's Uris using Uris returned from
 * <code>com.ibm.team.workitem.web.client.util</code> helper methods.
 */
dojo.declare("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController", null, {

	/*
	 * Internal constructor; should not be called by *any* non-Dojo code.
	 */
	constructor: function(workItemPageView) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		PlatformUI = net.jazz.ajax.ui.PlatformUI;

		this._currentOperation= null;
		this._defaultQuery= null;
		this.suspendDelaysUntil = 0;
		this._workItemPageView = workItemPageView;
		this.isGadget = workItemPageView ? workItemPageView.isGadget : false;
		com.ibm.team.workitem.web.ui.internal.module.WindowContext.isGadget = this.isGadget;
		this.pageController = this;
		this.mouseListeners = [];
		// Handle mouse motion events
		this.eventHandles = [];
		this.eventHandles.push(dojo.subscribe(EventTopics.SHOW_ERROR_PAGE, dojo.hitch(this, this._showErrorPage)));
		this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_COMPONENT_MENU_LOADED, dojo.hitch(this, this._initializeComponentMenus)));
		this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_ACTION_CREATE_FROM_TEMPLATE, dojo.hitch(this, this._createFromTemplate))); 
		this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_QUERY_VIEWED, dojo.hitch(this, this._queryViewed)));

		this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_ACTION_CLEAR_QUERY_EDITOR, dojo.hitch(this, this._queryClearEditor)));		
		this.eventHandles.push(dojo.subscribe(EventTopics.QUERY_EDITOR_CHANGED, dojo.hitch(this, this._queryChanged)));
		this.eventHandles.push(dojo.subscribe(EventTopics.QUERY_EDITOR_NAME_CHANGED, dojo.hitch(this, this._checkQueryNameInComponentMenu)));
		this.eventHandles.push(dojo.subscribe(EventTopics.QUERY_DELETED, dojo.hitch(this, this._queryDeleted))); 
		this.eventHandles.push(dojo.subscribe(EventTopics.BUILDER_POST_SAVE, dojo.hitch(this, this._querySaved)));
		this.eventHandles.push(dojo.subscribe(EventTopics.SHOW_WELCOME_PAGE, dojo.hitch(this, this.viewWelcome)));
		this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_SETPAGEDIRTY, dojo.hitch(this, this.setPageDirty)));

		if(this.isGadget) {
			this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._rpcNotifyOfWorkItemChanged)));
			this.eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_STORED, dojo.hitch(this, this._rpcNotifyOfWorkItemStored)));
		}
	},
	setPageDirty: function() {
		com.ibm.team.workitem.web.cache.internal.CacheUtil.setPageDirty();
	},
	destroyController: function() {
		com.ibm.team.workitem.web.cache.internal.Cache._pageController = null;
		for(var i = 0;i< this.eventHandles.length; i++) {
			dojo.unsubscribe(this.eventHandles[i]);
		} 
	},
	getCurrentProjectArea: function() {
		this._workItemPageView.getCurrentProjectArea();
	},
	validActions: ["search", "simpleQuery", "similarityQuery", "runSavedQuery", "runNewQuery", "newQuery", "editQuery", "runDefaultQuery", "showTemplateQuery", "newWorkItem", "newWorkItemSet",  "viewWorkItem", "viewWelcome", "viewQueries" ],
	// Keep a list of query actions so the query results know whether to refresh itself or not
	queryActions: ["search", "simpleQuery", "similarityQuery", "runSavedQuery", "runNewQuery", "newQuery", "editQuery", "runDefaultQuery", "showTemplateQuery", "viewQueries" ],
	getPageAction: function(functionName) {
		for(var i=0;i<this.validActions.length;i++) {
			if(this.validActions[i] == functionName) {
				return this[this.validActions[i]];
			}
		}
		return null;
	},
	_queryDeleted: function(itemId) {
		 com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + itemId);
	},
	_queryViewed: function(queryDto, isChanged, isSaved) {
		if(!isSaved && this._lastViewedQueryItemId == queryDto.itemId) {
			return;
		}
		/*
		 * When its saved, clear the last viewed so the next update marks the item dirty
		 */
		if(isSaved) {
			this._lastViewedQueryItemId = "_SYNTHETIC_UNDEFINED";
		} else {		
			this._lastViewedQueryItemId = queryDto ? queryDto.itemId || "-1" : "-1";
		}
		this._updateQueryComponentMenu(queryDto);
	},
	_checkQueryNameInComponentMenu: function(queryDto) {
		if(!this._queryChanged(/* skip query viewed */ true)) return; // return if destroyed or not initialized
		if(queryDto.name != this._lastQueryComponentMenuName) {
			this._updateQueryComponentMenu(queryDto);
		}
	},
	_updateQueryComponentMenu: function(queryDto) {
		this._checkChangedQueries(queryDto);
		/* Remove the current query */
		var queryItemId = queryDto.clientItemId || queryDto.itemId;
		var wasChanged = false;
		var _isBeingEdited = false;
		var oldProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + queryItemId);
		if(oldProxy) {
			wasChanged  = oldProxy.getValue({path: "attributes.isChanged"});
			_isBeingEdited  = oldProxy.getIsBeingEdited();
		} else {
			_isBeingEdited = window.location.href.indexOf("editQuery") != -1;
		}
		var options = {
			filterAttributes: [ "isQuery" ]
		}
		com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + queryItemId);
		var queryProxyItems = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems(options);
		queryProxyItems = queryProxyItems.reverse();
		var toBeRemoved = [];
		for(var i=0; i<queryProxyItems.length;i++) {
			var cachedQuery = queryProxyItems[i];
			cachedQuery.setValue({path: "attributes.isCurrent", attributeId: "isCurrent", value:false});
			if(i>8) {
				toBeRemoved.push(cachedQuery);
			}
		}
		for(var i=0; i<toBeRemoved.length;i++) {
			var cachedQuery = toBeRemoved[i];
			com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + cachedQuery.getItemId());
		}
		
		var hasNewQueryUrl = window.location.href.indexOf("action=com.ibm.team.workitem.newQuery") != -1;
		
		var proxy =  new com.ibm.team.workitem.web.cache.internal.QueryProxy({queryDto: queryDto});
		proxy.setIsBeingEdited(_isBeingEdited);
		proxy.setValue({path: "attributes.isCurrent", attributeId: "isCurrent", value:true});
		proxy.setValue({path: "attributes.isChanged", attributeId: "isChanged", value:wasChanged});
		proxy.setValue({path: "attributes.hasNewQueryUrl", attributeId: "hasNewQueryUrl", value:hasNewQueryUrl});
		proxy._queryUrl = window.location.href;
		
		this._lastQueryComponentMenuName = queryDto.name;
		
		com.ibm.team.workitem.web.cache.internal.Cache.addItem("workitems/queries/" + queryItemId, proxy);
		dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
	},
	resetQueryProxy: function(id) {
			var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/"+id);
			if(proxy) {
				proxy.setValue({path: "attributes.hasNewQueryUrl", attributeId: "hasNewQueryUrl", value:false});
			}
	},
	_checkChangedQueries: function(queryDto) {
		var itemId = queryDto.itemId || queryDto.clientItemId;
//		if(this._lastEditedQuery && (itemId == this._lastEditedQuery.itemId)) return;
		var isDirty = false;
		if(this._workItemPageView && this._workItemPageView.queryBuilderWidget && this._workItemPageView.queryBuilderWidget._query) {			
			var editedQuery= this._workItemPageView.queryBuilderWidget._query;
			var editedQueryItemId = editedQuery.clientItemId || editedQuery.itemId;
			isDirty = this._workItemPageView.queryBuilderWidget._isDirty;
		}
		if(editedQueryItemId != itemId) return;
		var options = {
			filterAttributes: [ "isQuery" ]
		}
		// Only one query can be changed at a time, so mark all the rest unchanged for now
		var queryProxyItems = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems(options);
		for(var i=0; i<queryProxyItems.length;i++) {
			var cachedQuery = queryProxyItems[i];
			if(isDirty && queryDto && cachedQuery.getItemId() == editedQueryItemId ) {
				cachedQuery.setValue({path: "attributes.isChanged", attributeId: "isChanged", value:true});	
			} else {
				cachedQuery.setValue({path: "attributes.isChanged", attributeId: "isChanged", value:false});
			}
		}
	},
	_removeQuery: function(parms) {
		com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + parms.query.itemId);
	},
	_resetQuery: function(parms) {
		var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + parms.query.itemId);
		if(!proxy) return;
		proxy.resetToStored();
		dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH);
	},
	_replaceQuery: function(parms) {
		var _isBeingEdited = false;
		var oldProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + parms.newQuery.itemId);
		if(oldProxy) {		
			_isBeingEdited = oldProxy._isBeingEdited;
		} else {
			_isBeingEdited = window.location.href.indexOf("editQuery") != -1;
		}
		com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + parms.oldQuery.itemId);
		var proxy =  new com.ibm.team.workitem.web.cache.internal.QueryProxy({queryDto: parms.newQuery});
		proxy.setValue({path: "attributes.isCurrent", attributeId: "isCurrent", value:true});
		proxy._isBeingEdited = _isBeingEdited;
		com.ibm.team.workitem.web.cache.internal.Cache.addItem("workitems/queries/" + parms.newQuery.itemId, proxy);
	},
	_setQueryName: function(queryDto) {
		if(!this._workItemPageView || !this._workItemPageView.queryBuilderWidget) return; 
		var item = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + queryDto.itemId);
		if(!item) return;			
	    item.object.name = this._workItemPageView.queryBuilderWidget.getName() || "";
	},
	_querySaved: function(queryDto) {
		if(!queryDto || !this._workItemPageView || !this._workItemPageView.queryBuilderWidget) return;
		var queryDto = this._workItemPageView.queryBuilderWidget._query;
		var item = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + queryDto.itemId);
		if(!item) return;
		item.setValue({path: "attributes.isChanged", attributeId: "isChanged", value:false});
		this._setQueryName(queryDto);
		this._queryViewed(queryDto, false, true);
	},
	_queryChanged: function(skipQueryViewed) {
		if(!this._workItemPageView || !this._workItemPageView.queryBuilderWidget || this._workItemPageView.queryBuilderWidget._isInitializing) {
		   return false;
		}
		var queryDto = this._workItemPageView.queryBuilderWidget._query;
		this._setQueryName(queryDto);
		if(!skipQueryViewed) {			
			this._queryViewed(queryDto, true);
		}
		return true;
	},
	_rpcNotifyOfWorkItemChanged: function(workItemProxy) {
		if(workItemProxy.isChanged()) {		
			com.ibm.team.workitem.web.ui.internal.module.WindowContext.rpcCallToContainer("com_ibm_team_workitem_rpc_onchange", null, workItemProxy, null, null);
		}
	},
	_rpcNotifyOfWorkItemStored: function(workItemProxy) {
		com.ibm.team.workitem.web.ui.internal.module.WindowContext.rpcCallToContainer("com_ibm_team_workitem_rpc_onstore", null, workItemProxy, null, null);
	},
 	_setPageTitle: function(title) {
 		if(PlatformUI && PlatformUI.getWorkbench) {
	    	PlatformUI.getWorkbench().setTitle(title);
 		}
 		if(this._workItemPageView.isGadget) {		
		    com.ibm.team.workitem.web.ui.internal.module.WindowContext.rpcSetTitle(title);
 		}
	}, 
	_setQueryPageTitle: function(queryName, isEditingQuery) {
		this._lastQueryName = queryName || this.messages.labelDefaultQueryName;
		this._setPageTitle(dojo.string.substitute(this.messages.pageTitleQuery, [ this._lastQueryName ]));
		dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
	},
	_setWorkItemPageTitle: function(workItem) {
		if(workItem && workItem.attributes && workItem.attributes.workItemType) {
				if (workItem.id < 0) {
					this._setPageTitle(this.messages.pageTitleNewWorkItem);
				}
				else {
					var title = this.messages.pageTitleWorkItem;
					var summary;
					if (workItem.attributes.summary.content) {
						var summaryContent = EditorUtil.restoreTags(workItem.attributes.summary.content);
						summary = summaryContent.substring(0, Math.min(workItem.attributes.summary.content.length, 200));
						if (summary.length >= 200) {
							title = dojo.string.substitute(this.messages.pageTitleWorkItemSummaryMore, [workItem.attributes.workItemType.label, workItem.id, summary]);
						} else {
							title = dojo.string.substitute(this.messages.pageTitleWorkItemSummary, [workItem.attributes.workItemType.label, workItem.id, summary]);
						}
					}
					this._setPageTitle(title);
				}
		} else {
			this._setPageTitle(this.messages.pageTitleWorkItem);
		} 
	},
	setWindowLocationHref: function(url, isReplace) {
//		console.log("pageController.setWindowLodationHref isGadget " + this.isGadget + " url:" + url);
		com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation3(this.isGadget, this.isFramed, url, this._workItemPageView.frameOrigin, this._workItemPageView.frameId, isReplace == true);
	},
	_updateUrl: function(url) {
		if(this.isFramed || this.isGadget) {
			this.setWindowLocationHref(url);
			return;
		}
	 	var split = url.split("#");
	 	var hash = split.length == 0 ? "" : split[1];
//	 	alert("update url at WorkItemPageController:" + hash + " from location" + window.location.href);
	 	dojo.hash(hash,true);
 	},
	
	/*
	 * Special method called by the Jazz web ui framework when the URL contains
	 * no 'action' request parameter.  In this scenario, we show the welcome page
	 * 
	 * This function name can be changed if we change 
	 * the page's defaultAction attribue in workitem.web/plugin.xml.
	 */
	_showDefaultView: function(args) {
		
		this._workItemPageView._registerNavigation();
		// If we are returning here from another page (e.g., planning), keep context
 		if(!this._isSubsequentView || this._forceShowingDefaultView){ // first time or forced
			this.viewWelcome({}); // TODO: Remove this when the location.replace works reliably. defect 77524
        	this._isSubsequentView = true;
  			if (dojo.isIE) {
				setTimeout(dojo.hitch(this,function(url){
					this._updateUrl(url);
					this._forceShowingDefaultViewMsg = null;
				} , WorkItemUtil.getWelcomePageUri (this._forceShowingDefaultViewMsg)),250);
			} else {
				this._updateUrl(WorkItemUtil.getWelcomePageUri (this._forceShowingDefaultViewMsg));
				this._forceShowingDefaultViewMsg = null;
			}
    	} else { // next time
			var info = this.getLastQueryInfo();
			if(info && info.url) {
				if (dojo.isIE) {
					setTimeout(dojo.hitch(this,this._updateUrl,info.url),0);
				} else {
					this._updateUrl(info.url);
				}
			}
		}
		// show whatever was the current view
	},

	
	_fetchAndRunDefaultQuery: function() {
		var self= this;
		var handlerObj= {
			success: function(query) {
				if (!query) {
					self.handleError(
						{message: self.messages.msgQueryNoDefaultQueryFound},
						self.messages.errorRunningDefaultQuery);
					dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
					return;
				}
				self._defaultQuery= query;
				self._internalRunQuery(self._defaultQuery);
				// initialize queries done in _internalRunQUery
			},
			failure: function(errorObj) {
				self.handleError(
					errorObj,
					self.messages.errorRunningDefaultQuery);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			}
		};
		var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
		QueryClient.getDefaultQuery(handler);
	},

	/*
	 * Creates an operation specified by the given name and id.
	 */
	createOperation: function(/* string */ name, /* string */ id) {
		checkNotNull(name, "name");
		checkNotNull(id, "id");
		return { name: name, id: id };
	},
	
	/*
	 * Marks the start of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	startOperation: function(operation) {
		if(PlatformUI && PlatformUI.getWorkbench()) {
			PlatformUI.getWorkbench().showMessage(operation.name);
		}
	},

	/*
	 * Marks the end of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	finishOperation: function(operation) {
		if(PlatformUI && PlatformUI.getWorkbench()) {
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},

	/*
	 * Handles an error in a standard way.
	 */
	handleError: function(obj, msg) {
		if(this._isReloading) return; // avoid repeated errors
		var humanFriendlyMsg= null;
		if (!obj) { obj= {}; }
		if (obj.status === 0) {
			humanFriendlyMsg= this.messages.errorServerNotResponding;
		}
		var summaryMsg= msg && dojo.isString(msg) ? msg : null;
		var detailedMsg= humanFriendlyMsg ? humanFriendlyMsg : obj.message;
		if(!detailedMsg || (detailedMsg.toLowerCase() == "not found")) {
			detailedMsg = this.messages.errorServerNotResponding;
		} else if(detailedMsg.toLowerCase() == "unknown") {
			detailedMsg = this.messages.errorUnknownDetails;
		}
		console.debug(dojo.string.substitute(this.messages.errorConsoleDetails, [ summaryMsg ? summaryMsg : this.messages.labelUnknownError , detailedMsg]));
		this._isReloading = true;

		this._openDefaultView(detailedMsg);
	},
	_openDefaultView: function(msg) {
		this._forceShowingDefaultView = true;
		this._forceShowingDefaultViewMsg = msg;
		
		if(msg && dojo.isIE) {
			setTimeout(dojo.hitch(this, function() { if(this._forceShowingDefaultViewMsg) alert(this._forceShowingDefaultViewMsg) } ),500);
		}
		
		var split = window.location.href.split("#");
		this._updateUrl(split[0] + "#action=jazz.viewPage&id=com.ibm.team.workitem");

	},
	/*
	 * QUICK SEARCH-related methods.
	 */

	/*
	 * Responds to the submission of the "quick search" form.  Any non-null,
	 * non-empty is valid.  If input is a positive int, the app will attempt to
	 * open a work item with that id; if input is anything else, the app will
	 * run a textual search against the input.
	 *
	 * This method should not be called directly.
	 */
//	_handleQuickSearch: function(searchText, openOnly) {
//		checkNotNull(searchText, "searchText");
//		var searchText= dojo.trim(searchText);
//		var quoteLess = dojo.trim(searchText.replace(/\"/g, "").replace(/\'/g, ""));
//		if(quoteLess.length == 0) {
//			this._openDefaultView();
//			return;
//			}
//		// If the value is a number, it is a work item number
//		if(!isNaN(searchText)) {
//			try{
//				var value = parseInt(searchText, 10);
//				if(value > 2147483647 || value < 1) {
//					searchText = '"' + value + '"';
//				}
//			} catch(e) {} // ignore parseInt failure
//		}
//		this.resetLastQueryString();
//		if (NON_NUMERIC_REGEX.test(searchText)) {
//			this._internalSearch(searchText, openOnly);
//		} else {
//			searchText = searchText.replace(/^[0]+/g,"");
//			this._internalViewWorkItem(searchText);
//		}
//	},
	
	/*
	 * Sets the browser's URI so that the Jazz framework calls the 'search'
	 * method.  May be called by other code within the com.ibm.team.workitem.web
	 * bundle.
	 */
	openSimilarityQuery: function(/* itemId: , summary: , content:, projectAreaItemid */ parms) {
		var href  =  WorkItemUtil.getSimilarityQueryUri(parms);
		if(parms.forceRefresh) {
			href = WorkItemUtil.reuniquifyUrl (href);
		}
		this.setWindowLocationHref(href);
	},
	
	/*
	 * Sets the browser's URI so that the Jazz framework calls the 'search'
	 * method.  May be called by other code within the com.ibm.team.workitem.web
	 * bundle.
	 */
	_internalSearch: function(input, openOnly) {
		checkNotNull(input, "input");
		this.setWindowLocationHref(WorkItemUtil.getSearchUri(input, openOnly, true));
	},
	_isOnQueryPage: function() {
		var location = window.location.href;
		for(var i=0;i<this.queryActions.length;i++) {
			if(location.indexOf(this.queryActions[i]) != -1) {
				return true;
			}
		}
		return false;
	},
	forceRefreshOfQueryResults: function() {
		/*
		 * Return true if we are showing query results, else false
		 * if false, refresh will happen the next time the query is run
		 */		
		if(this._isOnQueryPage()) {
			return true
		}
		this.resetLastQueryString();
		return false;
	},
	resetLastQueryString: function() {
		this._lastQueryKey = "";
	},
	/*
	 * Called from the SHOW_ERROR_PAGE dojo event
	 */
	_showErrorPage: function(message) {
		this.viewWelcome({error: message});
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Show the welcome page
	 */
	viewWelcome: function( kwArgs) {
		kwArgs = kwArgs || {};
		dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
		if(this._workItemPageView) {
			this._workItemPageView.showWelcomePage(kwArgs.error);			
		} else {
		    if(kwArgs && kwArgs.error && kwArgs.error.message) {
		    	alert(kwArgs.error.message);
		    }
		}
		this._isSubsequentView = true;
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.	
	 * Show the "Create from Template" dialog.
	 */
	newWorkItemSet: function(kwArgs) {
		if(!this._isSubsequentView) {
			this.viewWelcome({});
		}
		/* Refresh the component menu so another action will pick up the url change */
		dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH);
		this._createFromTemplate();
	},
	/*
	 * Show the queries page
	 *    queryType: either "myQueries" or "shared"
	 */
	viewQueries: function( kwArgs ) {
		this._workItemPageView.showQueriesPage(kwArgs);			
		dojo.publish(EventTopics.INITIALIZE_QUERIES, []);			
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Show the last query returned from "Create from Template"
	 */
	showTemplateQuery: function( kwArgs) {
		if(!this._lastTemplateQuery) {
			this._workItemPageView.showWelcomePage(this.messages.errorNoTemplateQuery);
			/* Since this probably occurs loading a bookmark, initialize the queries */
			dojo.publish(EventTopics.INITIALIZE_QUERIES, []);				
		} else {
			if(!this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
	    		this._showQueryBuilderOrCancel();
				return;
			}
			this.runQuery(this._lastTemplateQuery);
		}
	},
	_setTemplateQuery: function(query) {
		this._lastTemplateQuery = query;
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Searches for work items matching the string specified by 'kwArgs.q'.
	 */
	runDefaultQuery: function( kwArgs) {
		this._isSubsequentView = true;
		this._lastColumnArgs=null;
  		kwArgs.tq = kwArgs.tq == null ? "" : kwArgs.tq; 
		var queryKey = "runDefaultQuery:"+ kwArgs.tq;
		//cache If the search text is unchanged, show the same page again
		
		if (!kwArgs.index) {kwArgs.index= null;}
		if (!kwArgs.orderBy) {kwArgs.orderBy= null;}
		if (!kwArgs.direction) {kwArgs.direction= null;}
		if (!kwArgs.filterAttribute) {kwArgs.filterAttribute= null;}
		if (!kwArgs.filterValue) {kwArgs.filterValue= null;}
		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}

		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastFilterAttribute == kwArgs.filterAttribute
				&& this._lastFilterValue == kwArgs.filterValue) {

			this._setQueryPageTitle(this._lastQueryName, true);
			this._workItemPageView.showLastQueryResults();
			return;
		}

		this._lastEditedQuery = null;				
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryKey = queryKey;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastFilterAttribute = kwArgs.filterAttribute;
		this._lastFilterValue = kwArgs.filterValue;
		
		var op = this.createOperation(this.messages.operationRunningDefaultQuery, "runDefaultQueryOperation");
		this.startOperation(op);
		this._workItemPageView.showLastQueryResults();
		// run default query
		this._setPageTitle(""); // Blank for default view
		if (this._defaultQuery) {
			this._internalRunQuery(this._defaultQuery);
		} else {
			this._fetchAndRunDefaultQuery();
		}
		this.finishOperation(op);
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Searches for work items matching the string specified by 'kwArgs.q'.
	 */
	search: function(kwArgs) {
		checkNotNull(kwArgs.q, "kwArgs.q");
		this._isSubsequentView = true;
		var searchText= dojo.trim(kwArgs.q);
		var quoteLess = dojo.trim(searchText.replace(/\"/g, "").replace(/\'/g, ""));
		if(quoteLess.length == 0) {
			this._openDefaultView();
			return;
			}
		
		// If it is a search for a number, view the work item
		if(!NON_NUMERIC_REGEX.test(searchText)) {
			this._internalViewWorkItem(searchText, null, /* isSearch */ true);
			return;
			// initialize queries is done in viewWorkItem... which gets called eventually from internalViewWorkItem
		}
		
		if(!this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
			this._showQueryBuilderOrCancel();
			return;
		}
		
		var queryKey = "Search:"+searchText;
		//cache If the search text is unchanged, show the same page again
		var openOnly = kwArgs.o ? true : false;
		
		// if we're already caching the query editor editor, and the timestamp is the same, simply show the cached page
		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage) {
			this._setQueryPageTitle(this._lastQueryName, true);
			this._workItemPageView.showLastQueryResults();
			return;
		}
		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;
		this._lastEditedQuery = null;		
		this._lastQueryUrl = window.location.href;

	// If its the same query key, but different other factors, rerun the query
		if(queryKey == this._lastQueryKey) {
			this._internalRunQuery(this._lastQueryRun, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, true);
			return;
		}

		this._lastQueryKey = queryKey;
		 
		if (openOnly) {
			var queryName= dojo.string.substitute(this.messages.msgSearchResultsOpen, [( "" + kwArgs.q) ]);	
		} else {
			var queryName= dojo.string.substitute(this.messages.msgSearchResults, [( "" + kwArgs.q) ]);
		}
		
		var query= QueryClient.createFulltextQuery(queryName, searchText, openOnly);
		
		this._setQueryPageTitle(query.name, false);
		this._lastQueryRun = query;
		this._internalRunQuery(query, 0);	
	},
	
	/*
	 * QUERY-related methods.
	 */

	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the saved query with itemId specified by the paramter 'kwArgs.id'.
	 * Displays query results in a table.
	 */
  	runSavedQuery: function(kwArgs) {
		this._isSubsequentView = true;
		
		if(!this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
			this._showQueryBuilderOrCancel();
			return;
		}
		
		this._lastColumnArgs=null;
  		// kwArgs.action=runSavedQuery kwArgs.id=_uuid kwArgs.tq=[timestamp]
  		checkNotNull(kwArgs, "kwArgs");
  		checkNotNull(kwArgs.id, "kwArgs.id");
		
  		kwArgs.tq = kwArgs.tq == null ? "" : kwArgs.tq; 
		var queryKey = "runSaved:"+kwArgs.id + ":" + kwArgs.tq;
		//cache If the search text is unchanged, show the same page again
		
		if (!kwArgs.index) {kwArgs.index= null;}
		if (!kwArgs.orderBy) {kwArgs.orderBy= null;}
		if (!kwArgs.direction) {kwArgs.direction= null;}
		if (!kwArgs.itemsPerPage) {kwArgs.itemsPerPage= null;}
		if (!kwArgs.filterAttribute) {kwArgs.filterAttribute= null;}
		if (!kwArgs.filterValue) {kwArgs.filterValue= null;}
		if (kwArgs.preserveFilter == "true") {
			if(!kwArgs.filterAttribute && this._lastFilterAttribute) {
				kwArgs.filterAttribute = this._lastFilterAttribute;
			}
			if(!kwArgs.filterValue && this._lastFilterValue) {
				kwArgs.filterValue = this._lastFilterValue;
			}
			kwArgs.preserveFilter = true
			}

		if (!kwArgs.refresh && !this._lastEditedQuery && queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage
				&& this._lastFilterAttribute == kwArgs.filterAttribute
				&& this._lastFilterValue == kwArgs.filterValue) {

			this._setQueryPageTitle(this._lastQueryName, true);
			this._workItemPageView.showLastQueryResults();
			return;
		}
		this._lastEditedQuery = null;		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryKey = queryKey;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;
		this._lastFilterAttribute = kwArgs.filterAttribute;
		this._lastFilterValue = kwArgs.filterValue;
		
  		var self= this;
		
		var serviceObject= {
			success: function(query) {
				if (!query) {
					self._handleQueryNotFound();
					return;
				}
				self._setQueryPageTitle(query.name, query.id);
				self._internalRunQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter, kwArgs.filterAttribute, kwArgs.filterValue);
				/*
				 * If it is a refresh then replace the url with one that does not force refresh
				 */
				if(kwArgs.refresh) {
					var newUrl = window.location.href.replace(/\&refresh=true/, "");
					self.setWindowLocationHref(newUrl, true);
					return;
				}
				// Initialize queries is done in internalRunQuery
			},
			error: function(error) { 
				self.handleError(error, self.messages.errorQueryRunningSavedQuery);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};
			
		QueryClient.getQuery(new ServiceResponseHandler(serviceObject, "success", "error"), kwArgs.id);
  	},
	
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the saved query with its structure specified by the paramter 'kwArgs.q'.
	 * Displays query results in a table.
	 */
  	runNewQuery: function(kwArgs) {
		this._isSubsequentView = true;
  		checkNotNull(kwArgs, "kwArgs");
  		checkNotNull(kwArgs.q, "kwArgs.q");
		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}
  		var query= WorkItemUtil.deserializeQuery(kwArgs.q);
//		this._setQueryPageTitle("New Query"); not needed, set in internal run query
		this._internalRunQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter, undefined, undefined, this._lastColumnArgs);
  	},

	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Runs the find duplicates query
	 */	
	similarityQuery: function(kwArgs) {
		// kwargs.itemId = UUID of work item, unless it is new then null and only summary/content provied
		// kwargs.summary = summary of a new work item 
		// kwargs.content = description + comments of a work item

		if(!this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
			this._showQueryBuilderOrCancel();
			return;
		}

		this._isSubsequentView = true;
		this._lastColumnArgs=null;

		var name= kwArgs.name || this.messages.labelPotentialDuplicates;
		this._lastQueryName = name;
		var queryKey = "runSimilarity:"+kwArgs.itemId + ":" + kwArgs.content + ":" + kwArgs.summary + ":" + kwArgs.tq;
		//cache If the search text is unchanged, show the same page again		
		if (!kwArgs.index) {kwArgs.index= null;}
		if (!kwArgs.orderBy) {kwArgs.orderBy= null;}
		if (!kwArgs.direction) {kwArgs.direction= null;}
		if (!kwArgs.itemsPerPage) {kwArgs.itemsPerPage= null;}
		if (!kwArgs.filterAttribute) {kwArgs.filterAttribute= null;}
		if (!kwArgs.filterValue) {kwArgs.filterValue= null;}
		if (!kwArgs.projectAreaItemId) {kwArgs.projectAreaItemId= null;}
		if (kwArgs.preserveFilter == "true") {
			if(!kwArgs.filterAttribute && this._lastFilterAttribute) {
				kwArgs.filterAttribute = this._lastFilterAttribute;
			}
			if(!kwArgs.filterValue && this._lastFilterValue) {
				kwArgs.filterValue = this._lastFilterValue;
			}
			kwArgs.preserveFilter = true
			}
		
		this._lastEditedQuery = null;
		if (queryKey == this._lastQueryKey
				&& this._lastQueryIndex == kwArgs.index
				&& this._lastQueryOrderBy == kwArgs.orderBy
				&& this._lastQueryDirection == kwArgs.direction
				&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage
				&& this._lastProjectAreaItemId == kwArgs.projectAreaItemId
				&& this._lastFilterAttribute == kwArgs.filterAttribute
				&& this._lastFilterValue == kwArgs.filterValue) {

			this._setQueryPageTitle(name, true);
			this._workItemPageView.showLastQueryResults();
			return;
		}
		this._lastQueryItemId = "similarityQuery";
		this._lastQueryKey = queryKey;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;
		this._lastProjectAreaItemId = kwArgs.projectAreaItemId;
		this._lastFilterAttribute = kwArgs.filterAttribute;
		this._lastFilterValue = kwArgs.filterValue;

		var query;
		var currentWorkItemEditor;
		if(kwArgs.itemId) {
			currentWorkItemEditor = this._workItemPageView.getRecentWorkItemEditor(kwArgs.itemId);
		}
		
		/*
		 * If the work item is not in the cache, or
		 * the work item is not new and there are no changes
		 * then use the values from the keywords
		 */
		if(!currentWorkItemEditor || (!currentWorkItemEditor.isNewWorkItem() &&  !currentWorkItemEditor.isDirty())) {
			if (kwArgs.summary || kwArgs.content) { // this path may be dead
				query = QueryClient.createSimilarityQuery(name, kwArgs.itemId, kwArgs.summary, kwArgs.content);
			}
			else {
				if(kwArgs.itemId) {
					query = QueryClient.createSimilarityQuery(name, kwArgs.itemId);
				} else {
					this.handleError(/* errorObj */ null, this.messages.errorRunningPotentialDuplicatesQueryMissingArgument); 
				}
			}			
		} else {
			/* There is a current work item editor and it is dirty, so create a query from the current values in the editor */
			var summary, description = "", comment, comments = "";
			var currentWorkItem = currentWorkItemEditor._workItem;
			try  {
				// TODO: get summary, description and comment in a standard attribute way
				summary = currentWorkItemEditor.getAttributeValue("summary"); // _editorParts.header.summary.value();
				
				var descWidget = currentWorkItemEditor.findObjectWithProperty({
					property: "isDescriptionManager"
				});
				if(descWidget && descWidget.getDescriptionValue) {
					description=dojo.trim(descWidget.getDescriptionValue() || "");
				}
							
				if (currentWorkItem && currentWorkItem.attributes && currentWorkItem.attributes.internalComments && currentWorkItem.attributes.internalComments.items) {
						for (var i = 0; i < currentWorkItem.attributes.internalComments.items.length; i++) {
							item = currentWorkItem.attributes.internalComments.items[i];
							if (item.content) {
								comments += " " + item.content;
							}
						}
					}
					
				comment = currentWorkItemEditor.getAttributeValue(currentWorkItem.SYNTHETIC_COMMENT_ID);
				if(comment) {
					comments += " " + comment;
				}
				description += " " +  comments;
				query = QueryClient.createSimilarityQuery(name, currentWorkItem.itemId, dojo.trim(summary), dojo.trim(description));
				
			} catch(e) {
				// This is a fall back error path
				// kwArgs.itemId must not be null, but it could be null 
				if (!currentWorkItemEditor.isNewWorkItem()) {
					query = QueryClient.createSimilarityQuery(name, kwArgs.itemId);
				} else {
					this.handleError(/* errorObj */ null, this.messages.errorRunningPotentialDuplicatesQueryCouldNotConstruct);
				}
			}
		}

		if(kwArgs.projectAreaItemId) query.projectAreaItemId = kwArgs.projectAreaItemId;
		if (kwArgs.preserveFilter == "true") {kwArgs.preserveFilter = true}

		this._internalRunQuery(query, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, false, kwArgs.preserveFilter);
	},
	
	// temporary fix for #35544
	runQuery: function(query, columnArgs) {
		this._lastQueryRun= query;
		this._lastColumnArgs = columnArgs;
		this._internalRunQuery(query, undefined, this._lastQueryOrderBy, this._lastQueryDirection, undefined, true, undefined, this._lastFilterAttribute, this._lastFilterValue, columnArgs);
	},
	refreshLastQuery: function(query) {
		if(this._workItemPageView) {
			var queryEditor = this._workItemPageView._getQueryBuilderWidget();
			if(queryEditor && queryEditor._query && queryEditor._query.clientItemId == query.clientItemId) {
				queryEditor.runQuery();
			} else {
				this.runQuery(query, this._lastColumnArgs);				
			}
		} else {
			this.runQuery(query, this._lastColumnArgs);
		}
	},
	/*
	 * Runs the query specified by the parameter and displays the results in a
	 * table.  This method must only be called by code within the
	 * com.ibm.team.workitem.web bundle.
	 */
  	_internalRunQuery: function(query, index, orderByAttrib, orderDirection, itemsPerPage, keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs) {
		if (!this._currentOperation) {
			if(query.name) {
				var message = dojo.string.substitute(this.messages.operationRunningQueryNamed, [ query.name ]);
			} else {
				var message = this.messages.operationRunningQuery;
			}
			this._currentOperation= this.createOperation(message, query.itemId);
			this.startOperation(this._currentOperation);
		}
		this._lastQueryName = query.name ? query.name : this.messages.labelDefaultQueryName;
		this._lastQueryUrl = window.location.href;
		this._setQueryPageTitle(this._lastQueryName, query.id);
		// If the last query url doesn't have an "action" in it, then it was the default query and we need to pick it up differently
		if(this._lastQueryUrl.indexOf("action=") == -1) {
			this._lastQueryUrl = WorkItemUtil.getQueryUri(query)
		}
		
		if (columnArgs) {
			var sortOrder= columnArgs.sortOrder;
			if (sortOrder.length > 0) {
				orderByAttrib= sortOrder[0].id;
				orderDirection= sortOrder[0].ascending ? "ascending" : "descending";
			}
		} else if (com.ibm.team.workitem.web.client.util.isStringDefined(orderByAttrib) == false) {
			// Get the sort order defaults and then run the query
			// Note: This is sub-optimal performance-wise:
			// (1) The editable query DTO should be cached
			// (2) The REST service call should be specialized only to return 
			//     sorting specific information 
			this._getSortOrderDefaults(query, index, itemsPerPage, keepEditor, preserveFilter, 
					filterAttribute, filterValue, columnArgs);
			return;				
		}
		
		// Run query
		this._internalGetResultsSet(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs);
  	},
  	
	/*
 	 * Private Method
 	 * 
 	 * @param String query
 	 * @param Number index
 	 * @param String orderByAttrib
 	 * @param String orderDirection
 	 * @param String itemsPerPage
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_internalGetResultsSet: function(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs, sortOrder) {

   		var self= this;
		var queryUrl = window.location.href; 
		var handlerObj= {
			success: function(queryResultsSummaryPage) {
				// _querySuccessCallback will reset the last filter so keep it here
				var filterValue = self._lastFilter;
				self._querySuccessCallback(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor, queryUrl);
				if (!preserveFilter || !filterValue || filterValue == "") {
					self._workItemPageView.resetFilter();
				} else {
					self._workItemPageView.setFilter(filterValue)
				}
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [self]);
				self._isReloading = false;
			},
			error: function(error) { 
				self._queryFailureCallback(error);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				self._isReloading = false;
			}
		};

		var handler= new ServiceResponseHandler(handlerObj, "success", "error");

		var isAscending= orderDirection === "ascending";

		var args= new Object();
		args.startIndex= index ? index - 1 : 0;
		if (itemsPerPage) {
			args.maxResults= itemsPerPage == "all" ? WorkItemUtil._getMaximumQueryPageSize() : itemsPerPage;
		}
		else {
			args.maxResults= WorkItemUtil._getDefaultQueryPageSize();
		}
		args.filterAttribute= filterAttribute;
		args.filterValue= filterValue;
		if (columnArgs) {
				args.sortOrder= columnArgs.sortOrder;
				args.columnIdentifiers= columnArgs.columnIdentifiers;
		} else {
			if (sortOrder) {
				args.sortOrder = sortOrder;
			} else {
				if (orderByAttrib) {
					args.sortOrder= [ { id: orderByAttrib, ascending: isAscending ? true : false }];
				}
			}
		}
		
		QueryClient.getResultSet(query, args, handler);		
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param String query
 	 * @param Number index
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_getSortOrderDefaults: function(query, index, itemsPerPage, keepEditor, 
			preserveFilter, filterAttribute, filterValue, columnArgs) {
		// Create a reference to this object to use in the service object
		var self = this;
		// Create the service object
		var serviceObject = {
			// Service success
			success: function(dto) {
				self._processSortOrderDefaults(dto, query, index, itemsPerPage, keepEditor,
						preserveFilter, filterAttribute, filterValue, columnArgs);
			},
			// Service failure
			failure: function(errorObject) {
				self.handleError(errorObject, self.messages.errorFetchingSortOrderDefaults);
			}
		};
		// Create the handler
		var handler = new ServiceResponseHandler(serviceObject, "success", "failure");
		// Invoke the service
		QueryClient.getEditableQuery(handler, query.itemId);
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Object dto
 	 * @param String query
 	 * @param Number index
 	 * @param boolean keepEditor
 	 * @param boolean preserveFilter
 	 * @param String filterAttribute
 	 * @param String filterValue
 	 * @param Object columnArgs
 	 * @return void
 	 */	
	_processSortOrderDefaults: function(dto, query, index, itemsPerPage, keepEditor, 
			preserveFilter, filterAttribute, filterValue, columnArgs) {

		var orderByAttrib = null;
		var orderDirection = null;
		var id = null;
		var ascending = false;
		// Get sort order attributes if any are defined
		try {
			if(dto && dto.sorting && dto.sorting[0]) {				
				var id = dto.sorting[0].id;
				var ascending = dto.sorting[0].ascending;
			}
		} catch(errorObject) {
			// Fail silently and fallback on defaults
			// this.handleError(errorObject, "[Error] Processing DTO for sort order defaults failed");
			// Continue processing
		}
		// Set sort order attribute
		if (com.ibm.team.workitem.web.client.util.isStringDefined(id)) {
			orderByAttrib = id;
		} else {
			orderByAttrib = "modified";
		}
		// Set sort order direction
		if (ascending) {
			orderDirection = "ascending";
		} else {
			orderDirection = "descending";
		}
		
		var sorting = null;
		if(dto && dto.sorting) sorting = dto.sorting;
		// Run query
		this._internalGetResultsSet(query, index, orderByAttrib, orderDirection, itemsPerPage,
			keepEditor, preserveFilter, filterAttribute, filterValue, columnArgs, sorting);
	},		
  	/*
  	 * Called upon a successful query execution.  Displays the results in a
  	 * table.
  	 */
  	_querySuccessCallback: function(queryResultsSummaryPage, query, orderByAttrib, orderDirection, itemsPerPage, keepEditor, queryUrl) {
  		checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
  		checkNotNull(query, "query");
		// Fix to convert strings true/false to booleans
		var idRow = -1;
		for(var i=0;i<queryResultsSummaryPage.headers.length; i++) {
				var header = queryResultsSummaryPage.headers[i];
				if(header.iconOnly) {
					header.iconOnly = header.iconOnly == "true" || header.iconOnly === true ? true : false; 
				};
				if(header.isOrderable) {
					header.isOrderable = header.isOrderable == "true" || header.isOrderable === true ? true : false; 
				}
		}
		
		this._lastQueryName = query.name ? query.name : this.messages.labelDefaultQueryName;
		this._lastQueryItemId = query.itemId ? query.itemId : null;
		this._lastQueryItemId = this._lastQueryItemId || query.clientItemId;
		this._workItemPageView.showQueryResults(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, keepEditor, queryUrl);

  		if (this._currentOperation) {
			this.finishOperation(this._currentOperation);
			this._currentOperation= null;
		}
  	},
	_showQueryBuilderOrCancel: function() {
		var uri = this._workItemPageView._getQueryBuilderWidget().getQueryEditorUri();
		if(uri) {
			if(uri == window.location.href) {
				this._workItemPageView.showQueryInEditor(this._lastEditedQuery, /* keep */ true);
			} else {			
				this.setWindowLocationHref(uri); // show the query builder
			}
		} else {
			window.history.back(); // undo the navigation that caused the question
		}
	},
	_queryClearEditor: function(queryEditor) {
		var queryProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + queryEditor._query.itemId);
		if(!queryProxy) return; // query builder destroyed
		if(queryProxy.isNew()) {
			com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + queryEditor._query.itemId);
		} else {
			/* 
			 * If the unchanged query is the same, replace the old to undo changes
			 * else just leave the unchanged item in the cache
			 */
			if(queryEditor._query.itemId == queryEditor._unchangedQuery.itemId) {			
	 			this._replaceQuery({oldQuery: queryEditor._query, newQuery: queryEditor._unchangedQuery});
			} 
			this._workItemPageView.resetQueryEditor();
			this._lastEditedQuery = null;
			this._lastQueryKey = null;
		}
		dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
	},
	/*
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Shows the query editor for a new query.
	 */
	newQuery: function(kwArgs) {
		if(!kwArgs.tq) {
			com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation2(WorkItemUtil.getNewQueryUri(true));
			return; // will come back unique
		}
		/* If the user is asking for a new query and there already is an unchanged new query,
		 * show the current "new query" instead of asking if they want to abandon the unchanged new query
		 */
		var queryBuilderWidget = this._workItemPageView._getQueryBuilderWidget();
		if(queryBuilderWidget) {
			var builderQuery = queryBuilderWidget._query;
			if(builderQuery) {				
				if(builderQuery._uriTimestamp == kwArgs.tq && builderQuery.isNew && !queryBuilderWidget._isDirty) {
					this._showQueryBuilderOrCancel();
					return;
				}
				if(!builderQuery._isCancelled && !this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
					    this._showQueryBuilderOrCancel();
						return;
				}
			}
		}
		 
		if(this._lastNewQueryTimestamp != kwArgs.tq) {
			
		}
		this._lastNewQueryTimestamp = kwArgs.tq;
		this._isSubsequentView = true;
		var queryKey = "New Query:" + kwArgs.tq;
		this._lastEditedQuery = null;		
		
			// if we're already caching the query editor editor, and the timestamp is the same, simply show the cached page
			if (queryKey == this._lastQueryKey
					&& this._lastQueryIndex == kwArgs.index
					&& this._lastQueryOrderBy == kwArgs.orderBy
					&& this._lastQueryDirection == kwArgs.direction
					&& this._lastQueryItemsPerPage == kwArgs.itemsPerPage) {
				this._setQueryPageTitle(this._lastQueryName, true);
				this._workItemPageView.showLastQueryResults();
				return;
			} 
		
		this._lastQueryItemId = kwArgs.id;
		this._lastQueryIndex = kwArgs.index;
		this._lastQueryOrderBy = kwArgs.orderBy;
		this._lastQueryDirection = kwArgs.direction;
		this._lastQueryItemsPerPage = kwArgs.itemsPerPage;

	// If its the same query key, but different other factors, rerun the query
		if(queryKey == this._lastQueryKey) {
			this._internalRunQuery(this._lastQueryRun, kwArgs.index, kwArgs.orderBy, kwArgs.direction, kwArgs.itemsPerPage, true,  undefined, undefined, undefined, this._lastColumnArgs);
			return;
		}

		this._lastQueryKey = queryKey;
		this._lastColumnArgs = null;
		
		// Otherwise show a fresh query
		this.editQuery({ id: "", tq: kwArgs.tq, _isConfirmed: true});
	},
	
	/*
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Loads the query specified by 'kwArgs.id' in the query editor.
	 */
	editQuery: function(kwArgs) {
		this._isSubsequentView = true;
		checkNotNull(kwArgs, "kwArgs");
		checkNotNull(kwArgs, "kwArgs.id");
		if(!kwArgs._isConfirmed && !this._workItemPageView.confirmLeaveUnsavedQueryChanges(kwArgs)) {
			    this._showQueryBuilderOrCancel();
				return;
		}		
		if(kwArgs.id == "") {
			this._workItemPageView.resetQueryEditor();
		}
		
		var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + kwArgs.id);
		if(proxy) {
			proxy.setIsBeingEdited(true);
		}

		if(kwArgs.id == this._lastQueryId && this._lastEditedQuery) {
			this._workItemPageView.showQueryInEditor(this._lastEditedQuery, /* keep */ true);
			dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
			return;
		}
		this._lastQueryId = kwArgs.id;
		this._lastQueryUrl = window.location.href;
		
		/*
		 * If this was a "Save As Copy" then the query editor is showing this query already
		 */
		if(this._workItemPageView.isEditingQuery(kwArgs.id)) {
			this._workItemPageView.showQueryInEditor(this._lastEditedQuery, /* keep */ true);
			dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
			return;			
		}
		 
  		var self= this;
		var operation= this.createOperation(self.messages.operationOpeningQuery, "editQuery");
		this.startOperation(operation);
		
		var serviceObject = {
			success: function(query) {
				if (!query) {
					self._handleQueryNotFound();
					return;
				}
				if(kwArgs.id == "") {
					query.isNew = true;
				}
				var keepLastResults = false;
				self._lastEditedQuery = query;
				self._lastQueryName = query.name || self.messages.labelDefaultQueryName;
				self._lastQueryItemId = query.itemId;
				self.finishOperation(operation);
				self._setQueryPageTitle(query.name, /* isEditing */ true);
				self._workItemPageView.showQueryInEditor(query, keepLastResults, kwArgs);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
				dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
			},
			error: function(error) { 
				self.finishOperation(operation);
				self.handleError(error, self.messages.errorEditingQuery);
				dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
			}
		};
		QueryClient.getEditableQuery(new ServiceResponseHandler(serviceObject, "success", "error"), kwArgs.id);
	},
	
	_handleQueryNotFound: function() {
		this._showDefaultView();
		if(PlatformUI && PlatformUI.getWorkbench()) {
			PlatformUI.getWorkbench().showMessage(this.messages.msgQueryNotFound);
		}
	},
	  	
  	/*
  	 * Called if an error occurs while executing a query.
  	 */
  	_queryFailureCallback: function(error) {
		this.handleError(error, this.messages.errorRunningQuery);
		if (this._currentOperation) {
			this.finishOperation(this._currentOperation);
		}
  	},

	/*
	 * VIEW WORK ITEM-related methods.
	 */
	 
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Opens a work item editor for a new work item.
	 */
	newWorkItem: function(kwArgs) {
		this._isSubsequentView = true;
		this._setPageTitle(this.messages.pageTitleNewWorkItem);
		
		if(kwArgs.id) {
			kwArgs.ts = kwArgs.id;
		}
		if( kwArgs.ts == null) {
			var href = WorkItemUtil.reuniquifyUrlWithTimestamp(window.location.href);
			this._updateUrl(href.url);
			return;
		} 

		kwArgs.ts = Math.abs(kwArgs.ts);
		kwArgs.ts = kwArgs.ts * -1;
		// if we're already displaying a new work item editor, simply update the
		// currently displayed tab
		if (this._workItemPageView.isWorkItemEditorAvailable(kwArgs.ts) && !kwArgs.doReset) {
			this._workItemPageView.showEditorTab(kwArgs.ts, kwArgs.tab, kwArgs.doSave);
			return;
		}

		// if the page is loaded directly, wait until there is a default type (page fully loaded)
		if(this.defaultType) {
			this._newWorkItemPart2(kwArgs);	
		} else {
			var self=this;
			var then = new Date().getTime(); 
			var waitFunc = function() {
				if(self.defaultType != null) {
					self._newWorkItemPart2(kwArgs, interval);
				}
				var now = new Date().getTime();
				if((now - then) > 30000) { // time out after 30 seconds
					self._newWorkItemPart2(kwArgs, interval);
				}
			}
			var interval = setInterval(waitFunc, 200);
		}
	},
	_newWorkItemPart2: function(kwArgs, interval)	{
	
		if(interval) {
			clearInterval(interval);
		}
        // If the default type does not exist, go to the error page.
		if(kwArgs.type == null && this.defaultType == "") {
			this.handleError({message: this.messages.errorNoWorkItemTypes}, null);
			return;
		}
		
		// TODO: Remove this remap after url switching is fixed. defect 77524
		var welcomePage = com.ibm.team.workitem.web.cache.internal.Cache._workItemWelcome;
		if(welcomePage) {
		    // The following needs to be in a setTimeout to prevent the new item being processed twice
			setTimeout(dojo.hitch(welcomePage,welcomePage.makeUrlsUnique),200);
		}
		
		// otherwise fetch a template new work item object and display it
		var type = kwArgs.type || this.defaultType;
		var tempId =  kwArgs.ts || (new Date().getTime());
		this._showNewWorkItem(type, /* tempId */ kwArgs.ts, kwArgs.tab, kwArgs.draftId || null, kwArgs);
		
		},
	login: function() {
		this.setWindowLocationHref("https://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.https + window.location.pathname + window.location.search + window.location.hash);
	},
	isGuest: function() {
		return com.ibm.team.repository.web.client.session.isGuest();
	},
	checkLogin: function() {
		if (com.ibm.team.repository.web.client.session.isGuest()) {
			if (confirm(this.messages.promptLogInOrCancelWorkItemEdit)) {
				this.login(); /* Will not return */
			}
			return false;
		}
		return true;
	},
	_showNewWorkItem: function(type, tempId, tab, draftId, kwArgs) {
		if(!this.checkLogin()) {
			 this._showDefaultView();
			 return; // not reached
		}

		this._workItemPageView.showNewWorkItemEditor(type, tempId, tab, draftId, kwArgs);

	},
	_findProperty: function(props, attributeName) {
		if(!props) return [];
		for(var i=0;i<props.allValues.length; i++) {
			if(props.allValues[i].attributeName == attributeName) {
				return props.allValues[i].uiItems;
			}
		}
		return [];
	},
	_fetchWorkItemTypes: function() {
		var self=this;
		var serviceObj= {
			self: this,
			success: function(editProps) {
				this.self._handleTypesLoaded(editProps);
			},
			failure: function(errorObj) {
				// This path occurs when there are not types.
				this.self._handleTypesLoaded(null);
				// TODO: Handle the error
//				this.self.handleError(errorObj, self.messages.errorFetchingWorkItemProperties);
			}
		};
		var srh= new ServiceResponseHandler(serviceObj, "success", "failure");

		var appArgs = {newWorkItem: true, ids: ["workItemType"]};
		WorkItemClient.getAllValues(srh, appArgs);
	},
	_handleTypesLoaded: function(editProps) {
		var options = new Array();
		if(editProps != null) {
			options = editProps[0].uiItems;
			if(!options) {
				this._publishPageLoadedEvent();
				return;
			}		
			
		}
		
		if(options.length == 0) {
			this.defaultType = "";
		} else {
			this.defaultType = options[0].id;
		}
		
		this._publishPageLoadedEvent();
		
	},
	_publishPageLoadedEvent: function() {
		dojo.publish(EventTopics.WORKITEM_PAGE_LOADED, [ this ]);
	},
	_openNewWorkItem: function(typeId) {
		var url =  "#action=com.ibm.team.workitem.newWorkItem";
		if(typeId) {			
			var url = url + "&type=" + encodeURIComponent(typeId);
		}
		var ts = Date.parse(new  Date().toString());
		url = url + "&ts=" + ts + "0"
		this.setWindowLocationHref(url);
	},
	_openNewQuery: function(typeId) {
		var url =  WorkItemUtil.getNewQueryUri();
		this.setWindowLocationHref(url);
	},
	_createFromTemplate: function() {
		var action = new com.ibm.team.workitem.web.ui2.internal.action.CreateFromTemplateAction();
		action.run({ controller: this });
	},
 	/*
 	 * Take some attributes passed in workitem.attributes[theAttribute:value] and store them as workitem.theAttribute
 	 * Rule of thumb: Promote well known immutable attributes 
 	 */
	_promoteAttributes: function(workItem) {
		if(!workItem) return;
		workItem.attributes = workItem.attributes || {};
    	if(workItem.attributes.id && workItem.attributes.id.label ) {
			workItem.id = workItem.attributes.id.label.replace(/\,/g,"");
		} else {
		    workItem.id = -1;  // will act like a new work item
		}
	},
 
	/*
	 * Sets the browser's URI so that the Jazz framework calls the
	 * 'viewWorkItem' method.  May be called by other code within the
	 * com.ibm.team.workitem.web bundle.
	 *
	 * @param id work item id to open.
	 * @param tab (optional) indicates which tab ("Overview", etc.) the work
	 *  item editor should display initially.
	 */
	_internalViewWorkItem: function(id, tab, isSearch) {
		checkNotNull(id, "id");
		var numId= parseInt(id, 10);
		if (!numId || numId <= 0) {
			this.handleError({message: this.messages.errorOpeningWorkItem}, dojo.string.substitute(this.messages.errorOpeningWorkItemMessage, [ ("" + id)]));
		}
		this.setWindowLocationHref(WorkItemUtil.getViewWorkItemUri(id, tab), isSearch);
	},

	/**
	 * Remove the doSave parameter from the location because the save was cancelled
	 */
	_clearSaveParameter: function() {
		var loc = window.location.href.replace(/&doSave=\([0-9]+\)/, "");
		this.setWindowLocationHref(loc);
	} ,
	// @deprecated, use Cache directly
	getWorkingCopy: function(id, incrementUse){
	    console.debug("pageController.getWorkingCopy is deprecated. Use Cache instead.");
		var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
		if (!workingCopy) {
				return null;
		}
		if(incrementUse) workingCopy.incrementUseCount();
		return workingCopy;
	},
	/**
	 * PUBLISHED URI: DO NOT RENAME THIS METHOD.
	 *
	 * Opens the work item specified by 'kwArgs.id'.
	 *
	 * @param kwArgs map of arguments set by Jazz framework based on URI request
	 *  params.
	 * @param isRefresh (optional) specifies that the work item should be
	 *  reloaded even if it's currently open.
	 */
	viewWorkItem: function(kwArgs, isRefresh) {
		this._isSubsequentView = true;
		checkNotNull(kwArgs, "kwArgs");
		
		if(kwArgs.id == null && kwArgs.itemId == null) {
			console.error("[WorkItemPageController] Missing id or itemId in viewWorkItem url.");		
			checkNotNull(kwArgs.id, "kwArgs.id");
		}

		if(kwArgs.id == null && kwArgs.itemId != null) {
			this._viewWorkItemByItemId(kwArgs);
			return;
		}
		
		var id= parseInt(kwArgs.id, 10);
		if (!id || id <= 0) {
			this.handleError({message: this.messages.errorOpeningWorkItem}, dojo.string.substitute(this.messages.errorOpeningWorkItemMessage, [ (""+ kwArgs.id)]));
			return;
		}
		
		// if the work item editor is already open for the specified id, and
		// we're not doing an explicit refresh, then simply change the tab
		if (this._workItemPageView.isWorkItemEditorAvailable(id) && !isRefresh) {
			this._workItemPageView.showEditorTab(id, kwArgs.tab, kwArgs.doSave);
			return;
		}
		
		if(!kwArgs.doNotShowLoading && window.location.href.indexOf("showLoadingPage=false") == -1) {
			this._workItemPageView.showWelcomePage(dojo.string.substitute(this.messages.busyLoadingWorkItem, [kwArgs.id + ""]), true);
		} 
		this._workItemPageView.showWorkItemEditor(kwArgs.id, kwArgs.tab, isRefresh, kwArgs.doSave);
		
	},
	_locationUriPrefix: "itemName/com.ibm.team.workitem.WorkItem",
	_viewWorkItemByItemId: function(kwArgs) {
		var workingCopies = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({filterAttributes: [ "isWorkItem" ]});
		for(var i=0;i<workingCopies.length;i++) {
			if(workingCopies[i].getItemId() == kwArgs.itemId) {
				kwArgs.id = workingCopies[i].getId();
				this.viewWorkItem(kwArgs, false);
				return; 
			}
		}
			var self = this;
			var handlerObj = {
				success: function(workItem){
					//"https://10.0.1.2:9443/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/10"
					var i = workItem.locationUri.indexOf(self._locationUriPrefix);
					if(i == -1) {
						self.handleError({message:dojo.string.substitute(self.messages.errorOpeningWorkItemWithMessage, [ kwArgs.itemId ])} , dojo.string.substitute(self.messages.errorOpeningWorkItemWithMessage, [ kwArgs.itemId ]));
						return;
					}
					var id = workItem.locationUri.substring(i+self._locationUriPrefix.length + 1, workItem.locationUri.length);
					kwArgs.id = id;
					self.viewWorkItem(kwArgs, false);
				},
				error: function(error){
					self.handleError(error, dojo.string.substitute(self.messages.errorOpeningWorkItemWithMessage, [ kwArgs.itemId ]));
				}
			};
			
			var responseHandler = new ServiceResponseHandler(handlerObj, "success", "error");
			
			var args = {
				itemId: kwArgs.itemId,
				includeAttributes: false,
				includeLinks: false,
				includeApprovals: false,
				includeHistory: false
			};
			
			WorkItemClient.getWorkItem(responseHandler, args);
	},
	removeEditorTab: function(id, isCancel, workingCopy) {
		this._workItemPageView.removeEditorTab(id);
	},
	setIsDirty: function(workItemEditor, isDirty) {
		this._workItemPageView.setIsDirty(workItemEditor, isDirty);
	},
	getLastQueryInfo: function() {
		var currentWidgetId =this._workItemPageView.getCurrentWidgetId();
		var currentWorkItem =this._workItemPageView.getCurrentWorkItem();
		var isQuery = currentWidgetId ? currentWidgetId.indexOf("Query") != -1 : false; 
		var isWorkItem = !isQuery; 
		return { name: this._lastQueryName, itemId: this._lastQueryItemId, url: this._lastQueryUrl, currentWidgetId: currentWidgetId, isQuery: isQuery, isWorkItem: isWorkItem, currentWorkItem: currentWorkItem };
	},
	isInResults: function(workItemId) {
		if(this._workItemPageView.queryResultsWidget) { 
			var results = this._workItemPageView.queryResultsWidget.getResults();
		}
//		console.debug(results);
		if (!results)
			return false;
		for (var i = 0; i < results.length; i++) {
			if (results[i] && results[i].indices && results[i].indices[0] == workItemId)
				return true;
		}
		return false;
	},
	/*
	 * Return the currently displayed work item
	 */
	getCurrentWorkItem: function() {
		try{
			return this._workItemPageView._multipaneContentWidget.currentWidget._workItem;
		} catch(e) {
			return null;
		}
	},
	getCachedWorkItemEditor: function(id) {
		try{
			return this._workItemPageView._multipaneContentWidget.getCachedWidget("__jazzWorkItemEditor" , id, /* simple */ true);
		} catch(e) {
			return null;
		}
	},
	
	removeEditor: function(id, /* show next tab */ showNext, removeIfZero, workingCopy) {
		this._workItemPageView.removeEditorTab(id, showNext );
		if(removeIfZero && workingCopy.getUseCount() ==0) {
			workingCopy.destroy();
		}
		dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_REFRESH, [this]);
	},
	
	/*
	 * Get the list of "Recent work Items"
	 */
	getRecentWorkItemList: function() {
		var args= {};
		args.filterAttributes= ["isWorkItem"]; //$NON-NLS-1$
		var items= com.ibm.team.workitem.web.cache.internal.Cache.getAllItems(args);
		return items;
	},
	_initializeComponentMenus: function(componentMenu) {
		componentMenu.refreshMenu(this);
	}
	

});

})();
