/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.app.context");
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcome");      // Still used for error messages
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcomePage2"); // The new welcome page
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemShowQueriesPage");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.CompositeContentArea");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.QueryResults");
dojo.require("com.ibm.team.workitem.web.policy.internal.PolicyFactory"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.page.Landmark"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext");
dojo.require("jazz.app.context"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;

var PlatformUI;
var ActionRegistry		= null; // initialize later

var WorkItemUtil		= com.ibm.team.workitem.web.client.util;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

var WELCOME_PAGE_ID = "__jazzWorkItemWelcome";
var WELCOME_PAGE_ID2 = "__jazzWorkItemWelcome2";
var ERROR_PAGE_ID = "__jazzWorkItemError";
var WORK_ITEM_EDITOR_ID = "__jazzWorkItemEditor";
var QUERY_LIST_ID    = "__jazzQueryList";
var QUERY_RESULTS_ID    = "__jazzQueryResults";
var QUERY_BUILDER_ID    = "__jazzQueryBuilder";
var QUERY_COMPOSITE_ID  = "__jazzQueryComposite";

var SCOPE_ALL = "all";
var SCOPE_UNRESOLVED = "unresolved";
var MAX_SIZE = 11;

var SearchBox = jazz.ui.SearchBox;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var EditorUtil              = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/*
 * The main view for the work item web UI. Responsible for displaying left-nav
 * content and for managing the main content area.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView",
	[dijit._Widget, dijit._Templated],
	{
		/*
		 * Called by Dojo during widget creation lifecycle.
		 */
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.queryMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		},	
		postCreate: function() {
			PlatformUI = net.jazz.ajax.ui.PlatformUI;
			this.childWidgets = [];
		 	this.controller = new com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController(this);
			
			var landmark = new com.ibm.team.workitem.web.ui.internal.page.Landmark({parentWidget: this});
			this._landmarkNode.appendChild(landmark.domNode);
			
			// Initialize the multipane content widget
			var parms= {parentController: this.controller};
			this._multipaneContentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView(parms);
            this._addChild(this._multipaneContentWidget);
			                           			
            this._initializeHashChangeCallbacks();
/*
 * Initialize Work Item policies
 *  - policies are long-lasting singletons that subscribe to events on creation
 */
 			this._policies = com.ibm.team.workitem.web.policy.internal.PolicyFactory.initializePolicies();

			this._initializeNavigation();
		
//			this._bodyClickHandle= com.ibm.team.workitem.web.ui.internal.module.WindowContext.addBodyClickRedirectionListener(this);
		},
		_initializeHashChangeCallbacks: function() {
			if(!PlatformUI || !PlatformUI.getWorkbench()) return;
			//  initialize the ActionRegistry alias
			ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			
			// register actions
			ActionRegistry.registerAction("com.ibm.team.workitem._showDefaultView", this.controller, "_showDefaultView");

			var validActions = this.controller.validActions;
			
			var self = this;
			dojo.forEach(validActions,
				function(x) {
					ActionRegistry.registerAction("com.ibm.team.workitem." + x, self.controller, x); 
				}
			);

		},
		destroy: function() {
			if(this._changeProjectAreaHandle) dojo.unsubscribe(this._changeProjectAreaHandle);
			if(this._unloadHandle) dojo.disconnect(this._unloadHandle);
			if(this.controller && this.controller.destroyController)	this.controller.destroyController();

			dojo.forEach(this.childWidgets, function(child) {
				try {
					if(child.destroyWidget) {
						child.destroyWidget();
					} else {
						if(child.destroy) child.destroy();
					}
				} catch(e) {
					console.debug('Error destroying: ' + child);// ignore
				}
			});
			
			this._destroyQueryResultWidgets();					
 			this.inherited(arguments);
		},
		_destroyQueryResultWidgets: function() {
			if(this._queryResultPageControl) {
	//95375			this._queryResultPageControl.destroy();
	//95375			delete this._queryResultPageControl;
			}

			if(this._queryResultsWidget) {
				this.queryResultsWidget.destroy();
				this.queryResultWidget = null;
			}
			if(this._queryBottomPageControl) {
	//95375			this._queryBottomPageControl.destroy();
	//95375			delete this._queryBottomPageControl
			}
			if(this._workItemEditorBanner) {this._workItemEditorBanner.destroy() }
		},
		_addChild: function(childWidget) {
			if(!childWidget) {
				console.debug("Error adding null child widget in Work Item Page View");
			} else {
				this.childWidgets.push(childWidget);
			}
			
		},
		_initializeNavigation: function(navigationView) {
			
			this.controller._fetchWorkItemTypes();
						
			var queryArgs= {parentController: this.controller};
			
			this._registerNavigation();

			this._unloadHandle = dojo.connect(window, "onunload", dojo.hitch(this, this.unload)); // use dojo.connect here
			
			// Ping all the working copies to add them  to recently viewed
			var workingCopies = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({filterAttributes: [ "isWorkItem" ]});
			for(var i=0;i<workingCopies.length;i++) {
				if(dojo.isFunction(workingCopies[i].notifyOfItemChanged)) {
					workingCopies[i].notifyOfItemChanged();
				}	
			}
			
			if(this.isGadget) {
				var currentProjectArea = jazz.app.context.get();
			 	if(currentProjectArea && currentProjectArea.name) {
			 		var segment1 = {
								href: null,
								label: this.messages.labelChangeAndConfigurationManagement
							}
			 		var segment2 = {
							href: null,
							label: currentProjectArea.name
						}
			 		dojo.publish(EventTopics.WORKITEM_ACTION_SET_HEADER_BREADCRUMBS, [[ segment1, segment2 ]]);
			 	}
			}
			
		},
	getCurrentProjectArea: function() {
		return jazz.app.context.get();
	},
		unload: function() {
	//		this.destroy();  // DO NOT DESTROY - causes browser crashes
		},
		_registerNavigation: function() {
			if(!PlatformUI || !PlatformUI.getWorkbench()) return;
	//		if(WorkItemUtil.isDebugMode()) return; // too many clicks
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
		},
		changesPending: function() {
			if(com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({filterSelectors: [ "isChanged" ], filterAttributes: [ "isWorkItem"] }).length > 0) return true;
			if(this.queryBuilderWidget && this._getQueryBuilderWidget() && this._getQueryBuilderWidget().hasUnsavedChanges()) return true;
			return false;
		},
		/*
		 * This message gets called when navigating away from the web ui, e.g., to www.jazz.net
		 */
		_handleNavigateAway: function(e) {
			if(!this.changesPending()) return;
			var message = this.messages.promptUnsavedWorkItems + "\n" + this._getChangedItemList() + "\n\n";
			return message;
		},
		
		// TODO: Promote to AbtractFramedWidget
		_registerFramedNavigation: function() {
			var onBeforeUnloadHandler= dojo.hitch(this, function(unloadEvent) {
				var result= this._handleNavigateAway(); // The string message
				if (result != null){
					if (unloadEvent != null){
						unloadEvent.returnValue= result;
					}
					else {
						window.event.returnValue = result;
					}
				}
				if (result === null && dojo.isIE)
					return undefined; // must be undefined, not null
				return result;
			});
			window.onbeforeunload = onBeforeUnloadHandler;
		},

		_getChangedItemList: function() {
			var unsavedItems = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems({filterSelectors: [ "isChanged" ], filterAttributes: [ "isWorkItem"] });
			var message = this.messages.labelItemsChanged + "\n";
			
			for(var i=0;i<unsavedItems.length;i++){
				message += "\n  ";
				var summaryLabel = unsavedItems[i].getSummaryLabel();
				if(summaryLabel.length ==0) {
					if(!unsavedItems[i].isNew()) {
						summaryLabel = this.messages.labelNoSummary;
					}
				}

				if(unsavedItems[i].isNew()) {
					message += dojo.string.substitute(this.messages.labelNewWorkItem, [ summaryLabel]);
				} else {
					message += dojo.string.substitute(this.messages.labelNameSummary, [ unsavedItems[i].getLinkLabel(), summaryLabel]);
				}
			}
			if(this.queryBuilderWidget && this._getQueryBuilderWidget() && this._getQueryBuilderWidget().hasUnsavedChanges()) {			
				message += "\n  ";				
				message += dojo.string.substitute(this.messages.labelNameSummary, [this.messages.labelQuery, this._getQueryBuilderWidget().getName() || this.messages.labelDefaultQueryName]);
			} 
			return message;
		},
		//
		// Other work item code may call these properties and methods.
		//
		
		/*
		 * Returns the internal id of the widget currently displayed in the
		 * page's content area.
		 */
		getCurrentWidgetId: function() {
			return this._currentMainWidgetId;
		},
		
		/*
		 * Returns the work item object currently displayed in the page's
		 * content area.
		 */
		getCurrentWorkItem: function() {
			if(this._multipaneContentWidget && this._multipaneContentWidget.getCurrentWidget) {
				var widget = this._multipaneContentWidget.getCurrentWidget();
				if(widget && widget.workingCopy && widget.workingCopy.object) {
					return widget.workingCopy.object;
				}
			}
			return null;
		},
		/*
		 * Returns a loaded work item with the itemId, or null
		 */
		getRecentWorkItemEditor: function(itemId) {
			return this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, null, /* don't create if not */ true, itemId);
		},
		
		isWorkItemEditorAvailable: function(id) {
			return this._multipaneContentWidget.isWidgetAvailable(WORK_ITEM_EDITOR_ID, id)
		},
		
		/*
		 * Shows the query object specified by <code>query</code> in the query
		 * builder.
		 * 
		 * @param {Object} query the query object to display in the builder.
		 * @param {boolean} keepResults (optional) whether or not to keep the
		 *  current query results view intact.
		 */
		showQueryInEditor: function(query, keepResults, kwArgs) {
			var queryBuilder= this._getQueryBuilderWidget();
			if(query== null) {
				query = queryBuilder._query;
			}
			if (!keepResults) {
				this._getQueryCompositeWidget().clearContent(QUERY_RESULTS_ID);
			}
			this._setCompositeContent(QUERY_BUILDER_ID, queryBuilder.domNode, null);
			
			var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + query.itemId);
			if(proxy){
				proxy.setIsBeingEdited(true);
			} else {
				proxy =  new com.ibm.team.workitem.web.cache.internal.QueryProxy({queryDto: query});
				proxy.setValue({path: "attributes.isCurrent", attributeId: "isCurrent", value:true});
				proxy._isBeingEdited = true;
				com.ibm.team.workitem.web.cache.internal.Cache.addItem("workitems/queries/" + query.itemId, proxy);
			}
			
			if (!keepResults) {
				queryBuilder.showQuery(query, kwArgs);
			}
		},
		isEditingQuery: function(id) {
			var queryBuilder= this._getQueryBuilderWidget();
			if(!queryBuilder || !queryBuilder._query) return false;
			if(!this._multipaneContentWidget.isWidgetAvailable(QUERY_BUILDER_ID, id)) return false;
			return queryBuilder._query.itemId == id;
		},
		// a new query came in... clear the old name 
		resetQueryEditor: function() {
			this._getQueryBuilderWidget().clearName();
		},
		showQueriesPage: function(kwArgs) {
			if(this.showQueriesWidget == null) {
	 			this.showQueriesWidget = new com.ibm.team.workitem.web.ui.internal.page.WorkItemShowQueriesPage({parentController: this.controller});
				this._addChild(this.showQueriesWidget);
			}

			if(kwArgs.tab == null) {
				kwArgs.tab = this.showQueriesWidget.fLastTabId || "owned";
			}
			if(kwArgs.tab) {				
				this.showQueriesWidget.setQueryFilter(kwArgs.tab);
			}
			this.controller._setPageTitle(this.messages.pageTitleShowQueries);				
			this._setMainContent(QUERY_LIST_ID, this.showQueriesWidget, null, 0, /* updateNav */ false);
		},
		showWelcomePage: function(msg, isProgress, keepTitle) {
			if(this.welcomePageWidget == null) {				
				this.welcomePageWidget = new com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcome({});
				this._addChild(this.welcomePageWidget);
			}
			
			if(!keepTitle) {
				this.controller._setPageTitle(this.messages.pageTitleWorkItems);				
			}
			if (isProgress) {
				this.welcomePageWidget.setProgressMessage(msg);			
			}
			else {
				if (msg) {
					this.welcomePageWidget.setErrorMessage(msg);
				}
				else {
					this.welcomePageWidget.clearErrorMessage();
					this.showWelcomePage2(keepTitle);
					return;
				}
			}
			this._setMainContent(WELCOME_PAGE_ID, this.welcomePageWidget, null, 0, /* updateNav */ false, keepTitle || isProgress);
		},
		showWelcomePage2: function(keepTitle) {
			if(this.welcomePageWidget2 == null) {	
				this.welcomePageWidget2 = new com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcomePage2({parentWidget: this});
				this._addChild(this.welcomePageWidget2);
			}
	
			this._setMainContent(WELCOME_PAGE_ID2, this.welcomePageWidget2, null, 0, /* updateNav */ false, false);
		},
		showNewWorkItemEditor: function(type, tempId, tab, draftId, kwArgs) {
			var args = {
				isNew: true
			}
			if(type) args.type = type;
			if(tempId) args.tempId = tempId;
			if(tab) args.tab = tab;
			if(draftId) args.draftId = draftId;
			if(kwArgs.projectAreaItemId) args.projectAreaItemId = kwArgs.projectAreaItemId;
			if(kwArgs.addLinkType) args.addLinkType = kwArgs.addLinkType;
			if(kwArgs.addLinkIsSource) args.addLinkIsSource = kwArgs.addLinkIsSource;
			if(kwArgs.addLinkedItemId) {
				args.addLinkedItemId = kwArgs.addLinkedItemId;
				 var workItem = com.ibm.team.workitem.web.cache.internal.Cache.getItemByItemId(kwArgs.addLinkedItemId);
				 if(workItem) {
				 	workItem.setHasNewLinksPending(true);
				 }
			}
			if(kwArgs.addLinkedProjectAreaItemId) args.addLinkedProjectAreaItemId = kwArgs.addLinkedProjectAreaItemId;
			 
			this._showWorkItemEditor(args);
		},
		showWorkItemEditor: function(id, tab, isRefresh, doSave) {
			var args = {
				id: id,
				doSave: doSave || false
			}
			if(tab) args.tab = tab;
			if(isRefresh) args.isRefresh = isRefresh;
			this._showWorkItemEditor(args);
		},
		_showWorkItemEditor: function(parms) {
			var id = parms.id || parms.tempId;

			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
			if(!workItemEditorWidget) {
				workItemEditorWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor({isNew:true, parentController: this.controller});
			}
			
			parms.onSuccess= dojo.hitch(this,this._showWorkItemEditorComplete);
			parms.onError= dojo.hitch(this,this._showWorkItemEditorError)
			
		    workItemEditorWidget.initializeWorkItemEditor(parms);
		},
		_showWorkItemEditorError: function() {
	        dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
		},
		_showWorkItemEditorComplete: function(parms /* workItem, workItemEditor, [tempId] */ ) {
			
			dojo.publish(EventTopics.WORKITEM_ACTION_SET_BREADCRUMBS, [[ ]]);
			
			this.controller._setWorkItemPageTitle(parms.workItem);
			
			if(parms.isRefresh) {
				if(parms.workItem.attributes && parms.workItem.attributes[parms.workItemProxy.SYNTHETIC_COMMENT_ID]) {
					parms.workItemEditor.scrollTop = window.scrollY;
					parms.workItemEditor.scrollLeft = 0;
				} else {
					parms.workItemEditor.scrollTop = 0;
					parms.workItemEditor.scrollLeft = 0;
				}
			}

			var banner = this._getWorkItemEditorBanner(parms.workItem.id);
			this._setMainContent(
					WORK_ITEM_EDITOR_ID,
					parms.workItemEditor,
					 banner ? banner.domNode : null,
					parms.tempId || parms.workItem.id);
			

			if(parms.workItem.id < 1 && 	parms.workItemEditor._editorParts.header) {
				try {
					parms.workItemEditor._editorParts.header.summary.focus();
				} catch(e) {} // ignore... fails in FF due to timing
			}
			this.setIsDirty(parms.workItemEditor, parms.workItemEditor.workingCopy.isChanged());		
			
			/* if the url had an addLinkType parm set, add the link to the new work item */
			if(parms.addLinkType && parms.addLinkedProjectAreaItemId && parms.addLinkedItemId) {
				var workingCopy = parms.workItemEditor.workingCopy;
				workingCopy.addLinkTarget(parms);
			}
			// This is deferred until after the wi loads to speed up loading
			dojo.publish(EventTopics.INITIALIZE_QUERIES, []);

			if(parms.isRefresh) {
				/*
				 * When an editor is refreshed, check to make sure all other work items are in sych with the repository
				 */
				dojo.publish(EventTopics.WORKITEM_REFRESH_CACHE, [ [] ]); 
			}

			// The doSaveId shenanigans is to prevent the "Save" dialog from appearing after it is cancelled
			if (parms.doSave && parms.doSave != this._lastDoSaveId) {
				this._lastDoSaveId = parms.doSave;
				parms.workItemEditor.askToSave();
			}
		},
		/*
		 * For the work item identified by 'id', show the tab specified by
		 * 'tabName'. If the user clicked the 'Save' control on the left-nav,
		 * prompts user to confirm save.
		 * 
		 * @param {String} id the identifier for the work item editor for which
		 *  we'll show the specified tab.
		 * @param {String} tabName the identifier for the editor tab to show.
		 * @param {boolean} isDoingSave (optional) if 'true', confirms to save
		 *  the work item and saves on confirmation; defaults to 'false' if not
		 *  specified.
		 */
		_lastDoSaveId: 0,
		showEditorTab: function(id, tabName, doSaveId) {
			var isDoingSave = doSaveId != null;
			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
			this.controller._setWorkItemPageTitle(workItemEditorWidget._workItem);
			checkNotNull(workItemEditorWidget);
			
			var banner = this._getWorkItemEditorBanner(id);			
			workItemEditorWidget.showTab(tabName);
			this._currentMainWidgetId = WORK_ITEM_EDITOR_ID;
//  Remove extra show-content, which is done in "SetMainContent" 	
//		this._multipaneContentWidget.showContent(WORK_ITEM_EDITOR_ID, workItemEditorWidget, id);
			this._updateWorkItemEditorBanner(id);			
			this._setMainContent(
					WORK_ITEM_EDITOR_ID,
					workItemEditorWidget,
					banner ? banner.domNode : null,
					id,
					/* skipNavUpdate */ true // already done in showContent
					);
 			dojo.publish(EventTopics.WORKITEM_ACTION_SET_BREADCRUMBS , [[]]);
			// The doSaveId shenanigans is to prevent the "Save" dialog from appearing after it is cancelled
			if (isDoingSave && doSaveId != this._lastDoSaveId) {
				this._lastDoSaveId = doSaveId;
				workItemEditorWidget.askToSave();
			}
		},
		/*
		 * Open the prior view
		 * TODO: Open the prior view
		 */
		openPriorView: function() {
			this.controller._openDefaultView(); // TODO: Open the previous page instead of welcome page
		},
		/*
		 * 
		 */
		removeEditorTab: function(id, /* show next tab */ showNext) {
			this._multipaneContentWidget.removeCachedWidget(WORK_ITEM_EDITOR_ID, id);
			if(showNext) {
				// If we are not on the work items tab, swap in the Welcome page without switching to it
				if(window.location.href.indexOf("action=com.ibm.team.workitem") == -1 ) {
					this.showWelcomePage(/* message */ null, /* isProgress */ false, /* keep title */ true ); 
				} else {
					this.openPriorView();
				}
			}
			
		},
		
		
		/*
		 * For the specified work item editor widget, mark as dirty if 'isDirty'
		 * is 'true' or clean if 'isDirty' is false.
		 * 
		 * @param {Object} workItemEditorWidget the work item editor instance to
		 *  mark as clean or dirty.
		 * @param {boolean} isDirty if 'true', marks the specified editor as
		 *  dirty; if 'false', marks the specified editor as clean.
		 */
		setIsDirty: function(workItemEditorWidget, isDirty) {
			this._multipaneContentWidget.setIsDirty(WORK_ITEM_EDITOR_ID, workItemEditorWidget, workItemEditorWidget._workItem.id, isDirty);
		},
		
		//
		// No code outside this module should access these properties and
		// methods.
		//
		
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/page/templates/WorkItemPageView.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/page/templates/WorkItemPageView.css"),
		
		/*
		 * Required by Jazz web framework to make hyperlinking and back/forward
		 * work.
		 */
		getController: function() {
			return this.controller;
		},
		
		_getWorkItemEditorWidget: function(workItem, initialTab, url) {
			checkNotNull(workItem, "workItem");
			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, workItem.id);
			
			if (!workItemEditorWidget) {
				workItemEditorWidget=
					/* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor({parentController: this.controller, url: url, initialTab: initialTab});
//				this._addChild(workItemEditorWidget);  /* Not needed, destroyed by multipanecontentwidget */
			}

			return workItemEditorWidget;
		},
		setFilter: function(filterValue) {
			if(this._queryResultPageControl) { this._queryResultPageControl.setFilter(filterValue); }
			if(this._getQueryResultsWidget()) { this._getQueryResultsWidget().filter(filterValue); }
		},		
		showLastQueryResults: function() {
			this._setMainContent(QUERY_RESULTS_ID, this._getQueryCompositeWidget());
			var widget = this._getQueryResultsWidget();
			if(widget) { 
				widget.clearSelection(); 
				};
		},
		showQueryResults: function(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, keepEditor, queryUrl) {
			checkNotNull(query, "query");
			checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
			
			checkNonNegative(queryResultsSummaryPage.totalCount, "queryResultsSummaryPage.totalCount");
			checkNonNegative(queryResultsSummaryPage.startIndex, "queryResultsSummaryPage.startIndex");

			var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + query.itemId);
			if(proxy) {
				if(queryUrl && queryUrl.indexOf("editQuery") == -1) {
					proxy.setIsBeingEdited(false);
				} else {
					proxy.setIsBeingEdited(true);
				}
			}
			
			if (!keepEditor) {
				this._getQueryCompositeWidget().clearContent(QUERY_BUILDER_ID);

			}
			
			
				
			var pageControlParms= {
				parentView: this,
				query: query,
				queryResultsSummaryPage: queryResultsSummaryPage,
				queryResultsWidget: this._getQueryResultsWidget(),
				queryOrderBy: orderByAttrib ? orderByAttrib : null,
				queryOrderDirection: orderDirection ? orderDirection : null,
				queryItemsPerPage: itemsPerPage ? itemsPerPage : null,
				queryUrl: queryUrl ? queryUrl : null
				};
			
			var oldQueryResultPageControl = this._queryResultPageControl;
			var oldQueryResults = this.queryResultsWidget;

			this._queryResultPageControl = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl(pageControlParms);
			// destroyed individually
			
			var results = this._getQueryResultsWidget(true);
			results.setQueryResultsTable(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage);	

			this._setCompositeContent(
					QUERY_RESULTS_ID,
					results.domNode,
					this._queryResultPageControl.domNode);

			results.scrollResultsIntoView(); // must be after it is added to the dom
					
			if (oldQueryResults) {
				oldQueryResults.destroy();
			}
			if (oldQueryResultPageControl) {
				oldQueryResultPageControl.destroy();
			}
		},
		resetFilter: function() {
			if(!this._queryResultPageControl) { return };
			this._queryResultPageControl._resetFilterControl();
		}, 
		_getQueryResultsWidget: function(forceCreate) {
			if (forceCreate || !this.queryResultsWidget) {
				if(this.queryResultsWidget) this.queryResultsWidget.destroy();
				var parms= {parentController: this.controller, parentView: this};
				this.queryResultsWidget	= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.query.QueryResults(parms);

			}
			return this.queryResultsWidget;
		},
		showQueryBottomPageControl: function(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, domNode) {

			var pageControlParms= {
				parentView: this,
				query: query,
				queryResultsSummaryPage: queryResultsSummaryPage,
				queryResultsWidget: this._getQueryResultsWidget(),
				queryOrderBy: orderByAttrib,
				queryOrderDirection: orderDirection,
				queryItemsPerPage: itemsPerPage ? itemsPerPage : null,
				isBottomControl: true
				};
			
			var oldBottomPageControl = this._queryBottomPageControl;
			this._queryBottomPageControl =/* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.query.QueryResultPageControl(pageControlParms); 
			
			domNode.appendChild(this._queryBottomPageControl.domNode);
			if(oldBottomPageControl) {
				oldBottomPageControl.destroy();
			}
		},
		confirmLeaveUnsavedQueryChanges: function(kwArgs) {
			if (!this.queryBuilderWidget || !this.queryBuilderWidget._query) return true;
			if(this.queryBuilderWidget.isShowingNewQuery(kwArgs)) return true;
			if(!this.queryBuilderWidget._isDirty && !this.queryBuilderWidget._query.isNew) {
				if(kwArgs.id != null && kwArgs.id != this.queryBuilderWidget._query.itemId) {
				  this.controller.resetQueryProxy(this.queryBuilderWidget._query.itemId);
				}
				return true;
			}
			var result = this.queryBuilderWidget.confirmLeaveUnsavedChanges(kwArgs || {});
			if(result) {
				this.queryBuilderWidget._resetName();
			}
			return result;
		},
		_getQueryBuilderWidget: function() {
			if (!this.queryBuilderWidget) {
				var parms= {parentController: this.controller};
				this.queryBuilderWidget= new com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor(parms);
				this._addChild(this.queryBuilderWidget);
			}
			return this.queryBuilderWidget;
		},
		
		_getQueryCompositeWidget: function() {
			if (!this.queryCompositeWidget) {
				var parms= {parentController: this.controller};
				this.queryCompositeWidget= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.CompositeContentArea(parms);
				this._addChild(this.queryCompositeWidget);
			}
			return this.queryCompositeWidget;
		},
		
		_setMainContent: function(widgetId, mainAreaWidget, metaAreaWidgetDomNode, instanceId, /* updateNav */ skipNavUpdate, keepBookmarks) {
			if(widgetId != "__jazzWorkItemEditor") {
				if(mainAreaWidget != null && mainAreaWidget.isWelcomePage) {
					if(!keepBookmarks) dojo.publish(EventTopics.WORKITEM_ACTION_SET_BREADCRUMBS , [null]); // welcome page special case
				} else {
					if( widgetId =="__jazzQueryResults" || widgetId == "__jazzQueryComposite") {
 							var segment = {
 								href: WorkItemUtil.getShowQueriesUri(),
 								label: this.messages.pageTitleShowQueries
 							}
 							if(!keepBookmarks) dojo.publish(EventTopics.WORKITEM_ACTION_SET_BREADCRUMBS, [[ segment ]]);
 					} else {
						if(!keepBookmarks) dojo.publish(EventTopics.WORKITEM_ACTION_SET_BREADCRUMBS , [[]])
 					}
				}
			}
			this._currentMainWidgetId= widgetId;
			
			// Display the mainAreaWidget in the multi-pane content area
			this._multipaneContentWidget.showContent(widgetId, mainAreaWidget, instanceId, skipNavUpdate);
			
			// Set the content to display the multi-pane content area
			dojox.xml.parser.removeChildren(this._metaAreaContainer);
			if(metaAreaWidgetDomNode) {				
				this._metaAreaContainer.appendChild(metaAreaWidgetDomNode);
			}
			
			this._multipaneContentWidget.placeAt(this._appContainer);
		},
		/*
 		* Return the work item editor banner
 		* This banner is used on all work items and updated whenever a work item is loaded
 		* If the work item is found in the current query results, which always exist
 		* in the "Recently Viewed" cache, then show the banner, regardless of how
 		* the navigation to the work item occured.
 		*/
		_getWorkItemEditorBanner: function(workItemId) {

			if(this._workItemEditorBanner == null) {
				this._workItemEditorBanner = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner(
							{parentController: this.controller, parentView: this});
				this._addChild(this._workItemEditorBanner);
			}
			
			this._updateWorkItemEditorBanner(workItemId);
			return this._workItemEditorBanner;
		},
		_updateWorkItemEditorBanner: function (workItemId) {
			// If no query yet run, return null
			var results;
			if(this.queryResultsWidget) { 
				results = this.queryResultsWidget.getResults();
			}
			this._workItemEditorBanner.setQueryInfo(workItemId, this.controller._lastQueryName, this.controller._lastQueryUrl, results);
			
		},
		
		_setCompositeContent: function(widgetId, mainAreaWidget, metaAreaWidget) {
			var composite= this._getQueryCompositeWidget();
			composite.scrollLeft = 0;
			composite.scrollTop = 0;
			composite.setContent(widgetId, mainAreaWidget, metaAreaWidget);
			if (this._currentMainWidgetId != QUERY_COMPOSITE_ID)
				this._setMainContent(QUERY_COMPOSITE_ID, composite);
			else 
				// Display the mainAreaWidget in the multi-pane content area
				this._multipaneContentWidget.showContent(QUERY_COMPOSITE_ID, composite);

		}
	}
);

})();
