/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.WorkItemProcessPage");

dojo.require("dojo.i18n");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProjectAreaEditorPage"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.WorkItemConfigModel");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.type.TypeEditor");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var messages;
var AbstractProjectAreaEditorPage= com.ibm.team.process.web.ui.internal.admin.projects.AbstractProjectAreaEditorPage;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.WorkItemProcessPage",
	AbstractProjectAreaEditorPage,
{	
	fPendingChanges: false,
	templateString: '<div class="com-ibm-team-workitem-process"></div>',

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PageMessages");
		this.inherited(arguments);
	},
		
	postCreate: function(){	
		this.inherited(arguments);
		
		var self = this;
		
		// setup model
		var handleModelChanged = function(){
			self.fPendingChanges = true;
			self.setDirty(true);
		};
		var model = new com.ibm.team.workitem.web.process.ui.internal.page.WorkItemConfigModel();
		this.model = model;
		dojo.forEach([ "onTypeChanged", "onEnumerationChanged",
			"onPresentationChanged", "onWorkflowChanged", "onOSLCChanged" ],
			function(methodName){
				this.connect(model, methodName, handleModelChanged);
		}, this);
		this.connect(model, "onError", function(errorObj){
			self.displayErrorMessage(errorObj.message);
		});
		
		// create tab container
		var childTabContainer = new com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer({
			templateString: null,
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/page/templates/MainTabContainer.html")
		});
		this.domNode.appendChild(childTabContainer.domNode);
		this.childTabContainer = childTabContainer;
		
		// Add "Types & Attributes" tab
		var typeEditor = new com.ibm.team.workitem.web.process.ui.internal.view.type.TypeEditor({
			title: messages.TypeTitle,
			model: model
		});
		childTabContainer.addChild(typeEditor);

		// Add "Enumerations" tab in a lazy manner
		var enumEditorId = this.id+"_enum";
		childTabContainer.addChild({
			id: enumEditorId,
			title: messages.EnumTitle,
			createChild: function(){
				jazz.core.loader.load_sync("com.ibm.team.workitem.web.process.ui.internal.view.enumeration.EnumerationEditor");
				return new com.ibm.team.workitem.web.process.ui.internal.view.enumeration.EnumerationEditor({
					id: enumEditorId,
					model: model
				});
			}
		});
		typeEditor.connect(typeEditor, "editEnumeration", function(enumId){
			var enumEditor = childTabContainer.selectChild(enumEditorId);
			enumEditor.selectEnum(enumId);
		});

		// Add "Editor Presentations" tab in a lazy manner
		var presentationTabId = this.id+"_presentation";
		childTabContainer.addChild({
			id: presentationTabId,
			title: messages.PresentationTitle,
			createChild: function(){
				jazz.core.loader.load_sync("com.ibm.team.workitem.web.process.ui.internal.view.presentation.PresentationEditor");
				return new com.ibm.team.workitem.web.process.ui.internal.view.presentation.PresentationEditor({
					id: presentationTabId,
					model: model
				});
			}
		});
		typeEditor.connect(typeEditor, "editPresentation", function(editorId, tabId){
			var presentationEditor = childTabContainer.selectChild(presentationTabId);
			presentationEditor.selectEditorPresentation(editorId, tabId);
		});

		// Add "Workflows" tab in a lazy manner
		var workflowEditorId = this.id+"_workflow";
		childTabContainer.addChild({
			id: workflowEditorId,
			title: messages.WorkflowTitle,
			createChild: function(){
				jazz.core.loader.load_sync("com.ibm.team.workitem.web.process.ui.internal.view.workflow.WorkflowEditor");
				return new com.ibm.team.workitem.web.process.ui.internal.view.workflow.WorkflowEditor({
					id: workflowEditorId,
					model: model
				});
			}
		});
		typeEditor.connect(typeEditor, "editWorkflow", function(workflowId){
			var workflowEditor = childTabContainer.selectChild(workflowEditorId);
			workflowEditor.selectWorkflow(workflowId);
		});

		// Add "Change Management Type Bindings" tab in a lazy manner
		var oslcEditorId = this.id+"_oslc";
		childTabContainer.addChild({
			id: oslcEditorId,
			title: messages.OslcTitle,
			createChild: function(){
				jazz.core.loader.load_sync("com.ibm.team.workitem.web.process.ui.internal.view.oslc.OSLCBindingsEditor");
				return new com.ibm.team.workitem.web.process.ui.internal.view.oslc.OSLCBindingsEditor({
					id: oslcEditorId,
					model: model
				});
			}
		});
		
		typeEditor.connect(typeEditor, "displayMessage", function(msg, cssClass){
			if(!msg){
				self.clearMessage();
			}else if(cssClass == "errorMsg"){
				self.displayErrorMessage(msg);
			}
		});
	},
	
	refresh: function() {
		this.fPendingChanges = false;

		dojo.forEach(this.childTabContainer.getChildren(), function(tab){
			// this will only refreshes tabs that are instantiated (tabs that are not selected won't be refreshed) 
			if(tab && tab.refresh && dojo.isFunction(tab.refresh)){
				tab.refresh();
			}
		});
	},
	
	doSave: function(callback){
		this.model.postConfigurations({
			callback: dojo.hitch(this, function(){
				callback();
				this.refresh();
			}),
			error: function(errorObj){
				callback(errorObj);
			}
		});
	},
	
	canSave: function(){
		var msg = this.model.validate();
		if(msg && msg.length > 0){
			this.displayErrorMessage(msg);
			return false;
		}else{
			this.clearMessage();
			return true;
		}
	},
	
	changesPending: function(){
		return this.fPendingChanges;
	},
	
	destroy: function(preserveDom) {
		var childTabContainer = this.childTabContainer;
		if(childTabContainer){
			childTabContainer.destroy(preserveDom);
		}
		this.inherited(arguments); //$NON-NLS-1$
	},
	
	/*
	 * Override AbstractProjectAreaEditorPage
	 * don't call inherited as it will cause infinite loop when navigating to other tab (e.g. "Project Area" tab)
	 * and back to (e.g. "Project Area Management" tab)
	 */
	uninitialize: function(){
		delete this.itemUUID;
		delete this.tabID;
		delete this.tabTitle;
	}
});

})();