/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.models._EnumerationModel");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var SERVICE_ENTRY_POINT = "getAttributeTypes";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.models._EnumerationModel",
	com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel,
{
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages");
	},

	getAttributeTypes: function(callback, forceLoad){
		if(forceLoad !== true && this._attrTypes){
			callback(this._attrTypes);
			return;
		}		

		// actually make XHR call to fetch data from server
		var self = this;
		this.getConfig(SERVICE_ENTRY_POINT, function(configData){
			self._enumerationChanged = false;
			self._attrTypes = configData;
			callback(configData);
		});
	},

	validateEnumerations: function(){
		var msg = null;
		if(this._attrTypes){
			var enumTypes = this._attrTypes.enumerationTypes;
			if(enumTypes){
				var enumId, enumInfo;
				for(var j=0, len2=enumTypes.length; j<len2; j++){
					enumId = enumTypes[j].id;
					enumInfo = this.getEnumeration(enumId);
					if(enumInfo){
						var literals = enumInfo.literals;
						if(literals){
							var len = literals.length;
							if(len > 0){
								var hasDefault = false;
								for(var i=0; i<len; i++){
									if(literals[i].defaultValue === true){
										hasDefault = true;
										break;
									}
								}
								if(hasDefault === false){
									msg = dojo.string.substitute(messages.ErrNoDefaultLiteral, [this.getDisplayName(enumId)]);
									break;
								}
							}
						}
					}
				}
			}
		}
		return msg;
	},
	
	getEnumeration: function(enumId){
		if(this._attrTypes){
			var enumTypes = this._attrTypes.enumerationTypes;
			if(enumTypes){
				for(var i=0, len=enumTypes.length; i<len; i++){
					if(enumTypes[i].id == enumId){
						return enumTypes[i];
					}
				}
			}
		}
		return null;
	},
	
	_hasDuplicateName: function(name){
		if(this._attrTypes){
			var enumName;
			var enumTypes = this._attrTypes.enumerationTypes;
			var found = false;
			for(var i=0, len=enumTypes.length; i<len; i++){
				enumName = (enumTypes[i].displayName && enumTypes[i].displayName.length>0) ? enumTypes[i].displayName : enumTypes[i].id;
				if(enumName == name){
					if(found === false){
						found = true;
					}else{
						return true;
					}
				}
			}
		}
		return false;
	},
	
	getDisplayName: function(enumId){
		var enumInfo = this.getEnumeration(enumId);
		var displayName = null;
		if(enumInfo){
			displayName = enumInfo.displayName;
			displayName = (displayName && displayName.length>0) ? displayName : enumId;
			if(this._hasDuplicateName(displayName)){
				displayName = dojo.string.substitute(messages.EnumDupDisplayName, [ displayName, enumId ]);
			}
		}
		return displayName;
	},
	
	addEnumeration: function(enumInfo){
		if(this._attrTypes){
			enumInfo._eQualifiedClassName = "com.ibm.team.workitem.process.dto:EnumerationType";
			enumInfo.literals = [];
			if(!this._attrTypes.enumerationTypes){
				this._attrTypes.enumerationTypes = [];
			}
			this._attrTypes.enumerationTypes.push(enumInfo);
			this.onEnumerationChanged({ target: null });
		}
	},
	
	removeEnumeration: function(enumId){
		if(this._attrTypes){
			var enumTypes = this._attrTypes.enumerationTypes;
			for(var i=0, len=enumTypes.length; i<len; i++){
				if(enumTypes[i].id == enumId){
					enumTypes.splice(i, 1);
					this.onEnumerationChanged({ target: null });
					break;
				}
			}
		}
	},
	
	setEnumerationProperty: function(enumId, key, value){
		if(this._attrTypes){
			var enumTypes = this._attrTypes.enumerationTypes;
			for(var i=0, len=enumTypes.length; i<len; i++){
				if(enumTypes[i].id == enumId){
					enumTypes[i][key] = value;
					this.onEnumerationChanged({ target: "ENUMERATION", id: enumId, key: key, value: value });
					break;
				}
			}
		}
	},
	
	onEnumerationChanged: function(args){
		this._enumerationChanged = true;
	},
	
	addLiteral: function(enumId, literal){
		var enumInfo = this.getEnumeration(enumId);
		if(!enumInfo){ return; }
		literal._eQualifiedClassName = "com.ibm.team.workitem.process.dto:Literal";
		if(!enumInfo.literals){
			enumInfo.literals = [];
		}
		enumInfo.literals.push(literal);
		this.onEnumerationChanged({ target: "ENUMERATION", id: enumId, key: "literals", value: enumInfo.literals });
	},
	
	removeLiteral: function(enumId, literalId){
		var enumInfo = this.getEnumeration(enumId);
		if(!enumInfo){ return; }
		var literals = enumInfo.literals;
		for(var i=literals.length-1; i>=0; i--){
			if(literals[i].id == literalId){
				literals.splice(i, 1);
				this.onEnumerationChanged({ target: "ENUMERATION", id: enumId, key: "literals", value: literals });
				break;
			}
		}
	},
	
	setLiteral: function(enumId, literalId, args){
		var enumInfo = this.getEnumeration(enumId);
		if(!enumInfo){ return; }
		var literals = enumInfo.literals;
		for(var i=literals.length-1; i>=0; i--){
			if(literals[i].id == literalId){
				for(var key in args){
					literals[i][key] = args[key];
				}
				this.onEnumerationChanged({ target: "ENUMERATION", id: enumId, key: "literals", value: literals });
				break;
			}
		}
	},

	setLiterals: function(enumId, literals){
		var enumInfo = this.getEnumeration(enumId);
		if(!enumInfo){ return; }
		enumInfo.literals = literals;
		this.onEnumerationChanged({ target: "ENUMERATION", id: enumId, key: "literals", value: literals });
	},
	
	setEnumFinal: function(isFinal){
		if(this._attrTypes){
			this._attrTypes.isFinal = isFinal;
			this.onEnumerationChanged({ target:null });
		}
	},
	
	getEnumerationsPostData: function(){
		if(this._enumerationChanged !== true){ return null; }
		
		var attrTypes = this._attrTypes;
		if(!attrTypes){ return null; }
		var obj = {};
		if(attrTypes.isFinal === true){
			obj.isFinal = true;
		}
		if(attrTypes.enumerationTypes){
			obj.enumerations = attrTypes.enumerationTypes;
		}
		return dojo.toJson(obj);
	}
});

})();
