/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.models._OSLCModel");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel");

(function(){

var SERVICE_ENTRY_POINT = "getOSLCConfigurations";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.models._OSLCModel",
	com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel,
{
	getOSLCConfigurations: function(callback, forceLoad){
		if(forceLoad !== true && this._oslcConfig){
			callback(this._oslcConfig);
			return;
		}

		// actually make XHR call to fetch data from server
		var self = this;
		this.getConfig(SERVICE_ENTRY_POINT, function(configData){
			self._oslcChanged = false;
			self._oslcConfig = configData;
			callback(configData);
		});
	},

	onOSLCChanged: function(eventType){
		this._oslcChanged = true;
	},
	
	setOSLCTypeBinding: function(oslcTypeId, workItemTypeId){
		if(this._oslcConfig){
			var typeBindings = this._oslcConfig.typeBindings;
			if(typeBindings){
				var typeBindingArray = typeBindings.typeBinding;
				for(var i=0, l=typeBindingArray.length; i<l; i++){
					if(typeBindingArray[i].oslcTypeId === oslcTypeId){
						if(typeBindingArray[i].workItemTypeId != workItemTypeId){
							typeBindingArray[i].workItemTypeId = workItemTypeId;
							this.onOSLCChanged("BINDING_CHANGED", { oslcTypeId: oslcTypeId, workItemTypeId: workItemTypeId });
						}
						return;
					}
				}
				// not found ... maybe new oslc type
				var type = {oslcTypeId: oslcTypeId, workItemTypeId: workItemTypeId};
				typeBindingArray.push(type);
				this.onOSLCChanged("BINDING_ADDED", { oslcTypeId: oslcTypeId, workItemTypeId: workItemTypeId });
			}
		}
	},
	

	setOSLCStateGroup: function(stateGroupId, state, resolution){
		if(this._oslcConfig){
			var stateGroups = this._oslcConfig.stateGroups;
			if(stateGroups){
				var stateGroupArray = stateGroups.stateGroup || [];
				for(var i = 0, l = stateGroupArray.length; i < l; i++){
					if(stateGroupArray[i].id === stateGroupId){
						var changed = false;
						if(stateGroupArray[i].state != state){
							stateGroupArray[i].state = state;
							changed = true;
						}
						if(stateGroupArray[i].resolution != resolution){
							stateGroupArray[i].resolution = resolution;
							changed = true;
						}
						if(changed){
							this.onOSLCChanged("STATEGROUP_CHANGED", { stateGroupId: stateGroupId, state: state, resolution: resolution });
						}
						return;
					}
				}
				// not found ... maybe new state group
				var stateGroup = {id: stateGroupId, state: state, resolution: resolution};
				stateGroupArray.push(stateGroup);
				this.onOSLCChanged("STATEGROUP_ADDED", { stateGroupId: stateGroupId, state: state, resolution: resolution });
			}
		}
	},
	
	setOslcFinal: function(isFinal){
		if(this._oslcConfig){
			this._oslcConfig.isFinal = isFinal;
			this.onOSLCChanged();
		}
	},
	
	getOslcPostData: function(){
		var oslcObj = this._oslcConfig;
		if(this._oslcChanged === true && oslcObj){
			return dojo.toJson(oslcObj);
		}
		return null;
	}
});

})();