/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.models._PresentationModel");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel");

(function(){

var SERVICE_ENTRY_POINT = "getPresentations";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.models._PresentationModel",
	com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel,
{
    constructor: function(){
		this._presentations = null;
	},

	getEditorPresentations: function(callback, forceLoad){
		// if not forceLoad=true and there's cached data, don't make XHR call and just invoke callback
		if(forceLoad !== true && this._presentations){
			callback(this._presentations);
			return;
		}
		
		// actually make XHR call to fetch data from server
		var self = this;
		this.getConfig(SERVICE_ENTRY_POINT, function(configData){
			self._presentationChanged = false;
			self._presentations = configData;
			callback(configData);
		});
	},
	
	onPresentationChanged: function(obj){
		this._presentationChanged = true;
	},
	
	getEditors: function(){
		var presentations = this._presentations;
		if(presentations){
			return presentations.editor || [];
		}
		return [];
	},
	
	addEditor: function(obj){
		var presentations = this._presentations;
		if(!presentations){ return null; }
	
		if(!presentations.editor){
			presentations.editor = [];
		}
		presentations.editor.push({
			_eQualifiedClassName: "com.ibm.team.workitem.process.dto:Editor",
			id: obj.id,
			childTabs: []
		});
		
		this.onPresentationChanged({
			editorId: obj.id
		});
	},
	
	removeEditor: function(editorId){
		var presentations = this._presentations;
		if(!presentations){ return null; }
		
		var editorArray = presentations.editor;
		for(var i=0, len=editorArray.length; i<len; i++){
			if(editorArray[i].id == editorId){
				editorArray.splice(i,1);
				this.onPresentationChanged({});
				break;
			}
		}
	},
	
	_findEditorData: function(editorId){
		var presentations = this._presentations;
		if(!presentations){ return null; }
		
		var editorArray = presentations.editor;
		if(editorArray){
			for(var i=0, len=editorArray.length; i<len; i++){
				if(editorArray[i].id == editorId){
					return editorArray[i];
				}
			}
		}
		return null;
	},
	
	getEditorsForTabId: function(tabId){
		// returns an array of editor IDs that contain tabs with a specified tabId
		var editors = [];
		var allEditors = this.getEditors();
		if(allEditors){
			dojo.forEach(allEditors, function(editor){
				for(var i=editor.childTabs.length-1; i>=0; i--){
					if(editor.childTabs[i].tabId == tabId){
						editors.push(editor.id);
						break;
					}
				}
			});
		}
		return editors;
	},

	getEditorsForSectionId: function(sectionId){
		// returns an array of editor IDs that contain sections with a specified sectionId
		var editors = [];
		var allTabs = this.getAllTabs();
		dojo.forEach(allTabs, function(tab){
			if(tab.childSections){
				for(var i=tab.childSections.length-1; i>=0; i--){
					if(tab.childSections[i].sectionId == sectionId){
						dojo.forEach(this.getEditorsForTabId(tab.id), function(editorId){
							if(dojo.indexOf(editors, editorId) == -1){
								editors.push(editorId);
							}
						});
						break;
					}
				}
			}
		}, this);
		return editors;
	},
	
	isSharedTab: function(tabId){
		var count = 0;
		var allEditors = this.getEditors();
		for(var i=allEditors.length-1; i>=0; i--){
			var editor = allEditors[i];
			if(editor.childTabs){
				for(var j=editor.childTabs.length-1; j>=0; j--){
					if(editor.childTabs[j].tabId == tabId){
						count++;
						if(count > 1){
							return true;
						}
					}
				}
			}
		}
		return false;
	},
	
	isSharedSection: function(sectionId){
		var tabId = null;
		var allTabs = this.getAllTabs();
		for(var i=allTabs.length-1; i>=0; i--){
			var tab = allTabs[i];
			if(tab.childSections){
				for(var j=tab.childSections.length-1; j>=0; j--){
					if(tab.childSections[j].sectionId == sectionId){
						if(tabId){
							return true;
						}
						tabId = tab.id;
					}
				}
			}
		}
		return this.isSharedTab(tabId);
	},

	getTabs: function(editorId){
		// returns a list of tab info for a given editor
		var editorInfo = this._findEditorData(editorId);
		if(editorInfo && editorInfo.childTabs){
			return editorInfo.childTabs;
		}
		return [];
	},
	
	getAllTabs: function(){
		var presentations = this._presentations;
		if(presentations){
			return presentations.tab || [];
		}
		return [];
	},
	
	_findTabData: function(tabId){
		var tabArray = this.getAllTabs();
		for(var i=0,len=tabArray.length; i<len; i++){
			if(tabArray[i].id == tabId){
				return tabArray[i];
			}
		}
		return null;
	},
	
	getTabLayout: function(tabId){
		var tabInfo = this._findTabData(tabId);
		if(tabInfo){
			return tabInfo.layout;
		}
		return null;
	},
	
	setTabLayout: function(tabId, layout){
		var tabInfo = this._findTabData(tabId);
		if(tabInfo){
			tabInfo.layout = layout;
			this.onPresentationChanged({
				target: "EDITOR"
			});
		}
	},
	
	getTabProperty: function(editorId, tabIndex, key){
		var childTabs = this.getTabs(editorId);
		if(tabIndex < childTabs.length){
			return childTabs[tabIndex][key];
		}
		return null;
	},
	
	addTab: function(editorId, tabId, obj){
		// obj has following properties
		//		title : String
		//		layout : String
		//		properties : Array (optional)
		//		isHeader : boolean (optional)
		
		var editorInfo = this._findEditorData(editorId);
		if(!editorInfo){ return; }

		// add to editor element
		if(!editorInfo.childTabs){
			editorInfo.childTabs = [];
		}
		var tabInfo = {
			_eQualifiedClassName: "com.ibm.team.workitem.process.dto:TabReference",
			tabId: tabId,
			title: obj.title
		};
		if(obj.properties && obj.properties.length>0){
			tabInfo.properties = obj.properties;
		}
		var isHeader = (obj.layout == "internalHeader");
		if(isHeader && editorInfo.childTabs.length > 0){
			editorInfo.childTabs.splice(0, 0, tabInfo);
		}else{
			editorInfo.childTabs.push(tabInfo);
		}
		
		// add tab definition
		if(!this._presentations.tab){
			this._presentations.tab = [];
		}
		if(!this._findTabData(tabId)){
			this._presentations.tab.push({
				_eQualifiedClassName: "com.ibm.team.workitem.process.dto:Tab",
				id: tabId,
				layout: obj.layout,
				childSections: []
			});
		}
		
		var args = {
			target: "EDITOR",
			editorId: editorId
		};
		if(!isHeader){
			args.tabIndex = editorInfo.childTabs.length-1;
		}
		this.onPresentationChanged(args);
	},
	
	editTab: function(editorId, tabIndex, obj){
		// obj has following properties
		//		title : String
		//		layout : String
		//		properties : Array (optional)
		var editorInfo = this._findEditorData(editorId);
		var tabId = null;
		if(editorInfo){
			var tabArray = editorInfo.childTabs;
			if(tabArray && tabIndex < tabArray.length){
				tabId = tabArray[tabIndex].tabId;
				tabArray[tabIndex].title = obj.title;
				if(obj.properties && obj.properties.length>0){
					tabArray[tabIndex].properties = obj.properties;
				}else{
					delete tabArray[tabIndex].properties;
				}
			}
		}

		if(tabId){
			var tabInfo = this._findTabData(tabId);
			if(tabInfo){
				tabInfo.layout = obj.layout;
			}
		}

		this.onPresentationChanged({
			target: "EDITOR",
			editorId: editorId,
			tabIndex: tabIndex
		});
	},
	
	_removeUnreferencedTab: function(tabId){
		if(!tabId){ return; }
	
		// check if this is the last instance of this tabId
		var bFound = false;
		var editors = this.getEditors();
		if(editors){
			for(var i=editors.length-1; i>=0; i--){
				if(editors[i].childTabs){
					for(var j = editors[i].childTabs.length-1; j>=0; j--){
						if(editors[i].childTabs[j].tabId == tabId){
							bFound = true;
							break;
						}
					}
				}
				if(bFound){
					break;
				}
			}
		}
		
		// remove tab definition if last tab was removed
		if(!bFound){
			var tabs = this.getAllTabs();
			if(tabs){
				for(var k=tabs.length-1; k>=0; k--){
					if(tabs[k].id == tabId){
						tabs.splice(k,1);
						break;
					}
				}
			}
		}
	},
	
	removeTab: function(editorId, tabIndex){
		// remove tab instance from editor
		var tabId = null;
		var editorInfo = this._findEditorData(editorId);
		if(editorInfo){
			var tabArray = editorInfo.childTabs;
			if(tabArray && tabIndex < tabArray.length){
				tabId = tabArray[tabIndex].tabId;
				tabArray.splice(tabIndex, 1);
			}
		}
		
		this._removeUnreferencedTab(tabId);
		
		this.onPresentationChanged({
			target: "EDITOR",
			editorId: editorId
		});
	},
	
	renameTab: function(editorId, tabIndex, title){
		var editorInfo = this._findEditorData(editorId);
		if(editorInfo){
			var tabArray = editorInfo.childTabs;
			if(tabArray && tabIndex < tabArray.length){
				tabArray[tabIndex].title = title;
				this.onPresentationChanged({
					target: "TAB",
					editorId: editorId,
					tabIndex: tabIndex
				});
			}
		}
	},
	
	moveTab: function(editorId, oldTabIndex, newTabIndex){
		var editorInfo = this._findEditorData(editorId);
		if(editorInfo){
			var tabArray = editorInfo.childTabs;
			var len = tabArray.length;
			if(tabArray && 0 <= oldTabIndex && oldTabIndex < len && 0 <= newTabIndex && newTabIndex < len && oldTabIndex != newTabIndex){
				var tabInfo = tabArray[oldTabIndex];
				tabArray.splice(oldTabIndex, 1);
				tabArray.splice(newTabIndex, 0, tabInfo);
				this.onPresentationChanged({
					target: "EDITOR",
					editorId: editorId,
					tabIndex: newTabIndex // tabIndex to be selected
				});
			}
		}
	},
	
	getAllSections: function(){
		var presentations = this._presentations;
		if(presentations){
			return presentations.section || [];
		}
		return [];
	},
	
	getSections: function(tabId){
		var tabInfo = this._findTabData(tabId);
		if(tabInfo){
			return tabInfo.childSections;
		}
		return [];
	},
	
	addSection: function(tabId, sectionId, obj){
		// obj has the following properties
		//		title
		//		slot (optional)
		var tabInfo = this._findTabData(tabId);
		if(!tabInfo){
			return;
		}
		if(!tabInfo.childSections){
			tabInfo.childSections = [];
		}
		var sectionInfo = {
			_eQualifiedClassName: "com.ibm.team.workitem.process.dto:SectionReference",
			sectionId: sectionId,
			title: obj.title
		};
		if(obj.slot){
			sectionInfo.slot = obj.slot;
		}
		if(obj.properties){
			sectionInfo.properties = obj.properties;
		}
		tabInfo.childSections.push(sectionInfo);
		
		// add section definition
		if(!this._presentations.section){
			this._presentations.section = [];
		}
		if(!this._findSectionData(sectionId)){
			this._presentations.section.push({
				_eQualifiedClassName: "com.ibm.team.workitem.process.dto:Section",
				id: sectionId,
				presentations: []
			});
		}
		this.onPresentationChanged({
			target: "TAB",
			tabId: tabId
		});
	},
	
	editSection: function(tabId, sectionIndex, obj){
		var tabInfo = this._findTabData(tabId);
		if(tabInfo){
			var sectionArray = tabInfo.childSections;
			if(sectionIndex < sectionArray.length){
				sectionArray[sectionIndex].slot = obj.slot;
				sectionArray[sectionIndex].title = obj.title;
				if(obj.properties){
					sectionArray[sectionIndex].properties = obj.properties;
				}
				this.onPresentationChanged({
					target: "TAB",
					tabId: tabId
				});
			}
		}
	},
	
	_removeUnreferencedSection: function(sectionId){
		if(!sectionId){ return; }

		// check if this is the last section for a given section ID
		var bFound = false;
		var tabs = this.getAllTabs();
		if(tabs){
			for(var i = tabs.length-1; i>=0; i--){
				if(tabs[i].childSections){
					for(var j = tabs[i].childSections.length-1; j>=0; j--){
						if(tabs[i].childSections[j].sectionId == sectionId){
							bFound = true;
							break;
						}
					}
				}
				if(bFound){
					break;
				}
			}
		}
		
		// the last section has been removed, so delete section definition as well
		if(!bFound){
			var sections = this.getAllSections();
			for(var k = sections.length-1; k>=0; k--){
				if(sections[k].id == sectionId){
					sections.splice(k, 1);
					break;
				}
			}
		}
	},
	
	removeSection: function(tabId, sectionIndex){
		var sectionArray = this.getSections(tabId);
		if(sectionIndex < sectionArray.length){
			var sectionId = this.getSectionProperty(tabId, sectionIndex, "sectionId");
			sectionArray.splice(sectionIndex,1);
			this._removeUnreferencedSection(sectionId);

			this.onPresentationChanged({
				target: "TAB",
				tabId: tabId
			});
		}
	},
	
	moveSection: function(tabId, sectionIndex, toSlot, toIndex){
		// sectionIndex is index of sectionInfo within tab
		// sectionId cannot be used as an identifier, since there may be multiple sections having the same sectionId
		// toIndex is an index within the same slot
		var tabInfo = this._findTabData(tabId);
		if(tabInfo){
			if(!tabInfo.childSections){
				tabInfo.childSections = [];
			}
			var childSections = tabInfo.childSections;
			if(sectionIndex >= childSections.length){
				return;
			}
			var sectionInfo = dojo.clone(childSections[sectionIndex]);
			var slotCount = 0;
			var insertIndex = -1;
			for(var i=0, len=childSections.length; i<len; i++){
				if(childSections[i].slot == toSlot){
					if(slotCount == toIndex){
						insertIndex = i;
					}
					slotCount++;
				}
			}
			if(toIndex === slotCount){
				// if moving to the last of a different slot
				insertIndex = len-1;
			}
			if(insertIndex != -1){
				tabInfo.childSections.splice(sectionIndex, 1);
				sectionInfo.slot = toSlot;
				if(insertIndex == len-1){
					tabInfo.childSections.push(sectionInfo);
				}else{
					tabInfo.childSections.splice(insertIndex, 0, sectionInfo);
				}
				this.onPresentationChanged({
					target: "TAB",
					tabId: tabId
				});
			}
		}
	},

	getSectionProperty: function(tabId, sectionIndex, key){
		var childSections = this.getSections(tabId);
		if(sectionIndex < childSections.length){
			return childSections[sectionIndex][key];
		}
		return null;
	},
	
	addHeader: function(editorId, sectionId, properties){
		// add header tab
		var tabId = editorId+".internalHeader";
		this.addTab(editorId, tabId, {
			title: "none",  // this title is not visible in the UI
			layout: "internalHeader"
		});
		
		// add header section
		this.addSection(tabId, sectionId, {
			title: "none", // this title is not visible in the UI
			properties: properties
		});
	},
	
	removeHeader: function(editorId){
		// remove header tab from 'editor' element
		var tabs = this.getTabs(editorId);
		var tabId = null;
		for(var i=tabs.length-1; i>=0; i--){
			var tabInfo = this._findTabData(tabs[i].tabId);
			if(tabInfo && tabInfo.layout == "internalHeader"){
				tabId = tabs[i].tabId;
				tabs.splice(i, 1);
				break;
			}
		}
		
		if(!tabId){ return; }

		// keep section ID
		var sectionId = null;
		var childSections = this.getSections(tabId);
		if(childSections.length > 0){
			sectionId = childSections[0].sectionId; // there should only be only one child section under a header tab
		}

		// remove unreferenced tab/section
		this._removeUnreferencedTab(tabId);
		this._removeUnreferencedSection(sectionId);
				
		this.onPresentationChanged({
			target: "EDITOR",
			editorId: editorId
		});
	},
	
	_findSectionData: function(sectionId){
		var presentations = this._presentations;
		if(!presentations){ return null; }
		
		var sectionArray = presentations.section;
		if(sectionArray){
			for(var i=0,len=sectionArray.length; i<len; i++){
				if(sectionArray[i].id == sectionId){
					return sectionArray[i];
				}
			}
		}
		return null;
	},
	
	getPresentations: function(sectionId){
		var sectionInfo = this._findSectionData(sectionId);
		if(sectionInfo){
			return sectionInfo.presentations || [];
		}
		return [];
	},
	
	getKindMappings: function(){
		var presentations = this._presentations;
		if(!presentations){ return null; }
		return presentations.presentationMappings;
	},
	
	getUsedAttributeIDs: function(editorId){
		var idArray = [];
		var editorInfo = this._findEditorData(editorId);
		if(editorInfo){
			var self = this;
			var childTabs = editorInfo.childTabs;
			if(childTabs){
				dojo.forEach(childTabs, function(childTab){
					var tabInfo = self._findTabData(childTab.tabId);
					if(tabInfo){
						var childSections = tabInfo.childSections;
						if(childSections){
							dojo.forEach(childSections, function(childSection){
								var sectionInfo = self._findSectionData(childSection.sectionId);
								if(sectionInfo){
									var presentations = sectionInfo.presentations;
									if(presentations){
										dojo.forEach(presentations, function(presentation){
											if(presentation.attributeId){
												idArray.push(presentation.attributeId);
											}
										});
									}
								}
							});
						}
					}
				});
			}
		}
		return idArray;
	},
	
	addPresentation: function(sectionId, params){
		var sectionInfo = this._findSectionData(sectionId);
		if(sectionInfo){
			if(!sectionInfo.presentations){
				sectionInfo.presentations = [];
			}
			var presentationInfo = {
				_eQualifiedClassName: "com.ibm.team.workitem.process.dto:Presentation",
				kind: params.kind
			};
			if(params.attributeId){
				presentationInfo.attributeId = params.attributeId;
			}
			if(params.properties){
				presentationInfo.properties = params.properties;
			}
			if(params.label){
				presentationInfo.label = params.label;
			}
			if(params.desc){
				presentationInfo.description = params.desc;
			}
			if(params.id){
				presentationInfo.id = params.id;
			}
			sectionInfo.presentations.push(presentationInfo);
			this.onPresentationChanged({
				target: "SECTION",
				sectionId: sectionId
			});
		}
	},
	
	editPresentation: function(sectionId, params, index){
		var sectionInfo = this._findSectionData(sectionId);
		if(sectionInfo){
			var presentations = sectionInfo.presentations;
			if(presentations && presentations.length > index){
				var presentation = presentations[index];
				presentation.kind = params.kind;
				if(params.attributeId){
					presentation.attributeId = params.attributeId;
				}else{
					delete presentation.attributeId;
				}
				if(params.label){
					presentation.label = params.label;
				}else{
					delete presentation.label;
				}
				if(params.desc){
					presentation.description = params.desc;
				}else{
					delete presentation.description;
				}
				if(params.id){
					presentation.id = params.id;
				}else{
					delete presentation.id;
				}
				if(params.properties){
					presentation.properties = params.properties;
				}
			}
			this.onPresentationChanged({
				target: "SECTION",
				sectionId: sectionId
			});
		}
	},
	
	removePresentation: function(sectionId, index){
		var sectionInfo = this._findSectionData(sectionId);
		if(sectionInfo){
			var presentations = sectionInfo.presentations;
			if(presentations && presentations.length>index){
				presentations.splice(index,1);
				this.onPresentationChanged({
					target: "SECTION",
					sectionId: sectionId
				});
			}
		}
	},
	
	movePresentation: function(fromSectionId, fromIndex, toSectionId, toIndex){
		var fromSectionInfo = this._findSectionData(fromSectionId);
		var toSectionInfo = this._findSectionData(toSectionId);
		if(fromSectionInfo && toSectionInfo){
			var fromPresentations = fromSectionInfo.presentations;
			if(!toSectionInfo.presentations){
				toSectionInfo.presentations = [];
			}
			var toPresentations = toSectionInfo.presentations;
			if(fromPresentations && fromPresentations.length > fromIndex &&
				toPresentations && toPresentations.length >= toIndex){
				var presentation = fromPresentations[fromIndex];
				fromPresentations.splice(fromIndex,1);
				toPresentations.splice(toIndex, 0, presentation);
				this.onPresentationChanged({
					target: "SECTION",
					sectionId: fromSectionId
				});
				if(fromSectionId != toSectionId){
					this.onPresentationChanged({
						target: "SECTION",
						sectionId: toSectionId
					});
				}
			}
		}
	},
	
	setPresentationFinal: function(isFinal){
		if(this._presentations){
			this._presentations.isFinal = isFinal;
			this.onPresentationChanged({});
		}
	},
	
	getPresentationPostData: function(){
		if(this._presentationChanged !== true){ return null; }

		var presentations = this._presentations;
		if(!presentations){ return null; }
		var obj = null;
		dojo.forEach([ "editor", "tab", "section" ], function(key){
			if(presentations[key]){
				if(!obj){
					obj = {};
				}
				obj[key] = presentations[key];
			}
		});
		if(presentations.isFinal === true){
			if(!obj){
				obj = {};
			}
			obj.isFinal = true;
		}
		return (obj) ? dojo.toJson(obj) : null;
	}
});

})();