/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.models._TypeModel");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "ModelMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var SERVICE_ENTRY_POINT = "getWorkItemTypes";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.models._TypeModel",
	com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel,
{
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "ModelMessages");
	},
	
	getWorkItemConfig: function(callback, forceLoad){
		if(forceLoad !== true && this.workItemConfig){
			callback(this.workItemConfig);
			return;
		}		

		// actually make XHR call to fetch data from server
		var self = this;
		this.getConfig(SERVICE_ENTRY_POINT, function(configData){
			self._typeChanged = false;
			self.workItemConfig = configData;
			callback(configData);
		});
	},
	
	/*
	 * args is a Object having the following keys:
	 * target : "TYPE"/"TYPE_CATEGORY"/null (null will update all types)
	 * id : target ID (String)
	 * key : property name that got updated
	 * value : updated value
	 */
	onTypeChanged: function(args){
		this._typeChanged = true;
	},
	
	getWorkItemTypes: function(callback){
		if(this.workItemConfig){
			return this.workItemConfig.types;		
		}
		return null;
	},
	
	//
	getAllWorkItemCategories: function(){
		var categories = [];
		if(this.workItemConfig){
			var types = this.workItemConfig.types;
			if(types){
				for(var i = 0, l = types.length; i < l; i++){
					var c = types[i].category;
					if(dojo.indexOf(categories, c) < 0){
						categories.push(c);
					}
				}
			}
		}
		return categories; // not found 
	},

	getTypeProperty: function(workItemTypeId, key){
		if (this.workItemConfig) {
			var types = this.workItemConfig.types;
			for (var i = 0, l = types.length; i < l; i++) {
				if (types[i].id === workItemTypeId) {
					return types[i][key];
				}
			}
		}
		return null;
	},

	setTypeProperty: function(workItemTypeId, key, value){
		if (this.workItemConfig) {
			var types = this.workItemConfig.types;
			for (var i = 0, l = types.length; i < l; i++) {
				if (types[i].id === workItemTypeId) {
					types[i][key] = value;
					this.onTypeChanged({ target: "TYPE", id: workItemTypeId, key: key, value: value });
					return;
				}
			}
			
			// not found ... maybe new work item type
			var typeInfo = {id: workItemTypeId};
			typeInfo[key] = value;
			this.addType(typeInfo);
			this.onTypeChanged({ target: "TYPE", id: workItemTypeId, key: key, value: value });
		}
	},
	
	addType: function(typeInfo){
		var workItemConfig = this.workItemConfig;
		if(workItemConfig){
			if(!workItemConfig.types){
				workItemConfig.types = [];
			}
			typeInfo._eQualifiedClassName = "com.ibm.team.workitem.process.dto:WorkItemType";
			workItemConfig.types.push(typeInfo);
			this.onTypeChanged({ target:null });
		}
	},
	
	removeType: function(workItemTypeId){
		if (this.workItemConfig) {
			var types = this.workItemConfig.types;
			for (var i = types.length-1; i >= 0; i--) {
				if (types[i].id === workItemTypeId) {
					types.splice(i,1);
					this.onTypeChanged({ target:null });
					return;
				}
			}
		}
	},
	
	//
	setWorkItemCategory: function(workItemTypeId, category){
		if (this.workItemConfig) {
			var types = this.workItemConfig.types;
			for (var i = 0, l = types.length; i < l; i++) {
				if (types[i].id === workItemTypeId) {
					types[i].category = category;
					return;
				}
			}
			// not found ... maybe new work item type
			var type = {id: workItemTypeId, category: category};
			types.push(type);
		}
	},
	// 
	getWorkItemType: function(workItemTypeId){
		if (this.workItemConfig) {
			var types = this.workItemConfig.types;
			for (var i = 0, l = types.length; i < l; i++) {
				if (types[i].id === workItemTypeId) {
					return types[i];
				}
			}
		}
		return null; // not found
	},
	//
	getWorkItemCategory: function(workItemTypeId){
		var type = this.getWorkItemType(workItemTypeId);
		if(type){
			return type.category;
		}
		return null; // not found 
	},
	
	getAffectedTypes: function(typeId){
		// returns an array of WI type info for types that belongs to the same type category
		var category = this.getWorkItemCategory(typeId);
		return dojo.filter(this.getWorkItemTypes(), function(typeInfo){
			return (typeInfo.id != typeId && typeInfo.category == category);
		});
	},
	
	validateTypeName: function(name){
		if(!name || name.length === 0){
			// check empty name
			return messages.errNameEmpty;
		}else{
			var maxLength = 250;
			if(name.length > maxLength){
				// length of name needs to be under 250
				// see (com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypePart.AddEditTypeDialog)
				return dojo.string.substitute(messages.errNameInvalidLength, [maxLength]);
			}
		}
		return null;
	},
	
	//// attributes : start
	
	getAllCustomAttributes: function(){
		var attrs = [];
		if(this.workItemConfig){
			var customAttrs = this.workItemConfig.customAttributes;
			if(customAttrs){
				var _isDuplicateAttr = function(allAttrs, customAttr){
					return dojo.some(allAttrs, function(attr){
						return (attr.id == customAttr.id);
					});
				};
			
				dojo.forEach(customAttrs, function(customAttrInfo){
					if(customAttrInfo.attributes){
						dojo.forEach(customAttrInfo.attributes, function(customAttr){
							if(!_isDuplicateAttr(attrs, customAttr)){
								attrs.push(customAttr);
							}
						});
					}
				});
			}
		}
		return attrs;
	},
	
	getCustomAttributes: function(workItemTypeId){
		var category = this.getWorkItemCategory(workItemTypeId);
		if (category) {
			var attrss = this.workItemConfig.customAttributes;
			if(attrss){
				for (var i = 0, l = attrss.length; i < l; i++) {
					if (attrss[i].category === category) {
						return attrss[i].attributes; 
					}
				}
			}
		}
		return null; // not found 
	},
	
	setCustomAttributes: function(workItemTypeId, attributes){
		var category = this.getWorkItemCategory(workItemTypeId);
		if (category) {
			if(!this.workItemConfig.customAttributes){
				this.workItemConfig.customAttributes = [];
			}
			var attrss = this.workItemConfig.customAttributes;
			for (var i = 0, l = attrss.length; i < l; i++) {
				if (attrss[i].category === category) {
					attrss[i].attributes = attributes;
					this.onTypeChanged({ target: "TYPE_CATEGORY", id: category, key: "attributes", value: attributes });
					return;
				}
			}
			
			// not found, so add
			attrss.push({
				category: category,
				_eQualifiedClassName: "com.ibm.team.workitem.process.dto:CustomAttributes",
				attributes: attributes
			});
			this.onTypeChanged({ target: "TYPE_CATEGORY", id: category, key: "attributes", value: attributes });
		}
	},
	
	addCustomAttribute: function(workItemTypeId, attr){
		var attributes = this.getCustomAttributes(workItemTypeId);
		if(!attributes){
			attributes = [];
		}
		attr._eQualifiedClassName = "com.ibm.team.workitem.process.dto:WorkItemAttribute";
		attributes.push(attr);
		this.setCustomAttributes(workItemTypeId, attributes);
		
		if(!this.workItemConfig.attributeDefinitions){
			this.workItemConfig.attributeDefinitions = [];
		}
		this.workItemConfig.attributeDefinitions.push({
			_eQualifiedClassName: "com.ibm.team.workitem.process.dto:AttributeDefinition",
			id: attr.id,
			name: attr.name,
			typeId: attr.typeId
		});
	},
	
	removeCustomAttribute: function(workItemTypeId, attrId){
		var attributes = this.getCustomAttributes(workItemTypeId);
		for(var i=0, len=attributes.length; i<len; i++){
			if(attributes[i].id == attrId){
				attributes.splice(i, 1);
				this.setCustomAttributes(workItemTypeId, attributes);
				break;
			}
		}
		
		var attrDefs = this.workItemConfig.attributeDefinitions;
		if(attrDefs){
			for(var j=0, len2=attrDefs.length; j<len2; j++){
				if(attrDefs[j].id == attrId){
					attrDefs.splice(j, 1);
					break;
				}
			}
		}
	},
	
	setCustomAttributeProperty: function(workItemTypeId, attrId, key, value){
		// only used for key="name"
		var attributes = this.getCustomAttributes(workItemTypeId);
		for(var i=0,len=attributes.length; i<len; i++){
			if(attributes[i].id == attrId){
				attributes[i][key] = value;
				break;
			}
		}
		this.setCustomAttributes(workItemTypeId, attributes);
		
		var attrDef = this.findAttributeDefinition(attrId);
		if(attrDef){
			attrDef[key] = value;
		}		
	},
	
	getBuiltInAttributes: function(){
		if(this.workItemConfig){
			return this.workItemConfig.builtInAttributes;
		}
		return [];
	},
	
	setBuiltInAttributeProperty: function(attrId, key, value){
		// only used for key="name"

		// first check if attributeDefinition is specified or not
		var attrDef = this.findAttributeDefinition(attrId);
		if(attrDef){
			attrDef[key] = value;
			this.onTypeChanged({ target: "BUILTIN_ATTR" });
			return;
		}

		// attributeDefinition not found
		// built-in attributes are used as is, so add attributeDefinition element
		var builtIns = this.workItemConfig.builtInAttributes;
		if(builtIns){
			for(var i=0, len=builtIns.length; i<len; i++){
				if(builtIns[i].id == attrId){
					if(!this.workItemConfig.attributeDefinitions){
						this.workItemConfig.attributeDefinitions = [];
					}
					attrDef = {
						_eQualifiedClassName: "com.ibm.team.workitem.process.dto:AttributeDefinition",
						id: builtIns[i].id,
						name: builtIns[i].name,
						typeId: builtIns[i].typeId
					};
					attrDef[key] = value; // overwrite with the new value
					this.workItemConfig.attributeDefinitions.push(attrDef);
					
					builtIns[i][key] = value;
					this.onTypeChanged({ target: "BUILTIN_ATTR" });
				}
			}
		}
	},
	
	findAttributeDefinition: function(attrId){
		var attrDefs = this.workItemConfig.attributeDefinitions;
		if(attrDefs){
			for(var i=0, len=attrDefs.length; i<len; i++){
				if(attrDefs[i].id == attrId){
					return attrDefs[i];
				}
			}
		}
		return null;
	},
	
	//// attributes : end

	// prefix can be "workitemtype"/"enumeration"
	
	setTypeFinal: function(isFinal){
		if(this.workItemConfig){
			this.workItemConfig.isFinal = isFinal;
			this.onTypeChanged({ target:null });
		}
	},
	
	getTypesPostData: function(){
		if(this._typeChanged !== true){ return null; }
	
		// only send necessary data (remove built-in attributes data)
		var configObj = this.workItemConfig;
		if(!configObj){ return null; }
		var obj = {};
		if(configObj.isFinal === true){
			obj.isFinal = true;
		}
		if(configObj.types){
			obj.types = configObj.types;
		}
		if(configObj.customAttributes){
			obj.customAttributes = configObj.customAttributes;
		}
		if(configObj.attributeDefinitions){
			obj.attributeDefinitions = configObj.attributeDefinitions;
		}
		return dojo.toJson(obj);
	}
});

})();