/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.page.models._WorkflowModel");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var SERVICE_ENTRY_POINT = "getWorkflows";
var defaultStateGroups;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.page.models._WorkflowModel",
	com.ibm.team.workitem.web.process.ui.internal.page.models._AbstractModel,
{
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages");

		defaultStateGroups = dojo.map(["open", "inprogress", "closed"], function(group){
			return {
				id: group,
				name: messages[group]
			};
		});
	},

	getWorkflows: function(callback, forceLoad){
		if(forceLoad !== true && this.workflows){
			callback(this.workflows);
			return;
		}

		// actually make XHR call to fetch data from server
		var self = this;
		this.getConfig(SERVICE_ENTRY_POINT, function(configData){
			self._workflowChanged = false;
			self.workflows = configData;
			callback(configData);
		});
	},
	
	validateWorkflows: function(){
		// validate if there is startActionId
		var workflows = this.workflows;
		if(workflows && workflows.workflowDefinitions){
			var ids = dojo.map(workflows.workflowDefinitions, function(def){
				return def.id;
			});
			var workflow;
			for(var i=0, len=ids.length; i<len; i++){
				workflow = this.getWorkflowById(ids[i]);
				if(!workflow.startActionId || workflow.startActionId.length===0){
					var name = workflow.name || ids[i];
					return dojo.string.substitute(messages.ErrNoStartAction, [ name ]);
				}
			}
		}
		return null;
	},
	
	getStateGroups: function(){
		var workflows = this.workflows;
		if(workflows){
			var stateGroupDefinitions = workflows.stateGroupDefinitions;
			if(stateGroupDefinitions && stateGroupDefinitions.length > 0){
				return dojo.map(workflows.stateGroupDefinitions, function(def){
					return {
						id: def.id,
						name: def.name
					};
				});
			}
		}
		return defaultStateGroups;
	},
	
	//
	getWorkflowById: function(workflowId){
		if(this.workflows){
			var workflowDefs = this.workflows.workflowDefinitions;
			if(workflowDefs){
				for(var i = 0, l = workflowDefs.length; i < l; i++){
					var workflowDef = workflowDefs[i];
					if(workflowDef && workflowDef.id == workflowId){
						return workflowDef.workflows[0];
					}
				}
			}
		}
		return null; // not found 
	},
	
	addWorkflow: function(id, name){
		if(this.workflows){
			if(!this.workflows.workflowDefinitions){
				this.workflows.workflowDefinitions = [];
			}
			this.workflows.workflowDefinitions.push({
				_eQualifiedClassName: "com.ibm.team.workitem.process.dto:WorkflowDefinition",
				id: id,
				workflows: [
					{
						_eQualifiedClassName: "com.ibm.team.workitem.process.dto:Workflow",
						name: name,
						actions: [],
						states: [],
						resolutions: []
					}
				]
			});
			this.onWorkflowChanged();
		}
	},
	
	removeWorkflow: function(workflowId){
		if(this.workflows){
			var workflowDefs = this.workflows.workflowDefinitions;
			if(workflowDefs){
				for(var i = workflowDefs.length-1; i >= 0; i--){
					var workflowDef = workflowDefs[i];
					if(workflowDef && workflowDef.id == workflowId){
						workflowDefs.splice(i, 1);
						this.onWorkflowChanged();
					}
				}
			}
		}
	},

	setWorkflowProperty: function(workflowId, key, value){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			workflow[key] = value;
			this.onWorkflowChanged();
		}
	},
	
	//
	getStateById: function(workflowId, stateId){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			var states = workflow.states;
			for(var i=states.length-1; i>=0; i--){
				if(states[i].id == stateId){
					return states[i];
				}
			}
		}
		return null;
	},
	//
	onWorkflowChanged: function(args){
		this._workflowChanged = true;
	},
	
	setStateProperty: function(workflowId, stateId, key, value){
		var state = this.getStateById(workflowId, stateId);
		if(state){
			if(dojo.isString(key)){
				state[key] = value;
			}else if(dojo.isObject(key)){
				// 3rd argument is an object in the form of
				// { key1: value1, key2: value2,... }
				for(var i in key){
					state[i] = key[i];
				}
			}
			this.onWorkflowChanged(workflowId);
		}
	},
	
	setStateAction: function(workflowId, stateId, prevActionId, actionId){
		var state = this.getStateById(workflowId, stateId);
		if(state){
			if(!state.actionIds){
				state.actionIds = [];
			}
			
			// remove previous state action
			var index;
			if(prevActionId && prevActionId.length){
				index = dojo.indexOf(state.actionIds, prevActionId);
				if(index != -1){
					state.actionIds.splice(index, 1);
				}
			}

			// add new state action
			if(actionId && actionId.length > 0){
				index = dojo.indexOf(state.actionIds, actionId);
				if(index == -1){
					state.actionIds.push(actionId);
				}
			}
			this.onWorkflowChanged(workflowId);
		}
	},
	
	removeStateAction: function(workflowId, stateId, actionId){
		var state = this.getStateById(workflowId, stateId);
		if(state){
			if(state.actionIds && state.actionIds.length > 0){
				var index = dojo.indexOf(state.actionIds, actionId);
				if(index != -1){
					state.actionIds.splice(index, 1);
					this.onWorkflowChanged(workflowId);
				}
			}
		}
	},

	addState: function(workflowId, state){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			if(!workflow.states){
				workflow.states = [];
			}
			state._eQualifiedClassName = "com.ibm.team.workitem.process.dto:State";
			workflow.states.push(state);
			this.onWorkflowChanged(workflowId);
		}
	},
	
	removeState: function(workflowId, stateId){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			var states = workflow.states;
			var removedActionIds = [];
			var bRemoved = false;
			for(var i=states.length-1; i>=0; i--){
				if(states[i].id == stateId){
					states.splice(i, 1);
					removedActionIds = this.removeStatesAssociatedActions(workflow, stateId);
					bRemoved = true;
					break;
				}
			}
			// 128383: "null" shown in Workflow actions after removing a state 
			if(bRemoved){
				for(i=states.length-1; i>=0; i--){
					var actionIds = states[i].actionIds;
					if(actionIds){
						for(var j=actionIds.length-1; j>=0; j--){
							if(dojo.indexOf(removedActionIds, actionIds[j]) >= 0){
								actionIds.splice(j,1);
							}
						}
					}
				}
				this.onWorkflowChanged(workflowId);
			}
		}
	},
	
	// Method should only be called as a sub method of one that has already
	// checked the workflow. It doesn't call removeAction as it would just be
	// duplicating work.
	removeStatesAssociatedActions: function(workflow, stateId){
		var actionIds = [];
		if (workflow.actions){
			var actions = workflow.actions;
			for (var i = actions.length-1; i>=0; i--) {
				if (actions[i].state == stateId){
					actionIds.push(actions[i].id);
					actions.splice(i, 1);
				}
			}
		}
		return actionIds;
	},
	
	addAction: function(workflowId, action){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			if(!workflow.actions){
				workflow.actions = [];
			}
			action._eQualifiedClassName = "com.ibm.team.workitem.process.dto:Action";
			workflow.actions.push(action);
			this.onWorkflowChanged(workflowId);
		}
	},
	
	removeAction: function(workflowId, actionId){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			var actions = workflow.actions;
			for(var i=actions.length-1; i>=0; i--){
				if(actions[i].id == actionId){
					actions.splice(i, 1);
					this.onWorkflowChanged(workflowId);
					break;
				}
			}
		}
	},
	
	getActionProperty: function(workflowId, actionId, key){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			var actions = workflow.actions;
			for(var i=actions.length-1; i>=0; i--){
				if(actions[i].id == actionId){
					return actions[i][key];
				}
			}
		}
		return null;
	},
	
	setActionProperty: function(workflowId, actionId, key, value){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			var actions = workflow.actions;
			for(var i=actions.length-1; i>=0; i--){
				if(actions[i].id == actionId){
					if(dojo.isString(key)){
						actions[i][key] = value;
					}else if(dojo.isObject(key)){
						// 3rd argument is an object in the form of
						// { key1: value1, key2: value2,... }
						for(var j in key){
							actions[i][j] = key[j];
						}
					}
					this.onWorkflowChanged(workflowId);
					break;
				}
			}
		}
	},
	
	getAllResolutions: function(workflowId){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow && workflow.resolutions){
			return workflow.resolutions;
		}
		return [];
	},
	
	addResolution: function(workflowId, resolution){
		var workflow = this.getWorkflowById(workflowId);
		if(workflow){
			if(!workflow.resolutions){
				workflow.resolutions = [];
			}
			resolution._eQualifiedClassName = "com.ibm.team.workitem.process.dto:Resolution";
			workflow.resolutions.push(resolution);
			this.onWorkflowChanged(workflowId);
		}
	},
	
	removeResolution: function(workflowId, resolutionId){
		var resolutions = this.getAllResolutions(workflowId);
		for(var i=resolutions.length-1; i>=0; i--){
			if(resolutions[i].id == resolutionId){
				resolutions.splice(i, 1);
				this.onWorkflowChanged(workflowId);
				break;
			}
		}
	},
	
	getResolution: function(workflowId, resolutionId){
		var resolutions = this.getAllResolutions(workflowId);
		for(var i=resolutions.length-1; i>=0; i--){
			if(resolutions[i].id == resolutionId){
				return resolutions[i];
			}
		}
		return null;
	},
	
	setResolutionProperty: function(workflowId, resolutionId, key, value){
		var resolution = this.getResolution(workflowId, resolutionId);
		if(resolution){
			if(dojo.isString(key)){
				resolution[key] = value;
			}else if(dojo.isObject(key)){
				// 3rd argument is an object in the form of
				// { key1: value1, key2: value2,... }
				for(var j in key){
					resolution[j] = key[j];
				}
			}
			this.onWorkflowChanged(workflowId);
		}
	},
	
	setWorkflowFinal: function(isFinal){
		if(this.workflows){
			this.workflows.isFinal = isFinal;
			this.onWorkflowChanged();
		}
	},
	
	getWorkflowPostData: function(){
		if(this._workflowChanged === true && this.workflows){
			return dojo.toJson(this.workflows);
		}
		return null;
	}
});

})();