/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.SecondIconPreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.SwapListDialog");
dojo.require("jazz.util.html");
dojo.require("jazz.app.context");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {

var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

var _defCache = {};
var _labelCache = {};

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.SecondIconPreferenceProvider",
	CustomPreferenceProvider, {
		
	constructor: function(args) {
		this.viewlet = args.viewlet;
	},
		
	getLabel: function(value){
		var projectAreaItemId = this.projectAreaItemId;
		if (!projectAreaItemId) {
			projectAreaItemId = this.viewlet.getPreference("queryCategory");
			if (projectAreaItemId) {
				projectAreaItemId = projectAreaItemId.split("|")[0];
			} else if (this.viewlet.getPreference("defaultQuery") != -1) {
				// if there was no query category and there's a default query, we can assume current project
				projectAreaItemId = jazz.app.context.get().itemId;
			}
		}
		var label = "";
		if (projectAreaItemId) {
			if (!_labelCache[projectAreaItemId]) {
				_labelCache[projectAreaItemId] = "loading";
				this._fetchAttrs(projectAreaItemId).addCallback(dojo.hitch(this,function(options) {
					_labelCache[projectAreaItemId] = {};
					for (var i = 0; i < options.length; i++) {
						_labelCache[projectAreaItemId][options[i].id] = options[i].label;
					}
				}));
			}
			if (_labelCache[projectAreaItemId] === "loading") {
				return null;
			}
			value = this.viewlet._getSelectedIconsArray(value);
			for (var i = 0; i < value.length; i++) {
				label += (label ? ", " : "") + _labelCache[projectAreaItemId][value[i]];
			}
		}
		return this._buildLabel(label || messages.none);
	},
	
	getLoadingLabel: function() {
		return this._buildLabel(messages.loading);
	},
	
	_buildLabel: function (text) {
		return "<input type='text' readonly='readonly' value='" + jazz.util.html.escape(text).replace(/'/g,"&#39;") + "'/>";
	},

	createChooser: function(onOk, settings, viewlet) {
		var projectAreaItemId = settings.preferences.queryCategory;
		if (projectAreaItemId) {
			projectAreaItemId = projectAreaItemId.split("|")[0];
		} else if (settings.preferences.defaultQuery != -1) {
			// if there was no query category and there's a default query, we can assume current project
			projectAreaItemId = jazz.app.context.get().itemId;
		}
		// save the projectAreaItemId since we don't have as much info at getLabel time
		this.projectAreaItemId = projectAreaItemId;
		if (projectAreaItemId) {
			var dialog = new jazz.ui.SwapListDialog({
				dialogTitleText: messages.secondIconTitle,
				selectedListTitleText: messages.secondIconSelected,
				unselectedListTitleText: messages.secondIconUnselected,
				sortSelected: true,
				onOK: function(ref) {
					var results = ref.getSelectedEntries();
					for (var i = 0; i < results.length; i++) {
						results.splice(i,1,results[i].id);
					}
					onOk(dojo.toJson(results),"secondIcon");
				}
			});
			dialog.show();
			dialog.showLoading(true);
			this._fetchAttrs(projectAreaItemId).addCallback(dojo.hitch(this,function(options) {
				this._populateDialog(dialog,options,viewlet._getSelectedIconsArray(settings.preferences.secondIcon));
				return options;
			}))
		} else {
			alert("You must select a query first.");
		}
		
	},
	
	_populateDialog: function(dialog, options, selected) {
		var map = {};
		for (var i = 0; i < selected.length; i++)
			map[selected[i]] = i;
		selected = [];
		var unselected = [];
		for (var i = 0; i < options.length; i++) {
			var option = options[i];
			if (typeof map[option.id] === "number")
				selected[map[option.id]] = option;
			else
				unselected.push(option);
		}
		dialog.setSelectedEntries(selected);
		dialog.setUnselectedEntries(unselected);
		dialog.showLoading(false);
	},
	
	_fetchAttrs: function(projectAreaItemId) {
		var deferred = _defCache[projectAreaItemId];
		if (!deferred) {
			var deferred = new dojo.Deferred();
			_defCache[projectAreaItemId] = deferred;
			deferred.addCallback(function(response) {
				var options = [];
				var length = response.length;
				for (var i = 0; i < length; i++) {
					var current = response[i];
					if (current.id !== WorkItemAttributes.TYPE) {
						options.push({
							label: current.label,
							id: current.externalId
						});
					}
				}
				return options;
			});
			deferred.addErrback(function(error) {
				console.error(error);
			});
			var srh = new ServiceResponseHandler(deferred, "callback", "errback");
			WorkItemClient.getAttributes(srh, {
				projectAreaItemId: projectAreaItemId,
				iconsOnly: true
			});
		}
		return deferred;
	}
});

})();
