/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooser");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.string");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.ui.ModeledTree");
dojo.require("jazz.util.html");
dojo.require("jazz.app.context");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");

dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.ErrorUtility");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ErrorUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.ErrorUtility;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooser", null, {
	
	constructor: function(params){
		if (!dojo.isFunction(params.onOk)) {
			throw new Error("Missing function for 'onOk' parameter");
		}
		if (params.scope != DashboardConstants.SCOPE_CONTRIBUTOR && params.scope != DashboardConstants.SCOPE_TEAM_AREA && params.scope != DashboardConstants.SCOPE_PROJECT_AREA) {
			throw new Error("Missing or invalid required parameter 'scope'");
		}
		this._multiSelect = params.multiSelect == null || params.multiSelect; // null == true
		this._onOk = params.onOk;
		this._defaultValue = params.defaultValue ? params.defaultValue : null;
		this._defaultCategory = params.defaultCategory ? params.defaultCategory : null;
		this._suppressCrossRepoQueries = params.suppressCrossRepoQueries || false;
		
		// translate dashboard scope into work items scope
		switch (params.scope) {
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				params.scope = QueryClient.SCOPE_PERSONAL;
				break;
			case DashboardConstants.SCOPE_TEAM_AREA:
				params.scope = QueryClient.SCOPE_TEAM_AREA;
				break;
			case DashboardConstants.SCOPE_PROJECT_AREA:
				params.scope = QueryClient.SCOPE_PROJECT_AREA;
				break;
		}
		var widgetParams= {};		
		widgetParams.primaryTitle = params.title ? params.title : messages.chooserTitle;

		var onOk = dojo.hitch(this, function(queries) {
			this._onOk(queries);
			onCancel();
		});
		var onCancel = dojo.hitch(this, function() {
			this.floatingPane.close();
		});
		var onLoad = dojo.hitch(this, function() {
			this.floatingPane.updatePosition();
		});
		
		this.contentWidget = new com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserContent({onOk: onOk, onCancel: onCancel, onLoad: onLoad, multiSelect: this._multiSelect, scope: params.scope, scopeItemId: params.scopeItemId, defaultValue: this._defaultValue, defaultCategory: this._defaultCategory, suppressCrossRepoQueries: this._suppressCrossRepoQueries});
		widgetParams.contentNode = this.contentWidget.domNode;
		this.floatingPane = new jazz.ui.Dialog(widgetParams);
		this.contentWidget.init();
	}
});

var CATEGORY_INDEX_PROJECT = 0;
var CATEGORY_INDEX_TEAM = 1;

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserContent",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "ui/internal/templates/utils/WorkItemQueryChooser.html"),
	
	constructor: function(params){
		this.ok = messages.ok;
		this.cancel = messages.cancel;
		this.showQueriesFor = messages.showQueriesFor;
		this.description = messages.description;
		this._onOk = params.onOk;
		this._onCancel = params.onCancel;
		this._okButton = params.okButton;
		this._multiSelect = params.multiSelect;
		this._loadedCallback = params.loadedCallback;
		this._scope = params.scope;
		this._scopeItemId = params.scopeItemId;
		this._defaultValue = params.defaultValue;
		this._defaultCategory = params.defaultCategory ? params.defaultCategory.split("|") : [];
		this._suppressCrossRepoQueries = params.suppressCrossRepoQueries;
		this._itemNames = new Object();
		this._result = {};
		this._categoryId = 1;
	},
	
	postCreate: function() {
		if (this._okButton) {
			this._buttonSection.parentNode.removeChild(this._buttonSection);
			this._okCmd = this._okButton;
		} else {
			this.connect(this._okCmd, "onclick", this._doOnOk);
			if (this._onCancel)
				this.connect(this._cancelCmd, "onclick", this._onCancel);
			else
				this._cancelCmd.parentNode.removeChild(this._cancelCmd);
		}
		if (this._multiSelect)
			this._caption.innerHTML = messages.chooserInstructMulti;
		else
			this._caption.innerHTML = messages.chooserInstructSingle;
		this._populatePicker();
	},
	
	onError: function(error) {
		ErrorUtility.logError(error, this.id);
	},
	
	isMultiSelect: function() {
		return this._multiSelect;
	},
	
	init: function() {
		// if team area scope and a default team category is available use it
		// else if default project category is available
		// else use current project area for user scope, or the scope item for project/team scope
		var initialScope = (this._scope == QueryClient.SCOPE_TEAM_AREA && this._defaultCategory[CATEGORY_INDEX_TEAM]) ? this._defaultCategory[CATEGORY_INDEX_TEAM] :
							(this._defaultCategory[CATEGORY_INDEX_PROJECT] || (this._scope == QueryClient.SCOPE_PERSONAL ? jazz.app.context.get().itemId : this._scopeItemId || null));
		if (initialScope)
			this._runQuery(initialScope);
		else
			this._needsInitialScope = true;
	},
	
	treeSelectionChanged: function() {
		var selection = this._tree.getSelection();
		for (var i = 0; i < selection.length; i++) {
			if (selection[i].itemId) {
				var hasValidSelection = true;
				break;
			}
		}
		if (hasValidSelection) {
			this._okCmd.disabled = false;
			if (selection.length === 1) {
				dojo.place(document.createTextNode(selection[0].label),
				           this._itemDescription, 'only');
			} else {
				dojo.empty(this._itemDescription);
			}
		} else {
			this._okCmd.disabled = true;
			dojo.empty(this._itemDescription);
		}
	},
	
	getCurrentProjectArea: function() {
		if (this.areaSelect && this.areaSelect.value)
			return this.areaSelect.value;
	},
	
	getSelection: function() {
		if (this.isMultiSelect()) {
			var result = new Array();
			var treeSelection = this._tree.getSelection();
			for (var i = 0; i < treeSelection.length; i++) {
				var query = this._queriesMap[this._tree.getSelection()[i].getId()];
				if (query)
					result.push(query);
			}
			return result;
		}
		return this._queriesMap[this._tree.getSelection()[0].getId()];
	},
	
	_runQuery: function(scopeItemId) {
		this._currentQuery = scopeItemId;
		if (this._result[scopeItemId]) {
			if (this._result[scopeItemId] == "empty") {
				this._showEmpty();
			}
			else if (this._result[scopeItemId] != "pending") {
				this._tree = this._result[scopeItemId];
				this._showTree();
			}
			return;
		}
		var isTeamScope = this._scope == QueryClient.SCOPE_TEAM_AREA && this._scopeItemId == scopeItemId;
		this._showLoading();
		this._result[scopeItemId] = "pending";
		var serviceObject = {
			self: this,
			scopeItemId: scopeItemId,
			_successCallback: function(/*array*/ scopedQueriesDTOs) {
				var hasQueries = false;
				for (var i = 0; i < scopedQueriesDTOs.length; i++) {
					if (scopedQueriesDTOs[i].queries && scopedQueriesDTOs[i].queries.length > 0) {
						hasQueries = true;
						break;
					}
				}
				if (hasQueries) {
					// for team categories, store the team and the project, so the correct project can be restored if the viewlet is copied elsewhere
					var tree = this.self._createTree(scopedQueriesDTOs, isTeamScope ? jazz.app.context.get().itemId + "|" + this.scopeItemId : this.scopeItemId);
					if (this.self._currentQuery == this.scopeItemId) {
						this.self._tree = tree;
						this.self._showTree();
					}
					this.self._result[this.scopeItemId] = tree;
				} else {
					this.self._showEmpty();
					this.self._result[this.scopeItemId] = "empty";
				}
			},
			_errorCallback: function(error) {
				this.self.onError(error);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		if (this._scope == QueryClient.SCOPE_TEAM_AREA && isTeamScope)
			// don't need to suppress cross repo queries here since we don't get parent scopes for team scope anyway
			QueryClient.getScopedQueries(responseHandler, this._scope, scopeItemId, false, false); 
		else
			QueryClient.getScopedQueries(responseHandler, QueryClient.SCOPE_PROJECT_AREA, null, false, true, scopeItemId, this._suppressCrossRepoQueries);
	},
	
	_showTree: function() {
		this._clearContent();
		this._queryTreeArea.appendChild(this._tree.domNode);
		this.treeSelectionChanged();
		dojo.removeClass(this._queryTreeArea, "loading");
		dojo.removeClass(this._queryTreeArea, "padded");
	},
	
	_showLoading: function() {
		this._clearContent();
		this._queryTreeArea.appendChild(document.createTextNode(messages.loading));
		dojo.addClass(this._queryTreeArea, "loading");
		dojo.addClass(this._queryTreeArea, "padded");
	},
	
	_showEmpty: function() {
		this._clearContent();
		this._queryTreeArea.appendChild(document.createTextNode(messages.noQueriesFound));
		dojo.removeClass(this._queryTreeArea, "loading");
		dojo.addClass(this._queryTreeArea, "padded");
	},
	
	_clearContent: function() {
		while (this._queryTreeArea.firstChild)
			this._queryTreeArea.removeChild(this._queryTreeArea.firstChild);
	},
	
	_populatePicker: function() {
		var serviceObject = {
			self: this,
			_deferredCallback: function() {
				if (this.deferred) {
					this._successCallback(this.deferred);
					this.deferred = false;
				}
			},
			_successCallback: function(/*array*/ projects) {
				projects.sort(function(a,b) {
					return a.name < b.name ? -1 : 1;
				});
				if (this.self._needsInitialScope) {
					this.self._needsInitialScope = false;
					this.self._runQuery(projects[0].itemId);
				}
				var select = document.createElement("select");
				if (this.self._scope == QueryClient.SCOPE_PERSONAL || this.self._scope == QueryClient.SCOPE_PROJECT_AREA || !this.self._scopeItemId) {
					// if the scope is personal/project use the default if one exists, otherwise use current project area
					// also, if there is no scope item in team scope (remote viewlet), use the default project
					var selected = this.self._defaultCategory[CATEGORY_INDEX_PROJECT] || jazz.app.context.get().itemId;
				}
				else {
					if (!this.self._itemNames[this.self._scopeItemId]) {
						// if the scope is the team area and we have not retrieved the name yet, wait for now
						this.deferred = projects;
						return;
					} else {
						// if there is not a default team category or it doesn't match this team
						// select the default project category if it exists
						// if the default project category doesn't exist, default to the team category
						if (this.self._defaultCategory[CATEGORY_INDEX_PROJECT] &&
						(!this.self._defaultCategory[CATEGORY_INDEX_TEAM] || this.self._defaultCategory[CATEGORY_INDEX_TEAM] != this.self._scopeItemId)) 
							var selected = this.self._defaultCategory[CATEGORY_INDEX_PROJECT] || jazz.app.context.get().itemId;
						var option = document.createElement("option");
						option.value = this.self._scopeItemId;
						if (!selected) 
							option.selected = "selected";
						option.appendChild(document.createTextNode(this.self._itemNames[this.self._scopeItemId]));
						select.appendChild(option);
						var sep = document.createElement("optgroup");
						sep.label = "--------------------";
						select.appendChild(sep);
					}
				}
				// populate the projects and honor the selection if necessary
				for (var i = 0; i < projects.length; i++) {
					var option = document.createElement("option");
					option.value = projects[i].itemId;
					if (selected == projects[i].itemId)
						option.selected = "selected";
					option.appendChild(document.createTextNode(projects[i].name));
					select.appendChild(option);
				}
				this.self._areaPicker.innerHTML = "";
				this.self._areaPicker.appendChild(select);
				this.self.connect(select, "onchange", this.self._areaPickerChanged);
				this.self.areaSelect = select;
				if (this.self.onLoad)
					this.self.onLoad();
			},
			_errorCallback: function(error) {
				this.self.onError(error);
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		ProcessClient.getAllProjectAreas(responseHandler);
		if (this._scope == QueryClient.SCOPE_TEAM_AREA && this._scopeItemId && !this._itemNames[this._scopeItemId]) {
			var serviceObject2 = {
				_self: this,
				serviceObject: serviceObject,
				_successCallback: function(team){
					this._self._itemNames[team.itemId] = team.name;
					this.serviceObject._deferredCallback();
				},
				_errorCallback: function(error){
					this.self.onError(error);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObject2, "_successCallback", "_errorCallback");
			ProcessClient.getTeamAreaByUUID({uuid: this._scopeItemId}, responseHandler);
		}
		var html = ["<select disabled='disabled'><option value='loading' selected='true'>Loading...</option></select>"];
		this._areaPicker.innerHTML = html.join("<select><option value='loading'>Loading...</option></select>");
	},
	
	_areaPickerChanged: function(e) {
		this._runQuery(e.target.value);
	},
	
	_doOnOk: function(event) {
		this._onOk(this.getSelection());
	},
	
	_createTree: function(scopedQueriesDTOs, queryCategory) {
		var dataStore = this._createDataStore(scopedQueriesDTOs, queryCategory);
		this._tree = new com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserTree({
			model: dataStore,
			chooser: this,
			selection: this.isMultiSelect() ? "multi" : true
		});
		if (this._defaultValue && this._defaultItemValue){
			this._tree.expandTo(this._defaultValue, function(obj) {return obj.itemId || obj.label;});
			this._tree.setSelection(this._tree.getItem(this._defaultItemValue));
		}
		return this._tree;
	},
	
	_createDataStore: function(scopedQueriesDTOs, queryCategory) {
		var model = new Array();
		var crossRepoCat, personalCat;
		if (!this._queriesMap)
			this._queriesMap = {};
		for (var i = scopedQueriesDTOs.length-1; i >= 0; i--) {
			var category = this._createCategory(scopedQueriesDTOs[i], queryCategory);
			if (category.children.length > 0) {
				if (scopedQueriesDTOs[i].scope == QueryClient.SCOPE_CROSS_REPOSITORY)
					crossRepoCat = category;
				else if (scopedQueriesDTOs[i].scope == QueryClient.SCOPE_PERSONAL)
					personalCat = category;
				else
					model.push(category);
			}
		}
		model.reverse();
		// put cross repo category last
		if (crossRepoCat)
			model.push(crossRepoCat);
		// put personal category first
		if (personalCat)
			model.splice(0,0,personalCat);
		return model;
	},
	
	_createCategory: function (scopedQueriesDTO, queryCategory) {
		var category = new Object();
		category.label = this._getCategoryName(scopedQueriesDTO);
		category.iconClass = this._getCategoryClass(scopedQueriesDTO);
		var children = new Array();
		for (var i in scopedQueriesDTO.queries) {
			children.push({	label: jazz.util.html.escape(scopedQueriesDTO.queries[i].name), itemId: scopedQueriesDTO.queries[i].itemId, iconClass: "query", categoryId: this._categoryId });
			scopedQueriesDTO.queries[i].queryCategory = queryCategory;
			this._queriesMap[scopedQueriesDTO.queries[i].itemId + this._categoryId] = scopedQueriesDTO.queries[i];
			if (!this._defaultItemValue && this._defaultValue === scopedQueriesDTO.queries[i].itemId)
				this._defaultItemValue = this._defaultValue + this._categoryId;
		}
		category.children = children.sort(function(a, b) {
			return a.name < b.name ? -1 : a.name > b.name ? 1 : 0;
		});
		this._categoryId++;
		return category;
	},
	
	_getCategoryName: function (scopedQueriesDTO) {
		switch (scopedQueriesDTO.scope) {
			case QueryClient.SCOPE_PERSONAL:
				return messages.chooserHeadingPersonal;
			case QueryClient.SCOPE_OTHER_CONTRIBUTORS:
				return messages.chooserHeadingShared;
			case QueryClient.SCOPE_TEAM_AREA:
				return dojo.string.substitute(messages.chooserHeadingTeam, [ scopedQueriesDTO.scopeName ]);
			case QueryClient.SCOPE_PROJECT_AREA:
				return messages.chooserHeadingProject;
			default:
				return scopedQueriesDTO.scopeName;
		}
	},
	
	_getCategoryClass: function(scopedQueriesDTO) {
		if (scopedQueriesDTO.scope == QueryClient.SCOPE_TEAM_AREA)
			return "teamFolder";
		return "simpleFolder";
	},
	destroy: function() {
		for (var i in this._result) {
			if (this._result[i] != "pending" && this._result[i].destroy)
				this._result[i].destroy();
		}
		this.inherited(arguments);
	}
});


dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserTree", jazz.ui.ModeledTree, {
	createTreeItem: function(obj) {
		return new com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserTreeItem(obj)
	},
	_doSelection: function(e, item) {
		this.inherited(arguments);
		this.chooser.treeSelectionChanged();
	},
	onDblClick: function(item) {
		if (item.isLeaf())
			this.chooser._doOnOk();
		this.inherited(arguments);
	}
});


dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooserTreeItem", jazz.ui.tree.IconItem, {
	getId: function() {
		return (this.itemId || this.label) + (this.categoryId || "");
	}
});

})();
