/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooser");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility");
dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {

var CustomPreferenceProvider = com.ibm.team.dashboard.web.ui.CustomPreferenceProvider;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemQueryChooser = com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryChooser;
var UUIDUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility;

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider",
	CustomPreferenceProvider, {
		
	constructor: function(args) {
		this._labelMap = new Array();
		this._fetchingMap = new Array();
		this._prefId = args.prefId;
		this._queryCategoryPrefId = args.queryCategoryPrefId;
		this._suppressCrossRepoQueries = args.suppressCrossRepoQueries || false;
	},
	
	initDefault: function(queryDTO) {
		this._cacheLabel(queryDTO);
		this._defaultValue = queryDTO.itemId;
	},
		
	getLabel: function(value){
		var label = this.getTextLabel(value);
		if (label != "")
			return this._buildLabel(label);
		this._fetchLabel(value);
		return null;
	},

	getTextLabel: function(value) {
		// take a substring of the UUID because calm queries have two uuids appended together
		if (!value || !UUIDUtility.isValidUUID(value.substring(0,23))) {
			if (this._defaultValue)
				value = this._defaultValue;
			else
				return messages.providerNoQuery;
		}
		if (value in this._labelMap)
			return this._labelMap[value];
		return "";
	},
	
	// gets the name of the indicated query
	// flushes the cache for this query and updates it upon retrieving the result
	// the provided callback will be called with the result
	getQueryName: function(value, callback) {
		// if the value is invalid, ignore the call
		// take a substring of the UUID because calm queries have two uuids appended together
		if (!value || !UUIDUtility.isValidUUID(value.substring(0,23)))
			return;
		delete this._labelMap[value];
		if (!this._cacheCallbacks)
			this._cacheCallbacks = {};
		if (!this._cacheCallbacks[value])
			this._cacheCallbacks[value] = [];
		this._cacheCallbacks[value].push(callback);
		this._fetchLabel(value);
	},
	
	getLoadingLabel: function() {
		return this._buildLabel(messages.loading);
	},
	
	_buildLabel: function (text) {
		return "<input type='text' readonly='readonly' value='" + jazz.util.html.escape(text).replace(/'/g,"&#39;") + "'/>";
	},

	createChooser: function(onOk, settings, viewlet) {
		// Get scope
		var scope = viewlet.getScope() || DashboardConstants.SCOPE_CONTRIBUTOR;
		// Get scope item ID
		var scopeItemId = viewlet.getScopeItem();
		scopeItemId = scopeItemId ? scopeItemId.itemId : null;
		// Get current value
		var currentValue = settings.preferences[this._prefId];
		if ((!currentValue || !UUIDUtility.isValidUUID(currentValue.substring(0,23))) && this._defaultValue)
			currentValue = this._defaultValue;
		
		var params = {
			scope: scope,
			scopeItemId: scopeItemId,
			onOk: dojo.hitch(this,function(queryDTO) {
				this._cacheLabel(queryDTO);
				onOk(queryDTO.itemId, this._prefId);
				if (this._queryCategoryPrefId)
					onOk(queryDTO.queryCategory, this._queryCategoryPrefId);
			}),
			multiSelect: false,
			defaultValue: currentValue,
			defaultCategory: settings.preferences[this._queryCategoryPrefId],
			suppressCrossRepoQueries: this._suppressCrossRepoQueries
		};
		new WorkItemQueryChooser(params);
	},
	
	_fetchLabel: function (itemId) {
		
		if (itemId in this._fetchingMap)
			return;
		this._fetchingMap[itemId] = true;
		var self = this;
		var serviceObject = {
			_successCallback: function(/*array*/ queryDTO) {
				if (queryDTO == null)
					self._cacheLabel({itemId: itemId, name: messages.providerNoQuery});
				else
					self._cacheLabel(queryDTO);
			},
			_errorCallback: function(error) {
				if (error && error.status == 403) {
					self._cacheLabel({itemId: itemId, name: messages.providerPermissionDenied});
				}
				else {
					console.error(error);
				}
			}
		};
		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		// fake project area item id to trick the client
		QueryClient.getQuery2(responseHandler, {itemId: itemId, projectAreaItemId: true});
	},
	
	_cacheLabel: function(queryDTO) {
		delete this._fetchingMap[queryDTO.itemId];
		this._labelMap[queryDTO.itemId] = queryDTO.name;
		if (this._cacheCallbacks && this._cacheCallbacks[queryDTO.itemId]) {
			for (var i = 0; i < this._cacheCallbacks[queryDTO.itemId].length; i++)
				this._cacheCallbacks[queryDTO.itemId][i](queryDTO.name);
			delete this._cacheCallbacks[queryDTO.itemId];
		}
	}
});

})();
