/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var commonMessages;
var PROJECT_AREA_EDITOR = "com.ibm.team.process.editProjectArea"; //$NON-NLS-1$

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor",
	[ dijit._Widget, dijit._Templated ],
{
	model: null,
	pageCshId: null,
	
	postMixInProperties: function(){
		commonMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages");
		this.commonMessages = commonMessages; // so that it can refered from a template
		this.inherited(arguments);
	},

	postCreate: function(){
		// setup CSH
		this._childWidgets = [];
		if(this.pageCshId && this.csHelpNode){
			this._childWidgets.push(new jazz.app.ua.PageHoverHelp({
				uaKey: this.pageCshId
			}, this.csHelpNode));
		}
		
		this.inherited(arguments);
		
		// setup final checkbox
		var finalContainer = this.finalContainer;
		if(finalContainer){
			dojo.addClass(finalContainer, "finalContainer");
			var checkboxId = this.id+"_finalCheckbox";
			this.finalCheckbox = dojo.create("input", {
				id: checkboxId,
				type: "checkbox",
				onclick: dojo.hitch(this, "_handleFinalChanged")
			}, finalContainer);
			finalContainer.appendChild(dojo.doc.createTextNode(" "));
			dojo.create("label", {
				"for": checkboxId,
				className: "dijitInline formLabel",
				innerHTML: commonMessages.FinalCheckboxLabel
			}, finalContainer);
		}
	},
	
	toggleMainSection: function(bShow, /*String|DOM node*/msg){
		// If bShow=true, displays the main section and hides the messages section
		// If bShow=false, hides main the section and displays the messages section
		// message is a string to be displayed in the message section when bShow=false
		if(this.mainSection){
			this.mainSection.style.display = (bShow) ? "" : "none";
		}
		if(this.msgSection){
			this.msgSection.innerHTML = "";
			if(msg){
				if(dojo.isString(msg)){
					msg = dojo.doc.createTextNode(msg);
				}
				this.msgSection.appendChild(msg);
			}
			this.msgSection.style.display = (bShow) ? "none" : "";
		}
		
		if(this.finalContainer){
			this.finalContainer.style.display = bShow ? "" : "none";
		}
	},

	setLoading: function(isLoading){
		this.toggleMainSection(!isLoading, dojo.create("div", { className:"loadingNode", innerHTML:commonMessages.Loading }));
	},

	processCommonConfigData: function(configData){
		// processes data that is common across configuration data points, e.g. final
		var processProviderItemId = configData ? configData.processProviderItemId : null;
		var processProviderName = configData ? configData.processProviderName : null;
		this._showInheritedWarning(processProviderItemId, processProviderName);
		var isProcessProviderFinal = configData ? configData.isProcessProviderFinal : false;
		this._showFinalWarningMessage(isProcessProviderFinal, processProviderItemId, processProviderName);

		if(this.finalCheckbox){
			this.finalCheckbox.checked = (configData.isFinal === true);
		}
	},

	_getMasterProjectAnchor: function(processProviderItemId, processProviderName){
		var url = "#action=" + PROJECT_AREA_EDITOR + "&itemId=" + processProviderItemId; //$NON-NLS-1$ //$NON-NLS-2$
		return '<a href="'+url+'" title="'+processProviderName+'">'+processProviderName+'</a>';
	},

	_showInheritedWarning: function(processProviderItemId, processProviderName){
		var node = null;
		var isInherited = (processProviderItemId && processProviderItemId.length > 0 &&
			processProviderName && processProviderName.length > 0);
		if(isInherited){
			node = dojo.create("div");
			var anchorStr = this._getMasterProjectAnchor(processProviderItemId, processProviderName);
			dojo.create("div", {
				style: { marginBottom:"10px" },
				innerHTML: dojo.string.substitute(commonMessages.WarningInherited, [anchorStr])
			}, node);
			dojo.create("a", {
				href: "#",
				innerHTML: commonMessages.CustomizeAnchorText,
				title: commonMessages.CustomizeAnchorText,
				onclick: dojo.hitch(this, function(e){
					dojo.stopEvent(e);
					this.toggleMainSection(true);
				})
			}, node);
		}
		this.toggleMainSection(!isInherited, node);
	},
	
	_handleFinalChanged: function(e){
		console.debug("Subclass must override");
	},
	
	displayMessage: function(msg, cssClass){},
	
	displayErrorMessage: function(msg){
		this.displayMessage(msg, "errorMsg");
	},

	_showFinalWarningMessage: function(isProcessProviderFinal, processProviderItemId, processProviderName){
		if(isProcessProviderFinal){
			if(!this._finalWarning){
				var div = dojo.create("div", {
					className: "warningMessage"
				});
				this._finalWarning = div;
				dojo.place(div, this.domNode, "first");
				dojo.create("img", {
					src: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/page/templates/warning.gif").toString(),
					className: "dijitInline",
					alt: "",
					title: ""
				}, div);
				var anchorStr = this._getMasterProjectAnchor(processProviderItemId, processProviderName);
				dojo.create("span", {
					innerHTML: "&nbsp;"+dojo.string.substitute(commonMessages.warnFinalIgnored, [anchorStr])
				}, div);
			}
			this._finalWarning.style.display = "";
		}else if(this._finalWarning){
			this._finalWarning.style.display = "none";
		}
	},

	destroy: function(preserveDom){
		dojo.forEach(this._childWidgets, function(widget){
			widget.destroy(preserveDom);
		});
		this.inherited(arguments);
	}
});

})();