/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "DoubleListMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var _setSelect = function(sel, arr){
	var options = sel.options;
	options.length = 0;
	dojo.forEach(arr, function(item, idx){
		if(dojo.isString(item)){
			options[idx] = new Option(item, item);
			options[idx].title = item;
		}else{
			// item is an object in the following form :
			// { label: ..., value:... }
			options[idx] = new Option(item.label, item.value);
			options[idx].title = item.label;
		}
	});
};

var _moveSelectedOptions = function(select1, select2){
	// moves selected options from select1 to select2
	var options1 = select1.options;
	var options2 = select2.options;
	var index, opt;
	for(var i=options1.length-1; i>=0; i--){
		if(options1[i].selected === true){
			opt = options1[i].cloneNode(true);
			select2.appendChild(opt);
			select1.remove(i);
			index = i;
		}
	}
	
	// update selection
	select2.selectedIndex = options2.length-1;
	var len = options1.length;
	if(len > 0){
		var selectIndex = (index === len) ? (len-1) : index;
		options1[selectIndex].selected = true;
	}
};

var _moveAllOptions = function(select1, select2){
	// moves all options from select1 to select2
	var options1 = select1.options;		
	var options2 = select2.options;
	var len = options2.length;
	dojo.forEach(options1, function(opt, idx){
		options2[len+idx] = new Option(opt.text, opt.value);
		options2[len+idx].title = opt.text;
	});
	options1.length = 0;
};

var _reorderOption = function(select, targetIndex){
	// re-orders selected option to targetIndex
	var options = select.options;
	var len = options.length;
	if(targetIndex >= 0 && targetIndex < len){
		var selectedIndex = select.selectedIndex;
		var opt = options[selectedIndex].cloneNode(true);
		select.remove(selectedIndex);
		if(targetIndex === len-1){
			select.appendChild(opt);
		}else{
			select.insertBefore(opt, options[targetIndex]);
		}
		opt.selected = true;
	}
};

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox",
	[ dijit._Widget, dijit._Templated ],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/common/templates/DoubleListBox.html"),
	
	orderOnly: false, // if true, it will not show choice list, add/remove buttons, and only allows to order from value list
	choiceList: [],
	valueList: [],

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "DoubleListMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this.setChoiceList(this.choiceList);
		this.setValueList(this.valueList);
		var self = this;
		this.connect(this.choiceSelect, "onkeypress", function(e){
			if(e.keyCode == dojo.keys.ENTER){
				self._handleAddClicked();
			}
		});
		this.connect(this.valueSelect, "onkeypress", function(e){
			if(e.keyCode == dojo.keys.ENTER){
				self._handleRemoveClicked();
			}
		});
		
		if(this.orderOnly === true){
			dojo.addClass(this.containerRow, "orderOnly");
		}
	},
	
	setChoiceList: function(/*Array*/ choices){
		_setSelect(this.choiceSelect, choices);
	},
	
	setValueList: function(/*Array*/ values){
		_setSelect(this.valueSelect, values);
	},
	
	_handleAddAllClicked: function(e){
		_moveAllOptions(this.choiceSelect, this.valueSelect);
	},
	
	_handleAddClicked: function(e){
		_moveSelectedOptions(this.choiceSelect, this.valueSelect);
	},

	_handleRemoveClicked: function(e){
		if(!this.orderOnly){
			_moveSelectedOptions(this.valueSelect, this.choiceSelect);
		}
	},

	_handleRemoveAllClicked: function(e){
		_moveAllOptions(this.valueSelect, this.choiceSelect);
	},
	
	_handleChoiceSelected: function(e){
		var options = this.choiceSelect.options;
		for(var i=options.length-1; i>=0; i--){
			if(options[i].selected === true){
			}
		}
	},
	
	_handleTopClicked: function(e){
		_reorderOption(this.valueSelect, 0);
	},

	_handleUpClicked: function(e){
		var select = this.valueSelect;
		var index = select.selectedIndex;
		if(index>0){
			_reorderOption(select, index-1);
		}
	},
	
	_handleDownClicked: function(e){
		var select = this.valueSelect;
		var index = select.selectedIndex;
		if(index < select.options.length-1){
			_reorderOption(select, index+1);
		}
	},
	
	_handleBottomClicked: function(e){
		var select = this.valueSelect;
		_reorderOption(select, select.options.length-1);
	},
	
	getValues: function(){
		return dojo.map(this.valueSelect.options, function(opt){
			return opt.value;
		});
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox.openDialog = function(prop){
	// create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox({
		orderOnly: prop.orderOnly,
		choiceList: prop.choiceList,
		valueList: prop.valueList
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: prop.title,
		dialogContent: contentWidget,
		width: "680px",
		onOK: prop.onOK
	});
};

})();