/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.data.util.sorter");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.List");
dojo.require("com.ibm.team.workitem.web.process.client.internal.WorkItemConfigClient");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "IconSelectionMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var UPLOAD_SERVICE_URI   = "com.ibm.team.workitem.service.process.internal.rest.IProcessAttachmentRestService/";
var WorkItemConfigClient = com.ibm.team.workitem.web.process.client.internal.WorkItemConfigClient;
var CLASS_BLANK = "dijitInline iconNode blankIcon";
var basicComparator = dojo.data.util.sorter.basicComparator;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent",
	[dijit._Widget, dijit._Templated],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/common/templates/IconSelectionContent.html"),
	
	model: null,
	selectedPath: null,
	prefix: "/workitemtype",

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "IconSelectionMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		var self = this;
		var selectedIcon = self.selectedIcon;
		this.connect(this.iconListContainer, "onSelected", function(selected){
			selectedIcon.innerHTML = "";
			if(selected){
				dojo.create("img", {
					src: selected.iconUrl,
					title: selected.title
				}, selectedIcon);
			}else{
				dojo.create("span", {
					className: CLASS_BLANK,
					title: messages.NoIcon
				}, selectedIcon);
			}
		});
		
		// add icons
		this._populateIcons();
	},
	
	_populateIcons: function(){
		if(!this.model){ return; }
		
		var iconListContainer = this.iconListContainer;
		iconListContainer.clearItems();
		var selectedPath = this.selectedPath;
		this.model.getProcessAttachments(this.prefix, function(configData){
			if(!configData.processAttachments){ return; }
			var attachments = configData.processAttachments.sort(function(o1, o2){
				return basicComparator(o1.displayName.toLowerCase(), o2.displayName.toLowerCase());
			});
			dojo.forEach(attachments, function(attachment){
				var iconUrl = attachment.url;
				var item = iconListContainer.addItem({
					iconUrl: iconUrl,
					title: attachment.displayName,
					value: attachment.path
				});
				if(selectedPath && attachment.path == selectedPath){
					iconListContainer.selectItem(item);
				}
			});
		});
	},
	
	_clearIcon: function(){
		this.selectedPath = null;
		this.iconListContainer.selectItem(null);
	},
	
	_performUpload: function(e){
		if(e){
			dojo.stopEvent(e);
		}
	
		var self = this;

		if(this._currentBindRequest){
			this._currentBindRequest.cancel();
			this._currentBindRequest = null;
		}
		
		var setUploading = function(isUploading){
			self.fileInput.style.display = (isUploading) ? "none" : "";
			self.uploadingMsg.style.display = (isUploading) ? "" : "none";
		};
		setUploading(true);
		dojo["require"]("dojo.io.iframe");
		this._currentBindRequest = dojo.io.iframe.send({
			form: this._uploadForm,
			url: TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + WorkItemConfigClient.getCurrentProjectArea().projectAreaItemId+"&prefix="+this.prefix,
			content: {
				mimetype: "text/html",
				fileFields: "uploadFileInput"
			},
			handleAs: "json",
			error: function(response, ioArgs) {
				if(response.dojoType && response.dojoType == "cancel") {
					return; // this is a cancelled request, ignore it
				}
				self._currentBindRequest  = null; // no need to cancel it any more
				alert(messages.msgAttachmentUploadError);
				return;
			},
			handle: function(response, ioArgs){
				setUploading(false);
				if(response instanceof Error){
					if(response.dojoType && response.dojoType == "cancel"){
						return; // this is a cancelled request, ignore it
					}
					self._currentBindRequest = null; // no need to cancel it any more
					alert(messages.msgAttachmentUploadError);
					return;
				}
				
				self.fileInput.value = "";
				
				// If an error message display error and return.
				// If an informational message display and continue;
				if (response.errorMessage) {
					alert(response.errorMessage);
					return;
				}
				else if (response.informationMessage) {
					alert(response.informationMessage);
				}
				
				self.model.addProcessAttachment(self.prefix, {
					path: response.path,
					url: response.url,
					displayName: response.displayName
				});
				if(!self.selectedPath || self.selectedPath.length === 0){
					self.selectedPath = response.path;
				}
				self._populateIcons();
			}
		});
	},
	
	getValues: function(){
		var selected = this.iconListContainer.getSelectedItem();
		return {
			iconUrl: (selected) ? selected.iconUrl : "",
			title: (selected) ? selected.title : "",
			icon: (selected) ? selected.value : ""
		};
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog = function(prop){
	// create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent({
		prefix: prop.prefix,
		model: prop.model,
		selectedPath: prop.selectedPath
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: messages.EditIcon,
		dialogContent: contentWidget,
		onOK: prop.onOK
	});
};

})();