/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.List");

dojo.require("dijit._Widget");
dojo.require("dijit._KeyNavContainer");
dojo.require("jazz.util.html");

(function(){

var escapeHtml = jazz.util.html.escape;
var CLASS_SELECTED = "selected";
var CLASS_ACTION_COMMAND = "action-command ";
var CLASS_ICON = "dijitInline listIcon";
var CLASS_ITEM = "com-ibm-team-workitem-process-Item";
var CLASS_ANCHOR = "itemAnchor";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.List",
	[ dijit._Widget, dijit._KeyNavContainer ],
{
	_selected: null,
	
	postCreate: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "com-ibm-team-workitem-process-List");

		this.connect(this, "focusChild", "selectItem");
		this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
	},
	
	startup: function(){
		if(this._started){ return; }

		this.startupKeyNavChildren();

		this.inherited(arguments);
	},
	
	addItem: function(prop, /*int?*/ insertIndex){
		var item = new com.ibm.team.workitem.web.process.ui.internal.view.common.Item(prop);
		this.addChild(item, insertIndex);
		
		var self = this;
		item.connect(item, "onClick", function(){
			// don't call focusChild if clicked item is already selected
			// as it will InlineEditBox will stop editing
			if(self._selected == item){ return; }
			
			self.focusChild(item);
		});
		return item;
	},
	
	getSelectedItem: function(){
		return this._selected;
	},
	
	findItem: function(val){
		// returns the first item whose value matches val
		// returns null if not found
		var items = this.getChildren();
		for(var i=0, len=items.length; i<len; i++){
			if(items[i].value == val){
				return items[i];
			}
		}
		return null;
	},
	
	selectItem: function(item){
		if(this._selected){
			this._selected.setSelected(false);
		}
		this._selected = item;
		if(item){
			item.setSelected(true);
		}
		this.onSelected(item);
	},
	
	onSelected: function(selected){},
	
	clearItems: function(){
		this.selectItem(null);
		dojo.forEach(this.getChildren(), function(child){
			this.removeChild(child);
			child.destroy();
		}, this);
	},
	
	destroy: function(){
		this.clearItems();
		this.inherited(arguments);
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.Item",
	[dijit._Widget, dijit._Container, dijit._Contained],
{
	iconUrl: null, // string: icon URL
	title: '', // string
	actions: null, // array of objects
	value: null,
	_actionConnects: [],

	postCreate: function(){
		dojo.addClass(this.domNode, CLASS_ITEM);
		this.anchorNode = dojo.create("a", {
			tabIndex: 0,
			className: CLASS_ANCHOR
		}, this.domNode);
		this.focusNode = this.anchorNode;
		this.iconContainer = dojo.create("span", {
			className: CLASS_ICON
		}, this.anchorNode);
		
		this.inherited(arguments);
		
		this.setIcon(this.iconUrl, this.title);
		if(this.title){
			this.setTitle(this.title);
		}
		if(this.actions){
			this.setActions(this.actions);
		}
	},
	
	focus: function(){
		dijit.focus(this.focusNode);
	},
	
	createIconNode: function(iconUrl, title){
		// override to customize
		return dojo.create("img", {
			src: iconUrl,
			alt: title,
			title: title
		});
	},
	
	setIcon: function(iconUrl, title){
		this.iconContainer.innerHTML = "";
		title = title || "";
		var iconNode = this.createIconNode(iconUrl, title);
		if(iconNode){
			this.iconContainer.appendChild(iconNode);
			this.iconNode = iconNode;
		}
	},
	
	createTitleNode: function(escapedTitle){
		// override to customize
		return dojo.create("span", {
			innerHTML: escapedTitle
		});
	},
	
	setTitle: function(title){
		if(!this.titleNode){
			var titleNode = this.createTitleNode(escapeHtml(title));
			this.titleNode = titleNode;

			// place it to appropriate position
			if(this.iconContainer){
				dojo.place(this.titleNode, this.iconContainer, "after");
			}else if(this.actionNode){
				dojo.place(this.titleNode, this.actionNode, "before");
			}else{
				this.anchorNode.appendChild(this.titleNode);
			}
		}else{
			this.titleNode.innerHTML = escapeHtml(title);
		}
	},
	
	setActions: function(actionArray){
		var actionNode = this.actionNode;
		if(!actionNode){
			this.actionNode = dojo.create("span", {
				style: {
					"float": "right"
				}
			}, this.anchorNode);
		}else{
			dojo.forEach(this._actionConnects, dojo.disconnect);
			this._actionConnects = [];
			actionNode.innerHTML = "";
		}
		dojo.forEach(actionArray, function(action){
			var anchor = dojo.create("a", {
				title: action.title
			}, this.actionNode);
			var div = dojo.create("div", {
				className: CLASS_ACTION_COMMAND+action.iconClass
			}, anchor);
			dojo.create("span", {
				innerHTML: action.title,
				style: "display:none;"
			}, div);
			this._actionConnects.push(this.connect(div, "onclick", action.onClick));
		}, this);
	},
	
	setSelected: function(selected){
		dojo.toggleClass(this.anchorNode, CLASS_SELECTED, selected);
	}
});

})();