/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentController");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 			= net.jazz.ajax.toolbox.lang.checkNotEmpty;
var CACHE_SIZE = 10;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentController", null, {
	
	_widgetCache: [],
	constructor: function(multipaneContentView) {
		this._multipaneContentView = multipaneContentView;
	},
	destroy: function() {
		dojo.forEach(this._widgetCache, function(child) {
				try {
					if(child.destroyWidget) {
						child.destroyWidget();
					} else {
						if(child.destroy) child.destroy();
					}
				} catch(e) {
					alert('Error destroying: ' + child);// ignore
				}
			});
	},
	_destroyWidget: function(child) {
		if (child) {
			try {
				if (child.destroyWidget) {
					child.destroyWidget();
				}
				else {
					if (child.destroy) 
						child.destroy();
				}
			} 
			catch (e) {
				alert('Error destroying: ' + child);// ignore
			}
		} 
	},
	/*
	 * Return the currently cached widgets
	 */
	getWidgetCache: function () {
		return this._widgetCache;
	},	
	// Find the cached version of the widget
	getCachedWidget: function(widgetId, widget, instanceId, isSimple, instanceItemId) {
		if(instanceId == null && instanceItemId==null) return widget;
		var result;
		if (instanceItemId) {
			for (var i = 0; i < this._widgetCache.length; i++) {
				if (this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].widget && this._widgetCache[i].widget._workItem && this._widgetCache[i].widget._workItem.itemId == instanceItemId) {
					result = this._widgetCache[i].widget;
					break;
				}
			}
		}
		else {
			for (var i = 0; i < this._widgetCache.length; i++) {
				if (this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
					result = this._widgetCache[i].widget;
					break;
				}
			}	
		}
				
		// If it is simple, return the result
		if(isSimple) return result;
		
		// If it wasn't found and no widget provided, exit
		if(!result && widget == null) return null;
		 
		// remove the oldest non-dirty non-new item from the cache
		// and add this item to the cache 
		var cachedObject = { widgetId: widgetId, widget: widget, instanceId: instanceId, isDirty: false };
		
		/* if the cache is too big, remove the last, but not this one  */
		/* TODO: Use the last one touched from the cache, not last one created */
		if(this._getNonDirtyCacheSize() >= CACHE_SIZE) {
			var oldestItem = null;
			// Find the oldest non-dirty item
			for(var i=0; i< this._widgetCache.length; i++ ) {
				if(!this._widgetCache[i].isDirty
						&& parseInt(this._widgetCache[i].instanceId) > 0
						&&  this._widgetCache[i].instanceId != instanceId
						&& (!oldestItem  || (this._widgetCache[i]._creationTimestamp < oldestItem._creationTimestamp))) {
					oldestItem = this._widgetCache[i];
				}
			}
			// if there is an oldestItem, remove it
			if(oldestItem) {		
				this.removeCachedWidget(oldestItem.widgetId, oldestItem.instanceId);
			}
		}
		
		// If the widget wasn't in the cache already, add it
		if(!result) {
			cachedObject._creationTimestamp = new Date().getTime();
			this._widgetCache.push(cachedObject);
			result = widget;
		} 
		return result;		
	},
	_getNonDirtyCacheSize: function() {
		var count = 0;
		for(var i=0;i<this._widgetCache.length; i++) {
			if(!this._widgetCache[i].isDirty && parseInt(this._widgetCache[i].instanceId) > 0) count++;
		}
		return count;
	},
	removeCachedWidget: function(widgetId, instanceId) {
		if(instanceId == null) return;
		var newCache = [];
		var removedWidget;
		
		for(var i=0;i<this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				removedWidget =  this._widgetCache[i].widget;
				break;
			}
		}

		if (removedWidget) {
			this._destroyWidget(removedWidget);
		}		
		
		for(var i=0;i<this._widgetCache.length; i++) {
			if(!(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId)) {
				newCache.push(this._widgetCache[i]);
			}
		}
		this._widgetCache = newCache;
	},
	setIsDirty: function(widgetId, widget, instanceId, isDirty) {
		for(var i=0;i<this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				this._widgetCache[i].isDirty = isDirty;
				return;
			}
		}

	}
	
});

})();