/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var CLASS_TAB_SELECTED = "selected";
var escapeHtml = jazz.util.html.escape;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer",
	[ dijit._Widget, dijit._Templated, dijit._Container ],
{
	// summary:
	//		Jazz-style tab container
	//		(see com.ibm.team.process.web.ui.internal.admin.projects.AbstractProcessAreaEditor#createTab)
	//		with lazy-loading capability
	
	templateString: '<div class="com-ibm-team-workitem-process-TabContainer"><div dojoAttachPoint="tabButtonContainer" class="tabSection"></div><div dojoAttachPoint="containerNode"></div></div>',

	hasAddButton: false,

	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages");
		this.inherited(arguments);
	},

	postCreate: function(){
		this._tabInfo = [];
		this._selected = null;

		this.inherited(arguments);
		
		var self = this;
		if(this.hasAddButton === true){
			var button = dojo.create("a", {
				href: "#",
				className: "dijitInline addButtonAnchor",
				title: this.messages.AddTab,
				onclick: function(e){
					dojo.stopEvent(e);
					
					self.onAddTabButtonClicked();
				}
			});
			dojo.create("span", {
				className: "dijitInline addButton"
			}, button);
			this._addButton = button;
			this.tabButtonContainer.appendChild(button);
		}
		
		this.connect(this, "addChild", function(obj){
			if(!self._selected){
				// if no child has been selected yet, select first child
				self.selectChild(obj.id); // child creation might be delayed, so pass ID instead of widget instance
			}
		});
		this.connect(this, "removeChild", function(widget){
			self._removeTabButton(widget.id);
		});
	},
	
	// overridable function
	// by default, simply adds a new tab
	onAddTabButtonClicked: function(){
		var widget = new dijit._Widget({
			title: this.messages.NewTab
		});
		this.addChild(widget);
	},
	
	// override dijit._Container to support lazy loading of tabs
	// obj may be widget instance or
	// an object like { id:..., title:..., createChild:function(){...} } when lazy loading
	addChild: function(obj){
		// add tab button
		var tabButton = this.createTabButton(escapeHtml(obj.title));
		var self = this;
		tabButton.connect(tabButton.domNode, "onclick", function(e){
			dojo.stopEvent(e);
			self.selectChild(obj.id);
		});
		
		if(this.hasAddButton === true){
			dojo.place(tabButton.domNode, this._addButton, "before");
		}else{
			this.tabButtonContainer.appendChild(tabButton.domNode);
		}
		
		var tabInfo = {
			id: obj.id,
			tabButton: tabButton
		};
		this._tabInfo.push(tabInfo);
		
		if(!this._selected){
			dojo.addClass(tabButton.domNode, "firstTab");
		}
		
		if(obj && obj.id && obj.createChild && dojo.isFunction(obj.createChild)){
			// delay creation of child widget until tab gets selected
			tabInfo.createChild = obj.createChild;
		}else{
			// obj is a widget, so simply add as a child
			obj.domNode.style.display = "none";
			this.inherited(arguments);
			tabInfo.widget = obj;
		}
		
		return tabButton;
	},
	
	// overridable function
	// returns a tab button widget
	createTabButton: function(title){
		return new com.ibm.team.workitem.web.process.ui.internal.view.common.TabButton({
			title: title
		});
	},
	
	_getTabInfo: function(id){
		var tabInfo = this._tabInfo;
		for(var i=tabInfo.length-1; i>=0; i--){
			if(tabInfo[i].id == id){
				return tabInfo[i];
			}
		}
		return null;
	},
	
	getTabButton: function(index){
		if(index < this._tabInfo.length){
			return this._tabInfo[index].tabButton;
		}
		return null;
	},
	
	_removeTabButton: function(id){
		var tabInfo = this._tabInfo;
		for(var i=tabInfo.length-1; i>=0; i--){
			if(tabInfo[i].id == id){
				var tabButton = tabInfo[i].tabButton;
				if(tabButton){
					if(tabButton.domNode){
						this.tabButtonContainer.removeChild(tabButton.domNode);
					}
					tabButton.destroy();
				}
				tabInfo.splice(i,1);
				break;
			}
		}
	},
	
	// obj may be a widget or a string
	selectChild: function(obj){
		var widget = obj;
		var tabInfo;
		if(obj && dojo.isString(obj)){
			// if obj is a string
			tabInfo = this._getTabInfo(obj);
			if(tabInfo && tabInfo.createChild && dojo.isFunction(tabInfo.createChild)){
				// child has not yet been created yet, so create it
				widget = tabInfo.createChild();
				if(widget && widget.domNode){
					dijit._Container.prototype.addChild.apply(this, [widget]);
					delete tabInfo.createChild;
					tabInfo.widget = widget;
				}else{
					return;
				}
			}else{
				widget = tabInfo.widget;
			}
		}
		
		// actually select the child
		var selected = this._selected;
		if(selected != widget){
			if(selected){
				if(selected.domNode){
					selected.domNode.style.display = "none";
				}
				if(selected.id){
					tabInfo = this._getTabInfo(selected.id);
					if(tabInfo && tabInfo.tabButton && tabInfo.tabButton.domNode){
						dojo.toggleClass(tabInfo.tabButton.domNode, CLASS_TAB_SELECTED, false);
					}
				}
			}
			this._selected = widget;
			if(widget && widget.domNode){
				widget.domNode.style.display = "";
				tabInfo = this._getTabInfo(widget.id);
				if(tabInfo){
					dojo.toggleClass(tabInfo.tabButton.domNode, CLASS_TAB_SELECTED, true);
				}
			}
		}
		return widget;
	},
	
	getSelectedChild: function(){
		return this._selected;
	},
	
	getSelectedIndex: function(){
		for(var i=this._tabInfo.length-1; i>=0; i--){
			if(this._selected && this._tabInfo[i].widget == this._selected){
				return i;
			}
		}
		return -1;
	},
	
	clearTabs: function(){
		var id;
		for(var i=this._tabInfo.length-1; i>=0; i--){
			id = this._tabInfo[i].id;
			if(id){
				var widget = this._tabInfo[i].widget;
				if(widget){
					if(widget.domNode){
						this.removeChild(widget);
					}
					// somehow error is thrown for IE, so catch exception 
					try{
						widget.destroy();
					}catch(e){
					}
				}else{
					// tab content may not have been created (lazy load)
					this._removeTabButton(id);
				}
			}
		}
		this.selectChild(null);
	},
	
	destroy: function(preserveDom){
		this.clearTabs();
		this.inherited(arguments);
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.TabButton",
	[ dijit._Widget, dijit._Templated ],
{
	// default tab button
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/common/templates/TabButton.html"),
	
	title: '',
	
	setTitle: function(title){
		this.title = title;
		this.titleNode.innerHTML = "";
		this.titleNode.appendChild(dojo.doc.createTextNode(escapeHtml(title)));
	}
});

})();