/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._KeyNavContainer");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var CLASS_ROW_HOVER = "rowHover";
var CLASS_SELECTED = "selected";
var CLASS_TABLE_ROW = "com-ibm-team-workitem-process-Row";
var CLASS_ACTION_CELL = "actionCell";
var CLASS_ACTION_HEADER_CELL = "actionHeaderCell";

var escapeHtml = jazz.util.html.escape;
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;
var messages;
	
dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.Table",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/common/templates/Table.html"),
	
	hasHeader: true,
	columns: [], // an array of objects : [ { id:"col1", name:"Column 1" }, { id:"col2", name:"Column 2" },...]

	selectable: false,
	_selected: null,
	
	postCreate: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "CommonMessages");

		this.inherited(arguments);
		
		// setup columns
		this.setupColumns(this.columns);
	},
	
	setupColumns: function(columns){
		this.columns = columns;
		
		if(this.hasHeader){
			if(!this.headerRow){
				var thead = dojo.create("thead");
				this.headerRow = dojo.create("tr", {
					className: "headerRow"
				}, thead);
				dojo.place(thead, this.domNode, "first");
			}
			var headerRow = this.headerRow;
			dojo.forEach(columns, function(column){
				var prop = {};
				if(column.name){
					prop.innerHTML = column.name;
				}
				if(column.className){
					prop.className = column.className;
				}
				if(column.style){
					prop.style = column.style;
				}
				if(column.isAction){
					if(prop.className){
						prop.className += " " + CLASS_ACTION_HEADER_CELL;
					}else{
						prop.className = CLASS_ACTION_HEADER_CELL;
					}
				}
				dojo.create("th", prop, headerRow);
			});
		}
	},
	
	addItem: function(data){
		var item = new com.ibm.team.workitem.web.process.ui.internal.view.common.Row({
			columns: this.columns,
			data: data
		});
		this.addChild(item);

		if(this.selectable === true){
			var self = this;
			item.connect(item, "onClick", function(){
				self.selectItem(item);
			});
			// ondijitclick somehow absorbes events inside cells (hover actions, inline edit box), so use onkeydown instead
			item.connect(item.domNode, "onkeydown", function(e){
				if(e.keyCode == dojo.keys.ENTER){
					self.selectItem(item);
				}
			});
		}
		
		return item;
	},
	
	getSelectedItem: function(){
		return this._selected;
	},

	selectItem: function(item){
		if(this._selected){
			this._selected.setSelected(false);
		}
		this._selected = item;
		if(item){
			item.setSelected(true);
		}
		this.onSelected(item);
	},
	
	onSelected: function(selected){},
	
	clearItems: function(){
		this._selected = null;

		dojo.forEach(this.getChildren(), dojo.hitch(this, function(child){
			this.removeChild(child);
			child.destroy();
		}));
	},
	
	destroy: function(preserveDom){
		this.clearItems();
		this.inherited(arguments);
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.common.Row",
	[dijit._Widget, dijit._Contained],
{
	columns: [], // an array of objects : [ { id:"col1", name:"Column 1" }, { id:"col2", name:"Column 2" },...]
	data: null, // an object containing cell data : { col1:"Data 1", col2:"Data 2",...  }
	_cells: [],
	
	buildRendering: function(){
		this.domNode = dojo.create("tr", {
			tabIndex: 0,
			className: CLASS_TABLE_ROW
		});
		
		// descendant widget won't get destroyed by destroyRecursive without containerNode
		this.containerNode = this.domNode;
		this.focusNode = this.domNode;
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// set hover CSS class
		var domNode = this.domNode;
		this.connect(domNode, "onmouseover", function(){
			dojo.toggleClass(domNode, CLASS_ROW_HOVER, true);
		});
		this.connect(domNode, "onmouseout", function(){
			dojo.toggleClass(domNode, CLASS_ROW_HOVER, false);
		});
		
		// populate cells
		dojo.forEach(this.columns, function(column){
			var cellData = this.data[column.id];
			var td;
			var className = column.className;
			if(column.getCell){
				td = dojo.hitch(this, column.getCell)(cellData);
			}else{
				td = this.getCell(cellData);
			}
			if(td){
				if(column.isAction === true){
					if(!className){
						className = CLASS_ACTION_CELL;
					}else{
						className += " "+CLASS_ACTION_CELL;
					}
					this.setActions(cellData, td);
				}else{
					var cellContent;
					if(column.getCellContent){
						cellContent = dojo.hitch(this, column.getCellContent)(cellData);
					}else{
						cellContent = this.getCellContent(cellData);
					}
					if(cellContent){
						td.appendChild(cellContent);
					}
				}
				if(className){
					dojo.addClass(td, className);
				}
				this.domNode.appendChild(td);
				this._cells.push(td);
			}
		}, this);
	},
	
	onFocus: function(){
		dojo.addClass(this.domNode, CLASS_ROW_HOVER);
	},

	onBlur: function(){
		dojo.removeClass(this.domNode, CLASS_ROW_HOVER);
	},
	
	getCell: function(cellData){
		// return null for not adding null
		return dojo.create("td");
	},
	
	getCellContent: function(cellData){
		if(cellData && cellData.length>0){
			return dojo.doc.createTextNode(escapeHtml(cellData));
		}
		return null;
	},
	
	getCellByColumnId: function(colId){
		for(var i=0, len=this.columns.length; i<len; i++){
			if(this.columns[i].id == colId){
				return this._cells[i];
			}
		}
		return null;
	},

	setSelected: function(selected){
		dojo.toggleClass(this.domNode, CLASS_SELECTED, selected);
	},
	
	setActions: function(actionArray, container){
		if(!actionArray || actionArray.length === 0){
			container.appendChild(dojo.doc.createTextNode(messages.NA));
		}else{
			dojo.forEach(actionArray, function(action){
				container.appendChild(createActionIcon({
					iconClass: action.iconClass,
					onClick: dojo.hitch(this, action.onClick),
					title: action.title,
					owner: this
				}));
			}, this);
		}
	},
	
	destroy: function(preserveDom){
		this.destroyDescendants(preserveDom);
		this.inherited(arguments);
	}
});

})();