/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");

dojo.require("dojo.date");
dojo.require("dojox.xml.parser");
dojo.require("dojox.date.posix");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.ui.ImagePreview");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var EDITOR_PACKAGE			= "../com.ibm.team.workitem.web/ui/internal/view/editor";
var DATE_FORMAT_NO_TIME     = "%b %e, %Y";
var ImageClient             = com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var ATTACHMENT_TYPE= "com.ibm.team.workitem.linktype.attachment";
var ImagePreview			= jazz.ui.ImagePreview;

com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil= {

	UNCONFIGURED: "__com.ibm.team.workitem.web.UNCONFIGURED",
	messages:  dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages"),
	_createOption: function(displayText, value) {
		var option= new Option(displayText, value);
		var indent= this._getIndent(displayText);
		if (indent > 0) {
			option.style.paddingLeft= indent * 0.4 + "em";
		}
		return option;
	},
	checkLogin: function(msg) {
		if (com.ibm.team.repository.web.client.session.isGuest()) {
			msg = msg || this.messages.promptLogInOrCancelWorkItemEdit;
			if (confirm(msg)) {
				this.login(); /* Will not return */
			}
			return false;
		}
		return true;
	},
	_getFullName: function(list, index, labelAttr, originalLabelAttr) {
		if(!list || !list[index] || !list[index][labelAttr]) {
			return ""; // should never happen
		}
	 	var currentIndent= this._getTrueIndent(list[index][labelAttr]);
		// Remove leading and trailing whitespace
		var currentLabel = list[index][labelAttr].replace(/^\s+|\s+$/g,"")
		if(currentIndent === 0 ) return  currentLabel;
		
		var targetIndent = currentIndent - 1;
		var i = index-1;
		var workingLabel
		while(i>=0) {
			if(originalLabelAttr && list[i][originalLabelAttr]) {
				workingLabel =  list[i][originalLabelAttr];
			} else {
				workingLabel =  list[i][labelAttr];
			}
			var parentIndent = this._getTrueIndent(workingLabel);
			if(parentIndent <= targetIndent) {
				currentLabel = workingLabel.replace(/^\s+|\s+$/g,"") + " / " + currentLabel;
				if(targetIndent === 0)
				   break;
				targetIndent = targetIndent -1;
			}
			i--;
		}
		return currentLabel;
	},

	_getTrueIndent: function(str) {
		return this._getIndent(str) / 3;
	},
	_getIndent: function(str) {
	 	var indent= 0;
		if(!str || !dojo.isString(str)) return 0;
		while (indent < str.length && str.charAt(indent) === ' ') {
			indent++;
		}
		return indent;
	},
	getNewWorkItemTimestampLabel: function(workItem) {
		if(workItem._newIdLabel) return workItem._newIdLabel;
		var now = new Date();
		var hours = now.getHours();
		hours = hours < 10 ? "0" + hours : hours;
		var mins = now.getMinutes(); 
		mins = mins < 10 ? "0" + mins : mins;
		var seconds = now.getSeconds(); 
		seconds = seconds < 10 ? "0" + seconds : seconds;
		workItem._newIdLabel = "<" + hours  + ":" +  mins + ":" + seconds + ">";
		return workItem._newIdLabel;
	}, 
	_getOriginalTarget: function(e) {
		// normalize w3c (originalTarget) and srcElement (IE)
		return e.target || e.srcElement;
	},

	// adds an option to the end of a list; works both in IE and Firefox
	_addOption: function(select, option, isDefault) {
		var optionArr= select.options;
		optionArr[optionArr.length]= option;
		option.selected= (isDefault === true) ? true : false;
	},
	
	// adds a row to the end of a table; works both in IE and Firefox
	_addRow: function(table, values, actions) {
		 var row =table.insertRow(-1);
		optionArr[optionArr.length]= option;
		option.selected= (isDefault === true) ? true : false;
	},
	
	_getUriForTab: function(tabName, url) {
		var uri= "#";
		var parms= WorkItemUtil.getHashRequestParams(url);
		var addTabParam= true;
		function getPrefix(i) {
			return i === 0 ? "" : "&"; 
		}
		for (var i=0; i<parms.length; i++) {
			if (i !== 0 && tabName) {  }
			if (parms[i].name === "tab") {
				addTabParam= false;
				if (tabName) {
					uri += getPrefix(i) + parms[i].name + "=" + tabName;
				}
			} else {
				uri += getPrefix(i) + parms[i].name + "=" + parms[i].value;
			}
		}
		// remove the dosave portion
		uri  = uri.replace(/&doSave=\([0-9]+\)/, "");
		if (addTabParam && tabName) { uri += getPrefix(i) + "tab=" + tabName; }
		return uri;
	},
	findObjectInArray: function(/* object, ?property, array */ parms) {
		if(!parms || !parms.array) return -1;
		for(var i=0;i<parms.array.length;i++){
			if(parms.property) {
				if(parms.object[parms.property] == parms.array[i][parms.property]) return i;
			} else {
				if(parms.object == parms.array[i]) return i;
			}
		}
		return -1;
	},
	findObjectWithPropertyInObject: function(/* object, property, path, */ parms, caller) {
		if(!parms || !parms.object || !parms.property) return null;
		var actor;
		var originalPath = parms.path;
		for(var key in parms.object) {
			parms.path = originalPath || [];
			actor = parms.object[key];
			if(actor) {
				if(actor[parms.property]) return actor;
				// If the actor has objects with actions, ask it
				if(actor.findObjectWithProperty) {
					// Avoid loops
					if (this.findObjectInArray({
						array: parms.path,
						object: caller
					}) == -1) {
						parms.path.push(caller);
						actor = actor.findObjectWithProperty(parms);
						if (actor) 
							return actor;
					}
				}
			}
		}
		return null;
	},
	findObjectWithPropertyInArray: function(/* array, property, path */ parms, caller) {
		if(!parms || !parms.array || !parms.property) return null;
		var actor;
		parms.path = parms.path || [];
		for (var i = 0; i < parms.array.length; i++) {
			actor = parms.array[i];
			if (actor) {
				if (actor[parms.property]) 
					return actor;
				if (actor.findObjectWithProperty) {
					if (this.findObjectInArray({
						array: parms.path,
						item: caller
					}) == -1) {
						parms.path.push(caller);
						actor = actor.findObjectWithProperty( {
							property: parms.property,
							path: parms.path
						});
						if (actor) 
							return actor;
					}
				}
			}
		}
		return null;
	},	
	getSafeTextSpan: function(rawText, editorPart, links, workingCopy) {
		if(links && links.length > 0) {
			rawText = this.resolveLinks(rawText, links, workingCopy);
		}
		var span = document.createElement("SPAN"); 
		span.innerHTML = this.getSafeText(rawText, editorPart); /* innerHTML OK */
		
		return span;
	},
	resolveLinks: function(text, links, workingCopy) {
		if(!links || links.length==0) return text;
		// Go through the links in reverse offset order
		links.sort( function(a, b ) {
			if(!a || a.offset == null) return -1;
			if(!b || b.offset == null) return 1;
			if(a.offset > b.offset) return -1;
			return 1;
		})
		
		for(var i=0;i<links.length;i++) {
			if(!links[i]) continue;
			var target = "";
			var	uri = links[i].uri && links[i].uri[0]? links[i].uri[0] : null;
 			if(!uri || uri.length ==0) continue;
			if ((uri.indexOf("#action=com.ibm.team.workitem.") == -1 && uri.indexOf("internal.rest.IAttachmentRestService") == -1) || ((uri.indexOf("http") == 0) && (uri.indexOf(window.location.host) == -1))) {
						target = " target=\"_blank\"";
			}
			var phrase = text.substr(links[i].offset, links[i].length);
			var anchor = "<A _SYNTHETIC_ href='"+ // Tag synthetic links so they are not removed
					uri +
					"' class='inlinelink'" +
					target + ">" +
					phrase +
					"</A>";
			if(uri.indexOf("com.ibm.team.repository.Contributor") != -1 && workingCopy) {
				var match = uri.match(".+/com.ibm.team.repository.Contributor/(.+)");
				if(match && match.length > 0) {
					var user = workingCopy.findContributor(match[1]);
					var title = user ?  dojo.string.substitute(this.messages.titleUserNameWithId, [ user.name, user.userId]) : 
					 					dojo.string.substitute(this.messages.titleUserId, [ match[1] ]);
					anchor = anchor.replace(/_SYNTHETIC_/, "_SYNTHETIC_ title='" + title + "'");
				}	
			}
			text = text.substr(0,[links[i].offset]) +
				anchor +
				text.substr(links[i].offset + links[i].length, text.length);
		}
		return text;
	},
	linkAnchorPopUps: function(span, workItem) {
	    var childWidgets = []
		if(!workItem) return childWidgets;
		
		try{
		var attachmentNames = {};
		if (workItem._attachmentNames) {
			attachmentNames = workItem._attachmentNames;
		}
		else {
			var target, type;
			if (workItem.linkTypes) {
				for (var i = 0; i < workItem.linkTypes.length; i++) {
					if (workItem.linkTypes[i].id == ATTACHMENT_TYPE) {
						type = workItem.linkTypes[i];
						for(var j=0;j<type.linkDTOs.length;j++) {
							target = type.linkDTOs[j].target;
							if (!target || !target.id || !target.name) 
								continue;
							attachmentNames[target.id] = {
								name: target.name,
								size: target.contentLength
							};
						}
					}
				}
			}
			workItem._attachmentNames = attachmentNames;
		}
		
		// Add image preview to all image links
		var node, id, split, filename, index, ext, size;
		for(var i=0;i<span.childNodes.length;i++) {
			if(span.childNodes[i].nodeName && span.childNodes[i].nodeName.toLowerCase() == "a" && span.childNodes[i].href) {
				node = span.childNodes[i];
				if(!node.href ) continue
				// if it is not an attachment
				if(node.href.indexOf("com.ibm.team.workitem.service.internal.rest.IAttachmentRestService") == -1
				&& node.href.indexOf("com.ibm.team.workitem.common.internal.rest.IAttachmentRestService") == -1	) {
					 var args = { 
						 retainLinkText: true,
						 presentationUri: node.href 
					 };
					 childWidgets.push(new jazz.ui.ResourceLink(args, node));
				} else {
					split = node.href.split("/");
					if(split.length == 0) continue;
					id = split[split.length-1];
					
					if(!id) continue;
					if (isNaN(id)) {
						// /FileName.gif?itemId=_uuid&id=number
						split = id.split("?");
						if (!split || split.length == 0) 
							continue;
						filename = split[0];
						id=WorkItemUtil.getUrlParam(id, "id");
					} else {
						// The attachment is a number
						if (!attachmentNames[id]) 
							continue;
						filename = attachmentNames[id].name;
					}
					
					index = filename.lastIndexOf(".");
					ext = index == -1 ? null : filename.substring(index + 1);
					ext = ext ? ext.toLowerCase() : ext;
					if (ext == "gif" || ext == "png" || ext == "jpeg" || ext == "jpg") {
					childWidgets.push(new ImagePreview({
						around: [node],
						src: node.href,
						filename: filename
						}, document.createElement("div")));
					} else {
						if(attachmentNames[id] && attachmentNames[id].size) {
							size = "" + Math.ceil(attachmentNames[id].size / 1024);
							node.title = dojo.string.substitute(this.messages.titleFileNameSizeK, [filename, size ]); 
						}
					}
				}	
			}
		}
		} catch(e) {
			console.debug("Error creating attachment links: " + e.message); // $NON-NLS-1
		}
		return childWidgets;
	},
	getSummaryLabel: function(workItem, doNotShowType) {
		if(!workItem){
			return "";
		}
		
		var id = workItem.id;
		if(!id && workItem.attributes && workItem.attributes.id && workItem.attributes.id.id) {
			id = workItem.attributes.id.id;
		} 
		if(!id)	return this.messages.titleAWorkItem;
		if(id < 0)
			return this.messages.titleAnUnsavedNewWorkItem;  
		
		var label = "";
		var typeLabel = doNotShowType ? "" : workItem.attributes.workItemType.label;
		try {
			label = dojo.string.substitute(this.messages.pageTitleWorkItemSummary, [typeLabel,
			id + "", this.restoreTags(workItem.attributes.summary.content)]);
		} catch(e) {} // ignore	
		return label
	},
	restoreTags: function(value) {
		var rawText = value;
		rawText= rawText.replace(/<b>/gi,"");
		rawText= rawText.replace(/<\/b>/gi,"");
		rawText= rawText.replace(/<i>/gi,"");
		rawText= rawText.replace(/<\/i>/gi,"");
		rawText= rawText.replace(/\&gt;/gi,">");
		rawText= rawText.replace(/\&lt;/gi,"<");
		rawText= rawText.replace(/\&amp;/gi,"&");
		rawText= rawText.replace(/\&tilde;/gi,"~");
		rawText= rawText.replace(/\&quot;/gi,"\"");
		rawText= rawText.replace(/\&circ;/gi,"^");
		rawText= rawText.replace(/\&nbsp;/gi," ");
		rawText= rawText.replace(/<br\/>/gi,"\n");
		rawText= rawText.replace(/\&#09;/g, "\t");
		rawText= rawText.replace(/<a href="([^"]*)">([^<]*)<\/a>/g, "$2 <$1>"); // todo: should be externalized. too late for 3.0
		return rawText
	},
	
	cleanRawValue: function(value){
		var rawText = value;
		rawText = rawText.replace(/\r/g,""); // IE adds returns to the values so strip them for compare

		// value returned by rich text editor is pretty printed, so remove carriage returns etc.
		rawText = rawText.replace(/\n/g, "");
		rawText = rawText.replace(/<br \/>/g, "<br/>");
		return rawText;
	},
	
	createLoadingNode: function(){
		return dojo.create("div", {
			className: "loadingNode",
			innerHTML: this.messages.busyLoading
		});
	},

	/* 
	 * Given a value, return a text element with the HTML safely translated
	 */
	getSafeText: function(rawText, editorPart) {
		try{
		rawText= rawText.replace(/<synthetic>/g,"");
		rawText= rawText.replace(/<\/synthetic>/g,"");
		
//		rawText= rawText.replace(/>/g,"&gt;");
//		rawText= rawText.replace(/</g,"&lt;");
		
		// replace raw newlines with HTML line breaks
		// replace newlines (both native and xml encoded)
		rawText= rawText.replace(/(\n|\&#xA;)/g,"<br/>");
		// replace native spaces with non-breaking spaces (use two spaces
		// rather than one to avoid lines not wrapping)
		rawText= rawText.replace(/  /g, "&nbsp;&nbsp;");
		// replace xml encoded tabs with non-breaking spaces 
		rawText= rawText.replace(/(\t|\&#09);/g, "&nbsp;&nbsp;&nbsp;&nbsp;");
		
		rawText = jazz.util.html.removeScript(rawText);
		
		// Restore any <WBR>'s 
		rawText= rawText.replace(/\&lt;WBR\/\&gt;/g,"<WBR/>");
		rawText= rawText.replace(/\&lt;wbr\/\&gt;/g,"<wbr/>");
//		rawText= rawText.replace(/\&amp;nbsp;/g,"&nbsp;");

/* 				
		// Restore bold and italics
		rawText= rawText.replace(/\&amp;lt;B\&amp;gt;/g,"<B>");
		rawText= rawText.replace(/\&amp;lt;b\&amp;gt;/g,"<b>");
		rawText= rawText.replace(/\&amp;lt;\/B\&amp;gt;/g,"</B>");
		rawText= rawText.replace(/\&amp;lt;\/b\&amp;gt;/g,"</b>");
		rawText= rawText.replace(/\&amp;lt;I\&amp;gt;/g,"<I>");
		rawText= rawText.replace(/\&amp;lt;i\&amp;gt;/g,"<i>");
		rawText= rawText.replace(/\&amp;lt;\/I\&amp;gt;/g,"</I>");
		rawText= rawText.replace(/\&amp;lt;\/i\&amp;gt;/g,"</i>");
		rawText= rawText.replace(/\&lt;B\&gt;/g,"<B>");
		rawText= rawText.replace(/\&lt;I\&gt;/g,"<I>");
		rawText= rawText.replace(/\&lt;b\&gt;/g,"<B>");
		rawText= rawText.replace(/\&lt;i\&gt;/g,"<I>");
		rawText= rawText.replace(/\&lt;\/B\&gt;/g,"</B>");
		rawText= rawText.replace(/\&lt;\/I\&gt;/g,"</I>");
		rawText= rawText.replace(/\&lt;\/b\&gt;/g,"</B>");
		rawText= rawText.replace(/\&lt;\/i\&gt;/g,"</I>");
*/	
	// Restore <br/>
		rawText= rawText.replace(/\&lt;BR\/\&gt;/gi,"<BR/>");
		rawText= rawText.replace(/\&lt;BR \/\&gt;/gi,"<BR/>");

		// restore synthetic anchors, then clean them up below

		try {
			rawText = rawText.replace(/&lt;a _SYTNTHETIC_ (.+?)&gt;?/gim, "<a $1>"); // <a ... >
			var cursor = 0;
			var rawText2 = "";
			var regexp = new RegExp("<a _SYNTHETIC_ (.+?)>", "gim");
			var target, anchor, href, parms;
			
			var match = regexp.exec(rawText);
			if(match) { // if there is a match, also change the closing anchor
				rawText = rawText.replace(/&lt;\/a*&gt;/gim, "</a>"); // </a>
			}
			// [0] = the anchor
			// [1] = the stuff between the anchor
			// index = where in raw text be the anchor
			while (match) {
				target = "";
				// remove quotes
				parms = match[1].replace(/"/gim, "").replace(/'/gim, "") + " ";
				href = parms.match(/href=(.+?) /im);
				if (href && href.length>0 && href[1]) {
					href = href[1].replace(/&quot;/gi, "").replace(/\"/g,"");
					href = this.removeHrefScripting(href);
					// if its not a local action set the target 
					if ((href.indexOf("#action=com.ibm.team.workitem.") == -1  && href.indexOf("internal.rest.IAttachmentRestService") == -1) || ((href.indexOf("http") == 0) && (href.indexOf(window.location.host) == -1))) {
						target = "target=\"_blank\"";
					}
					href = " href=\"" + href + "\"";
					anchor = "<a " + target + href + ">";
				} else { // no href on the anchor
					anchor =match[0].replace(/>/gi,"&gt;").replace(/</gi, "&lt;");
				}

				
				rawText2 += rawText.substring(cursor, match.index) + anchor;
				cursor = match.index + match[0].length;
				
				var match = regexp.exec(rawText);
			}
			rawText = rawText2 + rawText.substring(cursor, rawText.length);
			if(editorPart) {
				rawText = this._swizzleAttachmentUrls(rawText, editorPart);
			}
		} 
		catch (e) {
			console.debug("Error parsing anchors");
			rawText = rawText.replace(/<a (.+?)>?/gim, "&lt;a $1&gt;"); // <a ... >
			rawText = rawText.replace(/<\/a>/gim, "&lt;/a&gt;"); // </a>
		}
		
		if (!dojo.isWebKit) {
			// Add <WBR>'s every 70 chars of text
			var _tempSpan = document.createElement("SPAN");
			_tempSpan.innerHTML = rawText;
			
			for (var i = 0; i < _tempSpan.childNodes.length; i++) {
				if (_tempSpan.childNodes[i].nodeName == "#text") {
					_tempSpan.childNodes[i].nodeValue = _tempSpan.childNodes[i].nodeValue.replace(/[^ ]{70}/g, '$&<wbr>');
				}
			}
			
			var _spanText = _tempSpan.innerHTML;
			rawText = _spanText.replace(/\&lt;wbr\&gt;/g, "<wbr/>")
		}
		/*
		 * IE 6 has a problem that strings that are too long push over the
		 * div and result in pushing down the whole content area. Short-term
		 * workaround is to break up the strings to allow wrapping.
		 */
		if ((dojo.isIE && dojo.isIE<7) ) {
			rawText= WorkItemUtil._breakupAllLongWords(rawText);
		}
		} catch(e) {
			console.debug("Error creating safe text in EditorUtil: " + rawText);
		}
		return rawText;
	},
	getPlainText: function(html) {
		var div= document.createElement("div");
		div.innerHTML= html;
		return div.textContent || div.innerText;
	},
	removeHrefScripting: function(origHref) {
		if(!origHref) return origHref;
		try {
			// remove any scripting punctuation
			var href = origHref.replace(/\n/gim, "").replace(/\r/gim, "").replace(/ /gim, "").replace(/\(/, "").replace(/\)/, "");
			href = href.replace(/>/gim, "_").replace(/</gim, "_");
			// remove scripting 
			if (href.match(/script:/gim)) {
				href = WorkItemUtil.getWelcomePageUri(dojo.string.substitute(this.messages.errorInvalidURI, [origHref])); // don't use it; 
			}
			return href;
		} 
		catch (e) {
			console.debug("Error parsing href");
		}
		return origHref;
	},
	
// before	https://jazz.net/jazz/service/com.ibm.team.workitem.common.internal.rest.IAttachmentRestService/itemName/com.ibm.team.workitem.Attachment/45
// after:  https://jazz.net/jazz/service/com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/repo/csid/Attachment/filename.txt?itemId=_theUUid&id=45
	_attachmentRegex:new RegExp("/service\/com.ibm.team.workitem.common.internal.rest.IAttachmentRestService\/itemName\/com.ibm.team.workitem.Attachment\/([0-9]+)\">", "g"),
	_swizzleAttachmentUrls: function(string, editorPart) {
		if(!editorPart || !editorPart.getAttachmentInfo) return string;
		// Attachment urls 
		//     in: ../../service/...Attachment/id
		//    out: ../../service/...Attachment/filename.typ?itemId=_uuid&id=id
		
		var regex = this._attachmentRegex; 
		var newString = string;
		var matches = regex.exec(string);
		while (matches) {
				var anchor = matches[0]; 
				var number = matches[1];
				var targetDTO = editorPart.getAttachmentInfo(number);
				if(targetDTO) {
					var suffix = "repo/csid/Attachment/" + targetDTO.name + "?itemId=" +targetDTO.itemId+"&id="+number; 
					var newAnchor = anchor.replace(number, suffix );
					newAnchor = newAnchor.replace(/itemName\/com.ibm.team.workitem.Attachment\//, "");
					newAnchor = newAnchor.replace(/com.ibm.team.workitem.common/,"com.ibm.team.workitem.service");
					newString = newString.replace(anchor, newAnchor); 
				}
			var matches = regex.exec(string);
		}
		return newString;

	},
// before	https://jazz.net/jazz/service/com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/repo/csid/Attachment/_theUUid
// after:  https://jazz.net/jazz/service/com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/repo/csid/Attachment/filename.txt?itemId=_theUUid&id=45
	_swizzleQuickInformationAttachmentUrls: function(href, workingCopy) { 
		var regex = new RegExp("\/service\/com.ibm.team.workitem.service.internal.rest.IAttachmentRestService\/repo\/csid\/Attachment\/([\_a-zA-Z0-9\-]+) ", "g")
		var newHref = href;
		var matches = regex.exec(href + " "); // add a string to make the match
		if (matches) {
				var itemId = matches[1];
				var targetDTO = workingCopy.getAttachmentInfoById(itemId);
				if(targetDTO) {
					var suffix = targetDTO.name + "?itemId=" +targetDTO.itemId+"&id="+targetDTO.id; 
					var newLink = href.replace(itemId, suffix );
					newHref = newLink.replace(href, newLink); 
				}
		}
		return newHref;

	},

	_getDateString: function(dateObject, includeTime) {
		var dateStr;
		if (dojo.i18n.normalizeLocale() == "en-us") {
			dateStr = dojox.date.posix.strftime(dateObject, DATE_FORMAT_NO_TIME);
			if (includeTime) {
				dateStr += " ";
				var hour = dojox.date.posix.strftime(dateObject, "%I");
				dateStr += hour.charAt(0) !== "0" ? hour : hour.substring(1);
				dateStr += ":" + dojox.date.posix.strftime(dateObject, "%M") + " ";
				dateStr += dojox.date.posix.strftime(dateObject, "%p").toUpperCase();
			}
		}
		else {
			if (includeTime) {
				dateStr = dojo.date.locale.format(dateObject, {formatLength: 'medium' })
//				dateStr = dojox.date.posix.strftime(dateObject, "%c"); // preferred date and time in current locale
			} else {
				dateStr =  dojo.date.locale.format(dateObject, {selector:'date', formatLength: 'medium'});
//				dateStr = dojox.date.posix.strftime(dateObject, "%x");  // preferred date in current locale
			}
		}
		
		return dateStr;
	},

	_resetTextInput: function(inputElement) {
		inputElement.value= "";
	},
	
	_resetSelect: function(select) {
		dojox.xml.parser.removeChildren(select);
		select.disabled= false;
		this._addOption(select, new Option(this.messages.busyLoading));
	},
	
	_bindLabelProviderToSelect: function(labelProvider, select, img) {
		dojox.xml.parser.removeChildren(select);
		labelProvider = labelProvider ? labelProvider : { label: "", id: ""};
		if(labelProvider.name && !labelProvider.label) {
			var label = labelProvider.name;
		} else {
			var label = "";
		}

		var option= new Option(label, labelProvider.id);

		this._addOption(select, option);
		if (img) {
			var iconUrl = labelProvider.iconUrl || this._getUnknownIcon();
			img.src = ImageClient.getImageUrl(iconUrl);
		}
	},
	
	_getIconUrlProp: function() { return "__jazzIconUrl"; },
	_getIconImgProp: function() { return "__jazzIconImg"; },
	_getLabelProp: function() { return "__jazzLabel"; },
	
	_bindValuesToSelect: function(values, select, img, moreLabel, moreTitle, moreId, selectedValueId) {
		if(!selectedValueId) {
			selectedValueId  = select.value;
			}
		
		dojox.xml.parser.removeChildren(select);
		if (!values || values.length === 0) {
			select.disabled= true;
			this._addOption(select, new Option(this.messages.optionNotAvailable, this.UNCONFIGURED));
			return;
		} else {
			select.disabled= false;
		}
		for (var i=0; i<values.length; i++) {
			if(values[i].label == null || !dojo.isString(values[i].label)) {
				values[i].label = values[i].id;
			}
			var option= this._createOption(values[i].label, values[i].id);
			if(values[i].title) {
				option.title=values[i].title;
			} 
			option[this._getLabelProp()]= values[i].label;
			if (img) {
				option[this._getIconImgProp()]= img;
				option[this._getIconUrlProp()]= values[i].iconUrl || this._getUnknownIcon();
			}
			var isDefault= selectedValueId && selectedValueId === values[i].id;
			if(isDefault) {
				select.currentDefaultIndex = i;
			} 
			this._addOption(select, option, isDefault);
			if(isDefault && values[i].title) {
				select.title = values[i].title;
			}
		}
		if(moreLabel) {
			var option= this._createOption(moreLabel, moreId);
			option.title = moreTitle;
			this._addOption(select, option, false);
		}
	},
	
	_updateSelectImage: function(select) {
		var curOpt= select.options[select.selectedIndex];
		var img= curOpt[this._getIconImgProp()];
		var iconUrl= curOpt[this._getIconUrlProp()];
		if (img) {
			img.src= ImageClient.getImageUrl(iconUrl);
		}
	},
	
	_getCurrentSelectProps: function(select) {
		var curOpt= select.options[select.selectedIndex];
		var o= {
			label: curOpt[this._getLabelProp()],
			id: select.value
		};
		if (curOpt[this._getIconUrlProp()]) {
			o.iconUrl= curOpt[this._getIconUrlProp()];
		}
		return o;
	},
	
	_getUnknownIcon: function() {
		return "/com.ibm.team.workitem.common/icons/obj16/type_none.gif";
	}
};

})();