/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.ScreenCaptureAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.AbstractAction"); //$NON-NLS-1$
dojo.require("dojo.io.iframe");//$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$

dojo.require("dojo.cookie");
dojo.require("dojo.string");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "ScreenCaptureMessages");

(function(){
	
	var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    
    var POLLING_INTERVAL = 1000;
    var POLLING_LIMIT = 60;
    var PollingCounter = 0;
    var PollingTimer;
    var PollingStartingPoint;
    
    var SkipInformationDialogCookie = "ScreenCapture.SkipInformationDialog";
    
    var Dialog= jazz.ui.Dialog;
    
    var MSGS = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "ScreenCaptureMessages"); //$NON-NLS-1$ //$NON-NLS-2$
    
	dojo.declare("com.ibm.team.workitem.web.ui.internal.view.editor.ScreenCaptureAction", com.ibm.team.workitem.web.ui2.internal.action.AbstractAction, { //$NON-NLS-1$
		
		_messages: MSGS,
		
		run: function() {
		
			/* Eagerly test if the Cookie is set to avoid a roundtrip (and help for Bug 106684) */
			if (dojo.cookie.isSupported() && dojo.cookie(SkipInformationDialogCookie) == "true") {
				this._launchScreenCapture(false);
				return;
			}
		
			/* Show an Information Dialog if this is the first time running, then start Screen Capture */
			var serviceRequest = new ServiceRequest("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService", "getSkipScreenCaptureInfoDialog", {});
			
			var self = this;
		    var handlerObj = {
		        success: function(result){
		    	
		    	   /* Show Information Dialog */
		           if (result == "false" || dojo.isIE) { //Workaround for Bug 106684: Always rely on the Cookie on IE because can not set location from background
		        	   var args= {};
		        	   
		        	   var dialogContainer= window.document.createElement("div");
		        	   dialogContainer.style.padding= "5px";
		        	   
		        	   var textContainer= window.document.createElement("div");
		        	   dialogContainer.appendChild(textContainer);
		        	   
		        	   textContainer.innerHTML= dojo.string.substitute(self._messages.captureInfo, [ "http://www.jazz.net/library/article/536" ]);

		        	   var buttonContainer= window.document.createElement("div");
		        	   dojo.style(buttonContainer, "margin", "20px 0 0 0" );
		        	   dialogContainer.appendChild(buttonContainer);
		        	   
		        	   var neverShowAgainInput= window.document.createElement("input");
		        	   neverShowAgainInput.type= "checkbox";
		        	   buttonContainer.appendChild(neverShowAgainInput);
		        	   
		        	   var neverShowAgainLabel= window.document.createElement("label");
		        	   neverShowAgainLabel.appendChild(window.document.createTextNode(self._messages.neverShowAgain));
		        	   dojo.style(neverShowAgainLabel, { "padding": "0 0 0 3px", "vertical-align": "top" } ); //For IE7
		        	   neverShowAgainLabel.setAttribute("style", "padding-left:3px;vertical-align:top;");
		        	   buttonContainer.appendChild(neverShowAgainLabel);
		        	   
		        	   var okButton= window.document.createElement("button");
		        	   okButton.appendChild(window.document.createTextNode(self._messages.okLabel));
		        	   dojo.style(okButton, { "float": "right", "margin": "0 0 5px 0" } );
		        	   buttonContainer.appendChild(okButton);
		        	   
		       		   args.contentNode= dialogContainer;
		       		   args.primaryTitle= self._messages.infoDialogTitle;
		       		   args.width= "40em"; //$NON-NLS-1$
		       		   args.destroyContents= true;
		       		   args.onClose= dojo.hitch(this, function() {
		       			   if (neverShowAgainInput.checked)
		       				   dojo.cookie(SkipInformationDialogCookie, "true", { expires: 5 * 365 });
		       			   self._launchScreenCapture(neverShowAgainInput.checked);
		       			   return true;
		       		   });
		       		   var infoDialog= new Dialog(args);
		       		   
		       		   if (dojo.isIE)
		       			   okButton.attachEvent("onclick", dojo.hitch(this, function() { infoDialog.close(); }), false);
		       		   else
		       			   okButton.addEventListener("click", dojo.hitch(this, function() { infoDialog.close(); }), false);
		           }
		           
		           /* Skip Information Dialog */
		           else {
		        	   dojo.cookie(SkipInformationDialogCookie, "true", { expires: 5 * 365 });
		        	   self._launchScreenCapture(false);
		           }
		        },
		        
		        failure: function(errorObj){
		        	console.log(errorObj);
		        }
		    };
		    
		    /* Invoke Service */
		    var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
		    TeamServerClient.invokeService(serviceRequest, responseHandler);
		},
		
		_launchScreenCapture: function(skipInfoDialog) {
			
			/* Screen Capture for a New Work Item: Only allow to attach to other Items */
			if (this.workingCopy.isNew()) {
				this._openUrl(TeamServerClient._getServicePrefix() + "com.ibm.team.workitem.service.internal.rest.IScreenCaptureRestService?projectId=" + this.workingCopy.getProjectAreaItemId());
			} 
			
			/* Screen Capture for an existing Work Item: Poll for changes to Attachment Count */
			else {
				
				/* Open Capture JNLP */
				if (skipInfoDialog)
					this._openUrl(TeamServerClient._getServicePrefix() + "com.ibm.team.workitem.service.internal.rest.IScreenCaptureRestService?projectId=" + this.workingCopy.getProjectAreaItemId() + "&itemNr=" + this.workingCopy.getId() + "&itemId=" + this.workingCopy.getItemId() + "&skipInfoDialog");
				else
					this._openUrl(TeamServerClient._getServicePrefix() + "com.ibm.team.workitem.service.internal.rest.IScreenCaptureRestService?projectId=" + this.workingCopy.getProjectAreaItemId() + "&itemNr=" + this.workingCopy.getId() + "&itemId=" + this.workingCopy.getItemId());
				
				/* Polling */
				var attachmentCount= this._getLocalAttachmentsCount();
				
				if (PollingTimer)
					window.clearTimeout(PollingTimer);
				
				PollingCounter= 0;
				PollingStartingPoint= document.location.toString();
				PollingTimer= window.setTimeout(dojo.hitch(this, "_checkForAddedAttachments", attachmentCount), POLLING_INTERVAL);
			}	
		},
		_openUrl: function(url) {
			
			if(!dojo.isIE && this._iframe) {
				dojo.io.iframe.setSrc(this._iframe,  url, /* replace */ true);
			} else {
				var name = "iframe_"  + Date.parse(new  Date().toString()); // for ie				
				this._iframe = dojo.io.iframe.create(name, "", url); // hidden $NON-NLS-1$
			}
		}, 
		_checkForAddedAttachments: function(attachmentCount) {
			
			/* Check for Terminating Condition */
			PollingCounter++;
			if (PollingCounter > POLLING_LIMIT || document.location.toString() != PollingStartingPoint)
				return;
			
			/* Construct Service Request */
			var serviceRequest = new ServiceRequest("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService", "getAttachmentsCount", {
				workItemId: this.workingCopy.getItemId()
	        });
			
		    var self = this;
		    var handlerObj = {
		        success: function(result){
		    	
		    	   /* Merge new Attachment into View */
		           if (parseInt(result) > attachmentCount)
		        	   self._onAttachmentAdded();
		           
		           /* Poll for new Attachments up to a limit */
		           else {
		        	   if (PollingCounter < POLLING_LIMIT && document.location.toString() == PollingStartingPoint)
		        		   PollingTimer= window.setTimeout(dojo.hitch(self, "_checkForAddedAttachments", attachmentCount), POLLING_INTERVAL);
		           }
		        },
		        
		        failure: function(errorObj){
		        	console.log(errorObj);
		        }
		    };
		    
		    /* Invoke Service */
		    var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
		    TeamServerClient.invokeService(serviceRequest, responseHandler);
		},
		
		_onAttachmentAdded: function() {
			this.workingCopy.mergeRepositoryChanges({isKeepingChangedValues: true});
		},
		
		_getLocalAttachmentsCount: function() {
			var attachmentDTOs = this.workingCopy.getValue({
				path: ["linkTypes", /* id */ "com.ibm.team.workitem.linktype.attachment", "linkDTOs"]
			});
			
			if (attachmentDTOs)
				return attachmentDTOs.length;
			
			return 0;
		},
	
		destroy: function() {
			if (PollingTimer)
				window.clearTimeout(PollingTimer);
			PollingCounter = POLLING_LIMIT;
		},
		
		isEnabled: function(){
	        return !this.workingCopy.isNew();
	    }
	});

})();