/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.Dialog");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {
	
var KEY_RETURN			= 13;
var KEY_ENTER			= 14;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog", [dijit._Widget], {
	constructor: function(params) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		if (!dojo.isFunction(params.onOk)) throw new Error("Missing function for 'onOk' parameter");
	},
	postCreate: function() {
		this._initContentWidget();
		this._initFloatingPane();
	},
	_onClose: function() {
		this.floatingPane.close();
	},
	_initContentWidget: function() {
		var self = this;
		var onOk = function(uri,comment) {
			var args = {
				uri: uri,
				comment: comment,
				action: self.action
			}
			self.onOk(args);
			dojo.hitch(self,self._onClose)();
		};
		var onCancel = dojo.hitch(this,this._onClose);
		var contentParams = {
			onOk: onOk,
			onCancel: onCancel,
			action: this.action,
			parentController: this.parentController
		};
		var content = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentContent(contentParams);
		this.dialogContent = content;
	},
	_initFloatingPane: function() {
		var floatingPaneParams = {
			width: "475px",
			onClose: this.onClose,
			contentNode: this.dialogContent.domNode
		};
		if (this.dialogTitle) {
			floatingPaneParams.primaryTitle = this.dialogTitle;
			floatingPaneParams.secondaryTitle = "";
		} else {
			floatingPaneParams.primaryTitle = this.messages.dialogTitleSelectUrl;
		}
		this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
		if(this.dialogContent) this.dialogContent.setFocus();
	},
	onClose: function () {
		
	}
});


/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/SelectUriAndCommentContent.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/SelectWorkItemContent.css"),
		constructor: function(params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.buttonOK = this.messages.buttonOK;
			this.buttonCancel = this.messages.buttonCancel;
		},
		postCreate: function() {
			var nameValueLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout();
			nameValueLayout.addRow({ labelText: this.messages.labelResourceUrl, valueNode: this.urlInputDiv });
			nameValueLayout.addRow({ labelText: this.messages.labelResourceComment, valueNode: this.commentInputDiv });
			this._childWidgets =   [ nameValueLayout ];
			// mark that keypresses from this control should be ignored w/r/t
			// dirtying the editor
			this.urlInput.__jazzIgnoreKeystroke= true;
			this.commentInput.__jazzIgnoreKeystroke= true;
			this.urlAndCommentSection.appendChild(nameValueLayout.domNode);
			this._connectEvents();
		},
		destroy: function() {
			dojo.forEach(this._childWidgets, function(widget) { widget.destroy() } );
			this.inherited(arguments);
		},
		setFocus: function() {
			this.urlInput.focus();
		},
		reset: function() {
			this.urlInput.value= "";
			this.commentInput.value= "";
			this.okCmd.disabled= true;
		},
		setKey: function(key) {
			this.key= key;
		},		
		_connectEvents: function() {
			this.connect(this.okCmd, "onclick", "_handleOk");
			this.connect(this.cancelCmd, "onclick",  "_handleCancel");
			this.connect(this.urlInput, "onkeyup",  "_handleKeyUp");
			this.connect(this.commentInput, "onkeyup",  "_handleKeyUp");
		},
		_handleKeyUp: function(e) {
//			if(e.keyCode === KEY_ENTER || e.keyCode === KEY_RETURN ) {// enter
//				this._handleOk();
//			} else {
				this.okCmd.disabled=this.urlInput.value.length == 0;
//			}
		},
		
		_handleOk: function() {
			this.onOk(this.urlInput.value, this.commentInput.value);
			this.reset();			
		},
		
		_handleCancel: function() {
			this.onCancel();
			this.reset();
		}
	}
);

})();