/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("jazz.core.loader");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var EditorUtil				= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil			= com.ibm.team.workitem.web.client.util;
var QueryClient				= com.ibm.team.workitem.web.client.internal.QueryClient;
var load_async				= jazz.core.loader.load_async;
var CLASS_KEYWORD_SEARCH	= "keywordSearch";

// delay between user input and running a search (in milliseconds)
var DELAY					= 750;

// cap max number of query results at 50
var MAX						= 50;

var KEY_RETURN			= 13;
var KEY_ENTER			= 14;
var KEY_SPACE			= 32;
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/SelectWorkItemContent.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/SelectWorkItemContent.css"),
		constructor: function(params) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.labelEnterSearchString = this.messages.labelEnterSearchString;
			this.buttonOK = this.messages.buttonOK;
			
			this.buttonCancel = this.messages.buttonCancel;
			this.htmlEnterSearchString = this.messages.labelEnterSearchString;
		},
		postCreate: function() {
			// mark that keypresses from this control should be ignored w/r/t
			// dirtying the editor
			this.queryInput.__jazzIgnoreKeystroke= true;
			var nameValueLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout();
			this._childWidgets = [ nameValueLayout ];
			this.projectAreaSelectDiv = document.createElement("span");
			this.projectAreaSelectDiv.className = "selectInput";
			var projLabel =  this.messages.labelProjectArea; 
			nameValueLayout.addRow( {labelText: projLabel, valueNode: this.projectAreaSelectDiv });
			
			this._initRadioButtons(nameValueLayout);

			this.typeSelectDiv = document.createElement("span");
			this.typeSelectDiv.className = "selectInput";
			var typeLabel =  this.messages.labelTypeFilter;
			this.typeSelectRow = nameValueLayout.addRow({labelText: typeLabel, valueNode: this.typeSelectDiv });
			this.nameValueSection.appendChild(nameValueLayout.domNode);
			this._initProjectAreaSelector();
			this._projectSelect.tabindex = 5;
			this.showRecentlyViewed = true;

			if(this.showRecentlyViewed) {
				this._setResultTitle(this.messages.labelRecentlyViewed);
			} else {
				this._setResultTitle(this.messages.labelMatching);
			}
			
			this._connectEvents();
		},
		destroy: function() {
			dojo.forEach(this._childWidgets, function(widget) { widget.destroy() } );
			this.inherited(arguments);
		},
		_setResultTitle: function(aMessage) {
			if(this.resultTitle) {
				this.resultTitle.innerHTML = aMessage;
			}
		},
		_initProjectAreaSelector: function() {
			this._currentProjectArea = jazz.app.context.get();
			this._projectSelect = document.createElement("select");
			this._projectSelect.className = "selectSelect";
			/*
			 * If the projectAreaItemId is provide, use that, don't show current
			 */
			if(!this.projectAreaItemId ||  this._currentProjectArea.itemId ==  this.projectAreaItemId) {
				var option = EditorUtil._addOption(this._projectSelect, new Option(this._currentProjectArea.name, this._currentProjectArea.itemId), true);				
			}

			this.projectAreaSelectDiv.appendChild(this._projectSelect);
			
			var srh= new ServiceResponseHandler(this, "_setProjectAreaOptions", "_handleError");
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getProcessAreaHierarchies(srh);
		},
		
		_initRadioButtons: function(nameValueLayout){
			// add radio buttons to select search by keyword or work item number or search by query
			var radioButtonContainer = dojo.create("div", {
				className: "RadioContainer"
			});
			nameValueLayout.addRow( { isWide: true, hideLabel: true, valueNode: radioButtonContainer });
			var keywordRadioContainer = dojo.create("span", {
				className: "dijitInline KeywordRadioSpan"
			}, radioButtonContainer);
			var keywordRadioId = this.id+"_radioKeyword";

			// Dynamically created radio buttons cannot be selected when using IE7
			// It has to be created in a special way (see http://msdn.microsoft.com/ja-jp/library/ms536389(en-us,VS.85).aspx)
			var addRadio = function(name, parentNode, checked){
				var inp = null;
				if(dojo.isIE <= 7){
					inp = dojo.doc.createElement('<input type="radio" name="'+name+'" />');
					inp.defaultChecked = checked;
				}else{
					inp = dojo.create("input", { type:"radio", name:name });
					inp.checked = checked;
				}
				parentNode.appendChild(inp);
				return inp;
			};

			var radioName = this.id+"_keywordQueryRadio";
			this.radioKeyword = addRadio(radioName, keywordRadioContainer, true);
			this.radioKeyword.id = keywordRadioId;
			this.connect(this.radioKeyword, "onclick", dojo.hitch(this, "_radioButtonClicked"));
			
			var label = dojo.create("label", {
				"for": keywordRadioId
			}, keywordRadioContainer);
			label.appendChild(document.createTextNode(" "+this.messages.labelSearchByKeyword));
			var queryRadioContainer = dojo.create("span", {
				className: "dijitInline QueryRadioSpan"
			}, radioButtonContainer);
			var queryRadioId = this.id+"_radioQuery";
			var queryRadio = addRadio(radioName, queryRadioContainer, false);
			queryRadio.id = queryRadioId;
			this.connect(queryRadio, "onclick", dojo.hitch(this, "_radioButtonClicked"));
			
			var label2 = dojo.create("label", {
				"for": queryRadioId
			}, queryRadioContainer);
			label2.appendChild(document.createTextNode(" "+this.messages.labelSearchByQuery));
			
			dojo.addClass(this.innerWrapper, CLASS_KEYWORD_SEARCH);
		},
		
		_radioButtonClicked: function(){
			var isKeywordChecked = this.radioKeyword.checked;
			this.searchInputSection.style.display = (isKeywordChecked) ? "" : "none";
			this.typeSelectRow.style.display = (isKeywordChecked) ? "" : "none";
			this.queryPickerSection.style.display = (isKeywordChecked) ? "none" : "";
			
			dojo.toggleClass(this.innerWrapper, CLASS_KEYWORD_SEARCH, isKeywordChecked);
			
			if(!isKeywordChecked && !this.queryPickerTree){
				var self = this;
				load_async("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserContent", function(){
					var projectSelect = self._projectSelect;
					var queryPickerTree = new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserContent({
						projectAreaItemId: projectSelect.value
					});
					self.queryPickerArea.appendChild(queryPickerTree.domNode);
					self.queryPickerTree = queryPickerTree;
					self._childWidgets.push(queryPickerTree);
					queryPickerTree.connect(queryPickerTree, "treeSelectionChanged", dojo.hitch(self, "_runQuery"));
					queryPickerTree.connect(projectSelect, "onchange", function(){
						// re-populate tree if project area changed
						queryPickerTree.populatePicker(projectSelect.value);
					});
				});
			}
		},
		
		_runQuery: function(){
			var query = this.queryPickerTree.getSelection();
			if(!query){
				return;
			}
			this._setStatusMessage(this.messages.busyRunningQuery);
			var responseHandler = new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");
			var args = {
				queryItemId: query.itemId
			};
			QueryClient.getResults(args, responseHandler);
		},
		
		_handleError: function(e) {
			console.debug("Error loading project areas. Option ignored."); // $NON-NLS-1
		},
		_setProjectAreaOptions: function(projectAreaTree) {
			var projectAreas = [];
			this._collectProcessAreas(projectAreaTree, projectAreas, "", false, false);
			projectAreas.sort(function(a, b){
				try{
					if(a && a.label && a.label.toLowerCase && b && b.label & b.label.toLowerCase) {
						return a.label.toLowerCase() > b.label.toLowerCase() ? 1 : -1;
					} else {
						return -1;
					}
				} catch(e) { 
					return -1 
				}
			});
			for(var i=0;i<projectAreas.length;i++) {
				if (projectAreas[i].id != this._currentProjectArea.itemId) {
					if (!this.projectAreaItemId || this.projectAreaItemId == projectAreas[i].id) {
						EditorUtil._addOption(this._projectSelect, new Option(projectAreas[i].label, projectAreas[i].id));
					}	
				}
			}
			
			if(this._projectSelect.childNodes.length === 0) this._projectSelect.style.visibility = "hidden";
			else if(this._projectSelect.childNodes.length === 1) this._projectSelect.disabled = true;
			this._isProjectAreaInitialized = true;
			this._resetTypeFilterSelector();
		},
		_collectProcessAreas: function(processAreas, collector, indent, includeChildren, disable){
			if (processAreas) {
				for (var i = 0; i < processAreas.length; i++) {
					var team = processAreas[i];
					var item = {
						id: team.itemId,
						label: indent + team.name,
						disabled: disable
					};
					collector.push(item);		
					if (includeChildren) 
						this._collectProcessAreas(team.children, collector, indent + "   ", true, false);
				}
			}
		},
		_resetTypeFilterSelector: function(doSearchAfter) {
			this._searchPending = doSearchAfter;
			if(this._typeSelect)	this.currentTypeSelection = this._typeSelect.value;
			this._typeSelect = document.createElement("select");
			this._typeSelect.className = "selectSelect";
			this.connect(this._typeSelect, "onchange", "_typeSelected");
			dojox.xml.parser.replaceChildren(this.typeSelectDiv,  this._typeSelect);
			if(!this._isProjectAreaInitialized) {
				this._enableCreateLink({isEnabled: true, isInitialized: false});
				return; // will call back after init
			}
			if(this._projectSelect.childNodes.length === 0){
				this._enableCreateLink({isEnabled: true, isInitialized: false});
			 	return; // no types to get
			}	
			var projectAreaItemId;
			if(this._projectSelect.value) projectAreaItemId = this._projectSelect.value;
			var srh= new ServiceResponseHandler(this, "_resetTypeFilterSelector2", "_handleError");
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getAllValues(srh, { newWorkItem: "true",  ids: [ "workItemType" ], projectAreaItemId: projectAreaItemId});
		},
		_resetTypeFilterSelector2: function(valueSet) {
			var workItemTypes= valueSet[0].uiItems;
			EditorUtil._addOption(this._typeSelect, new Option(this.messages.optionShowAll, -1));
			this._workItemTypes = workItemTypes;
			if(!workItemTypes || workItemTypes.length == 0) {
				this._enableCreateLink({isEnabled: false, isInitialized: true});
			 	return; // no types to get
			} else {
				this._setCreateLinkType(workItemTypes[0].id);
				this._enableCreateLink({isEnabled: true, isInitialized: true});
			}
			for(var i=0;i<workItemTypes.length;i++) {
				if(!this._isTypeIdShown(workItemTypes[i].id)) continue;
				EditorUtil._addOption(this._typeSelect, new Option(workItemTypes[i].label, workItemTypes[i].id), /* isDefault */ this.currentTypeSelection ==  workItemTypes[i].id);
			}
			
			if(this._projectSelect.childNodes.length === 0) this._projectSelect.style.visibility = "hidden";
			else if(this._projectSelect.childNodes.length === 1) this._projectSelect.disabled = true;
			
			if(this._searchPending) {
				this._searchPending = false;
				this._doSearch();
			}
		},
		_isTypeIdShown: function(workItemType) {
			if(!workItemType || !this.workItemTypeIdList || this.workItemTypeIdList.length == 0) return true;
			for(var i=0;i<this.workItemTypeIdList.length; i++) {
				if(workItemType == this.workItemTypeIdList[i]) return true;
			}
			return false
		},
		setFocus: function() {
			if(this.showRecentlyViewed) {
				this._showRecentlyViewedList();
			}	
			this.queryInput.focus();
		},
		reset: function() {
			this.queryInput.value= "";
			this.queryResults.length= 0;
			this._clearStatusMessage();
			this.okCmd.disabled= true;
		},
		setKey: function(key) {
			this.key= key;
		},
		
		setAllowsMultiple: function(allowsMultiple) {
			this.allowsMultiple= allowsMultiple;
		},
		
		setExclusionList: function(exclusionList) {
			this.exclusionMap= {};
			for (var i=0; i<exclusionList.length; i++) {
				var id= exclusionList[i];
				this.exclusionMap[id]= true;
			}
		},
		
		_connectEvents: function() {
			this.connect(this.okCmd, "onclick", "_handleOk");
			this.connect(this.queryResults, "ondblclick",  "_handleOk");
			this.connect(this.cancelCmd, "onclick",  "_handleCancel");
			this.connect(this.queryInput, "onkeyup", "_doCancellableSearch");
			this.connect(this.queryResults, "onkeyup",  "_handleQueryResultsKey");
			this.connect(this.queryResults, "onchange", "_handleSelectionChange");
			this.connect(this._projectSelect, "onchange", "_handleProjectChange");
		},
		_handleProjectChange: function(e){
			this._resetTypeFilterSelector(true /* do search */); // will force a search
		},
		_handleQueryResultsKey: function(e) {
			if(e.keyCode === KEY_ENTER || e.keyCode === KEY_RETURN || e.keyCode === KEY_SPACE) {// enter
				this._handleOk();
			}
		},
		
		_handleOk: function() {
			var retArr= [];
			var options= this.queryResults.options;
			for (var i=0; i<options.length; i++) {
				if (options[i].selected) {
					retArr.push(this._backingWorkItems[options[i].value]);
				}
			}
			this.reset(); // this must be BEFORE onOk as onOk will destroy contents and cause script error in this.reset()
			this.onOk(retArr, this.key);
		},
		
		_handleCancel: function() {
			this.reset(); // this must be BEFORE onCancel as onCancel will destroy contents and cause script error in this.reset()
			this.onCancel();
		},
		
		_handleSelectionChange: function() {
			// disable the 'OK' button if nothing is selected
			this.okCmd.disabled= this.queryResults.selectedIndex === -1;
			if (!this.allowsMultiple) {
				this._deselectPreviouslySelected();
			}
		},
		
		_deselectPreviouslySelected: function() {
			var options= this.queryResults.options;
			var prevOption= this._currentOption ? this._currentOption : null;
			var newOptionSet= false;

			for (var i=0; i<options.length; i++) {
				if (!options[i].selected) {
					continue;
				} else if (options[i] == prevOption || newOptionSet) {
					options[i].selected= false;
				} else {
					this._currentOption= options[i];
					newOptionSet= true;
				}
			}
		},
		
		_doCancellableSearch: function(e) {
			if(e.keyCode === 40){ // arrow down 
				this.queryResults.focus();
				return;
			} 
			if(e.keyCode === 38){ // arrow up 
				this._projectSelect.focus();
				return;
			} 
			if (this._currentId) {
				clearTimeout(this._currentId);
			}
			var self= this;
			var doSearchFunc= function() {
				self._doSearch();
			};
			this._currentId= setTimeout(doSearchFunc, DELAY);
		},
		_typeSelected: function(e) {
			this._setCreateLinkType(this._typeSelect.value);
			this._doSearch();
		},
		_doSearch: function() {
			this._setResultTitle(this.messages.labelMatching);

			// if there's no input, do nothing
			var value= this.queryInput.value;
			if (dojo.trim(value) === "") {
				this.queryResults.length= 0;
				this._clearStatusMessage();
				return;
			}
			var srh= new ServiceResponseHandler(this, "_loadQueryResults", "_queryError");
			var params;
			var projectAreaItemId = this._projectSelect.value;
			if (WorkItemUtil.isWorkItemId(value))
				params= { id: WorkItemUtil.getWorkItemId(value), projectAreaItemId: projectAreaItemId, scopeToProject: false };
			else
				params= { fullText: value, projectAreaItemId : projectAreaItemId };
			var workItemType = this._typeSelect.value;
			if(workItemType && workItemType != -1) {
				params.workItemType = workItemType;
			}
			this._setStatusMessage(this.messages.busyRunningSearch);
			QueryClient.getResults(params, srh);
		},
		/*
		 * Get the recent items that are workitems
		 *    id 
		 *    summary
		 */
		_getRecentWorkItems: function() {
			if(this.parentController && this.parentController.pageController)
					return this.parentController.pageController.getRecentWorkItemList(); // TODO: Move recent list to cache
			return [];
		},
		_getCurrentWorkItem: function() {
			var currentWorkItem;
			if(this.parentController && this.parentController.workingCopy) return this.parentController.workingCopy.object;		
			if(this.parentController && this.parentController.pageController) {
				currentWorkItem = this.parentController.pageController.getCurrentWorkItem();
			} else {
				if (this.parentController) {
					if(this.parentController.workItem) {
						currentWorkItem = this.parentController.workItem;
					} else {
						if(this.parentController._workItem) {
							currentWorkItem = this.parentController._workItem;
						}
					}
				}
			}
			return currentWorkItem;
		},
		/*
		 * Return the current work item id, using lazy initialization
		 */
		_getCurrentWorkItemId: function() {
			if(this._currentWorkItemId) return this._currentWorkItemId;
			var currentWorkItem = this._getCurrentWorkItem();
			
			var currentWorkItemId = -1;
			if(currentWorkItem) {
				currentWorkItemId = currentWorkItem.id;
			}
			this._currentWorkItemId = currentWorkItemId;
			return this._currentWorkItemId;
		},
		/*
		 * Show the recently viewed list
		 */
		_showRecentlyViewedList: function() {
			var currentWorkItemId = this._getCurrentWorkItemId();
			try{ 
				var currentWorkItem = this._getCurrentWorkItem();
				if(currentWorkItemId > 0 && currentWorkItem && currentWorkItem.attributes && currentWorkItem.attributes.workItemType && currentWorkItem.attributes.workItemType.label) {
					var currentLabel = dojo.string.substitute(this.messages.labelRecentlyViewedExcluding , [currentWorkItem.attributes.workItemType.label, "" + currentWorkItemId ]);
				} else {
					var currentLabel = this.messages.labelRecentlyViewedExcludingThisWorkItem;					
				}
				var title = currentLabel;
//				var title = dojo.string.substitute(this.messages.labelRecentlyViewedExcluding , [currentLabel]);
				if(this.resultTitle) {				
					dojox.xml.parser.replaceChildren(this.resultTitle, document.createTextNode(title));
				}
			} catch(ignore) {
				this._setResultTitle(this.messages.labelRecentlyViewed);
			};
			this._backingWorkItems= {};
			var proxyItems = this._getRecentWorkItems();
			var recentItems = [];
			for (var i=0; i<proxyItems.length; i++) {
				recentItems.push(proxyItems[i].object);
			}
			
			for (var i=0; i<recentItems.length; i++) {
				if(currentWorkItemId == recentItems[i].id || recentItems[i].id < 0) {
					continue; // don't put the current item on the list; don't put new items on the list
				}
				/*
				 * Filter to show only allowed types
				 */
				if(recentItems[i] && recentItems[i].attributes && recentItems[i].attributes.workItemType) {
					if(!this._isTypeIdShown(recentItems[i].attributes.workItemType.id)) continue;	
				}

				var title= EditorUtil.getSummaryLabel(recentItems[i], /* skip type */ true);
				var option= new Option(title, recentItems[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[recentItems[i].id]= recentItems[i];
				if (this.exclusionMap && (this.exclusionMap[recentItems[i].id] || this.exclusionMap[recentItems[i].itemId])) {
					this._disableOption(option,recentItems[i].id );
				}
			}
		},
		_loadQueryResults: function(summaryPage) {
		    if(!this.resultTitle) return; // destroyed while searching
			this._setResultTitle(this.messages.labelMatching);
			var summaries= summaryPage.workItemSummaryDTOs ?
					summaryPage.workItemSummaryDTOs : [];
			this.queryResults.length= 0;
			this._backingWorkItems= {};
			var displayedResults= 0;
			var typeId = (this._typeSelect && this.radioKeyword.checked) ? this._typeSelect.value : -1;
			for (var i=0; i<summaries.length; i++) {
				if(typeId != -1) {
					if(typeId != summaries[i].type2 ) continue;
				} else {
					/*
					 * Filter the results to the selected types
					 */
					 if(!this._isTypeIdShown(summaries[i].type2)) continue;
				}
				displayedResults++;
				var title= summaries[i].id + ": " + EditorUtil.restoreTags(summaries[i].summary);
				var option= new Option(title, summaries[i].id);
				EditorUtil._addOption(this.queryResults, option, /*isDefault*/false);
				this._backingWorkItems[summaries[i].id]= summaries[i];
				if (this.exclusionMap[summaries[i].id] || this.exclusionMap[summaries[i].workItemItemId]) {
					this._disableOption(option,summaries[i].id);
				}	
			}
			this._setStatusMessage(dojo.string.substitute(this.messages.msgResults, [ displayedResults ]));
		},
		_disableOption: function(option, id){
			var currentWorkItemId = this._getCurrentWorkItemId();
			option.style.color = "#CCCCCC";
			option.selected = false;
			if(id == currentWorkItemId) {
				option.title = this.messages.titleThisIsCurrent;
			} else {
				option.title = this.messages.titleThisIsLinked;
			}
			
		},
		_queryError: function(errorObj) {
			this._setStatusMessage(this.messages.errorFetchingSearchResults, /*isError*/ true);
		},
		
		_clearStatusMessage: function() {
			this.msgArea.innerHTML= ""; /* innerHTML OK */
		},
		
		_setStatusMessage: function(msg, isError) {
			dojox.xml.parser.replaceChildren(this.msgArea, document.createTextNode(msg));
			this.msgArea.className= isError ? "ErrorStatus" : "NormalStatus";
		},
		_enableCreateLink: function(parms) {
			if(this.action) {
				/*
				 * Duplicate of is a special case and should not allow creating a new work item that is a duplicate
				 * (Because this resolves the source)
				 */
				if(this.action.value.id == "com.ibm.team.workitem.linktype.duplicateworkitem" ) {
					parms.isEnabled = false;			
				} 
			} else {
				parms.isInitialized = false; // hide if no action
			}
			
			if(!parms.isInitialized || (this.parentController && this.parentController.workingCopy && this.parentController.workingCopy.isNew())) {
				this.createItemAnchor.style.display = "none";
				this.createItemAnchor._isEnabled = false;
			} else {
				this.createItemAnchor.style.display = "inline";
				
				if(parms.isEnabled) {
					this.createItemAnchor._isEnabled = true;
					dojo.addClass(this.createItemAnchor, "CreateItemAnchor");
					dojo.removeClass(this.createItemAnchor, "CreateItemAnchorDisabled");
				} else {
					this.createItemAnchor._isEnabled = false;
					dojo.addClass(this.createItemAnchor, "CreateItemAnchorDisabled");
					dojo.removeClass(this.createItemAnchor, "CreateItemAnchor");
				}
			}
		},
		_setCreateLinkType: function(itemTypeId) {
			var workItemTypes = this._workItemTypes;
			if(!workItemTypes) return;
			var len = workItemTypes.length;
			if(len===0) return;
			var itemType;
			for(var i=0; i<len; i++) {
				if(workItemTypes[i].id == itemTypeId) {
					itemType = workItemTypes[i];
					break;
				}
			}
			// 125923: OSLC work item picker: log entry when switching item type
			if(!itemType){
				itemType = workItemTypes[0];
			}
			
			var message = dojo.string.substitute(this.messages.labelCreateItem, [itemType.label]);
			dojox.xml.parser.replaceChildren(this.createItemLink, document.createTextNode(message));
			this.createItemAnchor._itemType = itemType;
		},
		_addItemClicked: function(e) {
			dojo.stopEvent(e);
			var anchor = e.currentTarget;
			if(!anchor || !anchor._isEnabled || !this.action) return;
			
			var itemToLinkTo= this.parentController.workingCopy;
			if (!itemToLinkTo) {
				return;
			}
			
			var args = {
				newWorkItemProjectAreaItemId: this._projectSelect.value,
				typeName: anchor._itemType.id,
				addLinkIsSource: !this.action.value.isSource, // create the opposite direction of link
				addLinkType: this.action.value.id,
				addLinkedProjectAreaItemId: itemToLinkTo.getProjectAreaItemId(),
				addLinkedItemId: itemToLinkTo.getItemId()
			}

			var url = WorkItemUtil.getNewWorkItemUri2(args);
			this._handleCancel();
			if(this.parentController && this.parentController.pageController) {
				this.parentController.pageController._updateUrl(url);
			}
		},
		__sentinal: true
	}
);

})();