/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.Dialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent");
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog", [dijit._Widget], {
	constructor: function(params) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		if (!dojo.isFunction(params.onOk)) throw new Error("Missing function for 'onOk' parameter");
	},
	postCreate: function() {
		this._initContentWidget();
		this._initFloatingPane();
	},
	destroy: function() {
		if(this.dialogContent) {
			this.dialogContent.destroy();
			this.dialogContent = null;
		}
		this.inherited(arguments);
	},
	_onClose: function() {
		this.floatingPane.close();
	},
	_initContentWidget: function() {
		var self = this;
		var onOk = function(workItems, key) {
			self.onOk(workItems, key);
			dojo.hitch(self,self._onClose)();
		};
		var onCancel = dojo.hitch(this,this._onClose);
		var contentParams = {
			action: this.action,
			onOk: onOk,
			onCancel: onCancel,
			allowsMultiple: this.allowsMultiple,
			workItemTypeIdList: this.workItemTypeIdList,
			showRecentlyViewed: this.showRecentlyViewed,
			key: this.action,
			parentController: this.parentController
		};
		if(this.projectAreaItemId) contentParams.projectAreaItemId = this.projectAreaItemId;
		var content = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemContent(contentParams);
		content.setExclusionList(this.exclusionList);
		this.dialogContent = content;
	},
	_initFloatingPane: function() {
		var self = this;
		var floatingPaneParams = {
			destroyContents: true,
			width: "500px",
			onClose: function() {
				self.onClose();
			},
			contentNode: this.dialogContent.domNode
		};
		if (this.dialogTitle) {
			floatingPaneParams.primaryTitle = this.dialogTitle;
			floatingPaneParams.secondaryTitle = (!this.allowsMultiple ? this.messages.dialogTitleSelectWorkItem2 : this.messages.dialogTitleSelectWorkItems2);
		} else {
			floatingPaneParams.primaryTitle = !this.allowsMultiple ?  this.messages.dialogTitleSelectWorkItem : this.messages.dialogTitleSelectWorkItems;
		}
		this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
		if(this.dialogContent) this.dialogContent.setFocus();
	},
	onClose: function () {
		if(this.dialogContent) {
			this.dialogContent.destroy();
			this.dialogContent = null;
		}
		this.destroy();
	}
});

})();