/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.SplitWorkItemAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.AbstractAction"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function(){
	
  var TeamContributorSelectionDialog= com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
  
  dojo.declare("com.ibm.team.workitem.web.ui.internal.view.editor.SplitWorkItemAction", com.ibm.team.workitem.web.ui2.internal.action.AbstractAction, { //$NON-NLS-1$
	  
      run: function(){
      
        this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
        var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
        
        // checking if timespent is 0 is moved to server as it cannot be done on the client 
        if(this.workingCopy.isChanged()) {
        
        	if (confirm (this.messages.msgSaveBeforeSplit)) {
        		
            	var self= this;
    			this.workingCopy.storeWorkItem({
    				instigator: this,
    				operationMsg: this.messages.busySaving,
    				skipErrorMessage: true,
    				onSuccess: function(parms) {
    					self.newOwnerWidget= self._splitItem(dojo.hitch(self,self._onOk));
    				    // the work item save would have been called, we need to refresh the editor in case the
    					// user decides not to split the work item now
    					dojo.connect(self.newOwnerWidget, "onClose", dojo.hitch(self, self._onCancel));
    	    		},
    	    		onError: dojo.hitch(this,this._splitActionShowMessage)
    			});
        	}
        } else {
        	this._splitItem(dojo.hitch(this,this._onOk));
        }
      },
      
      _splitItem: function(onOk) {
          var params = {
        		  allowMultiple : false,
        		  self: this,
                  isMultiselect: false,
                  useOkCancel: true, /* Not Add/Close */
                  onOK: function(selectedContribs){
                          if (selectedContribs && selectedContribs.length > 0) {
                        	  onOk(selectedContribs[0]);
                          }
                  }
          };
          return new TeamContributorSelectionDialog(params);
      },
      
      _onOk: function (value) {
       	  this.workingCopy.splitWorkItem({workingCopy: this.workingCopy, newOwner: value, skipErrorMessage: true, onError: dojo.hitch(this,this._splitActionShowMessage)});
      },
      
      _onCancel: function() {

    	  var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
    	  var workItemAttributePath = ["attributes", WorkItemAttributes.ID];
          var workItemId = this.workingCopy.getValue({path: workItemAttributePath});

          var args = { 
			id: workItemId.id,
			onSuccess: dojo.hitch(this, this._refreshComplete), 
			isRefresh: true,
			fetchEditorPresentation: false,
			fetchInlineEditorPresentation: false
			};
          this.workingCopy.fetchWorkItem(args);
      },
      
      _refreshComplete: function(parms) {
    	  // nothing to do
      }, 
			
      // until w.i. 106548 is implemented, any result message will be displayed as an alert and not in editor
      _splitActionShowMessage: function(error, parms) {
    	  var errorMessage = "";
    	  if (error && error.infos && error.infos.length > 0) {
    		  if ("" != error.infos[0].summary)
    			  errorMessage+= error.infos[0].summary;
    		  if ("" != error.infos[0].description)
    			  errorMessage+= ":" + error.infos[0].description;
    		  if ("" == errorMessage)
    			  errorMessage = error.message;
    	  }
 
    	  if ("" != errorMessage)
    		  this._showMessage(errorMessage);
      },
			
      isEnabled: function(){
    	  return true;
      }
  });
  
})();
