/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit._base.popup");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector",
	[dijit._Widget, dijit._Templated],
	{

		templateString: '<div class="com-ibm-team-workitem-web-tags">' +
						'<div dojoAttachPoint="_tagMatchArea"></div>' +
						'</div>',
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/TagSelector.css"),
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function(args, frag) {
			this.connect(this.domNode, "onclick", "_handleClick");
			this.connect(this._tagMatchArea, "onmouseover", "_handleMouseOver");
			this.connect(this._tagMatchArea, "onmouseout", "_handleMouseOut");
		},
		_handleClick: function(e) {
			dojo.stopEvent(e);
			var target= e.target || e.srcElement;
			if (target.tagName === "A") {
				this._doSelect(target);
			}
		},
		
		_doSelect: function(link) {
			if(!link._jazzTextIsMessage) {
				this._callback(link._jazzText);
			}
			this.close();
		},
		
		_handleMouseOver: function(e) {
			this.isMouseOver = true;
			var target= e.target || e.srcElement;
			if (target.tagName === "A") {
				this._highlightTag(target);
			}
		},
		_handleMouseOut: function(e) {
			this.isMouseOver = false;
		},
		
		open: function() {
			if (this.isShowingNow) { return; }
			dijit.popup.open({ 
						popup: this,
						parent: this.parentWidget,
						around: this._tagNode
			 });
			this._validateNode();
			this.isShowingNow = true;
		},
		MAX_SIZE: 11,
		showMatchingTags: function(searchTerm, excludeTerms) {
			
			// create a map of exclude terms for fast searching
			if (!excludeTerms) { excludeTerms= []; }
			var excludeTermMap= {};
			for (var i=0; i<excludeTerms.length; i++) {
				excludeTermMap[dojo.trim(excludeTerms[i])]= true;
			}
			
			var searchTermLower= searchTerm.toLowerCase();
			var matchArr= [];
			var foundCount = 0;
			for (var i=0; i<this._tags.length; i++) {
				if (this._tags[i].key.indexOf(searchTermLower) == 0) {
					var appendTerms= [];
					for (var tag in this._tags[i].o) {
						if (!excludeTermMap[tag]) {
							foundCount++;
							if(foundCount < this.MAX_SIZE) {
								appendTerms.push(tag);
							}
						}
					}
					matchArr= matchArr.concat(appendTerms);
				}
			}

			if (matchArr.length === 0 ||
					(matchArr.length === 1 && matchArr[0] === searchTerm)) {
				// if we have no matches or exactly one (case-sensitive) match, hide the tag picker 
				this.close();
			} else {
				// otherwise sort the results and show the matches
				matchArr.sort(function(a,b) { return a.toLowerCase() < b.toLowerCase() ? -1 : 1; } );
				if(foundCount > this.MAX_SIZE) {
					var item = {
						value: dojo.string.substitute(this.messages.optionPlusMore, [ "" + (foundCount - this.MAX_SIZE +1)]),
						isMessage: true
					};
					matchArr.push(item);
				}
				this._showMatches(matchArr, searchTerm);
			}
		},
		
		setAllTags: function(tagItemArr) {
			tagItemArr= dojo.map(tagItemArr ? tagItemArr : [], function(item) {
				return item.id;
			});
			
			// create a map of case-insensitive tags, mapped to their case-
			// sensitive matches
			// (e.g. ux can map to any of the following: ux, uX, Ux, UX)
			var tagMap= {};
			for (var i=0; i<tagItemArr.length; i++) {
				var tagItem= tagItemArr[i];
				var rawTags= tagItem.split(",");
				for (var j=0; j<rawTags.length; j++) {
					rawTags[j]= dojo.trim(rawTags[j]);
					var lower= rawTags[j].toLowerCase();
					if (!tagMap[lower]) {
						tagMap[lower]= {};
					}
					// the value of the property below doesn't matter; just want to
					// eliminate dups
					tagMap[lower][rawTags[j]]= true;
				}
			}
			
			// now turn them into arrays for searching (suboptimal, but good
			// enough for now)
			this._tags= [];
			for (var key in tagMap) {
				var entry= {key: key, o: tagMap[key]};
				this._tags.push(entry);
			}
		},
		
		highlightNextTag: function(direction) {
			// if nothing is currently selected, just return

			var tags= this._tagMatchArea.getElementsByTagName("A");
			
			if (tags.length < 2) {
				return;
			}

			var curIndex= this._selectedTagIndex;
			if (direction === "up") {
				var newIndex= curIndex === 0 ? tags.length - 1 : curIndex - 1;
			} else {
				var newIndex= curIndex === tags.length - 1 ? 0 : curIndex + 1;
			}
			this._highlightTag(tags[newIndex]);
		},
		
		selectCurrentTag: function() {
			if (this._selectedTagIndex >= 0) {
				var tags= this._tagMatchArea.getElementsByTagName("A");
				this._doSelect(tags[this._selectedTagIndex]);
			}
		},

		close: function() {
			if (this._validateTimeout) {
				clearTimeout(this._validateTimeout)
				eval("delete this._validateTimeout"); // eval for validator
			}
			if (this.isShowingNow) {
				dijit.popup.close(this);
				this.isShowingNow = false;
			}
		},

		uninitialize: function(){
			this.close();
		},
		
		_validateNode: function() {
			if (dojo.isDescendant(this._tagNode, document.body)) {
				this._validateTimeout = setTimeout(dojo.hitch(this,this._validateNode),500);
				return;
			}
			this.close();
		},
		
		_showMatches: function(matchArr, searchText) {
			dojox.xml.parser.removeChildren(this._tagMatchArea);
			this._selectedTagIndex= -1;
			for (var i=0; i<matchArr.length; i++) {

				var link= document.createElement("a");
				link.href= "addTag:" + matchArr[i];
				var matchingPrefix= document.createElement("span");
				matchingPrefix.className= "MatchingPrefix";
				if (!matchArr[i].isMessage) {
					matchingPrefix.appendChild(document.createTextNode(matchArr[i].substring(0, searchText.length)));
					var nonMatchingSuffix = document.createTextNode(matchArr[i].substring(searchText.length));
					link.appendChild(matchingPrefix);
					link.appendChild(nonMatchingSuffix);
				} else {
					var nonMatchingSuffix = document.createTextNode(matchArr[i].value);
					link.appendChild(matchingPrefix);
					link.appendChild(nonMatchingSuffix)
					link._jazzTextIsMessage = true;
				}
				link._jazzText= matchArr[i];
				this._tagMatchArea.appendChild(link);
			
				if (i === 0) {
					this._highlightTag(link);
				}
			}
			
			// close and open the tag picker in case it went from being
			// positioned above an element to below an element (as sometimes
			// happens when the list goes from long to short on a smaller
			// screen)
			this.close();
			this.open();
		},
		
		_highlightTag: function(link) {
			var tags= this._tagMatchArea.getElementsByTagName("A");
			for (var i=0; i<tags.length; i++) {
				if (tags[i] === link) {
					this._selectedTagIndex= i;
					tags[i].className= "OnHover";
				} else {
					tags[i].className= "";
				}
			}
		}
	}
);

var TagSelector= com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector;

})();
