/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("jazz.util.html");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WORKITEM_ACTION_SHOW_RICH_TEXT = com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_ACTION_SHOW_RICH_TEXT;
var editorProxyClass = com.ibm.team.workitem.web.ui.internal.view.richtext.getEditorProxyClass();

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemComment.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemComment.css"),
		// populate the template with data, connect events, and show initial
		// child widget
		isNew: false,
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},	
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-workitem-richtext");
			this._childWidgets = [];
			this.priorTextValue = "";
			this._commentHeader.title = this.messages.titleCommentCollapse;
			this._editorProxy = new editorProxyClass({
				editorArgs: {
					parentController: this.parentController,
					height: "11.2em" // same height as SelfSizingTextAreaWidget
				},
				parentNode: this._newCommentArea
			});
			var self = this;
			this.connect(this._editorProxy, "onEditorCreated", function(editor){
				editor.connect(editor, "onNormalizedDisplayChanged", dojo.hitch(this, function(){
					if(this.newCommentValue != this.getValue()){
						this._handleTextChange();
					}
				}));
				editor.connect(editor, "onBlur", dojo.hitch(self, "_handleTextBlur"));
				editor.subscribe(WORKITEM_ACTION_SHOW_RICH_TEXT, dojo.hitch(this, function(ed){
					if(editor !== ed){
						this.hideNewComment();
					}
				}));
			});

			if (this.comment == null) {
				this._commentHeader.style.display = "none";
				this._oldCommentArea.style.display = "none";
				this.isNew = true;
				if (this.workingCopy) {
					this.workingCopy.addListener({
						path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
						event: "onchange",
						listener: this,
						functionName: "_handleWorkingCopyChange"
					});
					var value = this._getAttributeValue();
					if(value) {
						this.setValue(value);
						this._setWorkingCopyValue(/* isQuiet */ false , /* notifyEvenIfUnchanged */ true);
					}
				}
				return;
			}
			 	
			this._bindCommentHeader(this.comment);
			this._bindCommentBody(this.comment);
		
		},
		_handleWorkingCopyChange: function(parms) {
			this.setValue(parms.attributeValue || "");
			if(parms.validationStatus)
				this._editorProxy.setProperty("status", parms.validationStatus);
		},	
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"] });			
		},
		destroyWidget: function() {
			this._stopNewCommentInterval();
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
			this.destroy();
		},
		destroy: function(){
			if(this._editorProxy){ this._editorProxy.destroy(); }
			this._editorProxy = null;
			this.inherited(arguments);
		},
		_bindCommentHeader: function(comment) {
			dojox.xml.parser.removeChildren(this._commentHeaderSPAN);
			
			// "<span class=\"commentNumber\">" + comment.commentNumber + ".</span>";
			var commentSpan = document.createElement("SPAN");
			commentSpan.className="commentNumber";
			commentSpan.appendChild(document.createTextNode(comment.commentNumber + "."));
						
			this._commentHeaderSPAN.appendChild(commentSpan);			
		
			var contribName= jazz.util.html.escape(comment.creator.name);
			var contribEmail= jazz.util.html.escape(comment.creator.emailAddress);

// "<span class=\"commentDetails\"><a title='Send an email to the user' href=\"mailto:" + contribEmail + "\">" + contribName + "</a> ";
			var emailSpan = document.createElement("SPAN");
			emailSpan.className="commentDetails";

			var emailSubject;
			if(this.parentController && this.parentController.workingCopy) {
				try {
					emailSubject =  this.parentController.workingCopy.object.attributes.workItemType.label + " " + this.parentController.workingCopy.getId() + ": " + EditorUtil.restoreTags(this.parentController.workingCopy.object.attributes.summary.content);
				}	catch(e) {} // ignore;			
			}    

			var emailHref = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil._getMailto(contribEmail, emailSubject); // subject can be null
			var emailAnchor = document.createElement("A");
			if (comment.creator.locationUri && this.parentController && this.parentController.workingCopy) {
					var resourceLinkArgs= { 
						userId: comment.creator.userId,
						uri: comment.creator.locationUri,
						presentationUri: emailHref,
						label: contribName
					};
					this._childWidgets.push(this.parentController.workingCopy.applyResourceLinkToAnchor(resourceLinkArgs, emailAnchor));
			} else {
				emailAnchor.href = emailHref;
				emailAnchor.appendChild(document.createTextNode(contribName));
			}

			var userIdText = "";
			if(comment.creator.userId) {
	            userIdText = comment.creator.userId;
			}
			// The userid must have a title otherwise it inherits the "collapse comment" title from the bar below	
			emailAnchor.title=  dojo.string.substitute(this.messages.titleUserNameWithId, [ contribName, userIdText ]);

			emailSpan.appendChild(emailAnchor);
			
			this._commentHeaderSPAN.appendChild(emailSpan);
			
			
//			var commentDate= dojo.date.fromIso8601(comment.creationDate);
			var commentDateStr = this.messages.labelNow;
			if(comment.creationDate) {
				var commentDate= WorkItemUtil.fromISOString(comment.creationDate);
				if(commentDate) {
					commentDateStr= EditorUtil._getDateString(commentDate, true);
				}	
			}
			
			var dateSpan = document.createElement("SPAN");
			dateSpan.className="CommentDate";
			dateSpan.appendChild(document.createTextNode(commentDateStr));
					
			this._commentHeaderSPAN.appendChild(dateSpan);

			if(this.isCollapsed) {
				var summarySpan = EditorUtil.getSafeTextSpan(comment.content, this, comment.links, this.parentController.workingCopy);
				this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(summarySpan, this.workItem));
				summarySpan.className="CommentSummary";
				this._commentHeaderSPAN.appendChild(summarySpan);
				summarySpan.title = "";
			}
			
		},

		_bindCommentBody: function(comment) {
            if(!comment || !comment.content || comment.content.length ==0) {
					comment = {
						content: this.messages.labelNoCommentContent
					};
					dojo.addClass(this._commentBodySPAN, "empty-comment");
			};
			var span = EditorUtil.getSafeTextSpan(comment.content, this, comment.links, this.parentController.workingCopy);
			dojox.xml.parser.replaceChildren(this._commentBodySPAN,span);
			this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(span, this.workItem)); // must be in dom
		},
		getAttachmentInfo: function(number) {
			return this.parentController.getAttachmentInfo(number);
		},
		expand: function() {
			this.isCollapsed = false;
			this._commentHeader.title = this.messages.titleCommentCollapse;
			if(this.isNew) {
				this.showNewComment();
//				this.commentTextArea.domNode.style.display="block";
//				this._workItemComment.className = "workItemNewComment";
			} else {
				this._commentBodySPAN.style.display="block";
				this._workItemComment.className = "workItemComment";
			}
			this._bindCommentHeader(this.comment);
		},
		collapse: function() {
			this.isCollapsed = true;
//			this._commentHeader.title = this.messages.titleCommentExpand;
			this._commentHeader.title = "";
			if(this.isNew) {
				this._workItemComment.className = "workItemCommentNewCollapsed";
				this._editorProxy.hideEditor();
			} else {
				this._workItemComment.className = "workItemCommentCollapsed";
				this._commentBodySPAN.style.display="none";
			}
			this._bindCommentHeader(this.comment);
		},
		handleClick: function(e) {
			if(!e || !e.target || this.isNew) return;
			if(e.target.tagName=="A") return;
			if(this.isCollapsed) {
				this.expand();
			} else {
				this.collapse();
			}
		},
		_handleDirtyingEvent: function(e) {

		},
		handleMouseOver: function(e) {
			if(this.isNew) return;
			this._commentHeader.className  = "workItemCommentHeaderMouseOver";
		},
		handleMouseOut: function(e) {
			if(this.isNew) return;
			this._commentHeader.className = "workItemCommentHeader";	
		},
		getValue: function() {
			return this._editorProxy.getValue();
		},
		setValue: function(value) {
			if(value == null) value = "";
			this._editorProxy.setValue(value);
			this.priorTextValue = "";
			this.newCommentValue = "";
		},
		clearTextValue: function() {
			this._editorProxy.setValue("");
			this.priorTextValue = "";
			this.newCommentValue = ""; 
		},
		showNewComment: function() {
			this.setNewTextEnabled(true);
			this._workItemComment.className = "workItemNewComment";
			this._commentHeader.style.display="";
			this._commentHeader.className = "workItemNewCommentHeader";
			this._newCommentArea.style.display="";
			this._showEditor(true);
//		"<span class=\"commentNumber\">&nbsp;</span><span class=\"commentDetails\"><B>New Comment</B></SPAN" ;

//			"<span class=\"newCommentHeader\"><B>New Comment</B></SPAN" ;

			this._commentHeaderSPAN = document.createElement("SPAN");
			this._commentHeaderSPAN.className="newCommentHeader";
			var b= document.createElement("B");
			b.appendChild(document.createTextNode(this.messages.buttonNewComment));
			this._commentHeaderSPAN.appendChild(b);
				
			// force the browser to show the save button
			if (this._saveCmdBottom) {
				try {
					var wasDisabled = this._saveCmdBottom.disabled;
					this._saveCmdBottom.disabled = false;
					this._saveCmdBottom.focus();
					this._saveCmdBottom.disabled = wasDisabled;
				} 
				catch (e) { /* OK, not visible */
				};
							}
			this._editorProxy.focus();
			
			// Watch the new comment value for changes
			this.newCommentValue = this.getValue();
			this._stopNewCommentInterval();
			this.newCommentInterval = setInterval(dojo.hitch(this, this._checkForNewCommentChanges), 200);
			
		},
		hideNewComment: function() {
			this._commentHeader.style.display="none";
			this._newCommentArea.style.display="none";
			this._showEditor(false);
			this._stopNewCommentInterval();
		},
		
		_showEditor: function(bShow){
			if(bShow){
				this._editorProxy.showEditor();
				
				if(this._readonlyNode){
					this._readonlyNode.style.display = "none";
				}
			}else{
				this._editorProxy.hideEditor();
				
				// if user value is not empty, show view only node
				var val = this.getValue();
				var bEmpty = (val == "");
				if(!bEmpty){
					if(!this._readonlyNode){
						this._readonlyNode = dojo.create("div", {
							tabIndex: 0,
							className: "readonly-text"
						}, this._workItemComment);
						this.connect(this._readonlyNode, "onfocus", dojo.hitch(this, function(){
							this.showNewComment();
						}));
					}else{
						this._readonlyNode.style.display = "";
					}
					this._readonlyNode.innerHTML = jazz.util.html.removeScript(val);
				}
				this.onEditorHidden(bEmpty);
			}
		},
		
		onEditorHidden: function(bEmpty){},
		
		setNewTextEnabled: function(isEnabled) {
			this._editorProxy.hideEditor();
			this._editorProxy.setProperty("enabled", isEnabled);
		},
		_checkForNewCommentChanges: function() {
			if(this.newCommentValue != this.getValue()) {
				this._stopNewCommentInterval();
				if(this.parentWidget){
					var e = {
						srcElement: this._editorProxy.getEditorNode()
					};
					this.parentWidget._handleChange(e);
				}
			}
		},
		_stopNewCommentInterval: function() {
			if(this.newCommentInterval) {
				window.clearInterval(this.newCommentInterval);	
			}
		},
		_handleTextBlur: function(e) {
			this._handleTextChange(e, /* notifyEvenIfUnchanged */ true);
		},
		_handleTextChange: function(e, notifyEvenIfUnchanged ) { // key up
			var isQuiet = this.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
			this._setWorkingCopyValue(isQuiet, notifyEvenIfUnchanged == true);
		},
		_setWorkingCopyValue: function(isQuiet, notifyEvenIfUnchanged){
			var value = this.getValue();
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
				attributeId: this.workingCopy.SYNTHETIC_COMMENT_ID,
				isInitialization: false,
				value: value,
				blankEqualsNull: true,
				isQuiet: isQuiet,
				notifyEvenIfUnchanged: notifyEvenIfUnchanged
			});
		}
	}
);

})();