/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserContent");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("jazz.ui.tree.IconItem");
dojo.require("jazz.ui.ModeledTree");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient = com.ibm.team.workitem.web.client.internal.QueryClient;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserContent",
	dijit._Widget,
{
	selection: true,
	defaultValue: null,
	suppressCrossRepoQueries: false,
	projectAreaItemId: null,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this._result = {};
		this._queryTreeArea = this.domNode;
		this.inherited(arguments);
		this.populatePicker(this.projectAreaItemId);
	},

	populatePicker: function(scopeItemId){
		if(this._result[scopeItemId]){
			if(this._result[scopeItemId] == "empty"){
				this._showEmpty();
			}else if(this._result[scopeItemId] != "pending"){
				this._tree = this._result[scopeItemId];
				this._showTree();
			}
			return;
		}
		this._showLoading();
		this._result[scopeItemId] = "pending";
		var self = this;
		var serviceObject = {
			_successCallback: function(/*array*/ scopedQueriesDTOs) {
				var hasQueries = false;
				for(var i=0; i<scopedQueriesDTOs.length; i++) {
					if(scopedQueriesDTOs[i].queries && scopedQueriesDTOs[i].queries.length > 0){
						hasQueries = true;
						break;
					}
				}
				if(hasQueries){
					// for team categories, store the team and the project, so the correct project can be restored if the viewlet is copied elsewhere
					var tree = self._createTree(scopedQueriesDTOs, scopeItemId);
					if(self._currentQuery == this.scopeItemId){
						self._tree = tree;
						self._showTree();
					}
					self._result[scopeItemId] = tree;
				}else{
					self._showEmpty();
					self._result[scopeItemId] = "empty";
				}
			},
			_errorCallback: function(error){
				self.onError(error);
			}
		};

		var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
		QueryClient.getScopedQueries(responseHandler, QueryClient.SCOPE_PERSONAL, null, false, true, scopeItemId, this.suppressCrossRepoQueries);
	},
	
	_createTree: function(scopedQueriesDTOs, queryCategory) {
		var dataStore = this._createDataStore(scopedQueriesDTOs, queryCategory);
		this._tree = new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserTree({
			model: dataStore,
			chooser: this,
			selection: this.selection
		});
		if(this.defaultValue){
			this._tree.expandTo(this.defaultValue, function(obj){
				return obj.itemId || obj.label;
			});
			this._tree.setSelection(this._tree.getItem(this.defaultValue));
		}
		return this._tree;
	},
	
	onError: function(error){
	},
	
	_createDataStore: function(scopedQueriesDTOs, queryCategory) {
		var model = [];
		var crossRepoCat, personalCat;
		if(!this._queriesMap){
			this._queriesMap = {};
		}
		for(var i = scopedQueriesDTOs.length-1; i >= 0; i--){
			var category = this._createCategory(scopedQueriesDTOs[i], queryCategory);
			if(category.children.length > 0){
				if(scopedQueriesDTOs[i].scope == QueryClient.SCOPE_CROSS_REPOSITORY){
					crossRepoCat = category;
				}else if(scopedQueriesDTOs[i].scope == QueryClient.SCOPE_PERSONAL){
					personalCat = category;
				}else{
					model.push(category);
				}
			}
		}
		model.reverse();
		// put cross repo category last
		if(crossRepoCat){
			model.push(crossRepoCat);
		}
		// put personal category first
		if(personalCat){
			model.splice(0,0,personalCat);
		}
		return model;
	},
	
	_createCategory: function (scopedQueriesDTO, queryCategory) {
		var category = {};
		category.label = this._getCategoryName(scopedQueriesDTO);
		category.iconClass = this._getCategoryClass(scopedQueriesDTO);
		var children = [];
		for (var i in scopedQueriesDTO.queries) {
			if (!this._queriesMap[scopedQueriesDTO.queries[i].itemId]) {
				children.push({	label: scopedQueriesDTO.queries[i].name, itemId: scopedQueriesDTO.queries[i].itemId, iconClass: "query" });
				scopedQueriesDTO.queries[i].queryCategory = queryCategory;
				this._queriesMap[scopedQueriesDTO.queries[i].itemId] = scopedQueriesDTO.queries[i];
			}
		}
		category.children = children.sort(function(a, b) {
			return a.name < b.name ? -1 : a.name > b.name ? 1 : 0;
		});
		return category;
	},
	
	_getCategoryName: function(scopedQueriesDTO){
		switch(scopedQueriesDTO.scope){
			case QueryClient.SCOPE_PERSONAL:
				return messages.chooserHeadingPersonal;
			case QueryClient.SCOPE_OTHER_CONTRIBUTORS:
				return messages.chooserHeadingShared;
			case QueryClient.SCOPE_TEAM_AREA:
				return dojo.string.substitute(messages.chooserHeadingTeam, [ scopedQueriesDTO.scopeName ]);
			case QueryClient.SCOPE_PROJECT_AREA:
				return messages.chooserHeadingProject;
			default:
				return scopedQueriesDTO.scopeName;
		}
	},

	_getCategoryClass: function(scopedQueriesDTO){
		if(scopedQueriesDTO.scope == QueryClient.SCOPE_TEAM_AREA){
			return "teamFolder";
		}
		return "simpleFolder";
	},
	
	_clearContent: function() {
		while(this._queryTreeArea.firstChild){
			this._queryTreeArea.removeChild(this._queryTreeArea.firstChild);
		}
	},
	
	_showTree: function() {
		this._clearContent();
		this._queryTreeArea.appendChild(this._tree.domNode);
		this.treeSelectionChanged();
		dojo.removeClass(this._queryTreeArea, "treeLoading");
		dojo.removeClass(this._queryTreeArea, "padded");
	},
	
	treeSelectionChanged: function(){},
	
	_showLoading: function() {
		this._clearContent();
		this._queryTreeArea.appendChild(document.createTextNode(messages.loading));
		dojo.addClass(this._queryTreeArea, "treeLoading");
		dojo.addClass(this._queryTreeArea, "padded");
	},
	
	_showEmpty: function(){
		this._clearContent();
		this._queryTreeArea.appendChild(document.createTextNode(messages.noQueriesFound));
		dojo.removeClass(this._queryTreeArea, "treeLoading");
		dojo.addClass(this._queryTreeArea, "padded");
	},
	
	getSelection: function(){
		var treeSelection = this._tree.getSelection();
		if(!treeSelection || treeSelection.length === 0){
			return null;
		}
		if(this.selection == "multi"){
			var result = [];
			for(var i = 0; i < treeSelection.length; i++){
				var query = this._queriesMap[this._tree.getSelection()[i].getId()];
				if(query){
					result.push(query);
				}
			}
			return result;
		}
		return this._queriesMap[this._tree.getSelection()[0].getId()];
	},
	
	destroy: function(){
		for(var i in this._result){
			if(this._result[i] != "pending" && this._result[i].destroy){
				this._result[i].destroy();
			}
		}
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserTree",
	jazz.ui.ModeledTree,
{
	// ported from com.ibm.team.workitem.viewlets.web.ui.internal.utils._WorkItemQueryChooser
	
	createTreeItem: function(obj){
		return new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserTreeItem(obj);
	},
	
	_doSelection: function(e, item){
		this.inherited(arguments);
		this.chooser.treeSelectionChanged();
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemQueryChooserTreeItem",
	jazz.ui.tree.IconItem,
{
	// ported from com.ibm.team.workitem.viewlets.web.ui.internal.utils._WorkItemQueryChooser
	getId: function(){
		return this.itemId || this.label;
	}
});

})(); 