/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that shows the Approvals Section
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="ApprovalsPart SectionPart">' +
						'<span dojoAttachPoint="_sectionMenuSPAN"></span>' +
						'<div dojoAttachPoint="_approvalsDIV"></div>' +
						'</div>',

		postCreate: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.layoutHints = {
				isWide: true,
				mustBeWide: true,
				mustBeMultiline: true
			}
				
			this._approvalWidgets = [];
			this._approvalsDIV.innerHTML = this.messages.busyLoading; /* innerHTML OK */
			this._approvalsDIV.style.color = "#CCCCCC";
			this._isInitialized = false;
			
			this.workingCopy.addListener({
				path: ["approvals"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyChange"
			});			
			

		},
		destroyWidget: function() {
			this.resetWidget();
			if(this.sectionMenuWidget) {
				this.sectionMenuWidget.destroy();
				this.sectionMenuWidget = null;
			}
			this.destroy();
		},
		resetWidget: function() {
			for(var i = 0; i< this._approvalWidgets.length; i++) {
				this._approvalWidgets[i].destroyWidget();
			}
			this._approvalWidgets = [];

		},
		_handleWorkingCopyChange: function(parms) {
			this._refreshApprovals();
		},
		/*
		 * Lazy load the tab when it is shown
		 */
		showWidget: function() {
			if(this._isInitialized) return;
			this._isInitialized = true;
			setTimeout(dojo.hitch(this, this.refresh), 100);
		},
		_getApprovals: function() {
			return this.workingCopy.getValue({
					path: ["approvals"],
					defaultValue: []
				});
		},
		_setWorkingCopyValue: function() {
			/* The approvals are updated in place, so just notify senders */
			this.workingCopy.setValue({
				forceNotify: true,
				caller: this,
				path: ["approvals"],
				value: 	this._getApprovals() // Current values
			});
		},
		/*
		 * Setup the page and read the states and types 
		 */
		refresh: function() {
			// The call to refresh can come
			// after the widget has been destroyed
			if(this._approvalsDIV == null) return;
			this.resetWidget();

			this._approvalsDIV.innerHTML = this.messages.busyLoading;
			this._approvalsDIV.style.color = "#CCCCCC";
			
			this._areTypesLoaded = false;
			this._areStatesLoaded = false;
			// Get the types from the server and update the child widget
			var fetchArgs = {
				serviceSelector: "getApprovalTypes",
				responseObject: this,
				responseSelector: "_typesLoaded",
				progressText: this.messages.busyLoadingApprovals
			};
			
			this.workingCopy.fetchSimpleInfo(fetchArgs);
			this._getApprovalStates();
			
		},
		_getApprovalStates: function(approvalTypes) {			
			// Get the state from the servers
			var fetchArgs = {
				serviceSelector: "getApprovalStates",
				responseObject: this,
				responseSelector: "_statesLoaded",
				progressText: this.messages.busyLoadingApprovals
			};
			
			this.workingCopy.fetchSimpleInfo(fetchArgs);
		},
		_statesLoaded: function(approvalStates) {
			this._approvalStates = approvalStates || [];
			this._areStatesLoaded = true;
			this._refreshApprovals();
		},
		_typesLoaded: function(approvalTypes) {
			this._approvalTypes = approvalTypes || [];
			this._areTypesLoaded = true;
			this._refreshApprovals();
		},
		_refreshApprovals: function() {
//			var start = new Date().getTime();
			if(!this._areTypesLoaded || !this._areStatesLoaded) return;

			this._approvalsDIV.style.color = "";

			// Can be called on initializtion, or on deleted approval
			var approvalStates = this._approvalStates || [];
			var approvalTypes = this._approvalTypes || [];
			this._approvalWidgets = [];
			
			this._createSectionMenu(approvalTypes);

			/*
			 * The states and types do not change, so 
			 * keep the actions as global.
			 */
			if (!this.stateActions || this.stateActions.length == 0) {
				this.stateActions = [];
				for (var i = 0; i < this._approvalStates.length; i++) {
					var state = approvalStates[i];
					var action = {
						isEnabled: true,
						receiverObject: this,
						receiverSelector: "changeApprovalState",
						label: state.label,
						iconSrc: state.iconUrl,
						value: state
					}
					this.stateActions.push(action);		
				}
			}

			if (!this.typeActions || this.typeActions.length == 0) {
				this.typeActions = [];
				for (var i = 0; i < this._approvalTypes.length; i++) {
					var type = this._approvalTypes[i];
					var action = {
						isEnabled: true,
						receiverObject: this,
						receiverSelector: "changeApprovalType",
						label: type.label,
						iconSrc: type.iconUrl,
						value: type
					}
					this.typeActions.push(action);
				}
			}
			
			var approvalsCount = 0;
			var deletedApprovalsCount = 0;
			var pendingApprovalsCount = 0;
			var approvedApprovalsCount = 0;
			var rejectedApprovalsCount = 0;
			
			var approvals = this._getApprovals();
			for (var i = 0; i < approvals.length; i++) {
				approvals[i].localId = i;
				approvalsCount++;
				if (approvals[i]._isDeleted) {
					if (!approvals[i]._isNew) {
						deletedApprovalsCount++;
					}
				}
				if (approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.pending") {
					pendingApprovalsCount++;
				}
				else {
					if (approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.rejected") {
						rejectedApprovalsCount++;
					}
					else {
						if (approvals[i].cumulativeState == "com.ibm.team.workitem.approvalState.approved") {
							approvedApprovalsCount++;
						}
					}
				}
			}			
			var prefix = "";

			var pendingMsg = pendingApprovalsCount ? dojo.string.substitute(this.messages.msgApprovalPending, [pendingApprovalsCount]) : "";
			prefix = pendingApprovalsCount ? prefix = ", " : "";

			var approvedMsg = approvedApprovalsCount ? prefix + dojo.string.substitute(this.messages.msgApprovalApproved, [approvedApprovalsCount]) : "";
			prefix = approvedApprovalsCount || pendingApprovalsCount ? prefix = ", " : "";
			
			var rejectedMsg = rejectedApprovalsCount ? prefix + dojo.string.substitute(this.messages.msgApprovalRejected, [rejectedApprovalsCount]) : "";

			prefix = pendingApprovalsCount || approvedApprovalsCount || rejectedApprovalsCount ?  prefix = " - " : "";
			
			if(this.sectionLayout) {
				if ((approvalsCount) == 0) {
					this.sectionLayout.sectionTitle.style.fontWeight = "";
					this.sectionLayout.sectionTitle.innerHTML = this.messages.headingApprovals;
					this._approvalsDIV.innerHTML = "<span style='color:#999999; margin-left: 7px; position:relative; top:10px'>" + this.messages.msgNoApprovals + "</span>";
					return;
				}
				else {
					this.sectionLayout.sectionTitle.style.fontWeight = "normal";
					this.sectionLayout.sectionTitle.innerHTML = "<b>" + dojo.string.substitute(this.messages.msgApprovalCount, [approvalsCount]) + "</b>" + prefix + pendingMsg + approvedMsg + rejectedMsg;
				}
			}
						
			dojox.xml.parser.removeChildren(this._approvalsDIV);
			
			if (!this._heading) {
				// Create a heading to show column titles
				var creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					parentWidget: this,
					isHeading: true,
					isFirst: true,
					className: "SectionListTableHeaderRow"
				};
				var approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);	
			} else {
				approvalWidget = this._heading;
			}
			this._approvalWidgets.push(approvalWidget);
			this._approvalsDIV.appendChild(approvalWidget.domNode);

			var listDiv = document.createElement("DIV");
			for(var i=approvals.length-1;i>=0;i--) {
				var isDeleted = approvals[i]._isDeleted || false;
//				if(this._approvals[i].widget && (isDeleted != this._approvals[i].widget._isDeleted)) {
//					this._approvals[i].widget.destroy();
//					this._approvals[i].widget = null;
//				}
//				if (!this._approvals[i].widget) {
				approvals[i]._approvalId = i;
				creationArgs = {
						isFirst: i == 0,
						parentWidget: this,
						parentController: this.parentController,
						workingCopy: this.workingCopy,
						approval: approvals[i],
						_isNew: approvals[i]._isNew || false,
						_isDeleted: isDeleted,
						approvalTypes: this._approvalTypes,
						approvalStates: this._approvalStates
					};
					
					approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);
					this._approvalWidgets.push(approvalWidget);
//					approvals[i].widget = approvalWidget; // TODO: Remove this connection
//				} else {
//					approvalWidget = this._approvals[i].widget;
//				}
				listDiv.appendChild(approvalWidget.domNode);
			}
			// Create a blank footer
			if (!this._footer) {
				var creationArgs = {
					parentWidget: this,
					parentController: this.parentController,
					isHeading: true,
					isFooter: true
				};
				approvalWidget = new com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalWidget(creationArgs);
				this._approvalWidgets.push(approvalWidget);
			} else {
				this._approvalWidgets.push(this.footer);
				approvalWidget = this._footer
			}
			listDiv.appendChild(approvalWidget.domNode);

			this._approvalsDIV.appendChild(listDiv);

		},
		_getTargetLabel: function() {
			var targetLabel = null;
			try {
			  targetLabel = this.parentController._editorParts.overview.allWidgets.target.getSelectedLabel();
			} catch(e) {} // ignore


			if(!targetLabel && this.workItem && this.workItem.attributes.target && this.workItem.attributes.target.label) {
				targetLabel = this.workItem.attributes.target.label;
			}
			
			if(!targetLabel || targetLabel.length == 0 || targetLabel.toLowerCase() == "unassigned" || targetLabel.toLowerCase() == "unknown") {
					targetLabel = "";
			}
			targetLabel = targetLabel ? targetLabel.replace(/ /g,"_") : "";
			return targetLabel;
		},
		// The default approval name is:
		//	Target ApprovalType (n)
		_getDefaultName: function(approval) {
			
			var targetLabel = this._getTargetLabel();
			
			
			var typeLabel = this._getTypeLabel(approval);
			
			var prefix = targetLabel.length > 0 ? targetLabel + "_" + typeLabel : typeLabel;
			
			// Make sure there are no other approvals with this label
			var result = prefix;
			var suffixCount = 2
			while(this._hasApprovalWithName(result, approval)) {
				result = prefix + dojo.string.substitute(this.messages.msgParenCount, [ suffixCount++ ]);
			}
			return result;
		},
		removeApproval: function(approval) {
			var approvals = this._getApprovals();
			for(var i=0;i<approvals.length; i++ ) {
				if(approvals[i].localId == approval.localId) {
//					if(this._approvals[i].widget)
//						this._approvals[i].widget.destroy();
					approvals.splice(i,1);
					break;
				}
			}
		},
		_getTypeLabel: function(approval){
			var typeId = approval.approvalType;
			var typeLabel = this.messages.msgApproval;
			for(var i=0;i<this._approvalTypes.length;i++ ) {
				if(typeId == this._approvalTypes[i].id)	{
					typeLabel = this._approvalTypes[i].label;
					break;
				}
			} 
			return typeLabel;
		},
		_hasApprovalWithName: function(name, approval) {
			var approvals = this._getApprovals();
			if(approvals.length ==  0 || !approval) return false;
			for(var j=0;j<approvals.length; j++) {
				if(approvals[j].name == name && approval.localId != approvals[j].localId) return true;
			}
			return false;
		},
		_getUpdatedDefaultName: function(approval) {
			try{
			
			var targetLabel = this._getTargetLabel();	
				
				var prefix = this._getDefaultName(approval);
				if(!approval || !approval.name) return prefix;
				var spaceIndex = approval.name.indexOf(" ");
				var colonIndex = approval.name.indexOf(":");
				if(spaceIndex == -1 && colonIndex == -1) {
					
					if(!this._isDefaultName(targetLabel, approval.name)) return null;
					
					return prefix;
				}
				
				var searchString;
				if (colonIndex == -1) {
					searchString = " ";
				}
				else {
					if (spaceIndex == -1) {
						searchString = ":";
					}
					else {
						searchString = spaceIndex < colonIndex ? " " : ":";
					}
				}
				
				var firstWord = approval.name.split(searchString)[0];
				
				// The first word must contain a type and target or we leave it alone
		
				if(!this._isDefaultName(targetLabel, firstWord)) return null;
				
				
				return approval.name.replace(firstWord,prefix);	
			} catch(e) {
				// ignore
			}
			return null;
		},
		_isDefaultName: function(targetLabel, name) {
				if(name.indexOf(targetLabel)== -1) return false;
				
				var found = false;
				for(var i=0;i<this._approvalTypes.length;i++ ) {
					var typeLabel = this._approvalTypes[i].label;
					if(name.indexOf(typeLabel) != -1) {
						found = true;
						break;	
					}
				}
				return found
		},
		/*
		 * Build the add approval menu  
		 */
		_createSectionMenu: function(approvalTypes) {
			if(this.workingCopy.isGuest()) return;
			if(this.sectionMenuWidget) {
				dojox.xml.parser.removeChildren(this._sectionMenuSPAN);
				this.sectionMenuWidget.destroy();
				this.sectionMenuWidget == null;
			}
			
			var addActions = [];
			for(var i=0;i<approvalTypes.length;i++){
				var type = approvalTypes[i];
				var action = {
					isEnabled: true,
					receiverObject: this,
					receiverSelector: "addNewApproval",
					label: type.label,
					iconSrc: type.iconUrl,
					value: type
				}
				addActions.push(action);
			}
			var actionNode = document.createElement("SPAN");
			var creationArgs = {
						parentWidget: this.parentWidget,
						actions: addActions,
						label: this.messages.buttonNew
					};
			var addActionWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget(creationArgs, actionNode);
			
			creationArgs = {
						parentWidget: this.parentWidget,
						actionWidgets: [ addActionWidget ]
					};
			
			this.sectionMenuWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget(creationArgs);
			this._sectionMenuSPAN.appendChild(this.sectionMenuWidget.domNode);
		},
		/*
		 * Add a new approval when the action is clicked from the section menu
		 */
		addNewApproval: function(action) {
			if(this.parentWidget && this.parentWidget.markDirty) {
					this.parentWidget.markDirty();
				} 

			if(this.parentWidget && this.parentWidget.parentController && this.parentWidget.parentController.markDirty) {
					this.parentWidget.parentController.markDirty();						
			}
			
			var approval = {
				approvalType: action.value.id,
				cumulativeState: "com.ibm.team.workitem.approvalState.pending",
//				dueDate: null,
//				id: null,
				_isNew: true
			}
			approval.name = this._getDefaultName(approval);
			this._getApprovals().push(approval);
			this._setWorkingCopyValue();
			this._refreshApprovals();
		},
		handleSaveComplete: function() {
			this._refreshApprovals();
		}
	}
);

})();
