/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.CategoryAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");

/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var EnumerationWidget = com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget;

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.CategoryAttributePart",
	EnumerationWidget,
	{
      constructor: function() {

	},
	postCreate: function()  {
		var values = this.values;
		var possibleValues = this._possibleValues || [];
		this._modifiers = this._modifiers || {};
		this._modifiers.allCategories = possibleValues;
		
		var currentValue = values ? values[0] : null;
	
		this._modifyCategoryValues({
			value: currentValue,
			possibleValues: possibleValues,
			categories: possibleValues
		});
		this.inherited(arguments);
		if(!(this.showGuessCategoryLink == false)) {
			this._createGuessCategoryLink();
		}
		//Set the id of the part so it can be labelled
		if(this.attributeId)
			this._select.setAttribute("id",this.attributeId);
	},
	onValueChanged: function(value, isInitialization) {
		return this._handleCategoryChange(this, value, isInitialization);
	},
	_handleCategoryChange: function(widget, value, isInitialization) {
			if(isInitialization) return;
					
			// always expand the categories after something is selected.
			if (value.id != "showall") {
				widget._select.title = widget._select.options[widget._select.selectedIndex].title;
			}
			
			// update the label for the selected item
			for (var i = 0; i < widget._modifiers.allCategories.length; i++) {
				if(!dojo.isIE && widget._modifiers.allCategories[i].id == value.id) {
					widget._modifiers.allCategories[i].label = widget._modifiers.allCategories[i].title.replace(/ \/ /g, "/");
				} else {
					widget._modifiers.allCategories[i].label = widget._modifiers.allCategories[i].shortLabel;
				}	
			}
			widget._possibleValues = widget._modifiers.allCategories;
			EditorUtil._bindValuesToSelect(widget._possibleValues, widget._select);
		},
	_createGuessCategoryLink: function() {
		
		// Make Category Combo a little smaller to leave room for guess-button
		dojo.addClass(this._select, "category-select-width");
		
		// Make outer Span for control visible
		this._enumGuessValue.style.display = "inline";
		
		// Create Control to Guess Category (Link with Image)
		var guessCategoryButton= document.createElement("a");
		guessCategoryButton.href = "#";
		guessCategoryButton.title = this.messages.labelGuessCategory;
		guessCategoryButton.className= "guessCategoryIcon";
		
		var guessCategoryImg= document.createElement("img");
		guessCategoryImg.src= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool.GUESS_CATEGORY;
		guessCategoryImg.alt = this.messages.labelGuessCategory;
		guessCategoryButton.appendChild(guessCategoryImg);
		this._enumGuessValue.appendChild(guessCategoryButton);
		
		// Guess Category on Click
		var self= this;
		this.connect(guessCategoryButton, "onclick", function(e) {
			dojo.stopEvent(e);
			guessCategoryButton.blur();
			self._onGuessCategory();
		});
	},
	setPossibleValues: function(possibleValues) {
			this._modifiers.allCategories = possibleValues;
			this._modifiers = this._modifiers || {};
			this._modifiers.allCategories = possibleValues;		
			var values = this.values;
			var firstPossibleValue = possibleValues.length > 0 ? possibleValues[0] : null;
			var currentValue = values ? values[0] : firstPossibleValue;
		
			this._modifyCategoryValues({
				value: currentValue,
				possibleValues: possibleValues,
				categories: possibleValues
			});
			
			this.inherited(arguments);
	},
	_onGuessCategory: function() {
		var itemSummary= dojo.trim(this.workingCopy.getValue({ path: "attributes.summary.content" }));
		var itemContent= dojo.trim(this.workingCopy.getValue({ path: "attributes.description.content" }));
		
		var itemComments= this.workingCopy.getValue({ path: "attributes.internalComments.items" });
		dojo.forEach(itemComments, function(entry) {
			itemContent += ("\n" + dojo.trim(entry.content));
		});
		
		var currentComment= this.workingCopy.getValue({ path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"] });
		if(currentComment) 
			itemContent += currentComment;
		
		var workItemId= this.workingCopy.getValue({ path: "itemId" });
		var projectAreaId= this.workingCopy.getValue({ path: "attributes.projectArea.id" });
		
		var serviceRequest = new ServiceRequest("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService", "postGuessCategory", 
			{ 
				projectAreaItemId : projectAreaId,
				workItemId : workItemId,
				itemSummary : itemSummary,
				itemContent : itemContent
			}
		);
		
		var self = this;
		var op = this.workingCopy.createOperation({
	        message: self.messages.labelGuessingCategory,
	        id: "guessingCategory"
	  });
			
    var handlerObj = {
	    success: function(result){
				self.workingCopy.finishOperation(op);
				if (result != null) {
					if(result.id != self.getValue()) {
						self.setAttributeValue(result, /* setSelectValue */ true);
					}
					self._select.focus();
				}
				else {
					alert(self.messages.labelGuessCategoryNoResult);
				}
	    },
	    failure: function(errorObj){
	    	if(errorObj) {
	        	var message = errorObj.message || self.messages.labelGuessCategoryNoResult;
	        	alert(message);
	    	}
			self.workingCopy.finishOperation(op);
	    }
    }
    
    var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
	this.workingCopy.startOperation(op);
    TeamServerClient.invokeService(serviceRequest, responseHandler);
	}
});
})();
