/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.EstimateAndCorrectionAttributePart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that provides the presentation and value
 * processing for Duration, presented in 
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var TOOLTIP;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.EstimateAndCorrectionAttributePart",
	[dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart],
	{
		
		templateString: '<div class="DurationAttributePart NarrowWidget EstimateAndCorrectionAttributePart">' +
						'</div>',

	constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			TOOLTIP= this.messages.msgEstimateValidation;
		},
		postCreate: function() {
			/*
			 * 
			 * values[0] is the Duration, values[1] is correctedEstimate
			 *    label
			 *    validationRegex
			 *    id
			 */
			
			this.estimate = this.workingCopy.getValue({ path: ["attributes", "duration"] });
			this.workingCopy.addListener({
				path: ["attributes", "duration"],
				event: "onchange",
				listener: this,
				functionName: "_handleEstimateAttributeChange"
			});

			this.estimate._attributeId = "duration";
					
			this.correction = this.workingCopy.getValue({ path: ["attributes", "correctedEstimate"] });
			this.workingCopy.addListener({
					path: ["attributes", "correctedEstimate"],
					event: "onchange",
					listener: this,
					functionName: "_handleCorrectionAttributeChange"
				});
			this.correction._attributeId = "correctedEstimate";
						
			var creationArgs = {
						parentWidget: this,
						onValueChanged: "onEstimateValueChanged",
						parentController: this.parentController,
						presentation: {
							attribute: {
								attributeTypeId: "ShortString"
							}
						},
						selection:  [ {
							id: this.estimate.label
							} 
						 ] ,
						 converter: this._TextWidget_durationConverter,
						 converterParms: {
						 		toolTip: TOOLTIP,
								value: this.estimate
						 },
						 statusIconContainerClass: "StatusIconEstimateTextContainer"
					};
					
			this.estimateWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget(creationArgs);

			this.domNode.appendChild(this.estimateWidget.domNode);
			
			var cDiv = document.createElement("DIV");
			
			var cSpan = document.createElement("SPAN");
			cSpan.appendChild(document.createTextNode(this.messages.labelCorrection));
			
			cDiv.appendChild(cSpan);
			
			creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						onValueChanged: "onCorrectionValueChanged",
						presentation: {
							attribute: {
								attributeTypeId: "ShortString"
							}
						},
						selection:  [ {
							id: this.correction.label
							} 
						 ] ,
						 converter: this._TextWidget_durationConverter,
						 converterParms: {
						 		toolTip: TOOLTIP, 
								value: this.correction
						 },
						 statusIconContainerClass: "StatusIconCorrectionTextContainer"
					};
					
			this.correctionWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget(creationArgs);

			cDiv.className = "EstimateCorrectionDiv";
			dojo.addClass(this.estimateWidget.domNode,"EstimateWidget");
			this.estimateWidget._textInput.className = "EstimateInput";
			cSpan.className = "EstimateCorrectionSpan";

			if(this.workingCopy.isGuest()) {
				dojo.addClass(this.correctionWidget._textInput, "GuestField");
				dojo.addClass(this.estimateWidget._textInput, "GuestField");
			}

			cDiv.appendChild(this.correctionWidget.domNode);
			this.domNode.appendChild(cDiv);	
		},
		_handleEstimateAttributeChange: function(parms) {
			var value = parms.attributeValue || ""; /* content */
			this.estimateWidget._textInput.value = value;
			if(parms.validationStatus)
				this.estimateWidget.setStatus(parms.validationStatus);		
		},
		_handleCorrectionAttributeChange: function(parms) {
			var value = parms.attributeValue || ""; /* content */
			this.correctionWidget._textInput.value = value;
			if(parms.validationStatus)
				this.correctionWidget.setStatus(parms.validationStatus);
		},
		onCorrectionValueChanged: function(e){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", "correctedEstimate"],
				savedAttributePath: ["attributes", "correctedEstimate"],
				attributeId: "correctedEstimate",
				isInitialization: false,
				value: this.correctionWidget._textInput.value
			});
			var status= this.workingCopy._validate("correctedEstimate");
			this.correctionWidget.setStatus(status);
		},
		onEstimateValueChanged: function(e){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", "duration"],
				savedAttributePath: ["attributes", "duration"],
				attributeId: "duration",
				isInitialization: false,
				value:  this.estimateWidget._textInput.value
			});
			var status= this.workingCopy._validate("duration");
			this.estimateWidget.setStatus(status);
		},
		destroy: function() {
			if(this.estimateWidget.destroy) this.estimateWidget.destroy();
			if(this.correctionWidget.destroy) this.correctionWidget.destroy();
			this.inherited(arguments);
		},
		setReadOnly: function(isReadOnly) {
			this.estimateWidget.setReadOnly(isReadOnly);
			this.correctionWidget.setReadOnly(isReadOnly);
		},
	     getValue: function() {
			try {
				var value1 = this.estimateWidget._textInput.value;
				var value2 = this.correctionWidget._textInput.value;
				return [ value1 , value2 ];
			} 
			catch (e) {
				console.debug("Error getting estimate and correction values." );
			}
			return []
		 },
	//* This runs in the context of TextWidget
		//
		_TextWidget_durationConverter: function (value, parms, isGettingValidValue) {
			return this.parentWidget._durationConverter(value, parms, isGettingValidValue);
		},
		_durationConverter: function (value, parms, isGettingValidValue) {
			var valid = false;
			if(value && parms.value && parms.value.validationRegex) {
				try{
					
					var regex = new RegExp("^" + parms.value.validationRegex + "$");
 					valid = value.search(regex) != -1;
//				valid = value.search(/^(([0-9]+(\.[0-9])?|\.[0-9]+)\s*(milliseconds|millisecond|millisecs|millisec|minutes|seconds|minute|second|weeks|hours|days|week|hour|mins|secs|day|wks|hrs|min|sec|ds|wk|ws|ms|d|w|h|m|s)\s*)*$/) != -1;
				} catch(e) {
					console.debug(dojo.string.substitute(this.messages.errorInvalidRegex, [this.validationRegex ]));
					valid = true;
				}

			}
			if(isGettingValidValue) {
				if(!value || value.length == 0) {
					return "";
				}				
				return value;
			}
			return {isCustom: true, isValid: valid };
		}
	}
);

})();
