/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.util.html");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemLinkTypes");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.require("com.ibm.team.calm.foundation.web.calmdialogs.RemoteItemPicker");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.calm.foundation.web.calmdialogs.ServiceProviderSelectionDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * This is a wrapper widget that shows the Links Section
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var SelectWorkItemDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
var ServiceProviderSelectionDialog = com.ibm.team.calm.foundation.web.calmdialogs.ServiceProviderSelectionDialog;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var WorkItemLinkTypes = com.ibm.team.workitem.web.client.internal.WorkItemLinkTypes;

// TODO eventually get this stuff from the server so we don't have duplicated
// model logic on the client
//

var linkTypeEditors = {
	    "WorkItem": "_addWorkItemLink",
		"_calmLink": "_addCalmLink",
		"_defaultUri": "_addUriLink"
}

var EDITABLE_LINK_TYPES =  { // Coordinate changes with constructor
				related: {  // Default
					key: "related",
					id: "com.ibm.team.workitem.linktype.relatedworkitem",
					isSingleValued: false,
					isSource: false
				},
				duplicateOf: {  // Used to generate this kind of link type
					key: "duplicateOf",
					id: "com.ibm.team.workitem.linktype.duplicateworkitem",
					isSingleValued: true,
					isSource: true
				}
			};
			
var headingColor = "#666668";
var columnLayout = {
	name: {
		style: {
			fontWeight: "bold",
			height: "1.2em",
			margin: "0 0 0 10px",
			padding: "3px 0 0 0",
			color: headingColor,
			width: "100%"
		}
	}
}

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="LinksPart SectionPart">' +
						'<span dojoAttachPoint="_sectionMenuSPAN"></span>' +
						'<div dojoAttachPoint="_contentNode"></div>' +
						'<div dojoAttachPoint="_noLinksNode" style="color:#999999;display:none;margin-left:5px">${_noLinksMessage}</div>' +
						'<BR></div>',
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this._noLinksMessage = this.messages.labelNoLinks;
			EDITABLE_LINK_TYPES.related.displayName = this.messages.labelRelated;
			EDITABLE_LINK_TYPES.duplicateOf.displayName= this.messages.labelDuplicateOf;
			columnLayout.name.label=  this.messages.labelLink;
		},
		postCreate: function() {
			this.layoutHints = {
				isWide: true,
				mustBeWide: false,
				mustBeMultiline: true
			}
			this._childWidgets = [];
			this.isLinkManager = true; //marker
			this._isInitialized = false;
			this._workItem = this.parentController._workItem;
						
			this.workingCopy.addListener({
				path: ["linkTypes"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyChange"
			});			
			
			var propertyMap = {};
			this.workingCopy.getPropertyMap(this.presentation,propertyMap);
			var linkTypeString = propertyMap["linkTypeFilter"];
			if(linkTypeString) {				
				// The array is in the form of a string "[ linkId1, linkId2 ]";
				linkTypeString = linkTypeString.substring(1, linkTypeString.length-1);
				linkTypeString = linkTypeString.replace(/ /g, "");
				this.linkTypeFilter = linkTypeString.split(",");
			}
	
			this._initializeEditableLinkTypes();
			
		},
		_initializeEditableLinkTypes: function() {
		
		var editableLinkTypes = this.workingCopy.getEditableLinkTypes();
		if(editableLinkTypes) {
			this._initializeEditableLinkTypesSuccess(editableLinkTypes);
			return;
		}
        var responseHandler = new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_initializeEditableLinkTypesSuccess", "_initializeEditableLinkTypesError");    
			var args = {
                projectAreaItemId: this.workingCopy.getProjectAreaItemId()
            }
            com.ibm.team.workitem.web.client.internal.WorkItemClient.getLinkTypes(responseHandler, args);
		},
		_initializeEditableLinkTypesError: function(errorObj) {
			alert(this.messages.errorFailedToLoadEditableLinkTypes + ". " + errorObj.message);
		},
		arrayContains: function(haystack, needle) {
			  var i = haystack.length;
			  while (i--) {
			    if (haystack[i] == needle) {
			      return true;
			    }
			  }
			  return false;
		},
		_initializeEditableLinkTypesSuccess: function(linkTypes) {
			this._editableLinkTypes = [];
			
			if(linkTypes && linkTypes.length > 0 && this.linkTypeFilter && this.linkTypeFilter.length > 0) {
				for(var i=0;i<linkTypes.length;i++) {
					if(this.arrayContains(this.linkTypeFilter, linkTypes[i].id)) {
						this._editableLinkTypes.push(linkTypes[i]);
					}
				}
			} else {
				this._editableLinkTypes = linkTypes || [];
			}
			
			this.workingCopy.setEditableLinkTypes(linkTypes || []);
			if(this._isRefreshPending) {
				this.refresh();
			}
		},
		_getWorkingCopyLinkTypes: function () {
			return this.workingCopy.getValue({
				path: ["linkTypes"],
				defaultValue: []
			});
		},
		_handleWorkingCopyChange: function(parms) {
			this.refresh();
		},
		_setWorkingCopyValue: function() {
			/* The linktypes are updated in place, so just notify senders */
			this.workingCopy.setValue({
				caller: this,
				path: ["linkTypes"],
				value: 	this._getWorkingCopyLinkTypes() // Current values
			});
		},
		destroyWidget: function() {
			this.resetWidget();
			this.destroy();
		},
		resetWidget: function() {
			for(var i = 0; i< this._childWidgets.length; i++) {
				if(this._childWidgets[i].destroyWidget)
					this._childWidgets[i].destroyWidget();
				this._childWidgets[i] = null;
			}

			this._childWidgets = [];
			if(this._contentNode) dojox.xml.parser.removeChildren(this._contentNode);
		},
		/*
		 * Load the tab when it is shown
		 */
		showWidget: function() {
			if(this._isInitialized) return;
			this._isInitialized = true;
			this.refresh();
		},
		
		/*
		 * Setup the page and read the states and types 
		 */
		refresh: function() {
			if(!this._editableLinkTypes) {
				this._isRefreshPending = true;
				return; // will get refreshed when the link types are loaded;
			}		
			this._isRefreshPending = false;
			var defaultAction;
			if (!this.workingCopy.isGuest()) {
				var actions = [];
				for (var i=0;i<this._editableLinkTypes.length;i++) {
				    var linkType= this._editableLinkTypes[i];
					if ((!linkType.itemType || linkTypeEditors[linkType.itemType] ) && linkType.isUserWriteable && linkType.endpointId != "successor") {
						if (linkType.itemType == null) {
							var calmServiceProviders= linkType.calmServiceProviders;
		    				if (dojo.isArray(calmServiceProviders) && calmServiceProviders.length > 0) {
								linkType.itemType = "_calmLink";
		    				} else {
								linkType.itemType = "_defaultUri";
							}
						} else {
							if (linkType.endpointId == "related") {
								defaultAction = linkType;
							}
						}
						actions.push(linkType);
					}
				}
				actions = actions.sort(this._sortLinkTypes);
				
				// insert separators by grouping items having the same projectLinkType or id
				var actions2 = [];
				var prevLinkId = null, prevProjectLinkType = null;
				dojo.forEach(actions, function(action, idx){
					if(idx > 0 && (
						(action.projectLinkType && action.projectLinkType != prevProjectLinkType) ||
						(!action.projectLinkType && action.id && action.id != prevLinkId)
						)){
						actions2.push(WidgetUtil.SEPARATOR);
					}
					actions2.push(action);
					prevLinkId = action.id;
					prevProjectLinkType = action.projectLinkType;
				});
				
				var actionClass = com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget;
				var menuClass = com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget;
				this._sectionMenuWidget = WidgetUtil.createSectionMenu({
					domNode: this._sectionMenuSPAN,
					parentWidget: this,
					defaultAction: defaultAction || false,
					menuClass: menuClass,
					actionClass: actionClass,
					actionSelector: "_handleAddLinkButtonPressed",
					actionLabelSelector: "displayName",
					menuLabel: this.messages.buttonAdd,
					actions: actions2
				});
				this._childWidgets.push(this._sectionMenuWidget);
			}
			this.resetWidget();		
			this._noLinksNode.style.display = "none";
			this._contentNode.style.display = "";
//			this._markDirtyIfDifferent();
			var creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						isHeading: false,
						isFirst: true,
						noPadding: true
					};
			var listWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget(creationArgs);
			var linkTypes = this._getWorkingCopyLinkTypes();
			if(linkTypes && linkTypes.length > 0 && this.linkTypeFilter && this.linkTypeFilter.length > 0) {
				var filteredLinkTypes = [];
				for(var i=0;i<linkTypes.length;i++) {
					if(this.arrayContains(this.linkTypeFilter, linkTypes[i].id)) {
						filteredLinkTypes.push(linkTypes[i]);
					}
				}
			} else {
				filteredLinkTypes = linkTypes || [];
			}
			linkTypes = filteredLinkTypes;
			
			var rowItems = [];
			for (var i = 0; i < linkTypes.length; i++) {
				// skip attachments
				if(linkTypes[i].id == "com.ibm.team.workitem.linktype.attachment" || linkTypes[i].isInternal) continue;
				
				var isLast = i == linkTypes.length;
				if (linkTypes[i] && this._hasLinks(linkTypes[i], true)) {
					var item = {};
					item.columnWidgets = [this._getRow(listWidget, linkTypes[i], /* isFirst */ i == 0, isLast) ];
					item.value  = linkTypes[i];
					item.value.isNotHighlighted = true;
					item.showDelete = false;
					rowItems.push(item);
					this._addLinkItems( linkTypes[i],rowItems, item, listWidget);
				}
			}
			listWidget.setRows(
					{
						itemList: rowItems,
						deleteAction: {
								selector: "_handleDeleteItemPressed",
								title: this.messages.titleLinkDelete
						},
						restoreAction: {
								selector: "_handleRestoreItemPressed",
								title: this.messages.titleLinkRestore
						}
					}
					)
			this._contentNode.appendChild(listWidget.domNode);
			if(rowItems.length == 0) {
				this._noLinksNode.style.display = "";
				this._contentNode.style.display = "none";
			}
			this._childWidgets.push(listWidget);	
			if( this.workingCopy.isGuest() && listWidget._tableWidget && listWidget._tableWidget._table) 
				listWidget._tableWidget._table.style.border="";
			this.parentController.updateHeaderValidation();

		},
		_sortLinkTypes: function(a, b) {
			var weightA= 0;
			var weightB= 0;
			
			if (a.projectLinkType)
				weightA++;
				
			if (b.projectLinkType)
				weightB++;
				
			if (weightA == weightB) {
				if (WorkItemLinkTypes.RELATED_WORK_ITEM == a.id) weightB++;
				if (WorkItemLinkTypes.RELATED_WORK_ITEM == b.id) weightA++;
			}
			if (weightA == weightB) {
				if (WorkItemLinkTypes.DUPLICATE_WORK_ITEM == a.id) weightB++;
				if (WorkItemLinkTypes.DUPLICATE_WORK_ITEM == b.id) weightA++;
			}				
			if (weightA == weightB) {
				if (a.id < b.id) weightB++;
				if (b.id < a.id) weightA++;
			} 	
			if (weightA == weightB) {
				(a.displayName > b.displayName) ? weightA++ : weightB++
			}
			return weightA - weightB;
		},
		_hasLinks: function(linkType, showDeleted) {
			if(!linkType || !linkType.linkDTOs || linkType.linkDTOs.length == 0) return false;
			for (var i = 0; i < linkType.linkDTOs.length; i++) {
				if(showDeleted || !linkType.linkDTOs[i]._isDeleted) return true;
			}
			return false;
		},
		_addLinkItems: function(linkType, rowItems, item, listWidget) {
			var links = item.value.linkDTOs
			if(!links || links.length == 0) return;
			var target, targetType, id, targetIconUrl;
			for (var j = 0; j < links.length; j++) {
				targetIconUrl = null;
				var link = links[j];
				if(!link) continue;
				link.linkTypeItemType = linkType.itemType;
				link.linkTypeId = linkType.id;
				link.linkTypeIsUserDeleteable = linkType.isUserDeleteable;
				link.linkTypeIsSource = linkType.isSource;
//				if(link._isDeleted) continue;
				var node = document.createElement("SPAN");
				node.className = "SectionListChildRow SectionListLink"
				target = link.target;
				id = null;
				var stateGroup = -1;
  				targetType = target ? target._eQualifiedClassName : null;
				if (target && target.attributes) {
					for (var i = 0; i < target.attributes.length; i++) {
						if (target.attributes[i].key == "id") {
							id = target.attributes[i].value.id;
						}
						if (target.attributes[i].key == "workItemType") {
							targetIconUrl = target.attributes[i].value.iconUrl;
						}
						if (target.attributes[i].key == "stateGroup") {
							stateGroup = target.attributes[i].value.id;
						}
					}
				}
				if(!targetIconUrl) {
					targetIconUrl = linkType.iconUrl;
				}
				var anchor;
				
				// if no comment, make the link the comment
				if((!link.comment || link.comment.length == 0)) {
					if(link.url) {
						link.comment = link.url;
					} else {
						if(link.linkTypeItemType) {
							link.comment = link.linkTypeItemType;	
						} else {
							if(link.linkTypeId) {
								link.comment = link.linkTypeId
							} else {
								link.comment = this.messages.labelLink;
							}
						}
					}
				}
								
				var img = document.createElement("IMG");
				img.src = ImagePool.LIST_ARROW;
				img.className = "linkArrow";
				node.appendChild(img);
				var targetIconNode;
				if(targetIconUrl) {					
					targetIconNode = document.createElement("IMG");
					targetIconNode.src = targetIconUrl;
					targetIconNode.className = "iconUrl";
					node.appendChild(targetIconNode);
					/*
					 * Capture the image append child to instead set the src
					 * See foundation Enhancement 110914.
					 */
					var _appendChild = function(args) {
						// Running in the context of the targetIconNode anchor
						this.src = args.src;
					}
					targetIconNode.appendChild = _appendChild;
				}
				
				anchor = document.createElement("a");
		  		anchor.__jazzValue = link;
		  		if(item.value && item.value.displayName && link.comment) {		  			
		  			dojo.attr(anchor, "aria-label", dojo.string.substitute(this.messages.ariaLabelLink, [ item.value.displayName , link.comment ]));
		  		}
		  		if(link.url) {
					var resourceLinkArgs= {
							iconNode: targetIconNode,
							uri: link.url, // canonical url of the work item
							presentationUri: link.url, // may be massaged in WorkItemProxy 
							label: jazz.util.html.escape(link.comment)
						};
					this._childWidgets.push(this.workingCopy.applyResourceLinkToAnchor(resourceLinkArgs, anchor));			  			
		  		} else {
		  		  // This path occurs when a link is added from the Web UI, before save
		  		   anchor.appendChild(document.createTextNode(link.comment));
		  			if(link.linkTypeItemType == "WorkItem" && link.id ) {
		  				anchor.href = "#action=com.ibm.team.workitem.viewWorkItem&id=" + link.id;
		  			}
		  		}
		  						
				this.connect(anchor, "onfocus", function (e) {
					if (anchor.parentNode && anchor.parentNode.parentNode && anchor.parentNode.parentNode.parentNode) {
						listWidget.setRowBackground(anchor.parentNode.parentNode.parentNode, true);	
					}	
				});
				this.connect(anchor, "onblur", function () {
					if (anchor.parentNode && anchor.parentNode.parentNode && anchor.parentNode.parentNode.parentNode) {
						listWidget.setRowBackground(anchor.parentNode.parentNode.parentNode, false);	
					}	
				});
				
								
				if(stateGroup == 2) {
					dojo.addClass(anchor,"Strikeout");	
				};
				
				node.appendChild(anchor);
				
				var fakeWidget = {
					domNode: node
				};
				
				var linkItem = {};

				linkItem.columnWidgets = [ fakeWidget ];
				linkItem.value  = link;
				
				var itemId = link.itemId;
				if(!itemId && target && target.attributes) {
	  				var id;
				  	for(var i=0;i<target.attributes.length;i++) {
				  		 if(target.attributes[i].key=="itemId") {
					 		itemId = target.attributes[i].value.itemId;
							break;
				 		}
			  		}
				}
				
				var typeIdKey = link.linkTypeId.replace(/\./g, "_").toLowerCase();
				var canBeDeleted = link.linkTypeIsUserDeleteable != false; 
				
				linkItem.itemId = itemId;
				linkItem.showDelete =canBeDeleted; 
				rowItems.push(linkItem);
			}
			
		},
		addDuplicateLink: function() {
			this._openAddWorkItemDialog({
				label: this.messages.labelDuplicateOf,
				value: EDITABLE_LINK_TYPES.duplicateOf
			});
		}, 
		removeDuplicateLink: function() {
				var linkTypes = this._getWorkingCopyLinkTypes();
				if(!linkTypes) return;
				for(var i = 0;i<linkTypes.length;i++ ){
					if(linkTypes[i].id == "com.ibm.team.workitem.linktype.duplicateworkitem" && linkTypes[i].isSource ) {
						for(var j=0;j<linkTypes[i].linkDTOs.length; j++) {
							if(!linkTypes[i].linkDTOs[j]._isDeleted)
								linkTypes[i].linkDTOs[j]._isDeleted = true;
						} 
					}
				}
				this.refresh();
		},
		_findInArray: function(item, array) {
			for(var i=0;i<array.length;i++){
				if(item.itemId == array[i].itemId) return true;
			}
			return false;
		},
//		_markDirtyIfDifferent: function() {
//			if(this.workingCopy.isChanged()){
//				this.parentWidget.parentController.markDirty();
//			}
//		},
		_getRow: function(listWidget, linkType, isFirst, isLast) {
			var layout = WidgetUtil.mergeObjects({}, columnLayout.name);
			layout.label = linkType.displayName || this.messages.labelUnknown;
			layout.iconUrl = linkType.iconUrl;
			layout.isFirst = isFirst;
			layout.isLast = isLast;
			return listWidget.getLabelWidget(layout);
		},
		_findLinkType: function(searchLinkType) {
			var linkType;
			var found = false;	
			var linkTypes = this._getWorkingCopyLinkTypes();		
			for(var i=0;i<linkTypes.length;i++){
				if(linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
					found = true;
					linkType =linkTypes[i]; 
					break;
				}
			}

			if (!found) {
				linkTypes = this._editableLinkTypes;
				for (var i = 0; i < linkTypes.length; i++) {
					if (linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
						found = true;
						linkType = dojo.clone(linkTypes[i]);
						this._getWorkingCopyLinkTypes().push(linkType);
						break;
					}
				}
			}

			if(!found) { // Should never happen
				linkType =  EDITABLE_LINK_TYPES.related;
				linkType.itemType = "WorkItem";
				linkType._isNew = true;
				this._getWorkingCopyLinkTypes().push(linkType);
			}
			
			if(!linkType.linkDTOs) {
				linkType.linkDTOs = [];
			}
			return linkType;
		},
		_addItemToLinks: function (linkType, item) {
			var links = linkType.linkDTOs
			var itemId = item.workItemItemId ? item.workItemItemId : item.itemId;
			var found = false;
			for(var i=0;i<links.length;i++){
				if(links[i].itemId == itemId) {
					found = true;
					links[i]._isDeleted = false;
					break;
				}
			}
			
			if (!found) {
				var link = {
					_isNew: true,
					id: item.id,
					itemId: itemId,
					comment: item._linkComment
				}
				var idAttr = {
					id: item.id
				}
				var target = {
					_eQualifiedClassName: "com.ibm.team.workitem.rest.dto:WorkItemDTO",
					attributes: [
							{ key: "id",
							  value: idAttr } ]
				}
				link.target = target;
				
				if (linkType.isSingleValued) {
					// mark all the other links deleted
					var newLinks = [];
					for (var i = 0; i < links.length; i++) {
						if (links[i].itemId != itemId) {
							if(!links[i]._isNew) {
								newLinks.push(links[i]);
							}
							links[i]._isDeleted = true;
						}
					}
					links = newLinks;
					linkType.linkDTOs = links; 
				}
				
				links.push(link);
			}	
		},
		hasDuplicateOfLink: function() {
				// If the work item has a duplicate link
				// and it hasn't been deleted
				// or it has been added
				// then it is ok, otherwise, show a message
				
				// Does the work item have duplicate links
				var currentDuplicates = [];
				var linkTypes = this._getWorkingCopyLinkTypes();
				if(!linkTypes) return false;
				for(var i = 0;i<linkTypes.length;i++ ){
					if(linkTypes[i].id == "com.ibm.team.workitem.linktype.duplicateworkitem" && linkTypes[i].isSource ) {
						for(var j=0;j<linkTypes[i].linkDTOs.length; j++) {
							if(!linkTypes[i].linkDTOs[j]._isDeleted)
								currentDuplicates.push(linkTypes[i].linkDTOs[j].itemId);
						} 
					}
				}
				
//				var foundNewDuplicate = false;
//				var saveCommands = this.parentController._getSaveCommands();
//				if(saveCommands.updateLinks) {
//					for(var i=0;i<saveCommands.updateLinks.length;i++) {
//						var cmd = saveCommands.updateLinks[i];
//						if(cmd.cmd == "addLink" && cmd.end == "source" && cmd.type=="com.ibm.team.workitem.linktype.duplicateworkitem") {
//							foundNewDuplicate = true;
//						}
//						if(cmd.cmd == "removeLink" && cmd.end == "source" && cmd.type=="com.ibm.team.workitem.linktype.duplicateworkitem") {
//							for(var j=0;j<currentDuplicates.length;j++) {
//								if(currentDuplicates[j] == cmd.itemId) {
//									currentDuplicates.splice(j);
//									break;
//								}
//							}
//						}
//					}
//				}
//				
//			 return currentDuplicates.length > 0 || foundNewDuplicate;

			return currentDuplicates.length > 0
		},	
		_handleDeleteItemPressed: function(e) {
			if(e && e.target && e.target.parentWidget && e.target.parentWidget._deleteLink) {
				e.target.parentWidget._deleteLink(e.target);
			}
		},
		_handleRestoreItemPressed: function(e) {
			if (e && e.target && e.target.parentWidget && e.target.parentWidget._restoreLink) {
				e.target.parentWidget._restoreLink(e.target);
			}
		},
		_getLinkTypeSpec: function(actionValue) {
		    if(actionValue.isSource) return actionValue; // already a link
			// Find or add the link type
			var linkType = {
			    	itemType: actionValue.linkTypeItemType,
					id: actionValue.linkTypeId,
					isUserDeleteable: actionValue.linkTypeIsUserDeleteable,
					isSource: actionValue.linkTypeIsSource
			}
			return linkType
		},
		_deleteLink: function(action) {
			var itemToDelete = action.value;
			var linkTypeSpec = this._getLinkTypeSpec(action.value);
			var linkType = this._findLinkType(linkTypeSpec);
			var links = linkType.linkDTOs; 
			var wasDeleted = false;
			if(itemToDelete.itemId) {
				for(var i=0;i<links.length;i++){
					if(links[i].itemId == itemToDelete.itemId) {
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						wasDeleted = true;
						break;
					}
				}
			} else {
				if(itemToDelete.url && itemToDelete.comment) {
					for(var i=0;i<links.length;i++){
					if(!links[i]._isDeleted && links[i].url && links[i].comment  && links[i].url == itemToDelete.url && links[i].comment == itemToDelete.comment) {
						wasDeleted = true;
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						break;
					}
				}	
				}		
			}
			
			if(wasDeleted) {
				this._parentMarkDirty();
			}
			this._setWorkingCopyValue();
			this.refresh();
		},
		_parentMarkDirty: function() {
			if(this.parentWidget && this.parentWidget.markDirty) {
					this.parentWidget.markDirty();
				} 

			if(this.parentWidget && this.parentWidget.parentController && this.parentWidget.parentController.markDirty) {
						this.parentWidget.parentController.markDirty();
				}
		},
		_restoreLink: function(action) {
			var itemToDelete = action.value;
			
			// Find or add the link type
			var linkTypeSpec = this._getLinkTypeSpec(action.value);
			var linkType = this._findLinkType(linkTypeSpec);
			var links = linkType.linkDTOs; 

			if(itemToDelete.itemId) {
				for(var i=0;i<links.length;i++){
					if(links[i].itemId == itemToDelete.itemId) {
						links[i]._isDeleted = false;
						break;
					}
				}
			} else {
				if(itemToDelete.url && itemToDelete.comment) {
					for(var i=0;i<links.length;i++){
					if(links[i]._isDeleted && links[i].url && links[i].comment  && links[i].url == itemToDelete.url && links[i].comment == itemToDelete.comment) {
						links[i]._isDeleted = false;
						break;
					}
				}	
				}		
			}
			this._setWorkingCopyValue();
			this.refresh();
		},
		_handleAddLinkButtonPressed: function(action) {
			if(!action || !action.value || !action.value.itemType) return;
			var selector = linkTypeEditors[action.value.itemType];
			if(selector) {
				this[selector](action);
			}
		},
		showLinksTab: function() {
			if(this.parentWidget.tabId) {
				var url = window.location.href;
				url = (url + "&").replace(/&&/g, "&");
				url = url.replace(/&tab=.+&/, "") + "&tab=" + this.parentWidget.tabId;
				
				if(this.parentController && this.parentController.pageController) {
					this.parentController.pageController.setWindowLocationHref(url);
				} else {					
					window.location.href = url;
				}
			}
		},
		_addCalmLink: function(action) {
			
			var self= this;
			var linkTypeDTO= action.value;
			
			var dialog= new ServiceProviderSelectionDialog({ 
				title: linkTypeDTO.displayName,
				iconUrl: linkTypeDTO.iconUrl,
				serviceProviderDTOs: linkTypeDTO.calmServiceProviders,
				linkTypeId: linkTypeDTO.id,
				projectAreaItemId : this.workingCopy.getProjectAreaItemId(),
				resultCallback : function(results) {
					self._addCalmLinksOK(action, results);
					}
				});
			dialog.open();
		},
		_addUriLink: function(action) {
		    var self= this;
			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
				dialogTitle: title,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				onOk: function(parms) {
					self._addUriLinkOK(parms);
				},
				onClose: function() {
					self.parentController.focus();
				},
				action: action
			};
			new com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog(params);

		},
		_addCalmLinksOK: function(action, links) {
			if(!action) return;
			var linkType = this._findLinkType(action.value);
			var existingLinks = linkType.linkDTOs; 

			for (var i= 0; i < links.length; i++) {
				var item= links[i];
				var dto = {
					url: item.url,
					comment: item.label,
					_isNew: true
				}
				// remove existing links of the same type with the same URL
				for (var k= existingLinks.length - 1; k >= 0 ; k--) {
					var existingLink= existingLinks[k];
					if (!existingLink._isDeleted && existingLink.url == dto.url) {
						existingLink._isDeleted = true;
						if (existingLink._isNew) {
							existingLinks.splice(i,1);
						}
					}
				}
				existingLinks.push(dto);
			}
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		_addUriLinkOK: function(parms) {
			if(!parms) return;
			var linkType = this._findLinkType(parms.action.value);
			var dto = {
				url: parms.uri,
				comment: parms.comment,
				_isNew: true
			}
			linkType.linkDTOs.push(dto);
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		_addWorkItemLink: function(action) {
			this._openAddWorkItemDialog(action);
		},
		_openAddWorkItemDialog: function(action) {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
				action: action,
				dialogTitle: title,
				allowsMultiple: !action.value.isSingleValued,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				exclusionList: this.getCurrentLinkItemIds(),
				onOk: function(selectedWorkItems, action) {
					self.addNewLink(selectedWorkItems, action);
					setTimeout(function() {self._destroySelectDialog(); }, 200);
				},
				onClose: function(args) {
					setTimeout(function() {self._destroySelectDialog(); }, 200);
					self.parentController.focus();
				},
				action: action
			};
			this._destroySelectDialog();
			this.selectWorkItemDialog = new SelectWorkItemDialog(params);
		},
		_destroySelectDialog: function() {
			if(this.selectWorkItemDialog) {
				this.selectWorkItemDialog.destroy(); 
				this.selectWorkItemDialog = null;
			}
		},
		addNewLink: function(workItems, action) {
			workItems.sort(
				function(a, b) {
					aId= parseInt(a.id, 10);
					bId= parseInt(b.id, 10);
					return aId - bId;
				});
				
			// These are either work items or wrappers from the dialog
			for (var i=0; i<workItems.length; i++) {
				if(workItems[i].attributes && workItems[i].attributes.summary) {
					workItems[i]._linkComment = EditorUtil.getSummaryLabel(workItems[i]);
				} else {
					workItems[i]._linkComment = workItems[i].id + ": " + EditorUtil.restoreTags(workItems[i].summary);
				}
			}
			
			// Find or add the link type
			var linkType = this._findLinkType(action.value); 
			
			// Add the work items to the type's linkDTOs
			var workItem, wasUpdated=false;
			for (var i=0; i<workItems.length; i++) {
				if(this.workingCopy.object.itemId == workItems[i].workItemItemId) {
					alert(this.messages.msgLinkCannotReferenceItself);	
				}	else {
					wasUpdated = true;
					this._addItemToLinks(linkType, workItems[i]);
				}
			};
			if(!wasUpdated) return;
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		/**
		 * Returns an array of the itemIds of the current linkTypes.
		 */
		getCurrentLinkItemIds: function() {
			var result= [];
			
			if(this.workingCopy && this.workingCopy.object && this.workingCopy.object.itemId) {
				result.push(this.workingCopy.object.itemId);				
			}
			var linkTypes = this._getWorkingCopyLinkTypes();
			for (var i=0; i<linkTypes.length; i++) {
				if(!linkTypes[i]._isDeleted && linkTypes[i].itemType == "WorkItem") result.push(linkTypes[i].itemId);
			}
			
			return result;
		},
		handleSaveComplete: function() {
			this.refresh();
		}
	}
);

})();
