/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.OwnerAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItem");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ContributorWidget = com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemAttributes		= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var EventTopics				= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.OwnerAttributePart",
	ContributorWidget,
	{
        constructor: function() {
			this.ownerMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
		},
		postCreate: function()  {
			this._warningMessage = this.ownerMessages.titleOwnerDoesNotBelongToTeamArea;
  			this.moreLabel = this.ownerMessages.buttonMorePrompt;
			this.moreTitle= this.ownerMessages.titleOwnerMore;
			
			this.__workItemAdapter= new com.ibm.team.workitem.web.client.internal.WorkItem(this.workingCopy);

			this.workingCopy.addListener({
				path: ["attributes", "owner"],
				event: "onchange",
				listener: this,
				functionName: "_handleOwnerChange"
			});
			
			this.workingCopy.addListener({
				path: ["attributes", "owner"],
				event: EventTopics.ON_DEPENDENCY_CHANGE,
				listener: this,
				functionName: "_fetchOwners"
			});
			this._fetchOwners();
			//Set the id so that labels can be set up for this part.
			if(this.attributeId)
				this._select.setAttribute("id",this.attributeId);

			this.inherited(arguments);			
		},
		_addCurrentOwnerToOwnerList: function(values, e) {
			return this._addExistingOwnerToOwnerList(values, this.workItem.attributes.owner); // use workItem, not workingCopy.
		},
		_addExistingOwnerToOwnerList: function(ownerList, otherOwner) {
			if(!otherOwner) return ownerList;
			var found = false;
			var otherId = otherOwner.id || otherOwner.itemId;
			for(var i=0;i<ownerList.length;i++) {
				var ownerId = ownerList[i].id || ownerList[i].itemId;
				if(ownerId == otherId) {
					found = true;
					break;
				}
			}
			if(!found) {
				var item = otherOwner;
				item.isOutOfProjectArea = true;
				item.id = item.id || item.itemId;
				item.label = item.label || item.name;
				ownerList.push(item);
			}
			return ownerList;
		},
		onSelectCreate: function(value){
			this._setAttribute(value, true);
			},
		_handleOwnerChange: function(event) {
			var ownerList = this._addCurrentOwnerToOwnerList(this.values);		
			var currentSelection = this.workingCopy.getValue( {
					path: ["attributes", "owner"]
			});
			ownerList = this._addExistingOwnerToOwnerList(ownerList, currentSelection);
			this.updateOptions(ownerList, currentSelection.id || currentSelection.itemId);
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent({target: this._select});
			}
		},
		_fetchOwners: function() {
			var self= this;
			var handlerObj= {
				success: function(values) {
					try {
						if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
						self.values = values;
						var ownerList = self._addCurrentOwnerToOwnerList(values);
						var currentSelection = self.workingCopy.getValue({
							path: ["attributes", "owner"]
						});
						ownerList = self._addExistingOwnerToOwnerList(ownerList, currentSelection);
						
						self.updateOptions(ownerList, currentSelection.id || currentSelection.itemId);
					} catch(e) {
						// ignore. this path can fail if the async op returns after the widget is destroyed
					}
			},
				error: function(errorObj) {
					// Only report in debug mode
					// ignore if it is in a dialog... isContentOnly (happens after the dialog is closed)
					// ignore if the parentController is no longer set (called after destroy)
					if (window.location.href.indexOf("debug=true") && self.parentController && self.parentController.isContentOnly != true && self.parentController.isDestroyed != true) {
						if (errorObj.message) {
							var msg = dojo.string.substitute(self.ownerMessages.errorFetchingOwners, [errorObj.message]);
						}
						else {
							var msg = dojo.string.substitute(self.ownerMessages.errorFetchingOwners, [" "]);
						}
						if(dojo.isFunction(self.parentController.handleError)) {
							self.parentController.handleError(errorObj, msg);							
						} else {
							console.log(msg);
						}
					}
				}
			};
			var srh= new ServiceResponseHandler(handlerObj, "success", "error");
			this.__workItemAdapter.getValueSet(WorkItemAttributes.OWNER, srh);
		},
		needsWarning: function(selectedValueId, defaultValue) {
			var needsWarning = false;
			for(var i=0;i<this._possibleValues.length;i++) { 
				if(this._possibleValues[i].id == selectedValueId) {
					if (this._possibleValues[i].isOutOfProjectArea) {
						needsWarning = i > 0; // Skip the "Unassigned" value
					}
					break;		
				}
			}
			return needsWarning;
		}
 	}
);

})(); 