/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaAnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaAnchorAttributePart",
	[com.ibm.team.workitem.web.ui.internal.view.editor.parts.AnchorAttributePart],
	{
		postCreate: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.inherited(arguments);
			this.refresh();
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "refresh"
			});			
		}, 
		refresh: function() {
			var value = this.workingCopy.getValue({path: ["attributes", this.attributeId]});
			var text;
			if(!value) {
				text = this.messages.labelUnassigned
			} else {
				text = value.label || value.name;
			}
			var isGrey = text == this.messages.labelUnassigned || text == this.messages.labelUnknown;
			this.setValueLabel(text, null, isGrey);	
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();		
		},
		_openChooseDialog: function(e) {
			this._openProcessAreaDialog();
		},
		_openProcessAreaDialog: function() {
			if(!EditorUtil.checkLogin()) return;

			var self= this;
//			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
//				dialogTitle: "Select Project/Process Area",
				projects: this.presentation.attributeType == "projectArea" ||this.presentation.attributeType == "processArea",
				teams: this.presentation.attributeType == "teamArea" ||this.presentation.attributeType == "processArea",
				allowsMultiple: false,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				onOk: function(selectedItem, action) {
					self.onProcessOk(selectedItem);
				},
				onClose: function() {
					self.parentController.focus();
				}
			};
			new com.ibm.team.process.web.ui.internal.ProcessAreaSelectionDialog(params);
		},
		onProcessOk: function(process) {
			if(!process) return;
			if(process.project) {
				if(this.presentation.attributeType == "teamArea") {
					alert(this.messages.promptSelectTeamArea);
					this._openProcessAreaDialog();
					return;
				}				
			} else {
				if(this.presentation.attributeType == "projectArea") {
					alert(this.messages.promptSelectProjectArea);
					this._openProcessAreaDialog();
					return;
				}
			}
			this.setWorkingCopyValue(process);
			this.refresh();
		},
		removeValue: function() {
			this.setWorkingCopyValue(null);
			this.refresh();
		},
		setWorkingCopyValue: function(value) {
			this.parentWidget._handleDirtyingEvent({target: this._valueDIV});
			
			var parms = {
					caller: this,
					path: ["attributes", this.attributeId],
					savedAttributePath: ["attributes", this.attributeId, "itemId"],
					attributeId: this.attributeId
				};
			if(value) parms.value = value; 
			this.workingCopy.setValue(parms);
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();
		}
	}
);

})();
