/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.StatusAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.StatusAttributePart",
	[com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget, com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart],
	{
		isStatusAttributePart: true,
        constructor: function() {
			this.attrMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},
		postCreate: function()  {
//			if(this.isNew()) this.setReadOnly(true);
			if(!this.attributeId) this.attributeId = "internalState";
			if(!this.attributeValue) this.attributeValue = this.workingCopy.getValue({
				path: "attributes.internalState"
			});

			this._getPossibleValues();
			
			// a state change can be an action
			this.workingCopy.addListener({
				path: ["attributes", "workflowAction", "id"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			// handle the editProps changing			
			this._workItemChangedHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._handleWorkItemChanged));
			this.inherited(arguments);
		},
		destroy: function() {
			dojo.unsubscribe(this._workItemChangedHandle);
			this.workingCopy.removeAllListeners({listener: this});
			this.inherited(arguments);
		},
		getMinimumWidth: function() {
			var currentValue = this._getCurrentSelectProps(this._select);
			if(currentValue && currentValue.label) {
			  if(currentValue.label.length < 14) return; // should fit OK	
			}
			/*
			 * Used in the header to set the width
			 * If asked, then it will be set, so we need to set the width of our select, too,
			 * but don't set it quite as wide since it needs room for the icon etc.
			 */
			this._select.style.width = "16em";
			return "16em"
		},
		getDefaultLabel: function() {
			return this.messages.labelState;
		},
		_handleWorkItemChanged: function(workingCopy) {
			if(workingCopy !== this.workingCopy) return
			this._handleAttributeChange();
		},
		_getEmptyValueLabel: function() {
			return this.attrMessages.labelUninitialized;
		},
		_isAddingValueToEmptyPossibleValues: function() {
			return false;
		},
		_getAttributeValue: function() {
			var id = this.workingCopy.getValue({path: "attributes.workflowAction.id"});
			if(!id) {
				id = this.workingCopy.getValue({path:"attributes.internalState.id"});	
			}
			return { id: id };
		},
		_handleAttributeChange: function() {
			this._getPossibleValues();
			if(this._possibleValues && this._possibleValues.length > 0) {
				this.setPossibleValues(this._possibleValues, this._select, this._image);
				var value = this._getAttributeValue();
				this.setValue(value.id, /* nonotify */ true);
			}			
		},
		_getPossibleValues: function(possibleStateValues) {
			this._possibleValues = [ ];
			/*
			 * If the workingCopy is doing a type change disable the field because
			 * the values can't make sense until the item is saved.
			 */
			if(this.workingCopy.isInTypeChange) {
				this._isAlwaysDisabled = true;
				this._select.disabled = true;
				var fakeItem = {
					iconUrl: false,
					id: "1",
					itemType: "smallString",
					label: this.attrMessages.labelUninitialized
				}
				this._possibleValues.push(fakeItem);
				return this._possibleValues;
			}	
			
			var state = this.workingCopy.getValue({path: "attributes.internalState"});
			/*
			 * If we are at the end of a move/copy, disable the state field.
			 */
			var storedProjectAreaItemId = null;
			if(this.workingCopy.storedObject && this.workingCopy.storedObject.attributes && this.workingCopy.storedObject.attributes.projectArea) {
				storedProjectAreaItemId = this.workingCopy.storedObject.attributes.projectArea.id;
			}

			if(this._possibleValues.length == 0 && storedProjectAreaItemId && this.workingCopy && this.workingCopy.moveProjectAreaItemId 
				&& this.workingCopy.moveProjectAreaItemId != storedProjectAreaItemId ) {
				// do nothing... empty 
			} else {			
				if(state) this._possibleValues.push(state); 
			}
			
			if(this.workingCopy && this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps) {
				editProps = this.workingCopy.workItemSpec.editProps;
				if(editProps) {				
					var allValues = this.getAllValueAttributeMap(editProps);
					if(allValues && allValues.actionValues) {
						for(var i=0;i<allValues.actionValues.length; i++) {
							this._possibleValues.push(allValues.actionValues[i]);
						}
					}
				}
			}			
			if (this._possibleValues.length == 0) {
				var fakeItem = {
					isDisabled:true,
					iconUrl: false,
					id: "1",
					itemType: "smallString",
					label: this._getEmptyValueLabel()
				}
				this._possibleValues.push(fakeItem);
				this._isAlwaysDisabled = true;
				this._select.disabled = true;
			}
			else {
				this._isAlwaysDisabled = false;
				this._select.disabled = false;
			}
			return this._possibleValues;
		},
		_isDisabled: function() {
			return this._possibleValues == null || this._possibleValues.length ==0 || this._possibleValues[0].isDisabled;
		},
		/*
		 * If there's a workflowAction use that
		 * else the internalState
		 */ 
		_getOverrideAttributeId: function() {
			if(this.workingCopy.getValue({path: "attributes.workflowAction"})) {
				return "workflowAction";
			}
			return this.attributeId;
		},
		_handleSelectChange: function(e) {
			// a state change is actually an action 
			this.workingCopy.setValue({
				path: ["attributes", "workflowAction", "id"],
				attributeId: "workflowAction",
				isInitialization: false,
				value: this.getValue()
			});
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent();
			}
		}
	}
);

})();
