/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemAnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil 		= com.ibm.team.workitem.web.client.util;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemAnchorAttributePart",
	[com.ibm.team.workitem.web.ui.internal.view.editor.parts.AnchorAttributePart],
	{
		postCreate: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "refresh"
			});			
			this.inherited(arguments);
			this.refresh();
		}, 
		refresh: function() {
			var value = this.workingCopy.getValue({path: ["attributes", this.attributeId]});
			if(value && value.attributes != null) {
				return this._refreshFromAttributes(value);
			}
			if(!value) {
				this.setValueLabel(this.messages.labelUnassigned, null, /* grey */ true);
				return;
			}
			if(value.label != null) {
				this.setValueLabel(value.label, null, /* grey */ value.label == this.messages.labelUnassigned);
				return;
			}
			var id = value.id + "";
			var typeName = value.typeName
			if(id) {
				var href = WorkItemUtil.getViewWorkItemUri(value.id);
				var text = dojo.string.substitute(this.messages.labelListWorkItem, [typeName, id])
				this.setValueLabel(text, href, /* grey */ false);
			}
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();
		},
		/* If the work item value is set, then it comes in as a work item instead of dto info */
		_refreshFromAttributes: function(value) {
			var properties = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil.getPropertyMap(value, {}, "attributes");
			if (properties && properties.workItemType && properties.workItemType.label && properties.id && properties.id.id) {
				var href = WorkItemUtil.getViewWorkItemUri(properties.id.id);
				var text = dojo.string.substitute(this.messages.labelListWorkItem, [properties.workItemType.label, properties.id.id])
				this.setValueLabel(text, href);
			} else {
				this.setValueLabel(this.messages.labelUnassigned, null, /* grey */ true);
			}
		},
		_openChooseDialog: function(e) {
			this._openWorkItemDialog();
		},
		_openWorkItemDialog: function() {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
//			var title = "Select Work Item";
			var params= {
//				dialogTitle: title,
				allowsMultiple: false,
				parentController: this.parentWidget.parentController  || this.parentWidget,
				parentElement: this.domNode,
				exclusionList: [ this.workingCopy.object.itemId ],
				onOk: function(selectedWorkItems, action) {
					self.onWorkItemOk(selectedWorkItems);
					setTimeout(function() {self._destroySelectDialog();}, 200);
				},
				onClose: function() {
					if(self.parentController && self.parentController.focus) {
						self.parentController.focus();	
					}
					setTimeout(function() {self._destroySelectDialog();}, 200);
				}
			};
			this._destroySelectDialog();
			this.selectWorkItemDialog = new com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog(params);
		},
		_destroySelectDialog: function() {
			if(this.selectWorkItemDialog) {
				this.selectWorkItemDialog.destroy(); 
				this.selectWorkItemDialog = null;
			}
		},
		onWorkItemOk: function(workItemArray) {
			if(!workItemArray || workItemArray.length ==0) return;
			this.setWorkingCopyValue(workItemArray[0]);
			this.refresh();
		},
		removeValue: function() {
			this.setWorkingCopyValue(null);
			this.refresh();
		},
		setWorkingCopyValue: function(value) {
			this.parentWidget._handleDirtyingEvent({target: this._valueDIV});

			var parms = {
					caller: this,
					path: ["attributes", this.attributeId],
					savedAttributePath: ["attributes", this.attributeId, "itemId"],
					attributeId: this.attributeId
				};
			if(value) {
				value.itemId = value.itemId || value.workItemItemId
				parms.value=value;
			}
			this.workingCopy.setValue(parms);
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();
		}
	}
);

})();
