/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemListPart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
/*
 * Handle presentation operations pertaining to the owner attribute
 */
(function() {

var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil 		= com.ibm.team.workitem.web.client.util;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemListPart",
	[com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart],
	{
		postCreate: function() {
//			this._chooseAnchor.style.display= "none";
//			var text = this.attributeValue ? this.attributeValue.label : "Unknown";
//			var href = WorkItemUtil.getViewWorkItemUri(this.attributeValue.attributes.itemId);
//			var href; //TODO
//			this.setValueLabel(text, href);
			this.inherited(arguments);
		},
		addWorkingCopyChangedListener: function() {			
			this.workingCopy.addListener({
				path: ["linkTypes"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyChange"
			});	
		},
		_getItemLabel: function(item) {
			// item can either be a WorkItemDTO or a WorkItemSummaryDTO
			if (item.typeName) { // WorkItemSummaryDTO
				var name = dojo.string.substitute(this.messages.labelListWorkItem, [ item.typeName, item.id])
					var name = item.typeName + " " + item.id;
			}
			else {
				var properties = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil.getPropertyMap(item, {}, "attributes");
				if (properties.workItemType) {
					var name = dojo.string.substitute(this.messages.labelListWorkItem, [ properties.workItemType.label, properties.id.label])
				} else {
					if(item.comment) {
						name = item.comment;
					} else if(item.target && item.target.name) {
						name = item.target.name;
					}
				}
			}
			return name || this.messages.labelUnknown;
		},
		_getItemHref: function(item) {
			var href;
			if(item.id) {
				href = WorkItemUtil.getViewWorkItemUri(item.id);
			} else {
				var properties = com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil.getPropertyMap(item, {}, "attributes");
				if(properties && properties.id && properties.id.id) {
					href = WorkItemUtil.getViewWorkItemUri(properties.id.id);
				} else {
					if(item.weburi) {
						href = item.weburi;
					}
				}
			}
			return href;
		},
		_openChooseDialog: function(e) {
			this._openWorkItemDialog();
		},
		_openWorkItemDialog: function() {
			this.openWorkItemDialog();
		},
		openWorkItemDialog: function() {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
			var params= {
				allowsMultiple: true,
				parentController: this.parentWidget.parentController  || this.parentWidget,
				parentElement: this.domNode,
//				exclusionList: [ this.workingCopy.object.itemId ], TODO
				exclusionList: [ ],
				onOk: function(selectedWorkItems, action) {
					self.onWorkItemOk(selectedWorkItems);
					setTimeout(function() {self._destroySelectDialog();}, 200);
				},
				onClose: function() {
					if(self.parentController && self.parentController.focus) {
						self.parentController.focus();	
					}
					setTimeout(function() {self._destroySelectDialog();}, 200);
				}
			};
			this._destroySelectDialog();
			this.selectWorkItemDialog = new com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog(params);
		},
		_destroySelectDialog: function() {
			if(this.selectWorkItemDialog) {
				this.selectWorkItemDialog.destroy(); 
				this.selectWorkItemDialog = null;
			}
		},
		onWorkItemOk: function(workItemArray) {
			if(!workItemArray || workItemArray.length ==0) return;
			for(var i=0;i<workItemArray.length;i++) {
				var item = workItemArray[i];
				item.itemId = item.itemId || item.workItemItemId;
			}
			this.workingCopy.addItems({
				items: workItemArray ,
				attributeId: this.attributeId,
				excludeCreator: false
			});
			this.refresh();
		},
		
		/*
		 * Open the link dialog to add a new link from the Refererence Work Item List
		 * TODO: Promote this to ReferenceWorkItemList
		 */
		addWorkItemLink: function(linkDTO /* linkTypeDTO */, currentItems) {
			var exclusionList = [];
			for(var i=0;i<currentItems.length;i++) {
				exclusionList.push(currentItems[i].itemId);
			}
			this._openAddWorkItemDialog2({label: linkDTO.displayName, value: linkDTO, exclusionList: exclusionList} );
		},
		_openAddWorkItemDialog2: function(action) {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
				action: action,
				dialogTitle: title,
				allowsMultiple: !action.value.isSingleValued,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				exclusionList: action.exclusionList || [],
//				exclusionList: [],
				onOk: function(selectedWorkItems, action) {
					self.addNewLink2(selectedWorkItems, action);
					setTimeout(function() {self.selectWorkItemDialog2.destroy(); self.selectWorkItemDialog2 = null; }, 500);
				},
				onClose: function() {
					if(self.parentController) self.parentController.focus();
					setTimeout(function() {self.selectWorkItemDialog2.destroy(); self.selectWorkItemDialog2 = null; }, 500);
				},
				action: action
			};
			this.selectWorkItemDialog2 = new com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog(params);
		},
		addNewLink2: function(workItems, action) {
			workItems.sort(
				function(a, b) {
					aId= parseInt(a.id, 10);
					bId= parseInt(b.id, 10);
					return aId - bId;
				});
				
			// These are either work items or wrappers from the dialog
			for (var i=0; i<workItems.length; i++) {
				if(workItems[i].attributes && workItems[i].attributes.summary) {
					workItems[i]._linkComment = EditorUtil.getSummaryLabel(workItems[i]);
				} else {
					workItems[i]._linkComment = workItems[i].id + ": " + EditorUtil.restoreTags(workItems[i].summary);
				}
			}
			
			// Find or add the link type
			var linkType = this._findLinkType(action.value); 
			
			// Add the work items to the type's linkDTOs
			var workItem, wasUpdated=false;
			for (var i=0; i<workItems.length; i++) {
				if(this.workingCopy.object.itemId == workItems[i].workItemItemId) {
					alert(this.messages.msgLinkCannotReferenceItself);	
				}	else {
					wasUpdated = true;
					this._addItemToLinks(linkType, workItems[i]);
				}
			};
			if(!wasUpdated) return;
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		/**
		 * Returns an array of the itemIds of the current linkTypes.
		 */
		getCurrentLinkItemIds2: function() {
			var result= [];
			
			if(this.workingCopy && this.workingCopy.object && this.workingCopy.object.itemId) {
				result.push(this.workingCopy.object.itemId);				
			}
			var linkTypes = this._getWorkingCopyLinkTypes();
			for (var i=0; i<linkTypes.length; i++) {
				if(!linkTypes[i]._isDeleted && linkTypes[i].itemType == "WorkItem") result.push(linkTypes[i].itemId);
			}
			
			return result;
		},
		_findLinkType: function(searchLinkType) {
			var linkType;
			var found = false;	
			var linkTypes = this._getWorkingCopyLinkTypes();		
			for(var i=0;i<linkTypes.length;i++){
				if(linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
					found = true;
					linkType =linkTypes[i]; 
					break;
				}
			}

			if (!found) {
				linkTypes = this._editableLinkTypes;
				for (var i = 0; i < linkTypes.length; i++) {
					if (linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
						found = true;
						linkType = linkTypes[i];
						this._getWorkingCopyLinkTypes().push(linkType);
						break;
					}
				}
			}

			if(!found) { // Should never happen
				linkType =  EDITABLE_LINK_TYPES.related;
				linkType.itemType = "WorkItem";
				linkType._isNew = true;
				this._getWorkingCopyLinkTypes().push(linkType);
			}
			
			if(!linkType.linkDTOs) {
				linkType.linkDTOs = [];
			}
			return linkType;
		},
		_getWorkingCopyLinkTypes: function () {
			return this.workingCopy.getValue({
				path: ["linkTypes"],
				defaultValue: []
			});
		},
		_addItemToLinks: function (linkType, item) {
			var links = linkType.linkDTOs
			var itemId = item.workItemItemId ? item.workItemItemId : item.itemId;
			var found = false;
			for(var i=0;i<links.length;i++){
				if(links[i].itemId == itemId) {
					found = true;
					links[i]._isDeleted = false;
					break;
				}
			}
			
			if (!found) {
				var link = {
					_isNew: true,
					id: item.id,
					itemId: itemId,
					comment: item._linkComment
				}
				var idAttr = {
					id: item.id
				}
				var target = {
					_eQualifiedClassName: "com.ibm.team.workitem.rest.dto:WorkItemDTO",
					attributes: [
							{ key: "id",
							  value: idAttr } ]
				}
				link.target = target;
				
				if (linkType.isSingleValued) {
					// mark all the other links deleted
					var newLinks = [];
					for (var i = 0; i < links.length; i++) {
						if (links[i].itemId != itemId) {
							if(!links[i]._isNew) {
								newLinks.push(links[i]);
							}
							links[i]._isDeleted = true;
						}
					}
					links = newLinks;
					linkType.linkDTOs = links; 
				}
				
				links.push(link);
			}	
		},
		_parentMarkDirty: function() {
			if(this.parentWidget && this.parentWidget.markDirty) {
					this.parentWidget.markDirty();
				} 

			if(this.parentWidget && this.parentWidget.parentController && this.parentWidget.parentController.markDirty) {
						this.parentWidget.parentController.markDirty();
				}
		},
		restoreItem: function(item) {
			item._isDeleted = false;
			this.refresh();
			if(this.attributeId) {				
				var status= this.workingCopy._validate(this.attributeId);
				this.statusIndicator.setStatus(status);
				// this.statusIndicator.test();
			}
		},
		deleteLinkDTO: function(linkDTO) {
			this._deleteLink(linkDTO);
		},
		_deleteLink: function(linkDTO) {
			var itemToDelete = linkDTO;
			var linkTypeSpec = this._getLinkTypeSpec(linkDTO);
			var linkType = this._findLinkType(linkTypeSpec);
			var links = linkType.linkDTOs; 
			var wasDeleted = false;
			if(itemToDelete.itemId) {
				for(var i=0;i<links.length;i++){
					if(links[i].itemId == itemToDelete.itemId) {
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						wasDeleted = true;
						break;
					}
				}
			} else {
				if(itemToDelete.url && itemToDelete.comment) {
					for(var i=0;i<links.length;i++){
					if(!links[i]._isDeleted && links[i].url && links[i].comment  && links[i].url == itemToDelete.url && links[i].comment == itemToDelete.comment) {
						wasDeleted = true;
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						break;
					}
				}	
				}		
			}
			
			if(wasDeleted) {
				this._parentMarkDirty();
			}
			this._setWorkingCopyValue();
			this.refresh();
		},
		_getLinkTypeSpec: function(actionValue) {
		    if(actionValue.isSource) return actionValue; // already a link
			// Find or add the link type
			var linkType = {
			    	itemType: actionValue.linkTypeItemType,
					id: actionValue.linkTypeId,
					isUserDeleteable: actionValue.linkTypeIsUserDeleteable,
					isSource: actionValue.linkTypeIsSource
			}
			return linkType
		},
		_findLinkType: function(searchLinkType) {
			var linkType;
			var found = false;	
			var linkTypes = this._getWorkingCopyLinkTypes();		
			for(var i=0;i<linkTypes.length;i++){
				if(linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
					found = true;
					linkType =linkTypes[i]; 
					break;
				}
			}

			if (!found) {
				linkTypes = this.workingCopy.workItemSpec.editableLinkTypes;
				for (var i = 0; i < linkTypes.length; i++) {
					if (linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
						found = true;
						linkType = linkTypes[i];
						this._getWorkingCopyLinkTypes().push(linkType);
						break;
					}
				}
			}

			if(!found) { // Should never happen
				linkType =  EDITABLE_LINK_TYPES.related;
				linkType.itemType = "WorkItem";
				linkType._isNew = true;
				this._getWorkingCopyLinkTypes().push(linkType);
			}
			
			if(!linkType.linkDTOs) {
				linkType.linkDTOs = [];
			}
			return linkType;
		},
		isWorkItemListPart: true,
		setWorkignCopyValue: function() {
			this._setWorkingCopyValue();
		},
		_setWorkingCopyValue: function() {
			/* The linktypes are updated in place, so just notify senders */
			this.workingCopy.setValue({
	//			caller: this,  // call back ourself
				notifyEvenIfUnchanged: true,
				path: ["linkTypes"],
				value: 	this._getWorkingCopyLinkTypes() // Current values
			});
		},
		__sentinal: true
	}
);

})();
