/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent",
	com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent,
{
	prefix: "/enumeration",

	postMixInProperties: function(){
		this.enumMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages");
		this.inherited(arguments);
	},
		
	postCreate: function(){
		this.inherited(arguments);
		
		// add "name" field
		var div = dojo.create("div", {
			style: {
				paddingBottom: "0.5em"
			}
		});
		var fieldId = this.id+"_literalNameField";
		dojo.create("label", {
			"for": fieldId,
			innerHTML: this.enumMessages.LiteralNameLabel
		}, div);
		dojo.create("span", {
			className: "requiredFlag",
			innerHTML: "*"
		}, div);
		this.literalNameField = dojo.create("input", {
			id: fieldId,
			style: {
				marginLeft: "10px",
				width: "300px"
			}
		}, div);
		dojo.place(div, this.domNode, "first");
	},
	
	getValues: function(){
		var values = this.inherited(arguments);
		values.literalName = this.literalNameField.value;
		return values;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent.openDialog = function(prop){
	// create edit icon dialog content
	var model = prop.model;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent({
		model: model
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.enumMessages.AddLiteral,
		onOpen: function(){
			dijit.focus(contentWidget.literalNameField);
		},
		validate: function(values, callback){
			var literalName = values.literalName;
			if(literalName && literalName.length>0){
				callback();
			}else{
				alert(contentWidget.enumMessages.errLiteralNameEmpty);
			}
		},
		onOK: function(values){
			var enumInfo = model.getEnumeration(prop.enumId);
			if(!enumInfo){ return; }
			
			var maxIndex = 1;
			var literals = enumInfo.literals;
			if(literals && literals.length > 0){
				var indexArray = dojo.map(literals, function(literal){
					var literalId = literal.id;
					if(literalId && literalId.length>0){
						var index = literalId.lastIndexOf(".");
						if(index != -1){
							literalId = literalId.substring(index+2);
						}
						var intId = parseInt(literalId, 10);
						if(!isNaN(intId)){
							return intId;
						}
					}
					return 0;
				}).sort(function(num1, num2){return num1 - num2});
				maxIndex = indexArray[indexArray.length-1]+1;
			}

			// auto-generate id like {enumeration ID}.literal.l{n} 
			var uniqueId = prop.enumId+".literal.l"+maxIndex;
			
			model.addLiteral(prop.enumId, {
				id: uniqueId, 
				name: values.literalName,
				iconUrl: values.iconUrl,
				title: values.title,
				icon: values.icon
			});
		}
	});
};