/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.enumeration.EnumerationEditor");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");
dojo.require("dojo.string");
dojo.require("dijit.InlineEditBox");
dojo.require("jazz.core.loader");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var CLASS_DISABLED = "disabled";
var CLASS_BLANK = "blankIcon";
var CLASS_INLINE = "dijitInline";  //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command";
var load_async = jazz.core.loader.load_async;
var escapeHtml = jazz.util.html.escape;
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;
var disableActionIcon = function(node, disable){
	dojo.toggleClass(node, CLASS_DISABLED, disable);
};

var CSH_KEYS = {
	ENUMERATION_EDITOR: "com.ibm.team.workitem.web.wi_enumerations_web_editor"
};

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.enumeration.EnumerationEditor",
	com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor,
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/enumeration/templates/EnumerationEditor.html"),
	pageCshId: CSH_KEYS.ENUMERATION_EDITOR,
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "EnumerationMessages");
		this.messages = messages;
		this.title = messages.EnumEditorTitle; // this would be used as tab title
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// create literals section toolbar
		var self = this;
		var literalsToolbar = this.literalsToolbar;
		this.downButton = createActionIcon({
			iconClass: "downIcon",
			onClick: dojo.hitch(self, "_moveLiteralDown"),
			title: messages.MoveDown,
			owner: this
		});
		literalsToolbar.appendChild(this.downButton);
		this.upButton = createActionIcon({
			iconClass: "upIcon",
			onClick: dojo.hitch(self, "_moveLiteralUp"),
			title: messages.MoveUp,
			owner: this
		});
		literalsToolbar.appendChild(this.upButton);
		literalsToolbar.appendChild(dojo.create("a", {
			href: "#",
			innerHTML: messages.Add,
			title: messages.AddLiteral,
			style: {
				verticalAlign: "middle",
				marginLeft: "8px"
			},
			onclick: function(e){
				dojo.stopEvent(e);
				self._handleAddLiteral();
			}
		}));

		var model = this.model;
		this.connect(model, "onEnumerationChanged", function(args){
			if(!args || !args.target){
				self._populateEnumList(false);
			}else{
				if(args.key == "literals"){
					self._populateLiterals(args.value);
				}else if(args.key == "displayName"){
					self._populateEnumList(false);
				}
			}
		});
		
		// connect to SELECT onchange event programmatically since connecting in template somehow doesn't work...
		this.connect(this.enumList, "onchange", "_handleEnumSelected");

		var literalsList = this.literalsList;
		literalsList.setupColumns([
			{
				id: "iconUrl",
				name: messages.IconColumn,
				className: "iconCell",
				getCellContent: function(cellData){
					var selectedIcon = this.data.icon;
					var literalId = this.data.id;
					var params = {
						onClick: function(){
							load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
								com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
									prefix: "/enumeration",
									model: model,
									selectedPath: selectedIcon,
									onOK: function(values){
										self.model.setLiteral(self.getSelectedEnum(), literalId, {
											icon: values.icon,
											iconUrl: values.iconUrl
										});
									}
								});
							});
						},
						title: messages.EditIcon,
						owner: this
					};
					if(cellData && cellData.length>0){
						params.iconUrl = cellData;
					}else{
						params.iconClass = CLASS_BLANK;
					}
					return createActionIcon(params);
				}
			},
			{	
				id: "name",
				name: messages.NameColumn,
				getCellContent: function(cellData){
					var literalId = this.data.id;
					var rowWidget = this;
					var editBox = new dijit.InlineEditBox({
						title: cellData,
						width: "200px",
						onChange: function(value){
							var isEmpty = false;
							if(!value || value.length===0){
								value = rowWidget.data.name;
								isEmpty = true;
							}
							var enumId = self.getSelectedEnum();
							model.setLiteral(enumId, literalId, {
								name: value
							});
							if(isEmpty){
								alert(messages.errLiteralNameEmpty);
							}
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:cellData }));
					dojo.style(editBox.domNode, {
						width: "200px"
					});
					return editBox.domNode;
				}
			},
			{ id:"actions", name:messages.ActionColumn, isAction:true }
		]);
		
		this.connect(literalsList, "onSelected", function(selected){
			var children = literalsList.getChildren();
			disableActionIcon(self.upButton, (!selected || children[0] == selected));
			disableActionIcon(self.downButton, (!selected || children[children.length-1] == selected));
		});
		
		this._populateEnumList(false); // forceLoad should be false as it get may loaded in other place (Types & Attributes page)
	},

	_handleFinalChanged: function(){
		this.model.setEnumFinal(this.finalCheckbox.checked);
	},
	
	_populateEnumList: function(forceLoad){
		this.setLoading(true);
		var enumList = this.enumList;
		var model = this.model;
		model.getAttributeTypes(dojo.hitch(this, function(configData){
			this.setLoading(false);

			if(forceLoad || !this._isInit){
				this.processCommonConfigData(configData);
			}
			
			this._isInit = true;
			
			var enumTypes = configData.enumerationTypes;
			
			// check previous selection to restore selection
			var options = enumList.options;

			var selectedId = null;
			if(enumTypes && enumTypes.length > 0){
				if(options.length > 0){
					if(enumTypes.length > options.length){
						// type has been added, so select added option
						selectedId = enumTypes[enumTypes.length-1].id;
					}else if(enumTypes.length < options.length){
						// type has been removed, so select first option
						selectedId = enumTypes[0].id;
					}else{
						// otherwise, select same option
						selectedId = options[options.selectedIndex].value;
					}
				}else{
					selectedId = enumTypes[0].id;
				}
			}

			// clean-up the previous list
			options.length = 0;

			if(enumTypes && enumTypes.length > 0){
				// populate updated types list
				dojo.forEach(enumTypes, function(enumInfo, idx){
					var enumId = enumInfo.id;
					options[idx] = new Option(model.getDisplayName(enumId), enumId);
				}, this);
				this.selectEnum(selectedId);
				this.removeBtn.disabled = false;
			}else{
				// no items, so clear selection
				this.selectEnum(null);
				this.removeBtn.disabled = true;
			}
		}), forceLoad);
	},
	
	refresh: function(){
		this._populateEnumList(true);
	},
	
	getSelectedEnum: function(){
		// returns ID for selected enumeration
		var options = this.enumList.options;
		return (options.length > 0) ? options[options.selectedIndex].value : null;
	},
	
	selectEnum: function(enumId){
		if(!enumId || this.getSelectedEnum() == enumId){
			this._handleEnumSelected();
		}else{
			var options = this.enumList.options;
			for(var i=0, len=options.length; i<len; i++){
				if(options[i].value == enumId){
					options[i].selected = true;
					this._handleEnumSelected();
					break;
				}
			}
		}
	},
	
	_handleEnumSelected: function(){
		var enumId = this.getSelectedEnum();

		// hide right section if no enumeration is selected
		this.detailsSection.style.display = (enumId) ? "" : "none";

		var enumInfo = this.model.getEnumeration(enumId);
		if(!enumInfo){ return; }
		
		var id = enumInfo.id;
		this.enumIdField.innerHTML = escapeHtml(id);
		var displayName = enumInfo.displayName;
		this.enumNameField.value = (displayName && displayName.length > 0) ? displayName : id;
		
		this._populateLiterals(enumInfo.literals);
	},
	
	_handleAddEnum: function(e){
		dojo.stopEvent(e);
		// create dialog content
		var model = this.model;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddEnumerationContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddEnumerationContent.openDialog({
				title: messages.AddEnum,
				model: model
			});
		});
	},
	
	_handleRemoveEnum: function(){
		var enumId = this.getSelectedEnum();
		if(enumId && enumId.length > 0){
			var enumInfo = this.model.getEnumeration(enumId);
			var enumName = (enumInfo && enumInfo.displayName && enumInfo.displayName.length>0) ? enumInfo.displayName : enumId;
			if(confirm(dojo.string.substitute(messages.warnRemoveEnum, [enumName]))){
				this.model.removeEnumeration(enumId);
			}
		}
	},
	
	_populateLiterals: function(literals){
		var defaultLiteralField = this.defaultLiteralField;
		defaultLiteralField.options.length = 0; // clear
		defaultLiteralField.options[0] = new Option(messages.None, "");
		var unassignedLiteralField = this.unassignedLiteralField;
		unassignedLiteralField.options.length = 0; // clear
		unassignedLiteralField.options[0] = new Option(messages.None, "");
		if(literals){
			dojo.forEach(literals, function(literal, idx){
				// populate default literal dropdown
				defaultLiteralField.options[idx+1] = new Option(literal.name, literal.id);
				if(literal.defaultValue === true){
					defaultLiteralField.options[idx+1].selected = true;
				}
				
				// populate unassigned literal dropdown
				unassignedLiteralField.options[idx+1] = new Option(literal.name, literal.id);
				if(literal.unassigned === true){
					unassignedLiteralField.options[idx+1].selected = true;
				}
			}, this);
		}
		
		// populate literals table
		var list = this.literalsList;
		var selected = list.getSelectedItem();
		list.clearItems();
		if(literals){
			var self = this;
			var model = this.model;
			var arr = dojo.map(literals, function(literal){
				var literalId = literal.id;
				var literalName = literal.name;
				return {
					id: literalId,
					icon: literal.icon,
					iconUrl: literal.iconUrl,
					name: literalName,
					actions: [{
						iconClass: CLASS_DELETE_COMMAND,
						title: messages.RemoveLiteral,
						onClick: function(e){
							if(literalId){
								if(confirm(dojo.string.substitute(messages.confirmRemoveLiteral, [ literalName ]))){
									model.removeLiteral(self.getSelectedEnum(), literalId);
								}
							}
						}
					}]
				};
			});
			var newSelectItem = null;
			dojo.forEach(arr, function(obj){
				var item = list.addItem(obj);
				
				// restore selection
				if(selected && selected.data.id && obj.id == selected.data.id){
					newSelectItem = item;
				}
			});
			list.selectItem(newSelectItem);
		}
	},
	
	_handleEnumNameChanged: function(e){
		var name = this.enumNameField.value;
		this.model.setEnumerationProperty(this.getSelectedEnum(), "displayName", name);
	},
	
	_handleDefaultLiteralChanged: function(e){
		var enumId = this.getSelectedEnum();
		var enumInfo = this.model.getEnumeration(enumId);
		if(!enumInfo){ return; }

		var sel = this.defaultLiteralField;
		var newDefaultId = sel[sel.selectedIndex].value;
		var literals = enumInfo.literals;
		// clear previous default literal
		var len = literals.length;
		for(var i=0; i<len; i++){
			if(literals[i].defaultValue === true){
				delete literals[i].defaultValue;
				break;
			}
		}
		// set new default literal
		for(var j=0; j<len; j++){
			if(literals[j].id == newDefaultId){
				literals[j].defaultValue = true;
				break;
			}
		}
		this.model.setLiterals(enumId, literals);
	},
	
	_handleUnassignedLiteralChanged: function(e){
		var enumId = this.getSelectedEnum();
		var enumInfo = this.model.getEnumeration(enumId);
		if(!enumInfo){ return; }

		var sel = this.unassignedLiteralField;
		var newUnassignedId = sel[sel.selectedIndex].value;
		var literals = enumInfo.literals;
		// clear previous unassigned literal
		var len = literals.length;
		for(var i=0; i<len; i++){
			if(literals[i].unassigned === true){
				delete literals[i].unassigned;
				break;
			}
		}
		// set new unassigned literal
		for(var j=0; j<len; j++){
			if(literals[j].id == newUnassignedId){
				literals[j].unassigned = true;
				break;
			}
		}
		this.model.setLiterals(enumId, literals);
	},
	
	_handleAddLiteral: function(){
		// create edit icon dialog content
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.enumeration.AddLiteralContent.openDialog({
				model: self.model,
				enumId: self.getSelectedEnum()
			});
		});
	},
	
	_moveLiteralDown: function(){
		if(dojo.hasClass(this.downButton, CLASS_DISABLED)){
			return;
		}
		
		var enumId = this.getSelectedEnum();
		var enumInfo = this.model.getEnumeration(enumId);
		if(!enumInfo){ return; }
		var literals = enumInfo.literals;
		
		var item = this.literalsList.getSelectedItem();
		if(item){
			var literalId = item.data.id;
			var len = literals.length;
			for(var i=len-1; i>=0; i--){
				var literal = literals[i];
				if(literal.id == literalId){
					if(i<len-1){
						literals.splice(i, 1);
						literals.splice(i+1, 0, literal);
						this.model.setLiterals(enumId, literals);
					}
					break;
				}
			}
		}else{
			alert(messages.msgLiteralNotSelected);
		}
	},
	
	_moveLiteralUp: function(){
		if(dojo.hasClass(this.upButton, CLASS_DISABLED)){
			return;
		}

		var enumId = this.getSelectedEnum();
		var enumInfo = this.model.getEnumeration(enumId);
		if(!enumInfo){ return; }
		var literals = enumInfo.literals;
		
		var item = this.literalsList.getSelectedItem();
		if(item){
			var literalId = item.data.id;
			var len = literals.length;
			for(var i=0; i<len; i++){
				var literal = literals[i];
				if(literal.id == literalId){
					if(i>0){
						literals.splice(i, 1);
						literals.splice(i-1, 0, literal);
						this.model.setLiterals(enumId, literals);
					}
					break;
				}
			}
		}else{
			alert(messages.msgLiteralNotSelected);
		}
	},
	
	_handleEditLiteralIcon: function(literal){
		// create edit icon dialog content
		if(literal){
			var self = this;
			load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
				com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
					prefix: "/enumeration",
					model: self.model,
					selectedPath: literal.icon,
					onOK: function(values){
						self.model.setLiteral(self.getSelectedEnum(), literal.id, {
							icon: values.icon,
							iconUrl: values.iconUrl
						});
					}
				});
			});
		}
	},
	
	destroy: function(preserveDom){
		if(this.literalsList){
			this.literalsList.destroy(preserveDom);
		}
		this.inherited(arguments);
	}
});

})();
