/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.xml.parser");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.api.common.Severity"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.api.common.Status"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.AbstractWidget");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");


dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");


(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.ContributorWidget",
	[dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.ui2.internal.widget.AbstractWidget],
	{
		templateString: '<div class="ContributorWidget NarrowWidget">' +
						'<span dojoAttachPoint="_statusIconContainer" class="StatusIconContainer">' +
						'<div dojoAttachPoint="_statusIcon"></div></span>' +
						'<span  style="display: none" dojoAttachPoint="_readOnlySelectValue" class="ReadOnlySelectValue"></span>' +						
						'<div class="fieldWrapper"><select style="display: none" dojoAttachPoint="_select" dojoAttachEvent="onchange: _handleSelectChange" class="Select"></select></div>' +
						'<span style="display: none" dojoAttachPoint="_anchorSPAN"></span>' +
						'<span style="display:none" dojoAttachPoint="_noValueLabelSPAN" class="LayoutValue GreyLabel"></span>' +
						'</div>',
     constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
			this.ownerMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
			this.editorMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},
		postCreate: function() {
			this.statusIndicator= new com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator({}, this._statusIcon);

  			this.moreLabel = this.ownerMessages.buttonMorePrompt;
			this.moreTitle= this.ownerMessages.titleOwnerMore;
			this._childWidgets = [];
			
			if(this.creationSpec && this.creationSpec.isDescription) {
					dojo.addClass(this.domNode,"MarginLeft10");
				}
			
			// If there are no possible values, but there is a value,
			// Add the value as a possible value
			if(this._possibleValues == null) {
				this._possibleValues= [];
			}
			
			var value = this.workingCopy.getValue({path: ["attributes", this.attributeId], attributeId: this.attributeId});
			this.attributeValue = value;
			this._updateValueSet(value);

			if(this._modifiers && this._modifiers.readonly == "true") {
				this._statusIconContainer.className="StatusIconReadOnlyContainer";
				this.createLabel();
			} else {
				this.createSelect();
			}
						
			this._isChanged = false;
			if(this.workingCopy.isGuest()) {
				dojo.addClass(this._select, "GuestField");
			}
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();			
		},
		destroy: function() {
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
			this._destroyDialog();
			if(this.statusIndicator) this.statusIndicator.destroy();
			this.statusIndicator = null;
			this.inherited(arguments);
		},
		_getAttributeValue: function() {
			var value = this.workingCopy.getValue({ path: ["attributes", this.attributeId] });
			// If the attributeValue is a contributor, convert it to a uiItem
			// uiItems come from a server request. Contributors come from the Select Contributor dialog.
			// this changes it in the working copy
			if(value.itemId && !value.id) {
				value.id = value.itemId;
				value.label = value.name;
			}
			this._updateValueSet(value);
			return value;			
		},
		
		_updateValueSet: function(value) {
			try {
				// if the value is not in possible values
				// then add the value as the first possible value
				if(value) {
					var found = false;
					for(var i=0;i<this._possibleValues.length;i++) {
						if(this._possibleValues[i].id == value.id || this._possibleValues[i].id == value.itemId) {
							found = true;
							break;
						}
					}
					if (!found) {
						this._possibleValues.unshift(value);
					}
				}
			} catch(e) { // ignore for now 
			}
		},
		
		_handleAttributeChange: function(parms) {
			if(!parms || !parms.attributeValue || !parms.attributeValue.id) return;
			if (this._select.value != parms.attributeValue.id) {
				this._select.value = parms.attributeValue.id;
				this._isChanged = true;
				var selectedValue = WidgetUtil._getCurrentSelectProps(this._select);
			
			/*
			 * If the selection wasn't in the possible values, add it
			 */
				if(!selectedValue || (selectedValue.id != parms.attributeValue.id)) {
					this._possibleValues = this._possibleValues || [];
					this._possibleValues.unshift(parms.attributeValue);
					if(this._modifiers && this._modifiers.readonly == "true") {
						this.createLabel();
					} else {
						this.createSelect(/* skip on create */ true);
					}
					this._select.value = parms.attributeValue.id;
					var selectedValue = WidgetUtil._getCurrentSelectProps(this._select);
				}
			
				if (selectedValue && selectedValue.label) {
					this._setReadOnlySelectValue(selectedValue.label);
				}
				this._setWarningIconVisibility(this._select.value);
				if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
					this.parentWidget._handleDirtyingEvent({target: this._select});
				}
			} 
			if(parms.validationStatus)
				this.statusIndicator.setStatus(parms.validationStatus);
		},
		setReadOnly: function(isReadOnly) {
			if(isReadOnly){
				if(this._modifiers && this._modifiers.readonly == "true") return;
				// Always show it if is changed
				if(this._isChanged) {
					this.setReadOnly(false);
					return;	
				}
				this._select.style.display = "none";
				this._readOnlySelectValue.style.display = "inline";
			}
			else {
				if(this._modifiers && this._modifiers.readonly == "true") return;
				this._select.style.display = "inline";
				this._readOnlySelectValue.style.display = "none";
			}
		},
		createLabel: function(showOnSelect) {
			var value = this._getAttributeValue();
			if(value.name == "Unassigned" && value.userId == "unassigned") {
				value.isNullValue = true;
			}
			
			if(!value || value.length == 0 || value.isNullValue) {
				value = this.messages.labelDefaultNullItem;
				if(this._modifiers.noValueLabel) {
					value = this._modifiers.noValueLabel;
				}
				if(this._modifiers.noValueTitle) {
					this._noValueLabelSPAN.title = this._modifiers.noValueTitle;
				} 
				this._noValueLabelSPAN.style.display="inline";
				dojox.xml.parser.replaceChildren(this._noValueLabelSPAN, document.createTextNode(value));
				return;
			} 
			var href = this._modifiers["href"];
			if(!href && value.emailAddress) {
					var emailSubject =  this._modifiers.emailSubject;
					var href = WidgetUtil._getMailto(value.emailAddress, emailSubject);
				}
				
			var name = value.name ? value.name : value.label;
			name = name ? name : labelDefaultNullItem;
				
// If there is an href, add the anchor, otherwise just add a label
// <a dojoAttachPoint="_anchor" class="LayoutValue Anchor"><span dojoAttachPoint="_labelSPAN" class=""></span></a>
			var labelTarget = this._anchorSPAN;
			var labelClass = "LayoutContributorLabel";		
			if(href) {
				var anchor = document.createElement("A");
				dojo.addClass(anchor, "RelativeValue");
				dojo.addClass(anchor, "Anchor");
				this._anchorSPAN.appendChild(anchor);
				labelTarget = anchor;
				labelClass = "RelativeValue";
				
				if (value.locationUri) {
					var resourceLinkArgs= { 
						uri: value.locationUri,
						presentationUri: href,
						userId: value.userId,
						label: jazz.util.html.escape(name)
					};
					this._childWidgets.push(this.workingCopy.applyResourceLinkToAnchor(resourceLinkArgs, anchor));
				} else {
					anchor.href = href;
				}
			}

			if(value.locationUri) {
				anchor.title= dojo.string.substitute(this.editorMessages.titleUserNameWithId, [ name, value.userId]);
			} else {
				var labelSPAN = document.createElement("SPAN");
				labelSPAN.appendChild(document.createTextNode(name));
				if(value.userId) {
					labelSPAN.title= dojo.string.substitute(this.editorMessages.titleUserNameWithId, [ name, value.userId]);
				}
				//Set the id so that labels can be set up for this span.
				if(this.attributeId)
					labelSPAN.setAttribute("id",this.attributeId);
				if(labelClass) dojo.addClass(labelSPAN, labelClass);
				labelTarget.appendChild(labelSPAN);
			}		
			
			
			this._anchorSPAN.style.display="inline";
		},
		createSelect: function(skipOnCreateSelect) {	
			var value = WidgetUtil._bindValuesToSelect(this, this._possibleValues, this._select, null,this.moreLabel,this.moreTitle);
			var props = WidgetUtil._getCurrentSelectProps(this._select);
//			if(!value || !props) return;
			this._setReadOnlySelectValue(props.label);
			this._setWarningIconVisibility(value.id);
			if(this.onSelectCreate && !skipOnCreateSelect) {
					this.onSelectCreate(value);
				}
			this.setReadOnly(false);
		},
		_setReadOnlySelectValue: function(value){
			dojox.xml.parser.removeChildren(this._readOnlySelectValue)
			this._readOnlySelectValue.appendChild(document.createTextNode(value));
		},
		updateOptions: function(newPossibleValues, newValueId) {
			this._possibleValues = newPossibleValues;
			var value = WidgetUtil._bindValuesToSelect(this, this._possibleValues, this._select, null,this.moreLabel,this.moreTitle);

			if(newValueId) {
				this._select.value = newValueId;				
			}
			this._setWarningIconVisibility(newValueId);
			
			var label = WidgetUtil._getCurrentSelectProps(this._select);
			// If there's a value, use it's label or leave it blank
			label = (label ? (label.label ? label.label : "") : "");
			// The title requires the possibleValues to include the userId
			if(label && value && value.userId) {
				this._select.title= label + " (@" + value.userId + ")";
			}

			this._setReadOnlySelectValue(label);
			if(this.onValueChanged) {
				var selectedValue = WidgetUtil._getCurrentSelectProps(this._select);
				this.onValueChanged(selectedValue);
			}

		},
		_setWarningIconVisibility: function(selectedValueId) {
			if(selectedValueId) {
				var needsWarning = this.needsWarning(selectedValueId, /* default */ false);
				if(needsWarning && this._warningMessage){ 
					this.statusIndicator.setStatus(new com.ibm.team.workitem.api.common.Status(com.ibm.team.workitem.api.common.Severity.WARNING, this._warningMessage));
				} else {
					var status= this.workingCopy._validate(this.attributeId);
					this.statusIndicator.setStatus(status);
				}
			}
		},
		/*
		 * Override in extension
		 */
		needsWarning: function(selectedValueId, defaultValue) {
			return defaultValue;
		},
		_handleSelectChange: function(e) {	
			this._isChanged=true;
			var select= EditorUtil._getOriginalTarget(e);
			var selectedValue = WidgetUtil._getCurrentSelectProps(select);
			if (selectedValue.id != -1) { // !More...
				this._setReadOnlySelectValue(selectedValue.label);
				this.workingCopy.setValue({
					path: ["attributes", this.attributeId],
					savedAttributePath: ["attributes", this.attributeId, "id"],
					attributeId: this.attributeId,
					value: selectedValue
				});
				if (this.onValueChanged) {
					this.onValueChanged(selectedValue);
				}
				this._setWarningIconVisibility(selectedValue.id);
				this.parentWidget._handleDirtyingEvent({target: this._select});
				if (select === this._select) {
					if (this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this, selectedValue);
					}
				}
			}
			if(selectedValue.id == -1) {// More...
				this.handleMoreSelected(e);
			}
		},
		handleMoreSelected: function(e) {
			this._openOwnerSelectionDialog();
		},
		_openOwnerSelectionDialog: function() {
		    this._destroyDialog();
			var self= this;
			var params= {
					title: this.ownerMessages.dialogTitleSelectOwner,
					isMultiselect: false,
					useOkCancel: true, /* Not Add/Close */
					disableUsers: [{
						itemId:  self.workingCopy.getValue({
							path: ["attributes", self.attributeId, "id"]
						})
					}]
				};
			this.userListWidget= new SelectContributorDialog(params);
			this.connect(this.userListWidget, "onClose", dojo.hitch(this, function() { self._select.focus();this._destroyDialog();}));
			this.connect(this.userListWidget, "onOK", dojo.hitch(this, function(selectedOwnerList) { self._setSelectedOwner(selectedOwnerList);}));
			this.userListWidget.searchText.focus();
			/* for now, reset the selection back, in case the user cancels */
			this._select.selectedIndex = this._select.currentDefaultIndex;
		},
		_destroyDialog: function() {
			if(this.userListWidget) this.userListWidget.destroy(false);
			this.userListWidget = null;
		},
		_setSelectedOwner: function(selectedOwnerList) {

			if(!selectedOwnerList || selectedOwnerList.length ==0) {
				return;
			}
			
			var select = this._select;
			var found = false;
			for(var i=0; i<select.options.length;i++ ) {
				if(select.options[i].valueObject && select.options[i].valueObject.id == selectedOwnerList[0].itemId) {
					found = true;
					break;
				}
			}
			var newPossibleValues = this._possibleValues;
			if(!found) {
				selectedOwnerList[0].id = selectedOwnerList[0].itemId;
				selectedOwnerList[0].label = selectedOwnerList[0].name;
				selectedOwnerList[0].isOutOfProjectArea = true;
				newPossibleValues.push(selectedOwnerList[0]);		
			}
			this.lastSelectedOwner = selectedOwnerList[0];
			this.updateOptions(newPossibleValues,selectedOwnerList[0].itemId );
			this._setAttribute(selectedOwnerList[0], false);
			this.parentWidget._handleDirtyingEvent({target: this._select});
		},
		_setAttribute: function(value, isInit){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId],
				savedAttributePath: ["attributes", this.attributeId, "id"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				value: value
			});
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();
		}
	}
);

})();
