/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItem");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.AbstractWidget");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");
dojo.require("com.ibm.team.workitem.api.common.Severity"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.api.common.Status"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var EventTopics	= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Severity= com.ibm.team.workitem.api.common.Severity;
var Status= com.ibm.team.workitem.api.common.Status;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget",
	[dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.ui2.internal.widget.AbstractWidget],
	{
		templateString: '<div class="EnumerationWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" />' +
						'<span dojoAttachPoint="_statusIconContainer" class="StatusIconContainer">' +
						'<div dojoAttachPoint="_statusIcon"></div></span>' +
						'<div class="fieldWrapper"><select dojoAttachPoint="_select" dojoAttachEvent="onkeyup: _handleKeyUp, onchange: _handleSelectChange, onclick:_handleSelectClick, onfocus:_handleSelectFocus, onblur: _handleSelectBlur" class="Select"></select><span style="display: none;" dojoAttachPoint="_enumGuessValue" class="EnumGuessValue"></span></div>' +
						'<span style="display: none" dojoAttachPoint="_readOnlySelectValue" class="ReadOnlySelectValue"></span>' +
						'</div>',
		SHOW_ALL_ID: "_synthetic_show_all",
		MORE_ID: "_synthetic_more",
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			WidgetUtil.setSpacerImage(this._image);
			
			this.__workItemAdapter= new com.ibm.team.workitem.web.client.internal.WorkItem(this.workingCopy);
			this.statusIndicator= new com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator({}, this._statusIcon);

			this._attributePath = this._getAttributePath();
			this._savedAttributePath = this._getSavedAttributePath();
			var attributeValue = this._getAttributeValue();
			if(attributeValue) {
				this.attributeValue = attributeValue;
			}
			if(this.creationSpec && this.creationSpec.isDescription) {
					dojo.addClass(this.domNode,"MarginLeft10");
				}
			this._modifiers = this._modifiers ? this._modifiers : {};
		
			// Set all versions of a null id to null
			if(this.attributeValue && (this.attributeValue.id == null || 
			   this.attributeValue.id == "" || 
			   this.attributeValue.isNull  || 
			   this.attributeValue.id=="unassigned")){
			   	 	 this.attributeValue.id = "";
  					 this.attributeValue.isNull = true;
			  }
			
			/*
			 * If there is no value or possible values, skip the rest of initialization
			 * ... some possible values may arrive later from async callback 
			 */
			if(this._initializePossibleValues()) return;
			this._completeInitialization();
			this._doValidation();
		},
_completeInitialization: function() {
	        if(this._isInitialized) return;
	        this._isInitialized = true;
			/*
			 * EditProperties are loaded in the background
			 */
			this._editPropsChangedSubscribeHandle= dojo.subscribe(EventTopics.WORKITEM_EDITABLE_PROPERTIES_CHANGED, this, dojo.hitch(this, this._handleEditPropertiesChanged));
			
			var label = "";
			try{
				this._lastSelectedValue = this._getCurrentSelectProps(this._select);
				label = this._lastSelectedValue.label;
			} catch(e) {}; // ignore
			this._setReadOnlySelectValue(label);
			if(this._modifiers && this._modifiers.onCreate) {
					this.parentWidget[this._modifiers.onCreate](this, value);
			}
			
			/* TODO: Delete the followining */
			if(this._modifiers && this._modifiers.onValueChanged) {
					this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ true);
				}
			/* ------------------------ */
			
			if(this.onValueChanged) {
				this.onValueChanged(this.attributeValue, /* isInitialization */ true);
			}			
			
			
			if (this.workingCopy.isGuest()) {
				dojo.addClass(this._select, "GuestField");
			}
			this._isChanged = false;
			
			this.workingCopy.addListener({
				path: this._attributePath,
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			this.workingCopy.addListener({
				path: this._attributePath,
				event: EventTopics.ON_DEPENDENCY_CHANGE,
				listener: this,
				functionName: "_fetchValueSet"
			});
						
			var propertyMap = this.workingCopy.getPropertyMap(this.presentation, {}, "properties");
			if(propertyMap.readonly == "true") this.setReadOnly(true);
		},
		destroy: function(preserveDom) {
			if(this.statusIndicator) this.statusIndicator.destroy(preserveDom);
			if(this._editPropsChangedSubscribeHandle) dojo.unsubscribe(this._editPropsChangedSubscribeHandle);
			this.statusIndicator = null;
            this.inherited(arguments);
		},
//		destroyWidget: function() {
//			this.inherited(arguments);
//		},
		/*
		 * Allow attributeId to be overridden 
		 */
		_getAttributeId: function() {
			if(this._getOverrideAttributeId) {
				return this._getOverrideAttributeId();
			}
			return this.attributeId;
		},
		_getAttributePath: function() {
			return ["attributes",  this._getAttributeId()];
		},
		_getSavedAttributePath: function() {
			return ["attributes",  this._getAttributeId(), "id" ];
		},
		_handleAttributeChange: function(parms) {
			if(parms.attributeValue && parms.attributeValue.id != null) {
				this.setValue(parms.attributeValue.id, false, false, parms.attributeValue);
			} else {
				this.setValue(parms.attributeValue, false, false, parms.attributeValue); // TODO: Rationalize attribute change
			}
			if(parms.validationStatus)
				this.statusIndicator.setStatus(parms.validationStatus);
		},
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { 
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this._select.style.display = "none";
				this._readOnlySelectValue.style.display = "inline";
			}
			else {
				this._select.style.display = "inline";
				this._readOnlySelectValue.style.display = "none";
			}
		},
		_setReadOnlySelectValue: function(value){
			if(value == null) {
				if(this._modifiers.noValueLabel != null) {
					value = this._modifiers.noValueLabel;
				} else {
					value = this.messages.labelDefaultNullEnumerationValue;
				}
			}
			 
			dojox.xml.parser.removeChildren(this._readOnlySelectValue)
			this._readOnlySelectValue.appendChild(document.createTextNode(value));
		},
		_handleKeyUp: function(e) {
			var select= EditorUtil._getOriginalTarget(e);
			var selectValue = this._getCurrentSelectProps(select);
			if(selectValue && this._lastSelectedValue) {
				if(selectValue.id != this._lastSelectedValue.id) {
					this._handleSelectChange(e);
				}
			} else {
				this._handleSelectChange(e);
			}
		},
		isShowAll: function(selectValue) {
			return false;
		},
		// doNotPropagateAgain means that this change came from a change path, so
		// do not propagate again.
		_handleSelectChange: function(e, doNotPropagateAgain, skipDirtyEvent) {
			if(this._isChangingOptions) return;
			this._isChanged=true;
			var select= EditorUtil._getOriginalTarget(e);
			var selectValue = this._getCurrentSelectProps(select);
			if(this.isShowAll(selectValue)) {
				this._isChangingOptions = true;
				// bindValuesToSelect will select this.attributeValue			
				value = this.setPossibleValues(this._possibleValues, this._select, this._image, true);
				this._isChangingOptions = false;
				return;
			}
			this._lastSelectedValue = selectValue;
			if (!selectValue) {
				if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
					this.parentWidget._handleDirtyingEvent(e);
				}
				return;
			}
			this._setReadOnlySelectValue(selectValue.label);
			if(selectValue.iconUrl) {
				this._image.src = selectValue.iconUrl;
				this._image.style.visibility="visible";
			} else {
				WidgetUtil.setSpacerImage(this._image);
			}
			
			if (select === this._select) {
				if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
					for(var i = 0;i<this._possibleValues.length;i++) {
						if(this._possibleValues[i].id == selectValue.id) {
							selectValue.value =this._possibleValues[i];
							break; 
						}
					}					
					this.parentWidget[this._modifiers.onValueChanged](this , selectValue, /* isInitialization */ false);
				}
			}
			/*------------------------------- */
			/* Call the onValueChanged method */
			if (select === this._select) {
				if(this.onValueChanged) {
					for(var i = 0;i<this._possibleValues.length;i++) {
						if(this._possibleValues[i].id == selectValue.id) {
							selectValue.value =this._possibleValues[i];
							break; 
						}
					}					
					this.onValueChanged(selectValue, /* isInitialization */ false);
				}
			}
			/*------------------------------- */

			
			this._handleSelectFocus(e);// Use the short form of labels.
			if(!doNotPropagateAgain) {
				this.setAttributeValue(selectValue)
			}
			if(!skipDirtyEvent && this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}
		},
		_handleSelectClick: function(e) {
		}, 
		_handleSelectFocus: function(e) {
			/* Special case for categories */
			if (dojo.isIE) return;
			try {
			if(!this._modifiers || !this._modifiers.allCategories) return;
			var select= EditorUtil._getOriginalTarget(e);
			var index  = select.selectedIndex;
			if(!index)return;
			var valueObject = this._possibleValues[index];
			if(!valueObject) return;
			if(select.options[index].__jazzWasClicked) return;
			select.options[index].__jazzWasClicked = true;
			var newLabel = valueObject.shortLabel;
			
			if(select.options[select.selectedIndex].childNodes[0] && 
			   select.options[select.selectedIndex].childNodes[0].nodeValue == newLabel) return;
			
			dojox.xml.parser.replaceChildren(select.options[select.selectedIndex], document.createTextNode(newLabel));
			var indent = EditorUtil._getIndent(newLabel);
			if(indent > 0) {
				select.options[select.selectedIndex].style.paddingLeft= (indent * 0.4) + "em";
			} else {
				select.options[select.selectedIndex].style.paddingLeft="";
			}
			} catch(e) {};
		},
		_handleSelectBlur: function(e) {
			if (dojo.isIE) return;
			try{
			/* Special case for categories */
			if(!this._modifiers.allCategories) return;
			var select= EditorUtil._getOriginalTarget(e);
			for(var i=0;i<select.options.length;i++){
				if(select.options[i].__jazzWasClicked){
					dojox.xml.parser.replaceChildren(select.options[i], document.createTextNode(select.options[i].__jazzLabel));
					var indent = EditorUtil._getIndent(select.options[i].__jazzLabel);
					if(indent > 0) {
						select.options[select.selectedIndex].style.paddingLeft= (indent * 0.4) + "em";
					} else {
						select.options[select.selectedIndex].style.paddingLeft="";
					}
					select.options[i].__jazzWasClicked = false;
				}
			}
			} catch(e) {};
		},
		getSelectValues: function(values, isShowAll) {
			return values;
		},
		setPossibleValues: function(possibleValues, select, image, isShowAll) {
			select = select || this._select;
			this._possibleValues = possibleValues;
			var result = this._bindValuesToSelect(this.getSelectValues(possibleValues, isShowAll), select, image);
			this._completeInitialization();
			return result;
		},
		_handleEditPropertiesChanged: function(parms) {
		 	if(parms.workingCopy != this.workingCopy) return;
		 	var allValues = this.workingCopy.getEditPropsAllValues();
			if (allValues) {
				possibleValues = [];
				for (var k = 0; k < allValues.length; k++) {
					var value = allValues[k];
					if (value && value.attributeName && value.uiItems) {
						if (value.attributeName == this.attributeId) {
							possibleValues = value.uiItems;
							break;
						}
					}
				}
			}
			this._isAlwaysDisabled=true;
			
			this._possibleValues = this._getPossibleValues(possibleValues);
			this._initializePossibleValues();

		},
		_getPossibleValues: function(editPropertyPossibleValues) {
			return editPropertyPossibleValues;
		},
		_getAttributeValue: function() {
			var attributeValue = this.workingCopy.getValue({path: this._attributePath});
			if(attributeValue) {
				// adapt non-UIItemDTOs
				if (!attributeValue.id && attributeValue.itemId)
					attributeValue.id= attributeValue.itemId;
				if (!attributeValue.label && attributeValue.name)
					attributeValue.label= attributeValue.name;
			}
			return attributeValue;
		},
		_initializePossibleValues: function() {
			// If there are no possible values, but there is a value,
			// Add the value as a possible value, but always be disabled
			if(this.isStatusAttributePart && !this.workingCopy.isInTypeChange) {				
				this._isAlwaysDisabled = false;
			}
		
			var isAddingValueToEmptyPossibleValues = true;
			if(dojo.isFunction(this._isAddingValueToEmptyPossibleValues)) {
				isAddingValueToEmptyPossibleValues = this._isAddingValueToEmptyPossibleValues();
			}
			var attributeValue = this._getAttributeValue();			
			this.attributeValue = attributeValue;
			
			if(this._possibleValues ==null || this._possibleValues.length == 0) {
				WidgetUtil.setSpacerImage(this._image);
				this._isAlwaysDisabled=true;
				if(attributeValue && isAddingValueToEmptyPossibleValues) { 
					this._possibleValues = [
						attributeValue
					];
				}
			}
			this._possibleValues = this._possibleValues || [];

			this._originalValueSet= dojo.clone(this._possibleValues);

			
			// Set all versions of a null id to an empty string
			for (var i = 0; i < this._possibleValues.length; i++) {
				if (this._possibleValues[i] &&
				(this._possibleValues[i].id == null ||
				this._possibleValues[i].id == "" ||
				this._possibleValues[i].isNull ||
				this._possibleValues[i].id == "unassigned")) {
					this._possibleValues[i].id = "";
					this._possibleValues[i].isNull = true;
				}
			}

			// if the attribute value is null, disable and return
			if(this.attributeValue == null) {
			  this._select.disabled = true;
			  if(this._possibleValues && this._possibleValues.length > 0) {
					this.setPossibleValues(this._possibleValues, this._select, this._image);
			  }
			  return true; // isDisabled
			};

			/*
			 * if the value is not in possible values
			 * then add the value as the first possible value
			 * do not add the value if moving from one PA to another, since the value is for the source PA
			 * 
			 */
			var isInMove = false;
			if(this.workingCopy && dojo.isFunction(this.workingCopy.isInMove)) {
				isInMove = this.workingCopy.isInMove();
			} 
			
			var value = this.attributeValue;
			if(!isInMove && value) {
				var found = false;
				for(var i=0;i<this._possibleValues.length;i++) {
					if(this._possibleValues[i].id == value.id) {
						found = true;
						break;
					}
				}
				if (!found && isAddingValueToEmptyPossibleValues) { 
					this._possibleValues.unshift(value);
				}
			}
			this._isAlwaysDisabled = this._isDisabled();
			this.setPossibleValues(this.getSelectValues(this._possibleValues, false), this._select, this._image);
			
			return false; // isDisabled
	},
	_isDisabled: function() {
		return false;
	},
		_bindValuesToSelect: function(possibleValues, select, image) {
			var resultValue = "";
			var	selectedValueId  = select.value;
			if(selectedValueId == "" || selectedValueId == this.SHOW_ALL_ID) {
				selectedValueId=this.attributeValue ? this.attributeValue.id : "";
			}
			dojox.xml.parser.removeChildren(select);
			
			if (!possibleValues || possibleValues.length === 0) {
				select.disabled= true;
				WidgetUtil.setSpacerImage(image);
				if(dojo.isFunction(this._getEmptyValueLabel)) {
					var label = this._getEmptyValueLabel();
				} else {
					var label = this.messages.labelDefaultNullEnumerationValue;					
				}
				if (this._modifiers.noValueLabel != null) {
					label = this._modifiers.noValueLabel;
				}
				EditorUtil._addOption(select, new Option(label, EditorUtil.UNCONFIGURED));
				return resultValue;
			} else {
				select.disabled= this._isAlwaysDisabled;
			}
			
			var defaultIndex = 0;
			for (var i = 0; i < possibleValues.length; i++) {
				if(!possibleValues[i]) continue;
				if(possibleValues[i].id == selectedValueId) {; // can be ""		
					defaultIndex= i;
					break;
				}
			}
			
			for (var i=0; i<possibleValues.length; i++) {
				if(!possibleValues[i]) continue;
				var option= EditorUtil._createOption(possibleValues[i].label, possibleValues[i].id);
				option.valueObject = possibleValues[i];
				if(possibleValues[i].title) {
					option.title=possibleValues[i].title;
				}
				 
				option[EditorUtil._getLabelProp()]= possibleValues[i].label;
	
				var isDefault = i== defaultIndex;
	
				if(isDefault) {
					resultValue = possibleValues[i];
					select.currentDefaultIndex = i;
					if(image) {
						if(possibleValues[i].iconUrl) {
							image.src = possibleValues[i].iconUrl;
							image.style.visibility="visible";	
						} else {
							WidgetUtil.setSpacerImage(image);
						}
					}
				} 
				EditorUtil._addOption(select, option, isDefault);
				if(isDefault && possibleValues[i].title) {
					select.title = possibleValues[i].title;
				}
			}
			var label = "";
			try {
				label = this._getCurrentSelectProps(this._select).label;
			} catch(e) {}; // ignore
			this._setReadOnlySelectValue(label);
		return resultValue;
	},
	setEnabled: function(isEnabled) {
		this._select.disabled = !isEnabled;
	},
	     getValue: function() {
		 	var value = this._select.value;
			return value;
		 },
		 getSelectedLabel: function() {
			if(!this._select) return "";
			return this._getCurrentSelectProps(this._select).label;
		},
		isSelectedValueNull: function() {
			var value = this.getValue();
			var isNull = false;
			if(this._possibleValues) {
				for(var i=0;i<this._possibleValues.length;i++) {
					if(this._possibleValues[i].id == value) {
						isNull = this._possibleValues[i].isNullValue;
						break;
					}
				}
			}
			return isNull;
		},
		 setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
		 		if (this._select.value != retainedState[this.attributeId]) {
					this._select.value = retainedState[this.attributeId];
					this._lastValue = retainedState[this.attributeId];
					this._isChanged = true;
//					this._setReadOnlySelectValue(this._getCurrentSelectProps(this._select).label);
					var selectValue = this._getCurrentSelectProps(this._select);
					this._setReadOnlySelectValue(selectValue.label);
					if(selectValue.iconUrl) {
						this._image.src = selectValue.iconUrl;
						this._image.style.visibility="visible";
					} else {
						WidgetUtil.setSpacerImage(this._image);
					}
				}
			}
		 },
		_getCurrentSelectProps: function(select) {
			if(select.selectedIndex == -1 || !select.options || select.options.length ==  0) return null;
			if(select.value == EditorUtil.UNCONFIGURED) return null;
			var currentOption= select.options[select.selectedIndex];
			if(!currentOption) return null;
			var props= {
				label: currentOption[EditorUtil._getLabelProp()],
				id: select.value
			};
			if(currentOption.valueObject) {
				props.iconUrl = currentOption.valueObject.iconUrl;
			}
			return props;
		},
		setAttributeValue: function(selectValue, setValue, value, skipPossibleValuesCheck ) {
		    if (setValue) {
				this.setValue(selectValue.id, false, false, value, skipPossibleValuesCheck);
			}
			var parms = {
				caller: this,
				path: this._attributePath,
				savedAttributePath: this._savedAttributePath,
				attributeId: this._getAttributeId(),
				value: selectValue
			};
			this.workingCopy.setValue(parms);
			this._doValidation();
		},
		setValue: function(valueId, noNotify, skipDirtyingEvent, value, skipPossibleValuesCheck) {
			if(valueId == "null") valueId = null;
			this._lastValue = valueId;
			this._select.value = valueId;
			var selectedValue = WidgetUtil._getCurrentSelectProps(this._select);
			/*
			 * If the new value is not in the select, add it
			 */
			if(!skipPossibleValuesCheck 
				&& (!selectedValue || (selectedValue.id != valueId && selectedValue != valueId) 
				&& !(selectedValue.id == EditorUtil.UNCONFIGURED && selectedValue.label == null))) {
				    this._possibleValues = this._possibleValues || [];
					this._possibleValues.unshift(value);
					if(this._modifiers && this._modifiers.readonly != "false") {
						this._setReadOnlySelectValue(value);
					} else {
						this.setPossibleValues(this._possibleValues, this._select, this._image)
					}
					this._select.value = valueId;
				}
			
			if(!noNotify) {
				this._handleSelectChange({target: this._select}, /* doNotPropagateAgain */ true, skipDirtyingEvent);
			}
		},
		
		_fetchValueSet: function() {
			var srh= this._createValueSetHandler();
			this.__workItemAdapter.getValueSet(this.attributeId, srh);
		},
		
		_createValueSetHandler: function() {
			var self= this;
			var serviceObj= {
				success: function(values) {
					if (!values) {
						values= [];
					}
					if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
					
					self._originalValueSet= dojo.clone(values);
					
					/* Make sure the current value is in the values */
					var index = self._select.selectedIndex;
					var currentValue = self._possibleValues[index];
					if(currentValue && currentValue.id) {
						var found = false;
						for(var i=0;i<values.length; i++) {
							if(values[i].id == currentValue.id) {
								found = true;
								break;
							}						
						}
						if(!found){
							values.unshift(currentValue);
						}
					}
					
					self.setPossibleValues(values, self._select, self._image);
// which does...	self._bindValuesToSelect(values, self._select, self._image);
					self._doValidation();
				},
				failure: function(errorObj) {
					if(self.parentWidget && self.parentWidget.isDestroyed) return; // returned after destroyed
					self.parentController.handleError(errorObj, "A problem occurred while trying to fetch enumeration.");
				}
			};
			return new ServiceResponseHandler(serviceObj, "success", "failure");
		},
		
		_doValidation: function() {
			var status= this.workingCopy._validate(this._getAttributeId());
			if (status != null && status.isOK() && !this._isApplicableValue()) {
				status= new Status(Severity.ERROR, this.messages.validationNotApplicable);
			}
			this.statusIndicator.setStatus(status);
		},
		
		_isApplicableValue: function() {
		
			var attribute= this.__workItemAdapter.getAttribute(this.attributeId);
			if (attribute && !attribute.isEnumeration())
				return true;
		
			var currentValue= this._getAttributeValue();
			var values= this._originalValueSet;
			
			if ((!currentValue || currentValue.id == "null") && values.length == 0)
				return true;
			
			if (currentValue.isArchived)
				return true;

			for(var i=0; i < values.length; i++) {
				if(values[i].id == currentValue.id) {
					return true;
				}						
			}
			return false;
		},
		
		__sentinel: null
	}
);

})();
