/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("jazz.core.loader");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var Util = com.ibm.team.workitem.web.client.util;
var EventTopics = com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var editorProxyClass = com.ibm.team.workitem.web.ui.internal.view.richtext.getEditorProxyClass();

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.HTMLWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="HtmlWidget" >' +
						'<span dojoAttachPoint="_statusIconSPAN" class="StatusIconContainer">' +		
						'<div dojoAttachPoint="_statusIcon"></div></span>' +						   
						'<span style="display:block" dojoAttachPoint="descriptionSPAN" dojoAttachEvent="onchange: _handleChange" tabIndex="0"></span>' +
						'<span style="display:block" dojoAttachPoint="descriptionTextSPAN" class="DescriptionTextSpan"></span>' +
						'</div>',					
		_isChanged: false,
		_wasChangeNotified: false,
		
		editorProxyClass: editorProxyClass,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-workitem-richtext");
			this.setSectionMenu = this.creationSpec.setSectionMenu == true || (this.isWiki && this.sectionLayout);
			var editorArgs = { parentController: this.parentController };
			if(!this.setSectionMenu){
				editorArgs.height = "8.8em"; // SelfSizingTextAreaShort
			}
			this._editorProxy = new this.editorProxyClass({
				editorArgs: editorArgs,
				parentNode: this.descriptionTextSPAN
			});
			this.connect(this._editorProxy, "onEditorCreated", "setupEditor");
		
			this._childWidgets = [];
			this._eventHandle = dojo.subscribe(EventTopics.WORKITEM_ABOUT_TO_SAVE, this, this._workItemAboutToSave);

			this._initValues();
			this._isReadOnly = this.workingCopy.isGuest() || this._hasReadOnlyProperty();
			this._isPreview = this._isReadOnly || !this.workingCopy.isNewWorkItem();

			this._isTextWidget = this.creationSpec.originalWidgetName == 'com.ibm.team.workitem.web.ui.internal.view.editor.parts.TextAttributePart';

			this.isDescription = this.creationSpec.isDescription == true || this.isWiki == true;
			
			this._descriptionSpanClassName = "DescriptionSpan";
			this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty";
			if(this._modifiers.isSpanningColumns) {
				this._descriptionSpanClassName = "DescriptionSpan_TwoColumn";
				this._descriptionSpanEmptyClassName = "DescriptionSpanEmpty_TwoColumn";
				dojo.addClass(this.domNode, "HtmlWidget_TwoColumn");
				this.descriptionTextSPAN.className = "DescriptionTextSpan_TwoColumn";
				this.descriptionSPAN.className = "DescriptionSpan_TwoColumn";
			}

			dojo.addClass(this.descriptionSPAN, "readonly-text");
						 
			if(this.sectionLayout && (this._modifiers.readonly != "true") && !this._isTextWidget && !this.workingCopy.isGuest() && !this._modifiers.isString && (this._modifiers.isSpanningColumns || (this.isDescription && !this.setSectionMenu))) {
					var editTitle;
					if(this.presentation && this.presentation.attributeLabel) {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.presentation.attributeLabel] );
					} else {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.attributeId ] ); // probably never used
					}
					this._editAction = this.sectionLayout.getAction(this.messages.buttonEdit,  /* icon */  null, /* disable icon */ null, /* icon title */ editTitle,  this, "_toggleHTMLEditor", /* enable */ true);
					dojo.addClass(this._editAction, "SectionActionEmbedded");
					this.descriptionTextSPAN.appendChild(this._editAction);
			}			

			if (!this._isReadOnly) {
				if (this.sectionLayout && this.setSectionMenu && !this._editAction) {
					var editTitle;
					if(this.presentation && this.presentation.attributeLabel) {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.presentation.attributeLabel] );
					} else {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.attributeId ] ); // probably never used
					}
					this._editAction = this.sectionLayout.addAction(this.messages.buttonEdit, /* icon */ null, /* disable icon */ null, /* icon title */ editTitle, this, "_toggleHTMLEditor", /* enable */ true);
					if(this.isWiki) {
						this._popupHelpSpan = this.sectionLayout.addPopupHelp(this.messages.helpWikiText,this.messages.titleWikiHelp);	
					}
				}
			}
			
			if(this._modifiers && this._modifiers.webHeight) {
				this.setHeight( this._modifiers.webHeight);
			}
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
				
			this.statusIndicator= new com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator({}, this._statusIcon);
			var status= this.workingCopy._validate(this.attributeId);
			this._setStatus(status);

			if(this._editAction){
				if(this.sectionLayout){
					dojo.addClass(this.sectionLayout.innerBorderDIV, "SectionActionRichText");
				}
			}else{
				if (!this._isReadOnly){
					this.connect(this.descriptionSPAN, "onfocus", "_showHTMLEditor");
				}
			}

			// 154752: New work item focus should be in Summary not Description
			this._setHTMLEditorState(this._isPreview, this.workingCopy.isNewWorkItem());
		},
		
		_initValues: function(){
			var value = this._getAttributeValue();
			var content;
			if(value && value.content !== null) {
				content = dojo.trim(value.content);
			} else {
				content = (value === null) ? "" : dojo.trim(value + ""); // Handle nonstring values;
			}

			// Links need to be resolved in the original read only value because the links refer to offsets 
			if(!this.isWiki && value && value.links && value.links.length > 0 ) {
				this._originalReadOnlyValue = EditorUtil.resolveLinks(content, value.links, this.workingCopy); // handle "bug 1234" in content
			} else  {
				this._originalReadOnlyValue = content;
			}
			this._origContent = EditorUtil.cleanRawValue(content);
			content = content.replace(/<br\/>/gi, "\n");
			
			if(this.isWiki){
				this.setValue(content);
			}else{
				this.setValue(this._origContent);
			}

			return content;
		},
		
		setupEditor: function(editor){
			editor.connect(editor, "onBlur", dojo.hitch(this, this._handleBlur));
			editor.connect(editor, "onNormalizedDisplayChanged", dojo.hitch(this, this._handleTextChange));
			
			// hide this editor if other rich text editor is opened
			editor.subscribe(EventTopics.WORKITEM_ACTION_SHOW_RICH_TEXT, dojo.hitch(this, function(ed){
				if(editor !== ed){
					this._setHTMLEditorState(true);
				}
			}));
		},
		getAttachmentInfo: function(number) {
			if(this.parentController && this.parentController.getAttachmentInfo) {
				return this.parentController.getAttachmentInfo(number);
			}
		},
		_hasReadOnlyProperty: function() {
			if (this.presentation && this.presentation.properties) {
				var props= Util.buildMap(this.presentation.properties);
				return props.readonly == "true";
			}
			return false;
		},
		_handleAttributeChange: function(parms) {
			var attrValue = parms.attributeValue || "";
			var value = this.getValue() || "";
			if(value == EditorUtil.cleanRawValue(attrValue)) return;

			this.setValue(attrValue);

			this._setHTMLEditorState(this._isReadOnly, /* skipFocus */ true );
			if(parms.validationStatus)
				this._setStatus(parms.validationStatus);
		},
		_setStatus: function(status) {
			this._editorProxy.setProperty("status", status);			
			this.statusIndicator.setStatus(status);			
		},
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.attributeId] });			
		},
		destroy: function(preserveDom) {
			this.workingCopy.removeListener({
				path: ["attributes", this.attributeId],
				listener: this
			});
			if(this._editorProxy){ this._editorProxy.destroy(); }
			this._editorProxy = null;
			if(this.statusIndicator) this.statusIndicator.destroy(preserveDom);
			this.statusIndicator = null;
			dojo.unsubscribe(this._eventHandle);
			this.clearChildWidgets();
            this.inherited(arguments);
		},
		destroyWidget: function() {
			// _updateListener is used by Wiki via _WikiMixin
			if(this._updateListener) dojo.disconnect(this._updateListener);
			this.destroy();
		},
		getValue: function() {
			return this._editorProxy.getValue();
		},
		_checkForChanges: function(e, notifyEvenIfUnchanged, skipHandleChange) {
			if (this._isValueChanged() || notifyEvenIfUnchanged) {
				var e = {
					srcElement: this._editorProxy.getEditorNode()
				};
				if(!skipHandleChange) {
					this._handleChange(e, notifyEvenIfUnchanged);
					return true;
				}
			}
			return false;
		},
		_isValueChanged: function() {
			var oldValue = this._origContent;
			var newValue = this.getValue();
			var attrValue = this._getAttributeValue();
			if(attrValue && attrValue.content) {
				return newValue != attrValue.content;
			}
			return (oldValue != newValue);
		},
		setValue: function(value) {
			this._editorProxy.setValue(value);
		},
		_cleanText:function(value) {
			return EditorUtil.restoreTags(value);
		},
		_handleBlur: function(e){
			if(!this._isValueChanged()) return;
			this._wasChangeNotified = false; // force notify
			if(this._checkForChanges(e, /* notifyEvenIfUnchanged */ true, /* skipHandleChange*/ false)) return;
			this._handleChange(e, /*notifyEvenIfUnchange*/ true);
		},
		_workItemAboutToSave: function(parms) {
			if(this._destroyed) return;
			if(!(parms.workItemProxy === this.workingCopy)) return;
			this._checkForChanges({}, /* notifyEvenIfUnchanged */ false, /* skipHandleChange*/ false);
		},
		_handleTextChange: function(e){
			if(this._destroyed) return;
			this._checkForChanges(e, /* notifyEvenIfUnchanged */ false, /* skipHandleChange*/ false);
		},
		_handleChange: function(e, notifyEvenIfUnchanged) {
			this._isChanged= true;
			notifyEvenIfUnchanged = notifyEvenIfUnchanged && !this._wasChangeNotified;
//			var isQuiet = this.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
			var isQuiet = false;
			this._setWorkingCopyValue(this.getValue(), /* isInit */ false, isQuiet, notifyEvenIfUnchanged == true);
			this._wasChangeNotified = true;
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent){
				this.parentWidget._handleDirtyingEvent(e);
			}
		},
		_setWorkingCopyValue: function(value, isInit, isQuiet, notifyEvenIfUnchanged){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "content"],
				savedAttributePath: ["attributes", this.attributeId, "content"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				isQuiet: isQuiet,
				notifyEvenIfUnchanged: notifyEvenIfUnchanged,
				value: value
			});
			var status= this.workingCopy._validate(this.attributeId);
			this._setStatus(status);
		},
		_toggleHTMLEditor: function() {
			this._setHTMLEditorState(!this._isPreview);
			},
		_showHTMLEditor: function() {
			this._setHTMLEditorState(false);
			},
		_showHTMLPreview: function() {
			this._setHTMLEditorState(true);
			},
						
		_setHTMLEditorState: function(isReadOnly, skipFocus) {
			this._isPreview = isReadOnly;
			if(/* !this._modifiers.isString && */ isReadOnly) {
				if (this._editAction) {
					dojox.xml.parser.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonEdit));
					var editTitle;
					if(this.presentation && this.presentation.attributeLabel) {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.presentation.attributeLabel] );
					} else {
						editTitle = dojo.string.substitute(this.messages.titleEdit, [ this.attributeId ] ); // probably never used
					}
					this._editAction._jazzAnchor.title = editTitle;
					if(this.isWiki && this._popupHelpSpan) this._popupHelpSpan.style.visibility = "hidden";
				}

				// If the value is changed, use the user's value, else the original value with links added.
				// The _getTextValue will not have phrases resolved to links (e.g., "bug 1234")
				// since that is done on the server
				var value;
				if(this._origContent != this.getValue()){
					value = dojo.trim(this.getValue());
					if(dojo.isIE) {
						value = value.replace(/\n/g,"<br _synthetic_/>"); // preserve line feeds
						// need to remove all scripts but should allow bold, italic, links
						value = this.escape(value);
						value = value.replace(/\n/g,""); // remove line ends added by escape
						value = value.replace(/\&lt;br _synthetic_\/\&gt;/g,"\n"); // put original line feeds back
					} else {
						// need to remove all scripts but should allow bold, italic, links
						value = this.escape(value);
					}
				}else{
					value = this._originalReadOnlyValue || "";	
				}

				var className = this._descriptionSpanClassName;
				var span;
				if(value.length === 0){
					className = this._descriptionSpanEmptyClassName;
					span = document.createElement("SPAN");
					if(this._modifiers.noValueLabel){
						value = this._modifiers.noValueLabel;
						span.appendChild(document.createTextNode(value));
					} else {
						span.innerHTML = "&nbsp;"; // innerHTML OK
					}
				}
				else {
					var span = this._getReadOnlySpan(value);
				}

				if(dojo.isWebKit) {
					span.style.wordWrap = "break-word";
				}
				dojox.xml.parser.replaceChildren(this.descriptionSPAN,span);
				this.clearChildWidgets();
				this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(span, this.parentWidget._workItem)); // must be in dom
				this.descriptionSPAN.className = className;
				dojo.addClass(this.descriptionSPAN, "readonly-text");
				this.descriptionSPAN.style.display = "block";
				this._statusIconSPAN.style.display = "block";
				this._showEditor(false);
				
//				this.sectionLayout.setActionIcon(this._editDescriptionAction, ImagePool.EDIT, "Edit the description");
			} else {
				if (this._editAction) {
					dojox.xml.parser.replaceChildren(this._editAction._jazzAnchor, document.createTextNode(this.messages.buttonPreview));
					var previewTitle;
					if(this.presentation && this.presentation.attributeLabel) {
						previewTitle = dojo.string.substitute(this.messages.titlePreview, [ this.presentation.attributeLabel] );
					} else {
						previewTitle = dojo.string.substitute(this.messages.titlePreview, [ this.attributeId ] ); // probably never used
					}
					this._editAction._jazzAnchor.title = previewTitle;
					if(this.isWiki && this._popupHelpSpan) this._popupHelpSpan.style.visibility = "visible";
				}
				
				if(this.domNode) {
					this.domNode.title = "";
				}
				this.descriptionSPAN.style.display = "none";
				this._statusIconSPAN.style.display = "none";
				this._showEditor(true);
				if(!skipFocus) this.focus();
			}
		},

		escape: function(value){
			return jazz.util.html.removeScript(value);
		},

		_showEditor: function(bShow){
			if(bShow){
				this._editorProxy.showEditor();
			}else{
				this._editorProxy.hideEditor();
			}
		},

		_getReadOnlySpan: function(value) {
			return EditorUtil.getSafeTextSpan(value, this);
		},
		_handleClick: function(e){
			if(!this._isPreview) return;
			// If you click on an anchor, just follow the link
			if(e.target.tagName == "A") return;
			this._setHTMLEditorState(false);
		},
		setWidth: function(width) {
			this._editorProxy.setProperty("width", width);
		},
		setHeight: function(height) {
			this._editorProxy.setProperty("height", height);
		},

		focus: function(){
			this._editorProxy.focus();
		},
	
		clearChildWidgets: function(){
			for(var i=0;i< this._childWidgets.length;i++) {
				if(this._childWidgets[i] && this._childWidgets[i].destroy) {
					this._childWidgets[i].destroy();			
				}
				this._childWidgets[i] = null;
			}
			this._childWidgets = [];
		},
		
		handleSaveComplete: function(){
			this._initValues();
			this._setHTMLEditorState(true);
		}
	}
);

})();
