/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
	
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SEVERITY_INFO = 0;
var SEVERITY_WARNING = 4;
var SEVERITY_ERROR = 8;

var checkNotNull=	net.jazz.ajax.toolbox.lang.checkNotNull;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout",
	[dijit._Widget, dijit._Templated], 
	{
	templateString: '<div class="com-ibm-team-workitem-view-layout-NameValueLayout">'
		+ '<table CELLPADDING="0" CELLSPACING=0 dojoAttachPoint="_nameValueTable" class="com-ibm-team-workitem-namevalue-table">'
		+ '</table></div>',
	postCreate: function() {
		// hasRequiredMarker
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		this.labelSpans = [];
		this._childWidgets = [];
		this.inherited(arguments);
	},
	destroy: function() {
		for(var i=0;i< this._childWidgets.length;i++) {
			if(this._childWidgets[i] && this._childWidgets[i].destroy) {
				this._childWidgets[i].destroy();			
			}
			this._childWidgets[i] = null;
		}
		this._childWidgets = [];
		this._valueNode = false;
		this.inherited(arguments);
	},
	setAttributeId: function(id) {
		if(this._lastRequiredMark) {
			this._lastRequiredMark.setAttributeId(id);
		}
	},
	addRow: function(parms /* labelText: labelText, valueNode, isWide, isOneLine, widget */ ) {
		if(!dojo.attr(parms.valueNode, "tabindex")) dojo.attr(parms.valueNode, "tabindex", 1);
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		if(parms.labelText && parms.labelText.indexOf(":") == -1 && parms.labelText.indexOf(String.fromCharCode(65306)) == -1 ) parms.labelText = dojo.string.substitute(this.messages.labelValue, [parms.labelText]);
		this._valueNode = parms.valueNode;
		if(!parms.isWide) {
			var labelCol = row.insertCell(-1);
			labelCol.className = "com-ibm-team-workitem-namevalue-label-td";
			var labelSpan = document.createElement("div");
			labelSpan.className = "com-ibm-team-workitem-namevalue-label-node";
			labelTextSpan = dojo.create("span", { className:"formLabel" });
			labelTextSpan.appendChild(document.createTextNode(parms.labelText || "" ));
			labelSpan.appendChild(labelTextSpan);
			this.labelSpans.push(labelTextSpan);
			if(this.hasRequiredMarker && (this.attributeId || (parms.widget && parms.widget.attributeId))) {
//				var reqCol = row.insertCell(-1);
//				reqCol.className = "com-ibm-team-workitem-namevalue-required-td";
				var attributeId = (parms.widget && parms.widget.attributeId) ? parms.widget.attributeId : this.attributeId;
				labelSpan.appendChild(this._getRequiredMarkerElement(attributeId));
			} else {
				this._lastRequiredMark = null;
			}
			
			if(parms.widget && parms.widget.attributeId) {
				var labelElement = document.createElement("LABEL");
				labelElement.setAttribute("for",parms.widget.attributeId)
				labelElement.appendChild(labelSpan);
				labelCol.appendChild(labelElement);
				parms.valueNode.setAttribute("id", parms.widget.attributeId);
			} else {				
				labelCol.appendChild(labelSpan);
			}
			
			var valueCol = row.insertCell(-1);
			valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
			valueCol.appendChild(parms.valueNode);
		} else {
			var sectionCol = row.insertCell(-1);
			sectionCol.colSpan = 3;
			if(!parms.hideLabel) {				
				var nameDiv = document.createElement("DIV");
				nameDiv.appendChild(document.createTextNode(parms.labelText))
				if(parms.widget && parms.widget.attributeId) {
					var labelElement = document.createElement("LABEL");
					if(parms.isOneLine) {
						dojo.addClass(labelElement, "com-ibm-team-workitem-namevalue-label-oneline");
					}
					labelElement.setAttribute("for",parms.widget.attributeId)
					labelElement.appendChild(nameDiv);
					sectionCol.appendChild(labelElement);

				} else {				
					if(parms.isOneLine) {
						dojo.addClass(nameDiv, "com-ibm-team-workitem-namevalue-label-oneline");
					}
					sectionCol.appendChild(nameDiv);
				}
			}
			if(parms.isOneLine) {
				dojo.addClass(parms.valueNode, "com-ibm-team-workitem-namevalue-value-oneline");
			}
			if(parms.widget && parms.widget.attributeId) {				
				parms.valueNode.setAttribute("id", parms.widget.attributeId);
			}
			sectionCol.appendChild(parms.valueNode);
		}
		return row;
	},
	addIconMessageRow: function(severity, message, isSmallImage, color) {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		
		var labelCol = row.insertCell(-1);
		labelCol.className = "com-ibm-team-workitem-namevalue-img-td";
		var labelSpan = document.createElement("div");
		labelSpan.className = "com-ibm-team-workitem-namevalue-img-node";
		
		var icon = document.createElement("IMG");
		if(isSmallImage) {
			switch(severity) {
				case SEVERITY_WARNING:
				icon.src = ImagePool.WARNING;
				break;			
				case SEVERITY_ERROR:
				icon.src = ImagePool.INFO;
				break;
				default:
				icon.src = ImagePool.INFO;
			}
			icon.className = "com-ibm-team-workitem-namevalue-img-small";
		} else {
			switch(severity) {
				case SEVERITY_WARNING:
				icon.src = ImagePool.WARNING36;
				break;			
				case SEVERITY_ERROR:
				icon.src = ImagePool.ERROR36;
				break;
				default:
				icon.src = ImagePool.INFO36;
			}
			icon.className = "com-ibm-team-workitem-namevalue-img-large";		
		}
		labelSpan.appendChild(icon);
		labelCol.appendChild(labelSpan);
		
		var valueCol = row.insertCell(-1);
		var valueSpan = document.createElement("span");
		valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
		valueSpan.appendChild(document.createTextNode(message || ""));
		if(color) valueSpan.style.color = color;
		valueSpan.className = "com-ibm-team-workitem-namevalue-value-message-span";
		valueCol.appendChild(valueSpan);
	},
	addMessageRow: function(severity, message, color) {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr";
		
		var valueCol = row.insertCell(-1);
		valueCol.COLSPAN = 2;
		valueCol.className = "com-ibm-team-workitem-namevalue-value-td";
		valueCol.appendChild(document.createTextNode(message || ""));
		if(color) valueCol.style.color = color;
	},
	addSeparatorRow: function() {
		var row = this._nameValueTable.insertRow(-1);
		row.className = "com-ibm-team-workitem-namevalue-tr-sep";
		var labelCol = row.insertCell(-1);
		labelCol.appendChild(this._getSeparatorElement());
		if(this.hasRequiredMarker) {
			var reqCol = row.insertCell(-1);
			reqCol.appendChild(this._getSeparatorElement());
		}
		var valueCol = row.insertCell(-1);
		valueCol.appendChild(this._getSeparatorElement());
	},
	setLabel: function(parms) {
		if(parms.index == null) parms.index = 0;
		if(!parms.hasColon && parms.labelText && parms.labelText.indexOf(":") == -1 && parms.labelText.indexOf(String.fromCharCode(65306)) == -1) parms.labelText = dojo.string.substitute(this.messages.labelValue, [parms.labelText]);
		
		var labelSpan = this.labelSpans[parms.index];
		dojox.xml.parser.removeChildren(labelSpan);
		labelSpan.appendChild(document.createTextNode(parms.labelText || "" ));
//		if(this._valueNode) this._valueNode.setAttribute("aria-label", parms.labelText);
	},
	setVisible: function(isVisible) {
		this._nameValueTable.style.visibility = isVisible ? "visible" : "hidden";
	},
	isVisible: function() {
		return this._nameValueTable.style.visibility != "hidden";
	},
	_getSeparatorElement: function() {
		var span = document.createElement("span");
		span.className = "com-ibm-team-workitem-namevalue-value-td-sep";
		return span;
	},
	_getRequiredMarkerElement: function(attributeId) {
		var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
			{workingCopy: this.workingCopy,
			 attributeId: attributeId}
		);
		this._childWidgets.push(reqMark);
		return reqMark.domNode;	
	}
});
	
})();