/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SectionLayout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");	
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var checkNotNull=	net.jazz.ajax.toolbox.lang.checkNotNull;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SectionLayout",
	[dijit._Widget, dijit._Templated], 
	{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/SectionLayout.html"),
  constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
			this.textDefaultTitle = this.messages.labelDefaultSectionTitle;
		},
	postCreate: function() {
		this._childWidgets = [];
		this._actionSpans = [];
		this._sectionTitleBarDIV.title = this.messages.titleCollapseSection;
		if(this.heading) {
			dojox.xml.parser.replaceChildren(this.sectionTitle, document.createTextNode(this.heading));
			this.sectionTitle.setAttribute("aria-label", this.heading);
		}
		if(this.column == "left") {
			dojo.addClass(this._sectionTitleBarDIV, "SectionLeft");				
		}
		if(this.column == "right") {
			dojo.addClass(this._sectionTitleBarDIV, "SectionRight");		
		} 
		this.setBorder(false);
		
		// Track section collapse state
		this._collapsed = false;
		if(this.initiallyCollapsed) {
			var arguments = {
				node: this._contentDIV, 
				duration: 0
			};
			var animation = dojo.fx.wipeOut(arguments);
			this.connect(animation, "onEnd", this._onCollapseAnimationEnd);
			animation.play();
		}
		this._addListenersSectionTitleBar();
		
	},
	destroyWidget: function() {
		for(var i=0;i< this._childWidgets.length;i++) {
			if(this._childWidgets[i] && this._childWidgets[i].destroy) {
				this._childWidgets[i].destroy();			
			}
			this._childWidgets[i] = null;
		}
		this._childWidgets = [];
		this.destroy();
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */		
	_addListenersSectionTitleBar: function() {
		// Add click listener
		this.connect(this._sectionTitleBarDIV, "onclick", this._handleEventSectionTitleBarClicked);
		this.connect(this.sectionTitle, "onkeypress", this._checkEnterForSectionTitleBarEvent);
		
		// Add animation listeners
		var arguments = {
			node: this._contentDIV, 
			duration: 200
		};
		
		// Expand animation listener
		this.contentExpandAnimation = dojo.fx.wipeIn(arguments);
		this.connect(this.contentExpandAnimation, "onEnd", this._onExpandAnimationEnd);
		
		// Collapse animation listener
		this.contentCollapseAnimation = dojo.fx.wipeOut(arguments);
		this.connect(this.contentCollapseAnimation, "onEnd", this._onCollapseAnimationEnd);
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_onExpandAnimationEnd: function() {
		this._collapsed = false;
		this._sectionTitleBarDIV.title = this.messages.titleCollapseSection;		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_onCollapseAnimationEnd: function() {
		dojo.addClass(this._contentDIV, "SectionContentCollapsed");
		dojo.addClass(this._sectionTitleBarDIV, "SectionTitleCollapsed");
		this._collapsed = true;
		this._sectionTitleBarDIV.title = this.messages.titleExpandSection;
	},
	
	_checkEnterForSectionTitleBarEvent: function(e) {
		var characterCode = e.keyCode;
		if(characterCode == 13 || characterCode == undefined) { //the key presed was 'enter'
			this._handleEventSectionTitleBarClicked(e);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * @return boolean
 	 */	
	_handleEventSectionTitleBarClicked: function(event) {
		// Prevent event bubbling
		if (event) {
			dojo.stopEvent(event);
		}

		if (this._collapsed) {
			this._doSectionTitleBarExpand();
		} else {
			this._doSectionTitleBarCollapse();
		}		

		return true;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_doSectionTitleBarExpand: function() {
		// Do expand
		dojo.removeClass(this._contentDIV, "SectionContentCollapsed");
		dojo.removeClass(this._sectionTitleBarDIV, "SectionTitleCollapsed");
		this._contentDIV.style.height = "1px";
		this.contentExpandAnimation.play();	
		this.sectionTitle.setAttribute("aria-expanded", "true");
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * @return void
 	 */	
	_doSectionTitleBarCollapse: function() {
		// Do collapse
		this.contentCollapseAnimation.play();
		this.sectionTitle.setAttribute("aria-expanded", "false");
	},
		
	/*
	 * Notify each child that an event occured
	 */
	_notifyChildren: function(selector, parm) {
		for(var i=0;i<this._childWidgets.length;i++) {
			if(this._childWidgets[i][selector]) {
				this._childWidgets[i][selector](parm);
			}
		}
	},
	/*
	 * Keep track of all the child widgets
	 */
	addChildWidget: function(widget) {
		this._childWidgets.push(widget);
	},
	/*
	 * Set the content
	 */
	setContent:function(domNode, className) {
		this._contentDIV.appendChild(domNode);
	},
	setMessage: function(message, isRight) {
		if(isRight) {
			dojox.xml.parser.replaceChildren(this._sectionRightMessage, document.createTextNode(message));	
		} else {
			dojox.xml.parser.replaceChildren(this._sectionMessage, document.createTextNode(message));
		}		
	},
	setRequiredAttributeId: function(attributeId, workingCopy) {
		var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
				{workingCopy: workingCopy,
			 		attributeId: attributeId}
				);
		this._childWidgets.push(reqMark);
		this._sectionRequiredMark.appendChild(reqMark.domNode);
	},
	clearMessage: function() {
		this._sectionMessage.innerHTML = ""; /* innerHTML OK */
	},
	setBackgroundClassName: function(backgroundClassName) {
		dojo.addClass(this.borderDIV, backgroundClassName);
	},
	setBorder: function(hasBorder) {
		if(hasBorder){
			dojo.removeClass(this.borderDIV, "SectionWithoutBorder");
			dojo.removeClass(this.innerBorderDIV, "SectionInnerBorderTop");
			dojo.addClass(this.borderDIV, "SectionWithBorder");
		} else {
			dojo.removeClass(this.borderDIV, "SectionWithBorder");
			dojo.addClass(this.borderDIV, "SectionWithoutBorder");
			dojo.addClass(this.innerBorderDIV, "SectionInnerBorderTop");
		}
	},
	addActionText: function(text, callbackObject, callbackSelector, title) {
		return this.addAction(text, null, null, title, callbackObject, callbackSelector); 
	},
	addActionIcon: function(iconUrl, iconDisabledUrl, iconTitle, callbackObject, callbackSelector) {
		return this.addAction(null, iconUrl, iconDisabledUrl, iconTitle, callbackObject, callbackSelector); 
	},
	setActionIcon: function(span, iconUrl, iconTitle) {
		if(!span) {
			console.debug("*** Error: Missing span in SectionLayout setActionIcon");
			return;
		}
		if(span._jazzImg) {
 			span._jazzImg.src=iconUrl;	
			span._jazzImg.title = iconTitle;
		}
	},
	addPopupHelp: function(message, title) {
		var span = document.createElement("SPAN");	
		var img = document.createElement("IMG");
		img.src = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool.PIXEL;
		img.title = title;
		span.className="SectionPopupHelpSpan";
		img.className="SectionPopupHelpImg";
		this.connect(img, "onclick", function(e) { dojo.stopEvent(e);}); // ignore the click
		span.appendChild(img);
		this._actionSpans.push(span);
		this._sectionActionArea.appendChild(span);
		this._childWidgets.push(new jazz.ui.HoverPopup({ around: [ span ], html: message, maxHeight: 480}));		
		return span;
	},
	addAction: function(text, iconUrl, iconDisabledUrl, iconTitle, callbackObject, callbackSelector, isEnabled) {

		var span = this.getAction(text, iconUrl, iconDisabledUrl, iconTitle, callbackObject, callbackSelector, isEnabled);
		this._actionSpans.push(span)		;
		
		if(this._actionSpans.length > 1 ) {
			dojo.addClass(this._actionSpans[this._actionSpans.length-2], "SectionActionBorderRight");			
		}

		this._sectionActionArea.appendChild(span);
		return span;
	},
	getAction: function(text, iconUrl, iconDisabledUrl, iconTitle, callbackObject, callbackSelector, isEnabled) {
		isEnabled = isEnabled == null ? true : isEnabled; 
		var span = document.createElement("SPAN");
		span.className = "SectionTitleActionImg";
		var anchor = document.createElement("A");
		anchor.href="#";
		anchor.title = iconTitle || text ||  this.messages.buttonDefaultRunAction;
		span.appendChild(anchor);

		if(iconUrl) {
			var img = document.createElement("IMG");
			img.src = iconUrl;
			img.title = iconTitle;
			img.className="SectionActionImg";
			img._jazzTarget = anchor;
			anchor.appendChild(img);
		}
		if(text) {
			var textNode = document.createTextNode(text);
			anchor.appendChild(textNode);
		}
		anchor.callbackObject = callbackObject;
		anchor.callbackSelector = callbackSelector;
		anchor._jazzIsEnabled = true;
		anchor._jazzTarget = anchor;
		span._jazzTarget = anchor;
		span._jazzAnchor = anchor;
		span._jazzImg = img;
		span._jazzImgUrl_enabled = iconUrl;
		span._jazzImgUrl_disabled = iconDisabledUrl;
		this.setActionEnabled(span, isEnabled);
		
		this.connect(span, "onclick", "_handleActionClick");

		return span;
	},
	setActionEnabled: function(span, isEnabled) {
		span._jazzAnchor._jazzIsEnabled = isEnabled;
		if(isEnabled) {
			dojo.removeClass(span, "SectionActionDisabled");
			dojo.addClass(span, "SectionActionEnabled");
			dojo.removeClass(span._jazzAnchor, "SectionActionAnchorDisabled");
			dojo.addClass(span._jazzAnchor, "SectionActionAnchorEnabled");

		} else {
			dojo.removeClass(span, "SectionActionEnabled");
			dojo.addClass(span, "SectionActionDisabled");
			dojo.removeClass(span._jazzAnchor, "SectionActionAnchorEnabled");
			dojo.addClass(span._jazzAnchor, "SectionActionAnchorDisabled");

		}
		if(span._jazzImg) {
			if(isEnabled) {
				span._jazzImg.src = span._jazzImgUrl_enabled;
			} else {
				span._jazzImg.src = span._jazzImgUrl_disabled;			
			}			
		}

	},
	setActionVisible: function(span, isVisible) {
		if(isVisible) {
			span.style.display = "inline";
			dojo.removeClass(span, "SectionActionVisible");
			dojo.addClass(span, "SectionActionHidden");
		} else {
			span.style.display = "none";
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event e
 	 * @return void
 	 */	
	_handleActionClick: function(e) {
		// Prevent event bubbling (up to section title bar)
		dojo.stopEvent(e);

		// Automatically expand collapsed sections if the action is clicked
		if (dojo.isIE && this._collapsed) {
			// IE: Must expand the title bar first before swapping content
			// nodes
			this._doSectionTitleBarExpand();
		}

		this._doAction(e);

		// Automatically expand collapsed sections if the action is clicked
		if (!dojo.isIE && this._collapsed) {
			this._doSectionTitleBarExpand();
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event e
 	 * @return void
 	 */	
	_doAction: function(e) {
		var target = e.target;
		if(!target.callbackObject) {
			target = target._jazzTarget;
		}
		if(!target._jazzIsEnabled) return;
		target.callbackObject[target.callbackSelector]();
	}

});
	
})();