/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.MenuSeparator");
	
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var ITEM_SLOT= "__jazzItem";

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="SectionMenuDropDownActionWidget">' +
						'<span dojoAttachPoint="_labelNode" class="SectionMenuLabel"></span>'+
						'<a href="#" dojoAttachPoint="_actionContainerNode" dojoAttachEvent="onclick: _handleDropDownClick" class="SectionMenuActionContainer">' +
						'<span dojoAttachEvent="onclick: _handleActionClick" dojoAttachPoint="_actionNodeImg" class="SectionMenuAction SectionMenuHeight"><span dojoAttachPoint="_actionNode"></span></span>' +
						'<span class="dijitInline SectionMenuDownArrowImageContainer SectionMenuHeight"><img dojoAttachPoint="_downArrowIMG" dojoAttachEvent="" class="SectionMenuDownArrowImage" /></span>' +
						'</a>' +
						'</div>',
 		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
			this.workItemMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},
		postCreate: function() {
			if(!this.label) {
				this.label = "";
				this._labelNode.style.display="none"; 
			} 
			if(!this.actions) this.actions = [];
			
			if (this.isBold) {
				this._actionContainerNode.style.fontWeight = "bold";
			}
			this._actionContainerNode.style.color = "#075F87";
			
			this._labelNode.appendChild(document.createTextNode(dojo.string.substitute(this.workItemMessages.labelValue , [ this.label] )));
			this._actionContainerNode.setAttribute("aria-label", dojo.string.substitute(this.workItemMessages.labelValue , [ this.label] ));
			dijit.setWaiRole(this._actionContainerNode, "menu");
			
			if(!this.actionLabelProperty) {
				this.actionLabelProperty = "label";
			}
			if(!this.actionIconSrcProperty) {
				this.actionIconSrcProperty = "iconSrc";
			}
			
			if(this.actions.length == 0) {
				var action = {
					isEnabled: false
				};
				action[this.actionLabelProperty] = this.messages.labelDefaultNullAction;
				this.actions.push(action);
			}

			this.setCurrentAction(this.actions[0]);
			this._downArrowIMG.src = ImagePool.POPUP_DOWN_ARROW;	
			
			if(this.defaultAction){
				this.setCurrentAction(this.defaultAction);
			}
			
		},
		destroy: function() {
			if(this._menuWidget) {
				if(this._menuWidget._onCloseActual) {
					this._menuWidget.onClose = this._menuWidget._onCloseActual;
					delete this._menuWidget._onCloseActual;
				}
				this._menuWidget.destroy();
			}
			this.inherited(arguments);
		},
		setCurrentAction: function(action) {
			this._currentAction = action;
			// TODO: Set action image
			dojox.xml.parser.replaceChildren(this._actionNode, document.createTextNode(action.label));
			if(action.iconSrc){
				this._actionNode.style.paddingLeft = "18px";
				var url = com.ibm.team.workitem.web.client.util.getBackgroundImageUrl(action[this.actionIconSrcProperty]);
 				this._actionNodeImg.style.backgroundImage = "url('" + url + "')";
				this._actionNodeImg.style.backgroundPosition = "0 center";
			}
			if(action.isEnabled){
				dojo.removeClass(this._actionNode, "SectionMenuAction_Disabled");
				dojo.addClass(this._actionNode, "SectionMenuAction_Enabled");
			} else {
				dojo.addClass(this._actionNode, "SectionMenuAction_Disabled");
				dojo.removeClass(this._actionNode, "SectionMenuAction_Enabled");
			}
			this._actionNode.title = dojo.string.substitute(this.workItemMessages.actionAddNewThing, [ action.label ] );
			this._actionContainerNode.setAttribute("aria-label", this._actionNode.title);
		},
		performAction: function(action) {
			if(action.receiverObject && action.receiverSelector && action.receiverObject[action.receiverSelector]) {
				action.receiverObject[action.receiverSelector](action);
			}
		},

		// Handle an action selected in the emenu
		_handleActionClick: function(e){
			dojo.stopEvent(e);
			var action = this._currentAction
			
			dojo.addClass(this._actionNode, "SectionMenuAction_clicked");
			
			setTimeout(dojo.hitch(this, function(){
							dojo.removeClass(this._actionNode, "SectionMenuAction_clicked");
							}), 10);
			this.performAction(action);						
		},
		_handleDropDownClick: function(e){
			dojo.stopEvent(e);
			if (!this._menuWidget) this._createMenuWidget();
			dojo.addClass(this._actionContainerNode,"SectionMenuActionContainer_open");
			dijit.popup.open({ popup: this._menuWidget, onCancel: dojo.hitch(this, this._handleMenuClose), parent: this, around: this._actionContainerNode, onExecute: dojo.hitch(this,this._handleMenuClose)});
			this._menuCloseOnBody = this.connect(this,"_onBlur",this._handleMenuClose);
			this._menuCloseOnWidget = this.connect(this.domNode,"onmouseup",this._handleMenuClose);
			this._menuWidget.focus();
		},
		_createMenuWidget: function() {
			this._hasIcon = false;
			this._menuWidget = new jazz.ui.Menu({});
			this.connect(this._menuWidget, "onItemClick", "_handleActionSelect");
			this._setMenuItems(this.actions);
			if(this._hasIcon){
				dojo.addClass(this._menuWidget.containerNode, "hasIcon");
			}
		},
		_setMenuItems: function(items) {
			if (items.length > 0) {
				for (var i= 0; i < items.length; i++) {
					if(items[i] == WidgetUtil.SEPARATOR){
						this._menuWidget.addChild(new dijit.MenuSeparator())
					}else{
						this._menuWidget.addChild(this._createMenuItem(items[i]));
					}
				}
			}
		},
		_createMenuItem: function(item) {
			var label = item[this.actionLabelProperty];

 			if(item[this.actionIconSrcProperty]) {
				try{
					var menuItem= new dijit.MenuItem( { label: label });
					menuItem.iconNode.src = item[this.actionIconSrcProperty];
					this._hasIcon = true;
					var div = menuItem.domNode/*TR*/.firstChild/*TD*/.firstChild/*DIV*/
					if(!dojo.isIE) {
						div.style.background = "url(" + item[this.actionIconSrcProperty] + ")";
					}
				} catch(e) {} // well, we tried
			} else {
				var menuItem= new dijit.MenuItem( { label: label, iconClass: "pixelIcon" }); // TODO: Support images
			}

			menuItem[ITEM_SLOT]= item;
			return menuItem;
		},
		_handleMenuClose: function() {
			dijit.popup.close(this._menuWidget);
			dojo.removeClass(this._actionContainerNode,"SectionMenuActionContainer_open");
			this.disconnect(this._menuCloseOnBody);
			this.disconnect(this._menuCloseOnWidget);
		},
		_handleActionSelect: function(/* MenuItem */ menuItem) {
			var action = menuItem[ITEM_SLOT];
			this.setCurrentAction(action);
			this.performAction(action);
			
		}
	}
);

})();
