/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SimpleInputWidget");
/*
 * This part is not used but will be used for short string attributes.
 */

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.xml.parser");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var StatusIndicator= com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SimpleInputWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="SimpleInputWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" >' +
						'<span dojoAttachPoint="_statusIconContainer" class="StatusIconContainer">' +
						'<div dojoAttachPoint="_statusIcon"></div></span>' +
						'<span dojoAttachPoint="_inputSet" style="display: inline">'+ 
						   '<input dojoAttachPoint="_input" class="SimpleInputWidgetInput OffsetInput WidgetInput TextInput"></input>'+
						   '<span class="UnitTextValue" dojoAttachPoint="_unitSpan1"></span>'+
						'</span>' +
						'<span dojoAttachPoint="_labelSet" style="display: none">' +
						   '<span dojoAttachPoint="_readOnlyValue" class="ReadOnlyStringValue"></span>&nbsp;'+
						    '<span class="ReadOnlyStringValue" dojoAttachPoint="_unitSpan2"></span>' +
						'</span>' +
						'</div>',
     	constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		postCreate: function() {
			
			this.setUnits(this.units);

			this._inputSet.style.display = "inline";
			this._labelSet.style.display = "none";
			
			WidgetUtil.setSpacerImage(this._image);

			var value = this.workingCopy.getValue({path: ["attributes" , this.attributeId, "content"], attributeId: this.attributeId});
			if(!value) {
				value = this.workingCopy.getValue({path: ["attributes" , this.attributeId, "id"], attributeId: this.attributeId});
			}
			this._setValue(value);			
			if(this._modifiers && this._modifiers.onCreate) {
					this.parentWidget[this._modifiers.onCreate](this, value);
			}
			
			if(this._modifiers && this._modifiers.onValueChanged) {
					this.parentWidget[this._modifiers.onValueChanged](this , value, /* isInitialization */ true);
				}
				
			if(this._modifiers && this._modifiers.readonly == "true") {
				this.setReadOnly(true);
			}	
				
			this.isChanged = false;
			if(this.inputWidth) {
				this._input.style.width = this.inputWidth;
			}
			if (this.workingCopy.isGuest()) {
				dojo.addClass(this._input, "GuestField");
			}
			this.workingCopy.addListener({
				path:  ["attributes",  this.attributeId, "content"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			this.workingCopy.addListener({
				path:  ["attributes",  this.attributeId, "id"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			
			this._updateListener= WidgetUtil.runAfterTyping(this._input, WidgetUtil.DEFAULT_DELAY, dojo.hitch(this, this._handleTextChange, {target: this._input}));
			
			this.statusIndicator= new StatusIndicator({}, this._statusIcon);
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
			// this.statusIndicator.test();
		},
		destroy: function() {
			this.workingCopy.removeListener({
				path:  ["attributes",  this.attributeId, "content"],
				listener: this
			});
			this.workingCopy.removeListener({
				path:  ["attributes",  this.attributeId, "id"],
				listener: this
			});
			if(this._updateListener) dojo.disconnect(this._updateListener);
			if(this.statusIndicator) this.statusIndicator.destroy();
			this.statusIndicator = null;
            this.inherited(arguments);
		},
		focus: function(){
			dijit.focus(this._input);
		},
		_handleAttributeChange: function(parms) {
			this._setValue(parms.attributeValue);
			if(parms.validationStatus)
				this.statusIndicator.setStatus(parms.validationStatus);
		},
		setValue: function(value){
			// needed because this gets called from WorkItemEditorHeader
			this._setValue(value);
		},
		_setValue: function(value) {
			var valueText;
			if(value != null) {
				if (value.content != null) {
					valueText = value.content;
				} else if (value.label != null) {
					valueText = value.label;
					} else {
						valueText = value;
					}
			} else {
				valueText = "";
			}
			if (this._getControlValue() != valueText) {
				this._setControlValue(valueText);
				this._setReadOnlyValue(valueText);
			}

		},
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this._inputSet.style.display = "none";
				this._labelSet.style.display = "inline";
			}
			else {
				if(this._modifiers && this._modifiers.isReadOnly) return;
				this._inputSet.style.display = "inline";
				this._labelSet.style.display = "none";
			}
		},
		setUnits: function(units){
			this.units = units;
			dojox.xml.parser.removeChildren(this._unitSpan1);
			dojox.xml.parser.removeChildren(this._unitSpan2);
			if(units) {
				var text1 = document.createTextNode(this.units);
				this._unitSpan1.appendChild(text1);
				var text2 = document.createTextNode(this.units);
				this._unitSpan2.appendChild(text2);
			}
		},
		_setReadOnlyValue: function(value){
			if(value==null) {
				value = this.messages.labelDefaultNullValue;
				dojox.xml.parser.removeChildren(this._unitSpan1);
				dojox.xml.parser.removeChildren(this._unitSpan2);
			} else {
				if (value.length == 0) {
//					value = this.messages.labelDefaultEmptyValue;
//					dojox.xml.parser.removeChildren(this._unitSpan1);
					dojox.xml.parser.removeChildren(this._unitSpan2);
				} else {
					this.setUnits(this.units);
				}		
			}

			dojox.xml.parser.removeChildren(this._readOnlyValue);
			if (this.attributeValue && this.attributeValue.isHTML) {
				this._readOnlyValue.innerHTML= value;
			} else {
				this._readOnlyValue.appendChild(document.createTextNode(value));
			}
		},
		_handleTextChange: function(e) {
			var controlValue = this._getControlValue();
			var oldValue = this.workingCopy.getValue({path:  ["attributes",  this.attributeId, "content"]});
			if(oldValue == controlValue) return;
			
			this.isChanged=true;
			if(this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}
			this._setReadOnlyValue(controlValue);
			
			if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
				this.parentWidget[this._modifiers.onValueChanged](this , controlValue, /* isInitialization */ false);
			}
			
			var parms = {
					caller: this,
					path:  ["attributes",  this.attributeId, "content"],
					savedAttributePath:  ["attributes",  this.attributeId, "content"],
					attributeId: this.attributeId,
					value: controlValue
				};
			this.workingCopy.setValue(parms);
			
			// TODO: Should also be handled by listening to the working copy changes
			var status= this.workingCopy._validate(this.attributeId);
			this.statusIndicator.setStatus(status);
		},
		getFormElementValues: function(formElements) {
			if (this.isChanged) {
				formElements[this.attributeId] = this._getControlValue();
			}
		},
		setFormElementValues: function(retainedState){
		 	if(retainedState[this.attributeId]) {
				if(this._getControlValue() != retainedState[this.attributeId]) {
					this._setControlValue(retainedState[this.attributeId]);
					this._setReadOnlyValue(this._getControlValue());
					this.isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , this._getControlValue());
					}
				}
			}
		 },
	     getValue: function() {
	     	if (this.isChanged) {
	     		return this._getControlValue();
	     	} else {
				return this.workingCopy.getValue({path:  ["attributes",  this.attributeId, "content"]});
			}
		 },
	     _getControlValue: function() {
			var value= this._input.value;
			// due to work item 155019
			if(this.attributeId == "summary") {
				value= jazz.util.html.escape(value);
			}
			return value;
		 },
		 _setControlValue: function(value) {
			// due to work item 155019
			if(this.attributeId == "summary") {
				value= EditorUtil.getPlainText(value);
			}
			this._input.value= value;
		 }
	}
);

})();
