/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SimpleRichTextWidget");

dojo.require("dijit._Widget");
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy");

(function() {

var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var StatusIndicator= com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator;
var editorProxyClass = com.ibm.team.workitem.web.ui.internal.view.richtext.getEditorProxyClass();
var WORKITEM_ACTION_SHOW_RICH_TEXT = com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_ACTION_SHOW_RICH_TEXT;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SimpleRichTextWidget",
	dijit._Widget,
{
	/*
	 * Rich text version of com.ibm.team.workitem.web.ui.internal.view.layout.SimpleInputWidget
	 * It shows toolbar for rich text editing when the field receives focus, and hides toolbar when focus is lost
	 */

	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-team-workitem-SimpleRichTextWidget");
		dojo.addClass(this.domNode, "com-ibm-team-workitem-richtext");
		this._childWidgets = [];
		var span = dojo.create("span", { className:"StatusIconContainer" }, this.domNode);
		this.readonlyContainer = dojo.create("div", null, this.domNode);
		this.readonlyNode = dojo.create("div", {
			tabIndex: 0,
			className: "readonly-text"
		}, this.readonlyContainer);
		this.connect(this.readonlyNode, "onfocus", dojo.hitch(this, function(){
			this._showEditor(true);
		}));

		this._editorProxy = new editorProxyClass({
			editorArgs: {
				parentController: this.parentController,
				IFRAME_BODY_CLASS: "SimpleRichTextBody",
				onKeyDown: function(e){
					if(e.keyCode === dojo.keys.ENTER){
						dojo.stopEvent(e);
						return;
					}
					this.inherited("onKeyDown", arguments);
				},
				height: "32px" // editor needs at least 32px in order to show horizontal scrollbar
			},
			parentNode: this.readonlyContainer
		});
		this.connect(this._editorProxy, "onEditorCreated", "setupEditor");
		
		this.inherited(arguments);

		var value = this.workingCopy.getValue({path: ["attributes" , this.attributeId], attributeId: this.attributeId});
		if(!value) {
			value = this.workingCopy.getValue({path: ["attributes" , this.attributeId, "id"], attributeId: this.attributeId});
		}
		this._setValue(value, true);
		var content = value.content || "";
		var _modifiers = this._modifiers;
		if(_modifiers && _modifiers.onCreate){
			this.parentWidget[_modifiers.onCreate](this, content);
		}
		if(_modifiers && _modifiers.onValueChanged) {
			this.parentWidget[_modifiers.onValueChanged](this, content, true);
		}
		if(_modifiers && _modifiers.readonly == "true"){
			this.setReadOnly(true);
		}
		
		this.workingCopy.addListener({
			path:  ["attributes",  this.attributeId, "content"],
			event: "onchange",
			listener: this,
			functionName: "_handleAttributeChange"
		});
		this.workingCopy.addListener({
			path:  ["attributes",  this.attributeId, "id"],
			event: "onchange",
			listener: this,
			functionName: "_handleAttributeChange"
		});

		this.statusIndicator= new StatusIndicator({}, dojo.create("div", null, span));
		var status= this.workingCopy._validate(this.attributeId);
		this.statusIndicator.setStatus(status);
	},

	setupEditor: function(editor){
		editor.connect(editor, "onNormalizedDisplayChanged", dojo.hitch(this, "_handleTextChange"));

		// hide this editor if other rich text editor is opened
		editor.subscribe(WORKITEM_ACTION_SHOW_RICH_TEXT, dojo.hitch(this, function(ed){
			if(editor !== ed){
				this._showEditor(false);
			}
		}));
	},
	
	getValue: function(){
		var value = this._editorProxy.getValue();
		value = this._stripBR(value);
		return value;
	},
	
	// strip out BR tag
	_stripBR: function(value){
		if(value){
			value = value.replace(/<br[^>]*>/gi, " "); // strip out BR tag
			value = dojo.trim(value);
		}
		return value;
	},
	
	setValue: function(value){
		this._editorProxy.setValue(value);
	},
	
	_handleAttributeChange: function(parms){
		this._setValue(parms.attributeValue);
		if(parms.validationStatus){
			this.statusIndicator.setStatus(parms.validationStatus);
		}
	},
	
	_setValue: function(value, bStore){
		var valueText;
		var valueContent;
		if(value !== null){
			if(value.content !== null){
				valueContent = this._stripBR(value.content);
				if(bStore){
					this._origContent = valueContent;
				}
				if(value.links && value.links.length>0){
					valueText = EditorUtil.resolveLinks(value.content, value.links, this.workingCopy); // handle "bug 1234" in content
				}else{
					valueText = valueContent;
				}
			}else if(value.label !== null){
				valueText = value.label;
			}else{
				valueText = value;
			}
		}else{
			valueText = "";
		}
//		if(this.attributeId == "summary"){
//			valueText = EditorUtil.restoreTags(valueText);
//		}
		var val = this.getValue();
		this.setValue(valueContent);
		if(bStore || val != this._origContent){
			this._setReadOnlyValue(valueText);
		}
	},
	
	setReadOnly: function(isReadOnly){
		dojo.toggleClass(this.readonlyContainer, "readonly", isReadOnly);
	},
		
	_setReadOnlyValue: function(value){
		value = (value && value.length > 0) ? value : "&nbsp;";
		
		this.readonlyNode.innerHTML = jazz.util.html.removeScript(value);
		
		this.clearChildWidgets();
		this._childWidgets = this._childWidgets.concat(EditorUtil.linkAnchorPopUps(this.readonlyNode, this.parentWidget._workItem));
	},

	_showEditor: function(bShow){
		// do nothing if readonly
		if(dojo.hasClass(this.readonlyContainer, "readonly")){ return; }
		
		if(bShow){
			this.readonlyNode.style.display = "none";
			this._editorProxy.showEditor();
			this.focus();
		}else{
			this.readonlyNode.style.display = "";
			this._editorProxy.hideEditor();
		}
	},
	
	_handleTextChange: function(){
		var val = this.getValue();

		if(val != this._origContent){
			this._setReadOnlyValue(val);
		}
		if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
			this.parentWidget[this._modifiers.onValueChanged](this , val, /* isInitialization */ false);
		}
		var oldValue = this.workingCopy.getValue({path:  ["attributes",  this.attributeId, "content"]});
		oldValue = this._stripBR(oldValue);
		if(oldValue == val){ return; }

		var parms = {
			caller: this,
			path: ["attributes",  this.attributeId, "content"],
			savedAttributePath:  ["attributes",  this.attributeId, "content"],
			attributeId: this.attributeId,
			value: val
		};
		this.workingCopy.setValue(parms);
			
		// TODO: Should also be handled by listening to the working copy changes
		var status= this.workingCopy._validate(this.attributeId);
		this.statusIndicator.setStatus(status);
	},
	
	focus: function(){
		this._editorProxy.focus();
	},

	handleSaveComplete: function(){
		var value = this.workingCopy.getValue({path: ["attributes" , this.attributeId], attributeId: this.attributeId});
		if(!value) {
			value = this.workingCopy.getValue({path: ["attributes" , this.attributeId, "id"], attributeId: this.attributeId});
		}
		this._setValue(value, true);
		
		this._showEditor(false);
	},
	
	clearChildWidgets: function(){
		for(var i=0;i< this._childWidgets.length;i++) {
			if(this._childWidgets[i] && this._childWidgets[i].destroy) {
				this._childWidgets[i].destroy();			
			}
			this._childWidgets[i] = null;
		}
		this._childWidgets = [];
	},
	
	destroy: function(){
		this.workingCopy.removeListener({
			path:  ["attributes",  this.attributeId, "content"],
			listener: this
		});
		if(this.statusIndicator){ this.statusIndicator.destroy(); }
		this.statusIndicator = null;
		if(this._editorProxy){ this._editorProxy.destroy(); }
		this._editorProxy = null;
		this.clearChildWidgets();
		this.inherited(arguments);
	}
});

})();