/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TeamAreaWidget");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojox.xml.parser");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
(function() {

var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var WORD_SIZE=18;
var WORD_SEPARATOR="<wbr/>";

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.TeamAreaWidget",
	[dijit._Widget, dijit._Templated],
	{
		templateString: '<div class="TeamAreaWidget NarrowWidget">' +
						'<img dojoAttachPoint="_spacer" class="PreFieldImage" />' +
						'<span dojoAttachPoint="_statusIconContainer" class="StatusIconReadOnlyContainer">' +
						'<div dojoAttachPoint="_statusIcon"></div></span>' +
						'<span dojoAttachPoint="_labelSPAN" class="LayoutValue" tabindex="0"></span>' +
						'</div>',
 		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		},
		/*
		 * TeamAreaWidget displays:
		 *   - if isProjectAreaWidget: just the project area
		 *   - if isTeamAreaWidget: just the team area
		 *   - if both: Both
		 *   - if neither: Both
		 */
		postCreate: function() {
			this.statusIndicator= new com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator({}, this._statusIcon);

			WidgetUtil.setSpacerImage(this._spacer);
			/*
			 * Set a space for the separator for browsers without WBR support
			 */
			if((dojo.isIE && dojo.isIE<7) || !(dojo.isWebKit || dojo.isMozilla)) {
				WORD_SEPARATOR=" ";
			}


			this.workingCopy.addListener({
				path: "attributes.teamArea",
				event: "onchange",
				listener: this,
				functionName: "_handleTeamAreaChange"
			});
			this.workingCopy.addListener({
				path: "attributes.projectArea",
				event: "onchange",
				listener: this,
				functionName: "_handleProjectAreaChange"
			});
			// Handle the case where neither is specified
			if(!this._modifiers.isProjectAreaWidget && !this._modifiers.isTeamAreaWidget) {
				this._modifiers.isProjectAreaWidget = true;
				this._modifiers.isTeamAreaWidget = true;
			}
			this._refresh();
		},
		destroy: function(preserveDom) {
			if(this.statusIndicator) this.statusIndicator.destroy(preserveDom);
			this.statusIndicator = null;
            this.inherited(arguments);
		},
		getMinimumWidth: function() {
			if(this._modifiers.isProjectAreaWidget && this._modifiers.isTeamAreaWidget) {
				return "16em"
			}
			return null;
		},
		_handleTeamAreaChange: function() {
			this._refresh();
		},
		_handleProjectAreaChange: function() {
			this._refresh();
		},
		_refresh: function() {
			var value;

			if(this._modifiers.isTeamAreaWidget && !this._modifiers.isProjectAreaWidget) {
				value = this._getTeamAreaLabel();
				var status= this.workingCopy._validate("teamArea");
				this.statusIndicator.setStatus(status);
			} else {
				value = this.workingCopy.getValue({path: "attributes.projectArea.label"});
				if(this._modifiers.isTeamAreaWidget) {
					value = dojo.string.substitute(this.messages.labelTeamAreaAndProjectArea, [this._getTeamAreaLabel() , value ]);
					var status= this.workingCopy._validate("projectArea");
					this.statusIndicator.setStatus(status);
				}
			}
			this.setValue(value);
			// this.statusIndicator.test();
		},
		_getTeamAreaLabel: function() {
			return this.workingCopy.getValue({path: "attributes.teamArea.label"});
		},
		shorten: function(text) {
			if(!text) return "";
			text = WorkItemUtil.plainToHTML(text);			
			if(text.length < WORD_SIZE) return text;
			var result = "", i=0;
			while(i<text.length) { 
				result = result + text.substring(i,Math.min(i+WORD_SIZE, text.length)) + WORD_SEPARATOR;
				i += WORD_SIZE;
			}
			if(i<text.length) {
				result = result + text.substring(i,text.length-1);
			}
			result = dojo.trim(result);
			return result;
		},
		setValue: function(value) {
			if(!value || value.length == 0) {
				value = this.messages.labelDefaultNullItem;
				dojo.addClass(this._labelSPAN, "GreyLabel")
			} else {
				dojo.removeClass(this._labelSPAN, "GreyLabel")
			}
			
			var split = value.split(" ");
			value = "";
			for (var i=0;i<split.length;i++) {
				var next = this.shorten(split[i]);
				value = value +  next + " ";
			}
			value = dojo.trim(value);
			this._labelSPAN.innerHTML = value;
		}
	}
);

})();
