/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.workitem.api.common.Status"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function(){
	
var STYLE_VALID= "validation-ok"
var STYLE_INVALID= "validation-failed"

var HoverPopup= jazz.ui.HoverPopup;
var WidgetUtil= com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.layout.TextWidget", [dijit._Widget, dijit._Templated], {

	templateString: '<div class="com-ibm-team-workitem-text-widget">' +
						'<span dojoAttachPoint="_statusIconContainer" class="StatusIconTextWidgetContainer">' +
						'<div dojoAttachPoint="_statusIcon"></div></span>' +
					'<div class="fieldWrapper" dojoAttachpoint="_textDiv"></div>' +
					'</div>',
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
		this.presentation= args.presentation;
		this.selection= args.selection;
		
		var converters= new Object();
		converters["integer"]= function(value) {
			var parsed= parseInt(test, 10);
			return (!isNaN(parsed) && parsed  == value);
		};
		
		converters["long"]= converters["integer"];
		
		converters["float"]= function(value) {
			var parsed= parseFloat(test, 10);
			return (!isNaN(parsed) && parsed  == value);
		};
		
		if(args.converter) {
			this._hasCustomConverter = true;
			this._converter = args.converter;
			this._converterParms = args.converterParms;
		} else {
			this._converter= converters[this.presentation.attribute.attributeTypeId];
		}
	},
	
	postCreate: function() {
		if(this.statusIconContainerClass) {
			this.statusIndicator= new com.ibm.team.workitem.web.ui2.internal.widget.StatusIndicator({}, this._statusIcon);
			this._statusIconContainer.className = this.statusIconContainerClass;
		// this.statusIndicator.test();
		}
//		<input class="text-control" type="text" dojoattachpoint="_textInput" dojoattachevent="onchange: valueChanged, onkeyup:doValidate"></input>
		this._textInput = document.createElement("INPUT");
		this._textInput.type = this.isPassword === true ? "PASSWORD" : "TEXT";
		if (this.presentation.displayattributes && this.presentation.displayattributes.disabled) {
			this._textInput.disabled = this.presentation.displayattributes.disabled;
		}
		dojo.addClass(this._textInput, STYLE_VALID);
		dojo.addClass(this._textInput, "text-control");
		
		this._textDiv.appendChild(this._textInput);
		
		if (this._converter) {
			
			this._toolTipMessage= this.messages.titleEnterNumber;
			if(this._hasCustomConverter)
				this._toolTipMessage = this._converterParms.toolTip;
				
//			this._tooltip = new HoverPopup({
//				around: [this._textInput],
//				html: tooltipMessage,
//				orientation: "L",
//				programmatic: true
//			});
		}
		
		if (this.selection)
			this._setValues(this.selection);
			
		this._updateListener= WidgetUtil.runAfterTyping(this._textInput, WidgetUtil.DEFAULT_DELAY, dojo.hitch(this, this.doValidate, {target: this._textInput}));
	},
	destroy: function() {
//		if(this._tooltip) {
//			this._tooltip.destroy();
//		}
		if(this.statusIndicator) this.statusIndicator.destroy();
		this.statusIndicator = null;
		if(this._updateListener) dojo.disconnect(this._updateListener);
		this.inherited(arguments);
	},
	/* String[] */ getValues: function() {
		var selection= new Array();
		var rawValues= this._getRawValues();
		for (var i= 0; i < rawValues.length; i++) {
			var value= this._convert(rawValues[i]);
			if (value)
				selection.push(value);
		}
		if(rawValues.length == 0){
			this._convert("");
		}
		return selection;
	},
	setStatus: function(status) {
		if(this.statusIndicator) {
			this.statusIndicator.setStatus(status);
			this._lastStatus= status; // used after validation
		}
	},
	valueChanged: function(e) {
		if(this.parentWidget && this.onValueChanged) {
			this.parentWidget[this.onValueChanged](e);
		}
	},
	doValidate: function(e) {
		if(!this.statusIndicator || !this._toolTipMessage) return;
		this.valueChanged(e);
		var inputNode= this._textInput;
		if (this._validateInput()) {
			this.statusIndicator.setStatus(this._lastStatus || com.ibm.team.workitem.api.common.Status.OK_STATUS);
//			dojo.disconnect(this._onBlur_handle);
//			dojo.removeClass(inputNode, STYLE_INVALID);
//			dojo.addClass(inputNode, STYLE_VALID);
//			if (this._tooltip)
//				this._tooltip.close();
		} else {
			this.statusIndicator.setStatus(new com.ibm.team.workitem.api.common.Status(com.ibm.team.workitem.api.common.Severity.ERROR, this._toolTipMessage));
//			dojo.removeClass(inputNode, STYLE_VALID);
//			dojo.addClass(inputNode, STYLE_INVALID);
//			if (this._tooltip) {
				if(this._hasCustomConverter) {
					var _self = this;
					
					this.connect(inputNode, "onblur", function(e) {
							_self._validateOnBlur(e);
						})				
				}
//				if (!this._tooltip._isOpen)
//					this._tooltip.open();
			}
//		}
		if(this.valueChangedListener && this.valueChangedFunction && dojo.isFunction(this.valueChangedListener[this.valueChangedFunction])) {
			this.valueChangedListener[this.valueChangedFunction]( this._textInput.value);
		}
	},
	
	_validateOnBlur: function(e) {
		dojo.disconnect(this._onBlur_handle);
//		dojo.removeClass(this._textInput, STYLE_INVALID);
//		dojo.addClass(this._textInput, STYLE_VALID);
//		this._tooltip.close(); 
		this._textInput.value = this._converter(this._textInput.value,  this._converterParms,  /* get valid */ true);

	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_convert: function(value) {
		if (value.length > 0 && this._converter) {
			return this._converter(value, this._converterParms);
		} else {
			if(this._hasCustomConverter) {
				return this._converter("", this._converterParms);
			}
		}
		return value;
	},
	
	_validateInput: function() {
		if (this._converter) {
			var values= this._getRawValues();
			for (var i= 0; i < values.length; i++) {
				var converted= this._convert(values[i])
				if(converted && converted.isCustom) {
					if(!converted.isValid) {
						return false;
					}
				} else {
					if (isNaN(converted) || (values[i] != converted))
						return false;			
				}
			}
		}
		return true;	
	},
	
	/* String[] */ _getRawValues: function() {
		var selection= new Array();
		var stringInput= this._textInput.value;
		var parts= stringInput.split(",");
		for (var i= 0; i < parts.length; i++) {
			var trimmed= dojo.trim(parts[i]);
			selection.push(trimmed);
		}
		return selection;
	},
	
	_setValues: function(/* UIItemDTO[] */ selection) {
		var list= new Array();
		for (var i= 0; i < selection.length; i++) {
			list.push(selection[i].id);
		}
		this._textInput.value= list.join(", ");
	}
});
})();
