/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget");
dojo.require("jazz.ui.SimpleToolbar");

(function() {

// Extend SimpleToolbar to support enable/disable

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.ToolbarWidget",
	[jazz.ui.SimpleToolbar],
	{
		postCreate: function() {
			this._twoStateButtonWidgets = [];
			this.inherited(arguments);
		},
		uninitialize: function() {
			this._twoStateButtonWidgets = [];
		},
		handleContextChanged: function(parms /* workingCopy: aWorkItemProxy */) {
			for(var i=0;i<this._twoStateButtonWidgets.length; i++) {
				this._twoStateButtonWidgets[i]._setEnabledFromContext(parms);
			}
		},
		addTwoStateButton: function(context /* action */, callback, args /* isEnabled, imgSrc, label, alt */) {
			if(!args.label && context.getLabel) args.label = context.getLabel();
			if(!args.alt && context.getAlt) args.alt = context.getAlt();
			if(args.isEnabled == "") args.isEnabled = false;

			var buttonWidget = this.addButton(context, callback, args);
			// { domNode: theNode }

			if(args.alt && buttonWidget.domNode.childNodes && buttonWidget.domNode.childNodes[0]) buttonWidget.domNode.childNodes[0].alt = args.alt;
			dojo.addClass(buttonWidget.domNode, "SimpleToolbar-button"); //$NON-NLS-1$
			var parentToolbar = this;
			var twoStateButtonWidget = {
				postCreate: function() {
					this._buttonWidget = buttonWidget;
					this.setEnabled(this.isEnabled);
					this.setVisible(this.isVisible);
				},
				isEnabled: args.isEnabled === false ? false : true,
				isVisible: args.isVisible === false ? false : true,
				setEnabled: function(isEnabled) {
					this.isEnabled = isEnabled;
					if(this._buttonWidget) {
						if(isEnabled) {
							dojo.removeClass(this._buttonWidget.domNode, "SimpleToolbar-button-disabled"); //$NON-NLS-1$
							dojo.addClass(this._buttonWidget.domNode, "SimpleToolbar-button-enabled"); //$NON-NLS-1$
						} else {
							dojo.addClass(this._buttonWidget.domNode, "SimpleToolbar-button-disabled"); //$NON-NLS-1$
							dojo.removeClass(this._buttonWidget.domNode, "SimpleToolbar-button-enabled");	//$NON-NLS-1$					
						}
					}
				},
				updateButton: function(updateArgs /* imgSrc, label */) {
					if(!this._buttonWidget) return;
					updateArgs.imgSrc ? args.imgSrc = updateArgs.imgSrc : null;
					updateArgs.label ? args.label = updateArgs.label : null;
					if(!this.isEnabled) return;
					parentToolbar.updateButton(this._buttonWidget, updateArgs)
				},
				setVisible: function(isVisible) {
					if(this._buttonWidget) this._buttonWidget.domNode.style.display = isVisible ? "" : "none";
				},
				_setEnabledFromContext: function(parms /* workingCopy */) {
					if(!context || !context.isEnabled) return;
					this.setEnabled(context.isEnabled(parms));
				}
			}
			twoStateButtonWidget.postCreate();
			var updateFunction = dojo.hitch(context, function(){
					twoStateButtonWidget.setVisible(context.isVisible());
					twoStateButtonWidget.setEnabled(context.isEnabled());
				});
			if(dojo.isFunction(context.onUpdate)){
				this.connect(context, "onUpdate", updateFunction);
			}
			if(dojo.isFunction(context.initialize)){
				context.initialize(updateFunction);
			}
			this._twoStateButtonWidgets.push(twoStateButtonWidget);
			return twoStateButtonWidget;
		},
		/*
		 * Fix a problem with separator where the image shifts slightly
		 */
		addSeparator: function(index) {
			var separator = document.createElement("div"); //$NON-NLS-1$
			separator.className = "jazz-ui-SimpleToolbar-separator"; //$NON-NLS-1$
			dojo.addClass(separator, "SimpleToolbar-separator"); //$NON-NLS-1$
			return this._insert({domNode: separator}, index);
		}
	}
);

})();
