/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.oslc.OSLCBindingsEditor");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "OSLCMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var CSH_KEYS = {
	OSLC_EDITOR: "com.ibm.team.workitem.web.wi_type_binding_web_editor"
};
var NONE_VALUE = "";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.oslc.OSLCBindingsEditor",
	com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor,
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/oslc/templates/OSLCBindingsEditor.html"),
	pageCshId: CSH_KEYS.OSLC_EDITOR,
	
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "OSLCMessages");
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);

		var model = this.model;
		if(model){
			var self = this;
			this.connect(model, "onTypeChanged", function(args){
				if(!args || !args.target){
					self._populateTypes(false);
				}else if(args.target == "TYPE" && (args.key == "name" || args.key == "iconUrl")){
					self._populateTypes(false);
				}
			});
			this.connect(model, "onWorkflowChanged", function(workflowId){
				self._populateTypes(false);
			});
		}
		
		this._populateTypes(false);
	},
	
	_populateTypes: function(forceLoad){
		this.setLoading(true);
		this.model.getWorkItemConfig(dojo.hitch(this, function(workItemConfig){
			// insert options to all selects
			var numTypes = (workItemConfig.types) ? workItemConfig.types.length : 0;
			if(numTypes > 0){
				dojo.forEach([ "defectSelect", "planItemSelect", "taskSelect", "requirementSelect" ], function(name){
					var target = this[name];
					target.options.length = 0; // clear
					if(name == "taskSelect" || name == "requirementSelect"){
						target.options[0] = new Option(this.messages.None, NONE_VALUE);
					}
					var startIndex = target.options.length;
					for(var i=0; i<numTypes; i++){
						var typeInfo = workItemConfig.types[i];
						target.options[i+startIndex] = new Option(typeInfo.name, typeInfo.id);
					}
				}, this);
			}

			this.model.getOSLCConfigurations(dojo.hitch(this, function(oslcConfig){
				this.setLoading(false);

				if(forceLoad || !this._isInit){
					this.processCommonConfigData(oslcConfig);
				}
				
				this._isInit = true;
			
				dojo.forEach(oslcConfig.typeBindings.typeBinding, dojo.hitch(this, function(typeBinding){
					if(typeBinding.oslcTypeId == "defect"){
						this._selectValue(this.defectSelect, typeBinding.workItemTypeId);
						this._onDefectChange({noUpdate: true});
					}
					else if(typeBinding.oslcTypeId == "planItem"){
						this._selectValue(this.planItemSelect, typeBinding.workItemTypeId);
						this._onPlanItemChange({noUpdate: true});
					}
					else if(typeBinding.oslcTypeId == "task"){
						this._selectValue(this.taskSelect, typeBinding.workItemTypeId);
						this._onTaskChange({noUpdate: true});
					}
					else if(typeBinding.oslcTypeId == "requirementChangeRequest"){
						this._selectValue(this.requirementSelect, typeBinding.workItemTypeId);
						this._onRequirementChange({noUpdate: true});
					}
				}));			
				dojo.forEach(oslcConfig.stateGroups.stateGroup, dojo.hitch(this, function(stateGroup){
					if(stateGroup.id == "com.ibm.team.rqm.stateGroup.readyForTesting.defect"){
						this._selectValue(this.readyDefectSelect, stateGroup.state || NONE_VALUE);
						this._onDefectStateChange({noUpdate: true});
						this._selectValue(this.resolutionSelect, stateGroup.resolution || NONE_VALUE);
						this._onDefectResolutionChange({noUpdate: true});
					}
					else if(stateGroup.id == "com.ibm.team.rqm.stateGroup.readyForTesting"){
						this._selectValue(this.readyPlanItemSelect, stateGroup.state || NONE_VALUE);
						this._onPlanItemStateChange({noUpdate: true});
					}
				}));
			}), forceLoad);
		}), forceLoad);
	},
	
	refresh: function(){
		this._populateTypes(true);
	},
	
	_handleFinalChanged: function(){
		this.model.setOslcFinal(this.finalCheckbox.checked);
	},
	
	_selectValue: function(select, value){
		for(var i = 0, len = select.options.length; i<len; i++){
			var option = select.options[i];
			if(option.value == value){
				select.selectedIndex = i;
				return true;
			}
		}
		return false;
	},
	
	_getWorkflowById: function(workItemTypeId){
		if(!workItemTypeId || workItemTypeId.length===0){ return null; }
		var model = this.model;
		var workflowId = model.getWorkItemType(workItemTypeId).workflowId;
		return model.getWorkflowById(workflowId);
	},
	
	loadStatesForType: function(workItemTypeId, sel, iconElement){
		var selectedValue = sel.value;
		
		// load states for work item type
		sel.length = 0; // clear
		sel.options[0] = new Option(this.messages.None, NONE_VALUE);
		var workflow = this._getWorkflowById(workItemTypeId);
		if(workflow && workflow.states && workflow.states.length > 0){
			dojo.forEach(workflow.states, function(state){
				sel.options[sel.length] = new Option(state.name, state.id);
			});
		}
		
		if(selectedValue){
			if(!this._selectValue(sel, selectedValue)){
				sel.selectedIndex = 0;
			}
			this._setIcon(iconElement, this._getIconByState(workItemTypeId, sel.options[sel.selectedIndex].value));
		}
	},
	
	loadResolutionForType: function(workItemTypeId, sel, iconElement){
		var selectedValue = sel.value;

		// load states for work item type
		sel.length = 0; // clear
		sel.options[0] = new Option(this.messages.None, NONE_VALUE);
		var workflow = this._getWorkflowById(workItemTypeId);
		if(workflow && workflow.resolutions && workflow.resolutions.length > 0){
			dojo.forEach(workflow.resolutions, function(resolution){
				sel.options[sel.length] = new Option(resolution.name, resolution.id);
			});
		}

		if(selectedValue){
			if(!this._selectValue(sel, selectedValue)){
				sel.selectedIndex = 0;
			}
			this._setIcon(iconElement, this._getIconByResolution(workItemTypeId, sel.options[sel.selectedIndex].value));
		}
	},
	
	_onDefectChange: function(event){
		var workItemTypeId = this.defectSelect.value;
		
		// update defect icon
		var iconUrl = null;
		if(workItemTypeId && workItemTypeId.length>0){
			var typeInfo = this.model.getWorkItemType(workItemTypeId);
			if(typeInfo){
				iconUrl = typeInfo.iconUrl;
			}

			// update states
			this.loadStatesForType(workItemTypeId, this.readyDefectSelect, this.readyDefectIcon);

			// update resolutions
			this.loadResolutionForType(workItemTypeId, this.resolutionSelect, this.resolutionDefectIcon);
		
			if(!event.noUpdate){
				// update model
				this.model.setOSLCTypeBinding("defect", workItemTypeId);
			}
		}
		this._setIcon(this.defectIcon, iconUrl);
	},
	
	_onPlanItemChange: function(event){
		var workItemTypeId = this.planItemSelect.value;

		// update defect icon
		var iconUrl = null;
		if(workItemTypeId && workItemTypeId.length>0){
			var typeInfo = this.model.getWorkItemType(workItemTypeId);
			if(typeInfo){
				iconUrl = typeInfo.iconUrl;
			}

			// update states
			this.loadStatesForType(workItemTypeId, this.readyPlanItemSelect, this.readyPlanItemIcon);

			if(!event.noUpdate){
				// update model
				this.model.setOSLCTypeBinding("planItem", workItemTypeId);
			}
		}
		this._setIcon(this.planItemIcon, iconUrl);
	},
	
	_onTaskChange: function(event){
		var workItemTypeId = this.taskSelect.value;

		// update defect icon
		var iconUrl = null;
		if(workItemTypeId && workItemTypeId.length>0){
			var typeInfo = this.model.getWorkItemType(workItemTypeId);
			if(typeInfo){
				iconUrl = typeInfo.iconUrl;
			}
			if(!event.noUpdate){
				// update model
				this.model.setOSLCTypeBinding("task", workItemTypeId);
			}
		}
		this._setIcon(this.taskIcon, iconUrl);
	},
	
	_onRequirementChange: function(event){
		var workItemTypeId = this.requirementSelect.value;

		// update defect icon
		var iconUrl = null;
		if(workItemTypeId && workItemTypeId.length>0){
			var typeInfo = this.model.getWorkItemType(workItemTypeId);
			if(typeInfo){
				iconUrl = typeInfo.iconUrl;
			}

			if(!event.noUpdate){
				// update model
				this.model.setOSLCTypeBinding("requirementChangeRequest", workItemTypeId);
			}
		}
		this._setIcon(this.requirementIcon, iconUrl);
	},
	
	_onDefectStateChange: function(event){
		var stateId = this.readyDefectSelect.value;
		
		// update state icon
		var workItemTypeId = this.defectSelect.value;
		this._setIcon(this.readyDefectIcon, this._getIconByState(workItemTypeId, stateId));
		
		if(!event.noUpdate){
			// update model
			var sel2 = this.resolutionSelect;
			var resolutionId = sel2.options[sel2.selectedIndex].value;
			this.model.setOSLCStateGroup("com.ibm.team.rqm.stateGroup.readyForTesting.defect", stateId, resolutionId);
		}
	},
	
	_onDefectResolutionChange: function(event){
		var sel = this.resolutionSelect;
		var resolutionId = sel.options[sel.selectedIndex].value;
		
		// update state icon
		var workItemTypeId = this.defectSelect.value;
		this._setIcon(this.resolutionDefectIcon, this._getIconByResolution(workItemTypeId, resolutionId));
		
		if(!event.noUpdate){
			// update model
			var stateId = this.readyDefectSelect.value;
			this.model.setOSLCStateGroup("com.ibm.team.rqm.stateGroup.readyForTesting.defect", stateId, resolutionId);
		}
	},
	
	_onPlanItemStateChange: function(event){
		var stateId = this.readyPlanItemSelect.value;

		// update state icon
		var workItemTypeId = this.planItemSelect.value;
		this._setIcon(this.readyPlanItemIcon, this._getIconByState(workItemTypeId, stateId));

		if(!event.noUpdate){
			// update model
			this.model.setOSLCStateGroup("com.ibm.team.rqm.stateGroup.readyForTesting", stateId, null);
		}
	},
	
	_getIconByState: function(workItemTypeId, stateId){
		var workflow = this._getWorkflowById(workItemTypeId);
		if(workflow){
			var states = workflow.states || [];
			for(var i=0, len=states.length; i<len; i++){
				if(states[i].id == stateId){
					return states[i].iconUrl;
				}
			}
		}
		return null;
	},
	
	_getIconByResolution: function(workItemTypeId, resolutionId){
		var workflow = this._getWorkflowById(workItemTypeId);
		if(workflow){
			var resolutions = workflow.resolutions || [];
			for(var i=0, len=resolutions.length; i<len; i++){
				if(resolutions[i].id == resolutionId){
					return resolutions[i].iconUrl;
				}
			}
		}
		return null;
	},
	
	_setIcon: function(img, src){
		if(src) {
			img.src = src;
			img.style.display = "";
		} else{
			img.style.display = "none";
		}
	},
	
	destroy: function(preserveDom){
		this.inherited(arguments);
	}
});

})();