/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.PresentationEditor");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.EditorController");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.TabController");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.SectionController");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.PresentationController");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var CSH_KEYS = {
	PRESENTATION_EDITOR: "com.ibm.team.workitem.web.wi_present_web_editor"
};
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.PresentationEditor",
	com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor,
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/templates/PresentationEditor.html"),
	pageCshId: CSH_KEYS.PRESENTATION_EDITOR,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		
		var self = this;
		var model = this.model;
		var editorController = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.EditorController({
			model: model,
			removeBtn: this.removeBtn,
			presentationList: this.presentationList
		});
		this._editorController = editorController;
		this.connect(editorController, "onEditorSelected", function(editorId){
			// hide right section if no presentation is selected
			self.detailsSection.style.display = (editorId) ? "" : "none";
		});
		this.connect(editorController, "setLoading", "setLoading");
		this.connect(editorController, "onEditorLoaded", function(configData, forceLoad){
			if(forceLoad || !self._isInit){
				self.processCommonConfigData(configData);
			}
			self._isInit = true;
		});

		// create header
		var headerContent = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderContent({}, this.headerNode);
		this.headerContent = headerContent;

		// create tabContainer
		var tabContainer = new com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer({
			hasAddButton: true,
			createTabButton: function(title){
				var tabButton = com.ibm.team.workitem.web.process.ui.internal.view.common.TabContainer.prototype.createTabButton.apply(this, arguments);

				// add dropdown arrow
				tabButton.titleNode.parentNode.appendChild(createActionIcon({
					iconClass: "dropDownArrow",
					title: messages.Menu,
					onClick: dojo.hitch(self._tabController, "onTabDropdownButtonClicked"),
					owner: tabButton
				}));
				
				return tabButton;
			},
			onAddTabButtonClicked: function(){
				self._tabController.addTab();
			}
		}, this.tabContainerNode);
		this.tabContainer = tabContainer;
		// add class so that work item editor styles get applied
		dojo.addClass(tabContainer.domNode, "com-ibm-team-workitem");
		dojo.addClass(tabContainer.containerNode, "workItemEditor");

		var tabController = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.TabController({
			model: model,
			headerContent: headerContent,
			tabContainer: tabContainer,
			editorController: editorController
		});
		this._tabController = tabController;

		var sectionController = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.SectionController({
			model: model,
			tabController: tabController
		});
		this._sectionController = sectionController;

		// add "Add section" button
		var anchor = dojo.create("a", {
			href: "#",
			className: "dijitInline addSectionButton"
		});
		this.connect(anchor, "onclick", function(e){
			dojo.stopEvent(e);
			if(!dojo.hasClass(anchor, "disabled")){
				sectionController.addSection(tabController.getSelectedTabId());
			}
		});
		dojo.create("span", {
			className: "dijitInline iconNode addIcon"
		}, anchor);
		anchor.appendChild(dojo.doc.createTextNode(" "+messages.AddSection));
		dojo.place(anchor, tabContainer.tabButtonContainer, "first");
		tabContainer.addSectionButton = anchor;
		
		this._presentationController = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.PresentationController({
			model: model,
			sectionController: sectionController,
			owner: this
		});

		editorController.populateEditors(false); // forceLoad should be false as it may get loaded in other place (Types & Attributes page)
	},
	
	refresh: function(){
		this._editorController.populateEditors(true);
	},
	
	_handleFinalChanged: function(){
		this.model.setPresentationFinal(this.finalCheckbox.checked);
	},
	
	selectEditorPresentation: function(editorId, tabId){
		var tabs = this.model.getTabs(editorId);
		var tabIndex = null;
		for(var i=0, len=tabs.length; i<len; i++){
			if(tabs[i].tabId == tabId){
				tabIndex = i;
				break;
			}
		}
		this._editorController.setSelectedEditor(editorId, tabIndex);
	},
		
	_handleAddEditor: function(e){
		this._editorController.addEditor();
	},

	_handleRemoveEditor: function(){
		this._editorController.removeEditor();
	},

	destroy: function(preserveDom){
		if(this._tabDropdownMenu){
			this._tabDropdownMenu.destroy(preserveDom);
		}
		this._editorController.destroy();
		this._tabController.destroy();
		this._sectionController.destroy();
		this._presentationController.destroy();
		this.headerContent.destroy(preserveDom);
		this.tabContainer.destroy(preserveDom);
		this.inherited(arguments);
	}
});

})();