/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.EditorController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.core.loader");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.EditorController",
	null,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		var model = args.model;
		this.model = model;
		var presentationList = args.presentationList;
		this.presentationList = presentationList;
		this.removeBtn = args.removeBtn;
		var self = this;
		this._connects = [
			// connect to SELECT onchange event programmatically since connecting in template somehow doesn't work...
			dojo.connect(presentationList, "onchange", function(e){
				self.onEditorSelected(self.getSelectedEditorId());
			}),
			dojo.connect(model, "onPresentationChanged", function(obj){
				if(!obj.target){
					self.populateEditors(false);
				}
			})
		];
	},
	
	populateEditors: function(forceLoad){
		this.setLoading(true);
		var self = this;
		this.model.getEditorPresentations(function(configData){
			self.setLoading(false);
			
			var editorData = self.model.getEditors();
	
			// check previous selection to restore selection
			var options = self.presentationList.options;
			var selectedId = null;
			if(editorData.length > 0){
				if(options.length > 0){
					if(editorData.length > options.length){
						// type has been added, so select added option
						selectedId = editorData[editorData.length-1].id;
					}else if(editorData.length < options.length){
						// type has been removed, so select first option
						selectedId = editorData[0].id;
					}else{
						// otherwise, select same option
						selectedId = options[options.selectedIndex].value;
					}
				}else{
					selectedId = editorData[0].id;
				}
			}

			// clean-up the previous list
			options.length = 0;

			if(editorData.length > 0){
				// populate updated types list
				dojo.forEach(editorData, function(editorInfo, idx){
					options[idx] = new Option(editorInfo.id, editorInfo.id);
				}, self);
				self.removeBtn.disabled = false;
			}else{
				// no items, so clear selection
				selectedId = null;
				self.removeBtn.disabled = true;
			}
			self.setSelectedEditor(selectedId);
			
			self.onEditorLoaded(configData, forceLoad);
		}, forceLoad);
	},
	
	setLoading: function(isLoading){},
	
	onEditorLoaded: function(configData, forceLoad){},
	
	getSelectedEditorId: function(){
		// returns ID for selected editor presentation
		var options = this.presentationList.options;
		return (options.length > 0) ? options[options.selectedIndex].value : null;
	},
	
	setSelectedEditor: function(editorId, tabIndex){
		if(!editorId || this.getSelectedEditorId() == editorId){
			this.onEditorSelected(editorId, tabIndex);
		}else{
			var options = this.presentationList.options;
			for(var i=0, len=options.length; i<len; i++){
				if(options[i].value == editorId){
					options[i].selected = true;
					this.onEditorSelected(editorId, tabIndex);
					break;
				}
			}
		}
	},
	
	onEditorSelected: function(editorId){},
	
	addEditor: function(){
		var model = this.model;
		jazz.core.loader.load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddEditorContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddEditorContent.openDialog({
				model: model
			});
		});
	},
	
	removeEditor: function(){
		// removes selected editor
		var editorId = this.getSelectedEditorId();
		if(editorId && editorId.length>0){
			if(confirm(dojo.string.substitute(messages.ConfirmRemoveEditor, [editorId]))){
				this.model.removeEditor(editorId);
			}
		}
	},
	
	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
	}
});

})();