/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.PresentationController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.PartsMapping");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var getDesigntimeWidget = com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.PartsMapping.getDesigntimeWidget;
var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.PresentationController",
	null,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		var model = args.model;
		this.model = model;
		var sectionController = args.sectionController;
		this.sectionController = sectionController;
		this.owner = args.owner; // PresentationEditor widget
		var self = this;
		this._connects = [
			dojo.connect(sectionController, "onSectionAdded", this, "populatePresentations"),
			dojo.connect(sectionController, "onAddPresentationButtonClicked", this, "addPresentation"),
			dojo.connect(model, "onPresentationChanged", function(obj){
				if(obj.target == "SECTION"){
					self.populatePresentations(obj.sectionId);
				}
			})
		];
	},
	
	populatePresentations: function(sectionId){
		var sectionLayouts = this.sectionController.getSections(sectionId);
		var hasBorder = false;
		var bgClassName = null;
		var presentations = dojo.map(this.model.getPresentations(sectionId), function(presentation){
			presentation.displayName = this.getPresentationLabel(presentation);
			var widgetInfo = this.getPresentationWidget(presentation);
			presentation.widget = widgetInfo;
			
			// if any one of the presentation widget has border (QuickInformationPart or DiscussionPart), this sectionLayout will have border
			if(widgetInfo && widgetInfo.hasBorder === true){
				hasBorder = true;
			}
			if(widgetInfo && widgetInfo.backgroundClassName){
				bgClassName = widgetInfo.backgroundClassName;
			}
			return presentation;
		}, this);

		var self = this;
		dojo.forEach(sectionLayouts, function(sectionLayout){		
			// render presentations within a section
			var contentWidget = sectionLayout.getContent();
			contentWidget.clearPresentations();
			var widgets = contentWidget.addPresentations(presentations);
			
			sectionLayout.setBackgroundClassName(bgClassName);
			// set border needs to be after setBackgroundClassName
			// since com.ibm.team.workitem.web.process.ui.internal.view.common.SectionLayout#setBackgroundClassName will clear-out border CSS class
			sectionLayout.setBorder(hasBorder === true);
			
			// make draggable
			contentWidget.makeDraggable();

			// add action popup			
			for(var i=0, len=widgets.length; i<len; i++){
				widgets[i].__sectionId = sectionId; // not a good way, but cannot get correct index somehow in other ways...
				LayoutUtil.bindActionPopup({
					widget: widgets[i],
					actions: [
						{
							iconClass: "edit-dlg-command",
							onClick: function(targetWidget){
								var sectionId = targetWidget.__sectionId;
								var index = self.sectionController.getPresentationIndex(sectionId, targetWidget);
								if(index >= 0){
									self.editPresentation(sectionId, index);
								}
							},
							title: messages.EditPresentation
						},
						{
							iconClass: "delete-command",
							onClick: function(targetWidget){
								var sectionId = targetWidget.__sectionId;
								var index = self.sectionController.getPresentationIndex(sectionId, targetWidget);
								if(index >= 0){
									self.removePresentation(sectionId, index);
								}
							},
							title: messages.RemovePresentation
						}
					],
					owner: self.owner
				});
			}
		});
	},
	
	addPresentation: function(sectionId){
		this.editPresentation(sectionId, null);
	},
	
	editPresentation: function(sectionId, index){
		if(this.sectionController.checkSharedSection(sectionId)){
			var self = this;
			jazz.core.loader.load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent", function(){
				com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent.openDialog({
					index: index,
					model: self.model,
					editorId: self.owner._editorController.getSelectedEditorId(), // needed to filter out already used attributeIDs
					sectionId: sectionId
				});
			});
		}
	},
	
	removePresentation: function(sectionId, index){
		var self = this;
		if(this.sectionController.checkSharedSection(sectionId)){
			var presentations = self.model.getPresentations(sectionId);
			if(presentations.length <= index){ return; }
		
			var label = self.getPresentationLabel(presentations[index]);
			var msg = (label) ? dojo.string.substitute(messages.ConfirmRemovePresentation, [label]) :
				messages.ConfirmRemovePresentationNoLabel; // TODO : need to show label for non-attribute-based kind
			if(confirm(msg)){
				self.model.removePresentation(sectionId, index);
			}
		}
	},
	
	getPresentationWidget: function(presentation){
		// returns a widget class name that is used for design time for a given "kind"
		if(LayoutUtil.isHiddenKind(presentation)){
			return {
				widgetClass: LayoutUtil.HIDDEN_WIDGET
			};
		}
		if(!this._widgetMappings){
			var widgetMappings = {};
			dojo.forEach(this.model.getKindMappings(), function(mapping){
				widgetMappings[mapping.kind] = {
					widget: mapping.widget
				};
				if(mapping.needsAttribute === true){
					widgetMappings[mapping.kind].needsAttribute = true;
				}
			}, this);
			this._widgetMappings = widgetMappings;
		}
		var widgetMapping = this._widgetMappings[presentation.kind];
		var runtimeWidget = widgetMapping ? widgetMapping.widget : null;
		return getDesigntimeWidget(runtimeWidget);
	},
	
	getPresentationLabel: function(presentation){
		var attributeLabel = presentation.attributeLabel;
		if(attributeLabel && attributeLabel.length>0){
			return attributeLabel;
		}
	
		var stringIdentifier = presentation.attributeId;
		if(!stringIdentifier){ return null; }

		// attribute-based
		var model = this.model;
		var builtInAttrs = model.getBuiltInAttributes();
		for(var i=builtInAttrs.length-1; i>=0; i--){
			if(builtInAttrs[i].stringIdentifier == stringIdentifier){
				var attrDef = model.findAttributeDefinition(builtInAttrs[i].id);
				if(attrDef){
					// built-in attribute name has been overwritten, so use that as label
					return attrDef.name;
				}else{
					// otherwise, use default built-in attribute name
					return builtInAttrs[i].name;
				}
			}
		}
			
		// not found in built-in attribute, so it's a custom attribute
		var customAttrs = model.getAllCustomAttributes();
		for(var j=customAttrs.length-1; j>=0; j--){
			if(customAttrs[j].id == stringIdentifier){
				return customAttrs[j].name;
			}
		}
		
		// not found
		return null;
	},
	
	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
	}
});

})();