/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.SectionController");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.SectionLayout");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderSectionContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var load_async = jazz.core.loader.load_async;
var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;
var SectionLayout = com.ibm.team.workitem.web.process.ui.internal.view.common.SectionLayout;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.SectionController",
	null,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		var model = args.model;
		this.model = model;
		var tabController = args.tabController;
		this.tabController = tabController;
		var self = this;
		this._connects = [
			dojo.connect(tabController, "clearTabs", function(){
				self._sectionInfoMap = {};
			}),
			dojo.connect(tabController, "onTabCreated", this, "populateSections"),
			dojo.connect(model, "onPresentationChanged", function(obj){
				if(obj.target == "TAB"){
					self.populateSections(obj.tabId);
				}
			})
		];
		this._sectionInfoMap = {};
	},
	
	populateSections: function(tabId){
		var tabArray = this.tabController.getTabs(tabId);
		var model = this.model;
		var sectionArray = model.getSections(tabId);
		dojo.forEach(sectionArray, function(sectionInfo){
			this._sectionInfoMap[sectionInfo.sectionId] = [];
		}, this);
		var layout = model.getTabLayout(tabId);
		var self = this;
		dojo.forEach(tabArray, function(tab){
			if(!tab.isHeader){
				tab.initSlots(layout);
			}
			tab.clearSections();
			dojo.forEach(sectionArray, function(sectionInfo){
				var sectionId = sectionInfo.sectionId;
				var isHeader = tab.isHeader;
				var title = (isHeader) ? messages.HeaderTitle : sectionInfo.title;
				var sectionLayout = this._createSection(tabId, tab, sectionId, title, !isHeader);
				var contentWidget;
				if(isHeader){
					dojo.addClass(sectionLayout.sectionTitle, "slotLabel");
					// disable toggling section for header
					contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderSectionContent();
					sectionLayout.setContent(contentWidget);
					tab.addSection(sectionLayout);
				}else{
					var slot = sectionInfo.slot;
					contentWidget = LayoutUtil.createSectionContent(model.getTabLayout(tabId), slot, title);
					sectionLayout.setContent(contentWidget);
					tab.addSection(sectionLayout, slot);
				}
				
				// support drag & drop of presentations
				var draggableContainers = contentWidget.getDraggableContainers();
				dojo.forEach(draggableContainers, function(fromContainer, idx){
					fromContainer.__sectionId = sectionId;
					contentWidget.connect(fromContainer, "move", function(fromIndex, toContainer, toIndex){
						var sectionFromIndex = -1;
						if(fromContainer.widgets && fromContainer.widgets.length > fromIndex){
							sectionFromIndex = self.getPresentationIndex(sectionId, fromContainer.widgets[fromIndex]);
						}
						var toSectionId = toContainer.__sectionId;
						var sectionToIndex = -1;
						if(toContainer.widgets && toContainer.widgets.length >= toIndex){
							// if fromContainer is different from toContainer, number of widgets of toContainer may increments by 1
							if(fromContainer == toContainer){
								sectionToIndex = self.getPresentationIndex(toSectionId, toContainer.widgets[toIndex]);
							}else{
								if(toContainer.widgets.length > 0){
									if(toIndex > 0){
										// find previous widget and increment index by 1
										sectionToIndex = self.getPresentationIndex(toSectionId, toContainer.widgets[toIndex-1])+1;
									}else{
										// set index of the first widget in toContainer
										sectionToIndex = self.getPresentationIndex(toSectionId, toContainer.widgets[0]);
									}
								} else {
									sectionToIndex= 0;
								}
								
								// 124973: Presentation is displaced to when drag & dropping from left table to right table in Details slot 
								if(sectionToIndex > 0){
									var fromContainerIndex = dojo.indexOf(draggableContainers, fromContainer);
									var toContainerIndex = dojo.indexOf(draggableContainers, toContainer);
									if(0 <= fromContainerIndex && fromContainerIndex < toContainerIndex){
										sectionToIndex--;
									}
								}
							}
						}
						if(sectionFromIndex >= 0 && sectionToIndex >= 0){
							if(self.checkSharedSection([ sectionId, toSectionId ])){
								model.movePresentation(sectionId, sectionFromIndex, toSectionId, sectionToIndex);
								return;
							}
						}
						fromContainer.onDndStop(false);
					});
				});
				
				this.onSectionAdded(sectionId);
			}, this);
		}, this);
	},

	getPresentationIndex: function(sectionId, targetWidget){
		var sectionLayouts = this.getSections(sectionId);
		for(var i=sectionLayouts.length-1; i>=0; i--){
			var contentWidget = sectionLayouts[i].getContent();
			if(contentWidget){
				var presentations = contentWidget.getPresentations();
				return dojo.indexOf(presentations, targetWidget);
			}
		}
		return -1;
	},

	_createSection: function(tabId, tab, sectionId, title, isCollapsible){
		var self = this;
		var sectionLayout = new SectionLayout({
			heading: title,
			isCollapsible: isCollapsible,
			actions: [
				{
					iconClass: "add-command",
					title: messages.AddPresentation,
					onClick: function(){
						self.onAddPresentationButtonClicked(sectionId);
					}
				},
				{
					iconClass: "edit-dlg-command",
					title: messages.EditSection,
					onClick: function(){
						var index = dojo.indexOf(tab.getSections(), sectionLayout);
						self.editSection(tabId, index);
					}
				},
				{
					iconClass: "delete-command",
					title: messages.RemoveSection,
					onClick: function(){
						var index = dojo.indexOf(tab.getSections(), sectionLayout);
						self.removeSection(tabId, index);
					}
				}
			]
		});
		this._sectionInfoMap[sectionId].push(sectionLayout);
		
		// 132591: No warning when editing a shared tab or section in the web ui
		var warningMessage = (this.model.isSharedSection(sectionId)) ? messages.ContentShared : null;
		sectionLayout.showWarningIcon(warningMessage);
		
		return sectionLayout;
	},
	
	onSectionAdded: function(sectionId){},
	
	checkSharedSection: function(/*String or Array*/sectionIds){
		var isShared = false;
		if(dojo.isArray(sectionIds)){
			for(var i=0, len=sectionIds.length; i<len; i++){
				if(this.model.isSharedSection(sectionIds[i])){
					isShared = true;
					break;
				}
			}
		}else{
			isShared = this.model.isSharedSection(sectionIds);
		}
		if(isShared){
			if(!confirm(messages.LockedSectionDesc)){
				return false;
			}
		}
		return true;
	},
	
	addSection: function(tabId){
		if(this.tabController.checkSharedTab(tabId)){
			var model = this.model;
			load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent", function(){
				com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openDialog({
					model: model,
					tabId: tabId
				});
			});
		}
	},
	
	editSection: function(tabId, sectionIndex){
		if(this.tabController.checkSharedTab(tabId)){
			var model = this.model;
			var isHeader = this.tabController.isHeader(tabId);
			load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent", function(){
				com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openEditDialog({
					model: model,
					isHeader: isHeader,
					tabId: tabId,
					sectionIndex: sectionIndex
				});
			});
		}
	},
	
	removeSection: function(tabId, sectionIndex){
		if(this.tabController.checkSharedTab(tabId)){
			var model = this.model;
			if(confirm(dojo.string.substitute(messages.ConfirmRemoveSection, [model.getSectionProperty(tabId, sectionIndex, "title")]))){
				if(this.tabController.isHeader(tabId)){
					this.tabController.removeHeader();
				}else{
					model.removeSection(tabId, sectionIndex);
				}
			}
		}
	},
	
	getSections: function(sectionId){
		// there may be multiple sections for the same sectionId
		return this._sectionInfoMap[sectionId] || [];
	},
	
	onAddPresentationButtonClicked: function(sectionId){},
	
	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
	}
});

})();