/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.TabController");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.TabContent");
//dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer");

dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
//var dndType = "tab";
var tabWidgetClassName = "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.TabContent";
var TabContent = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.TabContent;
var load_async = jazz.core.loader.load_async;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.TabController",
	null,
{
	constructor: function(args){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		var headerContent = args.headerContent;
		this.headerContent = headerContent;
		var tabContainer = args.tabContainer;
		this.tabContainer = tabContainer;
		var model = args.model;
		this.model = model;
		var editorController = args.editorController;
		this.editorController = editorController;
		var self = this;
		this._connects = [
			dojo.connect(headerContent, "onAddHeaderLinkClicked", this, "addHeader"),
			dojo.connect(editorController, "onEditorSelected", function(editorId, tabIndex){
				self.populateTabs(tabIndex);
			}),
			dojo.connect(model, "onPresentationChanged", function(obj){
				var target = obj.target;
				if(target == "EDITOR"){
					self.populateTabs(obj.tabIndex);
				}else if(target == "TAB" && obj.editorId && obj.tabIndex !== null && obj.tabIndex >= 0){
					// rename tab - only update tab title
					var tabIndex = obj.tabIndex;
					if(tabIndex !== -1){
						var tabButton = tabContainer.getTabButton(tabIndex);
						if(tabButton){
							tabButton.setTitle(model.getTabProperty(obj.editorId, tabIndex, "title"));
						}
					}
				}
			})
		];
		this._tabInfoMap = {};

		// for drag & drop
//		var draggableContainer = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer(tabContainer.tabButtonContainer, {
//			accept: [dndType]
//		});
//		draggableContainer.avatarClass = "com-ibm-team-workitem com-ibm-team-workitem-process";
//		this._draggable = draggableContainer;
	},
	
	clearTabs: function(){
//		var draggableContainer = this._draggable;
//		for(var i=this._draggable.widgets.length-1; i>=0; i--){
//			draggableContainer.remove(i);
//		}
		this.tabContainer.clearTabs();
		this._tabInfoMap = {};
	},
	
	populateTabs: function(selectIndex){
		this.headerContent.clearSections();

		// clear tabs
		this.clearTabs();
		this.disableAddSectionLink(true);
		
		var editorId = this.editorController.getSelectedEditorId();
		var tabArray = this.model.getTabs(editorId);
		var selectedTabWidgetId = null;
		var isIndexSpecified = (selectIndex !== null && selectIndex >= 0);
		if(!isIndexSpecified ||
			(selectIndex === 0 && this.hasHeader()) // if header is specified, select first tab
			){
			// if selectIndex is NOT specified, select first tab excluding header
			selectIndex = this.hasHeader() ? 1 : 0;
		}
		dojo.forEach(tabArray, function(tabInfo, index){
			var tabId = tabInfo.tabId;
			if(this.isHeader(tabId)){
				this._tabInfoMap[tabId] = [ this.headerContent ];
				this.onTabCreated(tabId);
			}else{
				var isLazyLoad = (index !== selectIndex);
				var tabWidgetId = this._doAddTab(editorId, tabInfo, isLazyLoad);
				if(!isLazyLoad){
					selectedTabWidgetId = tabWidgetId;
				}
			}
		}, this);
		
		// select new child
		this.tabContainer.selectChild(selectedTabWidgetId);
		if(selectedTabWidgetId){
			this.disableAddSectionLink(false);
		}
	},
	
	disableAddSectionLink: function(disable){
		dojo.toggleClass(this.tabContainer.addSectionButton, "disabled", disable);
	},
	
	isHeader: function(tabId){
		return (this.model.getTabLayout(tabId) == "internalHeader");
	},
	
	_doAddTab: function(editorId, tabInfo, isLazyLoad){
		var tabId = tabInfo.tabId;
		var title = tabInfo.title;
		var tabWidgetId = dijit.getUniqueId(tabWidgetClassName);
		
		var self = this;
		var _createTab = function(){
			var tab = new TabContent({
				id: tabWidgetId,
				title: title
			}, dojo.doc.createElement("div"));

			if(!self._tabInfoMap[tabId]){
				self._tabInfoMap[tabId] = [];
			}
			self._tabInfoMap[tabId].push(tab);
		
			// notify that tab is created
			self.onTabCreated(tabId);

			// support drag & drop of sections
			tab.connect(tab, "onSectionMoved", function(sectionIndex, toSlot, toIndex, draggableContainer){
				if(self.checkSharedTab(tabId)){
					self.model.moveSection(tabId, sectionIndex, toSlot, toIndex);
					return;
				}
				draggableContainer.onDndStop(false);
			});
			
			return tab;
		};
		
		var obj = null;
		if(isLazyLoad === true){
			// lazy load
			obj = {
				id: tabWidgetId,
				title: title,
				createChild: function(){
					return _createTab();
				}
			};
		}else{
			// create tab immediately
			obj = _createTab();
		}
		var tabButton = this.tabContainer.addChild(obj);
//		this._draggable.add(tabButton, { type:dndType });

		// 132591: No warning when editing a shared tab or section in the web ui
		var warningMessage = (this.model.isSharedTab(tabId)) ? messages.ContentShared : null;
		if(warningMessage){
			var icon = dojo.create("img", {
				src: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/page/templates/warning_ovr.gif").toString(),
				alt: warningMessage,
				title: warningMessage,
				style: {
					position: "absolute",
					top: "8px"
				}
			});
			dojo.place(icon, tabButton.titleNode, "before");
		}

		return tabWidgetId;
	},
	
	onTabCreated: function(tabId){},
	
	checkSharedTab: function(tabIds){
		var isShared = false;
		if(dojo.isArray(tabIds)){
			for(var i=0, len=tabIds.length; i<len; i++){
				if(this.model.isSharedTab(tabIds[i])){
					isShared = true;
					break;
				}
			}
		}else{
			isShared = this.model.isSharedTab(tabIds);
		}
		if(isShared){
			if(!confirm(messages.LockedTabDesc)){
				return false;
			}
		}
		return true;
		
	},

	onTabDropdownButtonClicked: function(tabDropdownNode){
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.tabButtonUtil", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.presentation.controllers.tabButtonUtil.openMenu(self, tabDropdownNode);
		});
	},
	
	getTabs: function(tabId){
		// there may be multiple tabs for the same tabId
		return this._tabInfoMap[tabId] || [];
	},
	
	getSelectedTabId: function(){
		// returns tabId of selected tab
		var selectedTab = this.tabContainer.getSelectedChild();
		for(var tabId in this._tabInfoMap){
			var tabArray = this.getTabs(tabId);
			for(var i=tabArray.length-1; i>=0; i--){
				if(tabArray[i] == selectedTab){
					return tabId;
				}
			}
		}
		return null;
	},
	
	hasHeader: function(){
		var editorId = this.editorController.getSelectedEditorId();
		var childTabs = this.model.getTabs(editorId);
		for(var i=0, len=childTabs.length; i<len; i++){
			if(this.isHeader(childTabs[i].tabId)){
				return true;
			}
		}
		return false;
	},
	
	getSelectedTabIndex: function(){
		// returns tabIndex that considers header
		var headerOffset = this.hasHeader() ? 1 : 0;
		return headerOffset + this.tabContainer.getSelectedIndex();
	},
	
	addTab: function(){
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent.openDialog({
				model: self.model,
				editorId: self.editorController.getSelectedEditorId()
			});
		});
	},
	
	editTab: function(){
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent.openEditDialog({
				model: self.model,
				editorId: self.editorController.getSelectedEditorId(),
				tabIndex: self.getSelectedTabIndex()
			});
		});
	},
	
	removeTab: function(){
		var editorId = this.editorController.getSelectedEditorId();
		var tabIndex = this.getSelectedTabIndex();
		if(tabIndex === -1){ return; }
		var model = this.model;
		var title = model.getTabProperty(editorId, tabIndex, "title") || model.getTabProperty(editorId, tabIndex, "tabId");
		if(confirm(dojo.string.substitute(messages.ConfirmRemoveTab, [title]))){
			// TODO : if this tab is shared with multiple editors, need to show such warning message
			model.removeTab(editorId, tabIndex);
		}
	},
	
	moveTabLeft: function(){
		var editorId = this.editorController.getSelectedEditorId();
		var tabIndex = this.getSelectedTabIndex();
		var canMoveLeft = (tabIndex > 0);
		if(!canMoveLeft ||
			(this.hasHeader() && tabIndex === 1)
			){ return; }
		this.model.moveTab(editorId, tabIndex, tabIndex-1);
	},
	
	moveTabRight: function(){
		var editorId = this.editorController.getSelectedEditorId();
		var tabIndex = this.getSelectedTabIndex();
		if(tabIndex === -1){ return; }
		this.model.moveTab(editorId, tabIndex, tabIndex+1);
	},
	
	renameTab: function(){
		var editorId = this.editorController.getSelectedEditorId();
		var tabIndex = this.getSelectedIndex();
		if(tabIndex === -1){ return; }
		var model = this.model;
		var origTitle = model.getTabProperty(editorId, tabIndex, "title") || "";
		var name = prompt(messages.RenameTabMsg, origTitle);
		if(name === null){ return; } // cancel
		if(name.length>0 && name != origTitle){
			model.renameTab(editorId, tabIndex, name);
		}else{
			// empty
			alert(messages.ErrEmptyTabTitle);
		}
	},
	
	addHeader: function(){
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openAddHeaderDialog({
				model: self.model,
				editorId: self.editorController.getSelectedEditorId()
			});
		});
	},
	
	removeHeader: function(){
		var editorId = this.editorController.getSelectedEditorId();
		this.model.removeHeader(editorId);
	},
	
	setTabLayout: function(tabId, layout){
		this.model.setTabLayout(tabId, layout);
	},
	
	destroy: function(){
		dojo.forEach(this._connects, dojo.disconnect);
		if(this._tabDropdownMenu){
			this._tabDropdownMenu.destroyRecursive();
		}
//		this._draggable.destroy();
	}
});

})();