/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("jazz.ui.Menu");
dojo.require("dojo.string");
dojo.require("dojo.data.util.sorter");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages, nonAttrKinds;
var ENUM_TYPE = "com.ibm.team.workitem.attributeType.enumerationTypes";
var HIDDEN_KIND = "com.ibm.team.workitem.kind.special.doNotShow";

var basicComparator = dojo.data.util.sorter.basicComparator;

// ignore the following attributes
// see com.ibm.team.workitem.ide.ui.internal.aspecteditor.editorpresentation.EditorPresentationAspectPart#setRequiredListsAndMaps
var ignoreAttrs = [
	"com.ibm.team.workitem.attribute.state",
	"com.ibm.team.workitem.attribute.resolution",
	"startDate" // TODO : need to check if this works right 
];

// hard-coded for now
// need to get this information from server
// see com.ibm.team.workitem.ide.ui.editorPresentations extension-point
var kindAttrMap = {
	// see com.ibm.team.apt.ide.ui/plugin.xml
	"com.ibm.team.workitem.kind.timeDuration" : [ "duration" ],
	"com.ibm.team.apt.kind.internal.constraintdate" : [ "timestamp" ],
	"com.ibm.team.apt.kind.internal.estimate" : [ "duration" ],
	"com.ibm.team.apt.kind.internal.timespent" : [ "duration" ],
	"com.ibm.team.apt.kind.internal.wiki" : [ "wiki" ],

	"com.ibm.team.tpt.kind.cost" : [ "long" ],

	// see com.ibm.team.workitem.ide.ui/plugin.xml
	"com.ibm.team.workitem.kind.enumeration" : [ "type", ENUM_TYPE ],
	"com.ibm.team.workitem.kind.string" : [ "string", "mediumString", "smallString", "html", "mediumHtml", "integer", "long" ],
	"com.ibm.team.workitem.kind.smallhtml" : [ "string", "mediumString", "smallString", "html", "mediumHtml" ],
	"com.ibm.team.workitem.kind.long" : [ "long" ],
	"com.ibm.team.workitem.kind.integer" : [ "integer" ],
	"com.ibm.team.workitem.kind.text" : [ "string", "mediumString", "smallString", "html", "mediumHtml" ],
	"com.ibm.team.workitem.kind.timestamp" : [ "timestamp" ],
	"com.ibm.team.workitem.kind.contributor" : [ "contributor" ],
	"com.ibm.team.workitem.kind.boolean" : [ "boolean" ],
	"com.ibm.team.workitem.kind.internal.tags" : [ "tags" ],
	"com.ibm.team.workitem.kind.internal.category" : [ "category" ],
	"com.ibm.team.workitem.kind.internal.target" : [ "interval" ],
	"com.ibm.team.workitem.kind.internal.deliverable" : [ "deliverable" ],
	"com.ibm.team.workitem.kind.internal.description" : [ "string", "mediumString", "smallString", "html", "mediumHtml" ],
	"com.ibm.team.workitem.kind.html" : [ "string", "mediumString", "smallString", "html", "mediumHtml" ],
	"com.ibm.team.workitem.kind.list" : [ "string", "mediumString", "smallString" ],
	"com.ibm.team.workitem.kind.list.checkbox" : [ "string", "mediumString", "smallString" ],
	"com.ibm.team.workitem.kind.radiogroup" : [ "type", ENUM_TYPE ],
	"com.ibm.team.workitem.kind.processAreaList" : [ "processAreaList", "projectAreaList", "teamAreaList" ],
	"com.ibm.team.workitem.kind.processArea" : [ "processArea", "projectArea", "teamArea" ],
	"com.ibm.team.workitem.kind.workItem" : [ "workItem" ],
	"com.ibm.team.workitem.kind.item.combo" : [ "workItem", "processArea", "projectArea", "teamArea", "contributor", "category", "interval", "deliverable" ],
	"com.ibm.team.workitem.kind.accesscontrol.combo" : [ "uuid" ],
	"com.ibm.team.workitem.kind.workItemList" : [ "workItemList" ],
	"com.ibm.team.workitem.kind.contributorList" : [ "subscriptions", "contributorList" ],
	"com.ibm.team.workitem.kind.eSignature" : [ "eSignature" ],
	"com.ibm.team.workitem.kind.item" : [ "item" ],
	"com.ibm.team.workitem.kind.itemList" : [ "itemList" ],
	"com.ibm.team.workitem.kind.internal.valuesetpicker" : [ "string", "mediumString", "smallString" ], 
	"com.ibm.team.workitem.kind.internal.valuesetcombo" : [ "string", "mediumString", "smallString" ]
};

// key is 'kind' and value is i18n key used in Eclipse client
// 116142: Some of com.ibm.team.workitem.ide.ui.editorPresentations extension point information should be used commonly between clients
var kindLabelMap = {
	// see com.ibm.team.workitem.ide.ui\plugin.xml
	"com.ibm.team.workitem.kind.enumeration": "editorPresentation.displayName.enumeration",
	"com.ibm.team.workitem.kind.workflow.state": "editorPresentation.displayName.workflowState",
	"com.ibm.team.workitem.kind.workflow.resolution": "editorPresentation.displayName.workflowResolution",
	"com.ibm.team.workitem.kind.string": "editorPresentation.displayName.string",
	"com.ibm.team.workitem.kind.smallhtml": "editorPresentation.displayName.smallHtml",
	"com.ibm.team.workitem.kind.long": "editorPresentation.displayName.long",
	"com.ibm.team.workitem.kind.integer": "editorPresentation.displayName.integer",
	"com.ibm.team.workitem.kind.text": "editorPresentation.displayName.text",
	"com.ibm.team.workitem.kind.timestamp": "editorPresentation.displayName.timestamp",
	"com.ibm.team.workitem.kind.contributor": "editorPresentation.displayName.contributor",
	"com.ibm.team.workitem.kind.boolean": "editorPresentation.displayName.boolean",
	"com.ibm.team.workitem.kind.internal.teamarea": "editorPresentation.displayName.teamarea",
	"com.ibm.team.workitem.kind.internal.tags": "editorPresentation.displayName.tags",
	"com.ibm.team.workitem.kind.internal.category": "editorPresentation.displayName.category",
	"com.ibm.team.workitem.kind.internal.target": "editorPresentation.displayName.iteration",
	"com.ibm.team.workitem.kind.internal.deliverable": "editorPresentation.displayName.deliverable",
	"com.ibm.team.workitem.kind.internal.description": "editorPresentation.displayName.description",
	"com.ibm.team.workitem.kind.html": "editorPresentation.displayName.html",
	"com.ibm.team.workitem.kind.internal.discussion": "editorPresentation.displayName.discussion",
	"com.ibm.team.workitem.kind.internal.quickInformation": "editorPresentation.displayName.quickInformation",
	"com.ibm.team.workitem.kind.internal.attachments": "editorPresentation.displayName.attachments",
	"com.ibm.team.workitem.kind.contributorList": "editorPresentation.displayName.contributorList",
	"com.ibm.team.workitem.kind.processAreaList": "editorPresentation.displayName.processAreaList",
	"com.ibm.team.workitem.kind.processArea": "editorPresentation.displayName.processArea",
	"com.ibm.team.workitem.kind.workItem": "editorPresentation.displayName.workItem",
	"com.ibm.team.workitem.kind.workItemList": "editorPresentation.displayName.workItemList",
	"com.ibm.team.workitem.kind.internal.links": "editorPresentation.displayName.links",
	"com.ibm.team.workitem.kind.internal.approvals": "editorPresentation.displayName.approvals",
	"com.ibm.team.workitem.kind.internal.history": "editorPresentation.displayName.history",
	"com.ibm.team.workitem.kind.separator": "editorPresentation.displayName.separator",
	"com.ibm.team.workitem.kind.list": "editorPresentation.displayName.list",
	"com.ibm.team.workitem.kind.list.checkbox": "editorPresentation.displayName.checkbox",
	"com.ibm.team.workitem.kind.radiogroup": "editorPresentation.displayName.radio",
	"com.ibm.team.workitem.kind.staticText": "editorPresentation.displayName.staticText",
	"com.ibm.team.workitem.kind.lineSeparator": "editorPresentation.displayName.lineSeparator",
	"com.ibm.team.workitem.kind.eSignature": "editorPresentation.displayName.eSignature",
	"com.ibm.team.workitem.kind.reference": "editorPresentation.displayName.references",
	"com.ibm.team.workitem.kind.item.combo": "editorPresentation.displayName.itemCombo",
	"com.ibm.team.workitem.kind.accesscontrol.combo": "editorPresentation.displayName.accesscontrolCombo",
	"com.ibm.team.workitem.kind.item": "editorPresentation.displayName.item",
	"com.ibm.team.workitem.kind.itemList": "editorPresentation.displayName.itemList",
	"com.ibm.team.workitem.kind.internal.valuesetpicker" : "editorPresentation.displayName.valueSetCombo", 
	"com.ibm.team.workitem.kind.internal.valuesetcombo" : "editorPresentation.displayName.valueSetPicker",
	
	// see com.ibm.team.apt.ide.ui\plugin.xml
	"com.ibm.team.apt.kind.internal.estimate": "editorPresentation.displayName.estimate",
	"com.ibm.team.workitem.kind.timeDuration": "editorPresentation.displayName.duration",
	"com.ibm.team.apt.kind.internal.timespent": "editorPresentation.displayName.timeSpent",
	"com.ibm.team.apt.kind.internal.progress": "editorPresentation.displayName.progress",
	"com.ibm.team.apt.kind.internal.wiki": "editorPresentation.displayName.wiki",
	"com.ibm.team.apt.kind.internal.constraintdate": "editorPresentation.displayName.constraintDate",
	"com.ibm.team.apt.kind.internal.plannedtime": "editorPresentation.displayName.plannedTime", 

	// see com.ibm.team.tpt.ide.ui\plugin.xml
	"com.ibm.team.tpt.kind.internal.timeSheet": "editorPresentation.displayName.timeSheet",
	"com.ibm.team.tpt.kind.internal.timesheetEntries": "editorPresentation.displayName.timeSheetEntries",
	"com.ibm.team.tpt.risk.kind.matrix": "editorPresentation.displayName.riskmatrix",
	"com.ibm.team.tpt.kind.cost": "editorPresentation.displayName.riskcost"
};

var getKindLabel = function(kind){
	if(!messages){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
	}
	if(messages && messages[kindLabelMap[kind]]){
		return messages[kindLabelMap[kind]];
	}else{
		return kind;
	}
};

var attrKindMap = {};
for(var kind in kindAttrMap){
	dojo.forEach(kindAttrMap[kind], function(attrTypeId){
		if(!attrKindMap[attrTypeId]){
			attrKindMap[attrTypeId] = [];
		}
		attrKindMap[attrTypeId].push({
			label: getKindLabel(kind),
			kind: kind
		});
	});
}

var propSpecs = {
	CONTRIBUTOR: {
		warningMessage: [],
		readonly: [ "true", "false" ],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	QUICK_INFO: {
		quickinformationConfiguration: []
	},
	LINKS: {
		linkTypeFilter: [], // to do need validation
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	APPROVALS: {
		toBeLinkedPartId: []
	},
	CHECKED_LIST: {
		enumeration: [],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	STATIC_TEXT: {
		text: [],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	ESIGNATURE: {
		timeout: [],
		readonly: [ "true", "false" ],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	REFERENCE: {
		endpointId: [],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	NONE: {},
	DEFAULT_ATTR: {
		readonly: [ "true", "false" ],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	DEFAULT: {
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	},
	VALUESETPICKER: {
		readonly: [ "true", "false" ],
		allowAnyValue: [ "true", "false"],
		labelVisible: [ "true", "false" ],
		hideIfEmpty: [ "true", "false" ],
		hideIfCreation: [ "true", "false" ],
		hideIfInState: [],
		hideIfEndpointEmpty: [],
		hideIfNoProjectLink: []
	}
};
// see com.ibm.team.workitem.ide.ui.presentationConfigurationlet extension-point in com.ibm.team.workitem.ide.ui\plugin.xml
var kindPropSpecMap = {
	"com.ibm.team.workitem.kind.contributor": "CONTRIBUTOR",
	"com.ibm.team.workitem.kind.internal.quickInformation": "QUICK_INFO",
	"com.ibm.team.workitem.kind.internal.links": "LINKS",
	"com.ibm.team.workitem.kind.internal.approvals": "APPROVALS",
	"com.ibm.team.workitem.kind.list": "CHECKED_LIST",
	"com.ibm.team.workitem.kind.list.checkbox": "CHECKED_LIST",
	"com.ibm.team.workitem.kind.staticText": "STATIC_TEXT",
	"com.ibm.team.workitem.kind.eSignature": "ESIGNATURE",
	"com.ibm.team.workitem.kind.reference": "REFERENCE",
	"com.ibm.team.workitem.kind.internal.discussion": "NONE",
	"com.ibm.team.workitem.kind.internal.attachments": "NONE",
	"com.ibm.team.workitem.kind.internal.history": "NONE",
	"com.ibm.team.workitem.kind.separator": "NONE",
	"com.ibm.team.workitem.kind.lineSeparator": "NONE",
	"com.ibm.team.workitem.kind.internal.valuesetpicker": "VALUESETPICKER", 
	"com.ibm.team.workitem.kind.internal.valuesetcombo": "DEFAULT_ATTR"
};

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/dialogs/templates/AddPresentationContent.html"),

	isEdit: false,
	attributeId: null,
	kind: null,
	usedAttrs: [],
	attributes: [],
	nonAttrKinds: [],
	label: null,
	desc: null,
	presentationId: null,
	properties: null,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){		
		this.inherited(arguments);

		var selectedAttrId = this.attributeId;
		
		// setup dropdown menu
		// don't instantiate menu widget until button is clicked
		var self = this;
		this.configAttrMenu.toggleDropDown = function(){
			if(!this.dropDown){
				var menu = new jazz.ui.Menu();
				dojo.addClass(menu.domNode, "com-ibm-team-workitem-process");
				self.configAttrMenu = new dijit.MenuItem({
					label: messages.ShowConfigAttr,
					onClick: dojo.hitch(self, "_onConfigAttrMenuClick")
				});
				menu.addChild(self.configAttrMenu);
				this.dropDown = menu; // 'this' is dijit.form.DropDownButton widget
				menu.startup();
			}
			dijit._HasDropDown.prototype.toggleDropDown.apply(this, arguments);
			dojo.style(this.dropDown.domNode.parentNode, {
				zIndex: 1002
			});
		}
		
		// populate attributes
		this._populateAttrbiutes(true);
		
		var selectedKind = this.kind;
		
		// populate non-attribute-based kinds
		var nonAttrKindOptions = this.kindField_nonAttr.options;
		nonAttrKindOptions[0] = new Option(messages.None, "");
		dojo.forEach(this.nonAttrKinds, function(kindInfo, idx){
			var kind = kindInfo.kind;
			nonAttrKindOptions[idx+1] = new Option(kindInfo.label, kind);
			if(selectedKind && selectedKind.length>0 && kind == selectedKind){
				nonAttrKindOptions[idx+1].selected = true;
			}
		}, this);
		
		// initialize radio buttons
		var isAttrBased = (!this.isEdit || (selectedAttrId && selectedAttrId.length>0));
		var radioBtn = (isAttrBased) ? this.attrBasedRadio : this.nonAttrBasedRadio;
		radioBtn.checked = true;
		this._onRadioChanged(isAttrBased);
		
		this._populateAttrBasedKinds();
		
		// optional label/description/id
		var label = this.label;
		if(label && label.length>0){
			this.presentationLabel.value = label;
		}
		var desc = this.desc;
		if(desc && desc.length>0){
			this.presentationDesc.value = desc;
		}
		var presentationId = this.presentationId;
		if(presentationId && presentationId.length>0){
			this.presentationIdField.value = presentationId;
		}
		
		// properties table
		this._handleKindChanged();
	},

	_handleRadioChanged: function(e){
		var isAttrBased = (e.target == this.attrBasedRadio);
		this._onRadioChanged(isAttrBased);
	},
	
	_onRadioChanged: function(isAttrBased){
		this.attrField.disabled = !isAttrBased;
		var selectedAttrId = this.attrField.value;
		var isAttrSelected = (selectedAttrId && selectedAttrId.length>0);
		this.kindField_attr.disabled = (!isAttrBased || !isAttrSelected);
		this.kindField_nonAttr.disabled = isAttrBased;
		dojo.toggleClass(this.radioContainer, "attrBased", isAttrBased);
	},
	
	_handleAttrChanged: function(e){
		this.attributeId = this.attrField.value;
		this._populateAttrBasedKinds();
	},
	
	_populateAttrbiutes: function(/*boolean*/ doFilterConfigAttrs){
		var attributesOptions = this.attrField.options;
		attributesOptions.length = 0;
		attributesOptions[0] = new Option(messages.None, "");
		var selectedAttrId = this.attributeId;
		var filteredAttrs = this.attributes;
		
		// filter out attributes already used in this editor
		if(doFilterConfigAttrs){
			var usedAttrs = this.usedAttrs;
			var isEdit = this.isEdit;
			filteredAttrs = dojo.filter(filteredAttrs, function(attr){
				var attributeId = attr.stringIdentifier || attr.id;
				if(isEdit && attributeId == selectedAttrId){
					return true; // don't filter me out if edit presentation
				}else{
					return (dojo.indexOf(usedAttrs, attributeId) == -1);
				}
			});
		}

		var isFound = false;
		var num = attributesOptions.length;
		dojo.forEach(filteredAttrs, function(attrInfo, idx){
			var attrTypeId = attrInfo.typeId;
			var attributeId = attrInfo.stringIdentifier || attrInfo.id;
			if((
				(attrTypeId && attrKindMap[attrTypeId]) || // only add attributes that don't have corresponding kinds
				attrInfo.isEnumeration === true   // add if it is a enumeration type
				) && (dojo.indexOf(ignoreAttrs, attributeId) == -1)){ // don't add if it's included in ignore list ('Resolution', 'Start Date', 'Status' should not show up)
				attributesOptions[num] = new Option(attrInfo.name, attributeId);
				if(selectedAttrId && selectedAttrId.length>0 && attributeId == selectedAttrId){
					attributesOptions[num].selected = true;
					isFound = true;
				}
				num++;
			}
		}, this);
		
		if(!isFound){
			this.attributeId = null;
		}
	},
	
	_onConfigAttrMenuClick: function(){
		var configAttrMenu = this.configAttrMenu;
		var isConfigAttrFiltered = !dojo.hasClass(configAttrMenu.iconNode, "selectedItem");
		this._populateAttrbiutes(!isConfigAttrFiltered);
		dojo.toggleClass(configAttrMenu.iconNode, "selectedItem", isConfigAttrFiltered);
		this._populateAttrBasedKinds();
	},
	
	_populateAttrBasedKinds: function(){
		// clear the options
		var attributeId = this.attributeId;
		var kind = this.kind;
		var selectedKind = (kind && kind.length>0) ? kind : this.getKind();
		var kindField_attr = this.kindField_attr;
		var attrKindOptions = kindField_attr.options;
		attrKindOptions.length = 0;
		attrKindOptions[0] = new Option(messages.None, "");
		
		if(!attributeId || attributeId.length === 0){
			kindField_attr.disabled = true;
			return;
		}else{
			kindField_attr.disabled = false;
		}
		
		var attributes = this.attributes;
		var attrTypeId = null;
		for(var i=attributes.length-1; i>=0; i--){
			var targetId = attributes[i].stringIdentifier || attributes[i].id;
			if(targetId == attributeId){
				if(attributes[i].isEnumeration === true){
					attrTypeId = ENUM_TYPE;
				}else{
					attrTypeId = attributes[i].typeId;
				}
				break;
			}
		}
		if(attrTypeId){
			var kinds = attrKindMap[attrTypeId];
			if(kinds){
				kinds = kinds.sort(function(o1, o2){
					return basicComparator(o1.label.toLowerCase(), o2.label.toLowerCase());
				});
				dojo.forEach(kinds, function(kindInfo, idx){
					var kind = kindInfo.kind;
					attrKindOptions[idx+1] = new Option(kindInfo.label, kind);
					if(selectedKind && selectedKind.length>0 && kind == selectedKind){
						attrKindOptions[idx+1].selected = true;
					}
				}, this);
				var len = attrKindOptions.length;
				attrKindOptions[len] = new Option(messages.Hidden, HIDDEN_KIND);
				if(selectedKind && selectedKind.length>0 && selectedKind == HIDDEN_KIND){
					attrKindOptions[len].selected = true;
				}
			}
		}
	},
	
	getKind: function(){
		var isAttrBased = (this.attrBasedRadio.checked === true);
		var kindSelect = (isAttrBased) ? this.kindField_attr : this.kindField_nonAttr;
		return kindSelect.value;
	},
	
	_handleKindChanged: function(){
		var kind = this.getKind();
		var spec = kindPropSpecMap[kind];
		if(!spec){
			spec = (this.attrBasedRadio.checked === true) ? "DEFAULT_ATTR" : "DEFAULT";
		}
		var propertiesTable = this.propertiesTable;
		if(!kind || spec == "NONE"){
			propertiesTable.domNode.style.display = "none";
		}else{
			propertiesTable.propSpec = propSpecs[spec];
			propertiesTable.populateProperties(this.properties);
			propertiesTable.domNode.style.display = "";
		}
	},
		
	validate: function(values, callback){
		if(this.attrBasedRadio.checked === true){
			var attributeId = values.attributeId;
			if(!attributeId || attributeId.length === 0){
				alert(messages.ErrAttrIdEmpty);
				return;
			}
		}
		
		if(!values.kind || values.kind.length === 0){
			alert(messages.ErrKindEmpty);
			return;
		}
		
		callback();
	},
	
	getValues: function(){
		var ret = {
			kind: this.getKind()
		};
		if(this.attrBasedRadio.checked === true){
			ret.attributeId = this.attrField.value;
		}
		var label = this.presentationLabel.value;
		if(label && label.length>0){
			ret.label = label;
		}
		var desc = this.presentationDesc.value;
		if(desc && desc.length>0){
			ret.desc = desc;
		}
		var id = this.presentationIdField.value;
		if(id && id.length>0){
			ret.id = id;
		}

		var properties = this.propertiesTable.getValues();
		if(properties){
			ret.properties = properties;
		}

		return ret;
	},
	
	destroy: function(){
		if(this.configAttrMenu && this.configAttrMenu.dropDown){
			this.configAttrMenu.dropDown.destroyRecursive();
		}		
		this.inherited(arguments);
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent.openDialog = function(prop){
	// attribute-based presentations
	var model = prop.model;
	var builtInAttrs = model.getBuiltInAttributes();
	var customAttrs = model.getAllCustomAttributes();
	
	// non-attribute-based presentations
	if(!nonAttrKinds){
		var filteredMappings = dojo.filter(model.getKindMappings(), function(mapping){
			return (mapping.needsAttribute !== true);
		});
		nonAttrKinds = dojo.map(filteredMappings, function(mapping){
			var kind = mapping.kind;
			return {
				label: getKindLabel(kind),
				kind: kind
			};
		}).sort(function(o1, o2){
			return basicComparator(o1.label.toLowerCase(), o2.label.toLowerCase());
		});
	}
	
	var index = prop.index;
	var isEdit = (index !== null && index >= 0);
	var params = {
		isEdit: isEdit,
		nonAttrKinds: nonAttrKinds
	};
	if(isEdit){
		// edit presentation
		var presentations = model.getPresentations(prop.sectionId);
		if(presentations && presentations.length > index){
			var presentation = presentations[index];
			params.attributeId = presentation.attributeId;
			params.kind = presentation.kind;
			params.label = presentation.label;
			params.desc = presentation.description;
			params.presentationId = presentation.id; // 'id' may conflict with widget ID, so use presentationId
			params.properties = dojo.clone(presentation.properties);
		}
	}

	params.usedAttrs = model.getUsedAttributeIDs(prop.editorId);
	params.attributes = builtInAttrs.concat(customAttrs).sort(function(o1, o2){
		return basicComparator(o1.name.toLowerCase(), o2.name.toLowerCase());
	});

	//create add tab dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddPresentationContent(params);

	// show dialog
	var dlgTitleKey = isEdit ? "EditPresentationDlgTitle" : "AddPresentationDlgTitle";
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: messages[dlgTitleKey],
		dialogContent: contentWidget,
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			if(isEdit){
				// edit presentation
				model.editPresentation(prop.sectionId, values, index);
			}else{
				// add presentation
				model.addPresentation(prop.sectionId, values);
			}
		}
	});
};

})();