/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;
var ButtonDialog = com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/dialogs/templates/AddSectionContent.html"),
	
	isHeader: false,
	sectionIndex: null,
	sectionId: null,
	title: null,
	slot: null,
	slots: [],
	sectionIDs: [],
	properties: null,
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		if(this.title && this.title.length>0){
			this.sectionTitle.value = this.title;
		}
		
		// don't show title row if this is add header section dialog
		if(this.isHeader){
			this.titleRow.style.display = "none";
		}
		
		// populate slots
		if(this.slots && this.slots.length > 0){
			var slotOptions = this.sectionSlot.options;
			dojo.forEach(this.slots, function(slot, idx){
				slotOptions[idx] = new Option(LayoutUtil.getSlotLabel(slot), LayoutUtil.getSlotValue(slot));
				if(this.slot && this.slot.length>0 && slot == this.slot){
					slotOptions[idx].selected = true;
				}
			}, this);
		}else{
			this.slotRow.style.display = "none";
		}

		if(this.sectionIndex !== null && this.sectionIndex >= 0){
			// edit section
			this.sectionIdRow.style.display = "";
			this.sectionIdFieldSet.style.display = "none";
		}else{
			// add section
			// populate existing section IDs
			var sectionIdOptions = this.reuseSectionID.options;
			dojo.forEach(this.sectionIDs, function(sectionID, idx){
				sectionIdOptions[idx+1] = new Option(sectionID, sectionID);
			});
				
			this.newSectionIdRadio.checked = true;
			this._onRadioChanged(true);
			
			this.sectionIdRow.style.display = "none";
			this.sectionIdFieldSet.style.display = "";
		}
		
		// properties table
		this.propertiesTable.propSpec = {
			noHeader: [ "true", "false" ],
			autocollect: [ "true", "false" ],
			width: [ "small", "wide" ],
			expanded: [ "true", "false" ]
		};
		this.propertiesTable.populateProperties(this.properties);
		
	},

	_handleRadioChanged: function(e){
		var isNewID = (e.target == this.newSectionIdRadio);
		this._onRadioChanged(isNewID);
	},
	
	_onRadioChanged: function(isNewID){
		this.newSectionID.readOnly = !isNewID;
		this.reuseSectionID.disabled = isNewID;
	},
	
	validate: function(values, callback){
		if(!values.title || values.title.length === 0){
			alert(messages.ErrSectionTitleEmpty);
			return;
		}
		
		if(!values.id || values.id.length === 0){
			alert(messages.ErrSectionIDEmpty);
			return;
		}
	
		callback();
	},
	
	getValues: function(){
		var ret = {
			title: this.sectionTitle.value,
			slot: this.sectionSlot.value,
			properties: this.propertiesTable.getValues()
		};
		if(this.sectionIndex !== null && this.sectionIndex >= 0){
			ret.sectionIndex = this.sectionIndex;
			ret.id = this.sectionId;
		}else{
			ret.id = (this.newSectionIdRadio.checked === true) ? this.newSectionID.value : this.reuseSectionID.value;
		}
		return ret;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openDialog = function(obj){
	var tabId = obj.tabId;
	var model = obj.model;
	var layout = model.getTabLayout(tabId);
	
	//create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent({
		slots: LayoutUtil.getAvailableSlots(layout),
		sectionIDs: dojo.map(model.getAllSections(), function(sectionInfo){
			return sectionInfo.id;
		}).sort()
	});

	// show dialog
	new ButtonDialog({
		primaryTitle: messages.AddSectionDialogTitle,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.sectionTitle);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: dojo.hitch(this, function(values){
			model.addSection(tabId, values.id, {
				title: values.title,
				slot: values.slot,
				properties: values.properties
			});
		})
	});
};

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openEditDialog = function(obj){
	var tabId = obj.tabId;
	var model = obj.model;

	var layout = model.getTabLayout(tabId);
	var sectionIndex = obj.sectionIndex;
	
	//create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent({
		slots: LayoutUtil.getAvailableSlots(layout),
		isHeader: obj.isHeader,
		sectionIndex: sectionIndex,
		sectionId: model.getSectionProperty(tabId, sectionIndex, "sectionId"),
		title: model.getSectionProperty(tabId, sectionIndex, "title"),
		slot: model.getSectionProperty(tabId, sectionIndex, "slot"),
		properties: model.getSectionProperty(tabId, sectionIndex, "properties")
	});

	// show dialog
	new ButtonDialog({
		primaryTitle: messages.EditSectionDialogTitle,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.sectionTitle);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: dojo.hitch(this, function(values){
			model.editSection(tabId, values.sectionIndex, {
				title: values.title,
				slot: values.slot,
				properties: values.properties
			});
		})
	});
};

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent.openAddHeaderDialog = function(obj){
	var model = obj.model;

	//create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddSectionContent({
		isHeader: true,
		sectionIDs: dojo.map(model.getAllSections(), function(sectionInfo){
			return sectionInfo.id;
		}).sort()
	});

	// show dialog
	new ButtonDialog({
		primaryTitle: messages.AddHeaderSectionDialogTitle,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.newSectionID);
		},
		validate: function(values, callback){
			if(!values.id || values.id.length === 0){
				alert(messages.ErrSectionIDEmpty);
				return;
			}
			callback();
		},
		onOK: dojo.hitch(this, function(values){
			model.addHeader(obj.editorId, values.id, values.properties);
		})
	});
};

})();