/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent",
	[ dijit._Widget, dijit._Templated ],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/dialogs/templates/AddTabContent.html"),
	
	tabs: [],
	tabId: null,
	title: null,
	layout: null,
	properties: null,
		
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.inherited(arguments);
	},
	
	postCreate: function(){
	
		this.inherited(arguments);
		
		if(this.tabId){
			// for edit tab
			if(this.title && this.title.length>0){
				this.tabTitle.value = this.title;
			}
			if(this.layout && this.layout.length>0){
				this.tabLayout.value = this.layout;
			}
			this.tabIdField.value = this.tabId;
			this.tabIdRow.style.display = "";
			this.tabIdFieldSet.style.display = "none";
		}else{
			// for add tab

			// Populate reuse existing tab dropdown
			if(this.tabs){
				var options = this.reuseTabID.options;
				dojo.forEach(this.tabs, function(tab, idx){
					options[idx+1] = new Option(tab, tab);
				});
			}
				
			this.newTabIdRadio.checked = true;
			this._onRadioChanged(true);
			this.tabIdRow.style.display = "none";
			this.tabIdFieldSet.style.display = "";
		}

		// update properties table
		this._handleLayoutChanged();
	},
	
	_handleRadioChanged: function(e){
		var isNewID = (e.target == this.newTabIdRadio);
		this._onRadioChanged(isNewID);
	},

	_onRadioChanged: function(isNewID){
		this.newTabID.readOnly = !isNewID;
		this.reuseTabID.disabled = isNewID;
	},
	
	_handleLayoutChanged: function(){
		var propertiesTable = this.propertiesTable;
		if(this.tabLayout.value == "builtInCustomAttributesLayout"){
			// tab properties are supported only for Custom Attributes Layout
			// see http://jazz.net/library/article/130#hideIfEmptyTab
			if(!propertiesTable){
				propertiesTable = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable({
					propSpec: {
						hideIfEmpty: [ "true", "false" ]
					}
				});
				this.propertiesTable = propertiesTable;
				this.domNode.appendChild(propertiesTable.domNode);
				dojo.style(propertiesTable.domNode, {
					width: "100%",
					marginTop: "10px"
				});
			}
			propertiesTable.domNode.style.display = "";
			propertiesTable.populateProperties(this.properties);
		}else{
			if(propertiesTable){
				propertiesTable.clearItems();
				propertiesTable.domNode.style.display = "none";
			}
		}
	},
	
	validate: function(values, callback){
		var id = values.id;
		if(!id || id.length === 0){
			alert(this.messages.ErrEmptyTabID);
			return;
		}

		var title = values.title;
		if(!title || title.length === 0){
			alert(this.messages.ErrEmptyTabTitle);
			return;
		}
		
		var layout = values.layout;
		if(!layout || layout.length === 0){
			alert(this.messages.ErrEmptyTabLayout);
			return;
		}
		
		callback();
	},
	
	getValues: function(){
		var ret = {
			title: this.tabTitle.value,
			layout: this.tabLayout.value
		};
		if(this.tabId){
			// for edit tab
			ret.id = this.tabId;
		}else{
			// for add tab
			ret.id = (this.newTabIdRadio.checked === true) ? this.newTabID.value : this.reuseTabID.value;
		}
		if(this.propertiesTable){
			ret.properties = this.propertiesTable.getValues();
		}
		return ret;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent.openDialog = function(prop){
	var model = prop.model;
	var editorId = prop.editorId;

	//create add tab dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent({
		tabs: dojo.map(model.getAllTabs(), function(tabInfo){
			return tabInfo.id;
		}).sort() // for reusing existing tabs
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: contentWidget.messages.AddTabDialogTitle,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.tabTitle);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			model.addTab(editorId, values.id, {
				layout: values.layout,
				title: values.title,
				properties: values.properties
			});
		}
	});
};

com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent.openEditDialog = function(prop){
	var model = prop.model;
	var editorId = prop.editorId;
	var tabIndex = prop.tabIndex;

	//create add tab dialog content
	var tabId = model.getTabProperty(editorId, tabIndex, "tabId");
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.AddTabContent({
		tabId: tabId,
		title: model.getTabProperty(editorId, tabIndex, "title"),
		layout: model.getTabLayout(tabId),
		properties: model.getTabProperty(editorId, tabIndex, "properties")
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: contentWidget.messages.EditTabDialogTitle,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.tabTitle);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			model.editTab(editorId, tabIndex, {
				layout: values.layout,
				title: values.title,
				properties: values.properties
			});
		}
	});
};