/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var CLASS_DELETE_COMMAND = "delete-command";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.dialogs.PropertiesTable",
	[ dijit._Widget, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/dialogs/templates/PropertiesTable.html"),
	
	properties: null,
	propSpec: null,
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		var self = this;
		var table = this._table;
		table.setupColumns([
			{
				id: "key",
				name: messages.Key,
				style: {
					width: "40%",
					paddingLeft: "7px"
				},
				getCellContent: function(cellData){
					// create options
					var sel = dojo.create("select");
					var options = sel.options;
					if(!cellData || cellData.length === 0){
						options[0] = new Option(messages.SelectPropKey, "");
					}
					var len = options.length;
					var bFound = false;
					var opt;
					dojo.forEach(self._getAvailableKeys(cellData), function(availableKey, idx){
						opt = new Option(availableKey, availableKey);
						options[idx+len] = opt;
						if(availableKey == cellData){
							opt.selected = true;
							bFound = true;
						}
					});
					// 125677: Customization Web UI: If unknown property is specified, Edit Presentation dialog shows invalid value
					if(!bFound && cellData && cellData.length>0){
						opt = new Option(cellData, cellData);
						options[options.length] = opt;
						opt.selected = true;
					}
					
					// listen to onchange event
					var rowWidget = this; // 'this' is a Row widget
					rowWidget.connect(sel, "onchange", function(){
						var index = dojo.indexOf(table.getChildren(), rowWidget);
						if(index >= 0){
							self.properties[index].key = sel.value;
							self.properties[index].value = ""; // clear the value
							self.populateProperties(self.properties);
						}
					});
					return sel;
				}
			},
			{
				id: "value",
				name: messages.Value,
				style: {
					paddingLeft: "7px"
				},
				getCellContent: function(cellData){
					var values = self.propSpec[cellData.key] || [];
					var rowWidget = this; // 'this' is a Row widget
					var selectedValue = cellData.value || "";
					if(values.length === 0){
						var inp = dojo.create("input", {
							type: "text",
							value: selectedValue,
							style: {
								width: "95%"
							}
						});
						
						rowWidget.connect(inp, "onchange", function(){
							var index = dojo.indexOf(table.getChildren(), rowWidget);
							if(index >= 0){
								self.properties[index].value = inp.value;
							}
						});
						
						return inp;
					}else{
						var sel = dojo.create("select", {
							style: {
								width: "100%"
							}
						});
						if(selectedValue.length === 0){
							sel.options[0] = new Option(messages.SelectPropKey, "");
						}
						var start = sel.options.length;
						var opt;
						dojo.forEach(values, function(value, idx){
							opt = new Option(value, value);
							if(selectedValue == value){
								opt.selected = true;
							}
							sel.options[idx+start] = opt;
						});
						
						rowWidget.connect(sel, "onchange", function(){
							var index = dojo.indexOf(table.getChildren(), rowWidget); 
							if(index >= 0){
								self.properties[index].value = sel.value;
							}
						});
						
						return sel;
					}
				}
			},
			{
				id: "actions",
				name: messages.Actions,
				style: {
					width: "60px",
					paddingLeft: "7px"
				},
				isAction: true
			}
		]);
		if(this.properties){
			this.populateProperties(this.properties);
		}
	},
	
	_getAvailableKeys: function(selectedKey){
		var usedKeys = (this.properties) ? dojo.map(this.properties, function(property){ return property.key; }) : [];
		var availableKeys = [];
		for(var key in this.propSpec){
			if(key == selectedKey || dojo.indexOf(usedKeys, key) === -1){
				availableKeys.push(key);
			}
		}
		return availableKeys;
	},
	
	populateProperties: function(properties){
		this.clearItems();
		this.properties = properties;
		dojo.forEach(properties, dojo.hitch(this, "_addPropertyRow"));
	},
	
	_addPropertyRow: function(property){
		var self = this;
		var key = property.key;
		var table = this._table;
		table.addItem({
			key: key,
			value: property,
			actions: [{
				iconClass: CLASS_DELETE_COMMAND,
				title: messages.RemoveProperty,
				onClick: function(){
					if(key && key.length > 0){
						if(!confirm(dojo.string.substitute(messages.ConfirmRemoveProp, [ key ]))){
							return;
						}
					}
					var index = dojo.indexOf(table.getChildren(), this); // 'this' is a Row widget
					if(index >= 0){
						self.properties.splice(index, 1);
						self.populateProperties(self.properties);
					}
				}
			}]
		});
	},
	
	_handleAddProperty: function(e){
		dojo.stopEvent(e);
		
		var len = this._getAvailableKeys().length;
		var values = this.getValues() || [];
		if(len === 0 || len <= this._table.getChildren().length - values.length){
			alert(messages.NoPropAvailable);
			return;
		}
		
		var property = {};
		if(!this.properties){
			this.properties = [];
		}
		this.properties.push(property);
		this._addPropertyRow(property);
	},
	
	clearItems: function(){
		this.properties = null;
		this._table.clearItems();
	},
	
	getValues: function(){
		if(this.properties){
			return dojo.filter(this.properties, function(property){
				var key = property.key;
				var val = property.value;
				return (key && key.length>0 && val && val.length>0);
			});
		}
		return null;
	}
});

})();