/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DetailsSectionContent");

dojo.require("dijit._Templated");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent");

(function(){

var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;
var getCell = com.ibm.team.workitem.web.process.ui.internal.view.common.Row.prototype.getCell;
var _isColSpan = function(presentation){
	return (LayoutUtil.isHiddenKind(presentation) || // "Hidden" kind, so don't add label cell
		(presentation && presentation.widget && presentation.widget.isSpanningColumns));
};

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DetailsSectionContent",
	[ com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/layout/templates/DetailsSectionContent.html"),
	
	postCreate: function(){
		this.inherited(arguments);
		
		// setup left/right table
		var columns = [
			{
				id: "label_data",
				className: "Column LabelColumn",
				getCell: function(presentation){
					if(_isColSpan(presentation)){
						return null;
					}else{
						return getCell.apply(this, arguments);
					}
				},
				getCellContent: function(presentation){
					var displayName = presentation.label || presentation.displayName;
					if(displayName && displayName.length>0){
						var labelSpan = dojo.create("span", {
							className: "formLabel"
						});
						LayoutUtil.addLabel(labelSpan, displayName);
						return labelSpan;
					}else if(presentation.widget && presentation.widget.htmlLabel){ // special case. displayName is empty
						return dojo.create("div", {
							className: "formLabel",
							innerHTML: presentation.widget.htmlLabel
						});
					}
					return null;
				}
			},
			{
				id: "value_data",
				className: "Column ValueColumn2",
				getCell: function(presentation){
					if(_isColSpan(presentation)){
						return dojo.create("td", { colSpan:2 });
					}else{
						return getCell.apply(this, arguments);
					}
				},
				getCellContent: function(presentation){
					if(presentation.widget) {
						// render teamarea widget in multiple row only for "Details" slot
						if(presentation.kind == "com.ibm.team.workitem.kind.internal.teamarea"){
							presentation.widget.widgetParams = {
								isSingleRow: false
							};
						}
						var valueDiv = dojo.create("div");
						
						// if span columns, add label in the same cell
						if(presentation.widget.isSpanningColumns){
							var displayName = presentation.label || presentation.displayName;
							if(displayName && displayName.length>0){
								var labelSpan = dojo.create("span", {
									className: "formLabel"
								}, valueDiv);
								LayoutUtil.addLabel(labelSpan, displayName);
								dojo.create("br", null, valueDiv);
							}
						}
						
						LayoutUtil.addPresentation(valueDiv, presentation);
						return valueDiv;
					}else{
						return dojo.create("SPAN", {
							innerHTML: "&nbsp;"
						});
					}
				}
			}
		];
		this._leftWidgetTable.setupColumns(columns);
		this._rightWidgetTable.setupColumns(columns);
	},
	
	addPresentations: function(presentations){
		this._widgets = [];
		var leftTable = this._leftWidgetTable;
		var rightTable = this._rightWidgetTable;
		var num = Math.floor(presentations.length/2);
		for(var i=0, len=presentations.length; i<len; i++){
			var presentation = presentations[i];
		
			// determine which column to add
			var isLeft = (i <= num);
			var tableWidget = isLeft ? leftTable : rightTable;

			var rowWidget = tableWidget.addItem({
				label_data: presentation,
				value_data: presentation
			});
			this._widgets.push(rowWidget);
		}
		
		return this._widgets;
	},
	
	getDraggableContainers: function(){
		var dndType = this.dndType;
		if(!this._leftDraggable){
			this._leftDraggable = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer(this._leftWidgetTable.containerNode, {
				accept: [dndType]
			});
			this._leftDraggable.avatarClass = "com-ibm-team-workitem com-ibm-team-workitem-process";
		}
		if(!this._rightDraggable){
			this._rightDraggable = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer(this._rightWidgetTable.containerNode, {
				accept: [dndType]
			});
			this._rightDraggable.avatarClass = "com-ibm-team-workitem com-ibm-team-workitem-process";
		}
		return [ this._leftDraggable, this._rightDraggable ];
	}
});

})();