/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent");

dojo.require("dijit._Widget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer");

(function(){

var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent",
	dijit._Widget,
{
	title: null,
	dndType: "presentation",
	
	postCreate: function(){
		this._widgets = [];
		this.inherited(arguments);
		if(!this.containerNode){
			this.containerNode = this.domNode;
		}
	},
	
	addPresentations: function(presentations){
		this._widgets = [];
		var sectionTitle = this.title;
		var containerNode = this.containerNode;
		var len=presentations.length;
		for(var i=0; i<len; i++){
			var borderNode = dojo.create("div", {
				className: "BottomPadded"
			});
			var widget = new dijit._Widget({}, borderNode);
			widget.containerNode = widget.domNode; // needed so that presentationWidget gets destroyed when 'widget' is destroyed
			var widgetNode = dojo.create("div", {}, borderNode);
			containerNode.appendChild(borderNode);
			this.addPresentation(widgetNode, presentations[i], i, len);
			this._widgets.push(widget);
		}
		return this._widgets;
	},
	
	// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout#_renderColumnSection
	addPresentation: function(containerNode, presentation, index, len){
		var label = presentation.label || presentation.displayName;
		var sectionTitle = this.title;
		if(LayoutUtil.isHiddenKind(presentation) ||
			(!label || label.length===0) || // no label
			len == 1 || // don't show label if there's only one presentation in this section see modifiers.presentationCount in com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout
			(index===0 && sectionTitle && sectionTitle.length>0 && sectionTitle == presentation.displayName)){
			// don't show label if its the first in the section and the label is the same as the heading label
			LayoutUtil.addPresentation(containerNode, presentation);
		}else{
			// name-value pair
			var table = dojo.create("table", {
				style: {
					width: "100%"
				},
				className: "com-ibm-team-workitem-namevalue-table"
			});
			var tbody = dojo.create("tbody", {}, table);
			var tr = dojo.create("tr", {
				className: "com-ibm-team-workitem-namevalue-tr"
			}, tbody);
			var td_label = dojo.create("td", {
				style: {
					width: "100px"
				},
				className: "com-ibm-team-workitem-namevalue-label-td"
			}, tr);
			
			var div_value = dojo.create("div", {
				className: "com-ibm-team-workitem-namevalue-label-node"
			}, td_label);
			var label = presentation.label || presentation.displayName;
			LayoutUtil.addLabel(div_value, label);
			var td_value = dojo.create("td", {
				className: "com-ibm-team-workitem-namevalue-value-td"
			}, tr);
			LayoutUtil.addPresentation(td_value, presentation);
			containerNode.appendChild(table);
		}
	},
		
	getPresentations: function(){
		return this._widgets;
	},
	
	clearPresentations: function(){
		dojo.forEach(this.getDraggableContainers(), function(draggableContainer){
			for(var i=draggableContainer.widgets.length-1; i>=0; i--){
				var widget = draggableContainer.widgets[i];
				draggableContainer.remove(i);
				widget.destroy();
			}
		});
	},
	
	getDraggableContainers: function(){
		// sub-class can override
		if(!this._draggable){
			var draggableContainer = new com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer(this.containerNode, {
				accept: [this.dndType]
			});
			draggableContainer.avatarClass = "com-ibm-team-workitem com-ibm-team-workitem-process";
			this._draggable = draggableContainer;
		}
		return [ this._draggable ];
	},
	
	makeDraggable: function(){
		var draggableContainers = this.getDraggableContainers();
		var containerNodes = dojo.map(draggableContainers, function(draggableContainer){
			return draggableContainer.node;
		});
		var dndType = this.dndType;
		dojo.forEach(this.getPresentations(), function(presentationWidget){
			var index = dojo.indexOf(containerNodes, presentationWidget.domNode.parentNode);
			if(index != -1){
				draggableContainers[index].add(presentationWidget, { type:dndType });
			}
		});
	},
	
	destroy: function(preserveDom){
		dojo.forEach(this.getPresentations(), function(presentationWidget){
			presentationWidget.destroyRecursive(preserveDom);
		});
		dojo.forEach(this.getDraggableContainers(), function(draggableContainer){
			draggableContainer.destroy(preserveDom);
		});
		this.inherited(arguments);
	}
});

})();