/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderSectionContent");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;
var iconContext = "com.ibm.team.workitem.web";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.HeaderSectionContent",
	[ com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent, dijit._Templated ],
{
	// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/layout/templates/HeaderSectionContent.html"),

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.containerNode = this.headerRight;
		
		this.inherited(arguments);
		
		// add extension registry toolbar icons
		var simpleToolbarContainer = this._simpleToolbarContainer;
		var addIcon = function(url){
			dojo.create("img", {
				className: "headerToolbarIcon",
				src: url.toString()
			}, simpleToolbarContainer);
		};
		var extensionActions = com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry.getContributedEditorToolbarActions();
		if(extensionActions.length > 0) {
			dojo.forEach(extensionActions, function(actionSpec){
				var iconUrl = dojo.moduleUrl(actionSpec.iconContext, actionSpec.iconUri);
				addIcon(iconUrl);
			});
			dojo.create("div", {
				className: "dijitInline SimpleToolbar-separator"
			}, simpleToolbarContainer);
		}
		
		// add default toolbar icons
		dojo.forEach([
			dojo.moduleUrl(iconContext, "ui/internal/view/editor/images/find-duplicates.gif"),
			dojo.moduleUrl(iconContext, "ui/internal/view/editor/images/subscribe.gif"),
			dojo.moduleUrl(iconContext, "ui/internal/view/editor/images/move-copy.gif"),
			dojo.moduleUrl(iconContext, "ui/internal/view/editor/images/refresh.gif")
		], addIcon);
	},
	
	_removeSummary: function(){
		var summaryWidget = this.summaryWidget;
		if(summaryWidget){
			this.headerSummary.removeChild(summaryWidget.domNode);
			summaryWidget.destroy();
			this.summaryWidget = null;
		}
	},
	
	clearPresentations: function(){
		// 'summary' is a special case, and won't be destroyed in "inherited"
		this._removeSummary();
		this.inherited(arguments);
	},
	
	addPresentations: function(presentations){
		this._widgets = [];
		var isLeft = true;
		var len=presentations.length;
		for(var i=0; i<len; i++){
			var widget = null;
			var label = presentations[i].label || presentations[i].displayName;
			if(presentations[i].attributeId == "com.ibm.team.workitem.attribute.summary"){
				// name-value pair
				var table = dojo.create("table", {
					style: {
						width: "100%"
					},
					className: "com-ibm-team-workitem-namevalue-table"
				});
				var tbody = dojo.create("tbody", {}, table);
				var tr = dojo.create("tr", {
					className: "com-ibm-team-workitem-namevalue-tr"
				}, tbody);
				var td_label = dojo.create("td", {
					style: {
						width: "100px"
					},
					className: "com-ibm-team-workitem-namevalue-label-td"
				}, tr);
				
				var div_value = dojo.create("span", {
					className: "com-ibm-team-workitem-namevalue-label-node"
				}, td_label);
				// com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader does not use attributeLabel, so we follow that too
				LayoutUtil.addLabel(div_value, messages.labelSummary);
				dojo.create("span", {
					className: "requiredFlag",
					innerHTML: "*"
				}, td_label);
				
				var td_value = dojo.create("td", {
					className: "com-ibm-team-workitem-namevalue-value-td"
				}, tr);
				dojo.create("input", {
					type: "text",
					style: {
						width: "100%"
					}
				}, td_value);
				widget = new dijit._Widget();
				this.summaryWidget = widget;
				widget.domNode.appendChild(table);				
				this.headerSummary.appendChild(widget.domNode);
				isLeft = false;
			}else if(presentations[i].kind == "com.ibm.team.workitem.kind.separator"){
				widget = new dijit._Widget();
				dojo.style(widget.domNode, {
					width: "3px",
					height: "15px"
				});
			}else{
				var targetDiv = isLeft ? this.headerLeft : this.headerRight;
				var attrDiv = dojo.create("div", {
					className: "dijitInline"
				}, targetDiv);
				var labelDiv = dojo.create("div", {
					className: "attr-label"
				}, attrDiv);
				if(label){
					LayoutUtil.addLabel(labelDiv, label);
				}
				var valueDiv = dojo.create("div", {
					className: "attr-value"
				}, attrDiv);
				var presentationWidget = LayoutUtil.addPresentation(valueDiv, presentations[i]);
				dojo.style(presentationWidget.domNode, {
					width: "100%"
				});
				widget = new dijit._Widget({}, attrDiv);
				widget.containerNode = widget.domNode; // needed so that presentationWidget gets destroyed when 'widget' is destroyed
				dojo.style(widget.domNode, {
					width: "12em"
				});
			}
			if(widget){
				this._widgets.push(widget);
			}
		}
		return this._widgets;
	},

	destroy: function(preserveDom){
		// 'summary' is a special case, and won't be destroyed in "inherited"
		this._removeSummary();
		this.inherited(arguments);
	}
});

})();