/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");

dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var getI18nMessage = function(key){
	if(!messages){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
	}
	return messages[key];
};
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;
var CLASS_HOVER = "com-ibm-team-workitem-process-presentationHover";

var tabSpec = {
	builtInOverviewLayout: [ "none", "details", "description", "discussion", "quickInfo" ], // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview
	builtInLinksLayout: [ "defaultSection", "attachments", "links", "subscribers" ], // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout
	builtInApprovalsLayout: [ "defaultSection" ], // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout
	builtInCustomAttributesLayout: [ "none", "left", "right" ], // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview
	builtInHLayout: [ "none", "top", "left", "right", "bottom" ], // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview
	builtInHistoryLayout: [ "defaultSection" ] // com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout
};

var GENERIC_SECTION = "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.GenericSectionContent";
var sectionSpec = {
	// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview#sectionMap
	details: {
		column: "left",
		contentWidget: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DetailsSectionContent"
	},
	linksSummary: { // Quick Information
		column: "right",
		contentWidget: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.QuickInfoSectionContent"
	},
	quickInfo: {  // Quick Information
		column: "right",
		contentWidget: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.QuickInfoSectionContent"
	},
	description: {
		column: "left",
		contentWidget: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DescriptionSectionContent"
	},
	discussion: {
		column: "left",
		contentWidget: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DescriptionSectionContent"
	},
		
	// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout
	defaultSection: {
		column: "single", // single, left or right
		contentWidget: GENERIC_SECTION,
		value: ""
	},
	links: {
		column: "left", // single, left or right
		contentWidget: GENERIC_SECTION
	},
	attachments: {
		column: "left", // single, left or right
		contentWidget: GENERIC_SECTION
	},
	subscribers: {
		column: "right", // single, left or right
		contentWidget: GENERIC_SECTION
	},
	
	none: {
		value: ""
	}
};
// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor#tabSpecs.builtInHLayout.slotMap
// see com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor#_renderCustomAttributes
sectionSpec.left = sectionSpec.details; // 'left' will be mapped to 'details' left in "H layout" or "Custom Attributes" layout
sectionSpec.right = sectionSpec.description; // 'right' will be mapped to 'description' left in "H layout" or "Custom Attributes" layout
sectionSpec.top = sectionSpec.quickInfo; // 'top' will be mapped to 'quickInfo' left in "H layout"
sectionSpec.bottom = sectionSpec.discussion; // 'bottom' will be mapped to 'discussion' left in "H layout"

var HIDDEN_KIND = "com.ibm.team.workitem.kind.special.doNotShow";

com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil =
{
	HIDDEN_WIDGET: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.HiddenWidget",

	getAvailableSlots: function(layout){
		return (layout && tabSpec[layout]) ? tabSpec[layout] : [];
	},
	
	getSlotLabel: function(slot){
		return getI18nMessage("slot_"+slot);
	},
	
	getSlotValue: function(slot){
		// returns "slot" value to be saved to process XML
		return sectionSpec[slot].value || slot;
	},
	
	getSectionLayoutColumn: function(layout, slot){
		if(slot && dojo.indexOf(this.getAvailableSlots(layout), slot) != -1){
			var slotInfo = sectionSpec[slot];
			if(slotInfo){
				return slotInfo.column;
			}
		}
		return null;
	},
	
	createSectionContent: function(layout, slot, title){
		var className = null;
		// get content widget class name
		if(dojo.indexOf(this.getAvailableSlots(layout), slot) != -1){
			// check if slot is supported in this tab layout. If not, use generic section content
			var slotInfo = sectionSpec[slot];
			if(slotInfo){
				className = slotInfo.contentWidget;
			}
		}

		if(!className){
			className = GENERIC_SECTION;
		}
		jazz.core.loader.load_sync(className);
		var cls = dojo.getObject(className);
		var params = {};
		if(title && title.length>0){
			params.title = title;
		}
		return new cls(params, dojo.create("div"));
	},
	
	isHiddenKind: function(presentation){
		return (presentation && presentation.kind == HIDDEN_KIND);
	},

	addPresentation: function(containerNode, presentation){
		var widgetInfo = presentation.widget;
		var className = widgetInfo.widgetClass;
		jazz.core.loader.load_sync(className);
		var cls = dojo.getObject(className);
		var params = widgetInfo.widgetParams || {};
		if(presentation.properties){
			params.properties = presentation.properties;
		}
		var widget = new cls(params);
		containerNode.appendChild(widget.domNode);
		dojo.addClass(containerNode, "presentationValue");
		return widget;
	},

	addLabel: function(containerNode, label, /*boolean?*/ noColon){
		var convertedLabel = (noColon !== true) ? dojo.string.substitute(getI18nMessage("LabelValue"), [ label ]) : label;
		var textNode = dojo.doc.createTextNode(convertedLabel);
		dojo.addClass(containerNode, "presentationLabel");
		containerNode.appendChild(textNode);
	},
	
	getProperty: function(properties, key){
		if(properties){
			for(var i=properties.length-1; i>=0; i--){
				if(properties[i].key == key){
					return properties[i].value;
				}
			}
		}
		return null;
	},
	
	bindActionPopup: function(obj){
		var widget = obj.widget;
		var owner = obj.owner;
		
		widget.connect(widget.domNode, "onmouseover", function(e){
			dojo.toggleClass(widget.domNode, CLASS_HOVER, true);
		
			var _popup = owner._popup;
			if(!_popup){
				_popup = new dijit._Widget();
				owner._popup = _popup;
				_popup.connect(owner, "destroy", function(){
					_popup.destroy();
				});
				dojo.addClass(_popup.domNode, CLASS_HOVER);
				dojo.style(_popup.domNode, {
					padding: "5px",
					height: "20px"
				});
				var wrapper = dojo.create("div", {
					className: "com-ibm-team-workitem-process"
				}, _popup.domNode);

				dojo.forEach(obj.actions, function(action){
					wrapper.appendChild(createActionIcon({
						iconClass: action.iconClass,
						onClick: function(){
							action.onClick(_popup._targetWidget);
						},
						title: action.title,
						owner: owner
					}));
				});
				
				
				// keep hover class while hovering over popup
				owner.connect(_popup.domNode, "onmouseout", function(e){
					if(!dojo.isDescendant(e.relatedTarget, _popup.domNode)){
						var targetWidget = _popup._targetWidget;
						if(targetWidget && !dojo.isDescendant(e.relatedTarget, targetWidget.domNode)){
							dojo.toggleClass(targetWidget.domNode, CLASS_HOVER, false);
							_popup._targetWidget = null;
							dijit.popup.close(_popup);
						}
					}
				});
			}
			
			_popup._targetWidget = widget;
			
			// change popup position to new around widget
			dijit.popup.open({
				popup: _popup,
				parent: owner,
				around: widget.domNode,
				orient: {"TR":"TL"}
			});
		});
		widget.connect(widget.domNode, "onmouseout", function(e){
			var _popup = owner._popup;
			if(_popup && dojo.isDescendant(e.relatedTarget, _popup.domNode)){
				// don't hide popup when mouseout to popup itself
				// see dojo._base.event for event.relatedTarget
				_popup._targetWidget = widget;
				dojo.toggleClass(widget.domNode, CLASS_HOVER, true);
			}else{
				// otherwise hide popup
				dijit.popup.close(_popup);
				_popup._targetWidget = null;
				dojo.toggleClass(widget.domNode, CLASS_HOVER, false);
			}
		});
	}
};

})();