/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.TabContent");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dijit._Widget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var LayoutUtil = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.LayoutUtil;
var DraggableContainer = com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.DraggableContainer;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.presentation.layout.TabContent",
	dijit._Widget,
{
	// summary:
	//		Tab content widget that contains sections
	//		TwoColumnLayout widget is used for layouting of sections 
	//		and drag & drop is supported
	
	dndType: "section",

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
		this.inherited(arguments);
	},

	postCreate: function(){
		this._slots = {};
		this._sectionLayouts = [];
		this.inherited(arguments);

		var columnLayout = new com.ibm.team.workitem.web.ui.internal.view.layout.TwoColumnLayout();
		this.domNode.appendChild(columnLayout.domNode);
		this.columnLayout = columnLayout;		
	},
	
	clearSections: function(){
		this._sectionLayouts = [];
		for(var slot in this._slots){
			var draggableContainer = this._slots[slot];
			for(var i=draggableContainer.widgets.length-1; i>=0; i--){
				var widget = draggableContainer.widgets[i];
				draggableContainer.remove(i);
				widget.destroy();
			}
		}
	},
	
	_clearSlots: function(){
		for(var slot in this._slots){
			var draggableContainer = this._slots[slot];
			var node = draggableContainer.node.parentNode;
			node.parentNode.removeChild(node);
			draggableContainer.destroy();
			delete this._slots[slot];
		}
		this._slots = {};
	},
	
	initSlots: function(/*String*/ layout){
		// remove sections
		this.clearSections();
	
		// re-create slots if layout changed
		if(layout && layout != this._layout){
			this._clearSlots();
			
			// add slots
			var columnLayout = this.columnLayout;
			var hiddenSlots = [];
			var slots = dojo.clone(LayoutUtil.getAvailableSlots(layout));
			for(var i=slots.length-1; i>=0; i--){
				// move "none" slots to last
				if(slots[i] == "none"){
					hiddenSlots.push(slots[i]);
					slots.splice(i, 1);
				}
			}
			dojo.forEach(slots.concat(hiddenSlots), function(slot){
				var column = LayoutUtil.getSectionLayoutColumn(layout, slot);
				var isHiddenSlot = false;
				if(!column){
					// if slot="none"
					column = "single";
					isHiddenSlot = true;
				}
				var containerDiv = (column=="single" || column=="left") ? columnLayout._leftDIV : columnLayout._rightDIV;
				var slotDiv = dojo.create("div", {}, containerDiv);
				if(slot != "defaultSection"){
					dojo.addClass(slotDiv, "sectionSlot");

					var label = (slot == "none") ? messages.slot_hidden : dojo.string.substitute(messages.SlotLabel, [ LayoutUtil.getSlotLabel(slot) ]);
					dojo.create("div", {
						className: "slotLabel",
						innerHTML: label
					}, slotDiv);
				}
				var sectionContainerDiv = dojo.create("div", {}, slotDiv);
				if(isHiddenSlot){
					dojo.addClass(sectionContainerDiv, "hiddenSection");
				}
				var draggableContainer = new DraggableContainer(sectionContainerDiv, { accept: [this.dndType] });
				draggableContainer.avatarClass = "com-ibm-team-workitem com-ibm-team-workitem-process";
				var self = this;
				this.connect(draggableContainer, "move", function(fromIndex, toContainer, toIndex){
					for(var toSlot in self._slots){
						if(self._slots[toSlot] == toContainer){
							if(draggableContainer.widgets.length > 0){
								var sectionIndex = dojo.indexOf(self._sectionLayouts, draggableContainer.widgets[fromIndex]);
								if(sectionIndex != -1){
									self.onSectionMoved(sectionIndex, toSlot, toIndex, draggableContainer);
								}
							}
							break;
						}
					}
				});
				this._slots[slot] = draggableContainer;
			}, this);
			this._layout = layout;
		}
	},
	
	onSectionMoved: function(sectionIndex, toSlot, toIndex){},
	
	addSection: function(sectionLayout, slot){
		if(!slot){
			slot = "defaultSection";
		}
		var draggableContainer = this._slots[slot];
		if(!draggableContainer){
			draggableContainer = this._slots.none;
		}
		draggableContainer.add(sectionLayout, { handle: sectionLayout._sectionTitleBarDIV, type:this.dndType });
		this._sectionLayouts.push(sectionLayout);
	},
	
	getSections: function(){
		return this._sectionLayouts;
	},
	
	destroy: function(preserveDom){
		// destroying DraggableContainer will destroy their child widgets
		this.clearSections();
		this._clearSlots();
		this.columnLayout.destroy(preserveDom);
		this.inherited(arguments);
	}
});

})();