/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.PartsMapping");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.EnumerationWidget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.EstimateAndCorrectionAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SeparatorWidget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.TemplatedWidget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TagWidget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TeamAreaWidget");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TextWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var getI18nMessage = function(key){
	if(!messages){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "PresentationMessages");
	}
	return messages[key];
};
var getI18nObj = function(/*Array|String*/obj){
	var ret = {};
	if(dojo.isString(obj)){
		ret[obj] = getI18nMessage(obj);
	}else if(dojo.isArray(obj)){
		dojo.forEach(obj, function(key){
			ret[key] = getI18nMessage(key);
		});
	}
	return ret;
};

var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var SIMPLE_TEXT_WIDGET = "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TextWidget";
var TEMPLATED_WIDGET = "com.ibm.team.workitem.web.process.ui.internal.view.common.TemplatedWidget";
var ENUM_WIDGET = "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.EnumerationWidget";

var mapping = {
//	"com.ibm.team.apt.web.ui.internal.parts.DurationPart": null, // cannot add from GUI?
	"com.ibm.team.apt.web.ui.internal.parts.ConstraintDatePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TimestampWidget",
	"com.ibm.team.apt.web.ui.internal.parts.WorkItemPlannedTimePresentationPart": {
		htmlLabel: '<div>'+getI18nMessage("PlannedStartDate")+'</div><div>'+getI18nMessage("PlannedEndDate")+'</div>',
		// empty
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div style="width:100%;height:100%"></div>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ApprovalsPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/ApprovalsPart.html"),
			messages: getI18nObj([ "NewApproval", "ApprovalLabel", "NoApprovals" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.AttachmentsPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/AttachmentsPart.html"),
			messages: getI18nObj([ "LabelAddAttachments", "LabelNoAttachments" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.BooleanAttributePart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<input type="checkbox" />'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.CategoryAttributePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.CategoryAttributePart",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.CheckedListAttributePart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div><div><input id="${id}_checkbox1" type="checkbox" /> <label for="${id}_checkbox1">option1</label></div><div><input id="${id}_checkbox2" type="checkbox" /> <label for="${id}_checkbox2">option2</label></div><div><input id="${id}_checkbox3" type="checkbox" /> <label for="${id}_checkbox3">option3</label></div></div>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.EnumerationAttributePart": ENUM_WIDGET,
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ContributorAttributePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.ContributorWidget",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.DiscussionPart": {
		hasBorder: true,
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/DiscussionsPart.html"),
			messages: getI18nObj([ "NoComments", "AddComment" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.HTMLAttributePart": {
		isSpanningColumns: true,
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<textarea style="width:100%; height:4.8em;"></textarea>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.layout.SimpleRichTextWidget": SIMPLE_TEXT_WIDGET,
	"com.ibm.team.apt.web.ui.internal.parts.EstimateAndCorrectionAttributePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.EstimateAndCorrectionAttributePart",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.HistoryPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div><div class="NoHistory">${messages.NoHistory}</div><hr /></div>',
			messages: getI18nObj("NoHistory")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.IntegerAttributePart":  {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<span>123</span>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ItemListPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="GreyLabel">${messages.NoItems}</div>',
			messages: getI18nObj("NoItems")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget": "com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/LinksPart.html"),
			messages: getI18nObj([ "LabelAddLinks", "LabelRelated" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ListAttributePart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<select size="3" multiple="true" style="width:100%;"><option>option1</option><option>option2</option><option>option3</option></select>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LongAttributePart": { // empty
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div style="width:100%;height:100%"></div>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaAnchorAttributePart": { // empty
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div style="width:100%;height:100%"></div>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaListPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="GreyLabel">${messages.NoProjArea}</div>',
			messages: getI18nObj("NoProjArea")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressBar": {
		htmlLabel: '<span>'+getI18nMessage("ProgressLabel")+'</span>',
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/StoryProgressBar.html"),
			messages: getI18nObj([ "NoWorkPlanned", "ProgressNumbers", "ProgressEstimated" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.QuickInformationPart": {
		hasBorder: true,
		backgroundClassName: "QuickInformationBackground",
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="NoLinks">${messages.QuickInfoNoLinks}</div>',
			messages: getI18nObj("QuickInfoNoLinks")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioGroupPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div><div><input type="radio" /> <label>option1</label></div><div><input type="radio" /> <label>option2</label></div></div>'
		}
	},
	"com.ibm.team.workitem.web.ui2.internal.presentation.ReferencedWorkItemList": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="GreyLabel">${messages.NoItems}</div>',
			messages: getI18nObj("NoItems")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ResolutionAttributePart": ENUM_WIDGET,
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.SeparatorPart": "com.ibm.team.workitem.web.ui.internal.view.layout.SeparatorWidget",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.StringAttributePart": SIMPLE_TEXT_WIDGET, // simple input field
	"com.ibm.team.workitem.web.ui.internal.view.layout.StaticTextWidget": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.StaticTextWidget",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.StatusAttributePart": ENUM_WIDGET,
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.SubscribersPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="NoSubscribers">${messages.NoSubscribers}</div>',
			messages: getI18nObj("NoSubscribers")
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.TextAttributePart": {
		isSpanningColumns: true,
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<textarea style="width:100%; height:4.8em;"></textarea>'
		}
	}, // TextArea
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.TagsAttributePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TagWidget",
	"com.ibm.team.workitem.web.ui2.internal.presentation.Target": ENUM_WIDGET,
	
	// runtime uses com.ibm.team.workitem.web.ui.internal.view.layout.TeamAreaWidget
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.TeamAreaAttributePart": {
		htmlLabel: '<div>'+getI18nMessage("LabelProjectArea")+'</div><div class="TeamAreaDiv">'+getI18nMessage("LabelTeamArea")+'</div>',
		widgetClass: "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TeamAreaWidget"
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.TimestampAttributePart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.TimestampWidget",
	"com.ibm.team.apt.web.ui.internal.parts.TimeSpentAttributePart": SIMPLE_TEXT_WIDGET, // simple input field, but label is somehow "Time Remaining"
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemAnchorAttributePart": { // empty
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div style="width:100%;height:100%"></div>'
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemListPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="GreyLabel">${messages.NoWorkItems}</div>',
			messages: getI18nObj("NoWorkItems")
		}
	},
	
	// Time Sheet tab in Formal Project Management template
	"com.ibm.team.tpt.web.ui.timesheet.internal.TimeSheetEntryPartWeb": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/TimeSheetEntryPartWeb.html"),
			messages: getI18nObj([
				"TimesheetAdd", "TimeCode", "TimesheetPrev", "TimesheetNext",
				"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday",
				"WeekTotal", "WITotal", "TimesheetOwner", "TimesheetActions", "NoTimesheet"
			]),
			timeCodeIcon: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/timecode.gif").toString(),
			calendarIcon: ImagePool.CALENDAR.toString()
		}
	},
	"com.ibm.team.tpt.web.ui.timesheet.internal.TimesheetEntriesSection": {
		hasBorder: true,
		backgroundClassName: "QuickInformationBackground",
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templateString: '<div class="GreyLabel" style="padding:0 5px;">${messages.NoTimesheet}</div>',
			messages: getI18nObj("NoTimesheet")
		}
	},
	"com.ibm.team.tpt.web.ui.risk.internal.view.editor.parts.RiskCostCurrencyPart": SIMPLE_TEXT_WIDGET,
	"com.ibm.team.tpt.web.ui.risk.internal.view.editor.RiskMatrixPart": {
		widgetClass: TEMPLATED_WIDGET,
		widgetParams: {
			templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/presentation/parts/templates/RiskMatrixPart.html"),
			messages: getI18nObj([ "RiskProbability", "RiskImpact" ])
		}
	},
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ValueSetPickerPart": "com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.ValueSetPickerWidget",
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.ValueSetComboPart": ENUM_WIDGET
};

com.ibm.team.workitem.web.process.ui.internal.view.presentation.parts.PartsMapping.getDesigntimeWidget = function(runtimeWidget){
	var ret = {};
	if(runtimeWidget){
		var widgetInfo = mapping[runtimeWidget];
		if(widgetInfo){
			if(dojo.isString(widgetInfo)){
				ret.widgetClass = widgetInfo;
			}else if(dojo.isObject(widgetInfo)){
				dojo.mixin(ret, widgetInfo);
			}
		}
	}
	if(!ret.widgetClass){
		// default
		ret = {
			widgetClass: TEMPLATED_WIDGET,
			widgetParams: {
				templateString: '<div class="GreyLabel" style="font-style:italic;">'+messages.NotAvailable+'</div>'
			}
		};
	}
	return ret;
};

})();