/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeTree");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.DeleteWorkItemsAction");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var EventTopics=com.ibm.team.workitem.web.ui.internal.page.EventTopics;	
var Term= com.ibm.team.workitem.web.client.internal.query.Term;
var AttributeTree= com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeTree;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/AttributeChooser.html"),
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this._addIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/add-condition.gif");

		var actionHelpAddContainer = "<span dojoattachpoint='_addConditionContainerInHelpText' class='addButton'></span>";
		if(!args.isOperationAttributeChooser) { // query
			this.textAddCondition = this.messages.labelAddCondition;
 			this.actionHelpText = this.messages.helpForQueries;
 			this.actionHelpText =dojo.string.substitute(this.actionHelpText, [actionHelpAddContainer] );
		} else { // bulk operation
			this.textAddCondition = this.messages.labelAddAttributeChange;
			var actionHelpAddImg = "<img class='addImage' src='" + this._addIconSrc +"' dojoattachpoint='_addConditionImage2' alt='+' title='" + this.textAddCondition + "'>";

			if(com.ibm.team.workitem.web.client.util.isCurrentContributorAnAdmin()) {
				var actionHelpDeleteContainer = "<span dojoattachpoint='_deleteConditionContainerInHelpText' class='addButton'></span>";
	 			this.actionHelpText = this.messages.helpForBulkOperations;
	 			this.actionHelpText =dojo.string.substitute(this.actionHelpText, [actionHelpAddContainer, actionHelpAddImg, actionHelpDeleteContainer] );
			} else {
	 			this.actionHelpText = this.messages.helpForBulkOperationsWithoutDelete;
	 			this.actionHelpText =dojo.string.substitute(this.actionHelpText, [actionHelpAddContainer, actionHelpAddImg] );			
			}
		}

		this.attributeTree= args.attributeTree;
		this.termSection= args.termSection;
		this._closeIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close.gif");
		this._closeIconHoverSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close-hov.gif");
	},
	
	postCreate: function() {
		this.subscribe(EventTopics.QUERY_EDITOR_CHANGED, "_updateHelpText");
		this._childWidgets = [];
		var addIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/add-condition_obj.gif")+"";
		
		var buttonArgs = {
			imgSrc: addIconSrc, 
			title: this.textAddCondition,
			label: this.textAddCondition,
			onClick: dojo.hitch(this, this.onAdd) 
		}
		var button = new jazz.ui.toolbar.Button(buttonArgs);
		this._childWidgets.push(button);
		this._addConditionButtonContainer.appendChild(button.domNode);		
		this._addConditionImage.style.display="none";

		buttonArgs = {
			imgSrc: addIconSrc, 
			title: this.textAddCondition,
			label: this.textAddCondition,
			onClick: dojo.hitch(this, this.onAdd) 
		}
		var button2 = new jazz.ui.toolbar.Button(buttonArgs);
		this._childWidgets.push(button2);
		this._addConditionContainerInHelpText.appendChild(button2.domNode);
		
		this._createAddTermButton();
		this._createTree();
		if(this.isOperationAttributeChooser) {
			buttonArgs = {
				imgSrc: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/common/images/delete.gif")+"", 
				title: this.messages.titleDeleteWorkItems,
				label: this.messages.labelDeleteWorkItems,
				onClick: dojo.hitch(this, this.onDeleteWorkItems) 
			}
			 this._deleteButton1 = new jazz.ui.toolbar.Button(buttonArgs);
			 this._childWidgets.push(this._deleteButton1);
			this._deleteWorkItemsButtonContainer.appendChild(this._deleteButton1.domNode);		

			buttonArgs = {
				imgSrc: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/common/images/delete.gif")+"", 
				title: this.messages.titleDeleteWorkItems,
				label: this.messages.labelDeleteWorkItems,
				onClick: dojo.hitch(this, this.onDeleteWorkItems) 
			}
			
			/*
			 * If a delete button span exists in the help text...
			 */
			if(this._deleteConditionContainerInHelpText) {
				this._deleteButton2 = new jazz.ui.toolbar.Button(buttonArgs);
				this._childWidgets.push(this._deleteButton2);
				this._deleteConditionContainerInHelpText.appendChild(this._deleteButton2.domNode);
			}
			
			this._changeHandle = dojo.subscribe(EventTopics.WORKITEM_OPERATION_SELECTIONS_CHANGED, dojo.hitch(this,this._handleSelectedItemsChange));
			this._addTermButtonNode.style.display="none"; // operation attributes don't use that button
			this._createAddAttributeButton(/* isOperation */ true);
			this._enableDeleteOperation([]); // disabled
		} else {
			this._deleteWorkItemsButtonContainer.style.display = "none";
			this._createAddAttributeButton();
		}
		
		// 161093: Query Editor style updates
		// don't show help text if there are no conditions
		this._updateHelpText();
	},
	
	destroy: function() {
		this._selectedItems = [];
		if(this._tree) this._tree.destroy();
		if(this.attributeTree) this.attributeTree.destroy();
		
		for(var i=0;i<this._childWidgets.length;i++) {
			this._childWidgets[i].destroy();
		}
		
		dojo.unsubscribe(this._changeHandle);		
		this.inherited(arguments);
	},
	
	onClose: function(e) {
		if(e) dojo.stopEvent(e);
		this._addConditionButtonContainer.style.display = "";
		this._addConditionImage.style.display="none";
		this._contentNode.style.display= "none";
		dojo.addClass(this._addConditionImage, "addCondition");
		dojo.removeClass(this._addConditionImage, "addConditionInViewlet");
		this._showOperationActions();
	},
	_showOperationActions: function() {
		dojo.addClass(this._operationsContainer, "com-ibm-team-workitem-queryeditor-actionChooser");
		/* If there are no terms, show the help */
		this._updateHelpText();
		if(this.isOperationAttributeChooser) {
			if(!this.termSection || this.termSection._conditionSections.length ==0 && com.ibm.team.workitem.web.client.util.isCurrentContributorAnAdmin()) {		
				this._deleteWorkItemsButtonContainer.style.display= "";
			} else {
				this._deleteWorkItemsButtonContainer.style.display= "none";
			}
		}
	},
	
	_updateHelpText: function(){
		this._actionHelp.style.display = ((!this.termSection || this.termSection._conditionSections.length ==0) && dojo.hasClass(this._operationsContainer, "com-ibm-team-workitem-queryeditor-actionChooser")) ? "" : "none";		
	},
	
	/*
	 * For bulk operations
	 */
	_handleSelectedItemsChange: function(selectedItems) {
		this._selectedItems = selectedItems;
		this._enableDeleteOperation(selectedItems);
	},
	_enableDeleteOperation: function(selectedItems) {
		if(!com.ibm.team.workitem.web.client.util.isCurrentContributorAnAdmin()) {
			this._deleteWorkItemsButtonContainer.style.display= "none";
			return;
		}
		if(!this._deleteButton1) return;
		if(selectedItems.length > 0) { // enabled
		   this._deleteButton1.attr("enabled", true);
		   this._deleteButton2.attr("enabled", true);
		} else {
		   this._deleteButton1.attr("enabled", false);
		   this._deleteButton2.attr("enabled", false);		   
		}
	},
	handleNoTermSections: function() {
		if(!dojo.hasClass(this._addConditionImage, "addCondition")) return;
		this._showOperationActions();	
	},
	onAdd: function(e) {
		if(e) dojo.stopEvent(e);
		this._addConditionButtonContainer.style.display = "none";
		this._addConditionImage.style.display="";
		this._deleteWorkItemsButtonContainer.style.display= "none";
		this._actionHelp.style.display= "none";
		this._contentNode.style.display= "block";
		dojo.removeClass(this._addConditionImage, "addCondition");
		dojo.addClass(this._addConditionImage, "addConditionInViewlet");
		dojo.removeClass(this._operationsContainer, "com-ibm-team-workitem-queryeditor-actionChooser");		
	},
	onAddCondition: function(e) {
	    this.onAdd(e);
	},
	checkEnterForAdd: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this.onAdd();
		}		
	},

	onDelete: function(e) {
		if(e) dojo.stopEvent(e);
		this._addConditionButtonContainer.style.display = "";
		this._addConditionImage.style.display="none";
		this._contentNode.style.display= "none";
		dojo.removeClass(this._addConditionImage, "addCondition");
		dojo.addClass(this._addConditionImage, "addConditionInViewlet");
		dojo.addClass(this._operationsContainer, "com-ibm-team-workitem-queryeditor-actionChooser");
	},
	
//	checkEnterForDelete: function(e) {
//		var characterCode;
//		characterCode = e.keyCode;
//		if(characterCode == 13) { //the key pressed was 'enter'
//			this.onAdd();
//		}		
//	},

	
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this.onClose();
		}		
	},
	checkEnterForAddAttribute: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key pressed was 'enter'
			this._handleAddAttributeCondition();
		}			
	},
	addAttributeSection: function(attributeId) {
		this.onAdd(); // make visible
		if(this.termSection) {
			this.termSection.addSection(attributeId); // the attribute identifier
		}
	},
	_createTree: function() {
		var tree= new AttributeTree( { model: this._createModel(this.attributeTree.children) } );
		this._tree= tree;
		this._treeNode.appendChild(tree.domNode);
		this.connect(tree.domNode, "ondblclick", this._handleAddAttributeCondition);
		this.connect(tree.domNode, "onkeypress", this.checkEnterForAddAttribute);
	},

	_createAddTermButton: function() {
		var self= this;
		var operator= this.termSection.getOperator() == Term.AND ? Term.OR : Term.AND;
		var addTermButtonParameters= {
			label: operator == Term.AND ? self.messages.labelAddOperatorConditionAND : self.messages.labelAddOperatorConditionOR,
			onClick: function() {
				self._handleAddTermCondition(operator);
				self.onClose();
			}
		};
		var addTermButton= new dijit.form.Button(addTermButtonParameters);
		this._addTermButtonNode.appendChild(addTermButton.domNode);
	},
	
	_createAddAttributeButton: function(isOperation) {
		var self= this;
		var label = isOperation ? self.messages.labelAddAttributeChange : self.messages.labelAddAttributeCondition; 
		var addAttributeButtonParameters= {
			label: label,
			onClick: function() {
				self._handleAddAttributeCondition();
			}
		};
		var addAttributeButton= new dijit.form.Button(addAttributeButtonParameters);
		this._addAttributeButton = addAttributeButton;
		this._addAttributeButtonNode.appendChild(addAttributeButton.domNode);
	},
	
	_handleAddAttributeCondition: function() {
		
		var nodes= this._tree.getSelection();
		var section= this.termSection;
		
		dojo.forEach(nodes, function(node) {
			if (node.item.isQueryable)
				section.addSection(node.item.id); // the attribute identifier
		});
	},
	
	_handleAddTermCondition: function(operator) {
		this.termSection.addTermSection({ operator: operator });
	},
	
	_createModel: function(/* TreeDTO */ dtos){
		var items= new Array();
		for (var i= 0; i < dtos.length; i++) {
			var attribute= dtos[i].item;
			if (dtos[i].children) {
				attribute.children= this._createModel(dtos[i].children);
			}
			if (attribute.isQueryable || attribute.children)
				items.push(attribute);
		}
		items.sort(this._sortAttributes);
		return items;
	},
	
	_sortAttributes: function(entry1, entry2){
		if (entry1.label < entry2.label) {
			return -1;
		}
		else if (entry1.label > entry2.label) {
			return 1;
		}
		return 0;
	},
	
	onCloseMouseOver: function(e) {
		this._closeImage.src = this._closeIconHoverSrc;	
	},
	
	onCloseMouseOut: function(e) {
		this._closeImage.src = this._closeIconSrc;
	},
	_handleDeleteWorkItems: function() {
	  	if(!this._deleteButton1.enabled) return;
	  	var ids = [];
	  	var itemIds = [];
	  	for(var i=0;i<this._selectedItems.length;i++) {
	  		ids.push(this._selectedItems[i].row._jazzRowWorkItemId);
	  		itemIds.push(this._selectedItems[i].row._jazzRowWorkItemItemId);
	  	}
		new com.ibm.team.workitem.web.ui2.internal.action.DeleteWorkItemsAction().run({actionSpec: {parameter: {ids: ids, itemIds: itemIds}}});
	},
	onDeleteWorkItems: function(e) {
		if(e) dojo.stopEvent(e);
		this._handleDeleteWorkItems();
	}
});
	
})();
