/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
// Derived from TermSection
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationConditionSection");
dojo.require("com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.query.Term");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var SectionContainer= com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer;
var AttributeChooser= com.ibm.team.workitem.web.ui.internal.view.queryeditor.AttributeChooser;
var OperationConditionSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationConditionSection;
var Term= com.ibm.team.workitem.web.client.internal.query.Term;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/OperationSection.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.parent= args.parent;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
//		this._menuItems = [{ operator: Term.AND, label: this.messages.labelAllMustMatch}, { operator: Term.OR, label: this.messages.labelAnyCanMatch}];
				
		this.operator= args.operator || Term.AND;
		this.term= args.term;
		this.attributes = args.attributes;	
		this._conditionSections= new Array();
		this._termSections= new Array();
	},
	
	postCreate: function() {
	// fix the attributes to only show editable attributes, and no children
		var trueAttributes = [];
		this._childWidgets = [];

		for(var i=0; i<this.attributes.children.length;i++) {
			if(this.attributes.children[i].item.isBulkEditable && !this.attributes.children[i].item.isMultiValued) {
				var attr = this.attributes.children[i];
				attr.children = [];
				trueAttributes.push(attr);
			}
		}
		this.attributes.children = trueAttributes;
		this.log = this.queryResultPageControl;

		if (this.term)
			this._setTerm(this.term);
		if(this._childAttributeChooser) this._childAttributeChooser.destroy();
		var attributeChooser= new AttributeChooser({ termSection: this, attributeTree: this.attributes, isOperationAttributeChooser: true });
		this._attributeChooserNode.appendChild(attributeChooser.domNode);
		this._childAttributeChooser = attributeChooser;
		this._getQueryResultsWidget().showCheckboxes(true,  this.attributes);
	},
	
	destroy: function() {
		/*//95375 */if(true) return; // disable destroy
		try {
			if(this._childAttributeChooser) this._childAttributeChooser.destroy();
			dojo.forEach(this._termSections, function(section) {
				if(section.destroy) section.destroy();
			});
			dojo.forEach(this._conditionSections, function(section) {
				if(section.destroy) section.destroy();
			});
			if(this.operationManager) this.operationManager.destroy();
			this.log = null;
			this.queryResultPageControl = null;
			for(var i=0;i<this._childWidgets.length;i++) {
				this._childWidgets[i].destroy();
			}
			this._childWidgets = [];
		} catch (e) {
			
		}	
		this.inherited(arguments);	
	},
	handleRowSelectionChanged: function() {
		var selectedRows = this.getSelectedRows();
		this.setTitle(selectedRows);
		if(this._conditionSections) {
			for(var i=0;i<this._conditionSections.length; i++) {
				this._conditionSections[i].handleRowSelectionChanged(selectedRows);
			}
		}
		dojo.publish(EventTopics.WORKITEM_OPERATION_SELECTIONS_CHANGED, [selectedRows]);
	},
	setRunButtonEnabled: function() {
		// run is enabled when at least one item and at least one condition
		var selectedRows = this.getSelectedRows();
		this.setTitle(selectedRows, true);
		var isEnabled = selectedRows.length > 0 && this.getOperations().length > 0;
		if(this.operationManager) {
			isEnabled = isEnabled && !this.operationManager.isExecuting;
		}
		if(this._site) {
			this._site.setRunButtonEnabled(isEnabled);
		}
	},
	getOperations: function() {
		var operations = [];
		for(var i=0; i< this._conditionSections.length; i++) {
			var operation = this._conditionSections[i].getOperation();
			if(operation) {
				operations.push(operation);
			}
		}
		return operations;
	},
	clearOperations: function() {
		var sections = [];
		for(var i=0;i<this._conditionSections.length;i++) {
			sections.push(this._conditionSections[i]);
		}
		for (var i = 0; i < sections.length; i++) {
			sections[i].clearCondition();
		}
		this._getQueryResultsWidget().clearMessages()
	},
	stopOperation: function() {
		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.STOP_BACKGROUND_OPERATIONS);
	},
	runOperation: function() {
		this.log.logSetStopButtonEnabled(true);
		var ops = this.getOperations();
		var attributeList = {};
		var action = false;
		for(var i=0;i<ops.length;i++) {
			if(ops[i].isAction) {
				action = ops[i].value
			} else {
				attributeList[ops[i].id] = ops[i].value;
			}

		} 
		this.getOperations();
		var rows = this.getSelectedRows();
		var workItemIdList = [];
		var workItemItemIdList = [];
		for(var i=0;i<rows.length;i++) {
			workItemIdList.push(rows[i].row._jazzRowWorkItemId);			
			workItemItemIdList.push(rows[i].row._jazzRowWorkItemItemId);
		}
		if(this.operationManager) {
			this.operationManager.stop();
			this.operationManager.destroy();
		}
		
		this.operationManager = new com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation({
			action: action,
			attributeList: attributeList,
			idList: workItemIdList,
			itemIdList: workItemItemIdList,
			log: this.queryResultPageControl,
			onSuccess: dojo.hitch(this, this._operationComplete),
			onError: dojo.hitch(this, this._operationCompleteWithErrors)
		});
		this.operationManager.execute();
	},
	_operationComplete: function() {
		// Schedule the query to refresh so the page can complete formatting
		this.log.logSetStopButtonEnabled(false);
		this.setRunButtonEnabled();
		setTimeout(dojo.hitch(this, this._promptToRefresh), 400);
	},
	_operationCompleteWithErrors: function() {
		this.log.logSetStopButtonEnabled(false);
		this.setRunButtonEnabled();
	},
	_promptToRefresh: function() {
		if(this.queryResultPageControl.refreshUrl && confirm(this.messages.promptOperationCompleteRefresh)) {
			this.queryResultPageControl.refresh();
		}
	},
	setSectionSite: function(site) {
		this._site= site;
		this.setTitle();
//		site.setupMenu("label", dojo.hitch(this, this._setOperator));
//		site.setMenuItems(this._menuItems, dojo.hitch(this, this._setOperator));
//		site.setSelectedMenuItem(this._menuItems[this.operator == Term.AND ? 0 : 1]);
	},
	_getQueryResultsWidget: function () {
		if(!this.queryResultPageControl) return null;
		return this.queryResultPageControl.getQueryResultsWidget();
	},
	setTitle: function(selectedRows, skipSettingRunButtonEnabled) {
		selectedRows = selectedRows || this.getSelectedRows();
		var selectionCount = selectedRows.length;
		var title;
		if(selectionCount == 0) {
			var title = this.messages.sectionTitleBulkOperationsNoneSelected;
		} else {
			if(selectionCount == 1) {
				var title = this.messages.sectionTitleBulkOperationsOneSelected;
			} else {
					var title = dojo.string.substitute(this.messages.sectionTitleBulkOperations, [selectionCount + ""]);
			}
		}
		if(this._site) {
			this._site.setTitle(title);
		}
		
		if(!skipSettingRunButtonEnabled) {
			this.setRunButtonEnabled();			
		}

	},
	getSelectedRows: function() {
		return this._getQueryResultsWidget().getSelectedRows();
	},
	deregister: function(section) {
		this._remove(this._conditionSections, section);
		this._remove(this._termSections, section);
		if(section && section.destroy) section.destroy();
		if(!this._conditionSections || this._conditionSections.length ==0) {
			this._childAttributeChooser.handleNoTermSections();
		}
		this.setRunButtonEnabled();
	},
	
	addSection: function(/* String */ attributeId, /* AttributeExpressionDTO */ expression) {
		
		// If the attribute is already in a section, don't add it again
		for(var i=0;i<this._conditionSections.length;i++) {
			if(this._conditionSections[i].attributeId == attributeId) {
				alert(this.messages.errorAttributeAlreadyAdded);
				return;
			}
		}
		
		var section= new OperationConditionSection({ attributeId: attributeId, parent: this, expression: expression });
		var container= new SectionContainer({ sectionWidget: section });
		dojo.addClass(container.domNode, "conditionLayout");
		
		this._childWidgets.push(section);
		this._childWidgets.push(container);
		
		this._conditionSections.push(section);
		
		var conditionsNode= this._conditionsNode;
		var attributeChooserNode= this._attributeChooserNode;
		conditionsNode.insertBefore(container.domNode, attributeChooserNode);
		
		this.setRunButtonEnabled();
		return section;
	},
	addBulkOperationAttribute: function(attributeId) {
		this._childAttributeChooser.addAttributeSection(attributeId);
	},
	addTermSection: function(args) {

		var term= args.term; /* TermDTO */
		var operator= term ? term.operator : args.operator;
		
		var section= new TermSection({ attributes: this.attributes, operator: operator, term: term, parent: this });
		var container= new SectionContainer({ sectionWidget: section });
		dojo.addClass(container.domNode, "termLayout");
		
		this._childWidgets.push(container);
			
		this._termSections.push(section);
		
		var conditionsNode= this._conditionsNode;
		conditionsNode.appendChild(container.domNode);
		
		this.setRunButtonEnabled();
		
		return section;
	},
	
	/* Term */ getTerm: function() {
		var conditionSections= this._conditionSections;
		var termSections= this._termSections;
		
		var term= new Term(this.operator);
		for (var i= 0; i < conditionSections.length; i++) {
			var expression= conditionSections[i].getExpression();
			if (expression)
				term.addAttributeExpression(expression);
		}
		for (var k= 0; k < termSections.length; k++) {
			var subTerm= termSections[k].getTerm();
			term.addTerm(subTerm);
		}
		return term;
	},
	
	getOperator: function() {
		return this.operator;
	},
	
	onClose: function() {
		if(this.queryResultPageControl) {
			this.queryResultPageControl.setTableEditMode(false);
		}
		if (this.parent) {
			this.parent.deregister(this);
			return true;
		}
		return false;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	_setTerm: function(/* TermDTO */ term) {
		
		this._setOperator( { operator: term.operator });
		
		var attributeExpressions= term.attributeExpressions;
		var termExpressions= term.termExpressions;
		
		if (attributeExpressions) {
			for (var i= 0; i < attributeExpressions.length; i++) {
				var expression= attributeExpressions[i];
				this.addSection(expression.attribute.id, expression);
			}
		}
		if (termExpressions) {
			for (var i= 0; i < termExpressions.length; i++) {
				var termExpression= termExpressions[i];
				this.addTermSection({
					term: termExpression
				});
			}
		}
	},
		
	_remove: function(arr, obj) {
		var idx= dojo.indexOf(arr, obj);
		if (idx > -1)
			arr.splice(idx, 1);
	},
	
	_setOperator: function(obj) {
		this.operator= obj.operator;
		if (this._site)
			this._site.setTitle(this._getTitle());
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_getTitle: function() {
//		return this.operator == Term.AND ? this.messages.labelAND : this.messages.labelOR;
		return this.messages.sectionTitleBulkOperationsGeneric; 
	}
	
});

var OperationSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSection;

})();
