/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
(function() {
	
var PopupButton= com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton;
var Messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
	
dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/OperationSectionContainer.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this._labelProperty= "name";
		this.sectionWidget= args.sectionWidget;
		this._labelClose= Messages.labelClose;
		this._labelRun= Messages.labelRun;
		this._labelClear= Messages.labelClear;
		this._labelStop= Messages.buttonStop;
	},
	
	postCreate: function() {
		this._operationsContainer.appendChild(this.sectionWidget.domNode);
		this.setStopButtonEnabled(false);
		if (this.sectionWidget.setSectionSite)
			this.sectionWidget.setSectionSite(this);	
		this._changeHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.QUERY_EDITOR_CHANGED, dojo.hitch(this,this._handleQueryChange));

		var clearHovHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/images/clear-hov.gif").uri;
		var clearHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/query/images/clear.gif").uri;		
		var closeHovHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close-hov.gif").uri;
		var closeHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close.gif").uri;		
		
		this._clearImg.setAttribute("src", clearHref);
		this._clearImg.setAttribute("onmouseover", "this.src='" + clearHovHref + "'");
		this._clearImg.setAttribute("onmouseout", "this.src='" + clearHref + "'");
		this._closeImg.setAttribute("src", closeHref);
		this._closeImg.setAttribute("onmouseover", "this.src='" + closeHovHref + "'");
		this._closeImg.setAttribute("onmouseout", "this.src='" + closeHovHref + "'");
	},
	_handleQueryChange: function() {
		this.sectionWidget.setRunButtonEnabled();
	},
	destroy: function() {
		if(this._menuWidget && this._menuWidget.destroy) this._menuWidget.destroy();
		if (this.sectionWidget && this.sectionWidget._site === this)
			this.sectionWidget.destroy();
		dojo.unsubscribe(this._changeHandle);
		this.inherited(arguments);
	},
	
	setTitle: function(title) {
		dojox.xml.parser.replaceChildren(this._headerText, document.createTextNode(title));
	},
	
	setupMenu: function(property, callback) {
		this._labelProperty= property;
		this._callback= callback;
	},
	
	setMenuItems: function(menuItems, callback) {
		var menu= this._getMenu();
		menu.setValues(menuItems);
	},
	
	setSelectedMenuItem: function(item) {
		var menu= this._getMenu();
		menu.setSelection(item);
	},
	setStopButtonEnabled: function(isEnabled) {
		if(isEnabled) {
			var stopHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/stop.gif").uri;
			var stopHovHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/stop-hov.gif").uri;
			this._stopAction.setAttribute("aria-disabled", "false");
			this._stopImg.setAttribute("src", stopHref);
			this._stopImg.setAttribute("onmouseover", stopHovHref);
			this._stopImg.setAttribute("onmouseout", stopHref);
		} else {
			this._stopAction.setAttribute("aria-disabled", "true");
			var stopDisabledHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/stop-disabled.gif").uri;
			this._stopImg.setAttribute("src", stopDisabledHref);
			this._stopImg.removeAttribute("onmouseover");
			this._stopImg.removeAttribute("onmouseout");
		}
		
	},
	setRunButtonEnabled: function(isEnabled) {
		if(isEnabled) {
			this._runAction.setAttribute("aria-disabled", "false");
			var runHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/run.gif").uri;
			var runHovHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/run-hov.gif").uri;
			this._runImg.setAttribute("src", runHref);
			this._runImg.setAttribute("onmouseover", runHovHref);
			this._runImg.setAttribute("onmouseout", runHref);
		} else {
			this._runAction.setAttribute("aria-disabled", "true");
			var runDisabledHref = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/run-disabled.gif").uri;
			this._runImg.setAttribute("src", runDisabledHref);
			this._runImg.removeAttribute("onmouseover");
			this._runImg.removeAttribute("onmouseout");
		}
		
	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_getMenu: function() {
		if (!this._menuWidget) {
			var menu= new PopupButton({ labelProperty: this._labelProperty, callback: this._callback });
			this._headerMenu.appendChild(menu.domNode);
			this._menuWidget= menu;
		}
		return this._menuWidget;
	},
	onClear: function(event) {
		dojo.stopEvent(event);
		this.sectionWidget.clearOperations();
	},
	onRun: function(event) {
		dojo.stopEvent(event);
		if(this._runAction.getAttribute("aria-disabled") == "true") return;
		var rowCount = this.sectionWidget.getSelectedRows().length;
		if(rowCount == 0) {
			console.debug("Error... no items selected for operation run.") // $NON-NLS-1 should never happen
			return;
		}
		var message = dojo.string.substitute(Messages.confirmRunOperation, [rowCount + ""]);
		if(!confirm(message)) return;
		this.setRunButtonEnabled(false);
		this.sectionWidget.runOperation();
		},
	onStop: function(event) {
		dojo.stopEvent(event);
		this.sectionWidget.stopOperation();
		},
	onClose: function(event) {
		dojo.stopEvent(event);
		var destroy= true;
		var sectionWidget= this.sectionWidget;
		if (sectionWidget && sectionWidget.onClose) {
			destroy= sectionWidget.onClose();
		}
		if (destroy) {
			this.destroy();
		}
	},
	checkEnterForRun: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onRun();
		}		
	},
	checkEnterForStop: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onStop();
		}		
	},
	checkEnterForClear: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onClear();
		}		
	},
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onClose();
		}		
	}
});

})();
