/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");	
dojo.require("dijit.Menu");
dojo.require("dojox.xml.parser");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

(function() {
	
var ITEM_SLOT= "__jazzItem";

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/PopupButton.html"),

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	labelSpan: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// Private fields
	
	_menuWidget: null,
	_currentSelection: null,
	_defaultValue: null,
	
	///////////////////////////////////////////////////////////////////////////////////
	// Called on creation
	
	constructor: function(args) {
		this.labelProperty= args.labelProperty;
		this.callback= args.callback;
		this.menuLabel= args.menuLabel;
	},
	
	postCreate: function() {
		this._menuWidget= new dijit.Menu({});
		this._childWidgets= [];
		dojo.addClass(this._menuWidget.domNode, "customDijitMenu");
		this.connect(this._menuWidget, "onItemClick", "onSelect");
		dijit.setWaiRole(this.domNode, "menubar");		
	},
	
	destroy: function() {
		if(this._menuWidget) this._menuWidget.destroy();
		for(var i=0;i<this._childWidgets.length;i++) {
			this._childWidgets[i].destroy();
		}
		this._childWidgets= [];
		this.inherited(arguments);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	setValues: function(items) {
		if (items.length > 0) {
			for (var i= 0; i < items.length; i++) {
				this._menuWidget.addChild(this._createMenuItem(items[i]));
			}
		}
	},
	
	getSelection: function() {
		return this._currentSelection;
	},
	
	setSelection: function(item) {
		this._updateSelection(item);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
	
	_createMenuItem: function(item) {
		var label= item[this.labelProperty];
		var menuItem= new dijit.MenuItem( { label: label });
		menuItem[ITEM_SLOT]= item;
		this._childWidgets.push(menuItem);
		return menuItem;
	},
	
	_updateSelection: function(item) {
		var label= item[this.labelProperty];
		dojox.xml.parser.replaceChildren(this.labelSpan, document.createTextNode(label));
		this._currentSelection= item;
		
		if (this.menuLabel) {
			this._menuAnchor.setAttribute("aria-label", this.menuLabel + label);
			dijit.setWaiRole(this._menuAnchor, "menuitem");
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Events
	
	onShowMenu: function(e) {
		this._menuWidget._openMyself(e);
	},
	
	onSelect: function(/* MenuItem2 */ menuItem) {
		this._updateSelection(menuItem[ITEM_SLOT]);
		if (this.callback)
			this.callback(this.getSelection());
	}
	
});

})();
