/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.client.internal.QueryClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TabController");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.UserSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TeamAreaSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.ResultColumnsSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp");

(function(){

var checkNotNull= net.jazz.ajax.toolbox.lang.checkNotNull;
var PlatformUI= net.jazz.ajax.ui.PlatformUI;

var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var QueryClient= com.ibm.team.workitem.web.client.internal.QueryClient;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImagePool= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

var TabController= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TabController;
var SectionContainer= com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer;
var TermSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TermSection;
var ConditionSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ConditionSection;

var EditableTableSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection;
var UserSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.UserSection;
var TeamAreaSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TeamAreaSection;
var ResultColumnsSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.ResultColumnsSection;
var OrderColumnsSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.OrderColumnsSection;
var TextAreaSection= com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection;
var WindowContext= com.ibm.team.workitem.web.ui.internal.module.WindowContext;

var isGuest= com.ibm.team.repository.web.client.session.isGuest;


dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.QueryEditor", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/QueryEditor.html"),
	
	/////////////////////////////////////////////////////////////////////////////
	// Widget creation
	
	constructor: function(args) {
		this.parentController= args.parentController;
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.titleCloseQuery = this.messages.titleCloseQuery;
		this.titleCancelQuery = this.messages.titleCancelQuery;
		this.textRun = this.messages.buttonRun;
		this.textCancel = this.messages.buttonCancel;
		this.textSave = this.messages.buttonSave;
		this.textSaveCopy = this.messages.buttonSaveCopy;
		this.textLabelName = this.messages.labelName;
		this.textTabTitleConditions = this.messages.tabTitleConditions;
		this.textTabTitleDetails = this.messages.tabTitleDetails;
		this.textTabTitleResultLayout = this.messages.tabTitleResultLayout;
		this.textSectionTitleDetails = this.messages.sectionTitleDetails;
		this.textLabelCreator = this.messages.labelCreator;
		this.textClose=this.messages.labelClose;
	},
	
	postCreate: function(){
		
		this._createTabs();
		this._createControls();
		
		this.typeIcon.src= ImagePool.WORKITEM_QUERY;
		
		this._dirtySubscription= dojo.subscribe(EventTopics.QUERY_EDITOR_CHANGED, this, this._setDirty);
		this._deleteQueryHandle= dojo.subscribe(EventTopics.QUERY_DELETED, this, this._handleQueryDeleted);

		this._updateListener= com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil.runAfterTyping(this._nameInput, 
				com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil.DEFAULT_DELAY, dojo.hitch(this, this._handleNameChange));

		var srh= new ServiceResponseHandler(this, "_setAttributeTree", "_handleAttributeError");
		QueryClient.getAttributeTree(srh);
		
		var helpContextId= com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics.QUERY_EDITOR_PAGE_CS_HELP;
		this._pageHelp= new jazz.app.ua.PageHoverHelp({uaKey: helpContextId});
		this._pageHelp.placeAt(this._contextHelp);
	},
	
	destroy: function() {
		try {
			dojo.unsubscribe(this._dirtySubscription);
			dojo.unsubscribe(this._deleteQueryHandle);
			if(this._updateListener) dojo.disconnect(this._updateListener);
			this._tabController.destroy();
			if(this._descriptionSection) this._descriptionSection.destroy();
			if(this._teamAreaAssociations) this._teamAreaAssociations.destroy();
			if(this._userAssociations) this._userAssociations.destroy();
			if(this._resultColumns) this._resultColumns.destroy();
			if(this._orderColumns) this._orderColumns.destroy();
			if(this._container) this._container.destroy();
			if(this._pageHelp) this._pageHelp.destroy();
			this._pageHelp = null;
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	_handleQueryDeleted: function(itemId) {
		if(this._query && this._query.stateId && this._query.itemId == itemId) {			
			this._query.stateId = null;
			this._setDirty();
		}
	},
	_createTabs: function(){
		
		var tabStyleName= "Selected";
		var tabController= new TabController();
		this._tabController= tabController;
		
		tabController.registerTab(this._tabConditions, this._tabConditionsContent, tabStyleName);
		tabController.registerTab(this._tabDetails, this._tabDetailsContent, tabStyleName);
		tabController.registerTab(this._tabResultLayout, this._tabResultLayoutContent, tabStyleName);
		
		tabController.showTab(this._tabConditions);
	},
	
	_createControls: function() {
		
		var descriptionSection= new TextAreaSection({ title: this.messages.sectionTitleDescription, editActionName: this.messages.buttonEditDescription, emptyMessage: this.messages.labelEmptyDescription });
		this._descriptionSectionDiv.appendChild(descriptionSection.domNode);
		this._descriptionSection= descriptionSection;

		var teamAreaAssociations= new TeamAreaSection({ title: this.messages.sectionTitleTeamAreaSharing, addActionName: this.messages.buttonAddTeamArea, emptyMessage: this.messages.labelEmptyTeamAreaSharing});
		this._teamAreaSharingColumn.appendChild(teamAreaAssociations.domNode);
		this._teamAreaAssociations= teamAreaAssociations;

		var userAssociations= new UserSection({ title: this.messages.sectionTitleUserSharing, addActionName: this.messages.buttonAddUser, emptyMessage: this.messages.labelEmptyUserSharing});
		this._userSharingColumn.appendChild(userAssociations.domNode);
		this._userAssociations= userAssociations;

		var resultColumns= new ResultColumnsSection({ title: this.messages.sectionTitleResultColumns, addActionName: this.messages.buttonAddColumn, emptyMessage: this.messages.labelEmptyResultColumns, supportOrdering: true });
		this._resultsColumn.appendChild(resultColumns.domNode);
		this._resultColumns= resultColumns;

		var orderColumns= new OrderColumnsSection({ title: this.messages.sectionTitleSortOrder, addActionName: this.messages.buttonAddSortColumn, emptyMessage: this.messages.labelEmptyOrderColumns, supportOrdering: true });
		this._orderColumn.appendChild(orderColumns.domNode);
		this._orderColumns= orderColumns;
	},
	
	_createTermPart: function(/* TermDTO */ term){
		
		if (this._container)
			this._container.destroy();
			
		var termSection= new TermSection({ attributes: this._attributeTree, term: term });
		var container= new SectionContainer({
			sectionWidget: termSection,
			isTermSection: true
		});
		this._termSection= termSection;
		this._tabConditionsContent.appendChild(container.domNode);
		
		this._container= container;
	},
	isShowingNewQuery: function(kwArgs) {
		if(!this._query || !this._query.isNew) return false;
		if(kwArgs.tq == this._query._uriTimestamp) return true;
		return false;
	},
	/////////////////////////////////////////////////////////////////////////////
	// API
	
	showQuery: function(query, kwArgs) {
		if(!this._unchangedQuery || this._unchangedQuery.id != query.id) {
			this._unchangedQuery = dojo.clone(query);
		}
		query._uriTimestamp = kwArgs.tq;
		if(query.isNew) {
			this._headingTitle = this.messages.sectionTitleWorkItemNewQuery;
			this._isDirty= false;
		} else {			
			this._headingTitle = this.messages.sectionTitleWorkItemEditQuery;
			this._isDirty= query.stateId ? false : true;
		}
		this._setTitle(this._headingTitle);
		this._clearMessage();
		this._internalShowQuery(query);
		if (!query.stateId) {
			this._nameInput.focus();
		}
		this._queryEditorUri = window.location.href;
	},
    getQueryEditorUri: function() {
    	return this._queryEditorUri; // can be null
    },
	_internalShowQuery: function(query, keepName) {
		this._query= query;
		this._query._oldName = query.name;
		
		this._suspendDirtyEvents= true;
		
		this._showName(query.name, keepName);
		if (this._isCopy) {
			this._nameInput.focus();
			this._nameInput.select();
			this._isCopy= false;
		}
		this._isInitializing = true;
		this._showCreator(query.creator);
		this._descriptionSection.setText(query.description);
		this._teamAreaAssociations.setInput(query.teamAreaAssociations);
		this._userAssociations.setInput(query.userAssociations);
		this._resultColumns.setInput(query.columns);
		this._orderColumns.setInput(query.sorting);
		
		if (this._attributeTree) {
			this._createTermPart(query.expression)
		}
		this._tabController.showTab(this._tabConditions);
		this._isInitializing = false;
		this._suspendDirtyEvents= false;
		this._updateDirtyState();
		this._suspendDirtyEvents= true;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []); // for component menu
		this._suspendDirtyEvents= false;
		dojo.publish(EventTopics.WORKITEM_QUERY_VIEWED, [query]);
	},
	
	_setTitle: function(title) {
		dojox.xml.parser.replaceChildren(this._titleTextSpan, document.createTextNode(title));
	},
	
	/////////////////////////////////////////////////////////////////////////////
	
	// Values
	clearName: function() {
		this._isClear = true;
		this._nameInput.value= "";
	},
	/*
	 * Only return a dirty query if it is changed and has changes
	 * (New queries are changed even if they have no data)
	 */
	hasUnsavedChanges: function() {
		if(!this._isDirty) return false;
		
		var args = this._collectSaveArgument(false);
	
		if(args.name != null) return true;
		if(	args.expression && args.expression._term) {
			if(args.expression._term.attributeExpressions && args.expression._term.attributeExpressions.length > 0) return true; 
			if(args.expression._term.similarityExpressions && args.expression._term.similarityExpressions.length > 0) return true; 
			if(args.expression._term.termExpressions && args.expression._term.termExpressions.length > 0) return true; 
		}
		if(args.description != null) return true;
		if(args.teamAreaAssociations != null) return true;
		if(args.contributorAssociations != null) return true;
		if(args.resultColumns != null) return true;
		if(args.sortColumns != null) return true;
		if(args.sortDirections != null) return true;
		
		return false;
	},
	_showName: function(name, keepName) {
		if (!((keepName || this._isClear) && name == "" && this._nameInput.value != "" )) {
			this._nameInput.value= name;
		}
		this._isClear = false;

	},
	getName: function() {
		return this._getName();
	},
	_getName: function() {
		return this._nameInput.value;
	},
	_resetName: function() {
		if(this._query._oldName) this._query.name=this._query._oldName; 
	},
		
	_hasNameChanged: function() {
		if(!this._query) return false;
		var result = this._query._oldName != this.getName();
		return this._query._oldName != this.getName();
	},
	
	_showCreator: function(creator) {
		if (creator)
			dojox.xml.parser.replaceChildren(this._queryCreator, document.createTextNode(creator.label));
	},
	
	/////////////////////////////////////////////////////////////////////////////
	// Callbacks
	
	/*
	 * If another query is loaded, then this one is no longer dirty
	 */
	resetDirtyState: function() {
		this._isDirty = false;
		this._updateDirtyState();
	},
	_setDirty: function() {
		if(this._isInitializing) return;
		if(!this._suspendDirtyEvents) {
			if (!this._isDirty) {
				this._isDirty= true;
				this._updateDirtyState();
				dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, [this._query]);
			}
		}
	},
	
	_updateDirtyState: function() {
		this._updateSaveButtonEnablement();
		if(this._query) this._query.isDirty = this._isDirty;
		this._setTitle(this._headingTitle + (this._isDirty ? " *" : ""));
		/*
		 * Notify the page that the state has changed
		 */
		if(this._wasDirty != this._isDirty) {
			var queryItemId = this._query.clientItemId || this._query.itemId;
			var queryProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + queryItemId);
			if(queryProxy) {
				queryProxy.setValue({path: "attributes.isChanged", attributeId: "isChanged", value:this._isDirty});
			}
			dojo.publish(EventTopics.WORKITEM_ACTION_COMPONENT_MENU_SETPAGEDIRTY, [this._query]);
		}
		this._wasDirty = this._isDirty;

	},
	
	_updateSaveButtonEnablement: function() {
		var value= this._getName();
		var isInvalid= isGuest() || !(value && value.length > 0);
		this._saveCmd.disabled= isInvalid || !this._isDirty;
		if (this._query && this._query.stateId) {
			this._saveCopyCmd.disabled= isInvalid;
			this._closeCmd.disabled= isInvalid;
		} else {
			this._saveCopyCmd.disabled= true;
			this._closeCmd.disabled= true;
		}
	},
	_handleNameBlur: function() {
		if (this._hasNameChanged() || this._lastBlurNameChange != this.getName()) {
			this._lastBlurNameChange = this.getName();
			this._query.name = this.getName();
			dojo.publish(EventTopics.QUERY_EDITOR_NAME_CHANGED, [this._query]);
		}
	},
	_handleNameChange: function() {
		var wasDirty = this._isDirty;
		this._setDirty(); // publishes QUERY_EDITOR_CHANGED on first dirty
		if (wasDirty && this._hasNameChanged()) {
			dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, [this._query]);
		}
		this._updateSaveButtonEnablement();
	},
	runQuery:function() {
		this._handleRunClick();
	},
	_handleCancelClick: function(e) {
//		if(this._query && this._isDirty) {
//			var msg;
//			if(this._query.name == "" || this._query.name == "New Query") {
//				msg = this.messages.confirmAbandonUnnamedQuery;
//			} else {
//				msg = dojo.string.substitute(this.messages.confirmAbandonNamedQuery, [ this._getName() ]);
//			}
//			if(!confirm(msg)) return;
//		}
		this._isDirty = false;
		this._updateDirtyState();
		this._query._isCancelled = true;
//		this._query = null;
		dojo.publish(EventTopics.WORKITEM_ACTION_CLEAR_QUERY_EDITOR, [this]);
		var uri = com.ibm.team.workitem.web.client.util.getShowQueriesUri();
		com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation2(uri);
	},
	_handleRunClick: function() {
		if (!this._termSection)
			return;
		
		var expression= this._termSection.getTerm();
		
		var query= new Object();
		query.name= this._getName();
		
		var clientItemId = this._query ? this._query.itemId || "" : "";
		query.clientItemId = clientItemId;
		this._query.clientItemId = clientItemId; 
		query.itemId = "";
		query.internalExpression= expression;
		
		var sortOrder= this._orderColumns.getValues();
		var resultColumns= this._resultColumns.getValues();
		var columnArgs= { sortOrder: sortOrder, columnIdentifiers: this._getIds(resultColumns)};
		
		this.parentController.runQuery(query, columnArgs);
	},
	
	_handleSaveClick: function() {
		this._handleSave(false);
	},
	
	_handleSaveCopyClick: function() {
		this._handleSave(true);
	},
	
	_handleSave: function(createCopy) {

		this._isCopy= createCopy;
		this._startOperation(dojo.string.substitute(this.messages.busySavingQuery, [ this._getName() ]), "operation.query.save");
		
		var args= this._collectSaveArgument(createCopy);
		var srh= new ServiceResponseHandler(this, "_querySaved", "_handleSaveError");
		QueryClient.saveQuery(srh, args);
	},
	
	_collectSaveArgument: function(createCopy) {
		
		var args= new Object();
		
		if (this._query && !createCopy) {
			if (this._query.stateId) {
				args.itemId= this._query.itemId;
				args.stateId= this._query.stateId;
			}
		}
		if (createCopy) {
			args.name= dojo.string.substitute(this.messages.copyOfNamePattern, [this._getName()]);
		}
		else if (this._hasNameChanged()) {
			args.name= this._getName();
		} 
		/*
		 * Handle the special case when the current query has been deleted
		 */
		if(!createCopy && this._query && !this._query.stateId && this._query.itemId) {
			args.name= this._getName();
			createCopy = true;
		}

		if (createCopy || this._descriptionSection.hasChanged()) {
			args.description= this._descriptionSection.getText();
		}
		/*
		 * When doing createCopy, do not copy the associations
		 * TODO: Ask the user whether to keep the association
		 */
		if(!createCopy) {
			if (this._teamAreaAssociations.hasChanged()) {
				var teamAreas= this._teamAreaAssociations.getValues();
				args.teamAreaAssociations= this._getIds(teamAreas);
			}
			if (this._userAssociations.hasChanged()) {
				var users= this._userAssociations.getValues();
				args.contributorAssociations= this._getIds(users);
			}
		}
		if (createCopy || this._resultColumns.hasChanged()) {
			var resultColumns= this._resultColumns.getValues();
			args.resultColumns= this._getIds(resultColumns);
		}
		if (createCopy || this._orderColumns.hasChanged()) {
			
			var orderColumns= this._orderColumns.getValues();
			args.sortColumns= this._getIds(orderColumns);
			
			var directions= new Array();
			for (var i= 0; i < orderColumns.length; i++) {
				directions.push(orderColumns[i].ascending);
			}
			args.sortDirections= directions;
		}
			
		args.expression= this._termSection.getTerm();
		
		return args;
	},
	
	_getIds: function(uiItems) {
		var ids= new Array();
		for (var i= 0; i < uiItems.length; i++) {
			ids.push(uiItems[i].id);
		}
		return ids;
	},
	
	confirmLeaveUnsavedChanges: function(kwArgs) {
		if (!this._query) return true;
		/*
		 * If the current query is not dirty and not new, or it's destroyed, then it is ok to replace it
		 */
		if ((!this._isDirty && !this._query.isNew)|| this._query.isDestroyed || this._query._isCancelled) return true;
		if(this._query.stateId == null) { // unsaved new query
			if(kwArgs.action=="com.ibm.team.workitem.newQuery" && kwArgs.tq == this.parentController.pageController._lastQueryTimestamp) return true; // same new query
		} else {
			if(kwArgs.id == this._query.itemId) return true; // same query
		}
		var name = this.getName();
		var message;
		if(name && name.length >0) {
			message = dojo.string.substitute(this.messages.confirmAbandonNamedQuery, [ name ]);
		} else {
			message = this.messages.confirmAbandonUnnamedQuery;
		}
		var result = confirm(message);
		if(result) {
			this._resetName();
			this.resetDirtyState();
			if(this._query.isNew) {				
				this.parentController._removeQuery({ query: this._query});
				this._query.isDestroyed = true;
			} else {
				this.parentController._resetQuery({ query: this._query});
			}
		}
		return result;
	},
	
	_clearMessage: function(){
		dojo.publish(EventTopics.WORKITEM_ACTION_SET_MESSAGEBOX);
	},
	
	_showReport: function(report){
		if(report.severity == QueryClient.PROCESS_REPORT_OK){
			this._clearMessage();
		} else {
			var details = [];
			if(report.infos){
				dojo.forEach(report.infos, function(info){
					details.push({ message: "<b>"+info.summary+"</b>" });
					details.push({ message: info.description });
				});
			}
			dojo.publish(EventTopics.WORKITEM_ACTION_SET_MESSAGEBOX, [{
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: report.message,
				details: details
			}]);
		}
	},
	
	_showError: function(message, error){
		this.showReport({
			severity: QueryClient.PROCESS_REPORT_ERROR,
			message: message,
			infos: [{
				summary: message,
				description: (error.message || "")
			}]
		});
	},
	
	_querySaved: function(operationReport) {

		if (operationReport.query) {
			operationReport.query._isSaved = true;
			this._headingTitle = this.messages.sectionTitleWorkItemEditQuery;
			this.parentController._replaceQuery({ oldQuery: this._query, newQuery: operationReport.query});
			this._unchangedQuery = operationReport.query;
			this._isDirty= false;
			this._internalShowQuery(operationReport.query);
			/*
			 * If we just did a Save as Copy, then the itemId of the query
			 * is not in the current url, so set the current url to proper value
			 */
			var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + this._query.itemId);
			proxy.setIsBeingEdited(true);
			var queryUrl = proxy.getUrl();
			if(queryUrl != window.location.href) {
				WindowContext.replaceWindowLocation(queryUrl);
			}
			
			
		}
		this._showReport(operationReport);
		
		this._query.isNew = false;
		
		/*
		 * If it was a new query that was saved, add a timestamp so the next new query uri request opens a new query rather than this one
		 */
		if(operationReport.query && window.location.href.indexOf("action=com.ibm.team.workitem.newQuery") !=-1 && window.location.href.indexOf("saved") == -1) {
			var href = window.location.href + "&savedId=" + operationReport.query.itemId; 
			WindowContext.setWindowLocation2(href);
		}
		this._queryEditorUri = window.location.href;		
		this._stopOperation();
		this.parentController._setQueryPageTitle(this._getName(), /* isEditing */ true);
		dojo.publish(EventTopics.BUILDER_POST_SAVE, [operationReport.query]);
	},

	_setAttributeTree: function(/* TreeDTO */ tree) {
		this._attributeTree= tree;
		this._resultColumns.setCachedAttributeTree(tree);
		this._orderColumns.setCachedAttributeTree(tree);
		if (this._query) {
			this._internalShowQuery(this._query, /* keepName */ true);
		}
	},
	
	_startOperation: function(name, id) {
		this._currentOperation= this.parentController.createOperation(name, id);
		this.parentController.startOperation(this._currentOperation);
		this._clearMessage();
	},
	
	_stopOperation: function() {
		this.parentController.finishOperation(this._currentOperation);
	},
	
	_handleAttributeError: function(e) {
		this._stopOperation();
		this._showError(this.messages.errorFetchingAttributes, e);
	},
	
	_handleSaveError: function(operationReport) {
		this._stopOperation();
		this._showReport(operationReport);
	}, 
	onClose: function(e) {
		if(e) dojo.stopEvent(e);
		if(!this._query) return;
		if(this._isDirty && !this._query.isNew) {
			var msg = dojo.string.substitute(this.messages.confirmAbandonNamedQuery, [ this._getName() ]);
			if(!confirm(msg)) return;
		}
		var uri;
		if(this._query.isNew) {
			uri = com.ibm.team.workitem.web.client.util.getShowQueriesUri();
			com.ibm.team.workitem.web.cache.internal.Cache.removeItem("workitems/queries/" + this._query.itemId);
			this._handleCancelClick();
			return;		
		}
		
		var proxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem("workitems/queries/" + this._query.itemId);
		if(proxy) proxy.setIsBeingEdited(false);
		this._query._isCancelled = true;
		this._updateDirtyState();
		uri = com.ibm.team.workitem.web.client.util.getRunSavedQueryUri(this._query.itemId);	
		com.ibm.team.workitem.web.ui.internal.module.WindowContext.setWindowLocation2(uri);
	}
});
	
})();
