/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {
	
var PopupButton= com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton;
var Messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
	
dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SectionContainer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/SectionContainer.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this._labelProperty= "name";
		this.sectionWidget= args.sectionWidget;		
		this._labelClose= Messages.labelClose;
		this._closeIconAlt = Messages.labelClose;
		this._closeIconTitle = Messages.labelClose;
		this._closeIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close.gif");
		this._closeIconHoverSrc = dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/images/close-hov.gif");
	},
	
	postCreate: function() {
		this._bodyContainer.appendChild(this.sectionWidget.domNode);
		if(this.isTermSection) {
			this._headerButtons.style.display = "none";
		}
		if (this.sectionWidget.setSectionSite)
			this.sectionWidget.setSectionSite(this);		
	},
	destroy: function() {
		if(this._menuWidget && this._menuWidget.destroy) this._menuWidget.destroy();
		if (this.sectionWidget)
			this.sectionWidget.destroy();
		this.inherited(arguments);
	},
	
	setTitle: function(title) {
		dojox.xml.parser.replaceChildren(this._headerText, document.createTextNode(title));
	},
	
	setupMenu: function(property, callback) {
		this._labelProperty= property;
		this._callback= callback;
	},
	
	setMenuItems: function(menuItems, callback) {
		var menu= this._getMenu();
		menu.setValues(menuItems);
	},
	
	setSelectedMenuItem: function(item) {
		var menu= this._getMenu();
		menu.setSelection(item);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_getMenu: function() {
		if (!this._menuWidget) {
			var menu= new PopupButton({ labelProperty: this._labelProperty, callback: this._callback, menuLabel: Messages.labelConditionTypeMenu });
			this._headerMenu.appendChild(menu.domNode);
			this._menuWidget= menu;
		}
		return this._menuWidget;
	},
	
	onClose: function(e) {
		if(e)	dojo.stopEvent(e);
		var destroy= true;
		var sectionWidget= this.sectionWidget;
		if (sectionWidget && sectionWidget.onClose) {
			destroy= sectionWidget.onClose();
		}
		if (destroy) {
			this.destroy();
		}
	},
	
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onClose();
		}		
	},
	
	onCloseMouseOver: function(e) {
		this._closeImage.src = this._closeIconHoverSrc;	
	},
	
	onCloseMouseOut: function(e) {
		this._closeImage.src = this._closeIconSrc;
	}
});

})();
