/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectTeamAreaDialog");

dojo.require("jazz.ui.Dialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {
	
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectTeamAreaDialog", null, {
	
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		
		if (!dojo.isFunction(params.onOk)) {
			throw new Error("Missing function for 'onOk' parameter");
		}
		var widgetParams = {
			primaryTitle: this.messages.dialogTitleSelectTeamArea
		};
		this._onOk= params.onOk;
		this._filterProjectArea= params.filterProjectArea;

		var self= this;
		var onOk= function(attributes) {
			self._onOk(attributes);
			onCancel();
		};
		var onCancel= function() {
			self.floatingPane.close();
		};
		
		this.contentWidget= new com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectTeamAreaDialogContent({
			onOk: onOk, 
			onCancel: onCancel, 
			filterProjectArea: this._filterProjectArea,
			allowProjectArea: params.allowProjectArea
			});
		widgetParams.contentNode = this.contentWidget.domNode;
		this.floatingPane = new jazz.ui.Dialog(widgetParams); 
		this.contentWidget.init();
	},
	destroy: function() {
		if(this.contentWidget) {			
			this.contentWidget.destroy();
		}
		this.contentWidget = null;
		if(this.floatingPane) this.floatingPane.destroy();
		this.floatingPane = null;
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.SelectTeamAreaDialogContent",
	[dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="padding: 1em 1em 1em 1em;">' +
						'<div dojoAttachPoint="_treeNode" class="com-ibm-team-workitem-queryeditor-tree" style="height: 20em; background-color: #FFFFFF; border: 1px solid #CCCCCC;"></div>' +
						'<div style="padding: 1em 0em; float: right;">' +
							'<button dojoAttachPoint="_okCmd" disabled="disabled">${textButtonOK}</button>' +
							'<button dojoAttachPoint="_cancelCmd">${textButtonCancel}</button>' +
						'</div>' +
					' </div>',
	
	constructor: function(params){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.textButtonOK = this.messages.buttonOK;
		this.textButtonCancel = this.messages.buttonCancel;
		this._onOk= params.onOk;
		this._onCancel= params.onCancel;
		this._filterProjectArea= params.filterProjectArea;
		this.allowProjectArea= params.allowProjectArea;
	},
	
	postCreate: function() {
		this.connect(this._okCmd, "onclick", this._doOnOk);
		this.connect(this._cancelCmd, "onclick", this._onCancel);
	},
	
	init: function() {
		var srh= new ServiceResponseHandler(this, "_setTeamAreas", "_handleError");
		WorkItemClient.getProcessAreaHierarchies(srh);
	},
	
	_isProjectArea: function(item) {
		return this._projectAreaSet[item.itemId];
	},
	
	_createTree: function(dataStore) {
		var treeStore= new dojo.data.ItemFileReadStore({
			data: dataStore
		});
		var self= this;
		var tree= new dijit.Tree({
			store: treeStore,
			widgetId: 'processAreaHierarchyTree',
			getIconClass: function(item) {
				if (!item)
					return "";
				if (self._isProjectArea(item))
					return "com-ibm-team-workitem-query-projectarea-icon";
				return "com-ibm-team-workitem-query-teamarea-icon"
			}
		});
		this._tree= tree;
		this._treeNode.appendChild(tree.domNode);
		
		this.connect(tree, "focusNode", this._onFocus);
		this.connect(tree, "_onEnterKey", this._doOnOk);
		this.connect(tree.domNode, "ondblclick", this._doOnOk);
	},
	
	_createDataStore: function(processAreaHierarchy) {
		var store= new Object();
		store.label= "name";
		store.identifier= "itemId";
		store.items= this._filter(processAreaHierarchy);
		return store;
	},
	
	_filter: function(processAreaHierarchy) {
		
		var filter= this._filterProjectArea;
		if (!filter)
			return processAreaHierarchy;
			
		var filtered= new Array();
		for (var i= 0; i < processAreaHierarchy.length; i++) {
			if (processAreaHierarchy[i].itemId == filter)
				filtered.push(processAreaHierarchy[i]);
		}
		return filtered;
	},
	
	_createProjectAreaSet: function(processAreaHierarchy) {
		var set= new Object();
		for (var i= 0; i < processAreaHierarchy.length; i++) {
			set[processAreaHierarchy[i].itemId]= true;
		}
		return set;
	},
	
	_setTeamAreas: function(processAreaHierarchy) {
		this._projectAreaSet= this._createProjectAreaSet(processAreaHierarchy);
		var store= this._createDataStore(processAreaHierarchy);
		this._createTree(store);
	},
	
	_doOnOk: function() {
		var selection= this._getSelectedItem();
		if (selection)
			this._onOk([ selection ]);
	},
	
	_onFocus: function(node) {
		if (node && this._isValidSelection(node.item)) {
			this._currentSelection= node.item;
//			console.debug(node.item);
			this._okCmd.disabled= false;
		} else {
			this._okCmd.disabled= true;
		}
	},
	
	_isValidSelection: function(item) {
		if (this.allowProjectArea)
			return true;
		return !this._isProjectArea(item);
	},
	
	_getSelectedItem: function() {
		return this._currentSelection;
	},
	
	_handleError: function(error) {
		console.error(error);
	}
	
});

})();
