/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

(function() {
	
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TextAreaSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/TextAreaSection.html"),
	
	constructor: function(args){
		this.title= args.title;
		this.editActionName= args.editActionName;
		this.emptyMessage= args.emptyMessage;
		this._textString= "";
	},
	
	postCreate: function() {
		dojox.xml.parser.replaceChildren(this._title, document.createTextNode(this.title));
		dojox.xml.parser.replaceChildren(this._editLink, document.createTextNode(this.editActionName));
		dojox.xml.parser.replaceChildren(this._emptyMessage, document.createTextNode(this.emptyMessage));
		
		this._updateEnablement();
	},
	
	setText: function(text) {
		
		if (!text)
			text= ""; 
		
		this._textString= text;
		this._textArea.value= text;
		dojox.xml.parser.replaceChildren(this._textDiv, document.createTextNode(text));
		
		this._isEditing= false;
		this._updateEnablement();
	},
	
	hasChanged: function() {
		return this._textString != this.getText();
	},
	
	getText: function() {
		return this._textArea.value;
	},
	
	onEdit: function(event) {
		dojo.stopEvent(event);
		this._isEditing= true;
		this._updateEnablement();
	},
	
	_onChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_updateEnablement: function() {
		
		var selectedEntry= this._selectedEntry;
		var index= selectedEntry ? dojo.indexOf(this._entries, selectedEntry) : -1;
		
		this._setVisible(this._emptyMessage, !this._isEditing && !this._textString);
		this._setVisible(this._textDiv, !this._isEditing);
		this._setVisible(this._textArea, this._isEditing);
		if(this._isEditing) this._textArea.focus();
		this._setVisible(this._actionArea, !this._isEditing);
	},
	
	_setVisible: function(node, show) {
		node.style.display= show ? "block" : "none";
	}
	
});

})();
