/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ListWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart");
(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImageClient= com.ibm.team.workitem.web.client.internal.ImageClient;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ListWidget", [dijit._Widget, dijit._Templated, com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/ListWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {

		this.presentation= args.presentation;
		this._selection= args.selection || [];
        this._pageControl = args.pageControl;		
		this._idLookup= new Object();
		this._selectMap= new Object();
		
		this._hasSelection= false;
	},
	postCreate: function() {
		this._priorValues = [];
		this._select.size= 6;
		if(!this.presentation.isOperationAssignment) {
			this._select.multiple= "multiple";
		}
		
		var attributeId= this.presentation.attribute.id;
		var srh= new ServiceResponseHandler(this, "_initValues", "_handleError");
		var args = { newWorkItem: true, ids: [ attributeId ], ignoreDependencies: true, includeArchived: !this.presentation.isOperationAssignment };
		WorkItemClient.getAllValues(srh, args);
	},
	
	/* String[] */ getValues: function() {
		
		if (!this._hasSelection)
			return this._getInitialSelectionValues();
		
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				selection.push(select[i].value);
			}
		}
		return selection;
	},
	handleRowSelectionChanged: function(selectedRows) {
		if(this.isOperationStateList()) {
			this._filterStatusElements(selectedRows);
		}		
	},
	isOperationStateList: function() {
		return this.presentation && this.presentation.isOperationAssignment && 
			this.presentation.attribute && this.presentation.attribute.id && this.presentation.attribute.id == "internalState"
	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	/*
	 * Status is a special case, the values shown change based on selection
	 */
	_filterStatusElements: function(selectedRows) {
		
		if(!selectedRows || selectedRows.length == 0) {
			if(this._pageControl) {
				var results = this._pageControl.getQueryResultsWidget().getResults();
				if(results && results.length > 0) {
					selectedRows = [results[0]]; 
				}
			}
		}
		if(!selectedRows || selectedRows.length == 0) { 
				this._showStatusInfoMessage();
				this._select.style.display = "none";
				return;
			}
		this._select.style.display = "";

		// Use the first selected element to offer status updates
		var id = selectedRows[0].row._jazzRowWorkItemId;
		var workingCopy = com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({
				id: id,
				createIfNeeded: true,
				doNotCache: true
			});	
		if(workingCopy.workItemSpec) {
			this._filterStatusElements3(workingCopy);
			return;
		}
		workingCopy.fetchPresentations({ // editProps only
			    fetchEditorPresentation: false, 
			    fetchInlineEditorPresentation: false,
				onSuccess: dojo.hitch(this, this._filterStatusElements2),
				onError: dojo.hitch(this,this._filterStatusElementsError)
			})
	},

	_filterStatusElements2: function(parms) {
		this._filterStatusElements3(parms.workItemProxy)
	},
	_filterStatusElements3: function(workingCopy) {
			var _possibleValues = [ ];			
			if(workingCopy && workingCopy.workItemSpec && workingCopy.workItemSpec.editProps) {
				var allValues = this.getAllValueAttributeMap(workingCopy.workItemSpec.editProps);
				if(allValues && allValues.actionValues) {
					for(var i=0;i<allValues.actionValues.length; i++) {
						_possibleValues.push(allValues.actionValues[i]);
					}
				}
			}
			
		var selection= new Array();
		for (var i= 0; i < _possibleValues.length; i++) {
			selection.push(_possibleValues[i]);
		}
		// If the list is the same as it was, do nothing
		if(this.presentation.isOperationAssignment) {
			if(this._priorValues.length == selection.length) {
				var isDifferent = false;
				for (var i= 0; i < selection.length; i++) {
					if(this._priorValues[i].id != selection[i].id) {
						isDifferent = true;
						break;
					}
				}
				if(!isDifferent) return;
			}
		}
		this._clearValues();
		this._selection = selection;
		this._selectValues(this._selection);
	},	
	_filterStatusElementsError: function(e) {
		e = e || {};
		msg = e.message || "";
		console.debug("*** ERROR fetching status in ListWidget: "+msg  )
	},
	_showStatusInfoMessage: function() {
		// TODO: show "Select row(s) to enable." 
	},
	_getInitialSelectionValues: function() {
		var values= this._selection;
		var selection= new Array();
		for (var i= 0; i < values.length; i++) {
			selection.push(values[i].id);
		}
		return selection;
	},
	
	_onSelectionChange: function() {
		this._currentSelection = this._select.value;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_initValues: function(/* UIItemSetDTO */ valueSet) {
		this.valueSet = valueSet;
		if (!valueSet || valueSet.length == 0)
			return;
		this._select.length= 0;
		if(!this.isOperationStateList()) {
			this._addValues(valueSet[0].uiItems);
			this._selectValues(this._selection);
		}
		
		var selectedRows = [];
		if(this._pageControl) {			
			selectedRows = this._pageControl.getQueryResultsWidget().getSelectedRows();
		}
		this.handleRowSelectionChanged(selectedRows); // init the display
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback("Loading values failed", e);
	},
	_clearValues: function() {
		dojox.xml.parser.removeChildren(this._select);
		this._selectMap = {};
		this._priorValues = [];
	},
	_addValues: function(/* UIItemDTO[] */ values, /* boolean */ select) {
		if (!values || values.length == 0)
			return;
		
		this._priorValues = values;
		var index= this._select.length;
		for (var i= 0; i < values.length; i++) {
			var option= this._createOption(values[i]);
			this._select[index++]= option;
			this._selectMap[values[i].id]= option;
			var isSelected =  select ? true : false;
			if(!isSelected && this.presentation.isOperationAssignment) {
				isSelected = this._currentSelection == values[i].id;
			}
			option.selected= isSelected;
		}
	},
	
	_selectValues: function(/* UIItemDTO[] */ selection) {
		this._hasSelection= true;
		var missing= new Array();
		for (var i= 0; i < selection.length; i++) {
			var item= selection[i];
			var option= this._selectMap[item.id];
			if (option) {
				option.selected= true;
			} else {
				if (item.arguments) // hack for category
					item.label= item.arguments;
				missing.push(item);
			}
		}
		var isSelection = !(this.presentation.isOperationAssignment == true);
		this._addValues(missing, isSelection);
	},
	
	_createOption: function(/* UIItemDTO */ item) {
		var option= new Option(item.label, item.id, false, false);
		var indent= this._getIndent(item);
		if (indent > 0) {
			option.setAttribute("style", "padding-left: " + indent * 5 + "px");
		}
		if (item.iconUrl) {
			var url= ImageClient.getImageUrl(item.iconUrl);
			option.setAttribute("style", "background-image: url('" + url + "'); padding-left: 20px");
		}
		if (item.disabled) {
			option.setAttribute("disabled", true);
		}
		return option;
	},
	
	_getIndent: function(/* UIItemDTO */ item) {
		var label= item.label;
		var indent= 0;
		while (indent < label.length && label[indent] == ' ') 
			indent++;
		return indent;
	}
	
});
	
})();
