/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.WorkItemWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var SelectWorkItemDialog= com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
var Messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.WorkItemWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/WorkItemWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.presentation= args.presentation;
		this.selection= args.selection;
		this.errorCallback= args.errorCallback;
		this._hasSelection= false;
		
		this.optionCreatorUIItemDTO= function(/* UIItemDTO */ dto) {
			return new Option(dto.label, dto.id, false, false);
		};
		
		this.optionCreatorWorkItemDTO= function(/* WorkItemDTO */ dto) {
			var label= dojo.string.substitute(Messages.labelWorkItem, [dto.id, dto.summary]);
			return new Option(label, dto.workItemItemId, false, false);
		};
	},
	
	postCreate: function() {
		
		this._select.size= 6;
		this._select.multiple= "multiple";

		this._createAddWorkItemButton();
		this._initValues(this.selection);
	},
	
	destroy: function() {
		try {
			if(this.childButton) this.childButton.destroy();
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	
	/* String[] */ getValues: function() {
		
		if (!this._hasSelection)
			return this._getInitialSelectionValues();
			
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				selection.push(select[i].value);
			}
		}
		return selection;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	_getInitialSelectionValues: function() {
		var values= this.selection || [];
		var selection= new Array();
		for (var i= 0; i < values.length; i++) {
			selection.push(values[i].id);
		}
		return selection;
	},
		
	_onSelectionChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_createAddWorkItemButton: function() {
		var self= this;
		var addWorkItemButtonParameters= {
			label: Messages.labelAddWorkItem,
			onClick: function() {
				self._handleAddWorkItem();
			}
		};
		var addWorkItemButton= new dijit.form.Button(addWorkItemButtonParameters);
		this._addWorkItemButton.appendChild(addWorkItemButton.domNode);
		
		this.childButton= addWorkItemButton; 
	},
	
	_handleAddWorkItem: function() {
		var self= this;
		var params= {
				dialogTitle: Messages.dialogTitleAddWorkItems,
				parentElement: this.domNode,
				exclusionList: [],
				onOk: function(selected) {
					self._addWorkItems(selected);
					setTimeout(function() {self.selectWorkItemDialog.destroy(); self.selectWorkItemDialog = null; }, 500);
				},
				onClose: function() {
					setTimeout(function() {self.selectWorkItemDialog.destroy(); self.selectWorkItemDialog = null; }, 500);
				}
			};
		this.selectWorkItemDialog = new SelectWorkItemDialog(params);
			
	},
	
	_addWorkItems: function(/* WorkItemDTO[] */ selected) {
		for (var i= 0; i < selected.length; i++) {
			var workItem= selected[i];
			var workItemOption= this._findWorkItemOption(workItem.workItemItemId);
			if (workItemOption) {
				workItemOption.selected= true;
			} else {
				this._addValues([ workItem ], true, this.optionCreatorWorkItemDTO);
			}
		}
		this._hasSelection= true;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	
	_addValues: function(/* Object[] */ values, /* boolean */ selectValue, /* function */ optionCreator) {
		if (values) {
			var index= this._select.length;
			for (var i= 0; i < values.length; i++) {
				var option= optionCreator(values[i]);
				if (selectValue)
					option.selected= true;
				this._select[index++]= option;
			}
		}
	},
	
	_findWorkItemOption: function(id) {
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].value == id) {
				return select[i];
			}
		}
		return null;
	},
	
	_initValues: function(/* UIItemDTO[] */ valueSet) {
		if (!valueSet || valueSet.length == 0)
			return;
		this._select.length= 0;
		this._addValues(valueSet, true, this.optionCreatorUIItemDTO);
		this._hasSelection= true;
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback(Messages.errorLoadingUsersFailed, e);
	}
	
});
	
})();
