/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.richtext.Editor");

dojo.require("dijit.Editor");
dojo.require("dojo.i18n");
dojo.require("dijit._editor.plugins.FullScreen");
dojo.require("dijit._editor.plugins.AlwaysShowToolbar");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.DropDownButton");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

(function(){

var messages;
var BR = "<br/>";
var EventTopics = com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.Editor",
	dijit.Editor,
{
	/*
	 * Rich text editor that has toolbar specific to Work Item
	 */
	parentController: null,
	
	IFRAME_BODY_CLASS: "RichTextBody",

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");

		this.plugins = [
			"com.ibm.team.workitem.web.ui.internal.view.richtext.FullScreen","|",
			"undo","redo","|",
			//"cut","copy","paste","|",
			"bold","italic",
			//"underline","strikethrough","|","insertOrderedList","insertUnorderedList","indent","outdent","|","justifyLeft","justifyRight","justifyCenter","justifyFull",
//			"com.ibm.team.workitem.web.ui.internal.view.richtext.EnterKeyHandling",
			"|",
			{
				name: "com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.DropDownButton",
				label: messages.insertUserLink,
				iconClass: "dijitEditorIcon com-ibm-team-workitem-UserLink",
				defaultActionIndex: 0, // use AtUserLink as default action
				plugins: [
					"com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.AtUserLink",
					"com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.WorkItemLink",
					"com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.ExternalLink"
				]
			}
		];
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.extraPlugins = [ "dijit._editor.plugins.AlwaysShowToolbar" ];
	
		// enable read-only view source only for debug=true
		var qstr = dojo.global.location.search.substr(1);
		if(qstr.length > 0 && dojo.queryToObject(qstr).debug == "true"){
			dojo["require"]("dijit._editor.plugins.ViewSource"); 
			dojo["require"]("dojox.editor.plugins.PrettyPrint");
			this.extraPlugins.push("|");
			this.extraPlugins.push({name: 'viewsource', readOnly: true});
			
// pretty print inserts unnecessary characters which makes value compare not working, so disable for now
//			this.extraPlugins.push({name: 'prettyprint', indentBy: 3, xhtml: true}); 
		}
		
		// 152240: Font size in rich text editor is too big
		this.styleSheets = dojo.query("head link[rel=\"stylesheet\"]").map(function(link){
			return link.href;
		}).join(";");
	
		this.inherited(arguments);
		
		if(dojo.isIE){
			this.contentPostFilters.push(this._convertP2BR);
		}
	},
	
	_convertP2BR: function(str){
		str = str.replace(/<p>(.*?)<\/p>/gi, "$1"+BR);
		return str;
	},
	
	_removeTrailingBR: function(str){
		str = str.replace(/<br[^<>]*>/gi, BR); // convert <br> to <br/>

		// remove trailing BR
		var idx = str.lastIndexOf(BR);
		if(0 <= idx && idx == (str.length-BR.length)){
			str = str.substring(0, idx);
		}
		return str;
	},
	
	// override
	_getIframeDocTxt: function(){
		var txt = this.inherited("_getIframeDocTxt", arguments);
		if(dojo.isWebKit){
			txt = txt.replace('<html', '<html class="dj_webkit"');
		}
		txt = txt.replace('<body', '<body class="'+this.IFRAME_BODY_CLASS+'"');
		return txt;
	},
/*
 * Repace unsupported tags with SPANs, remove CSS
 */
	_filterDisplay: function(node){
		var supportedTags = [ "b", "i", "br", "a"];
		if(dojo.isIE){
			supportedTags.push("p");
		}
		
		var hasUnsupportedTags = function(node){
			if(node== null || node.parentNode == null) return false;
			if(node.nodeType == 3) return false; // skip text nodes
			if(!node.childNodes || node.childNodes.length == 0) return false;

			var tagName = node.tagName.toLowerCase();
			node.className = ""; // remove css
			if(tagName != 'span' && tagName != "div" && dojo.indexOf(supportedTags, tagName) === -1) { 
				return true;
			}
				
			for(var i=0; i<node.childNodes.length;i++) {
				if(hasUnsupportedTags(node.childNodes[i])) return true;
			};
			return false;
		}
		if(!hasUnsupportedTags(node)) return;
		
		var convertNodeAndChildrenToSpans = function(node, isDrillDown){
			if(node== null || node.parentNode == null) return;			
			if(node.childNodes && node.childNodes.length > 0) {
				dojo.forEach(node.childNodes, function(child) {
					convertNodeAndChildrenToSpans(child, true);
				});
			}
			/* Don't change the first body statement */
			if(isDrillDown || node.tagName.toLowerCase() != "body") {				
				var newSpan = document.createElement("SPAN");
				if(node.nodeType != 3) { // skip text nodes
						var tagName = node.tagName.toLowerCase();
						if(dojo.indexOf(supportedTags, tagName) === -1) { // unsupported tag
							newSpan.innerHTML = node.innerHTML;				
							if(tagName == "p" || tagName == "div" || tagName == "li" || tagName == "td" || tagName == "th" || tagName == "tr") {
								newSpan.appendChild(dojo.create("br"));
							}
							node.parentNode.replaceChild(newSpan,node);
						}
				
					}
				}
		}
		convertNodeAndChildrenToSpans(node, false);

	},
	
	onNormalizedDisplayChanged: function(){
		var node = this.isClosed ? this.domNode : this.editNode;
		if(node){
			if(!this._prevValue || node.innerHTML != this._prevValue){ // check if value changed
				this._filterDisplay(node);
				this._prevValue = node.innerHTML;
			}
		}
		this.inherited(arguments);
	},
	
	save: function(){
		if(this.parentController){
			this.parentController._handleSaveClick();
		}
	},
	
	// override to make it work for Mac
	// 152247: Keyboard bindings for bold and italic need to be Command+B/I for OS X
	onKeyPress: function(e){
		var c = (e.keyChar && e.keyChar.toLowerCase()) || e.keyCode,
			handlers = this._keyHandlers[c],
			args = arguments;

		if(handlers && !e.altKey){
			dojo.some(handlers, function(h){
				if(!(h.shift ^ e.shiftKey) && !(h.ctrl ^ (e.ctrlKey || e.metaKey))){
					if(!h.handler.apply(this, args)){
						e.preventDefault();
					}
					return true;
				}
			}, this);
		}
		
		// When hitting ENTER key, IE inserts <P> tag and Chrome or Safari inserts <DIV> tag, so insert <BR> instead 
		if(e.keyCode === dojo.keys.ENTER){
			if(dojo.isIE){
				e.preventDefault();
				this._inserthtmlImpl("<br/>");
				return true;
			}else if(dojo.isWebKit){
				e.preventDefault();
				this.document.execCommand("insertlinebreak", false); // inserthtml <BR> somehow doesn't seem to work
				return true;
			}
		}

		if(!this._onKeyHitch){
			this._onKeyHitch = dojo.hitch(this, "onKeyPressed");
		}
		setTimeout(this._onKeyHitch, 1);
		return true;
	},
	
	focusAfterOnLoad: function(){
		dojo.addOnLoad(dojo.hitch(this, function(){
			setTimeout(dojo.hitch(this, function(){
				if(this.editNode){
					this.focus();
				}
			}), this.updateInterval);
		}));
	},

	resolveLinks: function(/*String*/ html){
		// TODO : need to stack to command queue
		// detects strings containing '://' and turns them into links
//		var html2 = html.replace(/([ >]?)([^ >]+:\/\/[^ <]+)([ <]?)/gi, '$1<a _synthetic="" href="$2" target="_blank">$2</a>$3');
		var html2 = html;
//		console.debug("resolve links:\n\n"+html+"\n\n"+html2);
		return html2;
	},
	
	/*
	 * Returns raw XHTML string without link elements
	 */
	getRawValue: function(){
		var html = this.attr("value");
		// strip out link elements with '_synthetic_' attributes
		var html2 = html ? html.replace(/<a[^>]* _synthetic[^>]*>([^<]*)<\/a>/gi, "$1") : html;

		html2 = this._removeTrailingBR(html2);

		return html2;
	},
	
	/*
	 * Sets raw XHTML string with link elements displayed in the editor
	 */
	_setRawValueAttr: function(/*String*/ html){
		var html2 = this.resolveLinks(html);
//console.debug("shima setRawValue:\n\n"+html+"\n\n"+html2);
		this.attr("value", html2);
	},
	
	_executeWithoutStyle: function(command, args){
		var ret;
		try{
			var oldValue = this.document.queryCommandValue("styleWithCSS");
			this.document.execCommand("styleWithCSS", false, false);
			ret = this.document.execCommand(command, false, args);
			this.document.execCommand("styleWithCSS", false, oldValue);
		}catch(e){
			try{
				this.document.execCommand("styleWithCSS", false, false);
			}catch(e){
				// throws exception in IE
			}
			ret = this.document.execCommand(command, false, args);
		}
		return ret;
	},
	
	// override to use <b> tag instead of "font-weight:bold" style so that it generates the same html as Eclipse client
	_boldImpl: function(args){
		return this._executeWithoutStyle("bold", args);
	},
	
	// override to use <i> tag instead of "font-style:italic" style so that it generates the same html as Eclipse client
	_italicImpl: function(args){
		return this._executeWithoutStyle("italic", args);
	},

	// methods to make compatible with SelfSizingTextArea
	_setStatusAttr: function(status){
		// TODO
//		console.debug("_setStatusAttr: "+dojo.toJson(status));
	},

	_setWidthAttr: function(width){
		// TODO
//		console.debug("_setWidthAttr: "+width);
	},

	_setHeightAttr: function(height){
		// TODO
//		console.debug("_setHeightAttr: "+height);
	},
	
	_setEnabledAttr: function(isEnabled){
		// TODO
//		console.debug("_setEnabledAttr: "+isEnabled);
	}
});

/*
 * Oveerride to notify resize event
 */
dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.FullScreen",
	dijit._editor.plugins.FullScreen,
{
	_setFullScreen: function(full){
		this.inherited(arguments);

		if(!full){
			setTimeout(function(){
				dojo.publish(EventTopics.WORKITEM_RICH_TEXT_RESIZED, [ { type: 'fullScreenClosed' } ]);
			}, 250);
		}
	},

	_resizeEditor: function(){
		this.inherited(arguments);

		dojo.publish(EventTopics.WORKITEM_RICH_TEXT_RESIZED, [ { type: 'fullScreenShown' } ]);
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.EnterKeyHandling",
	dijit._editor.plugins.EnterKeyHandling,
{
	singleLinePsToRegularPs: function(element){
		if(element){
			element = this.inherited(arguments);
		}
		return element;
	}
});

})();