/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy");
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.richtext.RichTextEditorProxy");

dojo.require("jazz.core.loader");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

(function(){

var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy",
	null,
{
	constructor: function(args){
		this.editorArgs = args.editorArgs;
		this.parentNode = args.parentNode;
		this._props = {};
		this._editor = null;
	},
	
	setProperty: function(key, value){
		this._props[key] = value;
		
		// if editor exists, apply attr value immediately
		// otherwise delay applying attr values until editor is being created
		if(this._editor){
			this._editor.attr(key, value);
		}
	},
	
	getValue: function(){
		if(this._editor){
			return this._editor.getValue();
		}
		return this._value;
	},
	
	setValue: function(value){
		this._value = value;
		
		if(this._editor){
			this._editor.setValue(value);
		}
	},
	
	getEditorNode: function(){
		return (this._editor) ? this._editor.textarea : null;
	},
	
	showEditor: function(){
		if(!this._editor){
			jazz.core.loader.load_async("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget", dojo.hitch(this, function(){
				var editorArgs = this.editorArgs;
				if(editorArgs && !editorArgs.height){
					editorArgs.height = "8.8em";
				}
				var editor = new com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget(editorArgs);
				this._editor = editor;
				this.parentNode.appendChild(editor.domNode);

				if(this._value){
					editor.setValue(this._value);
				}
				var key;
				for(key in this._props){
					editor.attr(key, this._props[key]);
				}

				this.onEditorCreated(editor);
			}));
		}else{
			this._editor.textarea.style.display = "";
		}
	},
	
	hideEditor: function(){
		if(this._editor){
			this._editor.textarea.style.display = "none";
		}
	},
	
	// stub function that can be connected
	onEditorCreated: function(editor){},
	
	focus: function(){
		if(this._editor){
			this._editor.focus();
		}
	},
	
	removeEditor: function(){
		if(this._editor){
			if(this._editor.domNode.parentNode){
				this._editor.domNode.parentNode.removeChild(this._editor.domNode);
			}
			this._editor.destroy();
			this._editor = null;
		}
	},
	
	destroy: function(){
		this.removeEditor();
	}
});

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.RichTextEditorProxy",
	com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy,
{
	constructor: function(args){
		// re-create rich text editor every time view changes (e.g. switch between Overview <-> Links tab, inline editor <-> full editor)
		// since dettaching and re-attaching rich text editor to the DOM tree causes onLoad to trigger which is problematic
		// See	154629: Rich text editor loses content when switching tabs
		// 		155590: Summary text lost in Inline Editor summary after drill down and back
		this._subscription = dojo.subscribe(EventTopics.WORKITEM_VIEW_CHANGED, dojo.hitch(this, function(){
			if(this._editor){
				this.showEditor();
			}
		}));

		this._loading = false;
		this._queue = [];
	},
	
	// destroy and create editors every time show/hide since remove/add to the DOM tree causes problems
	showEditor: function(){
		if(this._editor){
			this.removeEditor();
		}
		this._loading = true;
		if(!this._loadingNode){
			this._loadingNode = EditorUtil.createLoadingNode();
			this.parentNode.appendChild(this._loadingNode);
		}
		jazz.core.loader.load_async("com.ibm.team.workitem.web.ui.internal.view.richtext.Editor", dojo.hitch(this, function(){
			if(this._loadingNode && this._loadingNode.parentNode === this.parentNode){
				this.parentNode.removeChild(this._loadingNode);
			}

			var editor = new com.ibm.team.workitem.web.ui.internal.view.richtext.Editor(this.editorArgs);
			this._editor = editor;
			editor.connect(editor, "onNormalizedDisplayChanged", dojo.hitch(this, function(){
				this._props["rawValue"] = this.getValue();
			}));

			// rich text editor uses iframe so it needs to wait for iframe to finish loading
			this._onLoadConnect = editor.connect(editor, "onLoad", dojo.hitch(this, function(){
				for(var key in this._props){
					editor.attr(key, this._props[key]);
				}
				
				dojo.forEach(this._queue, function(func){
					func();
				});
				this._queue = [];
				this._loading = false;
				
				this.onEditorCreated(this._editor);
				dojo.publish(EventTopics.WORKITEM_ACTION_SHOW_RICH_TEXT, [this._editor]);
				/*
				 * After it is shown, broadcast a message so windows can resize
				 */
				dojo.publish(EventTopics.WORKITEM_RICH_TEXT_RESIZED, [{ editor: this._editor, type: 'editorShown' }]);
				editor.disconnect(this._onLoadConnect);
			}));
			this.parentNode.appendChild(editor.domNode);
		}));
	},
	
	hideEditor: function(){
		this._props["rawValue"] = this.getValue();
		this.removeEditor();
	},
	
	getEditor: function(){
		return this._editor;
	},
	
	getValue: function(){
		if(this._editor && this._editor.isLoaded){
			var val = this._editor.getRawValue();
			val = EditorUtil.cleanRawValue(val);
			if(val == "<br/>"){
				val = "";
			}
			return val;
		}else{
			return (this._props && this._props["rawValue"]) ? this._props["rawValue"] : "";
		}
	},
	
	setValue: function(value){	
		this.setProperty("rawValue", value);
	},
	
	focus: function(){
		if(this._loading){
			// if rich text editor is in middle of loading, delay focusing until editor is loaded
			this._queue.push(dojo.hitch(this, function(){
				this._editor.focusAfterOnLoad();
			}));
		}else{
			this.inherited(arguments);
		}
	},
	
	destroy: function(){
		dojo.unsubscribe(this._subscription);
		this._loading = false;
		this._queue = [];
		this.inherited(arguments);
	}
});

com.ibm.team.workitem.web.ui.internal.view.richtext.getEditorProxyClass = function(){
	var qstr = dojo.global.location.search.substr(1);
	var norichtext = (qstr.length > 0 && dojo.queryToObject(qstr).norichtext == "true");
	var userAgent = dojo.global.navigator.userAgent;
	var isMobile = (userAgent.indexOf("iPad") !== -1 || userAgent.indexOf("iPhone") !== -1 || userAgent.indexOf("Android") !== -1);
	if(norichtext || isMobile){
		return com.ibm.team.workitem.web.ui.internal.view.richtext.EditorProxy;
	}else{
		return com.ibm.team.workitem.web.ui.internal.view.richtext.RichTextEditorProxy;
	}
}

})();