/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.DropDownButton");

dojo.require("dijit._editor._Plugin");
dojo.require("dijit.form.Button");
dojo.require("jazz.core.loader");
dojo.require("jazz.ui.Menu");
dojo.require("dijit.MenuItem");

(function(){

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.richtext.plugins.DropDownButton",
	dijit._editor._Plugin,
{
	// summary:
	//		A rich text editor plugin that generates dropdown button having dropdown menu generated from 'plugins' argument
	//		This plugin itself does not invoke any actions (actions should be specified by 'plugins')
	//		The dropdown menu plugins would be loaded lazilly when dropdown button is clicked for the first time

	// Override _Plugin.buttonClass.   This plugin is controlled by a DropDownButton
	// (which triggers a TooltipDialog).
	buttonClass: dijit.form.DropDownButton,

	// Override _Plugin.useDefaultCommand... processing is handled by this plugin, not by dijit.Editor.
	useDefaultCommand: false,
	
	label: null,	// label to be shown as dropdown button tooltip
	iconClass: null,	// CSS class name for the dropdown button icon

	defaultActionIndex: -1, // specify the index of plugin whose action is to be used as default action
	
	// plugins: Object[]
	//		A list of plugin names (as strings) or instances (as objects)
	//		for this widget.
	//		similar to dijit.Editor#plugins
	//		plugin object should have the following properties :
	//			iconClass : String (optional)
	//			label : String
	//			actionFuncName : Name of the function to be triggered when dropdown menu is clicked
	plugins: [],
	
	constructor: function(/*Object?*/args, /*DomNode?*/node){
		if(this.defaultActionIndex >= 0){
			this.buttonClass = dijit.form.ComboButton;
		}
	},

	getLabel: function(/*String*/key){
		return this.label;
	},

	_initButton: function(){
		if(!this.button){
			if(!this.dropDown){
				// TODO : can we delay instantiation of dijit.Menu until button is clicked for the first time??
				this.dropDown = new jazz.ui.Menu();
				this.dropDown.startup();
			}
			var self = this;
			var props = dojo.mixin({
				label: this.getLabel(),
				showLabel: false,
				iconClass: this.iconClass,
				dropDown: this.dropDown,
				isLoaded: function(){
					return self._loaded;
				},
				loadDropDown: dojo.hitch(this, "_loadPlugins"),
				tabIndex: "-1"
			}, this.params || {});

			if(this.defaultActionIndex >= 0){
				// temporary onClick handler
				// called if button is clicked before loading the menu for the first time
				props.onClick = function(){
					// loading plugins will overwrite onClick handler
					// call actual button onClick handler after plugins are loaded
					self._loadPlugins();
					self.button.onClick();
				};
			}

			this.button = new this.buttonClass(props);
			
		}
	},

	_loadPlugins: function(loadCallback){
		// load plugins and add dropdown menus
		var self = this;
		dojo.forEach(this.plugins, function(pi, idx){
			var name;
			if(dojo.isString(pi)){
				name = pi;
				pi = { name: name };
			}else{
				// Object
				name = pi.name;
			}
			pi.showButton = false;

			jazz.core.loader.load_sync(name);
			self.editor.addPlugin(pi);

			var obj = self.editor._plugins[self.editor._plugins.length-1];
			if(obj){
				var prop = {
					label: obj.label,
					onClick: dojo.hitch(obj, obj.actionFuncName)
				};
				if(obj.iconClass){
					prop.iconClass = obj.iconClass;
				}
				self.dropDown.addChild(new dijit.MenuItem(prop));

				// if defaultActionIndex is specified, use that as default action
				if(self.defaultActionIndex === idx){
					self.button.onClick = dojo.hitch(obj, obj.actionFuncName);
				}
			}			
		});
		
		// menus are loaded, so call callback
		if(loadCallback && dojo.isFunction(loadCallback)){
			loadCallback();
		}
		this._loaded = true;
	},
	
	destroy: function(){
		if(this.dropDown){
			this.dropDown.destroyRecursive();
			this.dropDown = null;
		}
		this.inherited(arguments);
	}
});

})();