/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressLabelProvider");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "StoryMessages");

(function() {

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressLabelProvider", null, {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_progressInfo: null,
	
	constructor: function(progressInfo) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "StoryMessages");
		this._progressInfo= progressInfo;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getHoursTextNode: function() {
		var result= document.createElement("span");
		result.appendChild(this._textNode(this.messages.labelProgress, "type"));
		result.appendChild(this._labelNode(this.getWorkHoursDoneLabel()));
		result.appendChild(this._textNode("/", "numberSeparator"));
		result.appendChild(this._labelNode(this.getWorkHoursLabel()));
		if (this._progressInfo.isRealTimeSpecified()) {
			result.appendChild(this._textNode("|", "numberSeparator"));
			result.appendChild(this._labelNode(this.getDeltaLabel()));
		}
		result.appendChild(this._textNode("h", "numberSeparator"));

		return result;
	},

	getEstimateTextNode: function() {
		var result= document.createElement("span");
		result.appendChild(this._textNode(this.messages.labelEstimated, "type"));
		result.appendChild(this._labelNode(this.getQualityOfPlanningLabel()));

		return result;
	},

	getDeltaLabel: function() {
		var result= this._createDefaultLabel("-", this.messages.titleDifferenceActualToExpected);

		if (this._hasEstimatedWork()) {
			var workHoursDoneDelta= this._progressInfo.getWorkHoursDoneDelta();
			
			result.value= workHoursDoneDelta + "";
			if (workHoursDoneDelta > 0) 
				result.value= "+" + result.value;
			result.className= this._getWorkHoursDoneClass();
		}
		return result;
	},

	getWorkHoursLabel: function() {
		var result= this._createDefaultLabel("0", this.messages.titleTotalEstimatedWorkHours);
		result.value= this._progressInfo.getWorkHours() + "";
		result.className= null;
		return result;
	},

	getWorkHoursDoneLabel: function() {
		var result= this._createDefaultLabel("0", this.messages.titleWorkHoursDone);
		result.value= this._progressInfo.getWorkHoursDone() + "";
		result.className= null;
		return result;
	},

	getQualityOfPlanningLabel: function() {
		var result= this._createDefaultLabel("-", this.messages.titleQualityOfPlanning);

		if (this._hasWork() && !this._progressInfo.isAllWorkDone()) {
			result.value= Math.round(this._progressInfo.getQualityOfPlanning()) + "%";
			result.className= null;
		}
		
		return result;
	},

	getTooltipContent: function() {
		var lines= [];
		lines.push(this.messages.tooltipProgressReport.bold());
		lines.push(dojo.string.substitute(this.messages.tooltipWorkHoursDone, [ ""+this._progressInfo.getWorkHoursDone(), ""+this._progressInfo.getWorkHours()]));

		if (this._progressInfo.isRealTimeSpecified() && !this._progressInfo.isAllWorkDone()) {
			var deltaLabel= null;
			switch (this._getWorkHoursDoneClass()) {
				case "ahead":
					deltaLabel= dojo.string.substitute(this.messages.tooltipAheadBy, [""+this._progressInfo.getWorkHoursDoneExpected(), ""+Math.abs(this._progressInfo.getWorkHoursDoneDelta())]);
					break; 
				case "behind": 
					deltaLabel= dojo.string.substitute(this.messages.tooltipBehindBy, [""+this._progressInfo.getWorkHoursDoneExpected(), ""+Math.abs(this._progressInfo.getWorkHoursDoneDelta())]);
					break; 
				case "ontime": 
					deltaLabel= dojo.string.substitute( this.messages.tooltipOnTime, [""+this._progressInfo.getWorkHoursDoneExpected()]);
					break; 
			}
			if(deltaLabel) lines.push(deltaLabel);
		}
		if (!this._progressInfo.isAllWorkDone()) {
			lines.push("");
			lines.push(dojo.string.substitute(this.messages.tooltipItemsEstimated, [""+Math.round(this._progressInfo.getQualityOfPlanning())]));
			lines.push(dojo.string.substitute(this.messages.tooltipItemsCompleted, [""+this._progressInfo.getCloseCount(),""+this._progressInfo.getCount(), ""+Math.round(this._progressInfo.getClosedRatio())]));
		} else {
			lines.push(this.messages.tooltipItemsCompletedAll);
		}

		return lines.join("<br />");
	},

	getMessage: function() {
		if (this._progressInfo.isRealTimeSpecified() && this._progressInfo.getRealTimeLeft() == 0) {
			return this.messages.labelNoWorkTimeLeft;
		} if (!this._hasWork()) {
			return this.messages.labelNoWorkPlanned;
		} else if (!this._hasEstimatedWork() && this._progressInfo.getOpenCount() > 0) {
			return this.messages.labelNoOpenWorkEstimated;
		}

		return null;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_hasWork: function() {
		return this._progressInfo.getCount() > 0;
	},

	_hasEstimatedWork: function() {
		return this._hasWork() && this._progressInfo.getEstimatedCount() > 0;
	},

	_createDefaultLabel: function(noValue, tooltip) {
		return {
			value: noValue || this.messages.labelNotApplicable,
			className: "na",
			tooltip: tooltip
		};
	},

	_getWorkHoursDoneClass: function() {
		var workHoursProgress= this._progressInfo.getWorkHourProgress();
		var realTimeProgress= this._progressInfo.getRealTimeProgress();
		
		var result = "ontime";
		if (Math.abs(realTimeProgress - workHoursProgress) > 0.02) {
			result= realTimeProgress > workHoursProgress ? "behind" : "ahead";
		}
		return result;
	},
	
	_labelNode: function(label) {
		var result= document.createElement("div");
		dojo.addClass(result, "value");
		dojo.addClass(result, label.className || "");
		result.innerHTML= label.value;
		result.setAttribute("title", label.tooltip || ""); 
		return result;
	},

	_textNode: function(text, className) {
		var result= document.createElement("div");
		dojo.addClass(result, className || "");
		result.appendChild(document.createTextNode(text));
		return result;
	},


	__sentinel: null // terminates this class definition
});

})();
