/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.data.util.sorter");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var comparator = dojo.data.util.sorter.basicComparator;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/type/templates/AddAttributeContent.html"),
	
	name: "",
	attrId: "",
	type: "",
	model: null,
	workItemType: null,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// set "id" & "type" fields to readonly for edit attributes
		if(this.attrId && this.attrId.length>0 && this.type && this.type.length>0){
			this.idField.readOnly = true;
			this.typeSelect.disabled = true;
		}
		
		// populate attribute types dropdown
		var sel = this.typeSelect;
		sel.innerHTML = "";
		var selectedType = this.type;
		this.model.getAttributeTypes(function(configData){
			var enumerationTypes = configData.enumerationTypes || [];
			var types = configData.basicTypes.concat(enumerationTypes);
			types.sort(function(o1, o2){
				// 111126: Script error occurs when adding custom attributes
				// displayName may be null
				var name1 = (o1.displayName) ? o1.displayName : o1.id;
				var name2 = (o2.displayName) ? o2.displayName : o2.id;
				return comparator(name1.toLowerCase(), name2.toLowerCase());
			});
			dojo.forEach(types, function(attrType){
				var displayName = attrType.displayName || attrType.id;
				var opt = dojo.create("option", {
					value: attrType.id,
					innerHTML: displayName
				}, sel);
				
				if(selectedType && selectedType.length > 0){
					if(attrType.id == selectedType){
						opt.selected = true;
					}
				}else if(attrType.id == "string"){
					// make string default
					opt.selected = true;
				}
			});
		});
	},
	
	_handleNameChanged: function(e){
		var id = this.idField.value;
		if(!id || id.length === 0){
			this.idField.value = this.nameField.value;
		}
	},
	
	validate: function(values, callback){
		if(!values.name || values.name.length === 0){
			alert(messages.errAttrNameEmpty);
		}else if(!values.typeId || values.typeId.length === 0){
			alert(messages.errAttrTypeEmpty);
		}else{
			var id = values.id;
			if(!id || id.length === 0){
				alert(messages.errAttrIDEmpty);
			}else{
				var isAttrIDUsed = function(attr){
					return (
						(attr.id == id) ||
						(attr.stringIdentifier == id)
					);
				};
				
				// verify that the attribute id doesn't collide with an permission action name
				var isActionID = function(attr){
					var intPrefix= "internal";
					var prefixLength= intPrefix.length;
					var attrId= attr.id;
					if (attrId && attrId.length > prefixLength && attrId.substring(0, prefixLength) == intPrefix) {
						var actionName= attrId.substring(prefixLength, prefixLength + 1).toLowerCase() + attrId.substring(prefixLength + 1);
						return id == actionName;
					}
					return false;	
				};
				
				// 111584: Creation of identical custom attributes should not be allowed
				this.model.getWorkItemConfig(dojo.hitch(this, function(workItemConfig){
					// check if ID is used in built-in attributes
					var isBuiltInUsed = dojo.some(workItemConfig.builtInAttributes, isAttrIDUsed);
					if(isBuiltInUsed === true){
						alert(messages.errAttrIDUsed);
						return;
					}
					var isActionIdUsed = dojo.some(workItemConfig.builtInAttributes, isActionID);
					if (isActionIdUsed === true){
						alert(messages.errAttrIDUsed);
						return;
					}
					
					// check if ID is used in custom attributes
					var custAttrs = this.model.getCustomAttributes(this.workItemType);
					if(custAttrs){
						var isCustomUsed = dojo.some(custAttrs, isAttrIDUsed);
						if(isCustomUsed === true){
							alert(messages.errAttrIDUsed);
							return;
						}
					}
					
					// ID is not used, so call callback
					callback();
				}));
			}
		}
	},
	
	getValues: function(){
		var sel = this.typeSelect;
		var opt = sel.options[sel.selectedIndex];
		return {
			id: this.idField.value,
			name: this.nameField.value,
			typeId: opt.value,
			typeName: opt.innerHTML
		};
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent.openDialog = function(prop){
	var model = prop.model;
	
	// create dialog content
	var attrId = prop.attrId;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent({
		name: prop.name,
		attrId: attrId,
		type: prop.type,
		model: model,
		workItemType: prop.workItemType
	});
	
	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: prop.title,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.nameField);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			model.getAttributeTypes(function(configData){
				var attrTypeId = values.typeId;
				var enumerationTypes = configData.enumerationTypes || [];
				var isEnum = dojo.some(enumerationTypes, function(enumType){
					return (enumType.id == attrTypeId);
				});
				model.addCustomAttribute(prop.workItemType, {
					id: values.id,
					name: values.name,
					typeId: attrTypeId,
					typeName: values.typeName,
					isEnumeration: isEnum
				});
			});
		}
	});
};

})();