/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/type/templates/AddTypeContent.html"),

	model: null,
	
	postMixInProperties: function(){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages");
		this.inherited(arguments);
	},
	
	postCreate: function(){
		// populate existing categories select
		var options = this.categorySelect.options;
		dojo.forEach(this.model.getAllWorkItemCategories(), function(category, idx){
			options[idx] = new Option(category, category);
		});
		
		this.inherited(arguments);
		
		// add new type category by default
		 this.addCategoryRadio.checked = true;
		 this._onRadioChanged(true);
	},
	
	_handleIdChanged: function(e){
		var id = this.idField.value;
		
		// if name field is empty, use substring of ID as default name
		var name = this.nameField.value;
		if(!name || name.length===0){
			var index = id.lastIndexOf(".");
			if(index != -1){
				name = id.substring(index+1);
			}else{
				name = id;
			}
			this.nameField.value = name;
		}
		
		// if type category field is empty, use type ID as default type category ID
		if(this.addCategoryRadio.checked === true &&
			(!this.addCategoryField.value || this.addCategoryField.value.length===0)){
			if(this._checkDuplicateTypeCategory(id)){
				this.addCategoryField.value = id;
			}
		}
	},
	
	_checkDuplicateTypeCategory: function(categoryId){
		var categories = this.model.getAllWorkItemCategories();
		for(var i=0,len=categories.length; i<len; i++){
			if(categories[i] == categoryId){
				return false;
			}
		}
		return true;
	},
	
	_onRadioChanged: function(isAddCategory){
		this.addCategoryField.readOnly = !isAddCategory;
		this.categorySelect.disabled = isAddCategory;
	},
	
	_handleRadioChanged: function(e){
		var isAddCategory = (e.target == this.addCategoryRadio);
		this._onRadioChanged(isAddCategory);
	},
	
	validate: function(values, callback){
		// validate name
		var name = values.name;
		var msg = this.model.validateTypeName(name);
		if(msg && msg.length>0){
			alert(msg);
			return;
		}
		
		// validate ID
		var id = values.id;
		if(!id || id.length === 0){
			// check empty
			alert(this.messages.errIDEmpty);
			return;
		}else{
			// check if already used			
			var types = this.model.getWorkItemTypes();
			if(types){
				for(var i=0, len=types.length; i<len; i++){
					if(types[i].id == id){
						alert(this.messages.errIDUsed);
						return;
					}
				}
			}
		}
		
		// validate type category
		if(this.addCategoryRadio.checked === true){
			var category = values.category;
			if(!category || category.length === 0){
				alert(this.messages.errCategoryEmpty);
				return;
			}else{
				// check if type category exists
				if(!this._checkDuplicateTypeCategory(category)){
					alert(this.messages.errCategoryUsed);
					return;
				}
			}
		}
		
		callback();
	},
	
	getValues: function(){
		var category = (this.addCategoryRadio.checked === true) ? this.addCategoryField.value : this.categorySelect.value;
		return {
			id: this.idField.value,
			name: this.nameField.value,
			category: category
		};
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent.openDialog = function(prop){
	//create edit icon dialog content
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent({
		model: prop.model
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: contentWidget.messages.AddType,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.idField);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			// add work item type to the model
			prop.model.addType({
				id: values.id,
				category: values.category,
				name: values.name,
				icon: ""
			});
		}
	});
};