/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.type.TypeEditor");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");
dojo.require("dojo.string");
dojo.require("dojo.data.util.sorter");
dojo.require("dijit.InlineEditBox");
dojo.require("jazz.core.loader");
dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var CLASS_DELETE_COMMAND = "delete-command";
var CLASS_EDIT_COMMAND = "edit-command";
var CLASS_INLINE = "dijitInline";
var CLASS_ICON = CLASS_INLINE+" iconNode ";
var CLASS_BUILTIN_ATTR = "builtInAttr";
var CLASS_CUST_ATTR = "custAttr";
var CLASS_ROW_HOVER = "workFlowHover";
var CLASS_BLANK = "dijitInline iconNode blankIcon";

var basicComparator = dojo.data.util.sorter.basicComparator;
var load_async = jazz.core.loader.load_async;
var escapeHtml = jazz.util.html.escape;
var getCellContent = com.ibm.team.workitem.web.process.ui.internal.view.common.Row.prototype.getCellContent;
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;

var messages;
var TextHoverHelp = jazz.app.ua.TextHoverHelp;
var WI_PRESENTATION_BINDING_TARGET_MAIN = "__wi_presentation_binding_target_main"; // corresponds to main editor (target=null)
var CSH_KEYS = {
	TYPES_ATTR_EDITOR: "com.ibm.team.workitem.web.wi_type_web_editor",
	ALIASES: "com.ibm.team.workitem.web.wi_aliases_web",
	PRESENTATION_BINDINGS: "com.ibm.team.workitem.web.wi_present_binding_web_editor",
	"com.ibm.team.workitem.web.inline": "com.ibm.team.workitem.web.presentation_inline",
	"com.ibm.team.workitem.editor.lightweight": "com.ibm.team.workitem.web.presentation_lightweight",
	"com.ibm.team.apt.ui.internal.previewPane": "com.ibm.team.workitem.web.presentation_plan"
};
CSH_KEYS[WI_PRESENTATION_BINDING_TARGET_MAIN] = "com.ibm.team.workitem.web.presentation_main";

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.type.TypeEditor",
	com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor,
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/type/templates/TypeEditor.html"),
	
	title: "",
	pageCshId: CSH_KEYS.TYPES_ATTR_EDITOR,
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages");
		this.messages = messages; // keep it for template strings
		this.inherited(arguments);
		this._warningIconSrc = dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/page/templates/warning_ovr.gif").toString();
	},
	
	postCreate: function(){
		this._presentationBindingTargets = {};

		this.inherited(arguments);

		// setup CSH
		this._childWidgets.push(new TextHoverHelp({
			uaKey: CSH_KEYS.ALIASES,
			contentNode: this.aliasLabel
		}));
		this._childWidgets.push(new TextHoverHelp({
			uaKey: CSH_KEYS.PRESENTATION_BINDINGS,
			contentNode: this.presentationBindingsLabel
		}));
		
		// listen to model change
		var model = this.model;
		var self = this;
		this.connect(model, "onTypeChanged", function(args){
			if(!args){ return; }
			
			if(!args.target){
				// update all types
				self._populateTypesList(false);
			}else{
				var target = args.target;
				var key = args.key;
				if(target == "TYPE_CATEGORY"){
					if(key == "attributes"){
						self._populateAttributes();
					}
				}else if(target == "TYPE"){
					var value = args.value;
					switch(key){
						case "iconUrl":
							this._updateIcon(value);
							break;
						case "name":
							self._populateTypesList(false);
							break;
					}
				}
			}
		});
		this.connect(model, "onWorkflowChanged", function(workflowId){
			if(!workflowId){
				self._populateWorkflows();
			}
		});

		// setup presentation bindings table
		this.presentationBindingsTable.setupColumns([
			{
				id: "bindingLabel",
				className: "bindingLabel",
				getCellContent: function(bindingTarget){
					var label = dojo.create("label", {
						"for": this.id,
						className: "formLabel",
						innerHTML: bindingTarget.displayName
					});
					
					// add CSH widget
					var uaKey = (bindingTarget.target) ? CSH_KEYS[bindingTarget.target] : CSH_KEYS[WI_PRESENTATION_BINDING_TARGET_MAIN];
					if(uaKey){
						self._childWidgets.push(new TextHoverHelp({
							uaKey: uaKey,
							contentNode: label
						}));
					}
					
					return label;
				}
			},
			{
				id: "bindingSelect",
				getCellContent: function(bindingTarget){
					var select = dojo.create("select", {
						id: this.id,
						className: "presentationSelect"
					});
					self.connect(select, "onchange", "_handlePresentationEditorChanged");
					var target = bindingTarget.target || WI_PRESENTATION_BINDING_TARGET_MAIN;
					self._presentationBindingTargets[target] = {
						select: select,
						elementType: bindingTarget.elementType
					};
					return select;
				}
			},
			{
				id: "actions",
				className: "actionCell",
				name: messages.Actions,
				isAction:true
			}
		]);
		this.connect(model, "onPresentationChanged", function(){
			self._populateEditorPresentations(function(){
				// select binding after editor presentation bindings dropdown have been re-populated
				var id = self.getSelectedType();
				if(!id){ return; }
				var typeInfo = model.getWorkItemType(id);
				if(!typeInfo || !typeInfo.editorBindings){ return; }

				dojo.forEach(typeInfo.editorBindings, function(binding){
					var target = binding.target || WI_PRESENTATION_BINDING_TARGET_MAIN;
					var sel = self._presentationBindingTargets[target].select;
					sel.value = binding.id;
				});
			}, false);
		});
		
		// setup workflow list
		var workflowRow = this.workflowRow;
		this.connect(workflowRow, "onmouseover", function(){
			dojo.toggleClass(workflowRow, CLASS_ROW_HOVER, true);
		});
		this.connect(workflowRow, "onmouseout", function(){
			dojo.toggleClass(workflowRow, CLASS_ROW_HOVER, false);
		});
		var editWorkflowIcon = createActionIcon({
			iconClass: "edit-command editWorkflow",
			onClick: dojo.hitch(this, "_handleEditWorkflow"),
			title: messages.EditWorkflow,
			owner: this
		});
		dojo.place(editWorkflowIcon, this.workflowSelect, "after");

		// setup attributes table
		this.attrTable.setupColumns([
			{
				id: "isBuiltIn",
				name: " ",
				className: "isBuiltInCell",
				getCellContent: function(cellData){
					if(this.data.isBuiltIn === true){
						return dojo.create("span", {
							className: CLASS_ICON+CLASS_BUILTIN_ATTR,
							title: messages.BuiltInAttr
						});
					}else{
						return dojo.create("span", {
							className: CLASS_ICON+CLASS_CUST_ATTR,
							title: messages.CustAttr
						});
					}
				}
			},
			{
				id: "name",
				name: messages.AttrName,
				getCellContent: function(cellData){
					var isBuiltInAttr = this.data.isBuiltIn; // this is a Row widget
					var attrId = this.data.id;
					var editBox = new dijit.InlineEditBox({
						title: cellData,
						onChange: function(value){
							if(!value || value.length===0){
								this.attr("value", cellData);
								this.edit();
								alert(messages.errAttrNameEmpty);
								return;
							}

							if(isBuiltInAttr === true){
								// built-in attributes
								if(confirm(messages.warnBuiltin)){
									model.setBuiltInAttributeProperty(attrId, "name", value);
								}else{
									this.attr("value", cellData);
									this.edit();
								}
							}else{
								// custom attributes
								var typeId = self.getSelectedType();
								model.setCustomAttributeProperty(typeId, attrId, "name", value);
							}
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:cellData }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "typeName",
				name: messages.AttrType,
				style: {
					width: "20%"
				},
				getCellContent: function(cellData){
					// this is a Row widget
					if(this.data.isEnumeration === true){
						return dojo.create("a", {
							innerHTML: cellData,
							href: "#",
							onclick: dojo.hitch(this, function(e){
								dojo.stopEvent(e);
								self.editEnumeration(this.data.typeId); // this is a Row widget
							})
						});
					}else{
						return getCellContent.apply(this, arguments);
					}
				}
			},
			{
				id: "id",
				style: {
					width: "35%"
				},
				name: messages.AttrID
			},
			{ id:"actions", className:"actionCell", name:messages.Actions, isAction:true }
		]);
		this.connect(this.model, "onEnumerationChanged", "_populateAttributes");

		// connect to SELECT onchange event programmatically since connecting in template somehow doesn't work...
		this.connect(this.typesList, "onchange", "_handleTypeSelected");
		this.connect(this.workflowSelect, "onchange", "_handleWorkflowChanged");

		// load work items
		this._populateTypesList(false);
	},
	
	_handleFinalChanged: function(){
		this.model.setTypeFinal(this.finalCheckbox.checked);
	},
	
	_populateTypesList: function(forceLoad){
		// show loading message
		this.setLoading(true);
		var typesList = this.typesList;
		var self = this;
		this.model.getWorkItemConfig(dojo.hitch(this, function(workItemConfig){
			// hide loading message
			this.setLoading(false);

			if(forceLoad || !this._isInit){
				this.processCommonConfigData(workItemConfig);
			}
			
			this._isInit = true;
			
			this._builtInAttrs = workItemConfig.builtInAttributes;
			var types = workItemConfig.types;

			// check previous selection to restore selection
			var options = typesList.options;
			var selectedId = null;
			if(types && types.length > 0){
				if(options.length > 0){
					if(types.length > options.length){
						// type has been added, so select added option
						selectedId = types[types.length-1].id;
					}else if(types.length < options.length){
						// type has been removed, so select first option
						selectedId = types[0].id;
					}else{
						// otherwise, select same option
						selectedId = options[options.selectedIndex].value;
					}
				}else{
					selectedId = types[0].id;
				}
			}
			
			// clean-up the previous list
			options.length = 0;

			// populate work item type list
			if(types && types.length > 0){
				// populate updated types list
				dojo.forEach(types, function(typeInfo, idx){
					options[idx] = new Option(typeInfo.name, typeInfo.id);
				});
				this.removeBtn.disabled = false;
			}else{
				this.removeBtn.disabled = true;
			}
			
			// build presentation bindings table
			this._createPresentationBindingsTable(workItemConfig.presentationBindingTargets, function(){
				self.selectType(selectedId);
			}, forceLoad);
		}), forceLoad);
	},
	
	refresh: function(){
		this._populateTypesList(true);
	},
	
	_populateEditorPresentations: function(callback, forceLoad){
		this.model.getEditorPresentations(dojo.hitch(this, function(presentations){
			for(var target in this._presentationBindingTargets){
				var targetInfo = this._presentationBindingTargets[target];
				var sel = targetInfo.select;
				sel.options.length = 0; // clear previous options
				var elementType = targetInfo.elementType;

				// add "None" option for elementTypes other than "editor" (editor has default)
				if(elementType != "editor"){
					sel.options[0] = new Option(messages.None, "");
				}
				
				// populate dropdown for that elementType (editor/tab/section)
				dojo.forEach(presentations[elementType], function(def){
					sel.options[sel.options.length] = new Option(def.id, def.id);
				}, this);
			}
			
			// select work item type after presentation data has been loaded
			callback();
		}), forceLoad);		
	},
	
	_createPresentationBindingsTable: function(presentationBindingTargets, callback, forceLoad){
		var self = this;
		
		// create table
		var presentationBindingsTable = this.presentationBindingsTable;
		presentationBindingsTable.clearItems();
		dojo.forEach(presentationBindingTargets, function(bindingTarget){
			presentationBindingsTable.addItem({
				bindingLabel: bindingTarget,
				bindingSelect: bindingTarget,
				actions: [{
					iconClass: CLASS_EDIT_COMMAND,
					title: messages.EditPresentation,
					onClick: function(){
						self._handleEditPresentation(bindingTarget.target);
					}
				}]
			});
		}, this);
		
		// populate dropdown
		this._populateEditorPresentations(callback, forceLoad);
	},

	_handleAddType: function(e){
		// create edit icon dialog content
		var model = this.model;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.type.AddTypeContent.openDialog({
				model: model
			});
		});
	},
	
	_handleRemoveType: function(){
		var id = this.getSelectedType();
		if(id && id.length>0){
			var typeInfo = this.model.getWorkItemType(id);
			if(confirm(dojo.string.substitute(messages.warnRemoveType, [typeInfo.name]))){
				this.model.removeType(id);
			}
		}
	},
	
	getSelectedType: function(){
		// returns work item type ID for selected work item type
		var options = this.typesList.options;
		return (options.length > 0) ? options[options.selectedIndex].value : null;
	},
	
	selectType: function(typeId){
		if(!typeId || this.getSelectedType() == typeId){
			this._handleTypeSelected();
		}else{
			var options = this.typesList.options;
			for(var i=0, len=options.length; i<len; i++){
				if(options[i].value == typeId){
					options[i].selected = true;
					this._handleTypeSelected();
					break;
				}
			}
		}
	},
	
	_handleTypeSelected: function(){
		var id = this.getSelectedType();
		
		// if item is deseleted, hide the right pane
		this.detailsSection.style.display = (id) ? "" : "none";
		if(!id){
			return;
		}

		// update ID field
		var typeInfo = this.model.getWorkItemType(id);
		if(!typeInfo){
			return;
		}
		
		this.workItemIdField.innerHTML = escapeHtml(id);
		
		// update name field
		this.workItemNameField.value = typeInfo.name;
		
		// update type category field
		this.workItemCategoryField.innerHTML = escapeHtml(typeInfo.category);
		
		// update icon image
		this._updateIcon(typeInfo.iconUrl);
		
		// update alias
		var aliasStr = "";
		dojo.forEach(typeInfo.aliases, function(aliasName, idx){
			if(idx > 0){
				aliasStr += ", ";
			}
			aliasStr += aliasName;
		});
		this.aliasField.value = aliasStr;
		
		// update editor presentation bindings
		var editorBindings = typeInfo.editorBindings;
		for(var target in this._presentationBindingTargets){
			// find binding ID for that target
			var bindingId = "";
			if(editorBindings){ // some types don't have editorBindings
				for(var j=0, len2=editorBindings.length; j<len2; j++){
					if(
						(target == WI_PRESENTATION_BINDING_TARGET_MAIN && (!editorBindings[j].target || editorBindings[j].target.length === 0)) ||
						editorBindings[j].target == target){
						bindingId = editorBindings[j].id;
						break;
					}
				}
			}

			var sel = this._presentationBindingTargets[target].select;
			sel.value = bindingId;
		}
		
		// update workflow bindings
		this._populateWorkflows();
		this._updateAffectedTypesMessage(this.workflowWarnNode, messages.warnWorkflowAffected);
		
		// update custom attributes
		this._updateAffectedTypesMessage(this.attrWarnNode, messages.warnCustomAttrAffected);
		this._populateAttributes();
	},
	
	_populateWorkflows: function(){
		var id = this.getSelectedType();
		var typeInfo = this.model.getWorkItemType(id);
		var selectedWorkflowId = (typeInfo) ? typeInfo.workflowId : "";
		var workflowSelect = this.workflowSelect;
		var bFound = false;
		this.model.getWorkflows(dojo.hitch(this, function(workflows){
			workflowSelect.options.length = 0;
			dojo.forEach(workflows.workflowDefinitions, function(def, idx){
				var workflows = def.workflows;
				var displayName = (workflows && workflows.length>0 && workflows[0].name && workflows[0].name.length>0) ? workflows[0].name : def.id;
				workflowSelect.options[idx] = new Option(displayName, def.id);
				if(def.id == selectedWorkflowId){
					workflowSelect.options[idx].selected = true;
					bFound = true;
				}
			}, this);
			if(!bFound && workflowSelect.options.length > 0){
				// when new work item type has been added, set first workflow as default
				this.model.setTypeProperty(typeInfo.id, "workflowId", workflowSelect.options[0].value);
			}
		}));
	},
	
	_updateIcon: function(iconUrl){
		var iconNode = this.workItemIcon;
		iconNode.innerHTML = "";
		
		if(iconUrl && iconUrl.length > 0){
			var index = iconUrl.lastIndexOf("/");
			var title = (index != -1) ? iconUrl.substring(index+1) : iconUrl;
			dojo.create("img", {
				src: iconUrl,
				alt: title,
				title: title
			}, iconNode);
		}else{
			dojo.create("span", {
				className: CLASS_BLANK,
				title: messages.NoIcon
			}, iconNode);
		}
	},
	
	_validateTypeName: function(e){
		var name = this.workItemNameField.value;
		var msg = this.model.validateTypeName(name);
		if(msg && msg.length>0){
			this.displayErrorMessage(msg);
		}else{
			this.model.setTypeProperty(this.getSelectedType(), "name", name);
		}
	},
	
	_handleEditIcon: function(e){
		// create edit icon dialog content
		var type = this.getSelectedType();
		var selectedPath = null;
		var dimmedicon = null;
		var model = this.model;
		if(type){
			 var typeInfo = model.getWorkItemType(type);
			 if(typeInfo){
				 selectedPath = typeInfo.icon;
				 dimmedicon = typeInfo.dimmedicon;
			 }
		}
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
				prefix: "/workitemtype",
				model: model,
				selectedPath: selectedPath,
				onOK: function(values){
					var selectedType = self.getSelectedType();
					model.setTypeProperty(selectedType, "iconUrl", values.iconUrl);
					model.setTypeProperty(selectedType, "icon", values.icon);
					// 111120: "dimmedicon" value will be lost when saving work item configurations from Web UI
					// if icon is changed, should clear dimmedicon
					// see com.ibm.team.workitem.ide.ui.internal.aspecteditor.type.TypePart#editEvent
					if(dimmedicon && selectedPath != values.icon){
						model.setTypeProperty(selectedType, "dimmedicon", null);
					}
				}
			});
		});
	},
	
	_handleAliasChanged: function(e){
		var aliases = [];
		var aliasStr = this.aliasField.value;
		if(aliasStr && aliasStr.length>0){
			dojo.forEach(aliasStr.split(","), function(item){
				if(item){
					item = dojo.trim(item);
					if(item.length>0){
						aliases.push(item);
					}
				}
			});
		}
		this.model.setTypeProperty(this.getSelectedType(), "aliases", aliases);
	},
	
	_handlePresentationEditorChanged: function(e){
		var sel = e.target;

		// find target
		var target = null;
		for(var targetId in this._presentationBindingTargets){
			if(this._presentationBindingTargets[targetId].select == sel){
				if(targetId != WI_PRESENTATION_BINDING_TARGET_MAIN){ 
					// target should be null for WI_PRESENTATION_BINDING_TARGET_MAIN
					target = targetId;
				}
				break;
			}
		}
		
		var typeId = this.getSelectedType();
		var typeInfo = this.model.getWorkItemType(typeId);
		if(!typeInfo){ return; }
		if(!typeInfo.editorBindings){
			typeInfo.editorBindings = [];
		}
		var newBinding = sel.options[sel.selectedIndex].value;
		for(var j=0, len2=typeInfo.editorBindings.length; j<len2; j++){
			if(typeInfo.editorBindings[j].target == target){
				typeInfo.editorBindings[j].id = newBinding;
				this.model.setTypeProperty(typeId, "editorBindings", typeInfo.editorBindings);
				return;
			}
		}
		
		// not found, so add
		typeInfo.editorBindings.push({
			_eQualifiedClassName: "com.ibm.team.workitem.process.dto:EditorBinding",
			target: target,
			id: newBinding
		});
		this.model.setTypeProperty(typeId, "editorBindings", typeInfo.editorBindings);
	},
	
	_handleEditPresentation: function(target){
		var targetId = target || WI_PRESENTATION_BINDING_TARGET_MAIN;
		var targetInfo = this._presentationBindingTargets[targetId];
		
		var model = this.model;
		var selectedId = targetInfo.select.value;
		var editors = [];
		var elementType = targetInfo.elementType;
		switch(elementType){
			case "editor":
				// full-blown editor
				editors = [ selectedId ];
				break;
			case "tab":
				// tab may be shared between multiple editors
				editors = model.getEditorsForTabId(selectedId);
				break;
			case "section":
				// section may be shared between multiple editors
				editors = model.getEditorsForSectionId(selectedId);
				break;
		}
		
		if(editors.length == 1){
			// only single editor, so simply open it
			var tabId = null;
			if(elementType == "tab"){
				tabId = selectedId;
			}else if(elementType == "section"){
				var tabs = model.getAllTabs();
				for(var i=0, len=tabs.length; i<len; i++){
					if(tabs[i].childSections){
						for(var j=0, len2=tabs[i].childSections.length; j<len2; j++){
							if(tabs[i].childSections[j].sectionId == selectedId){
								tabId = tabs[i].id;
								break;
							}
						}
					}
					if(tabId){
						break;
					}
				}
			}
			this.editPresentation(editors[0], tabId);
		}else if(editors.length > 1){
			// multiple editors, so ask user to select one
			var self = this;
			load_async("com.ibm.team.workitem.web.process.ui.internal.view.type.SelectPresentationContent", function(){
				com.ibm.team.workitem.web.process.ui.internal.view.type.SelectPresentationContent.openDialog({
					model: model,
					elementType: elementType,
					selectedId: selectedId,
					editors: editors,
					onOK: dojo.hitch(self, "editPresentation")
				});
			});
		}else{
			alert(messages.NoEditorPresentation);
		}
	},
	
	editPresentation: function(/*String*/ editorId, /*String?*/ tabId){},

	_handleWorkflowChanged: function(e){
		var sel = this.workflowSelect;
		var workflowId = sel.options[sel.selectedIndex].value;
		var model = this.model;
		model.setTypeProperty(this.getSelectedType(), "workflowId", workflowId);
		// set workflowId for types that belong to the same type category
		dojo.forEach(model.getAffectedTypes(this.getSelectedType()), function(typeInfo){
			model.setTypeProperty(typeInfo.id, "workflowId", workflowId);
		});
	},
	
	_updateAffectedTypesMessage: function(warnNode, i18nStr){
		// show a warning message if there are affected types
		var affectedTypes = this.model.getAffectedTypes(this.getSelectedType());
		if(affectedTypes.length === 0){
			warnNode.style.display = "none";
		}else{
			var typeStr = "";
			dojo.forEach(affectedTypes, function(typeInfo, index){
				if(index>0){
					typeStr += ", ";
				}
				typeStr += "<b>"+typeInfo.name+"</b>";
			});
			if(!warnNode._popup){
				warnNode._popup = new jazz.ui.HoverPopup({
					around: [ warnNode ]
				});
				this._childWidgets.push(warnNode._popup);
			}
			warnNode._popup.onOpen = dojo.hitch(warnNode._popup, function(){
				this.content = dojo.create("span", {
					innerHTML: dojo.string.substitute(" "+i18nStr, [typeStr])
				});
			});
			warnNode.style.display = "";
		}
	},

	_handleEditWorkflow: function(){
		var sel = this.workflowSelect;
		this.editWorkflow(sel.options[sel.selectedIndex].value);
	},
	
	editWorkflow: function(workflowId){},
	
	_populateAttributes: function(){
		var attrTable = this.attrTable;
		attrTable.clearItems();
		
		var self = this;
		var model = this.model;
		
		model.getAttributeTypes(function(){
		
			var getAttributesArray = function(attrs, isBuiltIn){
				return dojo.map(attrs, function(attr){
					var attrId = attr.id;
					var name = attr.name;
					if(isBuiltIn){
						var attrDef = model.findAttributeDefinition(attrId);
						if(attrDef){
							// if attribute definition is specified, use that as label
							name = attrDef.name;
						}
					}
					
					var typeId = attr.typeId;
					var obj = {
						id: attrId,
						name: name,
						typeId: typeId,
						isEnumeration: attr.isEnumeration,
						isBuiltIn: isBuiltIn,
						actions: []
					};
					if(attr.isEnumeration){
						var dispName = model.getDisplayName(typeId);
						if(dispName){
							obj.typeName = dojo.string.substitute(messages.EnumType, [ dispName ]);
						}else{
							obj.typeName = attr.typeName;
						}
					}else{
						obj.typeName = attr.typeName;
					}
					
					if(isBuiltIn !== true){
						obj.actions.push({
							iconClass: CLASS_DELETE_COMMAND,
							title: messages.Remove,
							onClick: function(e){
								if(confirm(dojo.string.substitute(messages.confirmRemoveAttr, [name]))){
									var typeId = self.getSelectedType();
									model.removeCustomAttribute(typeId, attrId);
								}
							}
						});
					}
					return obj;
				}).sort(function(o1, o2){
					return basicComparator(o1.name.toLowerCase(), o2.name.toLowerCase());
				});
			};
		
			// add custom attributes
			var attrs = model.getCustomAttributes(self.getSelectedType());
			if(attrs && attrs.length > 0){
				dojo.forEach(getAttributesArray(attrs, false), dojo.hitch(attrTable, "addItem"));
			}
		
			// add built-in attributes
			if(self._builtInAttrs && self._builtInAttrs.length>0){
				dojo.forEach(getAttributesArray(self._builtInAttrs, true), function(attr){
					var row = attrTable.addItem(attr);
					dojo.addClass(row.domNode, "builtInRow");
				});
			}		
		
		}, false);
	},
	
	editEnumeration: function(enumId){},
	
	_handleAddAttr: function(e){
		dojo.stopEvent(e);
		
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent.openDialog({
				title: messages.AddAttrDlgTitle,
				name: "",
				attrId: "",
				type: "",
				model: self.model,
				workItemType: self.getSelectedType()
			});
		});
	},
	
	_handleCustomOnlyChanged: function(e){
		dojo.toggleClass(this.attrTable.containerNode, "customAttrOnly", (e.target.checked === true));
	},
	
	destroy: function(preserveDom){
		if(this.presentationBindingsTable){
			this.presentationBindingsTable.destroy(preserveDom);
		}
		if(this.attrTable){
			this.attrTable.destroy(preserveDom);
		}
		this.inherited(arguments);
	}
});

})();