/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent",
	com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent,
{
	prefix: "/workflow",
	workflowId: null,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages");
		this.workflowMessages = messages;
		this.inherited(arguments);
	},
		
	postCreate: function(){
		this.inherited(arguments);
		var table = dojo.create("table", {
			className: "workflowDetailsTable"
		});
		var tbody = dojo.create("tbody", null, table);
		
		// add "name" field
		var tr = dojo.create("tr", null, tbody);
		var fieldId = this.id+"_nameField";
		var th = dojo.create("th", null, tr);
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.NameLabel
		}, th);
		dojo.create("span", {
			className: "requiredFlag",
			innerHTML: "*"
		}, th);
		this.nameField = dojo.create("input", {
			id: fieldId
		}, dojo.create("td", null, tr));
		
		// add "description" field
		tr = dojo.create("tr", null, tbody);
		fieldId = this.id+"_descriptionField";
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.DescriptionLabel
		}, dojo.create("th", {
			style: {
				verticalAlign: "top"
			}
		}, tr));
		this.descriptionField = dojo.create("textarea", {
			id: fieldId,
			rows: 2
		}, dojo.create("td", null, tr));

		// add "Target State" field
		tr = dojo.create("tr", null, tbody);
		fieldId = this.id+"_targetState";
		th = dojo.create("td", null, tr);
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.TargetStateLabel
		}, th);
		dojo.create("span", {
			className: "requiredFlag",
			innerHTML: "*"
		}, th);
		var sel = dojo.create("select", {
			id: fieldId
		}, dojo.create("td", null, tr));
		this.targetStateField = sel;
		sel.options[0] = new Option(messages.None, "");
		var workflow = this.model.getWorkflowById(this.workflowId);
		dojo.forEach(workflow.states, function(state, idx){
			sel.options[idx+1] = new Option(state.name, state.id);
		});

		dojo.place(table, this.domNode, "first");		
	},
	
	getValues: function(){
		var values = this.inherited(arguments);
		values.name = this.nameField.value;
		values.description = this.descriptionField.value;
		values.targetState = this.targetStateField.value;
		return values;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent.openDialog = function(prop){
	// create edit icon dialog content
	var model = prop.model;
	var workflowId = prop.workflowId;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent({
		workflowId: workflowId,
		model: model
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.workflowMessages.AddActionTitle,
		onOpen: function(){
			dijit.focus(contentWidget.nameField);
		},
		validate: function(values, callback){
			var name = values.name;
			if(!name || name.length === 0){
				alert(contentWidget.workflowMessages.errActionNameEmpty);
				return;
			}
			
			var targetState = values.targetState;
			if(!targetState || targetState.length === 0){
				alert(contentWidget.workflowMessages.ErrTargetStateEmpty);
				return;
			}
			
			callback();
		},
		onOK: function(values){
			var workflowInfo = model.getWorkflowById(workflowId);
			if(!workflowInfo){ return; }

			var maxIndex = 1;
			var actions = workflowInfo.actions;
			if(actions && actions.length > 0){
				var indexArray = dojo.map(actions, function(action){
					var actionId = action.id;
					if(actionId && actionId.length>0){
						var index = actionId.lastIndexOf(".");
						if(index != -1){
							actionId = actionId.substring(index+2);
						}
						var intId = parseInt(actionId, 10);
						if(!isNaN(intId)){
							return intId;
						}
					}
					return 0;
				}).sort(function(num1, num2){return num1 - num2});
				maxIndex = indexArray[indexArray.length-1]+1;
			}

			// auto-generate id like {workflow ID}.action.a{n} 
			model.addAction(workflowId, {
				id: workflowId+".action.a"+maxIndex, 
				name: values.name,
				description: values.description,
				state: values.targetState,
				iconUrl: values.iconUrl,
				icon: values.icon
			});
		}
	});
};

})();