/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages"); //$NON-NLS-1$ //$NON-NLS-2$

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent",
	com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent,
{
	prefix: "/workflow",

	postMixInProperties: function(){
		this.workflowMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages");
		this.inherited(arguments);
	},
		
	postCreate: function(){
		this.inherited(arguments);
		var table = dojo.create("table", {
			className: "workflowDetailsTable"
		});
		var tbody = dojo.create("tbody", null, table);
		
		// add "name" field
		var tr = dojo.create("tr", null, tbody);
		var fieldId = this.id+"_resolutionNameField";
		var th = dojo.create("th", null, tr);
		dojo.create("label", {
			"for": fieldId,
			innerHTML: this.workflowMessages.NameLabel
		}, th);
		dojo.create("span", {
			className: "requiredFlag",
			innerHTML: "*"
		}, th);
		this.resolutionNameField = dojo.create("input", {
			id: fieldId
		}, dojo.create("td", null, tr));
		
		// add "description" field
		tr = dojo.create("tr", null, tbody);
		fieldId = this.id+"_resolutionDescriptionField";
		dojo.create("label", {
			"for": fieldId,
			innerHTML: this.workflowMessages.DescriptionLabel
		}, dojo.create("th", {
			style: {
				verticalAlign: "top"
			}
		}, tr));
		this.resolutionDescriptionField = dojo.create("textarea", {
			id: fieldId,
			rows: 2
		}, dojo.create("td", null, tr));

		dojo.place(table, this.domNode, "first");		
	},
	
	getValues: function(){
		var values = this.inherited(arguments);
		values.resolutionName = this.resolutionNameField.value;
		values.resolutionDescription = this.resolutionDescriptionField.value;
		return values;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent.openDialog = function(prop){
	// create edit icon dialog content
	var model = prop.model;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent({
		model: model
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: contentWidget.workflowMessages.AddResolution,
		onOpen: function(){
			dijit.focus(contentWidget.resolutionNameField);
		},
		validate: function(values, callback){
			var resolutionName = values.resolutionName;
			if(resolutionName && resolutionName.length>0){
				callback();
			}else{
				alert(contentWidget.workflowMessages.errResolutionNameEmpty);
			}
		},
		onOK: function(values){
			var workflowId = prop.workflowId;
			var workflowInfo = model.getWorkflowById(workflowId);
			if(!workflowInfo){ return; }

			var maxIndex = 1;
			var resolutions = workflowInfo.resolutions;
			if(resolutions && resolutions.length > 0){
				var indexArray = dojo.map(resolutions, function(resolution){
					var resolutionId = resolution.id;
					if(resolutionId && resolutionId.length>0){
						var index = resolutionId.lastIndexOf(".");
						if(index != -1){
							resolutionId = resolutionId.substring(index+2);
						}
						var intId = parseInt(resolutionId, 10);
						if(!isNaN(intId)){
							return intId;
						}
					}
					return 0;
				}).sort(function(num1, num2){return num1 - num2});
				maxIndex = indexArray[indexArray.length-1]+1;
			}

			// auto-generate id like {workflow ID}.resolution.r{n} 
			model.addResolution(workflowId, {
				id: workflowId+".resolution.r"+maxIndex, 
				name: values.resolutionName,
				description: values.resolutionDescription,
				iconUrl: values.iconUrl,
				icon: values.icon
			});
		}
	});
};