/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent");

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent",
	com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent,
{
	prefix: "/workflow",

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages");
		this.workflowMessages = messages;
		this.inherited(arguments);
	},
		
	postCreate: function(){
		this.inherited(arguments);
		var table = dojo.create("table", {
			className: "workflowDetailsTable"
		});
		var tbody = dojo.create("tbody", null, table);
		
		// add "name" field
		var tr = dojo.create("tr", null, tbody);
		var fieldId = this.id+"_stateNameField";
		var th = dojo.create("th", null, tr);
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.NameLabel
		}, th);
		dojo.create("span", {
			className: "requiredFlag",
			innerHTML: "*"
		}, th);
		this.stateNameField = dojo.create("input", {
			id: fieldId
		}, dojo.create("td", null, tr));
		
		// add "description" field
		tr = dojo.create("tr", null, tbody);
		fieldId = this.id+"_stateDescriptionField";
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.DescriptionLabel
		}, dojo.create("th", {
			style: {
				verticalAlign: "top"
			}
		}, tr));
		this.stateDescriptionField = dojo.create("textarea", {
			id: fieldId,
			rows: 2
		}, dojo.create("td", null, tr));

		// add "group" field
		tr = dojo.create("tr", null, tbody);
		fieldId = this.id+"_stateGroupField";
		dojo.create("label", {
			"for": fieldId,
			innerHTML: messages.GroupLabel
		}, dojo.create("td", null, tr));
		this.stateGroupField = dojo.create("select", {
			id: fieldId
		}, dojo.create("td", null, tr));

		var options = this.stateGroupField.options;
		options[0] = new Option(messages.None, "");
		dojo.forEach(this.model.getStateGroups(), function(group, idx){
			options[idx+1] = new Option(group.name, group.id);
		});

		dojo.place(table, this.domNode, "first");		
	},
	
	getValues: function(){
		var values = this.inherited(arguments);
		values.stateName = this.stateNameField.value;
		values.stateDescription = this.stateDescriptionField.value;
		values.stateGroup = this.stateGroupField.value;
		return values;
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent.openDialog = function(prop){
	// create edit icon dialog content
	var model = prop.model;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent({
		model: model
	});

	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		dialogContent: contentWidget,
		primaryTitle: messages.AddState,
		onOpen: function(){
			dijit.focus(contentWidget.stateNameField);
		},
		validate: function(values, callback){
			var stateName = values.stateName;
			if(stateName && stateName.length>0){
				callback();
			}else{
				alert(messages.errStateNameEmpty);
			}
		},
		onOK: function(values){
			var workflowInfo = model.getWorkflowById(prop.workflowId);
			if(!workflowInfo){ return; }
			
			var maxIndex = 1;
			var states = workflowInfo.states;
			if(states && states.length > 0){
				var indexArray = dojo.map(states, function(state){
					var stateId = state.id;
					if(stateId && stateId.length>0){
						var index = stateId.lastIndexOf(".");
						if(index != -1){
							stateId = stateId.substring(index+2);
						}
						var intId = parseInt(stateId, 10);
						if(!isNaN(intId)){
							return intId;
						}
					}
					return 0;
				}).sort(function(num1, num2){return num1 - num2});
				maxIndex = indexArray[indexArray.length-1]+1;
			}

			// auto-generate id like {workflow ID}.state.s{n} 
			var uniqueId = prop.workflowId+".state.s"+maxIndex;
			model.addState(prop.workflowId, {
				id: uniqueId, 
				name: values.stateName,
				description: values.stateDescription,
				group: values.stateGroup,
				iconUrl: values.iconUrl,
				icon: values.icon
			});
		}
	});
};

})();