/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.workflow.WorkflowEditor");

(function(){

dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor");
dojo.require("dojo.string");
dojo.require("dijit.InlineEditBox");
dojo.require("jazz.core.loader");
dojo.require("jazz.util.html");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Select");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.Table");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages;
var DEFAULT_DISPLAY_MAX = 10;
var NONE = "none"; //$NON-NLS-1$
var CLASS_DELETE_COMMAND = "delete-command";
var CLASS_INLINE = "dijitInline";  //$NON-NLS-1$
var CLASS_BLANK = "blankIcon";
var CLASS_TARGET_STATE = "dijitInline iconNode targetStateIcon";
var CSH_KEYS = {
	WORKFLOW_EDITOR: "com.ibm.team.workitem.web.wi_workflows_web_editor"
};

var substitute = dojo.string.substitute;
var InlineEditBox = dijit.InlineEditBox;
var escapeHtml = jazz.util.html.escape;
var load_async = jazz.core.loader.load_async;
var workflowActions = [ "startAction", "resolveAction", "reopenAction" ];
var createActionIcon = com.ibm.team.workitem.web.process.ui.internal.view.common.utils.createActionIcon;
var descNoValueIndicator;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.workflow.WorkflowEditor",
	com.ibm.team.workitem.web.process.ui.internal.view.common.AbstractEditor,
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/workflow/templates/WorkflowEditor.html"),
	pageCshId: CSH_KEYS.WORKFLOW_EDITOR,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "WorkflowMessages");
		this.messages = messages;
		descNoValueIndicator = '<span class="dijitInline noValueIndicator">'+messages.ClickToAddDesc+'</span>';
		this.title = messages.WorkflowEditorTitle; // this would be used as tab title
		this.inherited(arguments);
	},

	postCreate: function(){
		this._isInit = false;
		this._transitionConnects = [];

		this.inherited(arguments);
		
		var self = this;
		dojo.forEach(workflowActions, function(attachPoint){
			var select = self[attachPoint];
			self.connect(select, "onchange", function(e){
				var propName = attachPoint+"Id";
				self.model.setWorkflowProperty(self.getSelectedWorkflow(), propName, e.target.value);
			});
		});
		
		this._setupStateTable();
		this._setupActionTable();
		this._setupResolutionTable();

		this._populateWorkflowList(false); // forceLoad should be false as it may get loaded in other place (Types & Attributes page)

		// connect to SELECT onchange event programmatically since connecting in template somehow doesn't work...
		this.connect(this.workflowList, "onchange", "_handleWorkflowSelected");
		this.connect(this.model, "onWorkflowChanged", function(workflowId){
			if(workflowId){
				self._handleWorkflowSelected();
			}else{
				self._populateWorkflowList(false);
			}
		});
	},
	
	_populateWorkflowList: function(forceLoad){
		this.setLoading(true);
		var workflowList = this.workflowList;
		this.model.getWorkflows(dojo.hitch(this, function(configData){
			this.setLoading(false);

			if(forceLoad || !this._isInit){
				this.processCommonConfigData(configData);
			}
			this._isInit = true;

			var workflowDefs = configData.workflowDefinitions;
			
			// check previous selection to restore selection
			var options = workflowList.options;

			var selectedId = null;
			if(this._selectedWorkflowId){
				// handle a case for selectWorkflow has been called before workflow list has been populated
				selectedId = this._selectedWorkflowId;
			}else if(workflowDefs && workflowDefs.length > 0){
				if(options.length > 0){
					if(workflowDefs.length > options.length){
						// type has been added, so select added option
						selectedId = workflowDefs[workflowDefs.length-1].id;
					}else if(workflowDefs.length < options.length){
						// type has been removed, so select first option
						selectedId = workflowDefs[0].id;
					}else{
						// otherwise, select same option
						selectedId = options[options.selectedIndex].value;
					}
				}else{
					selectedId = workflowDefs[0].id;
				}
			}

			// clean-up the previous list
			options.length = 0;

			if(workflowDefs && workflowDefs.length > 0){
				// populate updated types list
				dojo.forEach(workflowDefs, function(workflowInfo, idx){
					var workflowId = workflowInfo.id;
					var workflows = workflowInfo.workflows;
					var displayName = (workflows && workflows.length>0 && workflows[0].name && workflows[0].name.length>0) ?
						workflows[0].name : workflowId;
					options[idx] = new Option(displayName, workflowId);
				}, this);
				this.selectWorkflow(selectedId);
				this.removeBtn.disabled = false;
			}else{
				// no items, so clear selection
				this.selectWorkflow(null);
				this.removeBtn.disabled = true;
			}
			this._selectedWorkflowId = null;
		}), forceLoad);
	},

	refresh: function(){
		this._populateWorkflowList(true);
	},

	_handleFinalChanged: function(){
		this.model.setWorkflowFinal(this.finalCheckbox.checked);
	},
	
	getSelectedWorkflow: function(){
		var options = this.workflowList.options;
		return (options.length > 0) ? options[options.selectedIndex].value : null;
	},
	
	selectWorkflow: function(workflowId){
		if(!workflowId || this.getSelectedWorkflow() == workflowId){
			this._handleWorkflowSelected();
		}else{
			if(this._isInit){
				// workflow list is already loaded, so select it immediately
				var options = this.workflowList.options;
				for(var i=0, len=options.length; i<len; i++){
					if(options[i].value == workflowId){
						options[i].selected = true;
						this._handleWorkflowSelected();
						break;
					}
				}
			}else{
				// workflow list is not yet loaded, so wait until it gets loaded
				this._selectedWorkflowId = workflowId;
			}
		}
	},

	_handleWorkflowSelected: function(){
		var workflowId = this.getSelectedWorkflow();

		// hide right section if no workflow is selected
		this.detailsSection.style.display = (workflowId) ? "" : "none";

		var workflowInfo = this.model.getWorkflowById(workflowId);
		if(!workflowInfo){ return; }
		
		this.workflowNameField.value = workflowInfo.name || "";
		this.workflowIdCell.innerHTML = escapeHtml(workflowId);
		this.workflowDescriptionField.value = workflowInfo.description || "";
		
		this._populateTransitions(workflowInfo);
		this._populateStates(workflowId, workflowInfo);
		this._populateActions(workflowId, workflowInfo);
		this._populateResolutions(workflowId, workflowInfo);
	},

	_handleAddWorkflow: function(e){
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddWorkflowContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddWorkflowContent.openDialog({
				model: self.model
			});
		});
	},
	
	_handleRemoveWorkflow: function(){
		var workflowId = this.getSelectedWorkflow();
		if(workflowId && workflowId.length>0){
			var workflowInfo = this.model.getWorkflowById(workflowId);
			var workflowName = (workflowInfo && workflowInfo.name) ? workflowInfo.name : workflowId;
			// TODO : need to check usage of this workflow in work item types. see RCP client behavior
			if(confirm(substitute(messages.ConfirmRemoveWorkflow, [workflowName]))){
				this.model.removeWorkflow(workflowId);
			}
		}
	},
	
	_handleWorkflowNameChanged: function(e){
		var workflowId = this.getSelectedWorkflow();
		this.model.setWorkflowProperty(workflowId, "name", this.workflowNameField.value);
	},
	
	_handleWorkflowDescriptionChanged: function(e){
		var workflowId = this.getSelectedWorkflow();
		this.model.setWorkflowProperty(workflowId, "description", this.workflowDescriptionField.value);
	},
	
	_handleAddState: function(e){
		dojo.stopEvent(e);
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddStateContent.openDialog({
				workflowId: self.getSelectedWorkflow(),
				model: self.model
			});
		});
	},
	
	_handleAddAction: function(e){
		dojo.stopEvent(e);
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddActionContent.openDialog({
				workflowId: self.getSelectedWorkflow(),
				model: self.model
			});
		});
	},
	
	_handleAddResolution: function(e){
		dojo.stopEvent(e);
		var self = this;
		load_async("com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent", function(){
			com.ibm.team.workitem.web.process.ui.internal.view.workflow.AddResolutionContent.openDialog({
				workflowId: self.getSelectedWorkflow(),
				model: self.model
			});
		});
	},
	
	_populateTransitions: function (workflowInfo, editHandler, removeHandler, showLess) {
		var self = this;
		var transitionHeader = this.transitionHeader;
		if (!transitionHeader) { return; }
		dojo.forEach(this._transitionConnects, dojo.hitch(this, "disconnect"));
		this._transitionConnects = [];
		dojo.empty(transitionHeader);
		dojo.empty(self.transitionArea);

		var states =  workflowInfo.states || [];
		var actions = workflowInfo.actions || [];
		
		// first row / first column
		var tr = dojo.create("tr", null, transitionHeader); //$NON-NLS-1$
		dojo.create("th", {
			innerHTML: messages.FromLabel,
			className: "transitionFromTo transitionFrom"
		}, tr); //$NON-NLS-1$
		dojo.create("th", {
			innerHTML: messages.ToLabel,
			className: "transitionFromTo transitionTo"
		}, tr); //$NON-NLS-1$

		if (states !== null && states.length > 0) {
			var getHeaderContent = function(state){
				var iconUrl = state.iconUrl;			
				var htmlStr = (iconUrl && iconUrl.length > 0) ?
					"<img align='top' src='" + iconUrl + "' style='margin-right: 5px;' />" :
					"<span class='dijitInline iconNode "+CLASS_BLANK+"' style='margin-right: 5px;'></span>";
				htmlStr += escapeHtml(state.name);
				return htmlStr;
			};
		
			// first row / rest of the columns
			var len = states.length;
			var s;
			for(var i = 0; i < len; i++){
				s = states[i];
				dojo.create("th", {
					innerHTML: getHeaderContent(s), 
					className: "transitionHeader"
				}, tr);
			}

			// content
			var trArray = [];
			for(i = 0; i < len; i++){
				trArray[i] = dojo.create("tr"); //$NON-NLS-1$
				// First column (State names)
				s = states[i];
				dojo.create("th", {
					innerHTML: getHeaderContent(s),
					colSpan: "2", 
					className: "transitionHeader"
				}, trArray[i]);
				if (showLess && i >= DEFAULT_DISPLAY_MAX) {
					tr.style.display = NONE;
				}
				self.transitionArea.appendChild(trArray[i]);
			}
			
			// column = target state 
			var td;
			for (i = 0; i < len; i++) {
				// rest of the columns
				var actionOptions = dojo.filter(actions, function(a){
					if (a.state === states[i].id) {
						return true;
					}
					return false;
				});

				// row = from state
				for (var l = 0; l < len; l++) {
					var actionName = messages.NoAction;
					td = dojo.create("td", {
						noWrap: "true",
						className: "transitionCol"
					}, trArray[l]); //$NON-NLS-1$
					if (l == i || !actionOptions) {
						dojo.style(td, {
							textAlign: "center"
						});
						td.innerHTML = escapeHtml(actionName);
					}else{
					
						var select = dojo.create("select", null, td);
						select.options[0] = new Option(messages.None, "");
						var model = this.model;
						dojo.forEach(actionOptions, function(o){
							var option = new Option(o.name, o.id);
							option.selected = dojo.indexOf(states[l].actionIds || [], o.id) >= 0;
							if(option.selected){
								select.__prevSelected = o.id;
							}
							select.options[select.options.length] = option;
						});

						// show icon for selected action
						var iconUrl = (select.__prevSelected) ?
							model.getActionProperty(self.getSelectedWorkflow(), select.__prevSelected, "iconUrl") : null;
						var iconNode;
						if(iconUrl && iconUrl.length>0){
							iconNode = dojo.create("img", {
								src: iconUrl,
								className: CLASS_TARGET_STATE
							});
						}else{
							iconNode = dojo.create("span", {
								className: CLASS_TARGET_STATE
							});
						}
						dojo.place(iconNode, select, "before");

						var stateId = states[l].id;
						select.__stateId = stateId;
						self._transitionConnects.push(self.connect(select, "onchange", function(e){
							var targetSelect = e.target;
							var val = targetSelect.value;
							if(val && val.length > 0){
								model.setStateAction(self.getSelectedWorkflow(), targetSelect.__stateId, targetSelect.__prevSelected, val);
							}else{
								model.removeStateAction(self.getSelectedWorkflow(), targetSelect.__stateId, targetSelect.__prevSelected);
							}
							targetSelect.__prevSelected = val;
						}));
					}
				}
			}

			// add invisible blank image to prop header column width
			tr = dojo.create("tr", null, self.transitionArea);
			td = dojo.create("td", { colSpan: 2 }, tr);
			dojo.create("img", {
				src: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/page/templates/blank.png").toString(),
				alt: "",
				title: "",
				style: {
					display: "block", // without this, there'll be unnecessary gaps on top and bottom
					width: "100px",
					height: "0px"
				}
			}, td);
			dojo.create("td", { colSpan: states.length }, tr);			
		}
	},

	_setupStateTable: function(){
		var self = this;
		var model = this.model;
		this.stateTable.setupColumns([
			{
				id: "iconUrl",
				name: messages.IconColumn,
				className: "iconCell",
				getCellContent: function(cellData){
					var selectedIcon = this.data.icon;
					var stateId = this.data.id;
					var params = {
						onClick: function(){
							load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
								com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
									prefix: "/workflow",
									model: model,
									selectedPath: selectedIcon,
									onOK: function(values){
										var workflowId = self.getSelectedWorkflow();
										model.setStateProperty(workflowId, stateId, {
											icon: values.icon,
											iconUrl: values.iconUrl
										});
									}
								});
							});
						},
						title: messages.EditIcon,
						owner: this
					};
					if(cellData && cellData.length>0){
						params.iconUrl = cellData;
					}else{
						params.iconClass = CLASS_BLANK;
					}
					return createActionIcon(params);
				}
			},
			{
				id: "name",
				name: messages.StateName,
				style: {
					width: "10%"
				},
				getCellContent: function(cellData){
					var stateId = this.data.id;
					var editBox = new InlineEditBox({
						title: cellData,
						onChange: function(value){
							if(!value || value.length===0){
								this.attr("value", cellData);
								this.edit();
								alert(messages.errStateNameEmpty);
								return;
							}
							var workflowId = self.getSelectedWorkflow();
							model.setStateProperty(workflowId, stateId, "name", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:cellData }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "group",
				name: messages.StateGroup,
				style: {
					width: "12%"
				},
				getCellContent: function(cellData){
					var stateId = this.data.id;
					var stateGroups = model.getStateGroups();
					var label;
					if(cellData && cellData.length){
						var bFound = false;
						for(var i=stateGroups.length-1; i>=0; i--){
							if(stateGroups[i].id == cellData){
								label = stateGroups[i].name;
								bFound = true;
								break;
							}
						}
						if(!bFound){
							label = messages.None;
						}
					}else{
						label = messages.None;
					}
					var options = [{
						label: messages.None,
						value: ""
					}];
					options = options.concat(dojo.map(stateGroups, function(group){
						var ret = {
							label: group.name,
							value: group.id
						};
						if(cellData == group.id){
							ret.selected = true;
						}
						return ret;
					}));
					var editBox = new InlineEditBox({
						editor: "com.ibm.team.workitem.web.process.ui.internal.view.common.Select",
						editorParams: {
							options: options
						},
						title: label,
						onChange: function(value){
							var workflowId = self.getSelectedWorkflow();
							model.setStateProperty(workflowId, stateId, "group", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:label }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "showResolution",
				name: messages.ShowResolution,
				className: "booleancell",
				getCellContent: function(cellData){
					var stateId = this.data.id;
					return dojo.create("input", {
						type: "checkbox",
						checked: cellData,
						onclick: function(e){
							model.setStateProperty(self.getSelectedWorkflow(), stateId, "showResolution", e.target.checked);
						}
					});
				}
			},
			{
				id: "description",
				name: messages.Description,
				getCellContent: function(cellData){
					var stateId = this.data.id;
					var label = cellData || "";
					var editBox = new InlineEditBox({
						title: label,
						noValueIndicator: descNoValueIndicator,
						onChange: function(value){
							var workflowId = self.getSelectedWorkflow();
							model.setStateProperty(workflowId, stateId, "description", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:label }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "workflowActions",
				name: messages.WorkflowActions,
				getCellContent: function(cellData){
					var workflowId = self.getSelectedWorkflow();
					var stateId = this.data.id;
					var stateInfo = model.getStateById(workflowId, stateId);
					var div = null;
					if(stateInfo && stateInfo.actionIds && stateInfo.actionIds.length > 0){
						div = dojo.create("div");
						var actionIds = stateInfo.actionIds;
						var str = "";
						dojo.forEach(actionIds, function(actionId){
							if(str.length>0){
								str += ", ";
							}
							str += model.getActionProperty(workflowId, actionId, "name");
						});
						if(str.length > 0){
							str += " ";
						}
						dojo.create("span", {
							innerHTML: str
						}, div);
					
						// edit workflow actions icon
						if(stateInfo.actionIds.length > 1){
							// only show pencil icon if there are more than 1 actions
							div.appendChild(createActionIcon({
								iconClass: "edit-dlg-command",
								onClick: function(){
									load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox", function(){
										com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox.openDialog({
											title: messages.EditActionOrder,
											orderOnly: true,
											valueList: dojo.map(actionIds, function(actionId){
												return {
													label: model.getActionProperty(workflowId, actionId, "name"),
													value: actionId
												};
											}),
											onOK: function(values){
												model.setStateProperty(workflowId, stateId, "actionIds", values);
											}
										});
									});
								},
								title: messages.EditActionOrder,
								owner: this
							}));
						}
					}
					return div;
				}
			},
			{ id:"actions", name:messages.Actions, isAction:true }
		]);
	},
	
	_populateStates: function (workflowId, workflowInfo) {
		var stateTable = this.stateTable;
		stateTable.clearItems();
		var model = this.model;
		if(workflowInfo.states){
			var states = dojo.map(workflowInfo.states, function(state){
				state.actions = [{
					iconClass: CLASS_DELETE_COMMAND,
					title: messages.RemoveState,
					onClick: function(e){
						// TODO : need to show "just show affected items" button
						var removeActionsText = ""; 
						var workflow = model.getWorkflowById(workflowId);
						if (workflow){
							var actions = workflow.actions;
							if(actions){
								for(var i=0, len=actions.length; i<len; i++){
									if(actions[i].state == state.id){
										removeActionsText += substitute(messages.RemoveStateActionList, [actions[i].name]);
									}
								}
							}
						}
						var message = "";
						if (removeActionsText.length === 0) {
							message = substitute(messages.ConfirmRemoveState, [state.name]);
						}else{
							var text = substitute(messages.RemoveStateWarning, [state.name]);
							message = substitute(messages.RemoveStateConfirmation, [text, removeActionsText]);
						}
						if(confirm(message)){
							model.removeState(workflowId, state.id);
						}
					}
				}];
				return state;
			});
			dojo.forEach(states, dojo.hitch(stateTable, "addItem"));
		}
	},
		
	_setupActionTable: function(){
		var self = this;
		var model = this.model;
		this.actionTable.setupColumns([
			{
				id: "iconUrl",
				name: messages.IconColumn,
				className: "iconCell",
				getCellContent: function(cellData){
					var selectedIcon = this.data.icon;
					var actionId = this.data.id;
					var params = {
						onClick: function(){
							load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
								com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
									prefix: "/workflow",
									model: model,
									selectedPath: selectedIcon,
									onOK: function(values){
										var workflowId = self.getSelectedWorkflow();
										model.setActionProperty(workflowId, actionId, {
											icon: values.icon,
											iconUrl: values.iconUrl
										});
									}
								});
							});
						},
						title: messages.EditIcon,
						owner: this
					};
					if(cellData && cellData.length>0){
						params.iconUrl = cellData;
					}else{
						params.iconClass = CLASS_BLANK;
					}
					return createActionIcon(params);
				}
			},
			{
				id: "name",
				name: messages.ActionName,
				style: {
					width: "12%"
				},
				getCellContent: function(cellData){
					var actionId = this.data.id;
					var editBox = new InlineEditBox({
						title: cellData,
						onChange: function(value){
							if(!value || value.length===0){
								this.attr("value", cellData);
								this.edit();
								alert(messages.errActionNameEmpty);
								return;
							}
							var workflowId = self.getSelectedWorkflow();
							model.setActionProperty(workflowId, actionId, "name", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:cellData }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "state",
				name: messages.ActionTarget,
				style: {
					width: "15%"
				},
				getCellContent: function(cellData){
					// TargetState is not editable in RCP client somehow
					var div = dojo.create("div", {
						className: "targetStateDiv"
					});

					var workflowId = self.getSelectedWorkflow();
					var state = model.getStateById(workflowId, cellData);
					
					// update state icon
					if(state){
						if(state.iconUrl && state.iconUrl.length>0){
							dojo.create("img", {
								src: state.iconUrl,
								className: CLASS_TARGET_STATE
							}, div);
						}else{
							dojo.create("span", {
								className: CLASS_TARGET_STATE+" "+CLASS_BLANK
							}, div);
						}
						div.appendChild(dojo.doc.createTextNode(state.name));
					}
					
					return div;
				}
			},
			{
				id: "description",
				name: messages.Description,
				getCellContent: function(cellData){
					var actionId = this.data.id;
					var label = cellData || "";
					var editBox = new InlineEditBox({
						title: label,
						noValueIndicator: descNoValueIndicator,
						onChange: function(value){
							var workflowId = self.getSelectedWorkflow();
							model.setActionProperty(workflowId, actionId, "description", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:label }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "resolutions",
				name: messages.ResolutionsColumnHeader,
				className: "resolutionCell",
				getCellContent: function(cellData){
					var workflowId = self.getSelectedWorkflow();
					var actionId = this.data.id;
					var resolutions = model.getActionProperty(workflowId, actionId, "resolutions");
					var groupMap = {};
					dojo.forEach(resolutions, function(resolutionRef){
						var group = resolutionRef.group;
						if(group){
							groupMap[resolutionRef.id] = group;
						}
					});
					
					var _handleEditResolution = function(){
						var valueList = [];
						var choiceList = dojo.map(model.getAllResolutions(workflowId), function(resolution){
							return {
								label: resolution.name,
								value: resolution.id
							};
						});
						dojo.forEach(resolutions, function(resolutionRef){
							for(var i=choiceList.length-1; i>=0; i--){
								if(choiceList[i].value == resolutionRef.id){
									valueList.push(choiceList[i]);
									choiceList.splice(i, 1);
									break;
								}
							}
						});
						load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox", function(){
							com.ibm.team.workitem.web.process.ui.internal.view.common.DoubleListBox.openDialog({
								title: messages.SelectResolutionsTitle,
								choiceList: choiceList,
								valueList: valueList,
								onOK: function(values){
									var newResolutions = dojo.map(values, function(value){
										var ret = { id:value };
										var group = groupMap[value];
										if(group){
											ret.group = group;
										}
										return ret;
									});
									model.setActionProperty(workflowId, actionId, "resolutions", newResolutions);
								}
							});
						});
					};
					
					var node;
					if(resolutions && resolutions.length > 0){
						node = dojo.create("div");
						var str = "";
						dojo.forEach(resolutions, function(resolution){
							if(str.length>0){
								str += ", ";
							}
							str += model.getResolution(workflowId, resolution.id).name;
						});
						if(str.length > 0){
							str += " ";
						}
						dojo.create("span", {
							innerHTML: str
						}, node);

						// edit resolution icon
						node.appendChild(createActionIcon({
							iconClass: "edit-dlg-command",
							onClick: _handleEditResolution,
							title: messages.EditActionResolutions,
							owner: this
						}));

					}else{
						node = dojo.create("a", {
							href: "#",
							innerHTML: messages.ClickToEditResolutionsDesc,
							onclick: function(e){
								dojo.stopEvent(e);
								_handleEditResolution();
							}
						});
					}
					
					return node;
				}
			},
			{ id:"actions", name:messages.Actions, isAction:true }
		]);
	},
	
	_populateActions: function(workflowId, workflowInfo) {
		var actionTable = this.actionTable;
		actionTable.clearItems();
		var model = this.model;
		var actions = workflowInfo.actions;
		
		// update start/resolve/reopen action dropdown
		dojo.forEach(workflowActions, function(attachPoint){
			// clear previous options first
			var options = this[attachPoint].options;
			options.length = 0;
			options[0] = new Option(messages.None, "");
			
			if(actions){
				var dupMap = {};
				dojo.forEach(actions, function(action){
					var name = action.name;
					if(dupMap[name] == "false"){
						dupMap[name] = "true";
					}else{
						dupMap[name] = "false";
					}
				});
				dojo.forEach(actions, function(action, idx){
					var name = action.name;
					var label = name;
					
					// check if there's a duplicate action name. If there is, show label as "${actionName} -> ${targetState}"
					if(dupMap[name] == "true"){
						var state = model.getStateById(workflowId, action.state);
						if(state){
							label += " -> "+state.name;
						}
					}

					options[idx+1] = new Option(label, action.id);
				
					var propName = attachPoint+"Id";
					if(workflowInfo[propName] == action.id){
						options[idx+1].selected = true;
					}
				});
			}
			
		}, this);
		
		// populate actions table
		if(workflowInfo.actions){
			var actionArray = dojo.map(actions, function(action){
				action.actions = [{
					iconClass: CLASS_DELETE_COMMAND,
					title: messages.RemoveAction,
					onClick: function(e){
						if(confirm(substitute(messages.ConfirmRemoveAction, [action.name]))){
							model.removeAction(workflowId, action.id);
						}
					}
				}];
				return action;
			});		
			dojo.forEach(actionArray, dojo.hitch(actionTable, "addItem"));		
		}
	},
	
	_setupResolutionTable: function(){
		var self = this;
		var model = this.model;
		this.resolutionTable.setupColumns([
			{
				id: "iconUrl",
				name: messages.IconColumn,
				className: "iconCell",
				getCellContent: function(cellData){
					var selectedIcon = this.data.icon;
					var resolutionId = this.data.id;
					var params = {
						onClick: function(){
							load_async("com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent", function(){
								com.ibm.team.workitem.web.process.ui.internal.view.common.IconSelectionContent.openDialog({
									prefix: "/workflow",
									model: model,
									selectedPath: selectedIcon,
									onOK: function(values){
										var workflowId = self.getSelectedWorkflow();
										model.setResolutionProperty(workflowId, resolutionId, {
											icon: values.icon,
											iconUrl: values.iconUrl
										});
									}
								});
							});
						},
						title: messages.EditIcon,
						owner: this
					};
					if(cellData && cellData.length>0){
						params.iconUrl = cellData;
					}else{
						params.iconClass = CLASS_BLANK;
					}
					return createActionIcon(params);
				}
			},
			{
				id: "name",
				name: messages.ResolutionName,
				style: {
					width: "25%"
				},
				getCellContent: function(cellData){
					var resolutionId = this.data.id;
					var editBox = new InlineEditBox({
						title: cellData,
						onChange: function(value){
							if(!value || value.length===0){
								this.attr("value", cellData);
								this.edit();
								alert(messages.errResolutionNameEmpty);
								return;
							}

							var workflowId = self.getSelectedWorkflow();
							model.setResolutionProperty(workflowId, resolutionId, "name", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:cellData }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{
				id: "description",
				name: messages.Description,
				getCellContent: function(cellData){
					var resolutionId = this.data.id;
					var label = cellData || "";
					var editBox = new InlineEditBox({
						title: label,
						noValueIndicator: descNoValueIndicator,
						onChange: function(value){
							var workflowId = self.getSelectedWorkflow();
							model.setResolutionProperty(workflowId, resolutionId, "description", value);
						}
					}, dojo.create("div", { className:CLASS_INLINE, innerHTML:label }));
					dojo.style(editBox.domNode, {
						width: "100%"
					});
					return editBox.domNode;
				}
			},
			{ id:"actions", name:messages.Actions, isAction:true }
		]);
	},
	
	_populateResolutions: function (workflowId, workflowInfo) {
		var resolutionTable = this.resolutionTable;
		resolutionTable.clearItems();
		var model = this.model;
		var resolutions = [];
		if(workflowInfo.resolutions){
			resolutions = dojo.map(workflowInfo.resolutions, function(resolution){
				resolution.actions = [{
					iconClass: CLASS_DELETE_COMMAND,
					title: messages.RemoveResolution,
					onClick: function(e){
						// TODO : need to show "just show affected items" button
						if(confirm(substitute(messages.ConfirmRemoveResolution, [resolution.name]))){
							model.removeResolution(workflowId, resolution.id);
						}
					}
				}];
				return resolution;
			});
		}
		dojo.forEach(resolutions, dojo.hitch(resolutionTable, "addItem"));		
	},

	destroy: function(preserveDom){
		this.inherited(arguments);
	}
});

})();