/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui2.internal.action.AbstractAction"); //$NON-NLS-1$
/*
 * This abstract class defines the methods that should be implemented when
 * creating an Action extension, such as a work item editor toolbar button.
 */

(function(){

	var DEFAULT_CONTEXT = "com.ibm.team.workitem.web"; //$NON-NLS-1$
    dojo.declare("com.ibm.team.workitem.web.ui2.internal.action.AbstractAction", null, { //$NON-NLS-1$
    	/*
    	 * Constructor is called when the action is created, such as when it is assigned to the
    	 * toolbar button, or when the button is pressed.
    	 * Be sure to end the subclass constructor with "this.inherited(arguments)" to 
    	 * call the template code below.
    	 */
        constructor: function(createArgs){
        	if(createArgs) {
				this.workingCopy = createArgs.workingCopy;  /* @deprecated, use the run parameter */
				this.actionSpec = createArgs.actionSpec;    /* @deprecated, use the run parameter */
        	}
        },
        /*
         * ActionSpec
         *   action: (String) id of the action class
         *   label: (String) Name of the action
         *   iconUri: (String) Relative path to the icon
         *   parameters: (String) Anything the configuration sends
         */
        actionSpec: null, 
        /*
         * WorkingCopy: Proxy representation of the work item. Only change data via methods
         */
        workingCopy: null,
        /*
         * Destroy is called when the action is destroyed, such as when the editor is closed.
         * Destroy must destroy all widgets or objects created and unsubscribe to any dojo events.
         */
        destroy: function() {
        },
        _doRun: function(event) {
			if(dojo.hasClass(event.currentTarget, "SimpleToolbar-button-disabled")) return; //$NON-NLS-1$

        	var args = {actionSpec: this.actionSpec, workingCopy: this.workingCopy};
        	if(this.isEnabled(args)) {
        		this.run(args);
        	}
        },
        /*
         * Run is called when the action is invoked. 
         * The value in parameters are the workingCopy and those specified in the extension point
         *   - parameter is a way to pass a string from the extension point to here
         */
        run: function(actionParms /* workingCopy: aWorkItemProxy, actionSpec: { id: string, action: string, label: string, iconContext: string, parameter: string } */) {
        	this._showMessage("Action " + actionParms.actionSpec.label + " run function is not yet implemented."); //$NON-NLS-1$
        },
        isVisible: function( parms /* workingCopy */) {
        	//example: return !parms.workingCopy.isNew();
        	return true;
        },
        /*
         * IsEnabled is called whenever the workingCopy is changed.
         */
        isEnabled: function(parms /* workingCopy */) {
        	//example:  return parms.workingCopy.isChanged();
        	return true;
        },
        /*
         * Get the icon. Should be 16x16 png, jpg, or gif
         */
         getIconUri: function() {
       		return this.actionSpec.iconUri ? dojo.moduleUrl(this.actionSpec.iconContext || DEFAULT_CONTEXT, this.actionSpec.iconUri) : this._getDefaultImageUri();
         },
        /*
         * Internal Functions
         */
        _showMessage: function(msg){
            alert(msg);
        },
        _getDefaultImageUri: function() {
	 		return dojo.moduleUrl("com.ibm.team.workitem.web", "ui2/internal/action/images/run.gif"); //$NON-NLS-1$
		}
    });
    
})();
