/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui2.internal.dialog.CreateFromTemplateDialogLauncher"); 
dojo.require("com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog");
dojo.require("com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy"); 
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.IconMessageWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WorkItemAttributes= com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare("com.ibm.team.workitem.web.ui2.internal.dialog.CreateFromTemplateDialogLauncher", null, {
openCreateFromTemplateDialog: function(parms) {
    if(!this.openCreateFromTemplateDialogMessages) this.openCreateFromTemplateDialogMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

	var templates = parms.templates;
	templates.sort(function(a, b) {
				try{
					if(!a || !a.label || !b || !b.label) return 1;
					return a.label > b.label ? 1 : -1;
				} catch(e) { 
					return -1 
				}
			});
   
    /* construct the list of templates */
    
	/* when a template is selected, show the description and variables */
	
    /* add the attribute variables */
    var self = this;
	var presentations = [
		{
			id: "listWidget",
			widget: "com.ibm.team.workitem.web.ui2.internal.widget.DialogListWidget",
			attributeId: "template",
			possibleValues: templates,
			detailsSelector: "description",
			messageSelector: "label",
			size: 6,
			accessibleLabel: dojo.string.substitute(self.openCreateFromTemplateDialogMessages.labelValue, [self.openCreateFromTemplateDialogMessages.labelTemplate]),
			initialize: function(parms) {
				var widget = parms.caller.parentWidget.getChildWidget(this.id);
				var message = dojo.string.substitute(self.openCreateFromTemplateDialogMessages.labelValue, [self.openCreateFromTemplateDialogMessages.labelTemplate]);
				var span = document.createElement("SPAN");
				span.style.marginLeft=  "5px";
				span.appendChild(document.createTextNode(message));
				widget.domNode.insertBefore(span, widget.domNode.childNodes[0]);		
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", // a template
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
				}
			},
			_handleTemplateChange: function(parms /* parentWidget: aPolicyDialog, workingCopy: aPolicyRequestProxy */) {
				parms.template = parms.value;
				self._validateRequestAndEnableCreateButton(parms); // self == the launcher		
			}
		},
		{
			id: "descriptionWidget",
			widget: "com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget",
			height: "8em",
			width: "97%",
			isReadOnly: true,
			className: "DialogListWidget",
			initialize: function(parms) {
				var widget = parms.caller.parentWidget.getChildWidget(this.id);
				var message = dojo.string.substitute(self.openCreateFromTemplateDialogMessages.labelValue, [self.openCreateFromTemplateDialogMessages.labelDescription]);
				var span = document.createElement("SPAN");
				span.style.marginLeft=  "5px";
				span.appendChild(document.createTextNode(message));
				widget.domNode.insertBefore(span, widget.domNode.childNodes[0]);
				dojo.attr(widget.textarea, "aria-label", self.openCreateFromTemplateDialogMessages.labelDescription);	
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template",
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
				}
			},
			_handleTemplateChange: function(parms) {
				parms.template = parms.value;
				var widget = parms.caller.parentWidget.getChildWidget(this.id);				
				widget.setValue(parms.value.description);
			}
		},
		{
			id: WorkItemAttributes.CATEGORY,
			label: "", // set in initialize
			widget: "com.ibm.team.workitem.web.ui.internal.view.editor.parts.CategoryAttributePart",
			creationArgs: {
				showGuessCategoryLink: false
			},
			attributeId: WorkItemAttributes.CATEGORY,
			initialize: function(parms) {
				parms.layoutWidget.setVisible(false);
				self._initializePossibleValues(parms);
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", 
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
					policyRequest.addListener({
						path: "attributes.category", 
						event: "onchange",
						listener: this,
						functionName: "_handleCategoryChange"
					});
				}
			},
			_handleCategoryChange: function(parms) {
				parms.attributeId = WorkItemAttributes.CATEGORY;
				parms.template = parms.workingCopy.object.attributes.template;
				self._validateRequestAndEnableCreateButton(parms);
			},
			_handleTemplateChange: function(parms) {
				parms.attributeId = WorkItemAttributes.CATEGORY;
				parms.template = parms.workingCopy.object.attributes.template;
				self._handleTemplateChange(parms);
			}
		},
		{   
			id:  WorkItemAttributes.TARGET,
			label: "", // set in initialize
			widget: "com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget",
			attributeId: WorkItemAttributes.TARGET,
			possibleValuesProvider: self._getTargetPossibleValues,
			initialize: function(parms) {
				parms.layoutWidget.setVisible(false);
				self._initializePossibleValues(parms);
				var policyRequest = parms.workingCopy;
				if(policyRequest) {
					policyRequest.addListener({
						path: "attributes.template", 
						event: "onchange",
						listener: this,
						functionName: "_handleTemplateChange"
					});
					policyRequest.addListener({
						path: "attributes.target", 
						event: "onchange",
						listener: this,
						functionName: "_handleTargetChange"
					});
				}
			},
			_handleTargetChange: function(parms) {
				parms.attributeId = WorkItemAttributes.TARGET;
				parms.template = parms.workingCopy.object.attributes.template;
				self._validateRequestAndEnableCreateButton(parms);
			},
			_handleTemplateChange: function(parms) {
				parms.attributeId = WorkItemAttributes.TARGET;
				parms.template = parms.workingCopy.object.attributes.template;
				self._handleTemplateChange(parms);
			}
		}
	];
	
	/*
	 * The policy request is initialized with dummy potential attributes
	 */
	var policyRequestObject = {
		attributes: {
			template: false,
			target: false,
			category: false
		}
	}
	var policyRequest =  new com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy({ 
		object: policyRequestObject
		}); 
	
	var dialogArgs = {
		dialogTitle: this.openCreateFromTemplateDialogMessages.labelTemplates,
		secondaryTitle: "",
		height: "32em",		
		workingCopy: policyRequest,
		okLabel: this.openCreateFromTemplateDialogMessages.labelCreateFromTemplate,
		okButtonHandler: this,
		onOk: function(args) {
				parms.onOk(args);
		},
		onCancel: function(args) {
		// ignore
		},
		presentations: presentations
	}
	this._dialog = new com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog(dialogArgs);
},
enableOkButton: function(parms) {
	var templateList = parms.parentWidget.getChildWidget("listWidget");
	var okButton = parms.parentWidget.getChildWidget(parms.parentWidget.OKBUTTON);
	var isEnabled = templateList.getSelected().length > 0;
	var hasCategoryVariable = false;
	var hasTargetVariable = false;
	var attributeVariables = parms.template ? parms.template.attributeVariables || [] : [];
	if(attributeVariables) {
		for(var i=0;i<attributeVariables.length;i++) {
			if(attributeVariables[i].id == WorkItemAttributes.CATEGORY) hasCategoryVariable = true;
			if(attributeVariables[i].id == WorkItemAttributes.TARGET) hasTargetVariable = true;			
		}
	}
	if(isEnabled && hasCategoryVariable) {
		var categoryWidget = parms.parentWidget.getChildWidget(WorkItemAttributes.CATEGORY);
		isEnabled = !categoryWidget.isSelectedValueNull();
	}
	if(isEnabled && hasTargetVariable) {
		var targetValue = parms.parentWidget.getChildWidget(WorkItemAttributes.TARGET).getValue();
		isEnabled = targetValue != "";
	}
	okButton.disabled = !isEnabled; 
},
_validateRequestAndEnableCreateButton: function(parms) {
	parms.parentWidget =this._dialog;
	this.enableOkButton(parms);
},
_handleTemplateChange: function(parms) {
	var policyRequest = parms.workingCopy;
	var template = parms.template;
	var templateMap = parms.caller.parentWidget.getPropertyMapFromArray({
		array: template.attributeVariables, 
		result: {}, 
		keySelector: "id"}).result;
	var widget = parms.caller.parentWidget.getChildWidget(parms.attributeId);
	var layout = parms.caller.parentWidget.getChildWidgetLayout(parms.attributeId);
	if(templateMap[parms.attributeId]) {
		/*
		 * The label is only known after the template is loaded
		 * so only show the widget after the label is known
		 */
		if(layout && !layout.isVisible()) {
			var labelText = templateMap[parms.attributeId].label;
			labelText = dojo.string.substitute(this.openCreateFromTemplateDialogMessages.labelValue, [ labelText ]);
			layout.setLabel({labelText: labelText, hasColon: true});
			dojo.attr(widget._select, "aria-label", labelText.replace(/: /g, ""));
		}
		widget.setEnabled(true);
		if(layout) {
			layout.setVisible(true);
		} else {
			widget.domNode.style.visibility = "visible";
		} 
	} else {
		widget.setEnabled(false);
	}
},
_initializePossibleValues: function(parms) {
		var policyRequest = parms.workingCopy;
		if(this._hasRequestedPossibleValues) return;
		this._hasRequestedPossibleValues = true;
		var op = parms.workingCopy.createOperation({
	        message: this.openCreateFromTemplateDialogMessages.busyLoading,
	        id: "loadingTemplatePossibleValues"
	  	});
	  	parms.workingCopy.startOperation(op);
		var self= this;
		var handlerObj= {
			success: function(/* UIItemSetDTO */ valueSet) {
				parms.workingCopy.finishOperation(op);
				var valueMap = parms.caller.parentWidget.getPropertyMapFromArray({
					array: valueSet, 
					result: {}, 
					keySelector: "attributeName"});
			    var categoryValues = valueMap.result.category ? valueMap.result.category.uiItems : [];
				var categoryWidget = parms.caller.parentWidget.getChildWidget(WorkItemAttributes.CATEGORY);
				categoryWidget.setPossibleValues(categoryValues);
				
			    var targetValues = valueMap.result.target ? valueMap.result.target.uiItems : [];
				var targetWidget = parms.caller.parentWidget.getChildWidget(WorkItemAttributes.TARGET);
				targetWidget.setPossibleValues(targetValues);
			},
			failure: function(errorObj) {
				parms.workingCopy.finishOperation(op);
		 		console.error("Error fetching dialog possible " + errorObj.message); // $NON-NLS-1$
			}
		};

		var handler= new ServiceResponseHandler(handlerObj, "success", "failure");
		WorkItemClient.getAllValues(handler, { newWorkItem: true, ids: [ WorkItemAttributes.CATEGORY, WorkItemAttributes.TARGET ] });
}
}
);

})();
