/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController");
dojo.require("dijit.InlineEditBox");

dojo.require("com.ibm.team.workitem.web.ui2.internal.dialog.CALMBackLinkProblemDialogLauncher"); 

dojo.require("dojox.xml.parser");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.ViewletContainer");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.TrimlessViewletContainer");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {
var WorkItemUtil		= com.ibm.team.workitem.web.client.util;
var slotMap = {}; // filled in with messages	
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

dojo.declare("com.ibm.team.workitem.web.ui2.internal.widget.InlineWorkItemEditor", [com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor,  com.ibm.team.workitem.web.ui2.internal.dialog.CALMBackLinkProblemDialogLauncher], {
	templateString: "<div class=\"com-ibm-team-workitem-widget-content com-ibm-team-workitem-web-inlineworkitemeditor\"></div>",
//	templateCSS: "/templates/InlineWorkItemEditor.css",
/*
 * Parameters:
 *    isTrimless: if true, do not show a container
 *    isShowingTopCloseButton: if false, do not show the close button (x)
 *    isShowingTopSaveButtons: if false, do not show the Save and Save and Close buttons at the top
 *    isShowingTopRefreshButton: if false, do not show the Save and Save and Close buttons at the top
 *    isShowingBottomSaveButtons: if true, show buttons along the bottom as a dialog: "OK"== Save & Close, "Cancel"
 *    fixedHeight: the desired height assigned to the style. The content will scroll.
 *    isContentOnly: if true, do only show the content
 * 
 */
	postCreate: function() {
		this.inherited(arguments);
	
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		this.pageController = this.parentWidget.pageController;
		if(!this.pageController) {
			this.pageController = com.ibm.team.workitem.web.cache.internal.Cache._pageController
		}
		if(!this.pageController) {
			this._createdPageController = true;
			this.pageController = new com.ibm.team.workitem.web.ui.internal.page.WorkItemPageController();
		}
		slotMap = {   
			"top": {
			 	attributeValueClass: "attr-value-wide-top",
			 	attributeLabelClass: "attr-label-wide",
				attributeContainerClass: "attr-container-top",
				isOneLine: true			
					},
			 "left": {},
			 "right": {},
			 "bottom": {
			 	attributeValueClass: "attr-value-wide-bottom",
			 	attributeLabelClass: "attr-label-wide-bottom",
				attributeContainerClass: "attr-container-bottom",
				isWide: true				
					},
			 "sidebar": {
			 	attributeValueClass: "attr-value-wide-bottom",
			 	attributeLabelClass: "attr-label-wide-bottom",
				attributeContainerClass: "attr-container-sidebar"				
					}
		}

		if(!this.workItemId) return;
		if(!this.projectArea) {
			this.projectArea = this.getCurrentProjectArea();
		}
		this.presentation = this.presentation || "com.ibm.team.workitem.web.inline";
		
		this._getThisWorkingCopy(this.workItemId, /*createIfNeeded*/ true);
		this.workItemId = this.workingCopy.getId();

		this.domNode.style.width = "100%";
//		if(!dojo.isIE) {
//			this.domNode.style.marginLeft = "1.7em";
//		} else {
//			this.domNode.style.marginLeft = "0.5em";			
//		}
		
		var headerPrimary = document.createElement("div");
		this.headerPrimary = headerPrimary;
		headerPrimary.className = "header-container-right";
		var titleSpan = document.createElement("span");
		titleSpan.appendChild(document.createTextNode(this.messages.busyEditingWorkItem));

		var isCreateRefreshButton = false;
		if (this.workingCopy.isNewWorkItem() && !(this.isShowingTopCloseButton === false)) {
			var cancel = document.createElement("span");
			dojo.create("button", {
				innerHTML: this.messages.buttonClose,
				onclick: dojo.hitch(this, "_promptToClose")
			}, cancel);
			cancel.className = "header-right";
			headerPrimary.appendChild(cancel);
			this.workingCopy.setEditorUrl(window.location.href);
		}
		else {
			if(!(this.isShowingTopRefreshButton === false)) {
				isCreateRefreshButton = true;
			}
		}

		if(!(this.isShowingTopSaveButtons === false)) {
			var saveSpan = document.createElement("span");
			this.saveButton = dojo.create("button", {
				innerHTML: this.messages.buttonSave,
				onclick: dojo.hitch(this, "_handleSaveClick")
			}, saveSpan);
			saveSpan.className = "header-right";
			headerPrimary.appendChild(saveSpan);
	
			var saveAndCloseSpan = document.createElement("span");			
			this.saveAndCloseButton = dojo.create("button", {
				innerHTML: this.messages.buttonSaveAndClose,
				onclick: dojo.hitch(this, "_handleSaveAndCloseClick")
			}, saveAndCloseSpan);
			saveAndCloseSpan.className = "header-right";
			headerPrimary.appendChild(saveAndCloseSpan);
	
		    this.setSaveEnabled(false);
		}	
		var headerLeft = document.createElement("div");
		this.idSpan  = document.createElement("SPAN");
		this._setIdLabel();
		
		this.idDiv = document.createElement("span");
		this.idDiv.appendChild(this.idSpan);
		this.idDiv.appendChild(document.createTextNode(" "));
		this.idDiv.className = "header-left";
		
		this.summaryDiv = document.createElement("div");
		this.summaryDiv.style.marginLeft = "0.5em";
		this.summaryDiv.className = "header-left";
		this.summaryDiv.appendChild(document.createTextNode(this.messages.busyLoading)); 
		
		headerLeft.appendChild(this.summaryDiv);
		headerPrimary.appendChild(this.idDiv);
		headerPrimary.appendChild(headerLeft);
			
		if(isCreateRefreshButton){
			this._createRefreshButton();
		}

		this._loadingDiv= document.createElement("div");
		this._loadingDiv.className = "loading-message"
		this._loadingDiv.innerHTML = this.messages.busyLoading; 
	
		this._eventHandles = this._eventHandles  || [];
		this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_EDITOR_CHANGED, this, this._workItemEditorChanged));
		this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_CHANGED, this, this._workItemChanged));
		this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_ACTION_NOTIFY_INLINE_TO_PUBLISH_ITEM_VIEWED, this, this._publishItemViewed));
	
		if (this.isContentOnly) {
			this.viewletContainer= new com.ibm.team.workitem.web.ui2.internal.widget.TrimlessViewletContainer({ parentWidget: this });
		} else {
			var args = {
			  headerHoverState: false,
			  primaryTitle: "",
			  headerHoverDefaultState: true,
		      parentWidget: this
			};
			if(this.isTrimless) {
				args.isTrimless = true;
				args.closable = false;
			}
		    this.viewletContainer = new com.ibm.team.workitem.web.ui2.internal.widget.ViewletContainer(args);
		}
		this.viewletContainer.setContent(this._loadingDiv, headerPrimary);
		this.domNode.appendChild(this.viewletContainer.domNode);
		var args = {
			id: this.workItemId,
			projectArea: this.projectArea,
			isForced: false,
			callback:  dojo.hitch(this, this._showWorkItemEditor),
			fetchEditorPresentation: false,
			fetchInlineEditorPresentation: true,
			inlinePresentationId: this.presentation,
			isInit: true
		}
		
		this.isShowingBottomSaveButtons = this.isShowingBottomSaveButtons === true;
		this.isInitialized = true;
		this.fetchWorkItem(args);

	},
	destroy: function() {
		if(this._createdPageController && this.pageController) {
			this.pageController.destroyController();
			this.pageController = null;
		}
		this.inherited(arguments);
	},
	_publishItemViewed: function(parms) {
		dojo.publish(EventTopics.WORKITEM_ITEM_VIEWED, [this.workingCopy, this]);
	},
	_createRefreshButton: function() {
		   if(this.refreshSpan) return;
			var refresh = document.createElement("span");
			var a = document.createElement("a");
			a.href = "#";
			a.className = "dijitInline action-icon";
			var txt = this.messages.buttonRefresh;
			dojo.create("img", {
				src: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/images/refresh.gif")+"",
				alt: txt,
				title: txt
			}, a);
			this.connect(a, "onclick", this._handleRefreshClick);
			refresh.appendChild(a);
			refresh.className = "header-right";
			dojo.place(refresh, this.idDiv, "before");
			this.refreshSpan = refresh;
	},
	_workItemEditorChanged: function(workItemProxy, editor, isClose ) {
		if(editor === this) return;
		if(workItemProxy === this.workingCopy && isClose ) {
			this._handleCancelClick();
		}
	},
	_workItemChanged: function(workItemProxy) {
		if(workItemProxy === this.workingCopy ) {
			this.setIsDirty(this.workingCopy.isChanged());
		}
	},
	_setIdLabel: function (isReset) {
		var idLabel = this.workingCopy.getIdLabel();
		var isModule = window.location.href.indexOf("_ajax-modules") > 0;
		if (isModule || this.workingCopy.isNewWorkItem()) {
			dojox.xml.parser.replaceChildren(this.idSpan, document.createTextNode(idLabel));
		}
		else {
			var idAnchor = dojo.create("A", { className:"idAnchor" });	
			idAnchor.appendChild(document.createTextNode(idLabel));
			idAnchor.href = this.workingCopy.getUrl(/* reset */ true);
			dojox.xml.parser.replaceChildren(this.idSpan, idAnchor);
		}
		
	},
	/*
	 * Ignore markDirty since this is handled by other notifications 
	 */ 
	markDirty: function() {
		
	},
	_handleWorkingCopySummaryChange: function(parms) {
			this._setSummaryValue(parms.attributeValue);
		},
	_handleWorkingCopyWorkItemTypeChange: function(parms) {
			this._setWorkItemType(parms.attributeValue);
		},
	_handleWorkingCopyCommentChange: function(parms) {
		this.setIsDirty(this.workingCopy.isChanged());
	},
	_handleWorkingCopyRefresh: function(parms) {
		this._showWorkItemEditor(parms);
		// remember the internal comment value
		if (this._currentLastComment && this._currentLastComment.length > 0) {
			this.workingCopy.setValue({
				path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
				attributeId: this.workingCopy.SYNTHETIC_COMMENT_ID,
				value: this._currentLastComment || "",
				isRefresh: true
			});
		}
		this.setIsDirty(this.workingCopy.isChanged());
	},
	_handleSummaryBlur: function(e) {
		this._handleSummaryChange(e, /* notifyEvenIfUnchanged */ true);
	},
	_handleSummaryChange: function(e, notifyEvenIfUnchanged) {
	// Only propagate the change if its the first change
		var isQuiet = this.workingCopy.isChanged() && !(notifyEvenIfUnchanged == true);
		this._setWorkingCopyValue(isQuiet, notifyEvenIfUnchanged == true);
	},
	_setWorkingCopyValue: function(isQuiet, notifyEvenIfUnchanged) {
		var value = this.summaryEditBox.value || "";
		var oldValue = 	this.workingCopy.getValue( {
			path: ["attributes", "summary", "content"]
		}) || "";
		if(value == oldValue) return;
		value = value.replace(/\&nbsp;/, " ")
	    this.workingCopy.setValue( {
			path: ["attributes", "summary", "content"],
			attributeId: "summary",
			notifyEvenIfUnchanged: notifyEvenIfUnchanged,
			isQuiet: isQuiet,
			value: value
		});
		this._handleDirtyingEvent();
	},
	_setSummaryValue: function(value) {
		if(this.summaryEditBox) {
			value = EditorUtil.restoreTags(value);
			this.summaryEditBox.value = value;
			this._handleDirtyingEvent();
		}
	},
	_setWorkItemType: function(value) {
		var iconUrl = value.iconUrl;
		if(this.viewletContainer) {
			this.viewletContainer.setHeaderIcon(iconUrl);
		}
		this._handleDirtyingEvent();
	},
	_showMissingPresentation: function() {
		this._loadingDiv.innerHTML = this.messages.labelNoEditorPresentationForThisType; 
		dojox.xml.parser.removeChildren(this.summaryDiv);
		if(this.refreshSpan) dojo.addClass(this.refreshSpan,  "save-disabled");
	},
	/*
	 * Called after the work item is fetched or refreshed
	 */
	_showWorkItemEditor: function(parms /* parms.workItem, parms.editProps, parms.inlinePresentationProps */ ) {
		
		/* If this is a refresh, remove all listeners and children */
		this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentWidget"});
		this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentController"});
		this.resetChildWidgets();
		
		parms = parms || {};

		if(!this.workingCopy || !this.workingCopy.workItemSpec || !this.workingCopy.workItemSpec.inlinePresentationProps
			 || !this.workingCopy.workItemSpec.inlinePresentationProps.pages || this.workingCopy.workItemSpec.inlinePresentationProps.pages.length == 0
			 || !this.workingCopy.workItemSpec.inlinePresentationProps.pages[0].sections || this.workingCopy.workItemSpec.inlinePresentationProps.pages[0].sections.length == 0) {
			this._showMissingPresentation();
			return;
		}
		
		var presentationProps = this.workingCopy.workItemSpec.inlinePresentationProps;
	    var sections =  this.workingCopy.workItemSpec.inlinePresentationProps.pages[0].sections;
		/*
		 * Presentation props are in the form 
		 *   []pages[0]
		 *       []sections
		 *          0 section
		 *          1 section
		 *               []presentations
		 *                   0 presentation
		 *                       attributeId
		 *                       attributeLabel
		 *                       attributeType "mediumHtml"
		 *                       partId 
		 *                       widget
		 *                       []properties
		 *                           0 prop
		 *                                key: labelVisible
		 *                                value: "false"
		 */
		
		var workItem = parms.workItem || this.workingCopy.object;
				
// The top summary line is a special case		
		this.workingCopy.addListener({
				path: ["attributes", "summary", "content"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopySummaryChange"
			});			

// The comment part is a special case
		this.workingCopy.addListener({
				path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyCommentChange"
			});			

		this.workingCopy.addListener({
				path: ["attributes", "workItemType"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyWorkItemTypeChange"
			});		

		this.workingCopy.addListener({
				path: [ "root" ],
				event: "onrefresh",
				listener: this,
				functionName: "_handleWorkingCopyRefresh"
			});	

		if(!workItem ) {
			console.debug("No work item available");   // $NON-NLS1
			return;
		}
		if(!presentationProps ) {
			console.debug("No presentationProps available");   // $NON-NLS1
			return;
		}

	    if(workItem && workItem.attributes && workItem.attributes.workItemType && workItem.attributes.workItemType.iconUrl) {
			var iconUrl = workItem.attributes.workItemType.iconUrl;
			this.viewletContainer.setHeaderIcon(iconUrl);
		}
		this.summaryDiv.style.visiblility = "none";
		dojox.xml.parser.removeChildren(this.summaryDiv);
//    this._showSummaryLine(); // show it in the header

		var content= document.createElement("div");
		/*
		 *     *--------------------------------------------------------------------------------*
		 *     | Title                                                               Actions  _ X
		 *     *--------------------------------------------------------------------------------*
		 *     | // Summary: Top slot
		 *     *---------------------------------------------------------------------------------
		 *     | Left Slot                              |  Right Slot
		 *     |   Label: Value Widget
		 *     |
		 *     *---------------------------------------------------------------------------------
		 *     * Description  Discussion Approvals Quick Information   Extention navigation bar
		 *     *---------------------------------------------------------------------------------
		 *     | Bottom Slot
		 *     |
		 *     *----------------------------------------------------------------------------------
		 */     
	
	    var columnContainerDiv = document.createElement("div");
		var sidebarContainerDiv = document.createElement("div");
		sidebarContainerDiv.className = "container-sidebar";
		columnContainerDiv.className = "container-columns";
		if(this.fixedHeight) {
			dojo.addClass(columnContainerDiv, "container-scrolled");
			columnContainerDiv.style.height = this.fixedHeight;
		}
		
		var table = document.createElement("TABLE");
		table.cellPadding = "0";
		table.cellSpacing = "0";
		table.border = "0";
		table.style.width ="100%";
		var row = table.insertRow(-1);
		var colLeft = row.insertCell(-1);
		colLeft.style.verticalAlign = "top";
		colLeft.style.border = "none";
		colLeft.style.width = "100%";
		var colRight = row.insertCell(-1);
		colRight.style.verticalAlign = "top";
		colRight.style.width = "100%";
		colRight.style.border = "none";
		colRight.appendChild(sidebarContainerDiv);
		colLeft.appendChild(columnContainerDiv);
		content.appendChild(table);
		
		
		// TODO: Put the sidebar on the side 			
		var sidebarDiv = document.createElement("div");
		sidebarDiv.className = "slot-sidebar";
		var layoutSpecs = this._getSlotLayoutSpecs(sections, "sidebar");
		for(var i=0;i<layoutSpecs.length;i++) {
			this._layoutSlot(layoutSpecs[i], sidebarDiv, "attr-sidebar",/* isOneLine */ false);
		}		
		sidebarContainerDiv.appendChild(sidebarDiv);
		
		// Top slot is full width
		var spacer = document.createElement("div");
		spacer.style.marginTop = "0.5em";
		columnContainerDiv.appendChild(spacer);
		dojo.addClass(this.domNode, "com-ibm-team-workitem");
		dojo.addClass(content, "workItemEditor");
		dojo.addClass(content, "workItemInlineEditor");
		
		var topDiv = document.createElement("div");
		topDiv.className = "slot-top";
		var layoutSpecs = this._getSlotLayoutSpecs(sections, "top");
		for(var i=0;i<layoutSpecs.length;i++) {
			this._layoutNameValueSlot(layoutSpecs[i], topDiv, "attr-top", /* isOneLine */ true, "attr-value-wide");
		}
		
		columnContainerDiv.appendChild(topDiv);
		
		var middleDiv = document.createElement("div");
		var leftDiv = document.createElement("div");
		leftDiv.className = "slot-left";
		var layoutSpecs = this._getSlotLayoutSpecs(sections, "left");
		for(var i=0;i<layoutSpecs.length;i++) {
			this._layoutNameValueSlot(layoutSpecs[i], leftDiv, "attr-left", /* isOneLine */ false);
		}

		middleDiv.appendChild(leftDiv);
		var rightDiv = document.createElement("div");
		rightDiv.className = "slot-right";
		var layoutSpecs = this._getSlotLayoutSpecs(sections, "right");
		for(var i=0;i<layoutSpecs.length;i++) {
			this._layoutNameValueSlot(layoutSpecs[i], rightDiv, "attr-right", /* isOneLine */ false);
		}
		middleDiv.appendChild(rightDiv);
		columnContainerDiv.appendChild(middleDiv);

		var bottomDiv = document.createElement("div");
		bottomDiv.className = "slot-bottom";
		var layoutSpecs = this._getSlotLayoutSpecs(sections, "bottom");
		for(var i=0;i<layoutSpecs.length;i++) {
						this._layoutSlot(layoutSpecs[i], bottomDiv, "attr-bottom",false);
		}
		columnContainerDiv.appendChild(bottomDiv);
	
		if(this.isShowingBottomSaveButtons) {
			var buttonDiv = document.createElement("div");
			buttonDiv.className = "buttonSection";
			
			var okButton = document.createElement("button");
			okButton.className = "Button";
			okButton.tabOrder = "5";
			okButton.disabled = "disabled";
			okButton.appendChild(document.createTextNode(this.messages.buttonOK));
			this.okButton = okButton;
			
			this.connect(okButton, "onclick", dojo.hitch(this, this._handleBottomOkClicked));

			var cancelButton = document.createElement("button");
			cancelButton.className = "Button";
			cancelButton.tabOrder = "6";
			cancelButton.appendChild(document.createTextNode(this.messages.buttonCancel));
		
			this.connect(cancelButton, "onclick", dojo.hitch(this, this._handleBottomCancelClicked));
			
			buttonDiv.appendChild(okButton);
			buttonDiv.appendChild(cancelButton);
			content.appendChild(buttonDiv);
		}
		if(this.params.heightHint) {
			dojo.style(content, "height",this.params.heightHint);
		}
		this.viewletContainer.setContent(content);
		var spacer = document.createElement("div");
		spacer.style.marginTop = "0.5em";

		content.appendChild(spacer);

		this.setIsDirty(this.workingCopy.isChanged());

		if(!parms.isQuiet) {
			dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, [this.workingCopy, this]);
		}

	},
	// @deprecated Summary is now shown in the top slot
	_showSummaryLine: function() {
		dojox.xml.parser.removeChildren(this.summaryDiv);
		var summaryValue = this.workingCopy.getValue( {
			path: ["attributes", "summary", "content"]
		}) || "";

		summaryValue = EditorUtil.restoreTags(summaryValue);
		var text = document.createElement("input");			
		text.type = "text";
		text.value = summaryValue;

		this.connect(text, "onkeyup", dojo.hitch(this, this._handleSummaryChange));
		this.connect(text, "onblur", dojo.hitch(this, this._handleSummaryBlur));
		dojo.addClass(text, "summary-text");
		this.summaryEditBox	= text;
			
		var isRequired = this._isRequired("summary");
		var requiredMarkerSpan = document.createElement("SPAN");
		requiredMarkerSpan.className = "RequiredMarker";
		requiredMarkerSpan.innerHTML = "*" /* innerHTML OK */
		requiredMarkerSpan.title = this.messages.titleRequiredMarker;
		if(isRequired) { 
			requiredMarkerSpan.style.display = "inline";
		} else {
			requiredMarkerSpan.style.display = "none";
		} 
		this.summaryDiv.appendChild(requiredMarkerSpan);

		this.summaryDiv.appendChild(text);
		this.summaryDiv.style.visiblility = "";

	},
	/* 
	 * Slot:
	 *          slotDiv
	 *             attributeDiv
	 *                 labelDiv   valueDiv
	 *                    #text       widget.domNode
	 */
	_layoutSlot: function(layoutSpec, slotDiv, className, isOneLine, attrClass ) {
		if(!layoutSpec || !layoutSpec.presentations) return;
		for(var i=0; i< layoutSpec.presentations.length;i++) {
			var attrContainerDiv = document.createElement("div");
			if(layoutSpec.presentations[i].partId == "com.ibm.team.workitem.kind.separator") {
				attrContainerDiv.className = "attr-row-separator";
			} else {
				attrContainerDiv.className = "attr-row";	
			}
			var attrDiv = document.createElement("div");
			attrDiv.className = className;
			layoutSpec.presentations[i].slot = layoutSpec.slot;
			this._layoutAttribute(layoutSpec.presentations[i],layoutSpec, attrDiv, attrClass);

			attrContainerDiv.appendChild(attrDiv);
			slotDiv.appendChild(attrContainerDiv);
//			slotDiv.appendChild(document.createElement("br"));
		}		
	},
	_isRequired: function(attributeId) {
		var isRequired = false;
		if(this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps && this.workingCopy.workItemSpec.editProps.requiredProperties ) {
			for(var i=0;i<this.workingCopy.workItemSpec.editProps.requiredProperties.length;i++ ) {
				if(this.workingCopy.workItemSpec.editProps.requiredProperties[i] == attributeId) {
					isRequired = true;
					break;
				}
			}
		}
		return isRequired;
	},
	_layoutAttribute: function (pres, sectionProperties, attrDiv, attrClass) {		
		var propertyKeyValues = this.getPropertyMap(pres);
		// labelVisible: "false"
		// hideIfEmpty: "true"
		
		var sectionPropertyKeyValues = sectionProperties && sectionProperties.properties ? this.getPropertyMap(sectionProperties) : {};		
		// title: "Description"

		var values;
		if(pres.widget != "com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget" && 
		   pres.widget != "com.ibm.team.workitem.web.ui.internal.view.layout.StaticTextWidget") {
			values = this._getPresentationValues(pres);
			if(propertyKeyValues.hideIfEmpty == "true"  && (!values || values.length == 0 || (values.length ==1 && values[0] == null)) ) {
				return;
			}			
			if(slotMap[pres.slot] &&  slotMap[pres.slot].attributeContainerClass) { 
				attrDiv.className = slotMap[pres.slot].attributeContainerClass;
			} else {
				attrDiv.className = "attr-container";
			}
		} else {
			values = [];
			attrDiv.className = "attr-container";
		}
		
		
		// Label
		var labelDiv = document.createElement("div");
		
        // Label Text
		var possibleValues = this._getPresentationPossibleValues(pres);
		var widgetDiv = document.createElement("div");
		dojo.mixin(pres, {properties: sectionProperties.properties})
		/*
		 * Special case for StaticText
		 */
		if(propertyKeyValues && pres.properties && propertyKeyValues.text) {
			pres.properties.push({ key: "text", value: propertyKeyValues.text });
		}

		
		var widget = this.getWidget(this.workingCopy.object, pres, widgetDiv, values, possibleValues );
		if(!widget) {
			console.debug("Missing widget: " + pres.widget); // $NON-NLS-1
			return;
		}


		if((widget.layoutHints && widget.layoutHints.mustBeWide && !slotMap[pres.slot].isWide)
			|| (widget.layoutHints && widget.layoutHints.mustBeMultiline && slotMap[pres.slot].isOneLine)) {
				console.debug("Widget " + pres.widget + " is too wide for the selected slot ("+ pres.slot+") and was not shown."); // $NON-NLS-1
				return;
		}
		
		
		var labelValue = "";
		var showLabel = !widget.layoutHints || widget.layoutHints.hideLabel != true;
		if(showLabel && propertyKeyValues.labelVisible != "false") {
			   if(pres.attributeLabel) {
			   		labelValue = dojo.string.substitute(this.messages.labelValue, [pres.attributeLabel]);
			   } else {
			   if(sectionPropertyKeyValues.title) {
						labelValue = dojo.string.substitute(this.messages.labelValue, [sectionPropertyKeyValues.title]).replace(/::/g,":");	   			
			 		} else { 
			   	    	if (dojo.isFunction(widget.getDefaultLabel)) {
							var defaultLabel = widget.getDefaultLabel();
							if (defaultLabel) {
								labelValue = dojo.string.substitute(this.messages.labelValue, [defaultLabel]).replace(/::/g,":");
						}
					}
				}
			   }
				
		}
		labelValue = labelValue || "";
		if(slotMap[pres.slot] &&  slotMap[pres.slot].attributeLabelClass) { 
			labelDiv.className = slotMap[pres.slot].attributeLabelClass;
		} else {
			labelDiv.className = "attr-label";
		}
		
		labelDiv.appendChild(document.createTextNode(labelValue));
		if (!sectionProperties.skipRequiredMarker && pres.attributeId) {
				var reqMark = new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget(
				{workingCopy: this.workingCopy,
			 		attributeId: pres.attributeId}
				);
				this._childWidgets.push(reqMark); // see AbstractEditor
				labelDiv.appendChild(reqMark.domNode);
		}

		if(labelValue.length == 0) {
			labelDiv.appendChild(document.createTextNode(this.messages.labelNone));
			if(showLabel) {				
				labelDiv.style.visibility = "hidden";
			} else {
				labelDiv.style.display = "none";
			}
			if(widget.isSeparator) {
				dojo.addClass(labelDiv,"SeparatorWidget");
			}
		}
		attrDiv.appendChild(labelDiv);
	
		if (slotMap[pres.slot] && slotMap[pres.slot].breakBeforeValue) {
		    attrDiv.appendChild(document.createElement("BR"));
		}
		var valueDiv = document.createElement("div");
		if(slotMap[pres.slot] &&  slotMap[pres.slot].attributeValueClass) { 
			valueDiv.className = slotMap[pres.slot].attributeValueClass;
		} else {
			valueDiv.className = "attr-value";
		}
		valueDiv.appendChild(widget.domNode);
		attrDiv.appendChild(valueDiv);
		if(pres.attributeId == "summary") {
			if(widget._input) {
				var self = this;
				setTimeout(dojo.hitch(this, function() {
					try{
						if(!widget || self.isDestroyed) return;
						widget._input.focus();} catch(e) {
						// ignore. This occurs on IE when the editor is hidden
					}
					}), 500);				
			} 
		}
		if(widget.isTall) {
			dojo.addClass(valueDiv, "attr-value-tall");
			dojo.addClass(labelDiv, "attr-label-tall");
		}
		if(widget.showWidget) widget.showWidget();
	},
	_layoutNameValueSlot: function(layoutSpec, slotDiv, className, isOneLine, attrClass) {
		if(!layoutSpec || !layoutSpec.presentations) return;
		
		if(isOneLine) return this._layoutOneLineNameValueSlot(layoutSpec, slotDiv);
		var nameValueLayoutWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout({
						workingCopy: this.workingCopy, 
						hasRequiredMarker: true });
		this._childWidgets[nameValueLayoutWidget.id] = nameValueLayoutWidget;
		for(var i=0; i< layoutSpec.presentations.length;i++) {
			if(layoutSpec.presentations[i].partId == "com.ibm.team.workitem.kind.separator") {
				nameValueLayoutWidget.addSeparatorRow(layoutSpec.presentations[i],layoutSpec);
			} else {
				layoutSpec.presentations[i].slot = layoutSpec.slot;
				this._layoutNameValueAttribute(nameValueLayoutWidget, layoutSpec.presentations[i], layoutSpec);
			}
		}
		slotDiv.appendChild(nameValueLayoutWidget.domNode);
		},
	_layoutOneLineNameValueSlot: function(layoutSpec, slotDiv) {
		var slotTable = document.createElement("table");
		slotTable.cellPadding = 0;
		slotTable.cellSpacing = 0;
		slotTable.className = "slot-top-table";
		var topRow = slotTable.insertRow(-1);
		topRow.className = "slot-top-tr";
		for(var i=0; i< layoutSpec.presentations.length;i++) {
			var nameValueLayoutWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.NameValueLayout({
						workingCopy: this.workingCopy, 
						hasRequiredMarker: true });
			this._childWidgets[nameValueLayoutWidget.id] = nameValueLayoutWidget;
			if(layoutSpec.presentations[i].partId == "com.ibm.team.workitem.kind.separator") {
				nameValueLayoutWidget.addSeparatorRow(layoutSpec.presentations[i],layoutSpec);
			} else {
				layoutSpec.presentations[i].slot = layoutSpec.slot;
				this._layoutNameValueAttribute(nameValueLayoutWidget, layoutSpec.presentations[i], layoutSpec);
			}
			var col = topRow.insertCell(-1);
			col.className = "slot-top-td";
			col.appendChild(nameValueLayoutWidget.domNode);
		}
		slotDiv.appendChild(slotTable);
		},
	_layoutNameValueAttribute: function (layoutWidget, pres, sectionProperties) {		
		var propertyKeyValues = this.getPropertyMap(pres);
		// labelVisible: "false"
		// hideIfEmpty: "true"
		
		var values;
		if(pres.widget != "com.ibm.team.workitem.web.ui.internal.view.layout.LineSeparatorWidget" && 
		   pres.widget != "com.ibm.team.workitem.web.ui.internal.view.layout.StaticTextWidget") {
			values = this._getPresentationValues(pres);
			if(propertyKeyValues.hideIfEmpty == "true"  && (!values || values.length == 0 || (values.length ==1 && values[0] == null)) ) {
				return;
			}			
		} else {
			values = []
		}

		var isRequired = this._isRequired(pres.attributeId);	

		var possibleValues = this._getPresentationPossibleValues(pres);
		var widgetDiv = document.createElement("div");
		if(sectionProperties && sectionProperties.properties) {
			pres.properties = pres.properties || [];			
			for(var i=0;i<sectionProperties.properties.length;i++) {
				pres.properties.push(sectionProperties.properties[i]);
			}
		}
//		dojo.mixin(pres, {properties: sectionProperties.properties})
		
		/*
		 * Special case for StaticText
		 */
		if(propertyKeyValues && pres.properties && propertyKeyValues.text) {
			pres.properties.push({ key: "text", value: propertyKeyValues.text });
		}

		var widget = this.getWidget(this.workingCopy.object, pres, widgetDiv, values, possibleValues );
		if(!widget) {
			console.debug("Missing widget: " + pres.widget); // $NON-NLS-1
			return;
		}

		if((widget.layoutHints && widget.layoutHints.mustBeWide && !slotMap[pres.slot].isWide)
			|| (widget.layoutHints && widget.layoutHints.mustBeMultiline && slotMap[pres.slot].isOneLine)) {
				console.debug("Widget " + pres.widget + " is too wide for the selected slot ("+ pres.slot+") and was not shown."); // $NON-NLS-1
				return;
		}

		var labelText = this._getLabelText(pres, propertyKeyValues, widget);
		layoutWidget.addRow( {
			labelText: labelText, 
			valueNode: widget.domNode, 
			isWide: (widget.layoutHints && widget.layoutHints.isWide), 
			isOneLine: (widget.layoutHints && widget.layoutHints.isOneLine),
			hideLabel: (widget.layoutHints && widget.layoutHints.hideLabel), 
			widget: widget });
		
		if(widget.showWidget) widget.showWidget();
	},
	_getLabelText: function(pres, propertyKeyValues, widget) {
		var labelValue;
		if(propertyKeyValues.labelVisible != "false") {
			   if(pres.attributeLabel) {
			   		labelValue = dojo.string.substitute(this.messages.labelValue, [pres.attributeLabel]);
			   } else {
			   	    if (dojo.isFunction(widget.getDefaultLabel)) {
						var defaultLabel = widget.getDefaultLabel();
						if (defaultLabel) {
							labelValue = dojo.string.substitute(this.messages.labelValue, [defaultLabel]).replace(/::/g,":");
						}
					}
			   }
				
		}
		return labelValue || "";
	},
	_getSlotLayoutSpec: function(sections, slotName) {
		for(var j=0;j<sections.length;j++) {
			if(sections[j].slot == slotName) return sections[j];
		}
	}, 
	_getSlotLayoutSpecs: function(sections, slotName) {
		var result = [];
		for(var j=0;j<sections.length;j++) {
			if(sections[j].slot == slotName) result.push(sections[j]);
		}
		return result;
	}, 
	_getPresentationValues: function (pres) {
		var attrId = pres.attributeId;
		var value = {};
		if(this.workingCopy.storedObject && this.workingCopy.storedObject.attributes) {
			value = this.workingCopy.storedObject.attributes[attrId];
		}
		if(!value) return []; // History has no attribute value;
		return [ value ];
	},
	_getPresentationPossibleValues: function (pres) {
		var attrId = pres.attributeId;
		var allValues = [];
		if(this.workingCopy && this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps && this.workingCopy.workItemSpec.editProps.allValues) {
				allValues = this.workingCopy.workItemSpec.editProps.allValues;
		}
		
		for(var i=0;i<allValues.length;i++) {
			if(allValues[i].attributeName == pres.attributeId) {
				return allValues[i].uiItems;
			}
		}
		
		return [  ];
	},
	_handleRefreshClick: function(e) {
		if(e != null){
			e.preventDefault();
		}
		this._currentOperation = this.workingCopy.createOperation({
			message: this.messages.busyLoading,
			id: "InlineWorkItemEditor/Loading",
			isModal: true,
			dialogTitle: this.messages.busyProgress
		});
		this.workingCopy.startOperation(this._currentOperation);
		this._currentLastComment = this.workingCopy.getValue({path: ["attributes" , this.workingCopy.SYNTHETIC_COMMENT_ID, "content" ]});
		this.refresh(/* force */ true);
	},
	_handleSaveClick: function(e) {
		if(e != null){
			e.preventDefault();
		}
		if(!this._isDirty) return;
		this.setSaveEnabled(false);
		this._saveWorkItem();
	},
	_saveWorkItem: function(isSaveAndClose) {
		if(this._isSaving) return; // Prevent double save which causes save failed due to status
		this._isSaving = true;
		this._currentOperation = this.workingCopy.createOperation({
			message: this.messages.busySaving,
			id: "InlineWorkItemEditor/Saving",
			isModal: true,
			dialogTitle: this.messages.busyProgress
		});
		this._currentLastComment = this.workingCopy.getValue({path: "attributes.comment.content"});
		this.workingCopy.startOperation(this._currentOperation);
		// Always refresh after save even if it is save and close to clean up the dirty bits.
		this.storeWorkItem(dojo.hitch(this, this._saveComplete), dojo.hitch(this, this.saveError), 
			{ instigator: this, op: this._currentOperation, isSaveAndClose: isSaveAndClose,  skipErrorMessage: true });

	},
	_saveComplete: function(parms) {
		// clear the internal comment value		
	    this.workingCopy.setValue( {
			path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
			attributeId: this.workingCopy.SYNTHETIC_COMMENT_ID,
			value: "",
			isRefresh: true
		});
		this._currentLastComment = "";
		this._isSaving = false;
		this.notifyChildren("handleSaveComplete")
		dojo.publish(EventTopics.WORKITEM_VIEW_CHANGED, [ ]);
		if(parms.isSaveAndClose) {
			this._handleCancelClick();
		}	
	},
	_finishCurrentOperation: function() {
		if (this._currentOperation) {
			this.workingCopy.finishOperation(this._currentOperation);
			this._currentOperation = null;
		}
	},
	_handleSaveAndCloseClick: function(e) {
		if(e != null){
			e.preventDefault();
		}
		if(!this._isDirty) return;
		this.setSaveEnabled(false);
		this._saveWorkItem(/* isSaveAndClose so no refresh */ true);
	},
	_promptToClose: function(e, isClose) {
		var useCount = this.workingCopy.getUseCount();
		if(useCount == 1 && (this.workingCopy.isNew() || this.workingCopy.isChanged()) && !confirm(this.messages.promptCloseWorkItemEdit)) {
			return false;
		}
		this._handleCancelClick(e, isClose);
		return true;
		},
	_handleCancelClick: function(e, isClose) {
		var workingCopy= this.workingCopy;
		this.destroy(isClose);
		if(this.parentWidget && this.parentWidget.hidePreviewEditor) { // TODO: change to hideInlineWorkItemEditor
				this.parentWidget.hidePreviewEditor(this.parentParms, null, isClose, workingCopy);
		}
		dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, [workingCopy, this]);
	},
	_handleBottomOkClicked: function(e) {
		this._handleSaveAndCloseClick(e);
	},
	_handleBottomCancelClicked: function(e) {
		this._handleCancelClick(e, /* isClose */ true);
	},
//	refreshInOperation: function(isForced, isInit, isQuiet, message, isSaveAndClose) {
//		if(this._currentOperation) {
//			this.workingCopy.setOperationMessage({ operation: this._currentOperation, message: message || this.messages.busyLoading});
//		} else {
//			this._currentOperation = this.workingCopy.createOperation({
//				message: message || this.messages.busyLoading,
//				id: "InlineWorkItemEditor/refresh",
//				isModal: !isQuiet==true,
//				dialogTitle: this.messages.busyProgress
//			});
//			this.workingCopy.startOperation(this._currentOperation);
//		}
//		return this.refresh(isForced, isInit, isQuiet, isSaveAndClose);
//	}, 
	refresh: function(isForced, isInit, isQuiet, isSaveAndClose) {
		var args = {
			id: this.workingCopy.getId(),
			projectArea: this.projectArea, 
			isForced: isForced, 
			isRefresh: true,
			callback: dojo.hitch(this, this._refreshComplete), 
			isInit: isInit, 
			isQuiet: isQuiet,
			isSaveAndClose: isSaveAndClose,
			fetchEditorPresentation: false,
			fetchInlineEditorPresentation: true,
			inlinePresentationId: this.presentation
			}
		if(this.workingCopy.isNewWorkItem()) {
			var type = this.workingCopy.getValue({path: "attributes.workItemType.id"});
			if(type) args.type = type;
		}
		this.fetchWorkItem(args);
	},
	_refreshComplete: function(parms /* parms.workItem, parms.editProps, parms.inlinePresentationProps */) {
		if(!parms.isSaveAndClose) {
			this._showWorkItemEditor(parms);	
		}
		if (this._currentOperation) {
			this.workingCopy.finishOperation(this._currentOperation);
			this._currentOperation = null;
		}
		// if refreshed in place with new to existing, add the refesh button 
		this._createRefreshButton();
		// remember the internal comment value		
	    this.workingCopy.setValue( {
			path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
			attributeId: this.workingCopy.SYNTHETIC_COMMENT_ID,
			value: this._currentLastComment || "",
			isRefresh: true
		});
		this._setIdLabel(/* reset */ true);
		if(this._currentLastComment && this._currentLastComment.length > 0) {
			this.setSaveEnabled(true);
		} 
		this._currentLastComment = "";
		if(parms.isSaveAndClose) {
			this._handleCancelClick();
		}
		/*
		 * When an editor is refreshed, check to make sure all other work items are in sych with the repository
		 */
		dojo.publish(EventTopics.WORKITEM_REFRESH_CACHE, [ [] ]); 
		
	},
	saveError: function(err, parms) {
		this._isSaving = false;
		// remember the internal comment value		
	    this.workingCopy.setValue( {
			path: ["attributes", this.workingCopy.SYNTHETIC_COMMENT_ID, "content"],
			attributeId: this.workingCopy.SYNTHETIC_COMMENT_ID,
			value: this._currentLastComment || ""
		});
		this._finishCurrentOperation();
		this.setSaveEnabled(true);
		var message = err.message || this.messages.errorUnknownDetails;
		if(message.length > 0 && message[message.length-1] != ".") {
			message = message + ".";
		}
		message += "\n";
	
		if(err && err.infos && err.infos) {
			for(var i=0; i< err.infos.length;i++) {
				if(err.infos[i].description) {
					message += err.infos[i].description;
					if(message.length > 0 &&  message[message.length-1] != ".") {
						message = message + ".";
					}
					message += "\n";
				}
			}
		}
		if(message) alert(message);
	},
	setIsDirty: function(isDirty) {
		var wasDirty = this._isDirty;
		this.inherited(arguments);
	    this.setSaveEnabled(isDirty, wasDirty);
	},
	setSaveEnabled: function(isDirty, wasDirty) {
		if(!this.isInitialized) return; // not initialized yet
		wasDirty = wasDirty == null ? this._isDirty : wasDirty;
		isDirty = isDirty && !this.workingCopy.isInOperation();
		if(dojo.isFunction(this.parentWidget.setSaveEnabled)) {
		   	this.parentWidget.setSaveEnabled(this.parentParms, isDirty);
		}
		if(isDirty) {
			    if(this.saveAndCloseButton) {
			    	this.saveAndCloseButton.disabled = false;
			    }
				if(this.saveButton) {
					this.saveButton.disabled = false;
				}
				if(this.okButton) {
					this.okButton.disabled = "";
				}
		} else {
			    if(this.saveAndCloseButton) {
			    	this.saveAndCloseButton.disabled = true;
				}
				if(this.saveButton) {
					this.saveButton.disabled = true;
				}
				if(this.okButton) {
					this.okButton.disabled = "disabled";
				}
		}
		
		if (wasDirty != this._isDirty) {
			if (this.workingCopy && this.workingCopy.storedObject) {
				dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, [this.workingCopy, this]);
			}
			else {
				dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, [null, this]);
			}
		}
	},
	getUrl: function() {
		return this.workingCopy.getUrl();
	}
});


})();
